@@ -2,7 +2,7 @@
 
   main.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -326,19 +326,19 @@ static void conv_data(int version, const char *data, long data_length, GB_VARIAN
 			{
 				case FIELD_TYPE_DATE:
 
-					sscanf(data, "%4hu-%2hu-%2hu", &date.year, &date.month, &date.day);
+					sscanf(data, "%4d-%2d-%2d", &date.year, &date.month, &date.day);
 					break;
 
 				case FIELD_TYPE_TIME:
 
-					sscanf(data, "%4hu:%2hu:%lf", &date.hour, &date.min, &sec);
+					sscanf(data, "%4d:%2d:%lf", &date.hour, &date.min, &sec);
 					date.sec = (short)sec;
 					date.msec = (short)((sec - date.sec) * 1000 + 0.5);
 					break;
 
 				case FIELD_TYPE_DATETIME:
 
-					sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
+					sscanf(data, "%4d-%2d-%2d %2d:%2d:%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
 					date.sec = (short)sec;
 					date.msec = (short)((sec - date.sec) * 1000 + 0.5);
 					break;
@@ -347,29 +347,29 @@ static void conv_data(int version, const char *data, long data_length, GB_VARIAN
 					switch(strlen(data))
 					{
 						case 14:
-							sscanf(data, "%4hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
+							sscanf(data, "%4d%2d%2d%2d%2d%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
 							date.sec = (short)sec;
 							date.msec = (short)((sec - date.sec) * 1000 + 0.5);
 							break;
 						case 12:
-							sscanf(data, "%2hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
+							sscanf(data, "%2d%2d%2d%2d%2d%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
 							date.sec = (short)sec;
 							date.msec = (short)((sec - date.sec) * 1000 + 0.5);
 							break;
 						case 10:
-							sscanf(data, "%2hu%2hu%2hu%2hu%2hu", &date.year, &date.month, &date.day, &date.hour, &date.min );
+							sscanf(data, "%2d%2d%2d%2d%2d", &date.year, &date.month, &date.day, &date.hour, &date.min );
 							break;
 						case 8:
-							sscanf(data, "%4hu%2hu%2hu", &date.year, &date.month, &date.day);
+							sscanf(data, "%4d%2d%2d", &date.year, &date.month, &date.day);
 							break;
 						case 6:
-							sscanf(data, "%2hu%2hu%2hu", &date.year, &date.month, &date.day);
+							sscanf(data, "%2d%2d%2d", &date.year, &date.month, &date.day);
 							break;
 						case 4:
-							sscanf(data, "%2hu%2hu", &date.year, &date.month);
+							sscanf(data, "%2d%2d", &date.year, &date.month);
 							break;
 						case 2:
-							sscanf(data, "%2hu", &date.year);
+							sscanf(data, "%2d", &date.year);
 							break;
 					}
 					if (date.year < 100)
@@ -452,6 +452,7 @@ static int do_query(DB_DATABASE *db, const char *error, MYSQL_RES **pres,
 	const char *query;
 	MYSQL_RES *res;
 	int ret;
+	unsigned long thread_id;
 
 	if (nsubst)
 	{
@@ -469,12 +470,22 @@ static int do_query(DB_DATABASE *db, const char *error, MYSQL_RES **pres,
 	if (DB.IsDebug())
 		fprintf(stderr, "mysql: %p: %s\n", conn, query);
 
-	if(mysql_query(conn, query)){
+	thread_id = mysql_thread_id(conn);
+	mysql_ping(conn);
+	if (mysql_thread_id(conn) != thread_id)
+	{
+		// Connection has been reestablished, set utf8 again
+		mysql_query(conn, "set names 'utf8'");
+	}
+	
+	if (mysql_query(conn, query))
+	{
 		ret = TRUE;
 		if (error)
 			GB.Error(error, mysql_error(conn));
 	}
-	else {
+	else 
+	{
 		res = mysql_store_result(conn);
 		ret = FALSE;
 		if (pres)
