@@ -3,7 +3,7 @@
   main.c
 
   (c) 2004-2007 Andrea Bortolan <andrea_bortolan@yahoo.it>
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -91,6 +91,7 @@ typedef struct
 		SQLUSMALLINT Function_exist;	//Does the Driver supports the SQLFetchScroll ?
 		SQLUSMALLINT Cursor_Scrollable;  //Is it possible to set a Scrollable cursor ?
 		ODBC_FIELDS *fields;
+		SQLLEN count;
 	}
 ODBC_RESULT;
 
@@ -392,7 +393,7 @@ static void conv_data(char *data, GB_VARIANT_VALUE * val, int type)
 				else
 					bc = 0;
 
-				sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year, &date.month,
+				sscanf(data, "%4d-%2d-%2d %2d:%2d:%lf", &date.year, &date.month,
 							 &date.day, &date.hour, &date.min, &sec);
 				date.sec = (short) sec;
 				date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
@@ -776,16 +777,16 @@ fflush(stderr);
 
 	if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO))
 	{
-		GB.Error("ODBC - Error - cannor allocate the handle");
+		GB.Error("Cannot allocate statement handle");
 		return retcode;
 	}
 
 
-	retcode =SQLSetStmtAttr(odbcres->odbcStatHandle, SQL_ATTR_CURSOR_SCROLLABLE, (SQLPOINTER) SQL_SCROLLABLE, 0);
+	retcode = SQLSetStmtAttr(odbcres->odbcStatHandle, SQL_ATTR_CURSOR_SCROLLABLE, (SQLPOINTER) SQL_SCROLLABLE, 0);
 
 	if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO))
 	{
-		odbcres->Cursor_Scrollable=SQL_FALSE;
+		odbcres->Cursor_Scrollable = SQL_FALSE;
 	} 
 	else odbcres->Cursor_Scrollable=SQL_TRUE;
 
@@ -799,13 +800,20 @@ fflush(stderr);
 	if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO))
 	{
 		SQLFreeHandle(SQL_HANDLE_STMT, odbcres->odbcStatHandle);
-		GB.Error("Error executing the statement");
+		GB.Error("Error while executing the statement");
 		return retcode;
 	}
 
 
 	if (res)
 	{
+		retcode = SQLRowCount(odbcres->odbcStatHandle, &odbcres->count);
+		if (retcode != SQL_SUCCESS && retcode != SQL_SUCCESS_WITH_INFO)
+		{
+			SQLFreeHandle(SQL_HANDLE_STMT, odbcres->odbcStatHandle);
+			GB.Error("Unable to retrieve row count");
+			return retcode;
+		}
 		*res = odbcres;
 	}
 	else
@@ -1011,7 +1019,6 @@ static void query_init(DB_RESULT result, DB_INFO * info, int *count)
 {
 
 	ODBC_RESULT *res = (ODBC_RESULT *) result;
-	SQLLEN rowsNum = -1;
 	SQLSMALLINT colsNum = 0;
 #ifdef ODBC_DEBUG_HEADER
 fprintf(stderr,"[ODBC][%s][%d]\n",__FILE__,__LINE__);
@@ -1023,9 +1030,9 @@ fflush(stderr);
 	if (!colsNum)
 		return;
 
-	SQLRowCount(res->odbcStatHandle, &rowsNum);
+	//SQLRowCount(res->odbcStatHandle, &rowsNum);
 	
-	*count = rowsNum;
+	*count = res->count;
 	info->nfield = colsNum;
 	query_make_result(res);
 }
