@@ -2,7 +2,7 @@
 
   main.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ extern "C"
 static char _buffer[32];
 
 static int _print_query = FALSE;
+static bool _need_field_type = FALSE;
 
 /*****************************************************************************
 
@@ -155,25 +156,25 @@ static void conv_data(const char *data, GB_VARIANT_VALUE * val, fType type)
 			switch (strlen(data))
 			{
 				case 14:
-					sscanf(data, "%4hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month,
+					sscanf(data, "%4d%2d%2d%2d%2d%lf", &date.year, &date.month,
 								 &date.day, &date.hour, &date.min, &sec);
 					date.sec = (short) sec;
 					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
 					break;
 				case 12:
-					sscanf(data, "%2hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month,
+					sscanf(data, "%2d%2d%2d%2d%2d%lf", &date.year, &date.month,
 								 &date.day, &date.hour, &date.min, &sec);
 					date.sec = (short) sec;
 					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
 					break;
 				case 10:
-					if (sscanf(data, "%4hu-%2hu-%2hu", &date.year, &date.month,
+					if (sscanf(data, "%4d-%2d-%2d", &date.year, &date.month,
 										 &date.day) != 3)
 					{
-						if (sscanf(data, "%4hu/%2hu/%2hu", &date.year, &date.month,
+						if (sscanf(data, "%4d/%2d/%2d", &date.year, &date.month,
 											 &date.day) != 3)
 						{
-							if (sscanf(data, "%4hu:%2hu:%lf", &date.hour, &date.min,
+							if (sscanf(data, "%4d:%2d:%lf", &date.hour, &date.min,
 												 &sec) == 3)
 							{
 								date.sec = (short) sec;
@@ -181,7 +182,7 @@ static void conv_data(const char *data, GB_VARIANT_VALUE * val, fType type)
 							}
 							else
 							{
-								sscanf(data, "%2hu%2hu%2hu%2hu%2hu", &date.year,
+								sscanf(data, "%2d%2d%2d%2d%2d", &date.year,
 											 &date.month, &date.day, &date.hour, &date.min);
 							}
 						}
@@ -189,24 +190,24 @@ static void conv_data(const char *data, GB_VARIANT_VALUE * val, fType type)
 
 					break;
 				case 8:
-					if (sscanf(data, "%4hu%2hu%2hu", &date.year, &date.month,
+					if (sscanf(data, "%4d%2d%2d", &date.year, &date.month,
 										 &date.day) != 3)
 					{
-						sscanf(data, "%2hu/%2hu/%2hu", &date.year, &date.month,
+						sscanf(data, "%2d/%2d/%2d", &date.year, &date.month,
 									 &date.day);
 					}
 					break;
 				case 6:
-					sscanf(data, "%2hu%2hu%2hu", &date.year, &date.month, &date.day);
+					sscanf(data, "%2d%2d%2d", &date.year, &date.month, &date.day);
 					break;
 				case 4:
-					sscanf(data, "%2hu%2hu", &date.year, &date.month);
+					sscanf(data, "%2d%2d", &date.year, &date.month);
 					break;
 				case 2:
-					sscanf(data, "%2hu", &date.year);
+					sscanf(data, "%2d", &date.year);
 					break;
 				default:
-					sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year,
+					sscanf(data, "%4d-%2d-%2d %2d:%2d:%lf", &date.year,
 								 &date.month, &date.day, &date.hour, &date.min, &sec);
 					date.sec = (short) sec;
 					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
@@ -294,8 +295,12 @@ static int do_query(DB_DATABASE *db, const char *error, Dataset **pres, const ch
 	va_list args;
 	int i;
 	const char *query;
-	Dataset *res = conn->CreateDataset();
-	int ret;
+	SqliteDataset *res = (SqliteDataset *)conn->CreateDataset();
+	int err;
+	int retry = 0;
+	int max_retry;
+	bool select;
+	bool success;
 
 	if (nsubst)
 	{
@@ -318,48 +323,51 @@ static int do_query(DB_DATABASE *db, const char *error, Dataset **pres, const ch
 	if (DB.IsDebug())
 		fprintf(stderr, "sqlite3: %p: %s\n", conn, query);
 
-	if (strncasecmp("select", query, 6) == 0)
-	{
-		if (res->query(query))
-		{
-			ret = FALSE;
-			if (pres)
-			{
-				*pres = res;
-			}
-		}
-		else
-		{
-			ret = TRUE;
-			GB.Error(error, conn->getErrorMsg());
-		}
-	}
+	if (db->timeout > 0)
+		max_retry = db->timeout * 5;
+	else if (db->timeout == 0)
+		max_retry = 600; // 120 s max
 	else
+		max_retry = 0;
+	
+	select = (strncasecmp("select ", query, 7) == 0);
+	
+	for(;;)
 	{
-		if (res->exec(query))
-		{
-			ret = FALSE;
-			if (pres)
-			{
-				*pres = res;
-			}
-		}
+		err = 0;
+
+		res->setNeedFieldType(_need_field_type);
+
+		if (select)
+			success = res->query(query);
 		else
+			success = res->exec(query);
+		
+		if (success)
 		{
-			ret = TRUE;
-			GB.Error(error, conn->getErrorMsg());
+			if (pres)
+				*pres = res;
+			break;
 		}
+		
+		err = conn->lastError();
+		
+		if (err != SQLITE_BUSY || retry >= max_retry)
+		{
+			GB.Error(error, conn->getErrorMsg());
+			break;
+		}
+		
+		retry++;
+		usleep(200000);
 	}
 	
 	if (!pres)
 		res->close();
 
-  if (ret)
-  	db->error = conn->lastError();
-  else
-  	db->error = 0;
-  
-	return ret;
+ 	db->error = err;
+	_need_field_type = FALSE;
+	return err != 0;
 }
 
 /* Internal function to check whether a file is a sqlite database file */
@@ -608,7 +616,7 @@ static const char *get_quote(void)
 
 *****************************************************************************/
 
-static int open_database(DB_DESC * desc, DB_DATABASE * db)
+static int open_database(DB_DESC *desc, DB_DATABASE * db)
 {
 	SqliteDatabase *conn = new SqliteDatabase();
 	char *name = NULL;
@@ -807,6 +815,7 @@ static void format_blob(DB_BLOB * blob, DB_FORMAT_CALLBACK add)
 
 static int exec_query(DB_DATABASE * db, const char *query, DB_RESULT * result, const char *err)
 {
+	_need_field_type = TRUE;
 	return do_query(db, err, (Dataset **) result, query, 0);
 }
 
@@ -884,6 +893,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 	int i;
 	const char *data;
 	GB_VARIANT value;
+	fType type;
 
 	if (!next)
 		res->seek(pos);							/* move to record */
@@ -892,8 +902,9 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 
 	for (i = 0; i < res->fieldCount(); i++)
 	{
-		//GB.NewString( &data, res->fv(res->fieldName(i)).get_asString().data(),0);
-		if (res->fv(i).get_isNull())
+		type = (fType) res->fieldType(i);
+		
+		if (type == ft_Blob || res->fv(i).get_isNull())
 			data = NULL;
 		else
 			data = res->fv(i).get_asString().data();
@@ -905,7 +916,7 @@ static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VAL
 
 		//if (field->type != FIELD_TYPE_NULL)
 		if (data)
-			conv_data(data, &value.value, (fType) res->fieldType(i));
+			conv_data(data, &value.value, type);
 
 		//GB.FreeString(&data);
 		GB.StoreVariant(&value, &buffer[i]);
@@ -1496,6 +1507,7 @@ static int table_is_system(DB_DATABASE * db, const char *table)
 /*****************************************************************************
 
   table_type()
+  
   Not Valid in Sqlite
 
   <handle> is the database handle.
@@ -1577,7 +1589,7 @@ static int table_create(DB_DATABASE * db, const char *table, DB_FIELD * fields,
 
 		if (fp->type == DB_T_SERIAL)
 		{
-			DB.Query.Add(" INTEGER PRIMARY KEY AUTOINCREMENT ");
+			DB.Query.Add(" INTEGER PRIMARY KEY AUTOINCREMENT");
 			no_pkey = TRUE;
 		}
 		else if (fp->type == DB_T_BLOB)
@@ -1644,7 +1656,9 @@ static int table_create(DB_DATABASE * db, const char *table, DB_FIELD * fields,
 			if (i > 0)
 				DB.Query.Add(",");
 
+			DB.Query.Add(QUOTE_STRING);
 			DB.Query.Add(primary[i]);
+			DB.Query.Add(QUOTE_STRING);
 		}
 
 		DB.Query.Add(")");
@@ -1677,8 +1691,7 @@ static int field_exist(DB_DATABASE * db, const char *table, const char *field)
 	Dataset *res;
 	int exist = 0;
 
-	if (do_query
-			(db, "Unable to find field: &1.&2", &res, query, 2, table, field))
+	if (do_query(db, "Unable to find field: &1.&2", &res, query, 2, table, field))
 	{
 		return FALSE;
 	}
@@ -1765,7 +1778,7 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 {
 	const char *query = "PRAGMA table_info('&1')";
 
-	Dataset *res;
+	Dataset *res, *res_autoinc;
 	GB_VARIANT def;
 	char *val;
 	char *_fieldName = NULL;
@@ -1774,15 +1787,19 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 	bool _fieldNotNull = FALSE;
 	int i, n;
 	//sqlite3 *db_handle = ((SqliteDatabase *)db->handle)->getHandle();
-	int autoinc;
 	fType type;
+	result_set *r;
+	char *schema;
+	char *p, *p2;
+	char *search;
+	bool autoinc;
 
 	if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
 	{
 		return TRUE;
 	}
 
-	result_set *r = (result_set *) res->getExecRes();
+	r = (result_set *) res->getExecRes();
 
 	if ((n = r->records.size()) == 0)
 	{
@@ -1815,11 +1832,47 @@ static int field_info(DB_DATABASE *db, const char *table, const char *field, DB_
 
 	info->name = NULL;
 
-						// This API is not always defined!
-	if (true) //(sqlite3_table_column_metadata(db_handle, NULL, table, field, NULL, NULL, NULL, NULL, &autoinc) != SQLITE_OK)
-	{
+	// This API is not always defined!
+	//(sqlite3_table_column_metadata(db_handle, NULL, table, field, NULL, NULL, NULL, NULL, &autoinc) != SQLITE_OK)
+
+	/*{
 		// [BM] We use INTEGER only when creating the AUTOINCREMENT field.
-		autoinc = strcasecmp(_fieldType, "INTEGER") == 0;
+		fprintf(stderr, "_fieldType = %s\n", _fieldType);
+		autoinc = strstr(_fieldType, "INTEGER") && strstr(_fieldType, "AUTOINCREMENT");
+	}*/
+	
+	autoinc = false;
+	
+	if (strstr(_fieldType, "INTEGER"))
+	{
+		if (!do_query(db, "Unable to get table schema: &1", &res_autoinc, "select sql from sqlite_master where type = 'table' and tbl_name = '&1'", 1, table))
+		{
+			schema = (char *)res_autoinc->fv(0).get_asString().data();
+			
+			search = (char *)alloca(2 + strlen(_fieldName));
+			sprintf(search, "'%s'", _fieldName);
+			
+			p = strchr(schema, '(');
+			if (p)
+			{
+				p = strstr(p, search);
+				if (p)
+				{
+					p2 = strchr(p, ',');
+					if (!p2)
+						p2 = strchr(p, ')');
+					if (p2)
+					{
+						p = strstr(p, "AUTOINCREMENT");
+						if (p && p < p2)
+							autoinc = true;
+					}
+				}
+			}
+			
+			res_autoinc->close();
+		}
+		
 	}
 	
 	type = GetFieldType(_fieldType, (unsigned int *) &info->length);
@@ -2062,17 +2115,21 @@ static int index_delete(DB_DATABASE * db, const char *table, const char *index)
 
 *****************************************************************************/
 
-static int index_create(DB_DATABASE * db, const char *table, const char *index, DB_INDEX * info)
+static int index_create(DB_DATABASE * db, const char *table, const char *index, DB_INDEX *info)
 {
 	DB.Query.Init();
 
 	DB.Query.Add("CREATE ");
 	if (info->unique)
 		DB.Query.Add("UNIQUE ");
-	DB.Query.Add("INDEX '");
+	DB.Query.Add("INDEX ");
+	DB.Query.Add(QUOTE_STRING);
 	DB.Query.Add(index);
-	DB.Query.Add("' ON ");
+	DB.Query.Add(QUOTE_STRING);
+	DB.Query.Add(" ON ");
+	DB.Query.Add(QUOTE_STRING);
 	DB.Query.Add(table);
+	DB.Query.Add(QUOTE_STRING);
 	DB.Query.Add(" ( ");
 	DB.Query.Add(info->fields);
 	DB.Query.Add(" )");
@@ -2280,6 +2337,10 @@ static int database_create(DB_DATABASE * db, const char *name)
 _CREATE_DATABASE:
 
 	conn2.setDatabase(fullpath);
+
+	if (DB.IsDebug())
+		fprintf(stderr, "sqlite3: create database: %s\n", fullpath);
+
 	GB.FreeString(&fullpath);
 
 	if (conn2.connect() != DB_CONNECTION_OK)
