@@ -2,7 +2,7 @@
 
   helper.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ static void check_message_now(DBusConnection *connection)
 
 static void check_message(DBusConnection *connection)
 {
-	GB.Post((GB_POST_FUNC)check_message_now, (intptr_t)connection);
+	GB.Post((GB_CALLBACK)check_message_now, (intptr_t)connection);
 }
 
 
@@ -712,14 +712,14 @@ bool DBUS_call_method(DBusConnection *connection, const char *application, const
 											const char *signature_in, const char *signature_out, GB_ARRAY arguments)
 {
 	DBusMessage *message;
-	int n;
+	//int n;
 	DBusMessageIter iter;
 	DBusMessage *reply;
 	DBusError error;
-	DBusSignatureIter siter;
+	//DBusSignatureIter siter;
 	bool ret;
-	GB_TYPE type;
-	int nparam;
+	//GB_TYPE type;
+	//int nparam;
 	
 	message = dbus_message_new_method_call(application, path, interface, method);
 	if (!message)
@@ -835,7 +835,7 @@ bool DBUS_reply(DBusConnection *connection, DBusMessage *message, const char *si
 {
 	bool ret = TRUE;
 	DBusMessage *reply;
-	DBusError error;
+	//DBusError error;
 	dbus_uint32_t serial = 0;
 
 	reply = dbus_message_new_method_return(message);
@@ -1069,7 +1069,7 @@ bool DBUS_register(DBusConnection *connection, const char *name, bool unique)
 
 	if (dbus_error_is_set(&error))
 	{
-		GB.Error("Unable to register application name");
+		GB.Error("Unable to register application name: &1", error.message);
 		dbus_error_free(&error);
 		return TRUE;
 	}
@@ -1083,15 +1083,14 @@ bool DBUS_register(DBusConnection *connection, const char *name, bool unique)
 bool DBUS_unregister(DBusConnection *connection, const char *name)
 {
 	DBusError error;
-	int ret, socket;
 	
 	dbus_error_init(&error);
 	
-	ret = dbus_bus_release_name(connection, name, &error);
+	dbus_bus_release_name(connection, name, &error);
 	
 	if (dbus_error_is_set(&error))
 	{
-		GB.Error("Unable to unregister application name");
+		GB.Error("Unable to unregister application name: &1", error.message);
 		dbus_error_free(&error);
 		return TRUE;
 	}
