@@ -2,7 +2,7 @@
 
   desktop.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -145,7 +145,7 @@ BEGIN_METHOD(CDESKTOP_sendkey, GB_STRING key; GB_BOOLEAN press)
 END_METHOD
 
 
-BEGIN_PROPERTY(CDESKTOP_root)
+BEGIN_PROPERTY(Desktop_Root)
 
 	if (X11_init())
 		return;
@@ -155,6 +155,17 @@ BEGIN_PROPERTY(CDESKTOP_root)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(Desktop_Time)
+
+	intptr_t time;
+	
+	GB.Component.GetInfo("TIME", POINTER(&time));
+	
+	GB.ReturnInteger((int)time);
+
+END_PROPERTY
+
+
 BEGIN_METHOD(CDESKTOP_get_window_property, GB_STRING name; GB_INTEGER window)
 
 	char *value;
@@ -333,7 +344,7 @@ BEGIN_METHOD(CDESKTOP_set_window_property, GB_STRING name; GB_STRING type; GB_VA
 							#if OS_64BITS
 							padded_data = (long *)alloca(sizeof(long) * count);
 							for (i = 0; i < count; i++)
-								padded_data[i] = *((long *)data + i);
+								padded_data[i] = *((int *)data + i);
 							data = padded_data;
 							#endif
 							break;
@@ -396,7 +407,7 @@ BEGIN_METHOD(CDESKTOP_intern_atom, GB_STRING atom; GB_BOOLEAN create)
 	if (X11_init())
 		return;
 	
-	GB.ReturnInteger(X11_intern_atom(GB.ToZeroString(ARG(atom)), !VARGOPT(create, FALSE)));
+	GB.ReturnInteger(X11_intern_atom(GB.ToZeroString(ARG(atom)), VARGOPT(create, FALSE)));
 
 END_METHOD
 
@@ -448,7 +459,7 @@ BEGIN_METHOD(CDESKTOP_send_client_message, GB_STRING message; GB_OBJECT data; GB
 				#if OS_64BITS
 					padded_data = (long *)alloca(sizeof(long) * count);
 					for (i = 0; i < count; i++)
-						padded_data[i] = *((long *)data + i);
+						padded_data[i] = *((int *)data + i);
 					data = padded_data;
 				#endif
 				break;
@@ -459,6 +470,7 @@ BEGIN_METHOD(CDESKTOP_send_client_message, GB_STRING message; GB_OBJECT data; GB
 				break;*/
 				
 			default:
+				fprintf(stderr, "gb.desktop: unsupported array datatype for 'Data' argument");
 				return;
 		}
 	}
@@ -583,6 +595,9 @@ END_METHOD
 
 BEGIN_METHOD(CDESKTOP_minimize_window, GB_INTEGER window; GB_BOOLEAN minimized)
 
+	if (X11_init())
+		return;
+	
 	if (VARG(minimized))
 	{
 		long state = IconicState;
@@ -598,6 +613,41 @@ BEGIN_METHOD(CDESKTOP_minimize_window, GB_INTEGER window; GB_BOOLEAN minimized)
 
 END_METHOD
 
+
+BEGIN_METHOD_VOID(Desktop_Flush)
+
+	if (X11_init())
+		return;
+	
+	XFlush(X11_display);
+
+END_METHOD
+
+
+BEGIN_METHOD_VOID(Desktop_Sync)
+
+	if (X11_init())
+		return;
+	
+	XSync(X11_display, FALSE);
+
+END_METHOD
+
+
+/*BEGIN_METHOD(Desktop_ActivateWindow, GB_INTEGER window)
+
+	intptr_t time;
+	
+	if (X11_init())
+		return;
+	
+	GB.Component.GetInfo("TIME", POINTER(&time));
+	
+	XSetInputFocus(X11_display, (Window)VARG(window), RevertToParent, (Time)time);
+
+END_METHOD*/
+
+
 GB_DESC CDesktopDesc[] =
 {
   GB_DECLARE("_Desktop", 0), GB_VIRTUAL_CLASS(),
@@ -608,20 +658,20 @@ GB_DESC CDesktopDesc[] =
   
 	GB_STATIC_METHOD("FindWindow", "Integer[]", CDESKTOP_find, "[(Title)s(Application)s(Role)s]"),
 	GB_STATIC_METHOD("SendKey", NULL, CDESKTOP_sendkey, "(Key)s(Press)b"),
-	GB_STATIC_PROPERTY_READ("RootWindow", "i", CDESKTOP_root),
+	GB_STATIC_PROPERTY_READ("RootWindow", "i", Desktop_Root),
+	GB_STATIC_PROPERTY_READ("Time", "i", Desktop_Time),
 	GB_STATIC_METHOD("GetWindowProperty", "v", CDESKTOP_get_window_property, "(Property)s[(Window)i]"),
-	//GB_STATIC_METHOD("GetWindowPropertyType", "s", CDESKTOP_get_window_property_type, "(Window)i(Property)s"),
 	GB_STATIC_METHOD("SetWindowProperty", NULL, CDESKTOP_set_window_property, "(Property)s(Type)s(Value)v[(Window)i]"),
 	GB_STATIC_METHOD("InternAtom", "i", CDESKTOP_intern_atom, "(Atom)s[(Create)b]"),
 	GB_STATIC_METHOD("GetAtomName", "s", CDESKTOP_get_atom_name, "(Atom)i"),
 	GB_STATIC_METHOD("SendClientMessageToRootWindow", NULL, CDESKTOP_send_client_message, "(Message)s[(Data)Array;(Window)i]"),
-  //GB_STATIC_PROPERTY_SELF("Root", "._Desktop.Window"),
-  //GB_STATIC_METHOD("_get", "._Desktop.Window"),
   GB_STATIC_PROPERTY("EventFilter", "b", CDESKTOP_event_filter),
   GB_STATIC_METHOD("WatchWindow", NULL, CDESKTOP_watch_window, "(Window)i(Watch)b"),
   GB_STATIC_METHOD("GetWindowGeometry", "Integer[]", CDESKTOP_get_window_geometry, "(Window)i"),
   GB_STATIC_METHOD("MakeIcon", "Image", CDESKTOP_make_icon, "(Data)Array;[(Width)i(Height)h]"),
   GB_STATIC_METHOD("MinimizeWindow", NULL, CDESKTOP_minimize_window, "(Window)i(Minimized)b"),
+  GB_STATIC_METHOD("Sync", NULL, Desktop_Sync, NULL),
+  GB_STATIC_METHOD("Flush", NULL, Desktop_Flush, NULL),
   
   GB_END_DECLARE
 };
@@ -671,7 +721,7 @@ static void enable_event_filter(bool enable)
 {
 	void (*set_event_filter)(void *) = NULL;
 	
-	GB.GetComponentInfo("SET_EVENT_FILTER", POINTER(&set_event_filter));
+	GB.Component.GetInfo("SET_EVENT_FILTER", POINTER(&set_event_filter));
 	if (set_event_filter)
 		(*set_event_filter)(enable ? x11_event_filter : 0);
 }
