@@ -36,9 +36,12 @@
 
 ***************************************************************************/
 
+static CIMAGE *_clipboard_image = NULL;
+
 BEGIN_METHOD_VOID(CCLIPBOARD_clear)
 
 	gClipboard::clear();
+	GB.StoreObject(NULL, POINTER(&_clipboard_image));
 
 END_METHOD
 
@@ -160,6 +163,9 @@ BEGIN_METHOD(CCLIPBOARD_copy, GB_VARIANT data; GB_STRING format)
 			goto _BAD_FORMAT;
 		
 		img = (CIMAGE *)VARG(data).value._object;
+		GB.Unref(POINTER(&_clipboard_image));
+		GB.Ref(img);
+		_clipboard_image = img;
 		gClipboard::setImage(CIMAGE_get(img));
 		return;
 	}
@@ -221,6 +227,7 @@ GB_DESC CClipboardDesc[] =
   GB_CONSTANT("Text", "i", 1),
   GB_CONSTANT("Image", "i", 2),
 
+  GB_STATIC_METHOD("_exit", 0, CCLIPBOARD_clear, 0),
   GB_STATIC_METHOD("Clear", 0, CCLIPBOARD_clear, 0),
 
   GB_STATIC_PROPERTY_READ("Format", "s", CCLIPBOARD_format),
@@ -242,6 +249,8 @@ GB_DESC CClipboardDesc[] =
 
 void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, char *format)
 {
+	gControl *dest;
+	
   if (GB.CheckObject(source))
     return NULL;
 
@@ -261,14 +270,14 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, char *format)
         goto _BAD_FORMAT;
     }
     
-    gDrag::dragText(source->widget, data->value._string, format);
+    dest = gDrag::dragText(source->widget, data->value._string, format);
   }
   else if (data->type >= GB_T_OBJECT && GB.Is(data->value._object, GB.FindClass("Image")))
   {
     if (format && *format)
       goto _BAD_FORMAT;
 
-		gDrag::dragImage(source->widget, CIMAGE_get((CIMAGE *)data->value._object));
+		dest = gDrag::dragImage(source->widget, CIMAGE_get((CIMAGE *)data->value._object));
   }
   else
     goto _BAD_FORMAT;
@@ -276,7 +285,7 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, char *format)
   //hide_frame(NULL);
   //GB.Post((GB_POST_FUNC)post_exit_drag, 0);
 
-  return GetObject(gDrag::getDestination());
+  return GetObject(dest);
 
 _BAD_FORMAT:
 
