@@ -36,6 +36,7 @@
 
 DECLARE_EVENT(EVENT_draw);
 
+
 /***************************************************************************
 
 	DrawingArea
@@ -44,10 +45,7 @@ DECLARE_EVENT(EVENT_draw);
 
 static void cleanup_drawing(intptr_t _object)
 {
-	if (THIS->painted)
-		PAINT_end();
-	else
-		DRAW_end();
+	PAINT_end();
 }
 
 static void Darea_Expose(gDrawingArea *sender,int x,int y,int w,int h) 
@@ -55,8 +53,6 @@ static void Darea_Expose(gDrawingArea *sender,int x,int y,int w,int h)
 	CWIDGET *_object = GetObject(sender);
 	GB_RAISE_HANDLER handler;
 	
-	//fprintf(stderr, "expose: %d %d %d %d\n", x, y, w, h);
-	
 	if (GB.CanRaise(THIS, EVENT_draw))
 	{
 		handler.callback = cleanup_drawing;
@@ -64,24 +60,12 @@ static void Darea_Expose(gDrawingArea *sender,int x,int y,int w,int h)
 			
 		GB.RaiseBegin(&handler);
 		
-		if (THIS->painted)
-		{
-			PAINT_begin(THIS);
-			PAINT_clip(x, y, w, h);
-			
-			GB.Raise(THIS, EVENT_draw, 0);
-			
-			PAINT_end();
-		}
-		else
-		{
-			DRAW_begin(THIS);
-			DRAW_get_current()->setClip(x, y, w, h);
-			
-			GB.Raise(THIS, EVENT_draw, 0);
-			
-			DRAW_end();
-		}
+		PAINT_begin(THIS);
+		PAINT_clip(x, y, w, h);
+		
+		GB.Raise(THIS, EVENT_draw, 0);
+		
+		PAINT_end();
 		
 		GB.RaiseEnd(&handler);
 	}
@@ -112,8 +96,10 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CDRAWINGAREA_focus)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(WIDGET->canFocus()); return; }
-	WIDGET->setCanFocus(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(WIDGET->canFocus());
+	else
+		WIDGET->setCanFocus(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
@@ -125,10 +111,16 @@ END_METHOD
 
 BEGIN_PROPERTY(CDRAWINGAREA_painted)
 
+	static bool deprecated = false;
+	
+	if (!deprecated)
+	{
+		deprecated = true;
+		GB.Deprecated("gb.gtk", "DrawingArea.Painted", NULL);
+	}
+	
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(THIS->painted);
-	else
-		THIS->painted = VPROP(GB_BOOLEAN);
+		GB.ReturnBoolean(true);
 
 END_PROPERTY
 
@@ -145,27 +137,41 @@ BEGIN_METHOD(DrawingArea_Refresh, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_I
 
 	int x, y, w, h;
 
-	x = VARGOPT(x, 0);
-	y = VARGOPT(y, 0);
-	w = VARGOPT(w, WIDGET->width());
-	h = VARGOPT(h, WIDGET->height());
-	
-	WIDGET->refresh(x,y,w,h);
+	if (MISSING(x) && MISSING(y) && MISSING(w) && MISSING(h))
+		WIDGET->refresh();
+	else
+	{
+		x = VARGOPT(x, 0);
+		y = VARGOPT(y, 0);
+		w = VARGOPT(w, WIDGET->width());
+		h = VARGOPT(h, WIDGET->height());
+		
+		WIDGET->refresh(x,y,w,h);
+	}
 
 END_METHOD
 
+BEGIN_PROPERTY(DrawingArea_Tablet)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(WIDGET->useTablet());
+	else
+		WIDGET->setUseTablet(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
 GB_DESC CDrawingAreaDesc[] =
 {
 	GB_DECLARE("DrawingArea", sizeof(CDRAWINGAREA)), GB_INHERITS("Container"),
 
 	GB_METHOD("_new", 0, CDRAWINGAREA_new, "(Parent)Container;"),
 
-	GB_PROPERTY("Arrangement", "i", CCONTAINER_arrangement),
-	GB_PROPERTY("AutoResize", "b", CCONTAINER_auto_resize),
-	GB_PROPERTY("Padding", "i", CCONTAINER_padding),
-	GB_PROPERTY("Spacing", "b", CCONTAINER_spacing),
-	GB_PROPERTY("Margin", "b", CCONTAINER_margin),
-	GB_PROPERTY("Indent", "b", CCONTAINER_indent),
+	GB_PROPERTY("Arrangement", "i", Container_Arrangement),
+	GB_PROPERTY("AutoResize", "b", Container_AutoResize),
+	GB_PROPERTY("Padding", "i", Container_Padding),
+	GB_PROPERTY("Spacing", "b", Container_Spacing),
+	GB_PROPERTY("Margin", "b", Container_Margin),
+	GB_PROPERTY("Indent", "b", Container_Indent),
   GB_PROPERTY("Invert", "b", Container_Invert),
 
 	GB_PROPERTY("Cached", "b", CDRAWINGAREA_cached),
@@ -173,13 +179,14 @@ GB_DESC CDrawingAreaDesc[] =
 	GB_PROPERTY("Focus","b",CDRAWINGAREA_focus),
 	GB_PROPERTY("Painted", "b", CDRAWINGAREA_painted),
 	GB_PROPERTY("NoBackground", "b", DrawingArea_NoBackground),
+	
+	GB_PROPERTY("Tablet", "b", DrawingArea_Tablet),
 
-	GB_METHOD("Clear", 0, CDRAWINGAREA_clear, 0),
+	GB_METHOD("Clear", 0, CDRAWINGAREA_clear, NULL),
 	GB_METHOD("Refresh", NULL, DrawingArea_Refresh, "[(X)i(Y)i(Width)i(Height)i]"),
 
 	GB_EVENT("Draw", 0, 0, &EVENT_draw),
 	
-	GB_INTERFACE("Draw", &DRAW_Interface),
 	GB_INTERFACE("Paint", &PAINT_Interface),
 	
 	DRAWINGAREA_DESCRIPTION,
