@@ -35,6 +35,9 @@ DECLARE_EVENT(EVENT_Click);
 DECLARE_EVENT(EVENT_Show);
 DECLARE_EVENT(EVENT_Hide);
 
+static CMENU *_popup_menu_clicked = NULL;
+static GB_FUNCTION _init_shortcut_func;
+
 
 static void send_click_event(void *_object)
 {
@@ -57,6 +60,13 @@ static void delete_later(gMenu *menu)
 
 static void delete_menu(gMenu *menu)
 {
+	void *_object = menu->hFree;
+	
+	if (!MENU)
+		return;
+	
+	THIS->widget = NULL;
+	
 	GB.Post((GB_CALLBACK)delete_later, (intptr_t)menu);
 }
 
@@ -76,14 +86,42 @@ static void cb_finish(gMenu *sender)
 static void cb_click(gMenu *sender)
 {
 	void *_object = sender->hFree;
+
 	GB.Ref(THIS);
-	GB.Post((GB_POST_FUNC)send_click_event, (intptr_t)THIS);
+	
+	if (gMenu::insidePopup())
+	{
+		GB.Unref(POINTER(&_popup_menu_clicked));
+		_popup_menu_clicked = THIS;
+	}
+	else
+		send_click_event(THIS);
 }
 
 static void cb_show(gMenu *sender)
 {
+	static bool init = FALSE;
+
 	void *_object = sender->hFree;
+
+	GB.Ref(THIS);
+
 	GB.Raise(THIS, EVENT_Show, 0);
+
+	if (!THIS->init_shortcut)
+	{
+		if (!init)
+		{
+			GB.GetFunction(&_init_shortcut_func, (void *)GB.FindClass("_Gui"), "_DefineShortcut", NULL, NULL);
+			init = TRUE;
+		}
+
+		THIS->init_shortcut = TRUE;
+		GB.Push(1, GB_T_OBJECT, THIS);
+		GB.Call(&_init_shortcut_func, 1, FALSE);
+	}
+
+	GB.Unref(POINTER(&_object));
 }
 
 static void cb_hide(gMenu *sender)
@@ -116,7 +154,7 @@ BEGIN_METHOD(CMENU_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 			return;
 		}
 
-		THIS->widget = new gMenu( (gMainWindow*)((CWINDOW*)parent)->ob.widget,hidden);
+		THIS->widget = new gMenu((gMainWindow*)((CWINDOW*)parent)->ob.widget, hidden);
 		goto __OK;
 	}
 	
@@ -128,8 +166,8 @@ BEGIN_METHOD(CMENU_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 			return;
 		}
 		
-		THIS->widget = new gMenu( (gMenu*)((CMENU*)parent)->widget,hidden);
-		MENU->onClick=cb_click;
+		THIS->widget = new gMenu((gMenu*)((CMENU*)parent)->widget, hidden);
+		MENU->onClick = cb_click;
 		goto __OK;
 	}
 	
@@ -139,9 +177,9 @@ BEGIN_METHOD(CMENU_new, GB_OBJECT parent; GB_BOOLEAN hidden)
 __OK:
 
 	MENU->hFree = (void*)THIS;
-	MENU->onFinish=cb_finish;
-	MENU->onShow=cb_show;
-	MENU->onHide=cb_hide;
+	MENU->onFinish = cb_finish;
+	MENU->onShow = cb_show;
+	MENU->onHide = cb_hide;
 
 	name = GB.GetLastEventName();
 	if (!name)
@@ -170,6 +208,9 @@ BEGIN_PROPERTY(CMENU_text)
 	}
 	MENU->setText(GB.ToZeroString(PROP(GB_STRING)));
 
+	if (!MENU->topLevel())
+		((CMENU *)GetObject((gMenu *)MENU->parent()))->init_shortcut = FALSE;
+
 END_PROPERTY
 
 
@@ -309,17 +350,18 @@ END_METHOD
 BEGIN_METHOD_VOID(CMENU_clear)
 
 	gMenu *mn;
-
-	int bucle,max;
+	int i, max;
 	
-	max=MENU->childCount();
+	max = MENU->childCount();
 	
-	for (bucle=0;bucle<max;bucle++)
+	for (i = 0; i < max; i++)
 	{
-		mn = MENU->childMenu(bucle);
+		mn = MENU->childMenu(i);
 		delete_menu(mn);
 	}
 
+	THIS->init_shortcut = FALSE;
+
 END_PROPERTY
 
 
@@ -329,6 +371,13 @@ BEGIN_METHOD(CMENU_popup, GB_INTEGER x; GB_INTEGER y)
 		MENU->popup(VARG(x), VARG(y));
 	else
 		MENU->popup();
+	
+	if (_popup_menu_clicked)
+	{
+		CMENU *menu = _popup_menu_clicked;
+		_popup_menu_clicked = NULL;
+		send_click_event(menu);
+	}
 
 END_METHOD
 
