@@ -3,16 +3,10 @@
   CScreen.h
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -34,11 +29,26 @@
 #include "gapplication.h"
 
 #ifndef __CSCREEN_CPP
-extern GB_DESC CDesktopDesc[];
-extern GB_DESC CApplicationTooltipDesc[];
-extern GB_DESC CApplicationDesc[];
+extern GB_DESC ScreenDesc[];
+extern GB_DESC ScreensDesc[];
+extern GB_DESC DesktopDesc[];
+extern GB_DESC ApplicationDesc[];
+extern GB_DESC StyleDesc[];
 
 extern char *CAPPLICATION_Theme;
-#endif
+
+#else
+
+#define SCREEN ((CSCREEN *)_object)
 
 #endif
+
+typedef
+	struct
+	{
+		GB_BASE ob;
+		int index;
+	}
+	CSCREEN;
+	
+#endif
