@@ -94,6 +94,11 @@ static int CTRAYICON_check(void *_object)
 	return TRAYICON == NULL;
 }
 
+static void cb_mouse_wheel(gTrayIcon *sender)
+{
+	GB.Raise(sender->hFree, EVENT_MouseWheel, NULL);
+}
+
 BEGIN_METHOD_VOID(CTRAYICON_new)
 
 	THIS->base.widget = new gTrayIcon();
@@ -103,6 +108,7 @@ BEGIN_METHOD_VOID(CTRAYICON_new)
 	
 	TRAYICON->onMousePress=Tray_press;
 	TRAYICON->onMouseRelease=Tray_release;
+	TRAYICON->onMouseWheel=cb_mouse_wheel;
 	TRAYICON->onMenu=Tray_menu;
 	TRAYICON->onDestroy=Tray_destroy;
 	TRAYICON->onFocusEnter=Tray_gotFocus;
@@ -122,6 +128,7 @@ static void destroy_tray_icon(CTRAYICON *_object)
 	{
 		delete TRAYICON;
 		THIS->base.widget = NULL;
+		MAIN_check_quit();
 	}
 }
 
@@ -180,13 +187,20 @@ END_METHOD
 BEGIN_METHOD_VOID(CTRAYICON_hide)
 
 	TRAYICON->hide();
+	MAIN_check_quit();
 
 END_METHOD
 
 BEGIN_PROPERTY(CTRAYICON_visible)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(TRAYICON->isVisible()); return; }
-	TRAYICON->setVisible(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(TRAYICON->isVisible());
+	else
+	{
+		TRAYICON->setVisible(VPROP(GB_BOOLEAN));
+		if (!VPROP(GB_BOOLEAN))
+			MAIN_check_quit();
+	}
 
 END_PROPERTY
 
@@ -282,7 +296,7 @@ GB_DESC CTrayIconDesc[] =
 	
 	GB_METHOD("Show", 0, CTRAYICON_show, 0),
 	GB_METHOD("Hide", 0, CTRAYICON_hide, 0),
-	GB_METHOD("Delete",0,CTRAYICON_destroy,0),
+	GB_METHOD("Delete",0, CTRAYICON_destroy,0),
 	
 	GB_PROPERTY("Visible","b",CTRAYICON_visible),
 	GB_PROPERTY("Picture","Picture",CTRAYICON_picture),
