@@ -54,6 +54,7 @@ DECLARE_EVENT(EVENT_Menu);
 DECLARE_EVENT(EVENT_Drag);
 DECLARE_EVENT(EVENT_DragMove);
 DECLARE_EVENT(EVENT_Drop);
+DECLARE_EVENT(EVENT_DragLeave);
 
 //Plug
 DECLARE_EVENT(EVENT_Plugged);
@@ -151,6 +152,8 @@ static int to_gambas_event(int type)
 		case gEvent_FocusOut: return EVENT_LostFocus;
 		case gEvent_Enter: return EVENT_Enter;
 		case gEvent_Leave: return EVENT_Leave;
+		case gEvent_DragMove: return EVENT_DragMove;
+		case gEvent_Drop: return EVENT_Drop;
 		default: fprintf(stderr, "warning: to_gambas_event: unhandled event: %d\n", type); return -1;
 	}
 }
@@ -178,8 +181,7 @@ bool gb_raise_MouseEvent(gControl *sender, int type)
 	switch(type)
 	{
 		case gEvent_MouseDrag:
-			if (gMouse::button() && GB.CanRaise(ob, EVENT_MouseDrag))
-				ret = GB.Raise(ob, EVENT_MouseDrag, 0);
+			ret = GB.Raise(ob, EVENT_MouseDrag, 0);
 			break;
 			
 		case gEvent_MouseMenu:
@@ -235,26 +237,54 @@ void gb_raise_FocusEvent(gControl *sender, int type)
 
 bool gb_raise_Drag(gControl *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
 	
-	if (!_ob) return false;
-	return GB.Raise((void*)_ob,EVENT_Drag,0);
+	if (!THIS)
+		return true;
+	
+	if (!GB.CanRaise(THIS, EVENT_Drag))
+	{
+		if (GB.CanRaise(THIS, EVENT_DragMove) || GB.CanRaise(THIS, EVENT_Drop))
+			return false;
+		else
+			return true;
+	}
+	
+	return GB.Raise(THIS, EVENT_Drag, 0);
+}
+
+void gb_raise_DragLeave(gControl *sender)
+{
+	CWIDGET *_object = GetObject(sender);
+	
+	GB.Raise(THIS, EVENT_DragLeave, 0);
 }
 
 bool gb_raise_DragMove(gControl *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
 	
-	if (!_ob) return false;
-	return GB.Raise((void*)_ob,EVENT_DragMove,0);
+	if (!THIS)
+		return true;
+	
+	if (!GB.CanRaise(THIS, EVENT_DragMove))
+		return !GB.CanRaise(THIS, EVENT_Drag);
+
+	return GB.Raise(THIS, EVENT_DragMove, 0);
 }
 
-void gb_raise_Drop(gControl *sender)
+bool gb_raise_Drop(gControl *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *_object = GetObject(sender);
 	
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Drop,0);
+	if (!THIS)
+		return false;
+	
+	if (!GB.CanRaise(THIS, EVENT_Drop))
+		return false;
+	
+	GB.Raise(THIS, EVENT_Drop, 0);
+	return true;
 }
 
 void DeleteControl(gControl *control)
@@ -288,6 +318,7 @@ void DeleteControl(gControl *control)
 
 void InitControl(gControl *control, CWIDGET *widget)
 {
+	static int n = 0;
 	char *name;
 	
 	GB.Ref((void*)widget);
@@ -298,15 +329,21 @@ void InitControl(gControl *control, CWIDGET *widget)
 	
 	name = GB.GetLastEventName();
 	if (!name)
-		name = GB.GetClassName((void *)widget);
-		
-	control->setName(name);
+	{
+		char buffer[16];
+		n++;
+		sprintf(buffer, "#%d", n);
+		control->setName(buffer);
+	}
+	else
+		control->setName(name);
 	
 	control->onFinish = DeleteControl;
 	control->onMouseEvent = gb_raise_MouseEvent;
 	control->onKeyEvent = gb_raise_KeyEvent;
 	control->onFocusEvent = gb_raise_FocusEvent;
 	control->onDrag = gb_raise_Drag;
+	control->onDragLeave = gb_raise_DragLeave;
 	control->onDragMove = gb_raise_DragMove;
 	control->onDrop = gb_raise_Drop;
 	control->onEnterLeave = gb_raise_EnterLeave;
@@ -335,6 +372,11 @@ CWIDGET *GetContainer(CWIDGET *control)
 	return control;
 }
 
+int CWIDGET_get_handle(void *_object)
+{
+	return CONTROL->handle();
+}
+
 int CWIDGET_check(void *_object)
 {
 	return (!CONTROL || CONTROL->isDestroyed());
@@ -436,7 +478,8 @@ BEGIN_PROPERTY(CCONTROL_font)
 	
 	if (READ_PROPERTY)
 	{
-    GB.ReturnObject(THIS->font);
+		CONTROL->font()->copyTo(((CFONT *)THIS->font)->font);
+		GB.ReturnObject(THIS->font);
 	}
 	else
 	{
@@ -473,15 +516,17 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CWIDGET_enabled)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(CONTROL->enabled()); return; }
-	CONTROL->setEnabled(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(CONTROL->isEnabled());
+	else
+		CONTROL->setEnabled(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CWIDGET_has_focus)
+BEGIN_PROPERTY(Control_HasFocus)
 
-	GB.ReturnBoolean(CONTROL == gApplication::activeControl());
+	GB.ReturnBoolean(CONTROL->hasFocus());
 
 END_PROPERTY
 
@@ -509,14 +554,10 @@ END_PROPERTY
 
 BEGIN_METHOD(CWIDGET_move, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)
 
-	CONTROL->move(VARG(x),VARG(y));
-	if (MISSING(w)) return;
-	if (MISSING(h))
-	{
-		CONTROL->resize(VARG(w),CONTROL->height());
-		return;
-	}
-	CONTROL->resize(VARG(w),VARG(h));
+	if (MISSING(w) && MISSING(h))
+		CONTROL->move(VARG(x), VARG(y));
+	else
+		CONTROL->moveResize(VARG(x), VARG(y), VARGOPT(w, CONTROL->width()), VARGOPT(h, CONTROL->height()));
 
 END_METHOD
 
@@ -909,7 +950,7 @@ GB_DESC CWidgetDesc[] =
 
 	GB_PROPERTY("Visible", "b", CWIDGET_visible),
 	GB_PROPERTY("Enabled", "b", CWIDGET_enabled),
-	GB_PROPERTY_READ("HasFocus", "b", CWIDGET_has_focus),
+	GB_PROPERTY_READ("HasFocus", "b", Control_HasFocus),
 	GB_PROPERTY_READ("Hovered", "b", Control_Hovered),
 
 	GB_PROPERTY("Expand", "b", CWIDGET_expand),
@@ -937,22 +978,23 @@ GB_DESC CWidgetDesc[] =
 	GB_PROPERTY_READ("Id", "i", CWIDGET_id),
 	GB_PROPERTY_READ("Handle", "i", CWIDGET_id),
 
-	GB_EVENT("Enter", 0, 0, &EVENT_Enter),
-	GB_EVENT("GotFocus", 0, 0, &EVENT_GotFocus),
-	GB_EVENT("LostFocus", 0, 0, &EVENT_LostFocus),
-	GB_EVENT("KeyPress", 0, 0, &EVENT_KeyPress),
-	GB_EVENT("KeyRelease", 0, 0, &EVENT_KeyRelease),
-	GB_EVENT("Leave", 0, 0, &EVENT_Leave),
-	GB_EVENT("MouseDown", 0, 0, &EVENT_MouseDown),
-	GB_EVENT("MouseMove", 0, 0, &EVENT_MouseMove),
-	GB_EVENT("MouseDrag", 0, 0, &EVENT_MouseDrag),
-	GB_EVENT("MouseUp", 0, 0, &EVENT_MouseUp),
-	GB_EVENT("MouseWheel", 0, 0, &EVENT_MouseWheel),
-	GB_EVENT("DblClick", 0, 0, &EVENT_DblClick),
-	GB_EVENT("Menu", 0, 0, &EVENT_Menu),
-	GB_EVENT("Drag", 0, 0, &EVENT_Drag),
-	GB_EVENT("DragMove", 0, 0, &EVENT_DragMove),
-	GB_EVENT("Drop", 0, 0, &EVENT_Drop),
+	GB_EVENT("Enter", NULL, NULL, &EVENT_Enter),
+	GB_EVENT("GotFocus", NULL, NULL, &EVENT_GotFocus),
+	GB_EVENT("LostFocus", NULL, NULL, &EVENT_LostFocus),
+	GB_EVENT("KeyPress", NULL, NULL, &EVENT_KeyPress),
+	GB_EVENT("KeyRelease", NULL, NULL, &EVENT_KeyRelease),
+	GB_EVENT("Leave", NULL, NULL, &EVENT_Leave),
+	GB_EVENT("MouseDown", NULL, NULL, &EVENT_MouseDown),
+	GB_EVENT("MouseMove", NULL, NULL, &EVENT_MouseMove),
+	GB_EVENT("MouseDrag", NULL, NULL, &EVENT_MouseDrag),
+	GB_EVENT("MouseUp", NULL, NULL, &EVENT_MouseUp),
+	GB_EVENT("MouseWheel", NULL, NULL, &EVENT_MouseWheel),
+	GB_EVENT("DblClick", NULL, NULL, &EVENT_DblClick),
+	GB_EVENT("Menu", NULL, NULL, &EVENT_Menu),
+	GB_EVENT("Drag", NULL, NULL, &EVENT_Drag),
+	GB_EVENT("DragMove", NULL, NULL, &EVENT_DragMove),
+	GB_EVENT("Drop", NULL, NULL, &EVENT_Drop),
+	GB_EVENT("DragLeave", NULL, NULL, &EVENT_DragLeave),
 
 	CONTROL_DESCRIPTION,
 
@@ -964,15 +1006,15 @@ GB_DESC CPluginDesc[] =
   GB_DECLARE("Embedder", sizeof(CPLUGIN)), GB_INHERITS("Control"),
 
   GB_METHOD("_new", 0, CPLUGIN_new, "(Parent)Container;"),
-  GB_METHOD("Embed", 0, CPLUGIN_embed, "(Client)i[(Prepared)b]"),
+  GB_METHOD("Embed", 0, CPLUGIN_embed, "(Client)i"),
   GB_METHOD("Discard", 0, CPLUGIN_discard, 0),
   
   GB_PROPERTY_READ("Client", "i", CPLUGIN_client),
   //GB_PROPERTY("Border", "i<Border>", CPLUGIN_border),
   
-  GB_EVENT("Embed", 0, 0, &EVENT_Plugged),
-  GB_EVENT("Close", 0, 0, &EVENT_Unplugged),
-  GB_EVENT("Error", 0, 0, &EVENT_PlugError), /* TODO */
+  GB_EVENT("Embed", NULL, NULL, &EVENT_Plugged),
+  GB_EVENT("Close", NULL, NULL, &EVENT_Unplugged),
+  GB_EVENT("Error", NULL, NULL, &EVENT_PlugError), /* TODO */
   
   EMBEDDER_DESCRIPTION,
   
