@@ -470,9 +470,9 @@ END_PROPERTY
 BEGIN_PROPERTY(Window_Utility)
 
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(WINDOW->getType() == _NET_WM_WINDOW_TYPE_UTILITY);
+		GB.ReturnBoolean(WINDOW->isUtility());
 	else
-		WINDOW->setType(VPROP(GB_BOOLEAN) ? _NET_WM_WINDOW_TYPE_UTILITY : _NET_WM_WINDOW_TYPE_NORMAL);
+		WINDOW->setUtility(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
@@ -714,6 +714,8 @@ BEGIN_METHOD_VOID(CFORM_new)
 
 	if (!GB.Parent(_object))
 		GB.Attach(_object, _object, "Form");
+	
+	WINDOW->setName(GB.GetClassName((void *)THIS));
 
 END_METHOD
 
@@ -834,12 +836,13 @@ GB_DESC CWindowDesc[] =
 	GB_PROPERTY("SkipTaskbar", "b", CWINDOW_skip_taskbar),
 	GB_PROPERTY("Opacity", "i", Window_Opacity),
 
-	GB_PROPERTY("Arrangement", "i", CCONTAINER_arrangement),
-	GB_PROPERTY("Padding", "i", CCONTAINER_padding),
-	GB_PROPERTY("Spacing", "b", CCONTAINER_spacing),
-	GB_PROPERTY("Margin", "b", CCONTAINER_margin),
-	GB_PROPERTY("AutoResize", "b", CCONTAINER_auto_resize),
-  GB_PROPERTY("Invert", "b", Container_Invert),
+	GB_PROPERTY("Arrangement", "i", Container_Arrangement),
+	GB_PROPERTY("AutoResize", "b", Container_AutoResize),
+	GB_PROPERTY("Padding", "i", Container_Padding),
+	GB_PROPERTY("Spacing", "b", Container_Spacing),
+	GB_PROPERTY("Margin", "b", Container_Margin),
+	GB_PROPERTY("Indent", "b", Container_Indent),
+	GB_PROPERTY("Invert", "b", Container_Invert),
 
 	//GB_PROPERTY("Type", "i", CWINDOW_type),
 	GB_PROPERTY("Utility", "b", Window_Utility),
@@ -868,8 +871,6 @@ GB_DESC CWindowDesc[] =
 	GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
 	//GB_EVENT("State", NULL, NULL, &EVENT_State),
 
-	GB_INTERFACE("Draw", &DRAW_Interface),
-
 	GB_END_DECLARE
 };
 
