@@ -2,7 +2,7 @@
 
   cpaint_impl.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -74,6 +74,109 @@ static cairo_surface_t *check_image(void *img)
 	return (cairo_surface_t *)IMAGE.Check((GB_IMG *)img, &_image_owner);
 }
 
+// Function partially taken from the GTK+ source code.
+
+static cairo_surface_t *gdk_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
+{
+	gint width = gdk_pixbuf_get_width (pixbuf);
+	gint height = gdk_pixbuf_get_height (pixbuf);
+	guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
+	int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+	int cairo_stride;
+	guchar *cairo_pixels;
+	cairo_format_t format;
+	cairo_surface_t *surface;
+	static const cairo_user_data_key_t key = { 0 };
+	int j;
+
+	if (n_channels == 3)
+		format = CAIRO_FORMAT_RGB24;
+	else
+		format = CAIRO_FORMAT_ARGB32;
+
+	cairo_stride = cairo_format_stride_for_width (format, width);
+	cairo_pixels = (uchar *)g_malloc (height * cairo_stride);
+	surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
+																								format,
+																								width, height, cairo_stride);
+
+	cairo_surface_set_user_data (surface, &key,
+						cairo_pixels, (cairo_destroy_func_t)g_free);
+
+	for (j = height; j; j--)
+	{
+		guchar *p = gdk_pixels;
+		guchar *q = cairo_pixels;
+
+		if (n_channels == 3)
+		{
+			guchar *end = p + 3 * width;
+			
+			while (p < end)
+				{
+					#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+					q[0] = p[2];
+					q[1] = p[1];
+					q[2] = p[0];
+					#else
+					q[1] = p[0];
+					q[2] = p[1];
+					q[3] = p[2];
+					#endif
+					p += 3;
+					q += 4;
+				}
+		}
+		else
+		{
+			guchar *end = p + 4 * width;
+			guint t1,t2,t3;
+				
+			#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
+
+			while (p < end)
+				{
+					#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+					MULT(q[0], p[2], p[3], t1);
+					MULT(q[1], p[1], p[3], t2);
+					MULT(q[2], p[0], p[3], t3);
+					q[3] = p[3];
+					#else	  
+					q[0] = p[3];
+					MULT(q[1], p[0], p[3], t1);
+					MULT(q[2], p[1], p[3], t2);
+					MULT(q[3], p[2], p[3], t3);
+					#endif
+					
+					p += 4;
+					q += 4;
+				}
+			
+			#undef MULT
+		}
+
+		gdk_pixels += gdk_rowstride;
+		cairo_pixels += cairo_stride;
+	}
+
+	return surface;
+}
+
+static GB_COLOR get_color(GB_PAINT *d, GB_COLOR col)
+{
+	if (col == GB_COLOR_DEFAULT)
+	{
+		if (GB.Is(d->device, CLASS_DrawingArea))
+			col = (((CWIDGET *)d->device)->widget)->realBackground();
+		else
+			col = 0xFFFFFF;
+	}
+
+	return col;
+}
+
+
 /**** Paint implementation ***********************************************/
 
 typedef
@@ -83,11 +186,20 @@ typedef
 		CFONT *font;
 		CFONT **font_stack;
 		cairo_matrix_t init;
+		double dx;
+		double dy;
+		double bx;
+		double by;
+		bool invert;
 		}
 	GB_PAINT_EXTRA;
 
 #define EXTRA(d) ((GB_PAINT_EXTRA *)d->extra)
 #define CONTEXT(d) EXTRA(d)->context
+//#define DX(d) EXTRA(d)->dx
+//#define DY(d) EXTRA(d)->dy
+#define DX(d) 0
+#define DY(d) 0
 
 static bool init_painting(GB_PAINT *d, cairo_surface_t *target, double w, double h, int rx, int ry)
 {
@@ -110,6 +222,8 @@ static bool init_painting(GB_PAINT *d, cairo_surface_t *target, double w, double
 		cairo_surface_destroy(target);
 	}
 	
+	cairo_set_line_width(CONTEXT(d), 1.0);
+	
 	dx->font = NULL;
 	dx->font_stack = NULL;
 	
@@ -173,6 +287,7 @@ static int Begin(GB_PAINT *d)
 	int rx = 96, ry = 96;
 	
 	EXTRA(d)->print_context = NULL;
+	EXTRA(d)->dx = EXTRA(d)->dy = 0;
 	
 	if (GB.Is(device, CLASS_Picture))
 	{
@@ -209,6 +324,8 @@ static int Begin(GB_PAINT *d)
 	{
 		gDrawingArea *wid = (gDrawingArea *)((CWIDGET *)device)->widget;
 		GdkDrawable *dr;
+		GtkAllocation *a;
+		double dx = 0, dy = 0;
 		
 		w = wid->width();
 		h = wid->height();
@@ -225,17 +342,31 @@ static int Begin(GB_PAINT *d)
 				GB.Error("Cannot paint outside of Draw event handler");
 				return TRUE;
 			}
+			
+			a = &wid->widget->allocation;
+			dx = a->x;
+			dy = a->y;
 			dr = wid->widget->window;
 		}
 
-		/*target = 
-			cairo_xlib_surface_create(gdk_x11_drawable_get_xdisplay(dr), gdk_x11_drawable_get_xid(dr), 
-				gdk_x11_visual_get_xvisual(gdk_drawable_get_visual(dr)), w, h);*/
-		
 		d->resolutionX = gDesktop::resolution(); //device->physicalDpiX();
 		d->resolutionY = gDesktop::resolution(); //device->physicalDpiY();
 		
 		EXTRA(d)->context = gdk_cairo_create(dr);
+		EXTRA(d)->dx = dx;
+		EXTRA(d)->dy = dy;
+		
+		cairo_translate(CONTEXT(d), dx, dy);
+		//cairo_surface_get_device_offset(cairo_get_target(CONTEXT(d)), &EXTRA(d)->dx, &EXTRA(d)->dy);
+		//fprintf(stderr, "Begin: device offset = %g %g ", EXTRA(d)->dx, EXTRA(d)->dy);
+		
+		//dx += EXTRA(d)->dx;
+		//dy += EXTRA(d)->dy;
+		//cairo_surface_set_device_offset(cairo_get_target(CONTEXT(d)), dx, dy);
+		//cairo_surface_get_device_offset(cairo_get_target(CONTEXT(d)), &dx, &dy);
+		//fprintf(stderr, "-> device offset = %g %g\n", dx, dy);
+		//EXTRA(d)->font = CFONT_create(wid->font());
+		//GB.Ref(EXTRA(d)->font);
 	}
 	else if (GB.Is(device, CLASS_Printer))
 	{
@@ -300,12 +431,10 @@ static void End(GB_PAINT *d)
 		GB.FreeArray(POINTER(&dx->font_stack));
 	GB.Unref(POINTER(&dx->font));
 	
-	cairo_destroy(dx->context);
-
 	if (GB.Is(device, CLASS_DrawingArea))
 	{
 		gDrawingArea *wid = (gDrawingArea *)((CWIDGET *)device)->widget;
-		if (wid->cached())
+		if (wid && wid->cached())
 			wid->setCache();
 	}
 	else if (GB.Is(device, CLASS_SvgImage))
@@ -313,6 +442,8 @@ static void End(GB_PAINT *d)
 		CSVGIMAGE *svgimage = ((CSVGIMAGE *)device);
 		SVGIMAGE_end(svgimage);
 	}
+	
+	cairo_destroy(dx->context);
 }
 
 static void Save(GB_PAINT *d)
@@ -357,34 +488,74 @@ static void Antialias(GB_PAINT *d, int set, int *antialias)
 // Font is used by X11!
 static void _Font(GB_PAINT *d, int set, GB_FONT *font)
 {
-	if (!EXTRA(d)->font)
-	{
-		EXTRA(d)->font = CFONT_create(new gFont());
-		GB.Ref(EXTRA(d)->font);
-	}
-		
 	if (set)
 	{
-		GB.Ref(*font);
 		GB.Unref(POINTER(&EXTRA(d)->font));
-		EXTRA(d)->font = (CFONT *)(*font);
+		if (*font)
+		{
+			EXTRA(d)->font = CFONT_create(((CFONT *)(*font))->font->copy());
+			GB.Ref(EXTRA(d)->font);
+		}
+		else
+			EXTRA(d)->font = NULL;
 	}
 	else
+	{
+		if (!EXTRA(d)->font)
+		{
+			if (GB.Is(d->device, CLASS_DrawingArea))
+			{
+				gDrawingArea *wid = (gDrawingArea *)((CWIDGET *)d->device)->widget;
+				EXTRA(d)->font = CFONT_create(wid->font()->copy());
+			}
+			else
+			{
+				EXTRA(d)->font = CFONT_create(new gFont());
+			}
+			
+			GB.Ref(EXTRA(d)->font);
+		}
+			
 		*font = (GB_FONT)EXTRA(d)->font;
+	}
 }
 
-/*static void init_path(GB_PAINT *d)
+
+static void Background(GB_PAINT *d, int set, GB_COLOR *color)
 {
-	switch (EXTRA(d)->fillRule)
+	if (set)
 	{
-		case GB_PAINT_FILL_RULE_WINDING:
-			PATH(d)->setFillRule(Qt::WindingFill); 
-			break;
-		case GB_PAINT_FILL_RULE_EVEN_ODD: 
-		default:
-			PATH(d)->setFillRule(Qt::OddEvenFill);
+		int r, g, b, a;
+		int col = get_color(d, *color);
+		GB_COLOR_SPLIT(col, r, g, b, a);
+		cairo_set_source_rgba(CONTEXT(d), r / 255.0, g / 255.0, b / 255.0, a / 255.0);
 	}
-}*/
+	else
+	{
+		double r, g, b, a;
+		if (cairo_pattern_get_rgba(cairo_get_source(CONTEXT(d)), &r, &g, &b, &a) != CAIRO_STATUS_SUCCESS)
+			*color = 0;
+		else
+			*color = GB_COLOR_MAKE((int)(r * 255.0), (int)(g * 255.0), (int)(b * 255.0), (int)(a * 255.0));
+	}
+}
+
+
+static void Invert(GB_PAINT *d, int set, int *invert)
+{
+	#if CAIRO_MAJOR >= 2 || (CAIRO_MAJOR == 1 && CAIRO_MINOR >= 10)
+	if (set)
+		cairo_set_operator(CONTEXT(d), *invert ? CAIRO_OPERATOR_DIFFERENCE : CAIRO_OPERATOR_OVER);
+	else
+		*invert = cairo_get_operator(CONTEXT(d)) == CAIRO_OPERATOR_DIFFERENCE;
+	#else
+	if (set)
+		EXTRA(d)->invert = *invert;
+	else
+		*invert = EXTRA(d)->invert;
+	#endif
+}
+
 
 static void Clip(GB_PAINT *d, int preserve)
 {
@@ -404,8 +575,8 @@ static void ClipExtents(GB_PAINT *d, GB_EXTENTS *ext)
 	double x1, y1, x2, y2;
 	cairo_clip_extents(CONTEXT(d), &x1, &y1, &x2, &y2);
 	
-	ext->x1 = (float)x1;
-	ext->y1 = (float)y1;
+	ext->x1 = (float)x1 - DX(d);
+	ext->y1 = (float)y1 - DY(d);
 	ext->x2 = (float)x2;
 	ext->y2 = (float)y2;
 }
@@ -431,15 +602,15 @@ static void PathExtents(GB_PAINT *d, GB_EXTENTS *ext)
 	double x1, y1, x2, y2;
 	cairo_path_extents(CONTEXT(d), &x1, &y1, &x2, &y2);
 	
-	ext->x1 = (float)x1;
-	ext->y1 = (float)y1;
+	ext->x1 = (float)x1 - DX(d);
+	ext->y1 = (float)y1 - DY(d);
 	ext->x2 = (float)x2;
 	ext->y2 = (float)y2;
 }
 
 static int PathContains(GB_PAINT *d, float x, float y)
 {
-	return cairo_in_fill(CONTEXT(d), (double)x, (double)y);
+	return cairo_in_fill(CONTEXT(d), (double)x + DX(d), (double)y + DY(d));
 }
 
 static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
@@ -452,7 +623,7 @@ static void Dash(GB_PAINT *d, int set, float **dashes, int *count)
 	
 	if (set)
 	{
-		if (!*dashes || *count == 0)
+		if (!*count)
 			cairo_set_dash(CONTEXT(d), NULL, 0, 0.0);
 		else
 		{
@@ -513,7 +684,7 @@ static void FillRule(GB_PAINT *d, int set, int *value)
 	{
 		cairo_fill_rule_t v;
 	
-		switch(*value)
+		switch (*value)
 		{
 			case GB_PAINT_FILL_RULE_EVEN_ODD: v = CAIRO_FILL_RULE_EVEN_ODD; break;
 			case GB_PAINT_FILL_RULE_WINDING: default: v = CAIRO_FILL_RULE_WINDING;
@@ -523,7 +694,7 @@ static void FillRule(GB_PAINT *d, int set, int *value)
 	}
 	else
 	{
-		switch(cairo_get_fill_rule(CONTEXT(d)))
+		switch (cairo_get_fill_rule(CONTEXT(d)))
 		{
 			case CAIRO_FILL_RULE_EVEN_ODD: *value = GB_PAINT_FILL_RULE_EVEN_ODD; break;
 			case CAIRO_FILL_RULE_WINDING: default: *value = GB_PAINT_FILL_RULE_WINDING;
@@ -531,6 +702,18 @@ static void FillRule(GB_PAINT *d, int set, int *value)
 	}
 }
 
+
+static void FillStyle(GB_PAINT *d, int set, int *style)
+{
+	/*if (set)
+	{
+		EXTRA(d)->fillRule = *value;
+	}
+	else
+		*value = EXTRA(d)->fillRule;*/
+}
+
+
 static void LineCap(GB_PAINT *d, int set, int *value)
 {
 	if (set)
@@ -673,43 +856,87 @@ static void ClosePath(GB_PAINT *d)
 }
 
 		
-static void Arc(GB_PAINT *d, float xc, float yc, float radius, float angle, float length)
+static void Arc(GB_PAINT *d, float xc, float yc, float radius, float angle, float length, bool pie)
 {
+	xc += DX(d);
+	yc += DY(d);
+	
 	cairo_new_sub_path(CONTEXT(d));
-	//angle = - angle;
+
+	if (pie)
+		cairo_move_to(CONTEXT(d), 0, 0);
+	
 	if (length < 0.0)
 		cairo_arc_negative(CONTEXT(d), xc, yc, radius, angle, angle + length);
 	else
 		cairo_arc(CONTEXT(d), xc, yc, radius, angle, angle + length);
+
+	if (pie)
+		cairo_close_path(CONTEXT(d));
+}
+
+static void Ellipse(GB_PAINT *d, float x, float y, float width, float height, float angle, float length, bool pie)
+{
+	x += DX(d);
+	y += DY(d);
+	
+	cairo_new_sub_path(CONTEXT(d));
+	
+	cairo_save(CONTEXT(d));
+
+	cairo_translate(CONTEXT(d), x + width / 2, y + height / 2);
+	cairo_scale(CONTEXT(d), width / 2, height / 2);
+
+	if (pie)
+		cairo_move_to(CONTEXT(d), 0, 0);
+	
+	if (length < 0.0)
+		cairo_arc_negative(CONTEXT(d), 0, 0, 1, angle, angle + length);
+	else
+		cairo_arc(CONTEXT(d), 0, 0, 1, angle, angle + length);
+	
+	if (pie)
+		cairo_close_path(CONTEXT(d));
+
+	cairo_restore(CONTEXT(d));
 }
 
 static void Rectangle(GB_PAINT *d, float x, float y, float width, float height)
 {
+	x += DX(d);
+	y += DY(d);
 	cairo_rectangle(CONTEXT(d), x, y, width, height);
 }
 
+static void ClipRect(GB_PAINT *d, int x, int y, int w, int h)
+{
+	ResetClip(d);
+	Rectangle(d, x, y, w, h);
+	Clip(d, FALSE);
+}
+
 static void GetCurrentPoint(GB_PAINT *d, float *x, float *y)
 {
 	double cx, cy;
 	
 	cairo_get_current_point(CONTEXT(d), &cx, &cy);
-	*x = (float)cx;
-	*y = (float)cy;
+	*x = (float)cx - DX(d);
+	*y = (float)cy - DY(d);
 }
 
 static void MoveTo(GB_PAINT *d, float x, float y)
 {
-	cairo_move_to(CONTEXT(d), x, y);
+	cairo_move_to(CONTEXT(d), x + DX(d), y + DY(d));
 }
 
 static void LineTo(GB_PAINT *d, float x, float y)
 {
-	cairo_line_to(CONTEXT(d), x, y);
+	cairo_line_to(CONTEXT(d), x + DX(d), y + DY(d));
 }
 
 static void CurveTo(GB_PAINT *d, float x1, float y1, float x2, float y2, float x3, float y3)
 {
-	cairo_curve_to(CONTEXT(d), x1, y1, x2, y2, x3, y3);
+	cairo_curve_to(CONTEXT(d), x1 + DX(d), y1 + DY(d), x2 + DX(d), y2 + DY(d), x3 + DX(d), y3 + DY(d));
 }
 
 static PangoLayout *create_pango_layout(GB_PAINT *d)
@@ -725,7 +952,7 @@ static PangoLayout *create_pango_layout(GB_PAINT *d)
 static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w, float h, int align, bool draw)
 {
 	char *html = NULL;
-  PangoLayout *layout;
+	PangoLayout *layout;
 	CFONT *font;
 	float tw, th, offx, offy;
 
@@ -734,6 +961,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	if (rich)
 	{
 		html = gt_html_to_pango_string(text, len, false);
+		pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
 		pango_layout_set_markup(layout, html, -1);
 		if (w > 0)
 			pango_layout_set_width(layout, (int)(w * PANGO_SCALE));
@@ -747,7 +975,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	if (align == GB_DRAW_ALIGN_DEFAULT)
 		align = ALIGN_TOP_NORMAL;
 	
-	if (w > 0 && h > 0)
+	if (w > 0 || h > 0)
 	{
 		gt_layout_alignment(layout, w, h, &tw, &th, align, &offx, &offy);
 		if (rich)
@@ -759,7 +987,7 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 		offy = -(font->font->ascentF());
 	}
 	
-	cairo_rel_move_to(CONTEXT(d), offx, offy);
+	cairo_rel_move_to(CONTEXT(d), offx + DX(d), offy + DY(d));
 	if (draw)
 		pango_cairo_show_layout(CONTEXT(d), layout);
 	else
@@ -794,6 +1022,7 @@ static void get_text_extents(GB_PAINT *d, bool rich, const char *text, int len,
 	if (rich)
 	{
 		html = gt_html_to_pango_string(text, len, false);
+		pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
 		pango_layout_set_markup(layout, html, -1);
 	}
 	else
@@ -828,6 +1057,24 @@ static void RichTextExtents(GB_PAINT *d, const char *text, int len, GB_EXTENTS *
 	get_text_extents(d, TRUE, text, len, ext, width);
 }
 
+static void TextSize(GB_PAINT *d, const char *text, int len, float *w, float *h)
+{
+	CFONT *font;
+	_Font(d, FALSE, (GB_FONT *)&font);
+	
+	*w = font->font->width(text, len);
+	*h = font->font->height(text, len);
+}
+
+static void RichTextSize(GB_PAINT *d, const char *text, int len, float sw, float *w, float *h)
+{
+	CFONT *font;
+	_Font(d, FALSE, (GB_FONT *)&font);
+
+	font->font->richTextSize(text, len, sw, w, h);
+}
+
+
 static void Matrix(GB_PAINT *d, int set, GB_TRANSFORM matrix)
 {
 	cairo_matrix_t *t = (cairo_matrix_t *)matrix;
@@ -841,7 +1088,10 @@ static void Matrix(GB_PAINT *d, int set, GB_TRANSFORM matrix)
 			if (EXTRA(d)->print_context)
 				cairo_set_matrix(CONTEXT(d), &EXTRA(d)->init);
 			else
+			{
 				cairo_identity_matrix(CONTEXT(d));
+				cairo_translate(CONTEXT(d), EXTRA(d)->dx, EXTRA(d)->dy);
+			}
 		}
 	}
 	else
@@ -854,6 +1104,28 @@ static void SetBrush(GB_PAINT *d, GB_BRUSH brush)
 	cairo_set_source(CONTEXT(d), (cairo_pattern_t *)brush);
 }
 
+static void BrushOrigin(GB_PAINT *d, int set, float *x, float *y)
+{
+	if (set)
+	{
+		cairo_pattern_t *brush;
+		cairo_matrix_t matrix;
+		
+		brush = cairo_get_source(CONTEXT(d));
+		cairo_pattern_get_matrix(brush, &matrix);
+		cairo_matrix_translate(&matrix, EXTRA(d)->bx, EXTRA(d)->by);
+		cairo_matrix_translate(&matrix, (- *x), (- *y));
+		cairo_pattern_set_matrix(brush, &matrix);
+		
+		EXTRA(d)->bx = *x;
+		EXTRA(d)->by = *y;
+	}
+	else
+	{
+		*x = EXTRA(d)->bx;
+		*y = EXTRA(d)->by;
+	}
+}
 
 static void BrushFree(GB_BRUSH brush)
 {
@@ -868,95 +1140,6 @@ static void BrushColor(GB_BRUSH *brush, GB_COLOR color)
 	*brush = (GB_BRUSH)cairo_pattern_create_rgba(r / 255.0, g / 255.0, b / 255.0, a / 255.0);
 }
 
-// Function partially taken from the GTK+ source code.
-
-static cairo_surface_t *gdk_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
-{
-  gint width = gdk_pixbuf_get_width (pixbuf);
-  gint height = gdk_pixbuf_get_height (pixbuf);
-  guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
-  int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-  int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-  int cairo_stride;
-  guchar *cairo_pixels;
-  cairo_format_t format;
-  cairo_surface_t *surface;
-  static const cairo_user_data_key_t key = { 0 };
-  int j;
-
-  if (n_channels == 3)
-    format = CAIRO_FORMAT_RGB24;
-  else
-    format = CAIRO_FORMAT_ARGB32;
-
-  cairo_stride = cairo_format_stride_for_width (format, width);
-  cairo_pixels = (uchar *)g_malloc (height * cairo_stride);
-  surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
-                                                 format,
-                                                 width, height, cairo_stride);
-
-  cairo_surface_set_user_data (surface, &key,
-			       cairo_pixels, (cairo_destroy_func_t)g_free);
-
-  for (j = height; j; j--)
-    {
-      guchar *p = gdk_pixels;
-      guchar *q = cairo_pixels;
-
-      if (n_channels == 3)
-	{
-	  guchar *end = p + 3 * width;
-	  
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      q[0] = p[2];
-	      q[1] = p[1];
-	      q[2] = p[0];
-#else	  
-	      q[1] = p[0];
-	      q[2] = p[1];
-	      q[3] = p[2];
-#endif
-	      p += 3;
-	      q += 4;
-	    }
-	}
-      else
-	{
-	  guchar *end = p + 4 * width;
-	  guint t1,t2,t3;
-	    
-#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
-
-	  while (p < end)
-	    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-	      MULT(q[0], p[2], p[3], t1);
-	      MULT(q[1], p[1], p[3], t2);
-	      MULT(q[2], p[0], p[3], t3);
-	      q[3] = p[3];
-#else	  
-	      q[0] = p[3];
-	      MULT(q[1], p[0], p[3], t1);
-	      MULT(q[2], p[1], p[3], t2);
-	      MULT(q[3], p[2], p[3], t3);
-#endif
-	      
-	      p += 4;
-	      q += 4;
-	    }
-	  
-#undef MULT
-	}
-
-      gdk_pixels += gdk_rowstride;
-      cairo_pixels += cairo_stride;
-    }
-
-	return surface;
-}
-
 static void BrushImage(GB_BRUSH *brush, GB_IMAGE image)
 {
 	gPicture *picture = CIMAGE_get((CIMAGE *)image);
@@ -1056,6 +1239,12 @@ static void TransformCreate(GB_TRANSFORM *matrix)
 	cairo_matrix_init_identity((cairo_matrix_t *)*matrix);
 }
 
+static void TransformCopy(GB_TRANSFORM *matrix, GB_TRANSFORM copy)
+{
+	GB.Alloc(POINTER(matrix), sizeof(cairo_matrix_t));
+	*(cairo_matrix_t *)*matrix = *(cairo_matrix_t *)copy;
+}
+
 static void TransformDelete(GB_TRANSFORM *matrix)
 {
 	GB.Free(POINTER(matrix));
@@ -1092,27 +1281,111 @@ static void TransformMultiply(GB_TRANSFORM matrix, GB_TRANSFORM matrix2)
 	cairo_matrix_multiply((cairo_matrix_t *)matrix, (cairo_matrix_t *)matrix, (cairo_matrix_t *)matrix2);
 }
 
-static void DrawImage(GB_PAINT *d, GB_IMAGE image, float x, float y, float w, float h)
+static void TransformMap(GB_TRANSFORM matrix, double *x, double *y)
+{
+	cairo_matrix_transform_point((cairo_matrix_t *)matrix, x, y);
+}
+
+
+static void DrawImage(GB_PAINT *d, GB_IMAGE image, float x, float y, float w, float h, float opacity, GB_RECT *source)
+{
+	cairo_t *cr = CONTEXT(d);
+	gPicture *picture = CIMAGE_get((CIMAGE *)image);
+	cairo_surface_t *surface;
+	cairo_pattern_t *pattern = NULL;
+	cairo_pattern_t *save;
+	cairo_matrix_t matrix;
+	
+	//gt_cairo_draw_pixbuf(CONTEXT(d), picture->getPixbuf(), x, y, w, h, opacity, source);
+	//return;
+
+	cairo_save(cr);
+	
+	save = cairo_get_source(cr);
+	cairo_pattern_reference(save);
+	
+	//if (source)
+	//	pixbuf = gdk_pixbuf_new_subpixbuf(pixbuf, source->x, source->y, source->w, source->h);
+	
+	surface = picture->getSurface();
+
+	pattern = cairo_pattern_create_for_surface(surface);
+
+	cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
+		
+	if (source && w >= source->w && h >= source->h && w == (int)w && h == (int)h && ((int)w % source->w) == 0 && ((int)h % source->h) == 0)
+		cairo_pattern_set_filter(pattern, CAIRO_FILTER_NEAREST);
+		
+		//gdk_cairo_set_source_pixbuf(cr, picture->getPixbuf(), x, y);
+		
+	cairo_matrix_init_identity(&matrix);
+	cairo_matrix_translate(&matrix, x, y);
+	if (source)
+	{
+		cairo_matrix_scale(&matrix, w / source->w, h / source->h);
+		cairo_matrix_translate(&matrix, -source->x, -source->y);
+	}
+	else if (w > 0 && h > 0)
+		cairo_matrix_scale(&matrix, w / cairo_image_surface_get_width(surface), h / cairo_image_surface_get_height(surface));
+	
+	cairo_matrix_invert(&matrix);
+	cairo_pattern_set_matrix(pattern, &matrix);
+	cairo_set_source(cr, pattern);
+		
+	cairo_rectangle(cr, x, y, w, h);
+	
+	if (opacity == 1.0)
+	{
+		cairo_fill(cr);
+	}
+	else
+	{
+		cairo_clip(cr);
+		cairo_paint_with_alpha(cr, opacity);
+	}
+	
+	cairo_set_source(cr, save);
+	cairo_pattern_destroy(save);
+	
+	cairo_restore(cr);
+	
+	cairo_pattern_destroy(pattern);
+}
+
+static void DrawPicture(GB_PAINT *d, GB_PICTURE picture, float x, float y, float w, float h, GB_RECT *source)
 {
 	cairo_pattern_t *pattern, *save;
 	cairo_matrix_t matrix;
-	gPicture *picture = CIMAGE_get((CIMAGE *)image);
+	gPicture *pic = ((CPICTURE *)picture)->picture;
+	
+	if (pic->type() != gPicture::PIXMAP || source)
+	{
+		gt_cairo_draw_pixbuf(CONTEXT(d), pic->getPixbuf(), x, y, w, h, 1.0, source);
+		return;
+	}
+	
+	x += DX(d);
+	y += DY(d);
 	
 	cairo_save(CONTEXT(d));
-	
 	save = cairo_get_source(CONTEXT(d));
 	cairo_pattern_reference(save);
 	
-	BrushImage((GB_BRUSH *)&pattern, image);
+	gdk_cairo_set_source_pixmap(CONTEXT(d), pic->getPixmap(), 0, 0);
+	
+	pattern = cairo_get_source(CONTEXT(d));
+	cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
+	
+	/*if (source && w >= source->w && h >= source->h && w == (int)w && h == (int)h && ((int)w % source->w) == 0 && ((int)h % source->h) == 0)
+		cairo_pattern_set_filter(pattern, CAIRO_FILTER_NEAREST);*/
 	
 	//gdk_cairo_set_source_pixbuf(CONTEXT(d), picture->getPixbuf(), x, y);
 	
 	cairo_matrix_init_identity(&matrix);
 	cairo_matrix_translate(&matrix, x, y);
-	cairo_matrix_scale(&matrix, w / picture->width(), h / picture->height());
+	cairo_matrix_scale(&matrix, w / pic->width(), h / pic->height());
 	cairo_matrix_invert(&matrix);
 	cairo_pattern_set_matrix(pattern, &matrix);
-	cairo_set_source(CONTEXT(d), pattern);
 	
 	cairo_rectangle(CONTEXT(d), x, y, w, h);
 	cairo_fill(CONTEXT(d));
@@ -1123,6 +1396,32 @@ static void DrawImage(GB_PAINT *d, GB_IMAGE image, float x, float y, float w, fl
 	cairo_restore(CONTEXT(d));
 }
 
+static void GetPictureInfo(GB_PAINT *d, GB_PICTURE picture, GB_PICTURE_INFO *info)
+{
+	gPicture *pic = ((CPICTURE *)picture)->picture;
+	
+	info->width = pic->width();
+	info->height = pic->height();
+}
+
+static void FillRect(GB_PAINT *d, float x, float y, float w, float h, GB_COLOR color)
+{
+	cairo_pattern_t *save;
+	
+	x += DX(d);
+	y += DY(d);
+	
+	save = cairo_get_source(CONTEXT(d));
+	cairo_pattern_reference(save);
+	
+	Background(d, TRUE, &color);
+	cairo_rectangle(CONTEXT(d), x, y, w, h);
+	cairo_fill(CONTEXT(d));
+	
+	cairo_set_source(CONTEXT(d), save);
+	cairo_pattern_destroy(save);
+}
+
 
 GB_PAINT_DESC PAINT_Interface = 
 {
@@ -1133,9 +1432,12 @@ GB_PAINT_DESC PAINT_Interface =
 	Restore,
 	Antialias,
 	_Font,
+	Background,
+	Invert,
 	Clip,
 	ResetClip,
 	ClipExtents,
+	ClipRect,
 	Fill,
 	Stroke,
 	PathExtents,
@@ -1143,6 +1445,7 @@ GB_PAINT_DESC PAINT_Interface =
 	Dash,
 	DashOffset,
 	FillRule,
+	FillStyle,
 	LineCap,
 	LineJoin,
 	LineWidth,
@@ -1151,6 +1454,7 @@ GB_PAINT_DESC PAINT_Interface =
 	NewPath,
 	ClosePath,
 	Arc,
+	Ellipse,
 	Rectangle,
 	GetCurrentPoint,
 	MoveTo,
@@ -1158,11 +1462,17 @@ GB_PAINT_DESC PAINT_Interface =
 	CurveTo,
 	Text,
 	TextExtents,
+	TextSize,
 	RichText,
 	RichTextExtents,
+	RichTextSize,
 	Matrix,
 	SetBrush,
+	BrushOrigin,
 	DrawImage,
+	DrawPicture,
+	GetPictureInfo,
+	FillRect,
 	{
 		BrushFree,
 		BrushColor,
@@ -1170,19 +1480,23 @@ GB_PAINT_DESC PAINT_Interface =
 		BrushLinearGradient,
 		BrushRadialGradient,
 		BrushMatrix,
-	},
-	{
-		TransformCreate,
-		TransformDelete,
-		TransformInit,
-		TransformTranslate,
-		TransformScale,
-		TransformRotate,
-		TransformInvert,
-		TransformMultiply
 	}
 };
 
+GB_PAINT_MATRIX_DESC PAINT_MATRIX_Interface =
+{
+	TransformCreate,
+	TransformCopy,
+	TransformDelete,
+	TransformInit,
+	TransformTranslate,
+	TransformScale,
+	TransformRotate,
+	TransformInvert,
+	TransformMultiply,
+	TransformMap
+};
+
 void PAINT_begin(void *device)
 {
 	DRAW.Paint.Begin(device);
@@ -1213,3 +1527,34 @@ cairo_t *PAINT_get_current_context()
 	return NULL;
 }
 
+void *PAINT_get_current_device()
+{
+	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
+	if (d)
+		return d->device;
+	
+	GB.Error("No current device");
+	return NULL;
+}
+
+bool PAINT_get_clip(int *x, int *y, int *w, int *h)
+{
+	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
+	GB_EXTENTS ext;
+	
+	ClipExtents(d, &ext);
+	
+	*x = ceilf(ext.x1);
+	*y = ceilf(ext.y1);
+	*w = floorf(ext.x2) - *x;
+	*h = floorf(ext.y2) - *y;
+	
+	return *w <= 0 || *h <= 0;
+}
+
+void PAINT_apply_offset(int *x, int *y)
+{
+	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
+	*x += EXTRA(d)->dx;
+	*y += EXTRA(d)->dy;
+}
