@@ -2,7 +2,7 @@
 
   cpaint_impl.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #ifndef __CPAINT_IMPL_C
 
 extern GB_PAINT_DESC PAINT_Interface;
+extern GB_PAINT_MATRIX_DESC PAINT_MATRIX_Interface;
 
 #endif
 
@@ -37,5 +38,8 @@ void PAINT_begin(void *device);
 void PAINT_end();
 void PAINT_clip(int x, int y, int w, int h);
 cairo_t *PAINT_get_current_context();
+void *PAINT_get_current_device();
+bool PAINT_get_clip(int *x, int *y, int *w, int *h);
+void PAINT_apply_offset(int *x, int *y);
 
 #endif
