@@ -39,223 +39,8 @@
 #include "gprinter.h"
 #include "gmainwindow.h"
 
-//#define DEBUG_IM 1
 //#define DEBUG_ENTER_LEAVE 1
 
-/*************************************************************************
-
-gKey
-
-**************************************************************************/
-
-bool gKey::_valid = false;
-bool gKey::_no_input_method = false;
-GdkEventKey gKey::_event;
-GtkIMContext *gKey::_im_context = NULL;
-gControl *gKey::_im_control = NULL;
-char *_im_text = NULL;
-
-const char *gKey::text()
-{
-	if (!_valid) 
-		return 0;
-	else
-		return _event.string;
-}
-
-int gKey::code()
-{
-	if (!_valid)
-		return 0;
-	
-	int code = _event.keyval;
-	
-	if (code >= GDK_a && code <= GDK_z)
-		code += GDK_A - GDK_a;
-	else if (code == GDK_Alt_L || code == GDK_Alt_R || code == GDK_Control_L || code == GDK_Control_R 
-		       || code == GDK_Meta_L || code == GDK_Meta_R || code == GDK_Shift_L || code == GDK_Shift_R)
-		code = 0;
-	
-	return code;
-}
-
-int gKey::state()
-{
-	if (!_valid)
-		return 0;
-	else
-		return _event.state;
-}
-
-bool gKey::alt()
-{
-	return state() & GDK_MOD1_MASK || _event.keyval == GDK_Alt_L || _event.keyval == GDK_Alt_R;
-}
-
-bool gKey::control()
-{
-	return state() & GDK_CONTROL_MASK || _event.keyval == GDK_Control_L || _event.keyval == GDK_Control_R;
-}
-
-bool gKey::meta()
-{
-	return state() & GDK_MOD2_MASK || _event.keyval == GDK_Meta_L || _event.keyval == GDK_Meta_R;
-}
-
-bool gKey::normal()
-{
-	return (state() & 0xFF) != 0;
-}
-
-bool gKey::shift()
-{
-	return state() & GDK_SHIFT_MASK || _event.keyval == GDK_Shift_L || _event.keyval == GDK_Shift_R;
-}
-
-int gKey::fromString(char *str)
-{
-	char *lstr;
-	int key;
-	
-	if (!str || !*str)
-		return 0;
-	
-	lstr = g_ascii_strup(str, -1);
-	key = gdk_keyval_from_name(lstr);
-	g_free(lstr);
-	if (key) return key;
-	
-	lstr = g_ascii_strdown(str, -1);
-	key = gdk_keyval_from_name(lstr);
-	g_free(lstr);
-	if (key) return key;
-
-	key = gdk_keyval_from_name(str);
-	return key;
-}
-
-void gKey::disable()
-{
-	if (!_valid)
-		return;
-		
-	_valid = false;
-	_event.keyval = 0;
-	_event.state = 0;
-	g_free(_event.string);
-}
-
-bool gKey::enable(gControl *control, GdkEventKey *event)
-{
-	bool filter;
-	
-	//if (widget != _im_widget)
-	//	return true;
-	
-	if (_valid)
-		disable();
-		
-	_valid = true;
-	_event = *event;
-	
-	if (_event.type == GDK_KEY_PRESS && !_no_input_method && control == _im_control)
-	{
-		#if DEBUG_IM
-		fprintf(stderr, "gKey::enable: event->string = '%s'\n", event->string);
-		#endif
-		filter = gtk_im_context_filter_keypress(_im_context, &_event);
-		#if DEBUG_IM
-		fprintf(stderr, "gKey::enable: filter -> %d event->string = '%s'\n", filter, event->string);
-		#endif
-	}
-	else
-		filter = false;
-  
-  if (filter && _im_text)
-  {
-		_event.string = g_strdup(_im_text);
-		//_event.keyval = 0;
-		filter = false;
-  }
-  else
-  	_event.string = g_strdup(_event.string);
-  
-  if (!filter)
-  {
-		//#if DEBUG_IM
-  	//fprintf(stderr, "gKey::enable: gtk_im_context_reset\n");
-		//#endif
-  	//gtk_im_context_reset(_im_context);
-  	if (_im_text)
-  	{
-  		g_free(_im_text);
-  		_im_text = NULL;
-  	}
-  }
-  
-  //fprintf(stderr, "gKey::enable: --> %d\n", filter);
-  return filter;
-}
-
-static void cb_im_commit(GtkIMContext *context, const gchar *str, gpointer pointer)
-{
-	#if DEBUG_IM
-	fprintf(stderr, "cb_im_commit: %s\n", str);
-	#endif
-	
-	if (_im_text)
-		g_free(_im_text);
-		
-	_im_text = g_strdup(str);
-}
-
-void gKey::init()
-{
-	_im_context = gtk_im_multicontext_new();
-  g_signal_connect (_im_context, "commit", G_CALLBACK(cb_im_commit), NULL);
-}
-
-void gKey::exit()
-{
-	disable();
-	if (_im_text)
-		g_free(_im_text);
-	g_object_unref(_im_context);
-}
-
-void gKey::setActiveControl(gControl *control)
-{
-	if (_im_control)
-	{
-		if (!_no_input_method)
-		{
-			#if DEBUG_IM
-			fprintf(stderr, "gtm_im_context_focus_out\n");
-			#endif
-	  	gtk_im_context_set_client_window (_im_context, 0);
-			gtk_im_context_focus_out(_im_context);
-		}
-		_im_control = NULL;
-		_no_input_method = false;
-	}
-	
-	if (control)
-	{
-		_im_control = control;
-		_no_input_method = control->noInputMethod();
-		
-		if (!_no_input_method)
-		{
-	  	gtk_im_context_set_client_window (_im_context, _im_control->widget->window);
-			gtk_im_context_focus_in(_im_context);
-			gtk_im_context_reset(_im_context);
-			#if DEBUG_IM
-			fprintf(stderr, "gtm_im_context_focus_in\n");
-			#endif
-		}		
-	}
-}
-
 /**************************************************************************
 	
 	Global event handler
@@ -266,10 +51,9 @@ static bool _focus_change = false;
 
 static bool check_button(gControl *w)
 {
-	return w && w->isVisible() && w->enabled();
+	return w && w->isVisible() && w->isEnabled();
 }
 
-#if 0
 static GtkWindowGroup *get_window_group(GtkWidget *widget)
 {
   GtkWidget *toplevel = NULL;
@@ -282,7 +66,6 @@ static GtkWindowGroup *get_window_group(GtkWidget *widget)
   else
     return gtk_window_get_group(NULL);
 }
-#endif
 
 /*static gboolean close_dialog(GtkButton *button)
 {
@@ -299,6 +82,7 @@ static bool raise_key_event_to_parent_window(gControl *control, int type)
 		win = control->parent()->window();
 		if (win->onKeyEvent && win->canRaise(win, type))
 		{
+			//fprintf(stderr, "onKeyEvent: %d %p %s\n", type, win, win->name());
 			if (win->onKeyEvent(win, type))
 				return true;
 		}
@@ -323,14 +107,80 @@ static bool check_crossing_event(GdkEvent *event)
 		return false;
 }
 
+static gControl *find_child(gControl *control, int rx, int ry, gControl *button_grab = NULL)
+{
+	gContainer *cont;
+	gControl *child;
+	int x, y;
+	
+	//fprintf(stderr, "find_child: %s ", control->name());
+	
+	if (gApplication::_control_grab)
+		return gApplication::_control_grab;
+	if (button_grab)
+		return button_grab;
+	
+	control = control->topLevel();
+	
+	while (control->isContainer())
+	{
+		control->getScreenPos(&x, &y);
+		cont = (gContainer *)control;
+		child = cont->find(rx - x, ry - y);
+		if (!child)
+			break;
+		control = child;
+	}
+
+	//fprintf(stderr, "-> %s\n", control->name());
+	
+	return control;
+}
+
+void gApplication::checkHoveredControl(gControl *control)
+{
+	if (gApplication::_enter != control)
+	{
+		#if DEBUG_ENTER_LEAVE
+		fprintf(stderr, "checkHoveredControl: %s\n", control->name());
+		#endif
+	
+		gControl *leave = gApplication::_enter;
+		
+		while (leave && leave != control && !leave->isAncestorOf(control))
+		{
+			#if DEBUG_ENTER_LEAVE
+			fprintf(stderr, "checkHoveredControl: leave: %s\n", leave->name());
+			#endif
+			leave->emitLeaveEvent();
+			leave = leave->parent();
+		}
+		
+		#if DEBUG_ENTER_LEAVE
+		fprintf(stderr, "checkHoveredControl: _enter <- %s\n", control ? control->name() : "ø");
+		#endif
+		gApplication::_enter = control;
+		
+		if (control)
+		{
+			#if DEBUG_ENTER_LEAVE
+			fprintf(stderr, "checkHoveredControl: enter: %s\n", control->name());
+			#endif
+			control->emitEnterEvent();
+		}
+	}
+}
+
 static void gambas_handle_event(GdkEvent *event)
 {
   GtkWidget *widget;
   GtkWidget *grab;
-	gControl *control;
-	int x, y, xc, yc;
+	gControl *control, *save_control;
+	gControl *button_grab;
+	int x, y, xs, ys, xc, yc;
 	bool cancel;
 	int type;
+	bool parent_got_it;
 	
 	if (gApplication::_fix_printer_dialog)
 	{
@@ -372,12 +222,51 @@ static void gambas_handle_event(GdkEvent *event)
 	
 	if (!((event->type >= GDK_MOTION_NOTIFY && event->type <= GDK_FOCUS_CHANGE) || event->type == GDK_SCROLL))
 		goto __HANDLE_EVENT;
-	
+
 	widget = gtk_get_event_widget(event);
 	if (!widget)
 		goto __HANDLE_EVENT;
 	
-	grab = gtk_grab_get_current();
+	button_grab = gApplication::_button_grab;
+	if (event->type == GDK_BUTTON_RELEASE)
+		gApplication::_button_grab = NULL;
+	
+	if (gApplication::_control_grab)
+	{
+		control = gApplication::_control_grab;
+		widget = control->border;
+		//fprintf(stderr, "[1] _control_grab: %s -> widget = %p\n", control->name(), widget);
+		goto __FOUND_WIDGET;
+	}
+
+	if (gMenu::currentPopup())
+	{
+		grab = GTK_WIDGET(gMenu::currentPopup()->child);
+		//fprintf(stderr, "[2] popup menu: grab = %p\n", grab);
+		if (get_window_group(grab) != get_window_group(widget) && (event->type == GDK_ENTER_NOTIFY || event->type == GDK_LEAVE_NOTIFY))
+			return;
+	}
+	else
+	{
+		//fprintf(stderr, "[3] popup: grab = %p / %p\n", gApplication::_popup_grab, grab);
+		grab = gtk_window_group_get_current_grab(get_window_group(widget));
+		if (!grab)
+			grab = gApplication::_popup_grab;
+		//fprintf(stderr, "[4] grab = %p\n", grab);
+		//fprintf(stderr, "search grab for widget %p -> group = %p -> grab = %p WINDOW = %d\n", widget, get_window_group(widget), grab, GTK_IS_WINDOW(grab));
+		//if (grab && grab != widget && !GTK_IS_WINDOW(grab))
+		//	goto __HANDLE_EVENT;
+
+		//if (!grab && gApplication::_popup_grab)
+		//	grab = gApplication::_popup_grab;
+	}
+		//gdk_window_get_user_data(gApplication::_popup_grab_window, (gpointer *)&grab);
+	
+	//if (grab && !gApplication::_popup_grab && !gApplication::_button_grab)
+	//	goto __HANDLE_EVENT;
+		
+	//if (event->type == GDK_BUTTON_RELEASE)
+	//	fprintf(stderr, "GDK_BUTTON_RELEASE #2\n");
 	
 	if (event->type == GDK_FOCUS_CHANGE)
 	{
@@ -393,7 +282,10 @@ static void gambas_handle_event(GdkEvent *event)
 			if (control)
 				gApplication::setActiveControl(control, event->focus_change.in);
 			else if (event->focus_change.in);
+			{
+				//fprintf(stderr, "GDK_FOCUS_CHANGE: setActiveWindow(NULL)\n");
 				gMainWindow::setActiveWindow(NULL);
+			}
 		}
 
 		if (event->focus_change.in && grab && widget != grab && !gtk_widget_is_ancestor(widget, grab))
@@ -407,30 +299,48 @@ static void gambas_handle_event(GdkEvent *event)
 		goto __HANDLE_EVENT;
 	}
 	
-	if (grab)
+	/*if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE || event->type == GDK_MOTION_NOTIFY)
 	{
-		if (widget != grab && !gtk_widget_is_ancestor(widget, grab))
-			widget = grab;
+		fprintf(stderr, "widget = %p grab = %p _popup_grab = %p _button_grab = %p\n", widget, grab, gApplication::_popup_grab, gApplication::_button_grab);
+		//fprintf(stderr, "widget = %p (%p) grab = %p (%p)\n", widget, widget ? g_object_get_data(G_OBJECT(widget), "gambas-control") : 0, 
+		//				grab, grab ? g_object_get_data(G_OBJECT(grab), "gambas-control") : 0);
+	}*/
+	
+	//fprintf(stderr, "grab = %p widget = %p %d\n", grab, widget, grab && !gtk_widget_is_ancestor(widget, grab));
+	
+	if (grab && widget != grab && !gtk_widget_is_ancestor(widget, grab))
+	{
+		//fprintf(stderr, "-> widget = grab\n");
+		widget = grab;
 	}
 	
-	//real = true;
 	while (widget)
 	{
 		control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
 		if (control)
 			break;
 		widget = widget->parent;
-		//real = false;
 	}
-
+	
 	/*if (event->type == GDK_BUTTON_PRESS)
 		fprintf(stderr, "GDK_BUTTON_PRESS: %p %s\n", widget, control ? control->name() : NULL);
-	else if (event->type == GDK_KEY_PRESS)
-		fprintf(stderr, "GDK_KEY_PRESS: %p %s (%s)\n", widget, control ? control->name() : NULL, gApplication::activeControl() ? gApplication::activeControl()->name() : NULL);*/
+	else if (event->type == GDK_BUTTON_RELEASE)
+		fprintf(stderr, "GDK_BUTTON_RELEASE: %p %s\n", widget, control ? control->name() : NULL);*/
+	
+	/*else if (event->type == GDK_KEY_PRESS)
+		fprintf(stderr, "GDK_KEY_PRESS: %p %s (%s)\n", widget, control ? control->name() : NULL, gApplication::activeControl() ? gApplication::activeControl()->name() : NULL);
+	*/
 	
 	if (!widget || !control)
 		goto __HANDLE_EVENT;
 	
+	//if (event->type == GDK_BUTTON_RELEASE)
+	//	fprintf(stderr, "GDK_BUTTON_RELEASE #3\n");
+	
+__FOUND_WIDGET:
+	
+	//fprintf(stderr, "control = %p %s\n", control, control->name());
+	
 	/*switch ((int)event->type)
 	{
 		case GDK_ENTER_NOTIFY:
@@ -472,6 +382,10 @@ static void gambas_handle_event(GdkEvent *event)
 	{
 		case GDK_ENTER_NOTIFY:
 			
+			control = find_child(control, (int)event->button.x_root, (int)event->button.y_root);
+			
+			//fprintf(stderr, "GDK_ENTER_NOTIFY: %s (%s)\n", control->name(), gApplication::_enter ? gApplication::_enter->name() : "ø");
+			
 			if (gApplication::_leave == control)
 			{
 				#if DEBUG_ENTER_LEAVE
@@ -486,14 +400,18 @@ static void gambas_handle_event(GdkEvent *event)
 					#if DEBUG_ENTER_LEAVE
 					fprintf(stderr, "enter: %s\n", control->name());
 					#endif
-					gApplication::_enter = control;
-					control->emitEnterEvent();
+					gApplication::checkHoveredControl(control);
 				}
 			}
+
 			break;
 		
 		case GDK_LEAVE_NOTIFY:
 			
+			//fprintf(stderr, "GDK_LEAVE_NOTIFY: %s\n", control->name());
+			
+			//control = find_child(control, (int)event->button.x_root, (int)event->button.y_root);
+			
 			if (gdk_events_pending() && gApplication::_leave == NULL)
 			{
 				if (check_crossing_event(event))
@@ -520,14 +438,19 @@ static void gambas_handle_event(GdkEvent *event)
 					control->emitLeaveEvent();
 				}
 			}
+			
+			//if (widget != control->border && widget != control->widget)
+			//	goto __RETURN;
+			
 			break;
 			
 		case GDK_BUTTON_PRESS:
 		case GDK_2BUTTON_PRESS:
 		case GDK_BUTTON_RELEASE:
 		{
+			save_control = control = find_child(control, (int)event->button.x_root, (int)event->button.y_root, button_grab);
+
 			bool menu = false;
-			gControl *save_control = control;
 			
 			switch ((int)event->type)
 			{
@@ -536,6 +459,19 @@ static void gambas_handle_event(GdkEvent *event)
 				default: type = gEvent_MouseRelease; break;
 			}
 			
+			if (event->type != GDK_BUTTON_RELEASE)
+			{
+				if (control->canFocus() && !control->hasFocus())
+					control->setFocus();
+				if (!control->_no_auto_grab)
+				{
+					//fprintf(stderr, "grab %s\n", control->name());
+					gApplication::_button_grab = control;
+					//gtk_grab_add(control->border);
+				}
+			}
+
+			
 		__BUTTON_TRY_PROXY:
 		
 			cancel = false;
@@ -545,16 +481,19 @@ static void gambas_handle_event(GdkEvent *event)
 				if (control->canRaise(control, type))
 				{
 					control->getScreenPos(&xc, &yc);
-					x = (int)event->button.x_root - xc;
-					y = (int)event->button.y_root - yc;
+					xs = (int)event->button.x_root;
+					ys = (int)event->button.y_root;
+					x = xs - xc;
+					y = ys - yc;
 					
 					gMouse::validate();
-					gMouse::setStart(x, y);
-					gMouse::setMouse(x, y, event->button.button, event->button.state);
+					gMouse::setEvent(event);
 					//gMouse::setValid(1,(int)event->x,(int)event->y,event->button,event->state,data->screenX(),data->screenY());
+					gMouse::setMouse(x, y, xs, ys, event->button.button, event->button.state);
 					switch ((int)event->type)
 					{
 						case GDK_BUTTON_PRESS: 
+							gMouse::setStart(x, y);
 							cancel = control->onMouseEvent(control, gEvent_MousePress);
 							break;
 						
@@ -577,8 +516,10 @@ static void gambas_handle_event(GdkEvent *event)
 				if (win->isPopup())
 				{
 					control->getScreenPos(&xc, &yc);
-					x = (int)event->button.x_root - xc;
-					y = (int)event->button.y_root - yc;
+					xs = (int)event->button.x_root;
+					ys = (int)event->button.y_root;
+					x = xs - xc;
+					y = ys - yc;
 				
 					if (x < 0 || y < 0 || x >= win->width() || y >= win->height())
 						win->close();
@@ -597,6 +538,7 @@ static void gambas_handle_event(GdkEvent *event)
 				if (control->_proxy_for)
 				{
 					control = control->_proxy_for;
+					//fprintf(stderr, "PRESS: try %s\n", control->name());
 					goto __BUTTON_TRY_PROXY;
 				}
 			}
@@ -613,48 +555,73 @@ static void gambas_handle_event(GdkEvent *event)
 			}
 			
 			if (cancel)
-				return;
+				goto __RETURN;
+			
+			if (widget != save_control->border && widget != save_control->widget)
+			{
+				//fprintf(stderr, "widget = %p, control = %p %p %s\n", widget, save_control->border, save_control->widget, save_control->name());
+				goto __RETURN;
+			}
 			
 			break;
 		}
 			
 		case GDK_MOTION_NOTIFY:
 
+			save_control = control = find_child(control, (int)event->motion.x_root, (int)event->motion.y_root, button_grab);
+			
+			//fprintf(stderr, "GDK_MOTION_NOTIFY: %s\n", control->name());
+			
+			gApplication::checkHoveredControl(control);
+			
 		__MOTION_TRY_PROXY:
 		
 			if (control->onMouseEvent && (control->canRaise(control, gEvent_MouseMove) || control->canRaise(control, gEvent_MouseDrag))
 					&& (control->isTracking() || (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))))
 			{
 				control->getScreenPos(&xc, &yc);
-				x = (int)event->motion.x_root - xc;
-				y = (int)event->motion.y_root - yc;
+				xs = (int)event->motion.x_root;
+				ys = (int)event->motion.y_root;
+				x = xs - xc;
+				y = ys - yc;
 				
 				gMouse::validate();
-				gMouse::setMouse(x, y, 0, event->motion.state);
+				gMouse::setEvent(event);
+				gMouse::setMouse(x, y, xs, ys, 0, event->motion.state);
+				
+				//fprintf(stderr, "pressure = %g\n", gMouse::getAxis(GDK_AXIS_PRESSURE));
+				
 				cancel = control->onMouseEvent(control, gEvent_MouseMove);
+			
 				//if (data->acceptDrops() && gDrag::checkThreshold(data, gMouse::x(), gMouse::y(), gMouse::startX(), gMouse::startY()))
-				if ((event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) 
+				if (!cancel && (event->motion.state & (GDK_BUTTON1_MASK | GDK_BUTTON2_MASK | GDK_BUTTON3_MASK))
 						//&& (abs(gMouse::x() - gMouse::y()) + abs(gMouse::startX() - gMouse::startY())) > 8)
 						&& gDrag::checkThreshold(control, gMouse::x(), gMouse::y(), gMouse::startX(), gMouse::startY()))
 				{
-					control->onMouseEvent(control, gEvent_MouseDrag);
+					cancel = control->onMouseEvent(control, gEvent_MouseDrag);
 				}
 				gMouse::invalidate();
 				
 				if (cancel)
-					return;
+					goto __RETURN;
 			}
 
 			if (control->_proxy_for)
 			{
 				control = control->_proxy_for;
+				//fprintf(stderr, "MOVE: try %s\n", control->name());
 				goto __MOTION_TRY_PROXY;
 			}
 			
+			//if (widget != save_control->border && widget != save_control->widget)
+			//	goto __RETURN;
+			
 			break;
 			
 		case GDK_SCROLL:
 			
+			save_control = control = find_child(control, (int)event->scroll.x_root, (int)event->scroll.y_root);
+			
 		__SCROLL_TRY_PROXY:
 		
 			if (control->onMouseEvent && control->canRaise(control, gEvent_MouseWheel))
@@ -662,8 +629,10 @@ static void gambas_handle_event(GdkEvent *event)
 				int dt, ort;
 				
 				control->getScreenPos(&xc, &yc);
-				x = (int)event->scroll.x_root - xc;
-				y = (int)event->scroll.y_root - yc;
+				xs = (int)event->scroll.x_root;
+				ys = (int)event->scroll.y_root;
+				x = xs - xc;
+				y = ys - yc;
 
 				switch (event->scroll.direction)
 				{
@@ -674,14 +643,15 @@ static void gambas_handle_event(GdkEvent *event)
 				}
 				
 				gMouse::validate();
-				gMouse::setMouse(x, y, 0, event->scroll.state);
+				gMouse::setEvent(event);
+				gMouse::setMouse(x, y, xs, ys, 0, event->scroll.state);
 				gMouse::setWheel(dt, ort);
 				cancel = control->onMouseEvent(control, gEvent_MouseWheel);
 				gMouse::invalidate();
 			}
 
 			if (cancel)
-				return;
+				goto __RETURN;
 				
 			if (control->_proxy_for)
 			{
@@ -689,6 +659,9 @@ static void gambas_handle_event(GdkEvent *event)
 				goto __SCROLL_TRY_PROXY;
 			}
 			
+			if (widget != save_control->border && widget != save_control->widget)
+				goto __RETURN;
+			
 			break;
 
 		case GDK_KEY_PRESS:
@@ -701,6 +674,8 @@ static void gambas_handle_event(GdkEvent *event)
 			
 			type =  (event->type == GDK_KEY_PRESS) ? gEvent_KeyPress : gEvent_KeyRelease;
 			
+			parent_got_it = false;
+			
 			if (control)
 			{
 			__KEY_TRY_PROXY:
@@ -709,20 +684,28 @@ static void gambas_handle_event(GdkEvent *event)
 				
 				if (!gKey::enable(control, &event->key))
 				{
-					if (gApplication::onKeyEvent)
-						cancel = gApplication::onKeyEvent(type);
-					if (!cancel)
-						cancel = raise_key_event_to_parent_window(control, type);
+					if (!parent_got_it)
+					{
+						parent_got_it = true;
+						
+						if (gApplication::onKeyEvent)
+							cancel = gApplication::onKeyEvent(type);
+						
+						if (!cancel)
+							cancel = raise_key_event_to_parent_window(control, type);
+					}
+					
 					if (!cancel && control->onKeyEvent && control->canRaise(control, type)) 
 					{
 						//fprintf(stderr, "gEvent_KeyPress on %p %s\n", control, control->name());
+						//fprintf(stderr, "onKeyEvent: %p %d %p %s\n", event, type, control, control->name());
 						cancel = control->onKeyEvent(control, type);
 					}
 				}
 				gKey::disable();
 				
 				if (cancel)
-					return;
+					goto __RETURN;
 				
 				if (control->_proxy_for)
 				{
@@ -735,26 +718,28 @@ static void gambas_handle_event(GdkEvent *event)
 					if (control->_grab)
 					{
 						gApplication::exitLoop(control);
-						return;
+						goto __RETURN;
 					}
 					
 					if (check_button(win->_cancel))
 					{
+						win->_cancel->setFocus();
 						win->_cancel->animateClick(type == gEvent_KeyRelease);
-						return;
+						goto __RETURN;
 					}
 				}
 				else if (event->key.keyval == GDK_Return || event->key.keyval == GDK_KP_Enter)
 				{
 					if (check_button(win->_default))
 					{
+						win->_default->setFocus();
 						win->_default->animateClick(type == gEvent_KeyRelease);
-						return;
+						goto __RETURN;
 					}
 				}
 				
 				if (control->_grab)
-					return;
+					goto __RETURN;
 			}
 
 			break;
@@ -764,6 +749,12 @@ static void gambas_handle_event(GdkEvent *event)
 __HANDLE_EVENT:
 
 	gtk_main_do_event(event);
+	
+__RETURN:
+	
+	(void)0;
+	//if (ungrab)
+	//	gtk_grab_remove(widget);
 }
 
 
@@ -779,32 +770,46 @@ int appEvents;
 bool gApplication::_busy = false;
 char *gApplication::_title = NULL;
 int gApplication::_in_popup = 0;
-GdkWindow *gApplication::_popup_grab_window = NULL;
+GtkWidget *gApplication::_popup_grab = NULL;
 int gApplication::_loopLevel = 0;
 void *gApplication::_loop_owner = 0;
 GtkWindowGroup *gApplication::_group = NULL;
 gControl *gApplication::_enter = NULL;
 gControl *gApplication::_leave = NULL;
+gControl *gApplication::_ignore_until_next_enter = NULL;
+gControl *gApplication::_button_grab = NULL;
+gControl *gApplication::_control_grab = NULL;
 gControl *gApplication::_active_control = NULL;
 gControl *gApplication::_old_active_control = NULL;
 bool (*gApplication::onKeyEvent)(int) = NULL;
 guint32 gApplication::_event_time = 0;
 bool gApplication::_close_next_window = false;
 bool gApplication::_fix_printer_dialog = false;
+gMainWindow *gApplication::_main_window = NULL;
+void (*gApplication::onEnterEventLoop)();
+void (*gApplication::onLeaveEventLoop)();
+
 
 void gApplication::grabPopup()
 {
-	if (!_popup_grab_window)
+	//fprintf(stderr, "grabPopup: %p\n", _popup_grab);
+	
+	if (!_popup_grab)
 		return;
 	
-  int ret = gdk_pointer_grab(_popup_grab_window, TRUE,
+	//gtk_grab_add(_popup_grab);
+	//return;
+	
+	GdkWindow *win = _popup_grab->window;
+		
+  int ret = gdk_pointer_grab(win, TRUE,
 			 (GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
 			 GDK_POINTER_MOTION_MASK),
 			 NULL, NULL, GDK_CURRENT_TIME);
 	
 	if (ret == GDK_GRAB_SUCCESS)
 	{
-		ret = gdk_keyboard_grab(_popup_grab_window, TRUE, GDK_CURRENT_TIME);
+		ret = gdk_keyboard_grab(win, TRUE, GDK_CURRENT_TIME);
 		
 		if (ret == GDK_GRAB_SUCCESS)
 			return;
@@ -817,7 +822,9 @@ void gApplication::grabPopup()
 
 void gApplication::ungrabPopup()
 {
-	_popup_grab_window = NULL;
+	//fprintf(stderr, "ungrabPopup: %p\n", _popup_grab);
+	//gtk_grab_remove(_popup_grab);
+	_popup_grab = NULL;
 	gdk_pointer_ungrab(GDK_CURRENT_TIME);
 	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
 }
@@ -865,16 +872,26 @@ bool gApplication::allEvents()
 	return true;
 }
 
-void gApplication::init(int *argc,char ***argv)
+static void do_nothing()
+{
+}
+
+void gApplication::init(int *argc, char ***argv)
 {
 	appEvents=0;
 	
-	gtk_init(argc,argv);
+	gtk_init(argc, argv);
  
 	gdk_event_handler_set((GdkEventFunc)gambas_handle_event, NULL, NULL);
 	
 	gClipboard::init();
 	gKey::init();
+	
+	onEnterEventLoop = do_nothing;
+	onLeaveEventLoop = do_nothing;
+
+	_group = gtk_window_group_new();
+	
 	_loop_owner = 0;
 }
 
@@ -911,25 +928,14 @@ int gApplication::controlCount()
 
 gControl* gApplication::controlItem(GtkWidget *wid)
 {
-	GList *iter;
+	gControl *control;
 	
-	if (!wid) return NULL;
-
-	if (gControl::controlList())
+	while (wid)
 	{
-		iter=g_list_first(gControl::controlList());
-		
-		while (iter)
-		{
-			if (((gControl*)iter->data)->border == wid )
-				return (gControl*)iter->data;
-				
-			if (((gControl*)iter->data)->widget == wid )
-				return (gControl*)iter->data;
-		
-			iter=iter->next;
-		}
-		
+		control = (gControl *)g_object_get_data(G_OBJECT(wid), "gambas-control");
+		if (control)
+			return control;
+		wid = gtk_widget_get_parent(wid);
 	}
 	
 	return NULL;
@@ -950,24 +956,24 @@ void gApplication::setBusy(bool b)
 	GList *iter;
 	gControl *control;
 
-  if (b == _busy)
-    return;
+	if (b == _busy)
+		return;
 
-  _busy = b;
-  
-  iter = g_list_first(gControl::controlList());
-  
-  while (iter)
-  {
-    control = (gControl *)iter->data;
-    
-    if (control->mustUpdateCursor())
-    	control->setMouse(control->mouse());
-    
-    iter = g_list_next(iter);
-  }
-  
-  MAIN_do_iteration_just_events();
+	_busy = b;
+
+	iter = g_list_first(gControl::controlList());
+
+	while (iter)
+	{
+		control = (gControl *)iter->data;
+
+		if (control->mustUpdateCursor())
+			control->setMouse(control->mouse());
+
+		iter = g_list_next(iter);
+	}
+
+	MAIN_do_iteration_just_events();
 }
 
 static bool _dirty = false;
@@ -1033,67 +1039,76 @@ void gApplication::exitGroup(GtkWindowGroup *oldGroup)
 	_group = oldGroup;	
 }
 
-void gApplication::enterLoop(void *owner, bool showIt)
+void gApplication::enterLoop(void *owner, bool showIt, GtkWindow *modal)
 {
 	void *old_owner = _loop_owner;
 	int l = _loopLevel;
-	GtkWindowGroup *oldGroup;
-	
-	oldGroup = enterGroup();
+	//GtkWindowGroup *oldGroup;
 	
 	if (showIt) ((gControl *)owner)->show();
 
+	//oldGroup = enterGroup();
+	
 	_loopLevel++;
 	_loop_owner = owner;
 	
+	(*onEnterEventLoop)();
 	do
 	{
 		MAIN_do_iteration(false);
 	}
 	while (_loopLevel > l);
+	(*onLeaveEventLoop)();
 	
 	_loop_owner = old_owner;
 
-	exitGroup(oldGroup);
+	//exitGroup(oldGroup);
 }
 
 void gApplication::enterPopup(gMainWindow *owner)
 {
 	void *old_owner = _loop_owner;
 	int l = _loopLevel;
-	GtkWindowGroup *oldGroup;
+	//GtkWindowGroup *oldGroup;
 	GtkWindow *window = GTK_WINDOW(owner->border);
+	GtkWidget *old_popup_grab;
 	
 	_in_popup++;
 	
-	oldGroup = enterGroup();
+	// Remove possible current button grab
+	_button_grab = NULL;
+//
+	//oldGroup = enterGroup();
 	
 	gtk_window_set_modal(window, true);
 	gdk_window_set_override_redirect(owner->border->window, true);
 	owner->show();
 	
+	old_popup_grab = _popup_grab;
+	_popup_grab = owner->border;
+	
 	if (_in_popup == 1)
-	{
-		_popup_grab_window = owner->border->window;
 		gApplication::grabPopup();
-	}
 	
 	_loopLevel++;
 	_loop_owner = owner;
 	
+	(*onEnterEventLoop)();
 	do
 	{
 		MAIN_do_iteration(false);
 	}
 	while (_loopLevel > l);
+	(*onLeaveEventLoop)();
 	
 	gApplication::ungrabPopup();
+	_popup_grab = old_popup_grab;
 	
 	_loop_owner = old_owner;
 
 	gdk_window_set_override_redirect(owner->border->window, false);
 	gtk_window_set_modal(window, false);
-	exitGroup(oldGroup);
+	//exitGroup(oldGroup);
 	
 	_in_popup--;
 }
@@ -1140,7 +1155,7 @@ void gApplication::updateLastEventTime(GdkEvent *e)
 
 static void post_focus_change(void *)
 {
-	gControl *current, *control;
+	gControl *current, *control, *next;
 	
 	//fprintf(stderr, "post_focus_change\n");
 	
@@ -1153,20 +1168,27 @@ static void post_focus_change(void *)
 		control = gApplication::_old_active_control;
 		while (control)
 		{
+			next = control->_proxy_for;
 			if (control->onFocusEvent)
 				control->onFocusEvent(control, gEvent_FocusOut);
-			control = control->_proxy_for;
+			control = next;
 		}
 		
+		current = gApplication::activeControl();
+		if (current == gApplication::_old_active_control)
+			break;
+		
 		gApplication::_old_active_control = current;
+		//fprintf(stderr, "post_focus_change: setActiveWindow\n");
 		gMainWindow::setActiveWindow(current);
 		
-		control = current;
+		control = gApplication::activeControl();
 		while (control)
 		{
+			next = control->_proxy_for;
 			if (control->onFocusEvent)
 				control->onFocusEvent(control, gEvent_FocusIn);
-			control = control->_proxy_for;
+			control = next;
 		}
 	}
 	
@@ -1196,21 +1218,25 @@ void gApplication::setActiveControl(gControl *control, bool on)
 
 int gApplication::getScrollbarSize()
 {
-	gint focus_line_width;
-	gint focus_padding;
+	//gint focus_line_width;
+	//gint focus_padding;
 	gint trough_border;
 	gint slider_width;
 	
-  gtk_style_get(gt_get_style("GtkRange", GTK_TYPE_RANGE), GTK_TYPE_RANGE,
+	if (g_type_from_name("OsBar"))
+	{
+		char *env = getenv("LIBOVERLAY_SCROLLBAR");
+		if (!env || *env != '0')
+			return 1;
+	}
+	
+  gtk_style_get(gt_get_style("GtkScrollbar", GTK_TYPE_SCROLLBAR), GTK_TYPE_SCROLLBAR,
 		"slider-width", &slider_width,
 		"trough-border", &trough_border,
+		/*"focus-line-width", &focus_line_width,
+		"focus-padding", &focus_padding,*/
 		(char *)NULL);
 	
-	gtk_style_get(gt_get_style("GtkWidget", GTK_TYPE_WIDGET), GTK_TYPE_WIDGET,
-		"focus-line-width", &focus_line_width,
-		"focus-padding", &focus_padding,
-		(char *)NULL);
-
 	return (trough_border) * 2 + slider_width;
 }
 
@@ -1227,15 +1253,26 @@ int gApplication::getScrollbarSpacing()
 
 int gApplication::getFrameWidth()
 {
-	return 2;
+	if (strcmp(getStyleName(), "oxygen-gtk") == 0)
+		return 3;
+	else
+		return 2;
 }
 
-int gApplication::getTextBoxFrameWidth()
+int gApplication::getInnerWidth()
+{
+	if (strcmp(getStyleName(), "oxygen-gtk") == 0)
+		return 2;
+	else
+		return 0;
+}
+
+void gApplication::getBoxFrame(int *w, int *h)
 {
 	GtkStyle *style;
 	gint focus_width;
 	gboolean interior_focus;
-	int v;
+	int inner;
 
 	style = gt_get_style("GtkEntry", GTK_TYPE_ENTRY);
 	
@@ -1244,9 +1281,27 @@ int gApplication::getTextBoxFrameWidth()
 		"interior-focus", &interior_focus,
 		(char *)NULL);
 	
-	v = MAX(style->xthickness, style->ythickness);
-	if (!interior_focus)
-		v += focus_width;
+	*w = style->xthickness;
+	*h = style->ythickness;
 	
-	return v;
+	if (!interior_focus)
+	{
+		*w += focus_width;
+		*h += focus_width;
+	}
+	
+	inner = getInnerWidth();
+	*w += inner;
+	*h += inner;
 }
+
+char *gApplication::getStyleName()
+{
+  GtkSettings *settings;
+	char *theme;
+
+  settings = gtk_settings_get_default();
+  g_object_get(settings, "gtk-theme-name", &theme, (char *)NULL);
+	return theme;
+}
+
