@@ -2,7 +2,7 @@
 
   gapplication.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ public:
 
 	static void setDirty();
 	static int loopLevel() { return _loopLevel; }
-	static void enterLoop(void *owner, bool showIt = false);
+	static void enterLoop(void *owner, bool showIt = false, GtkWindow *modal = NULL);
 	static void enterPopup(gMainWindow *owner);
 	static void exitLoop(void *owner);
 	static bool hasLoop(void *owner) { return _loop_owner == owner; }
@@ -72,28 +72,41 @@ public:
 	static int getScrollbarSize();
 	static int getScrollbarSpacing();
 	static int getFrameWidth();
-	static int getTextBoxFrameWidth();
+	static int getInnerWidth();
+	static void getBoxFrame(int *w, int *h);
+	static char *getStyleName();
 	
 	static void grabPopup();
 	static void ungrabPopup();
+	
+	static void setMainWindow(gMainWindow *win) { _main_window = win; }
+	static gMainWindow *mainWindow() { return _main_window; }
+	
+	static void checkHoveredControl(gControl *control);
 
 	//"Private"
 	static bool _busy;
 	static char *_title;
 	static int _loopLevel;
 	static int _in_popup;
-	static GdkWindow *_popup_grab_window;
+	static GtkWidget *_popup_grab;
 	static void *_loop_owner;
 	static GtkWindowGroup *_group;
 	static GtkWindowGroup *currentGroup();
 	//static void dispatchEnterLeave(gControl *enter);
 	static gControl *_enter;
 	static gControl *_leave;
+	static gControl *_ignore_until_next_enter;
 	static gControl *_active_control;
 	static gControl *_old_active_control;
+	static gControl *_button_grab;
+	static gControl *_control_grab;
 	static guint32 _event_time;
+	static gMainWindow *_main_window;
 	static bool _close_next_window;
 	static bool _fix_printer_dialog;
+	static void (*onEnterEventLoop)();
+	static void (*onLeaveEventLoop)();
 };
 
 #endif
