@@ -138,7 +138,7 @@ void gComboBox::create(bool readOnly)
 	lock();
 	
 	if (first)
-		border = gtk_event_box_new();
+		border = gtk_alignment_new(0, 0, 1, 1); //gtk_event_box_new();
 	else
 		ind = index();
 	
@@ -154,12 +154,13 @@ void gComboBox::create(bool readOnly)
 	{
 		widget = gtk_combo_box_new_with_model(GTK_TREE_MODEL(tree->store));
 		entry = NULL;
+		use_base = false;
 
 		cell = gtk_cell_renderer_text_new ();
 		g_object_ref_sink(cell);
 		gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(widget), cell, true);
 		//gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(widget), cell, "text", 0, (void *)NULL);
-		g_object_set(cell, "ypad", 0, (void *)NULL);
+		//g_object_set(cell, "ypad", 0, (void *)NULL);
 		gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(widget), cell, (GtkCellLayoutDataFunc)combo_cell_text, (gpointer)tree, NULL);
 	}
 	else
@@ -168,6 +169,7 @@ void gComboBox::create(bool readOnly)
 		
 		widget = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(tree->store), 0);
 		entry = gtk_bin_get_child(GTK_BIN(widget));
+		use_base = true;
 
 		g_signal_handler_disconnect(widget, g_signal_handler_find(widget, G_SIGNAL_MATCH_ID, g_signal_lookup("changed", G_OBJECT_TYPE(widget)), 0, 0, 0, 0));
 		
@@ -176,7 +178,7 @@ void gComboBox::create(bool readOnly)
 		g_list_free(cells);
 		g_object_ref(cell);
 		//gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(widget), cell, "text", 0, (void *)NULL);
-		g_object_set(cell, "ypad", 0, (void *)NULL);
+		//g_object_set(cell, "ypad", 0, (void *)NULL);
 		gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(widget), cell, (GtkCellLayoutDataFunc)combo_cell_text, (gpointer)tree, NULL);
 	}
 	
@@ -241,10 +243,10 @@ gComboBox::gComboBox(gContainer *parent) : gTextBox(parent, true)
 	
 	g_typ = Type_gComboBox;
 	
-	tree = new gTree(NULL);
+	tree = new gTree();
 	tree->addColumn();
 	//tree->addColumn();
-	tree->setHeaders(false);
+	//tree->setHeaders(false);
 	
 	create(false);
 }
@@ -465,7 +467,7 @@ void gComboBox::add(const char *text, int pos)
 
 	//g_signal_lookup("rowGTK_TYPE_COMBO_BOX);
 
-	row = tree->addRow(key, NULL, after, true);
+	row = tree->addRow(key, after, true);
 	if (row) 
 	{
 		cell = row->get(0);
@@ -584,3 +586,16 @@ void gComboBox::updateFocusHandler()
 	g_signal_connect(G_OBJECT(button), "focus-in-event", G_CALLBACK(button_focus_in), (gpointer)this);
 	g_signal_connect(G_OBJECT(button), "focus-out-event", G_CALLBACK(button_focus_out), (gpointer)this);
 }
+
+bool gComboBox::hasBorder() const
+{
+	gboolean v;
+	
+	g_object_get(G_OBJECT(widget), "has-frame", &v, 	(void *)NULL);
+	return v;
+}
+
+void gComboBox::setBorder(bool v)
+{
+	g_object_set(G_OBJECT(widget), "has-frame", v, 	(void *)NULL);
+}
