@@ -30,7 +30,6 @@
 #include "gapplication.h"
 #include "gdesktop.h"
 #include "gmainwindow.h"
-#include "gsplitter.h"
 #include "gcontainer.h"
 
 static gControl* get_next_child_widget (gContainer *gtk_control, int *gtk_list, int gtk_count)
@@ -105,7 +104,7 @@ static void resize_container(gControl *cont, int w, int h)
 #define GET_WIDGET_H(_widget)  (((gControl*)_widget)->height())
 #define MOVE_WIDGET(_object, _widget, _x, _y)  (((gControl*)_widget)->move( _x, _y))
 #define RESIZE_WIDGET(_object, _widget, _w, _h)  (((gControl*)_widget)->resize( _w, _h))
-#define MOVE_RESIZE_WIDGET(_object, _widget, _x, _y, _w, _h) (((gControl*)_widget)->move( _x, _y, _w, _h))
+#define MOVE_RESIZE_WIDGET(_object, _widget, _x, _y, _w, _h) (((gControl*)_widget)->moveResize( _x, _y, _w, _h))
 #define RESIZE_CONTAINER(_widget, _cont, _w, _h)  resize_container((gControl *)(_cont), _w, _h) 
 
 #define INIT_CHECK_CHILDREN_LIST(_widget) \
@@ -137,8 +136,15 @@ int gContainer::_arrangement_level = 0;
 
 void gContainer::performArrange()
 {
+	if (_no_arrangement)
+	{
+		_did_arrangement = true;
+		return;
+	}
+	
 	if (!gApplication::allEvents()) return;
 
+	_did_arrangement = false;
 	//if (!CAN_ARRANGE(this))
 	//	return;
 	
@@ -161,6 +167,7 @@ void gContainer::initialize()
 	_client_y = -1;
 	_client_w = 0;
 	_client_h = 0;
+	_no_arrangement = 0;
 	//onInsert = NULL;
 	
 	arrangement.mode = 0;
@@ -405,9 +412,9 @@ int gContainer::clientWidth()
 		{
 			//g_debug("clientWidth: %s: %d", name(), width());
 			GtkAllocation a = { x(), y(), width(), height() };
-			//gt_disable_warnings(true);
+			gt_disable_warnings(true);
 			gtk_widget_size_allocate(widget, &a);
-			//gt_disable_warnings(false);
+			gt_disable_warnings(false);
 		}
 		//g_debug("ClientWidth: %s -> %d", this->name(), cont->allocation.width);
 		if (cont->allocation.width > 0)
@@ -476,8 +483,8 @@ void gContainer::insert(gControl *child, bool realize)
 		gtk_widget_show_all(child->border);
 	}
     
-	if (hasBackground()) child->setBackground();
-	if (hasForeground()) child->setForeground();
+	if (hasBackground() && !child->_bg_set) child->setBackground();
+	if (hasForeground() && !child->_fg_set) child->setForeground();
   if (hasFont()) child->setFont(font());
 }
 
@@ -493,10 +500,25 @@ gControl *gContainer::find(int x, int y)
 	int i;
 	gControl *ch;
 	
-	for (i = 0; i < childCount(); i++)
+	x -= clientX();
+	y -= clientY();
+	
+	if (gApplication::_button_grab != this)
+	{
+		if (x < 0 || y < 0 || x >= clientWidth() || y >= clientHeight())
+			return NULL;
+	}
+	
+	if (_scroll)
+	{
+		x += scrollX();
+		y += scrollY();
+	}
+	
+	for (i = childCount() - 1; i >= 0; i--)
 	{
 		ch = child(i);
-		if (x >= ch->left() && y >= ch->top() && x < (ch->left() + ch->width()) && y < (ch->top() + ch->height()))
+		if (ch->isVisible() && x >= ch->left() && y >= ch->top() && x < (ch->left() + ch->width()) && y < (ch->top() + ch->height()))
 			return ch;
 	}
 	
@@ -513,7 +535,7 @@ void gContainer::setBackground(gColor color)
 	
 	for (i = 0; i < childCount(); i++)
 	{
-		ch = child(i);
+		ch = gContainer::child(i);
 		if (!ch->_bg_set)
 			ch->setBackground();
 	}	
@@ -528,8 +550,8 @@ void gContainer::setForeground(gColor color)
 	
 	for (i = 0; i < childCount(); i++)
 	{
-		ch = child(i);
-		if (!ch->_bg_set)
+		ch = gContainer::child(i);
+		if (!ch->_fg_set)
 			ch->setForeground();
 	}	
 }
@@ -577,8 +599,15 @@ void gContainer::resize(int w, int h)
 
 void gContainer::setVisible(bool vl)
 {
-	bool arr = vl && !isVisible();
+	bool arr;
+	
+	if (vl == isVisible())
+		return;
+	
+	arr = vl && !isVisible();
+	
 	gControl::setVisible(vl);
+	
 	if (arr)
 		performArrange();
 }
@@ -616,18 +645,21 @@ void gContainer::setFont(gFont *ft)
 	{
 		ch = child(i);
 		if (!ch->_font_set)
+		{
 			ch->setFont(ch->font());
+			ch->_font_set = false;
+		}
 	}	
 }
 
 void gContainer::moveChild(gControl *child, int x, int y)
 {
-	GtkWidget *parent = getContainer();
+	GtkWidget *cont = gtk_widget_get_parent(child->border); //getContainer();
 	
-	if (GTK_IS_LAYOUT(parent))
-		gtk_layout_move(GTK_LAYOUT(parent), child->border, x, y);
+	if (GTK_IS_LAYOUT(cont))
+		gtk_layout_move(GTK_LAYOUT(cont), child->border, x, y);
 	else
-		gtk_fixed_move(GTK_FIXED(parent), child->border, x, y);
+		gtk_fixed_move(GTK_FIXED(cont), child->border, x, y);
 }
 
 bool gContainer::hasBackground() const
@@ -653,3 +685,42 @@ void gContainer::setFullArrangement(gContainerArrangement *arr)
 	arrangement.locked = locked;
 	performArrange();
 }
+
+void gContainer::disableArrangement()
+{
+	if (_no_arrangement == 0)
+		_did_arrangement = false;
+	
+	_no_arrangement++;
+}
+
+void gContainer::enableArrangement()
+{
+	_no_arrangement--;
+	if (_no_arrangement == 0 && _did_arrangement)
+		performArrange();
+}
+
+void gContainer::hideHiddenChildren()
+{
+	int i;
+	gControl *child;
+	
+	for (i = 0;; i++)
+	{
+		child = gContainer::child(i);
+		if (!child)
+			break;
+		if (!child->isVisible())
+			gtk_widget_hide(child->border);
+		else if (child->isContainer())
+			((gContainer *)child)->hideHiddenChildren();
+	}
+}
+
+void gContainer::reparent(gContainer *newpr, int x, int y)
+{
+	fprintf(stderr, "gContainer::reparent\n");
+	gControl::reparent(newpr, x, y);
+	hideHiddenChildren();
+}
