@@ -45,36 +45,24 @@
 #include "gplugin.h"
 #include "gscrollbar.h"
 #include "gslider.h"
-#include "gsplitter.h"
 #include "gdesktop.h"
 #include "gdrag.h"
 #include "gmouse.h"
 #include "gcontrol.h"
 
-static GList *controls=NULL;
-static GList *controls_destroyed=NULL;
+static GList *controls = NULL;
+static GList *controls_destroyed = NULL;
 
-static const gchar _cursor_fdiag[] = {
+static const guchar _cursor_fdiag[] = {
    0x3f, 0x00, 0x1f, 0x00, 0x0f, 0x00, 0x0f, 0x00, 0x13, 0x00, 0x21, 0x00,
    0x40, 0x00, 0x80, 0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x84, 0x00, 0xc8,
    0x00, 0xf0, 0x00, 0xf0, 0x00, 0xf8, 0x00, 0xfc };
    
-static const gchar _cursor_bdiag[] = {
+static const guchar _cursor_bdiag[] = {
    0x00, 0xfc, 0x00, 0xf8, 0x00, 0xf0, 0x00, 0xf0, 0x00, 0xc8, 0x00, 0x84,
    0x00, 0x02, 0x00, 0x01, 0x80, 0x00, 0x40, 0x00, 0x21, 0x00, 0x13, 0x00,
    0x0f, 0x00, 0x0f, 0x00, 0x1f, 0x00, 0x3f, 0x00 };
    
-static const gchar _cursor_splith[] = {
-   0x60, 0x06, 0x60, 0x06, 0x60, 0x06, 0x60, 0x06, 0x64, 0x26, 0x66, 0x66,
-   0x67, 0xe6, 0x7f, 0xfe, 0x7f, 0xfe, 0x67, 0xe6, 0x66, 0x66, 0x64, 0x26,
-   0x60, 0x06, 0x60, 0x06, 0x60, 0x06, 0x60, 0x06 };
-   
-static const gchar _cursor_splitv[] = {
-   0xc0, 0x03, 0xe0, 0x07, 0xf0, 0x0f, 0x80, 0x01, 0x80, 0x01, 0xff, 0xff,
-   0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x80, 0x01,
-   0x80, 0x01, 0xf0, 0x0f, 0xe0, 0x07, 0xc0, 0x03 };
-
-
 // Geometry optimization hack - Sometimes fails, so it is disabled...
 #define GEOMETRY_OPTIMIZATION 0
 
@@ -208,7 +196,8 @@ void gControl::initAll(gContainer *parent)
 	bufX = 0;
 	bufY = 0;
 	curs = NULL;
-	fnt = NULL;
+	_font = NULL;
+	_resolved_font = NULL;
 	g_typ = 0;
 	dsg = false;
 	expa = false;
@@ -225,7 +214,7 @@ void gControl::initAll(gContainer *parent)
 	use_base = false;
 	mous = CURSOR_DEFAULT;
 	pr = parent;
-	_name=NULL;
+	_name = NULL;
 	visible = false;
 	_locked = 0;
 	_destroyed = false;
@@ -238,6 +227,8 @@ void gControl::initAll(gContainer *parent)
 	_proxy = _proxy_for = NULL;
 	_no_tab_focus = false;
 	_inside = false;
+	_no_auto_grab = false;
+	_no_background = false;
 
 	onFinish = NULL;
 	onFocusEvent = NULL;
@@ -246,6 +237,7 @@ void gControl::initAll(gContainer *parent)
 	onDrag = NULL;
 	onDragMove = NULL;
 	onDrop = NULL;
+	onDragLeave = NULL;
 	onEnterLeave = NULL;
 	canRaise = always_can_raise;
 
@@ -271,17 +263,6 @@ gControl::~gControl()
 {
 	gMainWindow *win = window();
 	
-	/* Useless, as the control won't raise events anymore apparently.
-	if (gApplication::_old_active_control == this)
-	{
-		if (onFocusEvent) 
-		{
-			fprintf(stderr, "focus out ??\n");
-			onFocusEvent(this, gEvent_FocusOut);
-		}
-		gApplication::_old_active_control = NULL;
-	}*/
-	
 	emit(SIGNAL(onFinish));
 	
 	if (win && win->focus == this)
@@ -296,20 +277,27 @@ gControl::~gControl()
 		gDrag::cancel();
 	
 	if (curs) { delete curs; curs=NULL; }
-	gFont::assign(&fnt);
+
+	if (_font)
+	{
+		gFont::assign(&_font);
+		gFont::assign(&_resolved_font);
+	}
+
 	setName(NULL);
 
 	controls = g_list_remove(controls, this);
 	controls_destroyed = g_list_remove(controls_destroyed, this);
 	
-	if (gApplication::_enter == this)
-		gApplication::_enter = NULL;
-	if (gApplication::_leave == this)
-		gApplication::_leave = NULL;
-	if (gApplication::_active_control == this)
-		gApplication::_active_control = NULL;
-	if (gApplication::_old_active_control == this)
-		gApplication::_old_active_control = NULL;
+	#define CLEAN_POINTER(_p) if (_p == this) _p = NULL
+
+	CLEAN_POINTER(gApplication::_enter);
+	CLEAN_POINTER(gApplication::_leave);
+	CLEAN_POINTER(gApplication::_active_control);
+	CLEAN_POINTER(gApplication::_old_active_control);
+	CLEAN_POINTER(gApplication::_button_grab);
+	CLEAN_POINTER(gApplication::_control_grab);
+	CLEAN_POINTER(gApplication::_ignore_until_next_enter);
 }
 
 void gControl::destroy()
@@ -328,9 +316,9 @@ void gControl::destroy()
 }
 
 
-bool gControl::enabled()
+bool gControl::isEnabled() const
 {
-	return GTK_WIDGET_SENSITIVE(border);
+	return gtk_widget_is_sensitive(border);
 }
 
 bool gControl::isReallyVisible()
@@ -340,11 +328,14 @@ bool gControl::isReallyVisible()
 
 void gControl::setEnabled(bool vl)
 {
-	gtk_widget_set_sensitive (border, vl);	
+	gtk_widget_set_sensitive(border, vl);
 }
 
 void gControl::setVisible(bool vl)
 {
+	if (vl == visible)
+		return;
+	
 	visible = vl;
 	
 	if (vl)
@@ -358,6 +349,8 @@ void gControl::setVisible(bool vl)
 	{
 		if (parent() && hasFocus())
 			gtk_widget_child_focus(GTK_WIDGET(gtk_widget_get_toplevel(border)), GTK_DIR_TAB_FORWARD);
+		if (gtk_widget_has_grab(border))
+			gtk_grab_remove(border);
 		gtk_widget_hide(border);
 	}
 	
@@ -370,25 +363,31 @@ POSITION AND SIZE
 
 ******************************************************************/
 
-void gControl::getScreenPos(int *x, int *y)
+bool gControl::getScreenPos(int *x, int *y)
 {
 	if (!border->window)
 	{
-		*x = *y = 0;
-		return;
+		*x = *y = 0; // widget is not realized
+		return true;
 	}
-	
-	gdk_window_get_origin(border->window, x, y);
-	
-	#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
+
+	gdk_window_get_origin(gtk_widget_get_window(border), x, y);
+
+	//fprintf(stderr, "getScreenPos: %s: %d %d: %d\n", name(), *x, *y, gtk_widget_get_has_window(border));
+
+	#if GTK_CHECK_VERSION(2, 18, 0)
 	if (!gtk_widget_get_has_window(border))
 	{
-		*x += border->allocation.x;
-		*y += border->allocation.y;
+		GtkAllocation a;
+		gtk_widget_get_allocation(border, &a);
+		*x += a.x;
+		*y += a.y;
 	}
 	#endif
-}
 
+	//fprintf(stderr, "getScreenPos: --> %d %d\n", *x, *y);
+	return false;
+}
 int gControl::screenX()
 {
 	int x,y;
@@ -403,26 +402,6 @@ int gControl::screenY()
 	return y;
 }
 
-int gControl::left()
-{
-	return bufX;
-}
-
-int gControl::top()
-{	
-	return bufY;
-}
-
-int gControl::width()
-{
-	return bufW;
-}
-
-int gControl::height()
-{
-	return bufH;
-}
-
 void gControl::setLeft(int l)
 {
 	move(l,top());
@@ -469,19 +448,10 @@ static void send_configure (gControl *control)
   gdk_event_free (event);
 }
 
-void gControl::move(int x, int y, int w, int h)
-{
-	move(x,y);
-	resize(w,h);
-}
-
 void gControl::move(int x, int y)
 {
 	//GtkLayout *fx;
 	
-	if (pr && pr->getClass() == Type_gSplitter) 
-		return;
-
 	if (x == bufX && y == bufY) 
 		return;
 	
@@ -492,9 +462,8 @@ void gControl::move(int x, int y)
 	_dirty_pos = true;
 	if (pr)
 	{
-    //fx = GTK_LAYOUT(gtk_widget_get_parent(border));
-	  //gtk_layout_move(fx, border, x, y);
-	  if (gtk_widget_get_parent(border) == pr->getContainer())
+		// TODO: check the following optimization to see if it can be enabled again
+	  //if (gtk_widget_get_parent(border) == pr->getContainer())
 	  	pr->performArrange();
   }
 	
@@ -509,9 +478,6 @@ void gControl::move(int x, int y)
 
 void gControl::resize(int w, int h)
 {
-	if (pr && pr->getClass() == Type_gSplitter) 
-		return;
-	
 	if (w < 1)
 		w = 0;
 		
@@ -562,6 +528,18 @@ void gControl::resize(int w, int h)
 	send_configure(this);
 }
 
+void gControl::moveResize(int x, int y, int w, int h)
+{
+	if (pr)
+		pr->disableArrangement();
+	
+	resize(w, h);
+	move(x, y);
+	
+	if (pr)
+		pr->enableArrangement();
+}
+
 void gControl::updateGeometry()
 {
 // 	if (_dirty_pos)
@@ -641,57 +619,88 @@ void gControl::setToolTip(char* vl)
 
 gFont* gControl::font()
 {
-	if (fnt)
-		return fnt;
-	if (pr)
+	if (_resolved_font)
+	{
+		//fprintf(stderr, "%s: font -> _resolved_font\n", name());
+		return _resolved_font;
+	}
+	else if (pr)
+	{
+		//fprintf(stderr, "%s: font -> parent\n", name());
 		return pr->font();
-	
-	return gDesktop::font();
+	}
+	else
+	{
+		//fprintf(stderr, "%s: font -> desktop\n", name());
+		return gDesktop::font();
+	}
 }
 
-
-void gControl::resolveFont(gFont *new_font)
+void gControl::actualFontTo(gFont *ft)
 {
-	gFont *font = new gFont();
-	gControl *ctrl = this;
-	
-	font->mergeFrom(new_font);
+	//fprintf(stderr, "actualFontTo: %s: %s / %s (_font = %s)\n", name(), ft->toString(), ft->toFullString(), _font ? _font->toFullString() : NULL);
+	font()->copyTo(ft);
+	ft->setAllFrom(_font);
+	//fprintf(stderr, "==> %s: %s / %s (_font = %s)\n", name(), ft->toString(), ft->toFullString(), _font ? _font->toFullString() : NULL);
+}
 
-	for(;;)
+void gControl::resolveFont()
+{
+	gFont *font;
+
+	if (_font)
 	{
-		if (font->isAllSet())
-			break;
+		font = new gFont();
+		font->mergeFrom(_font);
+		if (pr)
+			font->mergeFrom(pr->font());
+		else
+			font->mergeFrom(gDesktop::font());
 
-		ctrl = ctrl->parent();
-		if (!ctrl)
-			break;
-
-		if (ctrl->fnt)
-			font->mergeFrom(ctrl->fnt);
+		gFont::set(&_resolved_font, font);
 	}
-	
-	gtk_widget_modify_font(widget, font->desc());
-	
-	gFont::set(&fnt, font);
+	else
+		gFont::assign(&_resolved_font);
 }
 
 void gControl::setFont(gFont *ft)
 {
+	//fprintf(stderr, "setFont: %s: %s\n", name(), ft->toFullString());
 	if (ft)
-	{
-		resolveFont(ft);
-		_font_set = true;
-	}
-	else if (fnt)
-	{
-		gFont::assign(&fnt);
-		gtk_widget_modify_font(widget, NULL);
-		_font_set = false;
-	}
-	
+		gFont::assign(&_font, ft);
+	else if (_font)
+		gFont::assign(&_font);
+
+	gFont::assign(&_resolved_font);
+
+	updateFont();
+
 	resize();
+
+	//fprintf(stderr, "--> %s: _font = %s\n", name(), _font ? _font->toFullString() : NULL);
 }
 
+static void cb_update_font(GtkWidget *widget, gpointer data)
+{
+	PangoFontDescription *desc = (PangoFontDescription *)data;
+	gtk_widget_modify_font(widget, desc);
+}
+
+void gControl::updateFont()
+{
+	resolveFont();
+	gtk_widget_modify_font(widget, font()->desc());
+
+	if (!isContainer() && GTK_IS_CONTAINER(widget))
+		gtk_container_forall(GTK_CONTAINER(widget), (GtkCallback)cb_update_font, (gpointer)font()->desc());
+
+	refresh();
+	updateSize();
+}
+
+void gControl::updateSize()
+{
+}
 
 int gControl::mouse()
 {
@@ -730,42 +739,30 @@ void gControl::setCursor(gCursor *vl)
 
 void gControl::updateCursor(GdkCursor *cursor)
 {
-  if (GDK_IS_WINDOW(border->window))
-    gdk_window_set_cursor(border->window, cursor);
-  /*if (frame && GDK_IS_WINDOW(frame->window) && frame->window != border->window)
-    gdk_window_set_cursor(frame->window, cursor);
-  if (widget != frame && widget != border && GDK_IS_WINDOW(widget->window))
-    gdk_window_set_cursor(widget->window, cursor);*/
+  if (GDK_IS_WINDOW(border->window) && _inside)
+	{
+		//fprintf(stderr, "updateCursor: %s %p\n", name(), cursor);
+		if (!cursor && parent() && parent()->border->window == border->window)
+			parent()->updateCursor(parent()->getGdkCursor());
+		else
+			gdk_window_set_cursor(border->window, cursor);
+	}
 }
 
-void gControl::setMouse(int m)
+GdkCursor *gControl::getGdkCursor()
 {
-	GdkCursor *cr = NULL;
 	GdkPixmap *pix;
-	GdkColor col = {0,0,0,0};
-	
-	if (_proxy)
-	{
-		_proxy->setMouse(m);
-		return;
-	}
-	
-	mous = m;
+	GdkColor col = {0};
+	GdkCursor *cr = NULL;
+	int m = mous;
 	
 	if (gApplication::isBusy())
     m = GDK_WATCH;
 	
 	if (m == CURSOR_CUSTOM)
 	{
-		if (!curs || !curs->cur)
-		{
-			mous = CURSOR_DEFAULT;
-			updateCursor(NULL);
-		}
-		else
-			updateCursor(curs->cur);
-		
-		return;	
+		if (curs && curs->cur)
+			return curs->cur;
 	}
 	
 	if (m != CURSOR_DEFAULT) 
@@ -778,32 +775,39 @@ void gControl::setMouse(int m)
 		{
 			if (m == (GDK_LAST_CURSOR+1)) //FDiag
 			{
-				pix = gdk_bitmap_create_from_data(NULL,_cursor_fdiag,16,16);
-				cr = gdk_cursor_new_from_pixmap(pix,pix,&col,&col,0,0);
+				pix = gdk_bitmap_create_from_data(NULL, (const gchar *)_cursor_fdiag, 16, 16);
+				cr = gdk_cursor_new_from_pixmap(pix, pix, &col, &col, 0, 0);
 				g_object_unref(pix);
 			}
 			else if (m == (GDK_LAST_CURSOR+2)) //BDiag
 			{
-				pix = gdk_bitmap_create_from_data(NULL,_cursor_bdiag,16,16);
-				cr = gdk_cursor_new_from_pixmap(pix,pix,&col,&col,0,0);
-				g_object_unref(pix);
-			}
-			else if (m == (GDK_LAST_CURSOR+3)) //SplitH
-			{
-				pix = gdk_bitmap_create_from_data(NULL,_cursor_splith,16,16);
-				cr = gdk_cursor_new_from_pixmap(pix,pix,&col,&col,0,0);
-				g_object_unref(pix);
-			}
-			else if (m == (GDK_LAST_CURSOR+4)) //SplitV
-			{
-				pix = gdk_bitmap_create_from_data(NULL,_cursor_splitv,16,16);
-				cr = gdk_cursor_new_from_pixmap(pix,pix,&col,&col,0,0);
+				pix = gdk_bitmap_create_from_data(NULL, (const gchar *)_cursor_bdiag, 16, 16);
+				cr = gdk_cursor_new_from_pixmap(pix, pix, &col, &col, 0, 0);
 				g_object_unref(pix);
 			}
 		}
 	}
 	
-  updateCursor(cr);
+	return cr;
+}
+
+void gControl::setMouse(int m)
+{
+	if (_proxy)
+	{
+		_proxy->setMouse(m);
+		return;
+	}
+	
+	if (m == CURSOR_CUSTOM)
+	{
+		if (!curs || !curs->cur)
+			m = CURSOR_DEFAULT;
+	}
+	
+	mous = m;
+	
+	updateCursor(getGdkCursor());
 }
 
 
@@ -814,7 +818,7 @@ HANDLES
 
 ******************************************************************/
 
-bool gControl::isWindow()
+bool gControl::isWindow() const
 {
 	return g_typ == Type_gMainWindow; 
 }
@@ -881,7 +885,18 @@ void gControl::refresh(int x, int y, int w, int h)
 	if (x < 0 || y < 0 || w < 0 || h < 0)
 	  gtk_widget_queue_draw(border);
 	else	
-	  gtk_widget_queue_draw_area(border, x, y, w, h);
+	{
+		// Buggy GTK+
+	  // gtk_widget_queue_draw_area(border, x, y, w, h);
+	  GdkRectangle r;
+		
+		r.x = border->allocation.x + x;
+		r.y = border->allocation.y + y;
+		r.width = w;
+		r.height = h;
+  
+		gdk_window_invalidate_rect(border->window, &r, TRUE);
+	}
 
 	afterRefresh();
 }
@@ -966,14 +981,13 @@ gControl* gControl::previous()
 void gControl::lower()
 {
 	gpointer *p;
-	GList *iter;
 	GList *chd;
 	GtkWidget *child;
-	gControl *Br;
+	gControl *br;
 	int x,y;
+	GtkContainer *parent;
 
 	if (!pr) return;
-	if (pr->getClass()==Type_gSplitter) return;
 	
 	if (gtk_widget_get_has_window(border))
 	{
@@ -983,43 +997,36 @@ void gControl::lower()
 	}
 	else
 	{	
-		fprintf(stderr, "gb.gtk: warning: gControl::lower(): no window\n");
+		//fprintf(stderr, "gb.gtk: warning: gControl::lower(): no window\n");
 		
 		if (!(chd=gtk_container_get_children(GTK_CONTAINER(pr->getContainer())))) return;
-		chd=g_list_first(chd);
 		
-		while (chd)
+		chd = g_list_first(chd);
+		
+		while(chd)
 		{
-			child=(GtkWidget*)chd->data;
+			child = (GtkWidget*)chd->data;
 			
-			Br=NULL;
-			if (controls)
+			br = (gControl *)g_object_get_data(G_OBJECT(child), "gambas-control");;
+			
+			if (br && br != this)
 			{
-				iter=g_list_first(controls);
-				while (iter)
-				{
-					if ( ((gControl*)iter->data)->border == child)
-					{
-						Br=(gControl*)iter->data;
-						break;
-					}
-					iter=iter->next;
-				}
+				x = br->x();
+				y = br->y();
+				parent = GTK_CONTAINER(gtk_widget_get_parent(br->border));
+				g_object_ref(G_OBJECT(br->border));
+				gtk_container_remove(parent, br->border);
+				gtk_container_add(parent, br->border);
+				
+				if (GTK_IS_LAYOUT(parent))
+					gtk_layout_move(GTK_LAYOUT(parent), br->border, x, y);
+				else
+					gtk_fixed_move(GTK_FIXED(parent), br->border, x, y);
+				
+				g_object_unref(G_OBJECT(br->border));
 			}
 			
-			if (Br && (Br != this))
-			{
-				x=Br->left();
-				y=Br->top();
-				g_object_ref(G_OBJECT(Br->border));
-				gtk_container_remove(GTK_CONTAINER(pr->getContainer()),Br->border);
-				gtk_container_add(GTK_CONTAINER(pr->getContainer()),Br->border);
-				pr->moveChild(Br, x, y);
-				//gtk_layout_put(GTK_LAYOUT(pr->getContainer()),Br->border,x,y);
-				g_object_unref(G_OBJECT(Br->border));
-			}
-			
-			chd=g_list_next(chd);
+			chd = g_list_next(chd);
 		}
 	}
 	
@@ -1038,10 +1045,10 @@ void gControl::lower()
 
 void gControl::raise()
 {
-	int x,y;
+	int x, y;
+	GtkContainer *parent;
 
 	if (!pr) return;
-	if (pr->getClass()==Type_gSplitter) return;
 	
 	if (gtk_widget_get_has_window(border))
 	{
@@ -1051,19 +1058,25 @@ void gControl::raise()
 	}
 	else
 	{	
-		fprintf(stderr, "gb.gtk: warning: gControl::raise(): no window\n");
+		//fprintf(stderr, "gb.gtk: warning: gControl::raise(): no window\n");
 		
-		x=left();
-		y=top();
+		x = left();
+		y = top();
+		parent = GTK_CONTAINER(gtk_widget_get_parent(border));
 		g_object_ref(G_OBJECT(border));
-		gtk_container_remove(GTK_CONTAINER(pr->getContainer()),border);
-		gtk_container_add(GTK_CONTAINER(pr->getContainer()),border);
-		pr->moveChild(this, x, y);
+		gtk_container_remove(parent, border);
+		gtk_container_add(parent, border);
+
+		//pr->moveChild(this, x, y);
+		if (GTK_IS_LAYOUT(parent))
+			gtk_layout_move(GTK_LAYOUT(parent), border, x, y);
+		else
+			gtk_fixed_move(GTK_FIXED(parent), border, x, y);
+		
 		g_object_unref(G_OBJECT(border));
 	}
 	
 	g_ptr_array_remove(pr->_children, this);
-	
 	g_ptr_array_add(pr->_children, this);
 	
 	pr->updateFocusChain();
@@ -1147,10 +1160,14 @@ void gControl::setAcceptDrops(bool vl)
 	if (vl)
 	{
 		gtk_drag_dest_set(border, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+		//if (widget != border)
+		//	gtk_drag_dest_set(widget, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
 	}
 	else
 	{
 		gtk_drag_dest_unset(border);
+		//if (widget != border)
+		//	gtk_drag_dest_unset(widget);
 	}
 }
 
@@ -1164,10 +1181,10 @@ void gControl::connectParent()
 {
 	if (pr)
 	{
-    //gtk_widget_set_redraw_on_allocate(border, false);
-    
-  	pr->insert(this, true);
-  }
+		//gtk_widget_set_redraw_on_allocate(border, false);
+		
+		pr->insert(this, true);
+	}
   
 	// BM: Widget has been created, so we can set its cursor if application is busy
 	if (gApplication::isBusy() && mustUpdateCursor())
@@ -1179,12 +1196,18 @@ GList* gControl::controlList()
 	return controls; 
 }
 
-void gControl::drawBorder(GdkDrawable *win)
+gColor gControl::getFrameColor()
 {
+	return gDesktop::lightfgColor();
+}
+
+void gControl::drawBorder(GdkEventExpose *e)
+{
+	GdkDrawable *win;
 	GtkShadowType shadow;
 	gint x, y, w, h;
-	//GdkGC *gc;
 	GtkStyle* st;
+	GdkRectangle clip;
 	
 	if (getFrameBorder() == BORDER_NONE)
     return;
@@ -1194,7 +1217,7 @@ void gControl::drawBorder(GdkDrawable *win)
   w = width();
   h = height();
   
-	if (!win)
+	//if (!win)
 	{
 		GtkWidget *wid;
 		
@@ -1215,21 +1238,15 @@ void gControl::drawBorder(GdkDrawable *win)
   if (w < 1 || h < 1)
   	return;
   
-	st = gtk_widget_get_style(widget);
-  
 	switch (getFrameBorder())
 	{
     case BORDER_PLAIN:
     {
-			GdkGC *gc;
-			GdkGCValues values;
-
-			fill_gdk_color(&values.foreground, gDesktop::lightfgColor(), gdk_drawable_get_colormap(win));
-			gc = gtk_gc_get(gdk_drawable_get_depth(win), gdk_drawable_get_colormap(win), &values, GDK_GC_FOREGROUND);
+			cairo_t *cr;
 			
-			//gdk_draw_rectangle(win, use_base ? st->text_gc[GTK_STATE_NORMAL] : st->fg_gc[GTK_STATE_NORMAL], FALSE, x, y, w - 1, h - 1); 
-			gdk_draw_rectangle(win, gc, FALSE, x, y, w - 1, h - 1); 
-			gtk_gc_release(gc);
+			cr = gdk_cairo_create(win);
+			gt_cairo_draw_rect(cr, x, y, w, h, getFrameColor());
+			cairo_destroy(cr);
       return;
     }
     
@@ -1241,12 +1258,20 @@ void gControl::drawBorder(GdkDrawable *win)
       return;
 	}
 	
-  gtk_paint_shadow(st, win, GTK_STATE_NORMAL, shadow, NULL, NULL, NULL, x, y, w, h);
+	st = gtk_widget_get_style(widget);  
+	gdk_region_get_clipbox(e->region, &clip);
+  gtk_paint_shadow(st, win, GTK_STATE_NORMAL, shadow, &clip, NULL, NULL, x, y, w, h);
 }
 
 static gboolean cb_frame_expose(GtkWidget *wid, GdkEventExpose *e, gControl *control)
 {
-	control->drawBorder();
+	control->drawBorder(e);
+	return false;
+}
+
+static gboolean cb_draw_background(GtkWidget *wid, GdkEventExpose *e, gControl *control)
+{
+	control->drawBackground(wid, e);
 	return false;
 }
 
@@ -1291,6 +1316,49 @@ void gControl::registerControl()
 	g_object_set_data(G_OBJECT(border), "gambas-control", this);
 }
 
+static gboolean cb_clip_children(GtkWidget *wid, GdkEventExpose *e, gContainer *d)
+{
+	GdkRegion *me;
+	
+	me = gdk_region_rectangle((GdkRectangle *)&wid->allocation);
+	
+	gdk_region_intersect(e->region, me);
+	
+	gdk_region_destroy(me);
+	
+	if (gdk_region_empty(e->region))
+		return TRUE;
+	
+	return FALSE;
+}
+
+#if 0
+static gboolean cb_clip_by_parent(GtkWidget *wid, GdkEventExpose *e, gControl *d)
+{
+	GdkRegion *preg;
+	GdkRectangle prect = { 0, 0, d->parent()->width() - d->x(), d->parent()->height() - d->y() };
+	
+	fprintf(stderr, "area = %d %d %d %d  prect = %d %d %d %d\n",
+					e->area.x, e->area.y, e->area.width, e->area.height,
+					prect.x, prect.y, prect.width, prect.height);
+	
+	preg = gdk_region_rectangle(&prect);
+	
+	gdk_region_intersect(e->region, preg);
+	
+	gdk_region_destroy(preg);
+	
+	if (gdk_region_empty(e->region))
+		return TRUE;
+	
+	gdk_region_get_clipbox(e->region, &prect);
+	e->area = prect;
+	fprintf(stderr, "--> %d %d %d %d\n", prect.x, prect.y, prect.width, prect.height);
+	
+	return FALSE;
+}
+#endif
+
 void gControl::realize(bool make_frame)
 {
 	if (!_scroll)
@@ -1330,7 +1398,18 @@ void gControl::realize(bool make_frame)
 	
   if (frame)
 		g_signal_connect_after(G_OBJECT(frame), "expose-event", G_CALLBACK(cb_frame_expose), (gpointer)this);
-		
+	
+	if (!_no_background && !gtk_widget_get_has_window(border))
+		g_signal_connect(G_OBJECT(border), "expose-event", G_CALLBACK(cb_draw_background), (gpointer)this);
+	/*else if (!isTopLevel())
+	{
+		fprintf(stderr, "clip by parent\n");
+		g_signal_connect(G_OBJECT(border), "expose-event", G_CALLBACK(cb_clip_by_parent), (gpointer)this);
+	}*/
+	
+	if (isContainer() && !gtk_widget_get_has_window(widget))
+		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_clip_children), (gpointer)this);
+	
 	//if (isContainer() && widget != border)
 	//	g_signal_connect(G_OBJECT(widget), "size-allocate", G_CALLBACK(cb_size_allocate), (gpointer)this);
 	
@@ -1350,12 +1429,14 @@ void gControl::realize(bool make_frame)
 
 void gControl::realizeScrolledWindow(GtkWidget *wid, bool doNotRealize)
 {
-	border = gtk_event_box_new();
+	_scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
+	
+	border = GTK_WIDGET(_scroll);
 	widget = wid;
 	frame = 0;
-
-	_scroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
-	gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(_scroll));
+	_no_auto_grab = true;
+	
+	//gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(_scroll));
 	gtk_scrolled_window_set_policy(_scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(_scroll), widget);
 	
@@ -1473,7 +1554,7 @@ void gControl::setName(char *name)
 gColor gControl::realBackground()
 {
 	if (_bg_set)
-		return use_base ? get_gdk_base_color(widget) : get_gdk_bg_color(widget);
+		return use_base ? get_gdk_base_color(widget, isEnabled()) : get_gdk_bg_color(widget, isEnabled());
 	else if (pr)
 		return pr->realBackground();
 	else
@@ -1521,7 +1602,7 @@ void gControl::setBackground(gColor color)
 gColor gControl::realForeground()
 {
 	if (_fg_set)
-		return use_base ? get_gdk_text_color(widget) : get_gdk_fg_color(widget);
+		return use_base ? get_gdk_text_color(widget, isEnabled()) : get_gdk_fg_color(widget, isEnabled());
 	else if (pr)
 		return pr->realForeground();
 	else
@@ -1592,7 +1673,7 @@ void gControl::reparent(gContainer *newpr, int x, int y)
 		return;
 	
 	if (was_visible) hide();
-	gtk_widget_unrealize(border);
+	//gtk_widget_unrealize(border);
 	
 	oldpr = pr;
 	pr = newpr;	
@@ -1614,7 +1695,7 @@ void gControl::reparent(gContainer *newpr, int x, int y)
 		newpr->insert(this);
 	}
 
-	gtk_widget_realize(border);
+	//gtk_widget_realize(border);
 	move(x, y);
 	if (was_visible) show();
 }
@@ -1743,9 +1824,20 @@ int gControl::minimumHeight()
 	return 0;
 }
 
+bool gControl::isTracking() const
+{
+	if (_proxy)
+		return _proxy->isTracking();
+	else
+		return _tracking;
+}
+
 void gControl::setTracking(bool v)
 {
-	_tracking = v;
+	if (_proxy)
+		_proxy->setTracking(v);
+	else
+		_tracking = v;
 	/*
 	GtkWidget *wid;
 	
@@ -1778,9 +1870,11 @@ void gControl::setTracking(bool v)
 	*/
 }
 
-bool gControl::grab(bool showIt)
+bool gControl::grab()
 {
 	GdkWindow *win;
+	gControl *old_control_grab;
+	bool save_tracking;
 	
 	if (_grab)
 		return false;
@@ -1788,7 +1882,7 @@ bool gControl::grab(bool showIt)
 	win = border->window;
 	
 	if (gdk_pointer_grab(win, FALSE, (GdkEventMask)(GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK), NULL, 
-	#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
+	#if GTK_CHECK_VERSION(2, 18, 0)
 	    gdk_window_get_cursor(win),
 	#else
 		  NULL,
@@ -1806,26 +1900,37 @@ bool gControl::grab(bool showIt)
 	}
 
 	_grab = true;
+	save_tracking = _tracking;
+	_tracking = true;
 	
-	gApplication::enterLoop(this, showIt);
+	old_control_grab = gApplication::_control_grab;
+	gApplication::_control_grab = this;
 
+	gApplication::enterLoop(this);
+
+	gApplication::_control_grab = old_control_grab;
+	
 	gdk_pointer_ungrab(GDK_CURRENT_TIME);
 	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+	
+	_tracking = save_tracking;
 	_grab = false;
 	return false;
 }
 
 bool gControl::hovered()
 {
-	int x, y, xm, ym;
+	//int x, y, xm, ym;
 	
 	if (!isVisible())
 		return false;
+	else
+		return _inside;
 	
-	getScreenPos(&x, &y);
+	/*getScreenPos(&x, &y);
 	gMouse::getScreenPos(&xm, &ym);
 	
-	return (xm >= x && ym >= y && xm < (x + width()) && ym < (y + height()));
+	return (xm >= x && ym >= y && xm < (x + width()) && ym < (y + height()));*/
 }
 
 bool gControl::setProxy(gControl *proxy)
@@ -1861,20 +1966,108 @@ void gControl::setNoTabFocus(bool v)
 		pr->updateFocusChain();
 }
 
-void gControl::emitEnterEvent()
+void gControl::emitEnterEvent(bool no_leave)
 {
+	gContainer *cont;
+	
+	if (parent())
+		parent()->emitEnterEvent(true);
+	
+	if (!no_leave && isContainer())
+	{
+		cont = (gContainer *)this;
+		int i;
+		
+		for (i = 0; i < cont->childCount(); i++)
+			cont->child(i)->emitLeaveEvent();
+	}
+	
 	if (_inside)
 		return;
-
-	emit(SIGNAL(onEnterLeave), gEvent_Enter);
+	
 	_inside = true;
+
+	setMouse(mouse());
+
+	if (gApplication::_ignore_until_next_enter)
+	{
+		//fprintf(stderr, "ignore next enter for %s\n", name());
+		if (gApplication::_ignore_until_next_enter == this)
+			gApplication::_ignore_until_next_enter = NULL;
+		return;
+	}
+	
+	//fprintf(stderr, "RAISE ENTER: %s\n", name());
+	emit(SIGNAL(onEnterLeave), gEvent_Enter);
 }
 
 void gControl::emitLeaveEvent()
 {
 	if (!_inside)
 		return;
-
-	emit(SIGNAL(onEnterLeave), gEvent_Leave);
+	
+	if (isContainer())
+	{
+		gContainer *cont = (gContainer *)this;
+		int i;
+		
+		for (i = 0; i < cont->childCount(); i++)
+			cont->child(i)->emitLeaveEvent();
+	}
+	
 	_inside = false;
+	
+	if (parent()) parent()->setMouse(parent()->mouse());
+
+	if (gApplication::_ignore_until_next_enter)
+	{
+		//fprintf(stderr, "ignore next leave for %s\n", name());
+		return;
+	}
+
+	//fprintf(stderr, "RAISE LEAVE: %s\n", name());
+	emit(SIGNAL(onEnterLeave), gEvent_Leave);
 }
+
+bool gControl::isAncestorOf(gControl *child)
+{
+	if (!isContainer())
+		return false;
+	
+	for(;;)
+	{
+		child = child->parent();
+		if (!child)
+			return false;
+		else if (child == this)
+			return true;
+	}
+}
+
+void gControl::drawBackground(GtkWidget *widget, GdkEventExpose *e)
+{
+	if (background() == COLOR_DEFAULT)
+		return;
+	
+	cairo_t *cr = gdk_cairo_create(widget->window);
+
+	gdk_cairo_region(cr, e->region);
+	cairo_clip(cr);
+	gt_cairo_set_source_color(cr, background());
+
+	cairo_rectangle(cr, border->allocation.x, border->allocation.y, width(), height());
+	cairo_fill(cr);
+	
+  cairo_destroy(cr);
+}
+
+bool gControl::canFocus() const
+{
+	return GTK_WIDGET_CAN_FOCUS(widget);
+}
+
+void gControl::setCanFocus(bool vl)
+{
+	gtk_widget_set_can_focus(widget, vl);
+}
+
