@@ -70,7 +70,7 @@ static void set_font_from_string(gFont *font, const char *str)
 		else
 		{
 			size = atof(elt);
-			if (size != 0.0)
+			if (isdigit(*elt) && size != 0.0)
 				font->setSize(size);
 			else
 			{
@@ -204,6 +204,31 @@ void gFont::reset()
 	_underline_set = false;
 }
 
+void gFont::setAll(bool v)
+{
+	_bold_set = v;
+	_italic_set = v;
+	_name_set = v;
+	_size_set = v;
+	_strikeout_set = v;
+	_underline_set = v;
+}
+
+void gFont::setAllFrom(gFont *font)
+{
+	if (!font)
+		setAll(false);
+	else
+	{
+		_bold_set = font->_bold_set;
+		_italic_set = font->_italic_set;
+		_name_set = font->_name_set;
+		_size_set = font->_size_set;
+		_strikeout_set = font->_strikeout_set;
+		_underline_set = font->_underline_set;
+	}
+}
+
 void gFont::realize()
 {
   ct = NULL;
@@ -220,7 +245,7 @@ void gFont::initFlags()
 	
 	_bold_set = comp->bold() != bold();
 	_italic_set = comp->italic() != italic();
-	_name_set = strcmp(comp->name(), name());
+	_name_set = strcmp(comp->name(), name()) != 0;
 	_size_set = comp->size() != size();
 	_strikeout_set = comp->strikeout() != strikeout();
 	_underline_set = comp->underline() != underline();
@@ -424,7 +449,7 @@ void gFont::setGrade(int grade)
 {
 	double desktop = gDesktop::font()->size();
 	
-	if (grade < -4 || grade > 16)
+	if (grade < -4 || grade > 24)
 		return;
 	
 	setSize(GRADE_TO_SIZE(grade, desktop));
@@ -650,7 +675,7 @@ bool gFont::isAllSet()
 		&& _underline_set;
 }
 
-void gFont::richTextSize(char *txt, int len, int sw, int *w, int *h)
+void gFont::richTextSize(const char *txt, int len, float sw, float *w, float *h)
 {
 	PangoLayout *ly;
 	int tw = 0, th = 0;
@@ -660,14 +685,14 @@ void gFont::richTextSize(char *txt, int len, int sw, int *w, int *h)
 	{
 		ly = pango_layout_new(ct);
 		html = gt_html_to_pango_string(txt, len, false);
-		pango_layout_set_markup(ly, html, -1);	
 		pango_layout_set_wrap(ly, PANGO_WRAP_WORD_CHAR);
+		pango_layout_set_markup(ly, html, -1);	
 		if (sw > 0)
 			pango_layout_set_width(ly, sw * PANGO_SCALE);
-		pango_layout_get_pixel_size(ly, &tw, &th);
+		pango_layout_get_size(ly, &tw, &th);
 		g_free(html);
 	}
 	
-	if (w) *w = tw;
-	if (h) *h = th;
+	if (w) *w = (float)tw / PANGO_SCALE;
+	if (h) *h = (float)th / PANGO_SCALE;
 }
