@@ -23,6 +23,7 @@
 
 #include "widgets.h"
 #include "widgets_private.h"
+#include "gapplication.h"
 #include "gframe.h"
 
 static void cb_frame_resize (GtkWidget *wid, GtkAllocation *req, gFrame *data)
@@ -30,23 +31,116 @@ static void cb_frame_resize (GtkWidget *wid, GtkAllocation *req, gFrame *data)
 	gtk_widget_set_size_request(data->fr, req->width, req->height);
 } 
 
-
 /****************************************************************************
 
 Panel
 
 ****************************************************************************/
 
+void gPanel::create(void)
+{
+	int i;
+	GtkWidget *ch, *box;
+	bool doReparent = false;
+	bool was_visible = isVisible();
+	GdkRectangle rect;
+	int bg, fg;
+	gControl *nextSibling;
+	
+	if (border)
+	{
+		getGeometry(&rect);
+		bg = background();
+		fg = foreground();
+		nextSibling = next();
+		parent()->remove(this);
+		
+		for (i = 0; i < childCount(); i++)
+		{
+			ch = child(i)->border;
+			g_object_ref(G_OBJECT(ch));
+			gtk_container_remove(GTK_CONTAINER(widget), ch);
+		}
+		
+		_no_delete = true;
+		gtk_widget_destroy(border);
+		_no_delete = false;
+		doReparent = true;
+	}
+	
+	if (_bg_set)
+	{
+		border = gtk_event_box_new();
+		widget = gtk_fixed_new();
+		box = widget;
+		//gtk_widget_set_app_paintable(border, TRUE);
+		//gtk_widget_set_app_paintable(box, TRUE);
+	}
+	else
+	{
+		border = widget = gtk_fixed_new();
+		box = NULL;
+	}
+
+	frame = widget;
+	realize(true);
+	
+	//g_signal_connect(G_OBJECT(border), "size-allocate", G_CALLBACK(cb_size), (gpointer)this);
+	//g_signal_connect(G_OBJECT(border), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
+	
+	if (doReparent)
+	{
+		if (box)
+			gtk_widget_realize(box);
+		
+		setNext(nextSibling);
+		setBackground(bg);
+		setForeground(fg);
+		setFont(font());
+		bufX = bufY = bufW = bufH = -1;
+		setGeometry(&rect);
+		
+		for (i = 0; i < childCount(); i++)
+		{
+			ch = child(i)->border;
+			gtk_container_add(GTK_CONTAINER(widget), ch);
+			moveChild(child(i), child(i)->x(), child(i)->y());
+			g_object_unref(G_OBJECT(ch));
+		}
+		
+		if (was_visible)
+			show();
+		else
+			hide();
+		
+		//gApplication::checkHoveredControl(this);
+		
+		if (_inside)
+		{
+			_inside = false;
+			if (gApplication::_enter == this)
+				gApplication::_enter = NULL;
+			gApplication::_ignore_until_next_enter = this;
+		}
+	}
+}
+
 gPanel::gPanel(gContainer *parent) : gContainer(parent)
 {
 	g_typ = Type_gPanel;
-
-	border = gtk_event_box_new();
-	widget = gtk_fixed_new();
-	frame = widget;
-	realize(true);
+	border = NULL;
+	create();
 }
 
+void gPanel::setBackground(gColor color)
+{
+	bool set = _bg_set;
+	
+	gControl::setBackground(color);
+	
+	if (set != _bg_set)
+		create();
+}
 
 /****************************************************************************
 
@@ -58,8 +152,7 @@ gFrame::gFrame(gContainer *parent) : gContainer(parent)
 {
 	g_typ=Type_gFrame;
 
-	border = gtk_event_box_new();
-	widget = gtk_fixed_new();
+	border = widget = gtk_fixed_new();
 	
 	fr = gtk_frame_new(NULL);
 	label = gtk_frame_get_label_widget(GTK_FRAME(fr));
@@ -106,13 +199,6 @@ void gFrame::setText(char *vl)
 		gtk_label_set_text(GTK_LABEL(label), (const gchar*)vl);
 }
 
-void gFrame::setFont(gFont *ft)
-{
-	gControl::setFont(ft);
-	if (label)
-		gtk_widget_modify_font(label, fnt ? fnt->desc() : NULL);
-}
-
 void gFrame::setRealForeground(gColor color)
 {
 	gControl::setRealForeground(color);
