@@ -2,7 +2,7 @@
 
   gframe.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #define __GFRAME_H
 
 #include "gcontainer.h"
+#include "gcolor.h"
 
 class gPanel : public gContainer
 {
@@ -33,6 +34,11 @@ public:
 
 	int getBorder() { return getFrameBorder(); }
 	void setBorder(int vl) { setFrameBorder(vl); }
+	virtual void setBackground(gColor color = COLOR_DEFAULT);
+
+private:
+	
+	void create();
 };
 
 class gFrame : public gContainer
@@ -43,7 +49,6 @@ public:
 	char* text();
 	void setText(char* vl);
 
-	virtual void setFont(gFont *ft);
 	virtual void setRealForeground(gColor color);
 
 //"Private"
