@@ -49,21 +49,37 @@ static gboolean win_frame(GtkWidget *widget,GdkEventWindowState *event,gMainWind
 	return false;
 }
 
-static void cb_show (GtkWidget *widget, gMainWindow *data)
+static gboolean cb_show(GtkWidget *widget, gMainWindow *data)
 {
 	data->emitOpen();
+	
 	if (data->opened)
 	{
+		if (data->isTopLevel())
+		{
+			if (data->isModal() && data->isResizable())
+			{
+				GdkGeometry geometry;
+				
+				geometry.min_width = data->bufW;
+				geometry.min_height = data->bufH;
+				gdk_window_set_geometry_hints(gtk_widget_get_window(data->border), &geometry, (GdkWindowHints)(GDK_HINT_MIN_SIZE | GDK_HINT_POS));
+			}
+		}
+		
 		//data->performArrange();
+		data->emitResize();
 		data->emit(SIGNAL(data->onShow));
 		data->_not_spontaneous = false;
 	}
+	return false;
 }
 
-static void cb_hide (GtkWidget *widget, gMainWindow *data)
+static gboolean cb_hide(GtkWidget *widget, gMainWindow *data)
 {
 	data->emit(SIGNAL(data->onHide));
 	data->_not_spontaneous = false;
+	return false;
 	//if (data == gDesktop::activeWindow())
 	//	gMainWindow::setActiveWindow(NULL);
 }
@@ -76,22 +92,6 @@ static gboolean win_close(GtkWidget *widget,GdkEvent  *event,gMainWindow *data)
 	return true;
 }
 
-static gboolean resize_later(gMainWindow *data)
-{
-	data->bufW = data->_next_w;
-	data->bufH = data->_next_h;
-	data->_next_timer = 0;
-	data->configure();
-	data->performArrange();
-
-	if (data->onResize) 
-		data->onResize(data);
-	
-	//data->refresh();
-	
-	return false;
-}
-
 static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainWindow *data)
 {
 	gint x, y;
@@ -99,12 +99,17 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 	if (data->opened)
 	{
 		if (data->isTopLevel())
+		{
 			gtk_window_get_position(GTK_WINDOW(data->border), &x, &y);
+		}
 		else
 		{
 			x = event->x;
 			y = event->y;
 		}
+		
+		//fprintf(stderr, "cb_configure: %s: (%d %d %d %d) -> (%d %d %d %d) window = %p resized = %d\n", data->name(), data->bufX, data->bufY, data->bufW, data->bufH, x, y, event->width, event->height, event->window, data->_resized);
+	
 		if (x != data->bufX || y != data->bufY)
 		{
 			data->bufX = x;
@@ -114,24 +119,10 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 		
 		if ((event->width != data->bufW) || (event->height != data->bufH) || (data->_resized) || !event->window)
 		{
-			data->_next_w = event->width;
-			data->_next_h = event->height;
-/*			data->bufW=event->width;
-			data->bufH=event->height;
-			//g_debug("gMainWindow: cb_configure: %d", event->send_event);
-			data->performArrange();
-			if (data->onResize) data->onResize(data);*/
-			//if (data->_resized || data->parent())
-			//{
-				data->_resized = false;
-				resize_later(data);
-			/*}
-			else
-			{
-				//resize_later(data);
-				if (!data->_next_timer)
-					data->_next_timer = g_timeout_add(50, (GSourceFunc)resize_later, data);
-			}*/
+			data->_resized = false;
+			data->bufW = event->width;
+			data->bufH = event->height;
+			data->emitResize();
 		}
 	}
 	
@@ -140,10 +131,23 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 
 static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gMainWindow *data)
 {
-	if (data->_background)
+	if (data->_picture)
 	{
-		gdk_window_clear(data->border->window);
-		//gdk_window_clear(GTK_LAYOUT(data->widget)->bin_window);
+		cairo_t *cr = gdk_cairo_create(wid->window);
+		cairo_pattern_t *pattern;
+
+		gdk_cairo_region(cr, e->region);
+		cairo_clip(cr);
+
+		pattern = cairo_pattern_create_for_surface(data->_picture->getSurface());
+		cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
+
+		cairo_set_source(cr, pattern);
+		cairo_rectangle(cr, 0, 0, data->width(), data->height());
+		cairo_fill(cr);
+
+		cairo_pattern_destroy(pattern);
+		cairo_destroy(cr);
 	}
 	
 	return false;
@@ -181,15 +185,15 @@ void gMainWindow::initialize()
 	_not_spontaneous = false;
 	_skip_taskbar = false;
 	_current = NULL;
-	_background = NULL;
 	_style = NULL;
-	_next_timer = 0;
 	_xembed = false;
 	_activate = false;
 	_hidden = false;
 	_hideMenuBar = false;
 	_showMenuBar = true;
 	_popup = false;
+	_maximized = _minimized = _fullscreen = false;
+	_resize_last_w = _resize_last_h = -1;
 
 	onOpen = NULL;
 	onShow = NULL;
@@ -256,7 +260,7 @@ gMainWindow::gMainWindow(int plug) : gContainer(NULL)
 	gtk_widget_realize(border);
 	gtk_widget_show(widget);
 	
-	gtk_widget_set_size_request(border, 4, 4);
+	gtk_widget_set_size_request(border, 1, 1);
 }
 
 gMainWindow::gMainWindow(gContainer *par) : gContainer(par)
@@ -264,7 +268,7 @@ gMainWindow::gMainWindow(gContainer *par) : gContainer(par)
 	initialize();
 	g_typ = Type_gMainWindow;
 	
-	border = gtk_event_box_new();
+	border = gtk_alignment_new(0,0,1,1); //gtk_fixed_new(); //gtk_event_box_new();
 	widget = gtk_fixed_new();
 	
 	realize(false);
@@ -284,19 +288,18 @@ gMainWindow::~gMainWindow()
 			gApplication::exitLoop(this);
 	}
 	
-	if (_next_timer)
-		g_source_remove(_next_timer);
-	
 	gPicture::assign(&_picture);
 	gPicture::assign(&_icon);
 	if (_title) g_free(_title);
 	g_object_unref(accel);
-	if (_background) g_object_unref(_background);
 	if (_style) g_object_unref(_style);
 	
 	if (_active == this)
 		_active = NULL;
 	
+	if (gApplication::mainWindow() == this)
+		gApplication::setMainWindow(NULL);
+	
 	windows = g_list_remove(windows, (gpointer)this);
 }
 
@@ -358,13 +361,29 @@ void gMainWindow::setRealForeground(gColor color)
 
 void gMainWindow::move(int x, int y)
 {
+	gint ox, oy;
+	
 	if (isTopLevel())
 	{
 		if (x == bufX && y == bufY) 
 			return;
 	
+		#ifdef GDK_WINDOWING_X11
+		gdk_window_get_origin(border->window, &ox, &oy);
+		ox = x + ox - bufX;
+		oy = y + oy - bufY;
 		bufX = x;
 		bufY = y;
+		if (bufW > 0 && bufH > 0)
+		{
+			if (!X11_send_move_resize_event(GDK_WINDOW_XID(border->window), ox, oy, width(), height()))
+				return;
+		}
+		#else
+		bufX = x;
+		bufY = y;
+		#endif
+		
 		gtk_window_move(GTK_WINDOW(border), x, y);
 	}
 	else
@@ -383,6 +402,8 @@ void gMainWindow::resize(int w, int h)
 		
 	if (isTopLevel())
 	{
+		//fprintf(stderr, "resize: %s: %d %d\n", name(), w, h);
+	
 		//gdk_window_enable_synchronized_configure (border->window);
 		
 		bufW = w < 0 ? 0 : w;
@@ -395,15 +416,13 @@ void gMainWindow::resize(int w, int h)
 		}
 		else
 		{
-			//fprintf(stderr, "resize: %d %d (%d)\n", w, h, gtk_window_get_resizable(GTK_WINDOW(border)));
 			if (isResizable())
-				gtk_window_resize(GTK_WINDOW(border),w, h);
+				gtk_window_resize(GTK_WINDOW(border), w, h);
 			else
 				gtk_widget_set_size_request(border, w, h);
 				
 			if (visible)
 				gtk_widget_show(border);
-			
 		}
 	}
 	else
@@ -413,38 +432,57 @@ void gMainWindow::resize(int w, int h)
 	}
 }
 
+void gMainWindow::moveResize(int x, int y, int w, int h)
+{
+	//if (isTopLevel())
+	//	gdk_window_move_resize(border->window, x, y, w, h);
+	//else
+		gContainer::moveResize(x, y, w, h);
+}
+
 void gMainWindow::emitOpen()
 {
+	//fprintf(stderr, "emit Open: %p (%d %d) %d resizable = %d fullscreen = %d\n", this, width(), height(), opened, isResizable(), fullscreen());
+	
 	if (!opened)
 	{
-		//fprintf(stderr, "emit Open: %p (%d %d) resizable = %d\n", this, width(), height(), isResizable());
 		opened = true;
+		//_no_resize_event = true; // If the event loop is run during emitOpen(), some spurious configure events are received.
 		
-		if (isTopLevel()) 
+		// FIXME: the following does not work, it makes the window growing endlessly!
+			
+		/*if (isTopLevel()) 
 		{
-			gtk_widget_realize(border);
-			/*if (modal())
+			GdkGeometry geometry = { 0 };
+	
+			if (isModal())
 			{
-				GdkGeometry geometry;
-				
-				geometry.min_width = bufW;
-				geometry.min_height = bufH;
-				
-				fprintf(stderr, "gtk_window_set_geometry_hints: %d %d\n", bufW, bufH);
-				
-				gtk_window_set_geometry_hints(GTK_WINDOW(border), border, &geometry, GDK_HINT_MIN_SIZE);
-			}*/
-		}
+				geometry.min_width = 2; //bufW;
+				geometry.min_height = 2; //bufH;
+			}
+			else
+			{
+				geometry.min_width = 0;
+				geometry.min_height = 0;
+			}
+			
+			fprintf(stderr, "gtk_window_set_geometry_hints: %d %d\n", geometry.min_width, geometry.min_height);
+			gtk_window_set_geometry_hints(GTK_WINDOW(border), border, &geometry, GDK_HINT_MIN_SIZE);
+		}*/
 		
+		gtk_widget_realize(border);
+			
 		performArrange();
 		emit(SIGNAL(onOpen));
 		if (opened)
 		{
 			//fprintf(stderr, "emit Move & Resize: %p\n", this);			
 			emit(SIGNAL(onMove));
-			emit(SIGNAL(onResize));
+			emitResize();
 		}
 	}
+	
+	//_no_resize_event = false;
 }
 
 void gMainWindow::afterShow()
@@ -458,10 +496,13 @@ void gMainWindow::afterShow()
 
 void gMainWindow::setVisible(bool vl)
 {
-  gMainWindow *active;
+	if (vl == isVisible())
+		return;
 
 	if (vl)
 	{
+		bool arr = !isVisible();
+	
 		emitOpen();
 		if (!opened)
 			return;
@@ -475,13 +516,12 @@ void gMainWindow::setVisible(bool vl)
 			if (!_title || !*_title)
 				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
 			
-			if (!_xembed)
+			/*if (!_xembed)
 			{
+				fprintf(stderr, "gtk_window_group_add_window: %p -> %p\n", border, gApplication::currentGroup());
 				gtk_window_group_add_window(gApplication::currentGroup(), GTK_WINDOW(border));
-				active = gDesktop::activeWindow();
-				if (isModal() && active && active != this)
-					gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(active->topLevel()->border));
-			}
+				fprintf(stderr, "-> %p\n", gtk_window_get_group(GTK_WINDOW(border)));
+			}*/
 			
 			// Thanks for Ubuntu's GTK+ patching :-(
 			#if GTK_CHECK_VERSION(3,0,0)
@@ -499,7 +539,17 @@ void gMainWindow::setVisible(bool vl)
 			}
 			else
 				gtk_window_present(GTK_WINDOW(border));
-			//gtk_window_move(GTK_WINDOW(border), bufX, bufY);
+
+			if (isUtility())
+			{
+				gMainWindow *parent = _current;
+				
+				if (!parent && gApplication::mainWindow() && gApplication::mainWindow() != this)
+					parent = gApplication::mainWindow();
+				
+				if (parent)
+					gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(parent->border));
+			}
 		}
 		else 
 		{
@@ -518,6 +568,9 @@ void gMainWindow::setVisible(bool vl)
 		
 		if (skipTaskBar())
 			_activate = true;
+
+		if (arr)
+			performArrange();
 	}
 	else
 	{
@@ -526,7 +579,7 @@ void gMainWindow::setVisible(bool vl)
 			
 		_not_spontaneous = visible;
 		_hidden = true;
-		gControl::setVisible(false);
+		gContainer::setVisible(false);
 		
 		if (_popup)
 			gApplication::exitLoop(this);
@@ -537,58 +590,55 @@ void gMainWindow::setVisible(bool vl)
 void gMainWindow::setMinimized(bool vl)
 {
 	if (!isTopLevel()) return;
+	
+	_minimized = vl;
 	if (vl) gtk_window_iconify(GTK_WINDOW(border));
 	else    gtk_window_deiconify(GTK_WINDOW(border));
 }
 
 void gMainWindow::setMaximized(bool vl)
 {
-	if (!isTopLevel()) return;
-	if (vl) gtk_window_maximize(GTK_WINDOW(border));
-	else    gtk_window_unmaximize(GTK_WINDOW(border));
+	if (!isTopLevel())
+		return;
+
+	_maximized = vl;
+	
+	if (vl)
+		gtk_window_maximize(GTK_WINDOW(border));
+	else
+		gtk_window_unmaximize(GTK_WINDOW(border));
 }
 
 void gMainWindow::setFullscreen(bool vl)
 {
-	if (!isTopLevel()) return;
-	if (vl) gtk_window_fullscreen(GTK_WINDOW(border));
-	else    gtk_window_unfullscreen(GTK_WINDOW(border));
-}
-
-bool gMainWindow::minimized()
-{
-	if (isTopLevel())
-		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_ICONIFIED);
+	if (!isTopLevel())
+		return;
+	
+	_fullscreen = vl;
+	
+	if (vl)
+	{
+		gtk_window_fullscreen(GTK_WINDOW(border));
+		if (isVisible())
+			gtk_window_present(GTK_WINDOW(border));
+	}
 	else
-		return false;
-}
-
-bool gMainWindow::maximized()
-{
-	if (isTopLevel())
-		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_MAXIMIZED);
-	else
-		return false;
-}
-
-bool gMainWindow::fullscreen()
-{
-	if (isTopLevel())
-		return (bool)(gdk_window_get_state(border->window) &  GDK_WINDOW_STATE_FULLSCREEN);
-	else
-		return false;
+		gtk_window_unfullscreen(GTK_WINDOW(border));
 }
 
 void gMainWindow::center()
 {
-	int myx,myy;
+	GdkRectangle rect;
+	int x, y;
 	
 	if (!isTopLevel()) return;
 	
-	myx = (gDesktop::width() - width()) / 2;
-	myy = (gDesktop::height() - height()) / 2;
+	gDesktop::availableGeometry(screen(), &rect);
 	
-	move(myx, myy);
+	x = rect.x + (rect.width - width()) / 2;
+	y = rect.y + (rect.height - height()) / 2;
+	
+	move(x, y);
 }
 
 bool gMainWindow::isModal() const
@@ -605,24 +655,26 @@ void gMainWindow::showModal()
 	if (!isTopLevel()) return;
 	if (isModal()) return;
 	
-	save = _current;
-	_current = this;
-	
-	gtk_window_set_modal(GTK_WINDOW(border), true);
-  
-  center();
 	//show();
 
-	//fprintf(stderr, "showModal: begin %p\n", this);
+	gtk_window_set_modal(GTK_WINDOW(border), true);
+  center();
+	//show();
+	gtk_grab_add(border);
+	
+	if (_active)
+		gtk_window_set_transient_for(GTK_WINDOW(border), GTK_WINDOW(_active->topLevel()->border));
+	
+	save = _current;
+	_current = this;
 
 	gApplication::enterLoop(this, true);
 	
-	//fprintf(stderr, "showModal: end %p\n", this);
-
 	_current = save;
 	
+	gtk_grab_remove(border);
 	gtk_window_set_modal(GTK_WINDOW(border), false);
-	
+
 	if (!persistent)
 		destroyNow();
 	else
@@ -742,6 +794,11 @@ void gMainWindow::setBorder(bool b)
   	return;
 	
 	gtk_window_set_decorated(GTK_WINDOW(border), b);
+	/*#ifdef GDK_WINDOWING_X11
+	XSetWindowAttributes attr;
+	attr.override_redirect = !b;
+	XChangeWindowAttributes(GDK_WINDOW_XDISPLAY(border->window), GDK_WINDOW_XID(border->window), CWOverrideRedirect, &attr);
+	#endif*/
 }
 
 void gMainWindow::setResizable(bool b)
@@ -751,19 +808,16 @@ void gMainWindow::setResizable(bool b)
 	
 	if (b == isResizable())
 		return;
-	
-	//fprintf(stderr, "setResizable: %d %d %d\n", b, bufW, bufH);
-	
+
+	gtk_window_set_resizable(GTK_WINDOW(border), b);
+
 	if (b)
 		gtk_widget_set_size_request(border, 1, 1);
 	else
 		gtk_widget_set_size_request(border, bufW, bufH);
-
-	gtk_window_set_resizable(GTK_WINDOW(border), b);
 }
 
 
-
 void gMainWindow::setSkipTaskBar(bool b)
 {
 	_skip_taskbar = b;
@@ -844,7 +898,6 @@ void gMainWindow::remap()
 void gMainWindow::drawMask()
 {
 	GdkBitmap *mask;
-	GdkPixmap *back;
 	bool do_remap = false;
 	 
 	/*if (win_style) 
@@ -870,9 +923,6 @@ void gMainWindow::drawMask()
 	if (!isVisible())
 		return;
 	
-	//if (_background)
-	//	g_object_unref(_background);
-		
 	mask = (_mask && _picture) ? _picture->getMask() : NULL;
 	do_remap = !mask && _masked;
 	
@@ -883,32 +933,26 @@ void gMainWindow::drawMask()
 	//gdk_window_shape_combine_mask(border->window, mask, 0, 0);
   #endif
 
-	back = _picture ? _picture->getPixmap() : NULL;
-	gtk_widget_set_double_buffered(border, back == NULL);	
-	gtk_widget_set_double_buffered(widget, back == NULL);	
+	//back = _picture ? _picture->getPixmap() : NULL;
+	//gtk_widget_set_double_buffered(border, back == NULL);
+	//gtk_widget_set_double_buffered(widget, back == NULL);
 	
-	if (back)
+	if (_picture)
 	{
 		gtk_widget_realize(border);
 		gtk_widget_realize(widget);
-		gdk_window_set_back_pixmap(border->window, back, FALSE);
-		//gdk_window_set_back_pixmap(GTK_LAYOUT(widget)->bin_window, back, FALSE);
-		gdk_window_clear(border->window);
-		//gdk_window_clear(GTK_LAYOUT(widget)->bin_window);
-		gtk_widget_set_app_paintable(border, true);
-		gtk_widget_set_app_paintable(widget, true);
+		//gdk_window_set_back_pixmap(border->window, back, FALSE);
+		//gdk_window_clear(widget->window);
+		//gtk_widget_set_app_paintable(border, true);
+		//gtk_widget_set_app_paintable(widget, true);
+		for (int i = 0; i < controlCount(); i++)
+			getControl(i)->refresh();
 	}
 	else
 	{
 		setRealBackground(background());
 	}
 	
-	if (_background)
-		g_object_unref(_background);
-	_background = back;
-	if (_background)
-		g_object_ref(_background);
-	
 	_masked = mask != NULL;
 	
 	if (do_remap)
@@ -921,60 +965,6 @@ void gMainWindow::drawMask()
 			setSkipTaskBar(false);
 		}
 	}
-	
-	//gtk_widget_queue_draw(border);
-	
-	#if 0
-	GdkBitmap *map=NULL;
-	GdkPixmap *pix=NULL;
-	GdkColor black;
-	gint pw,ph;
-	GdkPixbuf *buf_mask = _picture ? _picture->getPixbuf() : NULL;	
-	
-	if (win_style) { 
-		g_object_unref(win_style); 
-		win_style=NULL; 
-	}
-	
-	if (buf_mask)
-	{
-		pw=gdk_pixbuf_get_width(buf_mask);
-		ph=gdk_pixbuf_get_height(buf_mask);
-		
-		map=gdk_pixmap_new(NULL,pw,ph,1);
-		gdk_pixbuf_render_threshold_alpha(buf_mask,map,0,0,0,0,pw,ph,128);
-		
-		pix=gdk_pixmap_new(border->window,pw,ph,-1);
-		GdkGC* gc=gdk_gc_new(border->window);
-		fill_gdk_color(&black,0);
-		gdk_gc_set_foreground (gc,&black);
-		gdk_gc_set_background (gc,&black);
-		gdk_draw_rectangle(pix,gc,true,0,0,pw,ph);
-		gdk_draw_pixbuf(pix,gc,buf_mask,0,0,0,0,pw,ph,GDK_RGB_DITHER_MAX,0,0);
-		g_object_unref(G_OBJECT(gc));
-	}
-	
-	if (buf_mask && _mask) gtk_widget_shape_combine_mask(border,map,0,0);
-	else        gtk_widget_shape_combine_mask(border,NULL,0,0);
-	
-	if (map)
-	{
-		win_style=gtk_style_copy(widget->style);
-		win_style->bg_pixmap[GTK_STATE_NORMAL]=pix;
-		gtk_widget_set_style(widget,win_style);
-		gdk_window_invalidate_rect( border->window,NULL,true);
-		g_object_unref(G_OBJECT(map));
-	}
-	else
-	{
-		gtk_widget_set_style(widget,NULL);
-		if (border->window) //GTK_WIDGET_REALIZED(widget))
-		{
-		  gtk_widget_reset_shapes(border);
-		  gdk_window_invalidate_rect(border->window,NULL,true);
-    }
-	}
-	#endif
 }
 
 int gMainWindow::menuCount()
@@ -1036,13 +1026,28 @@ bool gMainWindow::close()
 	return doClose();
 }
 
+static void hide_hidden_children(gContainer *cont)
+{
+	int i;
+	gControl *child;
+	
+	for (i = 0;; i++)
+	{
+		child = cont->child(i);
+		if (!child)
+			break;
+		if (!child->isVisible())
+			gtk_widget_hide(child->border);
+		else if (child->isContainer())
+			hide_hidden_children((gContainer *)child);
+	}
+}
+
 void gMainWindow::reparent(gContainer *newpr, int x, int y)
 {
 	GtkWidget *new_border;
 	int w, h;
 	gColor fg, bg;
-	int i;
-	gControl *child;
 	
 	if (_xembed)
 		return;
@@ -1075,18 +1080,12 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		
 		checkMenuBar();
 		
+		bufX = bufY = 0;
 		move(x, y);
 		gtk_widget_set_size_request(border, width(), height());
 		
 		// Hidden children are incorrectly shown. Fix that!
-		for (i = 0;; i++)
-		{
-			child = getControl(i);
-			if (!child)
-				break;
-			if (!child->isVisible())
-				child->setVisible(false);
-		}
+		hideHiddenChildren();
 	}
 	else if ((!isTopLevel() && !newpr)
 	         || (isTopLevel() && isPopup()))
@@ -1122,10 +1121,11 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		bufW = bufH = -1;
 		resize(w, h);
 		
+		hideHiddenChildren();
 		_popup = false; //type == GTK_WINDOW_POPUP;
 	}
 	else
-		gControl::reparent(newpr, x, y);	
+		gContainer::reparent(newpr, x, y);	
 }
 
 
@@ -1138,7 +1138,7 @@ int gMainWindow::controlCount()
 	while (list)
 	{
 		ctrl = (gControl *)list->data;
-		if (ctrl->window() == this)
+		if (ctrl->window() == this && !ctrl->isDestroyed())
 			n++;
 		list = g_list_next(list);
 	}
@@ -1154,7 +1154,7 @@ gControl *gMainWindow::getControl(char *name)
 	while (list)
 	{
 		ctrl = (gControl *)list->data;
-		if (ctrl->window() == this && !strcasecmp(ctrl->name(), name))
+		if (ctrl->window() == this && !strcasecmp(ctrl->name(), name) && !ctrl->isDestroyed())
 			return ctrl;
 		list = g_list_next(list);
 	}
@@ -1171,7 +1171,7 @@ gControl *gMainWindow::getControl(int index)
 	while (list)
 	{
 		ctrl = (gControl *)list->data;
-		if (ctrl->window() == this)
+		if (ctrl->window() == this && !ctrl->isDestroyed())
 		{
 			if (i == index)
 				return ctrl;
@@ -1196,15 +1196,10 @@ int gMainWindow::containerX()
 
 int gMainWindow::clientY()
 {
-	GtkRequisition req;
-
-	if (menuBar && GTK_WIDGET_VISIBLE(GTK_WIDGET(menuBar)))
-	{
-		gtk_widget_size_request(GTK_WIDGET(menuBar), &req);
-		return req.height;
-	}
-	
-	return 0;
+	if (isMenuBarVisible())
+		return menuBarHeight();
+	else
+		return 0;
 }
 
 int gMainWindow::containerY()
@@ -1226,9 +1221,11 @@ int gMainWindow::menuBarHeight()
 
 	if (menuBar)
 	{
-		gtk_widget_show(GTK_WIDGET(menuBar));
+		//gtk_widget_show(GTK_WIDGET(menuBar));
+		//fprintf(stderr, "menuBarHeight: gtk_widget_get_visible: %d\n", gtk_widget_get_visible(GTK_WIDGET(menuBar)));
 		gtk_widget_size_request(GTK_WIDGET(menuBar), &req);
 		h = req.height;
+		//fprintf(stderr, "menuBarHeight: %d\n", h);
 	}
 	
 	return h;
@@ -1262,27 +1259,25 @@ void gMainWindow::setActiveWindow(gControl *control)
 }
 
 #ifdef GDK_WINDOWING_X11
-int gMainWindow::getType()
+bool gMainWindow::isUtility()
 {
-	if (!isTopLevel())
-		return 0;
-	return _type; //X11_get_window_type(handle());
+	return gtk_window_get_type_hint(GTK_WINDOW(border)) == GDK_WINDOW_TYPE_HINT_UTILITY;
 }
 
-void gMainWindow::setType(int type)
+void gMainWindow::setUtility(bool v)
 {
 	if (!isTopLevel())
 		return;
-	X11_set_window_type(handle(), type);
-	_type = type;
+	
+	gtk_window_set_type_hint(GTK_WINDOW(border), v ? GDK_WINDOW_TYPE_HINT_UTILITY : GDK_WINDOW_TYPE_HINT_NORMAL);
 }
 #else
-int gMainWindow::getType()
+bool gMainWindow::isUtility()
 {
-	return 0;
+	return false;
 }
 
-void gMainWindow::setType()
+void gMainWindow::setUtility(bool v)
 {
 }
 #endif
@@ -1296,13 +1291,15 @@ void gMainWindow::configure()
 	
 	h = menuBarHeight();
 	
+	//fprintf(stderr, "configure: %s: %d %d - %d %d\n", name(), isMenuBarVisible(), h, width(), height());
+	
 	if (isMenuBarVisible())
 	{
 		gtk_fixed_move(layout, GTK_WIDGET(menuBar), 0, 0);
 		if (h > 1)
 			gtk_widget_set_size_request(GTK_WIDGET(menuBar), width(), h);
 		gtk_fixed_move(layout, widget, 0, h);
-		gtk_widget_set_size_request(widget, width(), height() - h);
+		gtk_widget_set_size_request(widget, width(), Max(0, height() - h));
 	}
 	else
 	{
@@ -1311,8 +1308,8 @@ void gMainWindow::configure()
 			if (menuBar)
 				gtk_fixed_move(layout, GTK_WIDGET(menuBar), 0, -h);
 			gtk_fixed_move(layout, widget, 0, 0);
-			gtk_widget_set_size_request(widget, width(), height());
 		}
+		gtk_widget_set_size_request(widget, width(), height());
 	}
 }
 
@@ -1329,6 +1326,7 @@ void gMainWindow::setMenuBarVisible(bool v)
 
 bool gMainWindow::isMenuBarVisible()
 {
+	//fprintf(stderr, "isMenuBarVisible: %d\n", !!(menuBar && !_hideMenuBar && _showMenuBar));
 	return menuBar && !_hideMenuBar && _showMenuBar; //|| (menuBar && GTK_WIDGET_MAPPED(GTK_WIDGET(menuBar)));
 }
 
@@ -1343,6 +1341,8 @@ void gMainWindow::checkMenuBar()
 	int i;
 	gMenu *menu;
 
+	//fprintf(stderr, "gMainWindow::checkMenuBar\n");
+	
 	if (menuBar)
 	{
 		_hideMenuBar = true;
@@ -1360,6 +1360,7 @@ void gMainWindow::checkMenuBar()
 	}
 		
 	configure();
+	performArrange();
 }
 
 void gMainWindow::embedMenuBar(GtkWidget *border)
@@ -1392,6 +1393,13 @@ void gMainWindow::embedMenuBar(GtkWidget *border)
 	}
 }
 
+/*void gMainWindow::getScreenPos(int *x, int *y)
+{
+	gContainer::getScreenPos(x, y);
+	//if (y && isMenuBarVisible())
+	//	*y += menuBarHeight();
+}*/
+
 double gMainWindow::opacity()
 {
 	if (isTopLevel())
@@ -1409,5 +1417,17 @@ void gMainWindow::setOpacity(double v)
 int gMainWindow::screen()
 {
 	gMainWindow *tl = topLevel();
-	return gdk_screen_get_monitor_at_window(gtk_window_get_screen(GTK_WINDOW(tl->border)), tl->border->window);
+	return gdk_screen_get_number(gtk_window_get_screen(GTK_WINDOW(tl->border)));
 }
+
+void gMainWindow::emitResize()
+{
+	if (bufW == _resize_last_w && bufH == _resize_last_h)
+		return;
+	
+	_resize_last_w = bufW;
+	_resize_last_h = bufH;
+	configure();
+	performArrange();
+	emit(SIGNAL(onResize));
+}
\ No newline at end of file
