@@ -2,7 +2,7 @@
 
   gmainwindow.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ public:
 //"Properties"
 	bool hasBorder();
 	bool isResizable();
-	int getType();
+	bool isUtility();
 	gPicture *icon() { return _icon; }
 	gPicture *picture() { return _picture; }
 	bool mask() { return _mask; }
@@ -45,9 +45,9 @@ public:
 	const char* text();
 	bool topOnly();
 	bool skipTaskBar();
-	bool minimized();
-	bool maximized();
-	bool fullscreen();
+	bool minimized() const { return _minimized; }
+	bool maximized() const { return _maximized; }
+	bool fullscreen() const { return _fullscreen; }
 	bool getSticky();
 	int  getStacking();
 	bool isPersistent() const { return persistent; }
@@ -63,7 +63,7 @@ public:
 
 	void setBorder(bool b);
 	void setResizable(bool b);
-	void setType(int type);
+	void setUtility(bool v);
 	void setIcon(gPicture *pic);
 	void setMask(bool vl);
 	void setPicture(gPicture *pic);
@@ -88,6 +88,8 @@ public:
 	virtual int clientY();
 	virtual int containerX();
 	virtual int containerY();
+
+	//virtual void getScreenPos(int *x, int *y);
 	
 	bool spontaneous() { return !_not_spontaneous; }
 	
@@ -105,6 +107,7 @@ public:
 	void raise();
 	virtual void move(int x, int y);
 	virtual void resize(int w, int h);
+	virtual void moveResize(int x, int y, int w, int h);
 	bool close();
 	virtual void reparent(gContainer *newpr, int x, int y);
 
@@ -139,6 +142,7 @@ public:
 	int menuBarHeight();
 	void configure();
 	void embedMenuBar(GtkWidget *border);
+	void emitResize();
 	
 	GtkWindowGroup *group;
 	GtkAccelGroup *accel;
@@ -149,15 +153,14 @@ public:
 	gPicture *_icon;
 	gPicture *_picture;
 	char *_title;
-	GdkPixmap *_background;
 	GtkStyle *_style;
 	
 	gControl *focus;
 	gButton *_default;
 	gButton *_cancel;
 	
-	int _next_w, _next_h;
-	guint _next_timer;
+	int _resize_last_w;
+	int _resize_last_h;
 	
 	unsigned _mask : 1;
 	unsigned top_only : 1;
@@ -175,6 +178,9 @@ public:
 	unsigned _hideMenuBar : 1;
 	unsigned _showMenuBar : 1;
 	unsigned _popup : 1;
+	unsigned _maximized : 1;
+	unsigned _minimized : 1;
+	unsigned _fullscreen : 1;
 };
 
 #endif
