@@ -34,7 +34,10 @@ typedef
 		}
 	MenuPosition;
 
-static GList *menus=NULL;
+gMenu *gMenu::_current_popup = NULL;
+int gMenu::_in_popup = 0;
+
+static GList *menus = NULL;
 
 static void mnu_destroy (GtkWidget *object, gMenu *data)
 {
@@ -378,7 +381,13 @@ void gMenu::initialize()
 }
 
 
-gMenu::gMenu(gMainWindow *par,bool hidden)
+static gboolean cb_menubar_changed(GtkWidget *widget, gMainWindow *data)
+{
+	data->configure();
+	return false;
+}
+
+gMenu::gMenu(gMainWindow *par, bool hidden)
 {
 	pr = (gpointer)par;
   initialize();
@@ -389,7 +398,9 @@ gMenu::gMenu(gMainWindow *par,bool hidden)
 	
 	if (!par->menuBar)
 	{
-		par->menuBar=(GtkMenuBar*)gtk_menu_bar_new();
+		par->menuBar = (GtkMenuBar*)gtk_menu_bar_new();
+		g_signal_connect_after(G_OBJECT(par->menuBar), "map", G_CALLBACK(cb_menubar_changed), (gpointer)par);
+		g_signal_connect(G_OBJECT(par->menuBar),"unmap", G_CALLBACK(cb_menubar_changed), (gpointer)par);
 		par->embedMenuBar(par->border);
 	}
 	
@@ -472,6 +483,9 @@ gMenu::~gMenu()
 	if (menu)
 		gtk_widget_destroy(GTK_WIDGET(menu));
 	
+	if (_current_popup == this)
+		_current_popup = NULL;
+	
 	if (onFinish) onFinish(this);
 }
 
@@ -515,7 +529,10 @@ void gMenu::updateVisible()
 	if (top_level && _style != MENU)
 		vl = false;
 	
-	g_object_set(G_OBJECT(menu),"visible",vl,(void *)NULL);
+	//fprintf(stderr, "gMenu::updateVisible: %s '%s' %d\n", name(), text(), vl);
+	
+	gtk_widget_set_visible(GTK_WIDGET(menu), vl);
+	//g_object_set(G_OBJECT(menu),"visible",vl,(void *)NULL);
 	
 	if (top_level && pr)
 		((gMainWindow *)pr)->checkMenuBar();
@@ -553,9 +570,9 @@ int gMenu::childCount()
 	GList *item;
 	gMenu *mn;
 	int ct=0;
-	
+
 	if (!menus) return 0;
-	
+
 	item=g_list_first(menus);
 	while (item)
 	{
@@ -563,7 +580,7 @@ int gMenu::childCount()
 		if (mn->pr == (void*)this) ct++;
 		item=g_list_next(item);
 	}
-	
+
 	return ct;
 }
 
@@ -572,9 +589,9 @@ gMenu* gMenu::childMenu(int pos)
 	GList *item;
 	gMenu *mn;
 	int ct=0;
-	
+
 	if (!menus) return NULL;
-	
+
 	item=g_list_first(menus);
 	while (item)
 	{
@@ -586,7 +603,7 @@ gMenu* gMenu::childMenu(int pos)
 		}
 		item=g_list_next(item);
 	}
-	
+
 	return NULL;
 }
 
@@ -606,6 +623,7 @@ static void position_menu(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, Me
 
 void gMenu::doPopup(bool move, int x, int y)
 {
+	gMenu *save_current_popup;
 	MenuPosition *pos = NULL;
 	
 	if (!child)
@@ -618,9 +636,23 @@ void gMenu::doPopup(bool move, int x, int y)
 		pos->y = y;
 	}
 	
+	save_current_popup = _current_popup;
+	_current_popup = this;
+	
+	_in_popup++;
+
 	gtk_menu_popup(child, NULL, NULL, move ? (GtkMenuPositionFunc)position_menu : NULL, (gpointer)pos, 0, gApplication::lastEventTime());
 	
-	while (child && GTK_WIDGET_MAPPED(child))
+	while (_current_popup && child && GTK_WIDGET_MAPPED(child))
+		MAIN_do_iteration(false);
+
+	_current_popup = save_current_popup;
+
+	_in_popup--;
+
+	// flush the event loop so that the main window is reactivated before the click menu event is raised
+
+	while (gtk_events_pending())
 		MAIN_do_iteration(false);
 }
 
@@ -869,3 +901,4 @@ bool gMenu::isTearOff() const
 	else
 		return false;
 }*/
+
