@@ -2,7 +2,7 @@
 
   gmenu.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@ public:
 	char *name() const { return _name; }
 	bool topLevel() const { return top_level; }
 	bool isSeparator() const { return _style == SEPARATOR; }
+	void *parent() const { return pr; }
 
 	void setChecked(bool vl);
 	void setToggle(bool vl);
@@ -78,6 +79,8 @@ public:
 	void popup();
 	void popup(int x, int y);
 	void destroy();
+	static bool insidePopup() { return _in_popup > 0; }
+	static gMenu *currentPopup() { return _current_popup; }
 
 // "Signals"
 	void (*onFinish)(gMenu *sender); // Special
@@ -102,7 +105,7 @@ public:
 	void initialize();
 	gMenuStyle style() const { return _style; }
   void hideSeparators();
-
+	
 private:
 
 	gMenuStyle _style, _oldstyle;
@@ -117,6 +120,9 @@ private:
 	unsigned top_level : 1;
 	unsigned _action : 1;
 	unsigned _visible : 1;
+
+	static gMenu *_current_popup;
+	static int _in_popup;
   
 	void doPopup(bool move, int x = 0, int y = 0);
   void update();
