@@ -42,6 +42,17 @@ static char *DIALOG_title=NULL;
 static gFont *DIALOG_font=NULL;
 static bool _dialog_show_hidden = false;
 
+
+static gboolean cb_show(GtkWidget *widget, gpointer data)
+{
+	GdkGeometry geometry;
+	geometry.min_width = gDesktop::scale() * 32;
+	geometry.min_height = gDesktop::scale() * 6;
+
+	gtk_window_set_geometry_hints(GTK_WINDOW(widget), widget, &geometry, (GdkWindowHints)(GDK_HINT_MIN_SIZE | GDK_HINT_POS));
+	return false;
+}
+
 static int run_dialog(GtkDialog *window)
 {
   gMainWindow *active;
@@ -58,7 +69,9 @@ static int run_dialog(GtkDialog *window)
 	gtk_window_present(GTK_WINDOW(window));
 	oldGroup = gApplication::enterGroup();
 	gApplication::_loopLevel++;
+	(*gApplication::onEnterEventLoop)();
 	ret = gtk_dialog_run(window);
+	(*gApplication::onLeaveEventLoop)();
 	gApplication::_loopLevel--;
 	gApplication::exitGroup(oldGroup);
 	return ret;
@@ -127,6 +140,8 @@ guint custom_dialog(const gchar *icon,GtkButtonsType btn,char *sg)
 	gdk_window_set_type_hint(msg->window,GDK_WINDOW_TYPE_HINT_UTILITY);
 	gtk_window_set_position(GTK_WINDOW(msg),GTK_WIN_POS_CENTER_ALWAYS);
 	
+	g_signal_connect(G_OBJECT(msg), "show", G_CALLBACK(cb_show), (gpointer)NULL);
+	
 	resp = run_dialog(GTK_DIALOG(msg));
 	gtk_widget_destroy(msg);
 	
@@ -503,7 +518,7 @@ bool gDialog::saveFile()
 	
 	if (DIALOG_path)
 	{
-		if (g_file_test(DIALOG_path, G_FILE_TEST_IS_DIR))
+		if (DIALOG_path[strlen(DIALOG_path) - 1] == '/' && g_file_test(DIALOG_path, G_FILE_TEST_IS_DIR))
 			gtk_file_chooser_set_current_folder((GtkFileChooser*)msg, DIALOG_path);
 		else
 			gtk_file_chooser_select_filename((GtkFileChooser*)msg, DIALOG_path);
