@@ -1,23 +1,23 @@
 /***************************************************************************
 
-  gpicture.cpp
+	gpicture.cpp
 
-  (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
+	(c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-  MA 02110-1301, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -35,13 +35,14 @@ gPicture
 
 #define LOAD_INC 65536L
 
-static bool pixbufFromMemory(GdkPixbuf **img, char *addr, unsigned int len, bool *trans)
+static bool pixbufFromMemory(GdkPixbuf **ppixbuf, char *addr, unsigned int len, bool *trans)
 {
-	GdkPixbufLoader* loader;
+	GdkPixbufLoader *loader;
+	GdkPixbuf *pixbuf;
 	GError *error = NULL;
 	gsize size;
 
-  *img = 0;
+	*ppixbuf = 0;
 
 	loader = gdk_pixbuf_loader_new();
 	
@@ -63,23 +64,24 @@ static bool pixbufFromMemory(GdkPixbuf **img, char *addr, unsigned int len, bool
 		goto __ERROR;
 	}
 	
-	(*img) = gdk_pixbuf_loader_get_pixbuf(loader);
-	g_object_ref(G_OBJECT(*img));
+	pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
+	g_object_ref(pixbuf);
 	
-	if (gdk_pixbuf_get_n_channels(*img) == 3)
+	if (gdk_pixbuf_get_n_channels(pixbuf) == 3)
 	{
 		// Rowstride breaks gb.image (it is rounded up so that a line is always a four bytes multiple).
- 		GdkPixbuf *aimg;
- 		aimg = gdk_pixbuf_add_alpha(*img, FALSE, 0, 0, 0);
- 		g_object_unref(G_OBJECT(*img));
-   	g_object_ref(G_OBJECT(aimg));
- 		*img = aimg;
+		GdkPixbuf *aimg;
+		aimg = gdk_pixbuf_add_alpha(pixbuf, FALSE, 0, 0, 0);
+		g_object_unref(pixbuf);
+		pixbuf = aimg;
 		*trans = false;
 	}
 	else
 		*trans = true;
 
 	g_object_unref(G_OBJECT(loader));
+
+	*ppixbuf = pixbuf;
 	return true;
 	
 __ERROR:
@@ -87,77 +89,43 @@ __ERROR:
 	return false;
 }
 
-/*static void render_pixbuf(gPicture *pic, GdkPixbuf *img)
-{
-	GdkColor col={0,0xFF00,0xFF00,0xFF00};
-	GdkColormap *map=gdk_colormap_get_system();
-	GdkGC *gc;
-	gint b_w,b_h;
-	gint depth;
-
-	pic->_transparent=false;
-
-	if (pic->pic) 
-	{
-	 g_object_unref(pic->pic);
-	 //delete pic->cache;
-	 //pic->cache = 0;
-	 pic->pic = 0;
-  }
-  
-	if (gdk_pixbuf_get_has_alpha(img)) {
-		pic->_transparent=true;
-		//pic->cache=new alphaCache(img);
-		//if (!pic->cache->active()) { delete pic->cache; pic->cache=NULL; }
-	}
-
-	b_w=gdk_pixbuf_get_width(img);
-	b_h=gdk_pixbuf_get_height(img);
-
-	depth=gdk_screen_get_system_visual(gdk_screen_get_default())->depth;
-	pic->pic=gdk_pixmap_new(NULL,b_w,b_h,depth);
-	gc=gdk_gc_new(pic->pic);
-	gdk_colormap_alloc_color(map,&col,FALSE,TRUE);
-	gdk_gc_set_foreground(gc,&col);
-	gdk_draw_rectangle(pic->pic,gc,TRUE,0,0,b_w,b_h);
-	g_object_unref(G_OBJECT(gc));
-	gdk_draw_pixbuf(pic->pic,NULL,img,0,0,0,0,b_w,b_h,GDK_RGB_DITHER_MAX,0,0);
-}*/
 
 void gPicture::initialize()
 {
-  pic = 0;
-  mask = 0;
-  img = 0;
-  _transparent = false;
-  _type = VOID;
-  _width = 0;
-  _height = 0;
+	pixmap = NULL;
+	mask = NULL;
+	pixbuf = NULL;
+	surface = NULL;
+	_transparent = false;
+	_type = VOID;
+	_width = 0;
+	_height = 0;
 }
 
 gPicture::gPicture() : gShare()
 {
-  initialize();
+	//fprintf(stderr, "gPicture(): %p\n", this);
+	initialize();
 }
 
 static GdkPixmap *create_pixmap(int w, int h)
 {
 	GdkScreen *scr;
 	gint depth;
-	GdkPixmap *pic;
+	GdkPixmap *pixmap;
 
-  scr = gdk_screen_get_default();
-  depth = (gdk_screen_get_system_visual(scr))->depth;
+	scr = gdk_screen_get_default();
+	depth = (gdk_screen_get_system_visual(scr))->depth;
 
-  pic = gdk_pixmap_new(NULL, w, h, depth);
-  gdk_drawable_set_colormap(GDK_DRAWABLE(pic), gdk_colormap_get_system());
-  return pic;
+	pixmap = gdk_pixmap_new(NULL, w, h, depth);
+	gdk_drawable_set_colormap(GDK_DRAWABLE(pixmap), gdk_colormap_get_system());
+	return pixmap;
 }
 
 void gPicture::createMask(bool opaque)
 {
 	GdkGC *gc;
-  GdkGCValues gc_values;
+	GdkGCValues gc_values;
 	
 	if (mask || !_transparent)
 		return;
@@ -165,38 +133,39 @@ void gPicture::createMask(bool opaque)
 	mask = gdk_pixmap_new(NULL, _width, _height, 1);
 	
 	gc_values.foreground.pixel = opaque ? 1 : 0;
-  gc = gdk_gc_new_with_values(mask, &gc_values, GDK_GC_FOREGROUND);
+	gc = gdk_gc_new_with_values(mask, &gc_values, GDK_GC_FOREGROUND);
 	
 	gdk_gc_set_fill(gc, GDK_SOLID);
-  gdk_draw_rectangle(mask, gc, TRUE, 0, 0, _width, _height);
-  
-  g_object_unref(gc);
+	gdk_draw_rectangle(mask, gc, TRUE, 0, 0, _width, _height);
+	
+	g_object_unref(gc);
 }
 
 gPicture::gPicture(gPictureType type, int w, int h, bool trans) : gShare()
 {
-  initialize();
-  
+	//fprintf(stderr, "gPicture(): %p: %d %d %d %d\n", this, type, w, h, trans);
+	initialize();
+	
 	_transparent = trans;
 
-  if (!type)
-    return;
+	if (!type)
+		return;
 
 	if (w <= 0 || h <= 0) return;
 
-  _type = type;
-  _width = w;
-  _height = h;
+	_type = type;
+	_width = w;
+	_height = h;
 
-  if (_type == SERVER)
-  {
-    pic = create_pixmap(w, h);
-    createMask(false);
-  }
-  else if (_type == MEMORY)
-  {
-  	img = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
-  }
+	if (_type == PIXMAP)
+	{
+		pixmap = create_pixmap(w, h);
+		createMask(false);
+	}
+	else if (_type == PIXBUF)
+	{
+		pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
+	}
 }
 
 
@@ -204,23 +173,24 @@ gPicture::gPicture(gPictureType type, int w, int h, bool trans) : gShare()
 
 gPicture::gPicture(GdkPixbuf *image, bool trans) : gShare()
 {
-  initialize();
-  if (!image)
-    return;
-    
-  _type = MEMORY;
-  _width = gdk_pixbuf_get_width(image);
-  _height = gdk_pixbuf_get_height(image);
-  _transparent = trans;
-  img = image;
+	//fprintf(stderr, "gPicture(image): %p: %p %d\n", this, image, trans);
+	initialize();
+	if (!image)
+		return;
+		
+	_type = PIXBUF;
+	_width = gdk_pixbuf_get_width(image);
+	_height = gdk_pixbuf_get_height(image);
+	_transparent = trans;
+	pixbuf = image;
 
-	if (gdk_pixbuf_get_n_channels(img) == 3)
+	if (gdk_pixbuf_get_n_channels(pixbuf) == 3)
 	{
 		GdkPixbuf *aimg;
-		aimg = gdk_pixbuf_add_alpha(img, FALSE, 0, 0, 0);
-		g_object_unref(G_OBJECT(img));
+		aimg = gdk_pixbuf_add_alpha(pixbuf, FALSE, 0, 0, 0);
+		g_object_unref(G_OBJECT(pixbuf));
 		//g_object_ref(G_OBJECT(aimg));
-		img = aimg;
+		pixbuf = aimg;
 		_transparent = false;
 	}
 
@@ -230,73 +200,105 @@ gPicture::gPicture(GdkPixbuf *image, bool trans) : gShare()
 
 gPicture::gPicture(GdkPixmap *pixmap) : gShare()
 {
-  initialize();
-  if (!pixmap)
-    return;
-    
-  _type = SERVER;
-  gdk_drawable_get_size((GdkDrawable *)pixmap, &_width, &_height);
-  pic = pixmap;
+	initialize();
+	if (!pixmap)
+		return;
+		
+	_type = PIXMAP;
+	gdk_drawable_get_size((GdkDrawable *)pixmap, &_width, &_height);
+	pixmap = pixmap;
 }
 
 gPicture::~gPicture()
 {
-  clear();
+	//fprintf(stderr, "~gPicture: %p\n", this);
+	clear();
 }
 
 void gPicture::invalidate()
 {
-	if (_type == MEMORY && pic) 
+	if (pixmap && _type != PIXMAP)
 	{
-    g_object_unref(G_OBJECT(pic));
-    pic = 0;
-    if (mask)
-    {
-    	g_object_unref(mask);
-    	mask = 0;
-    }
-  }
-  else if (_type == SERVER && img)
-  {
-    g_object_unref(G_OBJECT(img));
-    img = 0;
-  }
+		g_object_unref(G_OBJECT(pixmap));
+		pixmap = NULL;
+		if (mask)
+		{
+			g_object_unref(mask);
+			mask = NULL;
+		}
+	}
+	
+	if (pixbuf && _type != PIXBUF)
+	{
+		g_object_unref(G_OBJECT(pixbuf));
+		pixbuf = NULL;
+	}
+	
+	if (surface && _type != SURFACE)
+	{
+		cairo_surface_destroy(surface);
+		surface = NULL;
+	}
 }
 
 GdkPixbuf *gPicture::getPixbuf()
 {
-  if (_type == VOID)
-    return NULL;
-    
-  if (!img && pic)
-  {
-    img = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, width(), height());
-    gdk_pixbuf_get_from_drawable(img, pic, NULL, 0, 0, 0, 0, width(), height());
-    
-    if (mask)
-    {
-    	uchar *s, *d;
-    	int i;
-    	GdkPixbuf *alpha;
-	    
-	    alpha = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, width(), height());
-	    gdk_pixbuf_get_from_drawable(alpha, mask, NULL, 0, 0, 0, 0, width(), height());
-	    s = gdk_pixbuf_get_pixels(alpha);
-	    d = gdk_pixbuf_get_pixels(img) + 3;
-	    //fprintf(stderr, "mask -> pixbuf\n\n");
-	    for (i = 0; i < (_width * _height); i++)
-	    {
-	      //fprintf(stderr, "%08X%c", *((uint *)s), (1 + (i % _width)) == _width ? '\n' : ' ');
-	    	d[0] = s[0];
-	    	d += 4;
-	    	s += 4;
-	    }
-	    g_object_unref(alpha);
-    }
-  }
-  
-  _type = MEMORY;
-  return img;
+	if (_type == VOID)
+		return NULL;
+	
+	if (pixbuf)
+		return pixbuf;
+	
+	if (_type == PIXMAP)
+	{
+		pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, width(), height());
+		gdk_pixbuf_get_from_drawable(pixbuf, pixmap, NULL, 0, 0, 0, 0, width(), height());
+		
+		if (mask)
+		{
+			uchar *s, *d;
+			int i;
+			GdkPixbuf *alpha;
+			
+			alpha = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, width(), height());
+			gdk_pixbuf_get_from_drawable(alpha, mask, NULL, 0, 0, 0, 0, width(), height());
+			s = gdk_pixbuf_get_pixels(alpha);
+			d = gdk_pixbuf_get_pixels(pixbuf) + 3;
+			//fprintf(stderr, "mask -> pixbuf\n\n");
+			for (i = 0; i < (_width * _height); i++)
+			{
+				//fprintf(stderr, "%08X%c", *((uint *)s), (1 + (i % _width)) == _width ? '\n' : ' ');
+				d[0] = s[0];
+				d += 4;
+				s += 4;
+			}
+			g_object_unref(alpha);
+		}
+	}
+	else if (_type == SURFACE)
+	{
+		// Convert cairo surface to pixbuf
+		fprintf(stderr, "gb.gtk: warning: cairo surface to pixbuf conversion not implemented yet.\n");
+		return NULL;
+	}
+	
+	_type = PIXBUF;
+	return pixbuf;
+}
+
+cairo_surface_t *gPicture::getSurface()
+{
+	if (_type == VOID)
+		return NULL;
+	
+	if (surface)
+		return surface;
+	
+	getPixbuf();
+	surface = gt_cairo_create_surface_from_pixbuf(pixbuf);
+	
+	//_type = SURFACE;
+	return surface;
 }
 
 GdkPixmap *gPicture::getPixmap()
@@ -304,16 +306,21 @@ GdkPixmap *gPicture::getPixmap()
 	if (_type == VOID)
 		return NULL;
 	
-	if (!pic && img)
+	if (_type != PIXMAP)
 	{
-		// Normally, if pic is null, mask should be null too
+		if (_type != PIXBUF)
+			getPixbuf();
+	
+		if (pixmap)
+			g_object_unref(G_OBJECT(pixmap));
 		if (mask)
 			g_object_unref(G_OBJECT(mask));
-		gt_pixbuf_render_pixmap_and_mask(img, &pic, &mask, 128);
+
+		gt_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
 	}
 	
-	_type = SERVER;
-	return pic;
+	_type = PIXMAP;
+	return pixmap;
 }
 
 GdkBitmap *gPicture::getMask()
@@ -325,13 +332,16 @@ GdkBitmap *gPicture::getMask()
 
 gPicture *gPicture::fromMemory(char *addr, unsigned int len)
 {
-	GdkPixbuf *img;
+	GdkPixbuf *pixbuf;
 	bool trans;
 
-	if (!pixbufFromMemory(&img, addr, len, &trans))
+	if (!pixbufFromMemory(&pixbuf, addr, len, &trans))
 		return 0;
 	else
-		return new gPicture(img);
+	{
+		gPicture *pic = new gPicture(pixbuf);
+		return pic;
+	}
 }
 
 gPicture *gPicture::fromData(const char *data, int width, int height)
@@ -351,10 +361,10 @@ int gPicture::depth()
 {
 	int depth=0;
 
-	if (pic) 
-    depth = gdk_drawable_get_depth(GDK_DRAWABLE(pic));
-  else if (img)
-    depth = 32;
+	if (pixmap) 
+		depth = gdk_drawable_get_depth(GDK_DRAWABLE(pixmap));
+	else if (pixbuf)
+		depth = 32;
 
 	return depth;
 }
@@ -366,7 +376,7 @@ void gPicture::setTransparent(bool vl)
 		
 	_transparent = vl;
 	
-	if (_type == SERVER)
+	if (_type == PIXMAP)
 	{
 		if (_transparent)
 			createMask(true);
@@ -383,31 +393,31 @@ void gPicture::setTransparent(bool vl)
 
 void gPicture::fill(gColor col)
 {
-  if (_type == SERVER)
-  {
-  	gt_pixmap_fill(pic, col, NULL);
-  }
-  else if (_type == MEMORY)
-  {
-  	int r, g, b, a;
+	if (_type == PIXMAP)
+	{
+		gt_pixmap_fill(pixmap, col, NULL);
+	}
+	else if (_type == PIXBUF)
+	{
+		int r, g, b, a;
 		union
 		{
 			char c[4];
 			uint value;
 		}
 		color;
-  	
-  	gt_color_to_rgba(col, &r, &g, &b, &a);
-  	
-  	color.c[0] = a ^ 0xFF;
-  	color.c[1] = b;
-  	color.c[2] = g;
-  	color.c[3] = r;
-  	
-  	gdk_pixbuf_fill(img, color.value);
-  }
-  
-  invalidate();
+		
+		gt_color_to_rgba(col, &r, &g, &b, &a);
+		
+		color.c[0] = a ^ 0xFF;
+		color.c[1] = b;
+		color.c[2] = g;
+		color.c[3] = r;
+		
+		gdk_pixbuf_fill(pixbuf, color.value);
+	}
+	
+	invalidate();
 }
 
 
@@ -474,45 +484,45 @@ int gPicture::save(const char *path, int quality)
 
 
 /***********************************************************************
- The following function tries to load an icon from predefined or "stock"
- items. It accepts the format: StockSize/IconName, where StockSize can be:
+The following function tries to load an icon from predefined or "stock"
+items. It accepts the format: StockSize/IconName, where StockSize can be:
 
- "Menu", "SmallToolBar","LargeToolBar","Button","Dnd","Dialog"
+"Menu", "SmallToolBar","LargeToolBar","Button","Dnd","Dialog"
 
- And IconName can be:
+And IconName can be:
 
- "Add","Apply","Bold","Cancel",
- "CDRom","Clear","Close","ColorPicker",
- "Convert","Copy","Cut","Delete",
- "DialogAuthentication""DialogError","DialogInfo","DialogQuestion",
- "DialogWarning","Dnd","DndMultiple", "Execute",
- "Find","FindAndReplace","Floppy","GotoBottom",
- "GotoFirst", "GotoLast","GotoTop","GoBack",
- "GoDown","GoForward","GoUp","HardDisk"
- "Help","Home","Indent","Index",
- "Italic","JumpTo","JustifyCenter", "JustifyFill",
- "JustifyLeft","JustifyRight","MissingImage","Network",
- "New","No","Ok","Open",
- "Paste","Preferences","Print","PrintPreview",
- "Properties","Quit","Redo","Refresh",
- "Remove","RevertToSaved","Save","SaveAs",
- "SelectColor","SelectFont","SortAscending","SortDescending",
- "SpellCheck","Stop","StrikeThrough","Undelete",
- "Underline","Undo","Unindent","Yes",
- "Zoom100","ZoomFit","ZoomIn","ZoomOut"
+"Add","Apply","Bold","Cancel",
+"CDRom","Clear","Close","ColorPicker",
+"Convert","Copy","Cut","Delete",
+"DialogAuthentication""DialogError","DialogInfo","DialogQuestion",
+"DialogWarning","Dnd","DndMultiple", "Execute",
+"Find","FindAndReplace","Floppy","GotoBottom",
+"GotoFirst", "GotoLast","GotoTop","GoBack",
+"GoDown","GoForward","GoUp","HardDisk"
+"Help","Home","Indent","Index",
+"Italic","JumpTo","JustifyCenter", "JustifyFill",
+"JustifyLeft","JustifyRight","MissingImage","Network",
+"New","No","Ok","Open",
+"Paste","Preferences","Print","PrintPreview",
+"Properties","Quit","Redo","Refresh",
+"Remove","RevertToSaved","Save","SaveAs",
+"SelectColor","SelectFont","SortAscending","SortDescending",
+"SpellCheck","Stop","StrikeThrough","Undelete",
+"Underline","Undo","Unindent","Yes",
+"Zoom100","ZoomFit","ZoomIn","ZoomOut"
 *************************************************************************/
 
 
 
 /***********************************************************************
- The following function tries to load an icon from predefined system
- paths
- ***********************************************************************/
+The following function tries to load an icon from predefined system
+paths
+***********************************************************************/
 gPicture* gPicture::fromNamedIcon(const char *name, int len)
 {
 	GtkIconTheme* theme;
 	GdkPixbuf *buf;
-	gPicture *pic=NULL;
+	gPicture *pixmap=NULL;
 	int r_type=32;
 	char *c_name, *r_name;
 	
@@ -541,53 +551,57 @@ gPicture* gPicture::fromNamedIcon(const char *name, int len)
 	g_free(c_name);
 	if (!buf) return NULL;
 
-	pic=gPicture::fromPixbuf(buf);
+	pixmap=gPicture::fromPixbuf(buf);
 	g_object_unref(buf);
 
-	return pic;
+	return pixmap;
 }
 
 void gPicture::clear()
 {
-  //fprintf(stderr, "gPicture::clear: %p (%d %d) pic = %p img = %p\n", this, _width, _height, pic, img);
-  _width = 0;
-  _height = 0;
-  _type = VOID;
-  
-  if (pic)
-  	g_object_unref(G_OBJECT(pic));
-  if (mask)
-  	g_object_unref(G_OBJECT(mask));
-	if (img)
-		g_object_unref(G_OBJECT(img));
+	//fprintf(stderr, "gPicture::clear: %p (%d %d) pixmap = %p pixbuf = %p\n", this, _width, _height, pixmap, pixbuf);
+
+	_width = 0;
+	_height = 0;
+	_type = VOID;
 	
-  pic = 0;
-  img = 0;
-  mask = 0;
+	if (pixmap)
+		g_object_unref(G_OBJECT(pixmap));
+	if (mask)
+		g_object_unref(G_OBJECT(mask));
+	if (pixbuf)
+		g_object_unref(G_OBJECT(pixbuf));
+	if (surface)
+		cairo_surface_destroy(surface);
+	
+	pixmap = NULL;
+	pixbuf = NULL;
+	mask = NULL;
+	surface = NULL;
 }
 
 void gPicture::resize(int w, int h)
 {
-  if (_width <= 0 || _height <= 0)
-  {
-    clear();
-    return;
-  }
+	if (_width <= 0 || _height <= 0)
+	{
+		clear();
+		return;
+	}
 
-  if (_type == SERVER)
-  {
-  	GdkPixmap *buf;
-  	GdkBitmap *save;
-  	GdkGC *gc;
+	if (_type == PIXMAP)
+	{
+		GdkPixmap *buf;
+		GdkBitmap *save;
+		GdkGC *gc;
 
-    buf = create_pixmap(w, h);
+		buf = create_pixmap(w, h);
 		
 		gc=gdk_gc_new(buf);
-		gdk_draw_drawable(buf, gc, pic, 0, 0, 0, 0, w, h);
+		gdk_draw_drawable(buf, gc, pixmap, 0, 0, 0, 0, w, h);
 		g_object_unref(gc);
 		
-		g_object_unref(G_OBJECT(pic));
-		pic = buf;
+		g_object_unref(G_OBJECT(pixmap));
+		pixmap = buf;
 		
 		if (_transparent)
 		{
@@ -601,49 +615,49 @@ void gPicture::resize(int w, int h)
 			
 			g_object_unref(save);
 		}
-  }
-  else if (_type == MEMORY)
-  {
-  	GdkPixbuf *buf;
-  	
+	}
+	else if (_type == PIXBUF)
+	{
+		GdkPixbuf *buf;
+		
 		if (w > width() || h > height())
 		{
 			buf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
 			if (w > width()) w = width();
 			if (h > height()) h = height();
-			gdk_pixbuf_copy_area(img, 0, 0, w, h, buf, 0, 0);
+			gdk_pixbuf_copy_area(pixbuf, 0, 0, w, h, buf, 0, 0);
 		}
 		else
 		{
-			buf = gdk_pixbuf_new_subpixbuf(img, 0, 0, w, h);
+			buf = gdk_pixbuf_new_subpixbuf(pixbuf, 0, 0, w, h);
 		}
 		
-		g_object_unref(G_OBJECT(img));
-		img = buf;
-  }
-  
-  _width = w;
-  _height = h;
-  
-  invalidate();
+		g_object_unref(G_OBJECT(pixbuf));
+		pixbuf = buf;
+	}
+	
+	_width = w;
+	_height = h;
+	
+	invalidate();
 }
 
 
 gPicture *gPicture::copy(int x, int y, int w, int h)
 {
-  gPicture *ret = NULL;
-  
-  if (_type == VOID || w <= 0 || h <= 0)
-    return new gPicture();
+	gPicture *ret = NULL;
+	
+	if (_type == VOID || w <= 0 || h <= 0)
+		return new gPicture();
 
-  if (_type == SERVER)
-  {
-    GdkGC *gc;
-    
-	  ret = new gPicture(_type, w, h, _transparent);
-  
-		gc=gdk_gc_new(ret->pic);
-		gdk_draw_drawable(ret->pic, gc, pic, x, y, 0, 0, w, h);
+	if (_type == PIXMAP)
+	{
+		GdkGC *gc;
+		
+		ret = new gPicture(_type, w, h, _transparent);
+	
+		gc=gdk_gc_new(ret->pixmap);
+		gdk_draw_drawable(ret->pixmap, gc, pixmap, x, y, 0, 0, w, h);
 		g_object_unref(gc);
 		
 		if (ret->mask)
@@ -652,22 +666,22 @@ gPicture *gPicture::copy(int x, int y, int w, int h)
 			gdk_draw_drawable(ret->mask, gc, mask, x, y, 0, 0, w, h);
 			g_object_unref(gc);
 		}
-  }
-  else if (_type == MEMORY)
-  {
-  	GdkPixbuf *buf;
-  	if (x == 0 && y == 0 && w == width() && h == height())
-  		buf = gdk_pixbuf_copy(img);
-  	else
-  	{
+	}
+	else if (_type == PIXBUF)
+	{
+		GdkPixbuf *buf;
+		if (x == 0 && y == 0 && w == width() && h == height())
+			buf = gdk_pixbuf_copy(pixbuf);
+		else
+		{
 			buf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
-			gdk_pixbuf_copy_area(img, x, y, w, h, buf, 0, 0);
+			gdk_pixbuf_copy_area(pixbuf, x, y, w, h, buf, 0, 0);
 		}
 		
-  	ret = new gPicture(buf, _transparent);
-  }
-  
-  return ret;
+		ret = new gPicture(buf, _transparent);
+	}
+	
+	return ret;
 }
 
 gPicture *gPicture::copy()
@@ -685,7 +699,7 @@ void gPicture::putPixel(int x, int y, gColor col)
 	if ( (x<0) || (x>width()) ) return;
 	if ( (y<0) || (y>height()) ) return;
 
-  image = getPixbuf();
+	image = getPixbuf();
 	
 	nchannels=gdk_pixbuf_get_n_channels(image);
 	rowstride=gdk_pixbuf_get_rowstride(image);
@@ -696,12 +710,12 @@ void gPicture::putPixel(int x, int y, gColor col)
 	if (nchannels>1) p[1]=( (col>>8) & 0xFF );
 	if (nchannels>2) p[2]=( col & 0xFF );
 	//if (nchannels>3) p[3]=(col>>24);*/
-  p[0]=((col>>16) & 0xFF);
-  p[1]=((col>>8) & 0xFF);
-  p[2]=(col & 0xFF);
-  if (nchannels>3) p[3]=255 - (col >> 24);   
+	p[0]=((col>>16) & 0xFF);
+	p[1]=((col>>8) & 0xFF);
+	p[2]=(col & 0xFF);
+	if (nchannels>3) p[3]=255 - (col >> 24);   
 
-  invalidate();
+	invalidate();
 }
 
 gColor gPicture::getPixel(int x, int y)
@@ -715,7 +729,7 @@ gColor gPicture::getPixel(int x, int y)
 	if ( (x<0) || (x>width()) ) return 0;
 	if ( (y<0) || (y>height()) ) return 0;
 
- 	image = getPixbuf();
+	image = getPixbuf();
 	
 	nchannels=gdk_pixbuf_get_n_channels(image);
 	rowstride=gdk_pixbuf_get_rowstride(image);
@@ -792,8 +806,8 @@ gPicture* gPicture::flip(bool mirror)
 				s += rowstride;
 			}
 		}
-  }
-  
+	}
+	
 	return ret;
 }
 
@@ -834,7 +848,7 @@ static void rotate_image(double mat11, double mat12, double mat21, double mat22,
 		}
 		m21ydx += m21;
 		m22ydy += m22;
-   }
+	}
 }
 
 gPicture* gPicture::rotate(double angle)
@@ -881,7 +895,7 @@ gPicture* gPicture::rotate(double angle)
 	dy = height() / 2.0 - cy;
 	
 	GdkPixbuf *src = getPixbuf();
-	gPicture *npic = new gPicture(MEMORY, nw, nh, isTransparent());
+	gPicture *npic = new gPicture(PIXBUF, nw, nh, isTransparent());
 	GdkPixbuf *dst = npic->getPixbuf();
 	
 	npic->fill(isTransparent() ? -1 : 0);
@@ -895,10 +909,10 @@ gPicture* gPicture::rotate(double angle)
 
 gPicture *gPicture::stretch(int w, int h, bool smooth)
 {
-  gPicture *ret;
-  GdkPixbuf *image;
+	gPicture *ret;
+	GdkPixbuf *image;
 	int ws, hs;
-  
+	
 	if (w <= 0 && h <= 0)
 		return new gPicture();
 	
@@ -907,16 +921,16 @@ gPicture *gPicture::stretch(int w, int h, bool smooth)
 	else if (h < 0)
 		h = height() * w / width();
 	
-  if (w <= 0 || h <= 0)
-    return new gPicture();
-  
-  ret = copy();
-  if (ret->isVoid())
-    return ret;
-    
-  image = ret->getPixbuf();
-  
-  if (smooth)
+	if (w <= 0 || h <= 0)
+		return new gPicture();
+	
+	ret = copy();
+	if (ret->isVoid())
+		return ret;
+		
+	image = ret->getPixbuf();
+	
+	if (smooth)
 	{
 		ws = w;
 		hs = h;
@@ -926,45 +940,45 @@ gPicture *gPicture::stretch(int w, int h, bool smooth)
 			hs = h * 4;
 		if (ws != w || hs != h)
 		{
-			ret->img = gdk_pixbuf_scale_simple(image, ws, hs, GDK_INTERP_NEAREST);
+			ret->pixbuf = gdk_pixbuf_scale_simple(image, ws, hs, GDK_INTERP_NEAREST);
 			g_object_unref(G_OBJECT(image));
-			image = ret->img;
+			image = ret->pixbuf;
 		}
-    ret->img = gdk_pixbuf_scale_simple(image, w, h, GDK_INTERP_BILINEAR);
+		ret->pixbuf = gdk_pixbuf_scale_simple(image, w, h, GDK_INTERP_BILINEAR);
 	}
-  else
-    ret->img = gdk_pixbuf_scale_simple(image, w, h, GDK_INTERP_NEAREST);
+	else
+		ret->pixbuf = gdk_pixbuf_scale_simple(image, w, h, GDK_INTERP_NEAREST);
 
-  g_object_unref(G_OBJECT(image));
-  
-  ret->_width = w;
-  ret->_height = h;
+	g_object_unref(G_OBJECT(image));
+	
+	ret->_width = w;
+	ret->_height = h;
 
-  ret->invalidate();
+	ret->invalidate();
 
 	return ret;
 }
 
 
-void gPicture::draw(gPicture *pic, int x, int y, int w, int h, int sx, int sy, int sw, int sh)
+void gPicture::draw(gPicture *pixmap, int x, int y, int w, int h, int sx, int sy, int sw, int sh)
 {
-	if (isVoid() || pic->isVoid())
+	if (isVoid() || pixmap->isVoid())
 		return;
 
-	GT_NORMALIZE(x, y, w, h, sx, sy, sw, sh, pic->width(), pic->height());
+	GT_NORMALIZE(x, y, w, h, sx, sy, sw, sh, pixmap->width(), pixmap->height());
 
-  if (x >= width() || y >= height())
-  	return;
+	if (x >= width() || y >= height())
+		return;
 	
 
-	if (_type == SERVER)
+	if (_type == PIXMAP)
 	{
 		GdkPixmap *dst = getPixmap();
 		
-		if (pic->type() == gPicture::SERVER && !pic->isTransparent() && w == sw && h == sh)
+		if (pixmap->type() == gPicture::PIXMAP && !pixmap->isTransparent() && w == sw && h == sh)
 		{
 			GdkGC *gc = gdk_gc_new(GDK_DRAWABLE(dst));
-			GdkPixmap *src = pic->getPixmap();
+			GdkPixmap *src = pixmap->getPixmap();
 			gdk_draw_drawable(GDK_DRAWABLE(dst), gc, src, sx, sy, x, y, sw, sh);
 			g_object_unref(G_OBJECT(gc));
 		}
@@ -975,23 +989,23 @@ void gPicture::draw(gPicture *pic, int x, int y, int w, int h, int sx, int sy, i
 			if (w != sw || h != sh)
 			{
 				gPicture *pic2;
-				pic2 = pic->copy(sx, sy, sw, sh);
-				pic = pic2->stretch(w, h, true);
+				pic2 = pixmap->copy(sx, sy, sw, sh);
+				pixmap = pic2->stretch(w, h, true);
 				delete pic2;
 				del = true;
 				sx = 0; sy = 0; sw = w; sh = h;
 			}
 			
-			gdk_draw_pixbuf(GDK_DRAWABLE(dst), NULL, pic->getPixbuf(), sx, sy, x, y, sw, sh, GDK_RGB_DITHER_MAX, 0, 0); 
+			gdk_draw_pixbuf(GDK_DRAWABLE(dst), NULL, pixmap->getPixbuf(), sx, sy, x, y, sw, sh, GDK_RGB_DITHER_MAX, 0, 0); 
 			
 			if (del)
-				delete pic;
+				delete pixmap;
 		}
 	}
-	else if (_type == MEMORY)
+	else if (_type == PIXBUF)
 	{
 		GdkPixbuf *dst = getPixbuf();
-		GdkPixbuf *src = pic->getPixbuf();
+		GdkPixbuf *src = pixmap->getPixbuf();
 		double scale_x, scale_y, offset_x, offset_y;
 		
 		scale_x = (double)w / sw;
@@ -1028,32 +1042,32 @@ GHashTable *gPictureCache::cache = 0;
 
 static void destroy_key(char *key)
 {
-  g_free(key);
+	g_free(key);
 }
 
-static void destroy_value(gPicture *pic)
+static void destroy_value(gPicture *pixmap)
 {
-	//fprintf(stderr, "gPictureCache: destroy_value %p\n", pic);
-  pic->unref();
+	//fprintf(stderr, "gPictureCache: destroy_value %p\n", pixmap);
+	pixmap->unref();
 }
 
 void gPictureCache::init()
 {
-  cache = g_hash_table_new_full((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal, (GDestroyNotify)destroy_key, (GDestroyNotify)destroy_value);
+	cache = g_hash_table_new_full((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal, (GDestroyNotify)destroy_key, (GDestroyNotify)destroy_value);
 }
 
 void gPictureCache::exit()
 {
-  g_hash_table_destroy(cache);
+	g_hash_table_destroy(cache);
 }
 
-void gPictureCache::put(const char *key, gPicture *pic)
+void gPictureCache::put(const char *key, gPicture *pixmap)
 {
 	if (!key || !*key) return;
-  
-	//fprintf(stderr, "gPictureCache: put %p\n", pic);
-	pic->ref();
-	g_hash_table_replace(cache, (gpointer)g_strdup(key), (gpointer)pic);
+	
+	//fprintf(stderr, "gPictureCache: put %p\n", pixmap);
+	pixmap->ref();
+	g_hash_table_replace(cache, (gpointer)g_strdup(key), (gpointer)pixmap);
 }
 
 gPicture *gPictureCache::get(const char *key)
@@ -1065,8 +1079,8 @@ gPicture *gPictureCache::get(const char *key)
 
 void gPictureCache::flush()
 {
-  exit();
-  init();
+	exit();
+	init();
 }
 
 void gPicture::makeGray()
