@@ -531,6 +531,12 @@ void gPrinter::getPrintPages(int *from, int *to) const
 	GtkPageRange *range;
 	int nrange;
 	
+	if (gtk_print_settings_get_print_pages(_settings) == GTK_PRINT_PAGES_ALL)
+	{
+		*from = *to = -1;
+		return;
+	}
+	
 	range = gtk_print_settings_get_page_ranges(_settings, &nrange);
 	
 	if (nrange <= 0)
@@ -589,6 +595,7 @@ static char *unescape_uri(const char *uri)
 
 const char *gPrinter::outputFileName() const
 {
+	//fprintf(stderr, "outputFileName: %s\n", gtk_print_settings_get(_settings, GTK_PRINT_SETTINGS_OUTPUT_URI));
 	return unescape_uri(gtk_print_settings_get(_settings, GTK_PRINT_SETTINGS_OUTPUT_URI));
 }
 
@@ -666,6 +673,7 @@ static void dump_tree(GtkWidget *wid, GtkPrintUnixDialog *dialog)
 			char *name;
 			
 			path = unescape_uri(gtk_print_settings_get(gPrinter::_current->_settings, GTK_PRINT_SETTINGS_OUTPUT_URI));
+			//fprintf(stderr, "dump_tree: path = %s\n", path);
 			if (path)
 			{
 				name = g_path_get_basename(path);
@@ -696,3 +704,15 @@ void gPrinter::fixPrintDialog(GtkPrintUnixDialog *dialog)
 	dump_tree(GTK_WIDGET(dialog), dialog);
 }
 
+static gboolean find_all_printers(GtkPrinter *gtk_printer, bool (*callback)(const char *, bool))
+{
+	if (strcmp(G_OBJECT_TYPE_NAME(gtk_printer_get_backend(gtk_printer)), "GtkPrintBackendFile"))
+		return (*callback)(gtk_printer_get_name(gtk_printer), gtk_printer_is_default(gtk_printer));
+
+	return FALSE;
+}
+
+void gPrinter::enumeratePrinters(bool (*callback)(const char *, bool))
+{
+	gtk_enumerate_printers((GtkPrinterFunc)find_all_printers, (gpointer)callback, NULL, TRUE);
+}
