@@ -28,26 +28,34 @@
 
 gboolean gSeparator_expose(GtkWidget *wid, GdkEventExpose *e, gSeparator *data)
 {
-	gint w, h;
+	gint x, y, w, h;
+	gColor color;
 
+	x = wid->allocation.x;
+	y = wid->allocation.y;
 	w = data->width();
 	h = data->height();
 	
 	if (w == 1 || h == 1)
 	{
-		GdkGC *gc;
-		GdkGCValues values;
-
-		fill_gdk_color(&values.foreground, gDesktop::lightfgColor(), gdk_drawable_get_colormap(wid->window));
-		gc = gtk_gc_get(gdk_drawable_get_depth(wid->window), gdk_drawable_get_colormap(wid->window), &values, GDK_GC_FOREGROUND);
+		cairo_t *cr;
 		
-		gdk_draw_rectangle(wid->window, gc, TRUE, e->area.x, e->area.y, e->area.width, e->area.height);
-		gtk_gc_release(gc);
+		cr = gdk_cairo_create(wid->window);
+		
+		color = data->foreground();
+		if (color == COLOR_DEFAULT)
+			color = gDesktop::lightfgColor();
+		
+		gt_cairo_set_source_color(cr, color);
+		
+		cairo_rectangle(cr, e->area.x, e->area.y, e->area.width, e->area.height);
+		cairo_fill(cr);
+		cairo_destroy(cr);
 	}
 	else if (w>=h)
-		gtk_paint_hline(wid->style,wid->window,GTK_STATE_NORMAL,&e->area,wid,NULL,0,w,h/2);
+		gtk_paint_hline(wid->style, wid->window, GTK_STATE_NORMAL, &e->area, wid, NULL, x, x + w, y + h / 2);
 	else
-		gtk_paint_vline(wid->style,wid->window,GTK_STATE_NORMAL,&e->area,wid,NULL,0,h,w/2);
+		gtk_paint_vline(wid->style, wid->window, GTK_STATE_NORMAL, &e->area, wid, NULL, y, y + h, x + w / 2);
 
 	return false;
 } 
@@ -56,17 +64,11 @@ gSeparator::gSeparator(gContainer *parent) : gControl(parent)
 {
 	g_typ=Type_gSeparator;
 	
-	border=gtk_event_box_new();
-	widget=gtk_drawing_area_new();
-	gtk_container_add(GTK_CONTAINER(border),widget);
+	border = widget = gtk_fixed_new();
 
-	gtk_widget_add_events(widget,GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
-	gtk_widget_add_events(widget,GDK_POINTER_MOTION_MASK);
-	
-	connectParent();
-	initSignals();
+	realize(false);
 
-	g_signal_connect(G_OBJECT(widget),"expose-event",G_CALLBACK(gSeparator_expose),(gpointer)this);
+	g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(gSeparator_expose), (gpointer)this);
 }
 
 
