@@ -23,61 +23,19 @@
 
 #include "widgets.h"
 #include "widgets_private.h"
-#include "gdraw.h"
 #include "gscrollbar.h"
 #include "gslider.h"
 
-void slider_Change(GtkRange *range,gSlider *data)
+static void cb_change(GtkRange *range,gSlider *data)
 {
-	data->_value = gtk_range_get_value(GTK_RANGE(data->widget));
-	if (data->onChange) data->onChange(data);
-}
+	int new_value = gtk_range_get_value(GTK_RANGE(data->widget));
 
-gboolean slider_Expose(GtkWidget *widget,GdkEventExpose *event,gSlider *data)
-{
-	GtkAdjustment* adj=gtk_range_get_adjustment(GTK_RANGE(data->widget));
-	int max=(int)(adj->upper-adj->lower);
-	int b;
-	int fact=1;
-	int myh;
+	if (data->_value == new_value)
+		return;
 	
-	if (!data->_mark) return false;
-	
-	if ( GTK_WIDGET_TYPE(data->widget)==GTK_TYPE_HSCALE )
-	{
-		myh=(data->height()-20)/2;
-		if (myh<=0) myh=1;
-		if (max) fact = data->width()/max;
-		gDraw *dr=new gDraw();
-		dr->connect(data);
-		dr->setForeground(get_gdk_fg_color(data->border));
-		for (b=0;b<data->width();b+=data->_step)
-		{
-			dr->line(b*fact,0,b*fact,myh);
-			dr->line(b*fact,data->height(),b*fact,data->height()-myh);
-		}
-		dr->disconnect();
-		delete dr;
-	}
-	else
-	{
-		myh=(data->width()-20)/2;
-		if (myh<=0) myh=1;
-		if (max) fact = data->height()/max;
-		gDraw *dr=new gDraw();
-		dr->connect(data);
-		dr->setForeground(get_gdk_fg_color(data->border));
-		for (b=0;b<data->height();b+=data->_step)
-		{
-			dr->line(0,b*fact,myh,b*fact);
-			dr->line(data->width(),b*fact,data->width()-myh,b*fact);
-		}
-		dr->disconnect();
-		delete dr;
-	}
-	
-	
-	return false;
+	data->_value = new_value;
+	if (data->onChange) 
+		data->onChange(data);
 }
 
 void gSlider::init()
@@ -119,7 +77,7 @@ gSlider::gSlider(gContainer *par, bool scrollbar) : gControl(par)
 	_max = 100;
 	_tracking = true;
 	
-	border = gtk_event_box_new();
+	border = gtk_alignment_new(0, 0, 1, 1);
 	
 	if (scrollbar)
 		return;
@@ -133,8 +91,8 @@ gSlider::gSlider(gContainer *par, bool scrollbar) : gControl(par)
 	
 	onChange = NULL;
 	
-	g_signal_connect(G_OBJECT(widget),"value-changed",G_CALLBACK(slider_Change),(gpointer)this);
-	g_signal_connect_after(G_OBJECT(border),"expose-event",G_CALLBACK(slider_Expose),(gpointer)this);
+	g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(cb_change), (gpointer)this);
+	//g_signal_connect_after(G_OBJECT(border),"expose-event",G_CALLBACK(slider_Expose),(gpointer)this);
 }
 
 gScrollBar::gScrollBar(gContainer *par) : gSlider(par, true)
@@ -143,8 +101,11 @@ gScrollBar::gScrollBar(gContainer *par) : gSlider(par, true)
 	widget = gtk_hscrollbar_new(NULL);
 	realize(false);
 	
+	init();
+	onChange = NULL;
+	
 	gtk_range_set_update_policy(GTK_RANGE(widget),GTK_UPDATE_CONTINUOUS);
-	g_signal_connect(G_OBJECT(widget),"value-changed",G_CALLBACK(slider_Change),(gpointer)this);
+	g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(cb_change), (gpointer)this);
 }
 
 bool gSlider::mark()
@@ -152,12 +113,32 @@ bool gSlider::mark()
 	return _mark;
 }
 
+void gSlider::updateMark()
+{
+	int i;
+	int step;
+	
+	if (!_mark)
+		return;
+
+	gtk_scale_clear_marks(GTK_SCALE(widget));
+	
+	step = _page_step;
+	while (step < ((_max - _min) / 20))
+		step *= 2;
+	
+	for (i = _min; i <= _max; i += step)
+		gtk_scale_add_mark(GTK_SCALE(widget), i, isVertical() ? GTK_POS_TOP : GTK_POS_RIGHT, NULL);
+}
+
 void gSlider::setMark(bool vl)
 {
+	
 	if (vl == _mark) return;
 	
 	_mark = vl;
-	gtk_widget_queue_draw(widget);
+	gtk_scale_clear_marks(GTK_SCALE(widget));
+	updateMark();
 }
 
 int gSlider::step()
@@ -187,7 +168,7 @@ void gSlider::setPageStep(int vl)
 	
 	_page_step = vl;
 	init();
-	if (_mark) gtk_widget_queue_draw(widget);
+	updateMark();
 }
 
 int gSlider::max()
@@ -211,6 +192,7 @@ void gSlider::setMax(int vl)
 	if (_min > _max)
 		_min = _max;
 	init();
+	updateMark();
 }
 
 void gSlider::setMin(int vl)
@@ -219,6 +201,7 @@ void gSlider::setMin(int vl)
 	if (_min > _max)
 		_max = _min;
 	init();
+	updateMark();
 }
 
 bool gSlider::tracking()
@@ -239,8 +222,13 @@ void gSlider::setValue(int vl)
 	else if (vl > _max)
 		vl = _max;
 	
+	if (_value == vl)
+		return;
+	
 	_value = vl;
 	init();
+	
+	emit(SIGNAL(onChange));
 }
 
 
@@ -268,7 +256,7 @@ void gSlider::orientation(int w,int h)
 		gtk_scale_set_draw_value(GTK_SCALE(widget), false);
 		gtk_widget_show(widget);
 		widgetSignals();
-		g_signal_connect(G_OBJECT(widget),"value-changed",G_CALLBACK(slider_Change),(gpointer)this);
+		g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(cb_change), (gpointer)this);
 		
 		g_object_unref(adj);
 		
@@ -296,17 +284,20 @@ void gScrollBar::resize(int w, int h)
 		adj = gtk_range_get_adjustment(GTK_RANGE(widget));
 		g_object_ref(adj);
 
+		g_object_ref(widget);
+		gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(widget)), widget);
 		gtk_widget_destroy(widget);
+		g_object_unref(widget);
 		
 		if (type == GTK_TYPE_VSCROLLBAR)
 			widget = gtk_vscrollbar_new(adj);
 		else
 			widget = gtk_hscrollbar_new(adj);
-
+		
 		gtk_container_add(GTK_CONTAINER(border), widget);
 		gtk_widget_show(widget);
 		widgetSignals();
-		g_signal_connect(G_OBJECT(widget),"value-changed",G_CALLBACK(slider_Change),(gpointer)this);
+		g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(cb_change), (gpointer)this);
 		
 		g_object_unref(adj);	
 		
@@ -325,3 +316,8 @@ int gSlider::getDefaultSize()
 	else
 		return req.height;
 }
+
+bool gSlider::isVertical() const
+{
+	return (GTK_WIDGET_TYPE(widget) == GTK_TYPE_VSCALE);
+}
