@@ -28,7 +28,7 @@
 #include "gdesktop.h"
 #include "gtabstrip.h"
 
-
+#if 0
 static int gTabStrip_buttonRelease(GtkWidget *wid,GdkEventButton *e,gTabStrip *d)
 {
 	if (!gApplication::userEvents()) return false;
@@ -59,6 +59,7 @@ static int gTabStrip_buttonPress(GtkWidget *wid,GdkEventButton *e,gTabStrip *d)
 	}
 	return false;
 }
+#endif
 
 static void cb_click(GtkNotebook *nb,GtkNotebookPage *pg,guint pnum,gTabStrip *data)
 {
@@ -70,26 +71,35 @@ static void cb_size_allocate(GtkWidget *wid, GtkAllocation *alloc, gTabStrip *da
 {
 	if (wid == data->getContainer() && (alloc->width != data->_client_w || alloc->height != data->_client_h))
 	{
-		data->_client_x = alloc->x;
-		data->_client_y = alloc->y;
+		int tx, ty, px, py;
+
+		if (data->getScreenPos(&tx, &ty))
+			return;
+
+		gdk_window_get_origin(gtk_widget_get_window(wid), &px, &py);
+
+		data->_client_x = px - tx + alloc->x;
+		data->_client_y = py - ty + alloc->y;
 		data->_client_w = alloc->width;
 		data->_client_h = alloc->height;
-		data->performArrange();
 	}
 }
 
+static gboolean cb_button_fix(GtkWidget *wid, GdkEventExpose *e, gTabStrip *data)
+{
+	gtk_button_set_relief(GTK_BUTTON(wid), GTK_RELIEF_NONE);
+	return false;
+}
+
 static gboolean cb_button_expose(GtkWidget *wid, GdkEventExpose *e, gTabStrip *data)
 {
-	GdkGC        *gc;
-	GdkPixbuf    *img;
-	GdkRectangle rpix={0,0,0,0};
-	GdkRectangle rect={0,0,0,0};
-	gint         py,px;
-	bool         rtl,bcenter=false;
+	cairo_t *cr;
+	GdkPixbuf *img;
+	GdkRectangle rpix = {0,0,0,0};
+	GdkRectangle rect = {0,0,0,0};
+	gint py, px;
 	gint dx, dy;
 
-	rtl = gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL;
-
 	rect = wid->allocation;
 	px = rect.width;
 
@@ -120,33 +130,16 @@ static gboolean cb_button_expose(GtkWidget *wid, GdkEventExpose *e, gTabStrip *d
 	
 	py = (rect.height - rpix.height)/2;
 	
-	gc = gdk_gc_new(wid->window);
-	gdk_gc_set_clip_origin(gc,0,0);
-	gdk_gc_set_clip_rectangle(gc,&e->area);
-
-	bcenter = true; //!(data->text()) || !(*data->text());
+	cr = gdk_cairo_create(wid->window);
 	
-	if (bcenter) 
-	{	
-		gdk_draw_pixbuf(GDK_DRAWABLE(wid->window),gc,img,0,0,rect.x + (px-rpix.width)/2, rect.y + py,
-																			-1,-1,GDK_RGB_DITHER_MAX,0,0);
-		g_object_unref(gc);
-		return false;
-	}
-
-	if (rtl)
-		gdk_draw_pixbuf(GDK_DRAWABLE(wid->window),gc,img,0,0,rect.x + rect.width - 6, rect.y + py,
-																			-1,-1,GDK_RGB_DITHER_MAX,0,0);
-	else
-		gdk_draw_pixbuf(GDK_DRAWABLE(wid->window),gc,img,0,0,rect.x + 6, rect.y + py,
-																			-1,-1,GDK_RGB_DITHER_MAX,0,0);
-
-	g_object_unref(G_OBJECT(gc));
+	gdk_cairo_region(cr, e->region);
+	cairo_clip(cr);
 	
-	//rect.width -= rpix.width;
-	//rect.x += rpix.width;
+	gt_cairo_draw_pixbuf(cr, img, rect.x + (px - rpix.width) / 2, rect.y + py, -1, -1, 1.0, NULL);
 	
-	return FALSE;
+	cairo_destroy(cr);
+	
+	return false;
 }
 
 static void cb_button_clicked(GtkWidget *wid, gTabStrip *data)
@@ -222,10 +215,10 @@ gTabStripPage::gTabStripPage(gTabStrip *tab)
 	updateColors();
 	updateFont();
 	
-	g_signal_connect(G_OBJECT(widget),"button-press-event",G_CALLBACK(gTabStrip_buttonPress),(gpointer)parent);
-	g_signal_connect(G_OBJECT(widget),"button-release-event",G_CALLBACK(gTabStrip_buttonRelease),(gpointer)parent);
-	g_signal_connect(G_OBJECT(fix),"button-press-event",G_CALLBACK(gTabStrip_buttonPress),(gpointer)parent);
-	g_signal_connect(G_OBJECT(fix),"button-release-event",G_CALLBACK(gTabStrip_buttonRelease),(gpointer)parent);
+	//g_signal_connect(G_OBJECT(widget),"button-press-event",G_CALLBACK(gTabStrip_buttonPress),(gpointer)parent);
+	//g_signal_connect(G_OBJECT(widget),"button-release-event",G_CALLBACK(gTabStrip_buttonRelease),(gpointer)parent);
+	//g_signal_connect(G_OBJECT(fix),"button-press-event",G_CALLBACK(gTabStrip_buttonPress),(gpointer)parent);
+	//g_signal_connect(G_OBJECT(fix),"button-release-event",G_CALLBACK(gTabStrip_buttonRelease),(gpointer)parent);
 	g_signal_connect_after(G_OBJECT(widget), "size-allocate", G_CALLBACK(cb_size_allocate), (gpointer)parent);	
 	
 	g_object_ref(widget);
@@ -412,7 +405,8 @@ void gTabStripPage::updateButton()
 	if (v && !_button)
 	{
 		_button = gtk_button_new();
-		gtk_button_set_relief(GTK_BUTTON(_button), GTK_RELIEF_NONE);
+		gtk_button_set_focus_on_click(GTK_BUTTON(_button), false);
+		g_signal_connect(G_OBJECT(_button), "expose-event", G_CALLBACK(cb_button_fix), (gpointer)parent);
 		g_signal_connect_after(G_OBJECT(_button), "expose-event", G_CALLBACK(cb_button_expose), (gpointer)parent);
 		g_signal_connect(G_OBJECT(_button), "clicked", G_CALLBACK(cb_button_clicked), (gpointer)parent);
 		g_object_set_data(G_OBJECT(_button), "gambas-tab-page", (void *)widget);
@@ -451,18 +445,21 @@ gTabStrip::gTabStrip(gContainer *parent) : gContainer(parent)
 	onClick = NULL;
 	onClose = NULL;
 	
-	border = gtk_event_box_new();
+	//border = gtk_event_box_new();
 	//gtk_event_box_set_visible_window(GTK_EVENT_BOX(border), false);
 	
-	widget = gtk_notebook_new();
+	border = widget = gtk_notebook_new();
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(widget), TRUE);
 	gtk_drag_dest_unset(widget);
 	
 	realize();
 
+	gtk_widget_add_events(border, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK
+		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK | GDK_SCROLL_MASK);
+	
 	setCount(1);
   
-	gtk_widget_add_events(widget,GDK_BUTTON_RELEASE_MASK);
 	g_signal_connect_after(G_OBJECT(widget), "switch-page", G_CALLBACK(cb_click), (gpointer)this);
 }
 
@@ -685,7 +682,6 @@ gControl *gTabStrip::tabChild(int ind, int n) const
 	}
 	
 	return NULL;
-	
 }
 
 int gTabStrip::childCount() const
@@ -785,4 +781,18 @@ void gTabStrip::setClosable(bool v)
 		get(i)->updateButton();
 }
 
-
+int gTabStrip::findIndex(gControl *child) const
+{
+	int i;
+	GtkWidget *page;
+	
+	page = gtk_widget_get_parent(child->border); 
+	
+	for (i = 0; i < count(); i++)
+	{
+		if (get(i)->widget == page)
+			return i;
+	}
+	
+	return -1;
+}
