@@ -44,7 +44,7 @@ Conversion between GDK and long type colors
 #define SCALE(i) ((int)(i * 255.0 / 65535.0 + 0.5))
 #define UNSCALE(d) ((int)(d / 255.0 * 65535.0 + 0.5))
 
-static GtkStateType _color_style_bg[] = { GTK_STATE_INSENSITIVE, GTK_STATE_NORMAL };
+static GtkStateType _color_style_bg[] = { GTK_STATE_INSENSITIVE, GTK_STATE_ACTIVE, GTK_STATE_PRELIGHT, GTK_STATE_NORMAL };
 static GtkStateType _color_style_fg[] = { GTK_STATE_ACTIVE, GTK_STATE_PRELIGHT, GTK_STATE_NORMAL };
 
 void fill_gdk_color(GdkColor *gcol,gColor color, GdkColormap *cmap)
@@ -94,12 +94,12 @@ static void set_color(GtkWidget *wid, gColor color, void (*func)(GtkWidget *, Gt
 	}
 }
 
-gColor get_gdk_fg_color(GtkWidget *wid)
+gColor get_gdk_fg_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
 
-	st=gtk_widget_get_style(wid);	
-	return get_gdk_color(&st->fg[GTK_STATE_NORMAL]);
+	st=gtk_widget_get_style(wid);
+	return get_gdk_color(&st->fg[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
 
 void set_gdk_fg_color(GtkWidget *wid, gColor color)
@@ -107,12 +107,12 @@ void set_gdk_fg_color(GtkWidget *wid, gColor color)
 	set_color(wid, color, gtk_widget_modify_fg, true);
 }
 
-gColor get_gdk_bg_color(GtkWidget *wid)
+gColor get_gdk_bg_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
 
 	st=gtk_widget_get_style(wid);	
-	return get_gdk_color(&st->bg[GTK_STATE_NORMAL]);
+	return get_gdk_color(&st->bg[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
 
 void set_gdk_bg_color(GtkWidget *wid,gColor color)
@@ -120,12 +120,12 @@ void set_gdk_bg_color(GtkWidget *wid,gColor color)
 	set_color(wid, color, gtk_widget_modify_bg, false);
 }
 
-gColor get_gdk_text_color(GtkWidget *wid)
+gColor get_gdk_text_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
 
 	st=gtk_widget_get_style(wid);	
-	return get_gdk_color(&st->text[GTK_STATE_NORMAL]);
+	return get_gdk_color(&st->text[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
 
 void set_gdk_text_color(GtkWidget *wid,gColor color)
@@ -133,12 +133,12 @@ void set_gdk_text_color(GtkWidget *wid,gColor color)
 	set_color(wid, color, gtk_widget_modify_text, true);
 }
 
-gColor get_gdk_base_color(GtkWidget *wid)
+gColor get_gdk_base_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
 
 	st=gtk_widget_get_style(wid);
-	return get_gdk_color(&st->base[GTK_STATE_NORMAL]);
+	return get_gdk_color(&st->base[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
 
 void set_gdk_base_color(GtkWidget *wid,gColor color)
@@ -337,11 +337,6 @@ gPicture *gt_grab_window(GdkWindow *win, int x, int y, int w, int h)
 	GdkPixbuf *buf = NULL;
 	gPicture *pic;
 	
-	ow = w;
-	oh = h;
-	
-	dx = dy = 0;
-	
 	gdk_window_get_geometry(win, 0, 0, &ww, &hh, 0);
 	
 	if (w <= 0 || h <= 0)
@@ -350,6 +345,10 @@ gPicture *gt_grab_window(GdkWindow *win, int x, int y, int w, int h)
 		h = hh;
 	}
 	
+	dx = dy = 0;
+	ow = w;
+	oh = h;
+	
 	if (x < 0)
 	{
 		w += x;
@@ -376,7 +375,7 @@ gPicture *gt_grab_window(GdkWindow *win, int x, int y, int w, int h)
 	if (w == ow && h == oh)
 		return new gPicture(buf);
 		
-	pic = new gPicture(gPicture::MEMORY, ow, oh, false);
+	pic = new gPicture(gPicture::PIXBUF, ow, oh, false);
 	pic->fill(0);
 	
 	if (w <= 0 || h <= 0)
@@ -706,7 +705,7 @@ void gt_shortcut_parse(char *shortcut, guint *key, GdkModifierType *mods)
 	 g_free(res);
 }
 
-#define MAX_FREE_LATER 32
+#define MAX_FREE_LATER 16
 
 static char *_free_later_ptr[MAX_FREE_LATER] = { 0 };
 static int _free_later_index = 0;
@@ -1174,7 +1173,7 @@ void gt_pixmap_fill(GdkPixmap *pix, gColor col, GdkGC *gc)
 	
 	fill_gdk_color(&color, col);
 	
-	#if GTK_MAJOR_VERSION >= 3 || (GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 24)
+	#if GTK_CHECK_VERSION(2, 24, 0)
 	gdk_pixmap_get_size(pix, &w, &h);
 	#else
 	gdk_drawable_get_size(GDK_DRAWABLE(pix), &w, &h);
@@ -1388,57 +1387,6 @@ void gt_pixbuf_make_gray(GdkPixbuf *pixbuf)
 		p[0] = p[1] = p[2] = (p[0] * 11 + p[1] * 16 + p[2] * 5) / 32;
 }
 
-static void gt_pixbuf_make_alpha_from_white(GdkPixbuf *pixbuf)
-{
-	guchar *p;
-	int i, n;
-	
-	p = gdk_pixbuf_get_pixels(pixbuf);
-	n = gdk_pixbuf_get_width(pixbuf) * gdk_pixbuf_get_height(pixbuf);
-
-	for (i = 0; i < n; i++, p += 4)
-	{
-		p[3] = (p[0] * 11 + p[1] * 16 + p[2] * 5) / 32;
-	}
-}
-
-
-
-GdkBitmap *gt_make_text_mask(GdkDrawable *dr, int w, int h, PangoLayout *ly, int x, int y)
-{
-	GdkBitmap *mask;
-	GdkColor color;
-	GdkPixmap *tmp_pixmap;
-	GdkPixbuf *tmp_pixbuf;
-	GdkGC *tmp_gc;
-	
-	tmp_pixmap = gdk_pixmap_new(dr, w, h, -1);
-	
-	tmp_gc = gdk_gc_new(tmp_pixmap);
-	
-	gt_pixmap_fill(tmp_pixmap, 0, tmp_gc);
-	
-	fill_gdk_color(&color, 0, gdk_drawable_get_colormap(dr));
-	gdk_gc_set_background(tmp_gc, &color);
-	
-	fill_gdk_color(&color, 0xFFFFFF, gdk_drawable_get_colormap(dr));
-	gdk_gc_set_foreground(tmp_gc, &color);
-	
-	gdk_draw_layout(tmp_pixmap, tmp_gc, x, y, ly);
-	
-	g_object_unref(tmp_gc);
-	
-	tmp_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
-	gdk_pixbuf_get_from_drawable(tmp_pixbuf, tmp_pixmap, NULL, 0, 0, 0, 0, w, h);
-	g_object_unref(tmp_pixmap);
-	
-	gt_pixbuf_make_alpha_from_white(tmp_pixbuf);
-	gt_pixbuf_render_pixmap_and_mask(tmp_pixbuf, NULL, &mask, 64);
-	g_object_unref(tmp_pixbuf);
-	
-	return mask;
-}
-
 static void disabled_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer data)
 {
 	// Print only debugging messages
@@ -1450,12 +1398,12 @@ static GLogFunc old_handler = NULL;
 
 void gt_disable_warnings(bool disable)
 {
-	fprintf(stderr, "disable warnings\n");
+	//fprintf(stderr, "disable warnings\n");
 	if (disable)
 		old_handler = g_log_set_default_handler(disabled_handler, NULL);		
 	else
 		g_log_set_default_handler(old_handler, NULL);
-	fprintf(stderr, "enable warnings\n");
+	//fprintf(stderr, "enable warnings\n");
 }
 
 void gt_set_cell_renderer_text_from_font(GtkCellRendererText *cell, gFont *font)
@@ -1615,7 +1563,7 @@ void gt_layout_alignment(PangoLayout *layout, float w, float h, float *tw, float
 	}
 }
 
-#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
+#if GTK_CHECK_VERSION(2, 18, 0)
 #else
 void
 gtk_widget_set_can_focus (GtkWidget *widget,
@@ -1635,3 +1583,240 @@ gtk_widget_set_can_focus (GtkWidget *widget,
     }
 }
 #endif
+
+static void add_again(GtkWidget *widget, GtkWidget *ignore)
+{
+	int x, y;
+	GtkContainer *parent;
+	gControl *control;
+	gContainer *parent_control;
+
+	if (widget == ignore)
+		return;
+	
+	parent = GTK_CONTAINER(gtk_widget_get_parent(ignore));
+	control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
+	parent_control = (gContainer *)g_object_get_data(G_OBJECT(parent), "gambas-control");
+	
+	if (control && parent_control)
+	{
+		x = control->x();
+		y = control->y();
+	}
+	
+	g_object_ref(G_OBJECT(widget));
+	gtk_container_remove(parent, widget);
+	gtk_container_add(parent, widget);
+	g_object_unref(G_OBJECT(widget));
+	
+	if (control && parent_control)
+		parent_control->moveChild(control, x, y);
+}
+
+void gt_lower_widget(GtkWidget *widget)
+{
+	GtkContainer *parent;
+	
+	parent = GTK_CONTAINER(gtk_widget_get_parent(widget));
+	if (parent)
+		gtk_container_foreach(parent, (GtkCallback)add_again, widget);
+}
+
+#if GTK_CHECK_VERSION(2, 22, 0)
+#else
+int gdk_device_get_source(GdkDevice *device)
+{
+	return device->source;
+}
+
+//GtkWidget *_gtk_window_group_get_current_grab(GtkWindowGroup *window_group);
+
+GtkWidget *gtk_window_group_get_current_grab(GtkWindowGroup *window_group)
+{
+  if (window_group->grabs)
+    return GTK_WIDGET(window_group->grabs->data);
+  return NULL;
+}
+#endif
+
+void gt_cairo_set_source_color(cairo_t *cr, GB_COLOR color)
+{
+	int r, g, b, a;
+	
+	GB_COLOR_SPLIT(color, r, g, b, a);
+	cairo_set_source_rgba(cr, r / 255.0, g / 255.0, b / 255.0, a / 255.0);
+}
+
+void gt_cairo_draw_rect(cairo_t *cr, int x, int y, int w, int h, GB_COLOR color)
+{
+	gt_cairo_set_source_color(cr, color);
+	
+	cairo_rectangle(cr, x, y, w, 1);
+	cairo_fill(cr);
+	if (h <= 1)
+		return;
+	cairo_rectangle(cr, x, y + h - 1, w, 1);
+	cairo_fill(cr);
+	if (h <= 2)
+		return;
+	cairo_rectangle(cr, x, y + 1, 1, h - 2);
+	cairo_fill(cr);
+	cairo_rectangle(cr, x + w - 1, y + 1, 1, h - 2);
+	cairo_fill(cr);
+}
+
+// Function partially taken from the GTK+ source code.
+
+cairo_surface_t *gt_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
+{
+	gint width = gdk_pixbuf_get_width (pixbuf);
+	gint height = gdk_pixbuf_get_height (pixbuf);
+	guchar *gdk_pixels = gdk_pixbuf_get_pixels (pixbuf);
+	int gdk_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	int n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+	int cairo_stride;
+	guchar *cairo_pixels;
+	cairo_format_t format;
+	cairo_surface_t *surface;
+	static const cairo_user_data_key_t key = { 0 };
+	int j;
+
+	//fprintf(stderr, "gt_cairo_create_surface_from_pixbuf: %d %d\n", width, height);
+	
+	if (n_channels == 3)
+		format = CAIRO_FORMAT_RGB24;
+	else
+		format = CAIRO_FORMAT_ARGB32;
+
+	cairo_stride = cairo_format_stride_for_width (format, width);
+	cairo_pixels = (uchar *)g_malloc (height * cairo_stride);
+	surface = cairo_image_surface_create_for_data ((unsigned char *)cairo_pixels,
+																								format,
+																								width, height, cairo_stride);
+
+	cairo_surface_set_user_data (surface, &key,
+						cairo_pixels, (cairo_destroy_func_t)g_free);
+
+	for (j = height; j; j--)
+	{
+		guchar *p = gdk_pixels;
+		guchar *q = cairo_pixels;
+
+		if (n_channels == 3)
+		{
+			guchar *end = p + 3 * width;
+			
+			while (p < end)
+				{
+					#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+					q[0] = p[2];
+					q[1] = p[1];
+					q[2] = p[0];
+					#else
+					q[1] = p[0];
+					q[2] = p[1];
+					q[3] = p[2];
+					#endif
+					p += 3;
+					q += 4;
+				}
+		}
+		else
+		{
+			guchar *end = p + 4 * width;
+			guint t1,t2,t3;
+				
+			#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
+
+			while (p < end)
+				{
+					#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+					MULT(q[0], p[2], p[3], t1);
+					MULT(q[1], p[1], p[3], t2);
+					MULT(q[2], p[0], p[3], t3);
+					q[3] = p[3];
+					#else	  
+					q[0] = p[3];
+					MULT(q[1], p[0], p[3], t1);
+					MULT(q[2], p[1], p[3], t2);
+					MULT(q[3], p[2], p[3], t3);
+					#endif
+					
+					p += 4;
+					q += 4;
+				}
+			
+			#undef MULT
+		}
+
+		gdk_pixels += gdk_rowstride;
+		cairo_pixels += cairo_stride;
+	}
+
+	return surface;
+}
+
+
+void gt_cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *pixbuf, float x, float y, float w, float h, float opacity, GB_RECT *source)
+{
+	cairo_surface_t *surface;
+	cairo_pattern_t *pattern = NULL, *save;
+	cairo_matrix_t matrix;
+	
+	cairo_save(cr);
+	
+	save = cairo_get_source(cr);
+	cairo_pattern_reference(save);
+	
+	if (source)
+		pixbuf = gdk_pixbuf_new_subpixbuf(pixbuf, source->x, source->y, source->w, source->h);
+	
+	if (w < 0 || h < 0)
+	{
+		//fprintf(stderr, "gdk_cairo_set_source_pixbuf: %d %d\n", gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf));
+		gdk_cairo_set_source_pixbuf(cr, pixbuf, x, y);
+		cairo_rectangle(cr, x, y, gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf));
+	}
+	else
+	{
+		surface = gt_cairo_create_surface_from_pixbuf(pixbuf);
+		pattern = cairo_pattern_create_for_surface(surface);
+		cairo_surface_destroy(surface);
+		cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
+		
+		if (source && w >= source->w && h >= source->h && w == (int)w && h == (int)h && ((int)w % source->w) == 0 && ((int)h % source->h) == 0)
+			cairo_pattern_set_filter(pattern, CAIRO_FILTER_NEAREST);
+		
+		//gdk_cairo_set_source_pixbuf(cr, picture->getPixbuf(), x, y);
+		
+		cairo_matrix_init_identity(&matrix);
+		cairo_matrix_translate(&matrix, x, y);
+		cairo_matrix_scale(&matrix, w / gdk_pixbuf_get_width(pixbuf), h / gdk_pixbuf_get_height(pixbuf));
+		cairo_matrix_invert(&matrix);
+		cairo_pattern_set_matrix(pattern, &matrix);
+		cairo_set_source(cr, pattern);
+		
+		cairo_rectangle(cr, x, y, w, h);
+	}
+	
+	if (opacity == 1.0)
+	{
+		cairo_fill(cr);
+	}
+	else
+	{
+		cairo_clip(cr);
+		cairo_paint_with_alpha(cr, opacity);
+	}
+	
+	cairo_set_source(cr, save);
+	cairo_pattern_destroy(save);
+	
+	cairo_restore(cr);
+	
+	if (pattern)
+		cairo_pattern_destroy(pattern);
+	
+	if (source)
+		g_object_unref(pixbuf);
+}
