@@ -2,7 +2,7 @@
 
   gtools.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "widgets.h"
 #include "gpicture.h"
 #include "gcontrol.h"
+#include "gb.paint.h"
 
 void stub(const char *function);
 
@@ -111,10 +112,6 @@ void gt_pixbuf_render_pixmap_and_mask(GdkPixbuf *pixbuf, GdkPixmap **pixmap_retu
 void gt_pixbuf_make_alpha(GdkPixbuf *pixbuf, gColor color);
 void gt_pixbuf_make_gray(GdkPixbuf *pixbuf);
 
-// Makes a bitmap mask from a Pango layout
-
-GdkBitmap *gt_make_text_mask(GdkDrawable *dr, int w, int h, PangoLayout *ly, int x, int y);
-
 // Enable/disable warning messages
 
 void gt_disable_warnings(bool disable);
@@ -138,10 +135,25 @@ gPicture *gt_grab_window(GdkWindow *win, int x = 0, int y = 0, int w = 0, int h
 
 void gt_layout_alignment(PangoLayout *layout, float w, float h, float *tw, float *th, int align, float *offX, float *offY);
 
-#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
+#if GTK_CHECK_VERSION(2, 18, 0)
 #else
 void
 gtk_widget_set_can_focus(GtkWidget *widget, gboolean can_focus);
 #endif
 
+void gt_lower_widget(GtkWidget *widget);
+
+#if GTK_CHECK_VERSION(2, 22, 0)
+#else
+int gdk_device_get_source(GdkDevice *device);
+GtkWidget *gtk_window_group_get_current_grab(GtkWindowGroup *window_group);
+#endif
+
+// Cairo support
+
+cairo_surface_t *gt_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf);
+void gt_cairo_set_source_color(cairo_t *cr, GB_COLOR color);
+void gt_cairo_draw_rect(cairo_t *cr, int x, int y, int w, int h, GB_COLOR color);
+void gt_cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *pixbuf, float x, float y, float w, float h, float opacity, GB_RECT *source);
+
 #endif
