@@ -30,6 +30,7 @@
 #include "gb.image.h"
 #include "gb.gtk.h"
 #include "watcher.h"
+#include "gglarea.h"
 
 #include "x11.h"
 #include "CScreen.h"
@@ -57,14 +58,11 @@
 #include "CPictureBox.h"
 #include "CTextBox.h"
 #include "CTextArea.h"
-#include "CListBox.h"
-#include "CTreeView.h"
 #include "CSlider.h"
 #include "CTabStrip.h"
 #include "CTrayIcon.h"
 #include "CScrollView.h"
 #include "CSpinBox.h"
-#include "CSplitter.h"
 #include "CStock.h"
 #include "CSeparator.h"
 #include "cprinter.h"
@@ -84,20 +82,25 @@ GB_CLASS CLASS_SvgImage;
 static void my_lang(char *lang,int rtl1);
 static void my_error(int code,char *error,char *where);
 static void my_quit (void);
-static void my_main(int *argc, char **argv);
+static void my_main(int *argc, char ***argv);
 static void my_timer(GB_TIMER *timer,bool on);
 static void my_wait(int duration);
 static void my_post(void);
 static int my_loop();
 static void my_watch(int fd, int type, void *callback, intptr_t param);
 
+static GtkWidget *GTK_CreateGLArea(void *_object, void *parent, void (*init)(GtkWidget *));
+
 static bool _post_check = false;
 static bool _must_check_quit = false;
 
 static bool _application_keypress = false;
 static GB_FUNCTION _application_keypress_func;
 
+static void *_old_hook_main;
+
 int MAIN_scale = 0;
+bool MAIN_debug_busy = false;
 
 extern "C"
 {
@@ -116,8 +119,6 @@ extern "C"
 		CArrangeDesc,
 		CBorderDesc,
 		CScrollDesc,
-		CLineDesc,
-		CFillDesc,
 		CColorDesc,
 		CFontsDesc,
 		CFontDesc,
@@ -128,6 +129,7 @@ extern "C"
 		CDragDesc,
 		CCursorDesc,
 		CMouseDesc,
+		CPointerDesc,
 		CMessageDesc,
 		CDialogDesc,
 		CWatcherDesc,
@@ -136,8 +138,6 @@ extern "C"
 		CContainerDesc,
 		CDrawingAreaDesc,
 		CFrameDesc,
-		CHSplitDesc,
-		CVSplitDesc,
 		CUserControlDesc,
 		CUserContainerDesc,
 		CPanelDesc,
@@ -166,25 +166,15 @@ extern "C"
 		CToolButtonDesc,
 		CPictureBoxDesc,
 		CMovieBoxDesc,
-		CListBoxDesc,
-		CListBoxItemDesc,
 		CTextBoxSelectionDesc,
 		CTextBoxDesc,
 		CTextAreaDesc,
 		CTextAreaSelectionDesc,
 		CComboBoxDesc,
 		CComboBoxItemDesc,
-		CListViewItemDesc,
-		CListViewDesc,
-		CTreeViewItemDesc,
-		CTreeViewDesc,
-		CColumnViewItemDesc,
-		CColumnViewColumnDesc,
-		CColumnViewColumnsDesc,
-		CColumnViewDesc,
 		CTabStripDesc,
-		CTabDesc,
-		CTabChildrenDesc,
+		CTabStripContainerDesc,
+		CTabStripContainerChildrenDesc,
 		CPluginDesc,
 		CScrollViewDesc,
 		CSpinBoxDesc,
@@ -198,24 +188,24 @@ extern "C"
 	void *GB_GTK_1[] EXPORT =
 	{
 		(void *)GTK_INTERFACE_VERSION,
-		(void *)my_main,
 		(void *)GTK_GetPicture,
 		(void *)GTK_GetImage,
-		(void *)DRAW_get_drawable,
-		(void *)DRAW_get_style,
-		(void *)DRAW_get_state,
-		(void *)DRAW_get_shadow,
-		(void *)DRAW_set_state,
-		(void *)DRAW_set_shadow,
+		(void *)GTK_CreateGLArea,
 		NULL
 	};
 
-	const char *GB_INCLUDE EXPORT = "gb.draw";
+	const char *GB_INCLUDE EXPORT = "gb.draw,gb.gui.base";
 
 	int EXPORT GB_INIT(void)
 	{
+		char *env;
+		
+		env = getenv("GB_GUI_BUSY");
+		if (env && atoi(env))
+			MAIN_debug_busy = true;
+		
 		GB.Hook(GB_HOOK_QUIT, (void *)my_quit);
-		GB.Hook(GB_HOOK_MAIN, (void *)my_main);
+		_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)my_main);
 		GB.Hook(GB_HOOK_WAIT, (void *)my_wait);
 		GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
 		GB.Hook(GB_HOOK_TIMER,(void *)my_timer);
@@ -224,12 +214,9 @@ extern "C"
 		GB.Hook(GB_HOOK_ERROR,(void*)my_error);
 		GB.Hook(GB_HOOK_LANG,(void*)my_lang);
 
-		// Thanks again to GTK+ 2.18 :-(
-		#if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION >= 18
-		//putenv((char *)"GDK_NATIVE_WINDOWS=1");
-		#endif
-		
-		GB.LoadComponent("gb.draw");
+		GB.Component.Load("gb.draw");
+		GB.Component.Load("gb.image");
+		GB.Component.Load("gb.gui.base");
 		GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
 		IMAGE.SetDefaultFormat(GB_IMAGE_RGBA);
 		DRAW_init();
@@ -249,7 +236,13 @@ extern "C"
 		CLASS_Image = GB.FindClass("Image");
 		CLASS_SvgImage = GB.FindClass("SvgImage");
 		
-		return -1;
+#if !defined(GLIB_VERSION_2_36)
+		g_type_init();
+#endif /* !defined(GLIB_VERSION_2_36) */
+		
+	my_lang(GB.System.Language(), GB.System.IsRightToLeft());
+
+	return -1;
 	}
 
 	void EXPORT GB_EXIT()
@@ -293,23 +286,79 @@ extern "C"
 			#endif
 			return TRUE;
 		}
+		else if (!strcasecmp(key, "GET_HANDLE"))
+		{
+			*value = (void *)CWIDGET_get_handle;
+			return TRUE;
+		}
+		else if (!strcasecmp(key, "SET_EVENT_FILTER"))
+		{
+			*value = (void *)0;
+			return TRUE;
+		}
+		else if (!strcasecmp(key, "TIME"))
+		{
+			*value = (void *)(intptr_t)gtk_get_current_event_time(); //gdk_x11_display_get_user_time(gdk_display_get_default());
+			return TRUE;
+		}
 		else
 			return FALSE;
 	}
+
+	static void activate_main_window(intptr_t)
+	{
+		if (gMainWindow::_active)
+			gtk_window_present(GTK_WINDOW(gMainWindow::_active->topLevel()->border));
+	}
+	
+	void EXPORT GB_SIGNAL(int signal, void *param)
+	{
+		static GtkWidget *save_popup_grab = NULL;
+		
+		switch(signal)
+		{
+			case GB_SIGNAL_DEBUG_BREAK:
+				if (gApplication::_popup_grab)
+				{
+					save_popup_grab = gApplication::_popup_grab;
+					gApplication::ungrabPopup();
+				}
+				break;
+				
+			case GB_SIGNAL_DEBUG_FORWARD:
+				//while (qApp->activePopupWidget())
+				//	delete qApp->activePopupWidget();
+				if (gdk_display_get_default())
+				gdk_display_sync(gdk_display_get_default());
+				break;
+				
+			case GB_SIGNAL_DEBUG_CONTINUE:
+				GB.Post((GB_CALLBACK)activate_main_window, 0);
+				if (save_popup_grab)
+				{
+					gApplication::_popup_grab = save_popup_grab;
+					save_popup_grab = NULL;
+					gApplication::grabPopup();
+				}
+				break;
+		}
+	}
 }
 
 void my_quit (void)
 {
-	CWINDOW *win = CWINDOW_Main;
+	while (gtk_events_pending())
+		gtk_main_iteration();
+  
+	CWINDOW_delete_all();
+	gControl::cleanRemovedControls();
 
-	gApplication::suspendEvents(false);
-	if (win)
-	{
-		while (gtk_events_pending ()) gtk_main_iteration();
-		if (win->ob.widget) ((gMainWindow*)win->ob.widget)->destroy();
-		while (gtk_events_pending ()) gtk_main_iteration();
-	}
+	CWatcher::Clear();
+	gApplication::exit();
 
+	#ifdef GDK_WINDOWING_X11
+  	X11_exit();
+  #endif
 }
 
 static bool global_key_event_handler(int type)
@@ -318,16 +367,20 @@ static bool global_key_event_handler(int type)
 	return GB.Stopped();
 }
 
-static void my_main(int *argc, char **argv)
+static void my_main(int *argc, char ***argv)
 {
 	static bool init = false;
 	
 	if (init)
 		return;
 	
-	gApplication::init(argc, &argv);
+	gApplication::init(argc, argv);
 	gApplication::setDefaultTitle(GB.Application.Title());
 	gDesktop::init();
+	
+	gApplication::onEnterEventLoop = GB.Debug.EnterEventLoop;
+	gApplication::onLeaveEventLoop = GB.Debug.LeaveEventLoop;
+		
 	MAIN_scale = gDesktop::scale();
 	#ifdef GDK_WINDOWING_X11
   	X11_init(gdk_x11_display_get_xdisplay(gdk_display_get_default()), gdk_x11_get_default_root_xwindow());
@@ -340,6 +393,8 @@ static void my_main(int *argc, char **argv)
 	}
 
 	init = true;
+
+	CALL_HOOK_MAIN(_old_hook_main, argc, argv);
 }
 
 /*static void raise_timer(GB_TIMER *timer)
@@ -350,10 +405,11 @@ static void my_main(int *argc, char **argv)
 
 typedef
 	struct {
+		int source;
 		GTimer *timer;
 		int timeout;
 		}
-	MyTimerTag;
+	MyTimerId;
 
 gboolean my_timer_function(GB_TIMER *timer)
 {
@@ -363,15 +419,16 @@ gboolean my_timer_function(GB_TIMER *timer)
 		
 		if (timer->id)
 		{
-			MyTimerTag *tag = (MyTimerTag *)timer->tag;
-			GTimer *t = tag->timer;
-			int elapsed = (int)(g_timer_elapsed(t, NULL) * 1000) - tag->timeout;
+			MyTimerId *id = (MyTimerId *)timer->id;
+			GTimer *t = id->timer;
+			int elapsed = (int)(g_timer_elapsed(t, NULL) * 1000) - id->timeout;
 			int next = timer->delay - elapsed;
 			if (next < 10)
 				next = 10;
-			tag->timeout = next;
+			id->timeout = next;
 			g_timer_start(t);
-			timer->id = (intptr_t)g_timeout_add(next, (GSourceFunc)my_timer_function,(gpointer)timer);
+			id->source = g_timeout_add(next, (GSourceFunc)my_timer_function,(gpointer)timer);
+			//timer->id = (intptr_t)g_timeout_add(next, (GSourceFunc)my_timer_function,(gpointer)timer);
 			//fprintf(stderr, "elapsed = %d  delay = %d  next = %d\n", elapsed, timer->delay, next);
 		}
 	}
@@ -381,22 +438,22 @@ gboolean my_timer_function(GB_TIMER *timer)
 
 static void my_timer(GB_TIMER *timer,bool on)
 {
-	if (timer->id) {
-		MyTimerTag *tag = (MyTimerTag *)timer->tag;
-		g_source_remove(timer->id);
-		g_timer_destroy(tag->timer);
-		g_free(tag);
+	if (timer->id)
+	{
+		MyTimerId *id = (MyTimerId *)timer->id;
+		g_source_remove(id->source);
+		g_timer_destroy(id->timer);
+		g_free(id);
 		timer->id = 0;
-		timer->tag = 0;
 	}
 
 	if (on)
 	{
-		MyTimerTag *tag = g_new(MyTimerTag, 1);
-		tag->timer = g_timer_new();
-		tag->timeout = timer->delay;
-		timer->tag = (intptr_t)tag;
-		timer->id = (intptr_t)g_timeout_add(timer->delay,(GSourceFunc)my_timer_function,(gpointer)timer);
+		MyTimerId *id = g_new(MyTimerId, 1);
+		id->timer = g_timer_new();
+		id->timeout = timer->delay;
+		id->source = (intptr_t)g_timeout_add(timer->delay,(GSourceFunc)my_timer_function,(gpointer)timer);
+		timer->id = (intptr_t)id;
 		return;
 	}
 }
@@ -420,22 +477,15 @@ static int my_loop()
 	{
 		if (_must_check_quit)
 		{
-			if (CWINDOW_must_quit() && CWatcher::count() == 0)
+			if (CWINDOW_must_quit() && CWatcher::count() == 0 && gTrayIcon::visibleCount() == 0)
 				break;
 			_must_check_quit = false;
 		}
 		MAIN_do_iteration(false);
 	}
 
-	while (gtk_events_pending())
-		gtk_main_iteration();
-  
-	CWINDOW_delete_all();
-	gControl::cleanRemovedControls();
-
-	CWatcher::Clear();
-	gApplication::exit();
-
+	my_quit();
+	
   return 0;
 }
 
@@ -464,12 +514,12 @@ static void my_error(int code,char *error,char *where)
 	g_free(showstr);
 }
 
-static void my_lang(char *lang,int rtl)
+static void my_lang(char *lang, int rtl)
 {
 	int bucle, ct;
 	gControl *iter;
 
-	if (rtl==1)
+	if (rtl)
 		gtk_widget_set_default_direction(GTK_TEXT_DIR_RTL);
 	else
 		gtk_widget_set_default_direction(GTK_TEXT_DIR_LTR);
@@ -485,8 +535,8 @@ static void my_lang(char *lang,int rtl)
 
 void MAIN_do_iteration_just_events()
 {
-	if (gtk_events_pending ())
-		gtk_main_iteration_do (false);
+	if (gtk_events_pending())
+		gtk_main_iteration_do(false);
 }
 
 void MAIN_do_iteration(bool do_not_block, bool do_not_sleep)
@@ -519,3 +569,12 @@ void MAIN_do_iteration(bool do_not_block, bool do_not_sleep)
 	
 	gControl::cleanRemovedControls();
 }
+
+static GtkWidget *GTK_CreateGLArea(void *_object, void *parent, void (*init)(GtkWidget *))
+{
+	gControl *ctrl = new gGLArea(CONTAINER(parent), init);
+	InitControl(ctrl, (CWIDGET*)_object);
+	//WIDGET->onExpose = Darea_Expose;
+	return ctrl->widget;
+}
+
