@@ -36,7 +36,8 @@
 
 #ifndef __CCURL_C
 
-extern GB_DESC CCurlDesc[];
+extern GB_DESC CurlDesc[];
+extern GB_DESC CurlSSLDesc[];
 extern GB_STREAM_DESC CurlStream;
 
 #endif
@@ -58,17 +59,25 @@ typedef
 	struct {
 		GB_BASE ob;
 		GB_STREAM stream;
+		GB_LIST list; // List of async curl objects
 		int status;
 		CURL *curl;
 		char *url;
 		FILE *file;
 		CPROXY proxy;
 		CURL_USER user;
-		bool async;
 		int timeout;
 		int method; // 0->Get, 1->Put
-		bool debug;
 		char *data;
+		int64_t dltotal;
+		int64_t dlnow;
+		int64_t ultotal;
+		int64_t ulnow;
+		unsigned async : 1;
+		unsigned in_list : 1;
+		unsigned debug : 1;
+		unsigned ssl_verify_peer : 1;
+		unsigned ssl_verify_host : 1;
 	}
 	CCURL;
 
@@ -97,7 +106,10 @@ void CURL_stop(void *_object);
 void CURL_manage_error(void *_object, int error);
 
 void CURL_init_stream(void *_object);
+void CURL_init_options(void *_object);
 
 bool CURL_check_active(void *_object);
 
+void CURL_set_progress(void *_object, bool progress);
+
 #endif
