@@ -3,7 +3,7 @@
   CHttpClient.c
 
   (c) 2003-2008 Daniel Campos Fernández <dcamposf@gmail.com>
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@
 
 #define SEND_POST 1
 #define SEND_PUT 2
-
+#define SEND_FILE 4
 
 static void http_parse_header(CHTTPCLIENT *_object)
 {
@@ -96,7 +96,7 @@ static int http_header_curl(void *buffer, size_t size, size_t nmemb, void *_obje
 	{
 		THIS_STATUS = NET_RECEIVING_DATA;
 		GB.Ref(THIS);
-		GB.Post(CURL_raise_connect,(long)THIS);
+		GB.Post(CURL_raise_connect, (intptr_t)THIS);
 	}
 
 	return size * nmemb;
@@ -139,7 +139,7 @@ static int http_write_curl(void *buffer, size_t size, size_t nmemb, void *_objec
 	if (THIS->async)
 	{
 		GB.Ref(THIS);
-		GB.Post(CURL_raise_read,(long)THIS);
+		GB.Post(CURL_raise_read, (intptr_t)THIS);
 	}
 	
 	return nmemb;
@@ -149,6 +149,7 @@ static int http_write_curl(void *buffer, size_t size, size_t nmemb, void *_objec
 static void http_reset(void *_object)
 {
 	GB.FreeString(&THIS->data);
+	
 	GB.Unref(&THIS_HTTP->headers);
 	GB.Unref(&THIS_HTTP->sent_headers);
 	
@@ -160,9 +161,14 @@ static void http_reset(void *_object)
 	
 	if (THIS_HTTP->data)
 	{
-		GB.Free((void**)POINTER(&THIS_HTTP->data));
+		if (THIS_HTTP->send_file)
+			GB.ReleaseFile(THIS_HTTP->data, THIS_HTTP->len_data);
+		else
+			GB.Free(POINTER(&THIS_HTTP->data));
 		THIS_HTTP->data = NULL;
 	}
+	
+	THIS_HTTP->send_file = FALSE;
 }
 
 
@@ -188,7 +194,7 @@ static void http_initialize_curl_handle(void *_object, GB_ARRAY custom_headers)
 		curl_easy_setopt(THIS_CURL, CURLOPT_TIMEOUT,THIS->timeout);
 	}
 	
-	curl_easy_setopt(THIS_CURL, CURLOPT_VERBOSE, THIS->debug);
+	curl_easy_setopt(THIS_CURL, CURLOPT_VERBOSE, (bool)THIS->debug);
 	curl_easy_setopt(THIS_CURL, CURLOPT_PRIVATE,(char*)_object);
 	curl_easy_setopt(THIS_CURL, CURLOPT_USERAGENT, THIS_HTTP->sUserAgent);
 	curl_easy_setopt(THIS_CURL, CURLOPT_ENCODING, THIS_HTTP->encoding);
@@ -203,10 +209,10 @@ static void http_initialize_curl_handle(void *_object, GB_ARRAY custom_headers)
 	else
 		curl_easy_setopt(THIS_CURL, CURLOPT_COOKIEJAR, NULL);
 
-	CURL_proxy_set(&THIS->proxy.proxy,THIS_CURL);
+	CURL_proxy_set(&THIS->proxy.proxy, THIS_CURL);
 	CURL_user_set(&THIS->user, THIS_CURL);
-	curl_easy_setopt(THIS_CURL, CURLOPT_URL,THIS_URL);
-
+	curl_easy_setopt(THIS_CURL, CURLOPT_URL, THIS_URL);
+	
 	THIS_HTTP->return_code = 0;
 	GB.FreeString(&THIS_HTTP->return_string);
 
@@ -220,6 +226,7 @@ static void http_initialize_curl_handle(void *_object, GB_ARRAY custom_headers)
 		GB.Ref(custom_headers);
 	}
 	
+	CURL_init_options(THIS);
 	CURL_init_stream(THIS);
 }
 
@@ -250,11 +257,11 @@ static bool check_request(void *_object, char *contentType, char *data, int len)
 		return TRUE;
 	}
 
-	if (!data || !len)
+	/*if (!data || !len)
 	{ 
 		GB.Error(ERR_INVALID_DATA); 
 		return TRUE; 
-	};
+	};*/
 	
 	return FALSE;
 }
@@ -295,6 +302,7 @@ static void http_get(void *_object, GB_ARRAY custom_headers, char *target)
 	}
 	
 	curl_easy_setopt(THIS_CURL, CURLOPT_HTTPHEADER, headers);
+	CURL_set_progress(THIS, TRUE);
 
 	if (THIS->async)
 	{
@@ -302,7 +310,7 @@ static void http_get(void *_object, GB_ARRAY custom_headers, char *target)
 		return;
 	}
 	
-	CURL_manage_error(_object,curl_easy_perform(THIS_CURL));
+	CURL_manage_error(_object, curl_easy_perform(THIS_CURL));
 }
 
 
@@ -325,23 +333,47 @@ static void http_send(void *_object, int type, char *sContent, char *sData, int
 			return;
 		}
 	}
-
+	
 	http_initialize_curl_handle(_object, custom_headers);
 
-	mylen=strlen(sContent) + strlen("Content-Type: ") + 1;
-	GB.Alloc((void*)&THIS_HTTP->sContentType,mylen);
-	GB.Alloc((void*)&THIS_HTTP->data,lendata+1);
-	strncpy(THIS_HTTP->data,sData,lendata);
-	THIS_HTTP->sContentType[0]=0;
-	strcpy(THIS_HTTP->sContentType,"Content-Type: ");
-	strcat(THIS_HTTP->sContentType,sContent);
+	if (type & SEND_FILE)
+	{
+		if (GB.LoadFile(sData, lendata, &THIS_HTTP->data, &mylen))
+			return;
+		
+		THIS_HTTP->len_data = mylen;
+		THIS_HTTP->send_file = TRUE;
+	}
+	else
+	{
+		THIS_HTTP->send_file = FALSE;
+		THIS_HTTP->len_data = lendata;
+		
+		if (lendata)
+		{
+			GB.Alloc((void*)&THIS_HTTP->data, lendata + 1);
+			strncpy(THIS_HTTP->data, sData, lendata);
+		}
+		else
+			THIS_HTTP->data = NULL;
+		
+	}
 	
-	THIS->method=1;
-	headers = curl_slist_append(headers,THIS_HTTP->sContentType );
+	THIS_HTTP->len_sent = 0;
+
+	mylen = strlen(sContent) + strlen("Content-Type: ") + 1;
+	GB.Alloc((void*)&THIS_HTTP->sContentType, mylen);
+	
+	THIS_HTTP->sContentType[0] = 0;
+	strcpy(THIS_HTTP->sContentType, "Content-Type: " );
+	strcat(THIS_HTTP->sContentType, sContent);
+	
+	THIS->method = 1;
+	headers = curl_slist_append(headers, THIS_HTTP->sContentType);
 	
 	if (THIS_HTTP->sent_headers)
 	{
-		for(i = 0; i < GB.Array.Count(THIS_HTTP->sent_headers); i++)
+		for (i = 0; i < GB.Array.Count(THIS_HTTP->sent_headers); i++)
 			headers = curl_slist_append(headers, *(char **)GB.Array.Get(THIS_HTTP->sent_headers, i));
 	}
 	
@@ -350,9 +382,6 @@ static void http_send(void *_object, int type, char *sContent, char *sData, int
 	curl_easy_setopt(THIS_CURL, CURLOPT_READFUNCTION, http_read_curl);
 	curl_easy_setopt(THIS_CURL, CURLOPT_READDATA, _object);
 	
-	THIS_HTTP->len_data = lendata;
-	THIS_HTTP->len_sent = 0;
-
 	if (type == SEND_PUT)
 	{
 		curl_easy_setopt(THIS_CURL, CURLOPT_INFILESIZE_LARGE, (curl_off_t)lendata);
@@ -364,6 +393,8 @@ static void http_send(void *_object, int type, char *sContent, char *sData, int
 		curl_easy_setopt(THIS_CURL, CURLOPT_POSTFIELDSIZE, lendata);
 	}
 
+	CURL_set_progress(THIS_CURL, TRUE);
+
 	if (THIS->async)
 	{
 		CURL_start_post(THIS);
@@ -420,7 +451,7 @@ BEGIN_PROPERTY(HttpClient_CookiesFile)
 END_PROPERTY
 
 
-BEGIN_PROPERTY (HttpClient_Auth)
+BEGIN_PROPERTY(HttpClient_Auth)
 
 	if (READ_PROPERTY)
 	{
@@ -502,7 +533,7 @@ END_METHOD
 
 BEGIN_METHOD_VOID(HttpClient_free)
 
-	CURL_stop(_object);
+	//CURL_stop(_object);
 	http_reset(THIS);
 	
 	GB.FreeString(&THIS_HTTP->sUserAgent);
@@ -527,9 +558,16 @@ BEGIN_METHOD(HttpClient_Post, GB_STRING contentType; GB_STRING data; GB_OBJECT h
 END_METHOD
 
 
+BEGIN_METHOD(HttpClient_PostFile, GB_STRING contentType; GB_STRING file; GB_OBJECT headers; GB_STRING target)
+
+	http_send(THIS, SEND_POST | SEND_FILE, GB.ToZeroString(ARG(contentType)), STRING(file), LENGTH(file), VARGOPT(headers, NULL), MISSING(target) ? NULL : GB.ToZeroString(ARG(target)));
+
+END_METHOD
+
+
 BEGIN_METHOD(HttpClient_Put, GB_STRING contentType; GB_STRING data; GB_OBJECT headers; GB_STRING target)
 
-	http_send(THIS, SEND_PUT, GB.ToZeroString(ARG(contentType)), STRING(data), LENGTH(data), VARG(headers), MISSING(target) ? NULL : GB.ToZeroString(ARG(target)));
+	http_send(THIS, SEND_PUT, GB.ToZeroString(ARG(contentType)), GB.ToZeroString(ARG(data)), 0, VARG(headers), MISSING(target) ? NULL : GB.ToZeroString(ARG(target)));
 
 END_METHOD
 
@@ -554,6 +592,7 @@ GB_DESC CHttpClientDesc[] =
   GB_METHOD("Get", NULL, HttpClient_Get, "[(Headers)String[];(TargetFile)s]"),
   GB_METHOD("Post", NULL, HttpClient_Post, "(ContentType)s(Data)s[(Headers)String[];(TargetFile)s]"),
   GB_METHOD("Put", NULL, HttpClient_Put, "(ContentType)s(Data)s[(Headers)String[];(TargetFile)s]"),
+  GB_METHOD("PostFile", NULL, HttpClient_PostFile, "(ContentType)s(Path)s[(Headers)String[];(TargetFile)s]"),
 
   GB_PROPERTY("Auth", "i", HttpClient_Auth),
   GB_PROPERTY("CookiesFile", "s",HttpClient_CookiesFile),
@@ -570,7 +609,7 @@ GB_DESC CHttpClientDesc[] =
   GB_CONSTANT("_Group", "s", "Network"),
   GB_CONSTANT("_Properties", "s", HTTP_PROPERTIES),
   GB_CONSTANT("_DefaultEvent", "s", "Read"),
-
+  
   GB_END_DECLARE
 };
 
