@@ -2,7 +2,7 @@
 
   libsmtp_comm.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -168,11 +168,11 @@ int libsmtp_int_send (GString *libsmtp_send_gstring, struct libsmtp_session_stru
 	int libsmtp_int_bytes;
 
 	#ifdef LIBSMTP_DEBUG
-		printf ("DEBUG in send: %s", libsmtp_send_gstring->str);
-	#endif
-
+		fprintf(stderr, "-> %.*s\n", (int)libsmtp_send_gstring->len, libsmtp_send_gstring->str);
+	#else
 	if (libsmtp_session->debug && type > 0)
 		fprintf(stderr, "-> %.*s\n", (int)libsmtp_send_gstring->len, libsmtp_send_gstring->str);
+	#endif
 
 	if (libsmtp_session->stream)
 	{
@@ -213,11 +213,11 @@ int libsmtp_int_send_body(char *libsmtp_send_string, unsigned int libsmtp_int_le
 	int libsmtp_int_bytes;
 
 	#ifdef LIBSMTP_DEBUG
-		printf ("DEBUG in body send : %s\n", libsmtp_send_string);
-	#endif
-
-	if (libsmtp_session->debug && libsmtp_session->Stage < LIBSMTP_DATA_STAGE)
 		fprintf(stderr, "-> %.*s\n", libsmtp_int_length, libsmtp_send_string);
+	#else
+		if (libsmtp_session->debug && libsmtp_session->Stage < LIBSMTP_DATA_STAGE)
+			fprintf(stderr, "-> %.*s\n", libsmtp_int_length, libsmtp_send_string);
+	#endif
 
 	if (libsmtp_session->stream)
 	{
@@ -230,11 +230,11 @@ int libsmtp_int_send_body(char *libsmtp_send_string, unsigned int libsmtp_int_le
 	if (libsmtp_int_bytes<0)
 	{
 		libsmtp_close(libsmtp_session);
-		libsmtp_session->ErrorCode=LIBSMTP_ERRORSENDFATAL;
+		libsmtp_session->ErrorCode = LIBSMTP_ERRORSENDFATAL;
 		return LIBSMTP_ERRORSEND;
 	}
 	/* Update statistics */
-	libsmtp_session->BodyBytes+=libsmtp_int_bytes;
+	libsmtp_session->BodyBytes += libsmtp_int_bytes;
 
 	return LIBSMTP_NOERR;
 }
@@ -277,7 +277,7 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
 	/* Ok, now lets give him the sender address */
 
 	g_string_sprintf (libsmtp_temp_gstring, "mail from: %s\r\n", \
-											libsmtp_session->From->str);
+											strchr(libsmtp_session->From->str, '<'));
 
 	if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 2))
 	{
@@ -313,7 +313,7 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
 			printf ("To: %s\n", (char *)libsmtp_temp_glist->data);
 		#endif
 		g_string_sprintf (libsmtp_temp_gstring, "rcpt to: %s\r\n", \
-				(char *)libsmtp_temp_glist->data);
+				strchr((char *)libsmtp_temp_glist->data, '<'));
 
 
 		/* Every recipient gets sent to the server */
@@ -353,9 +353,10 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
 	for (libsmtp_temp=0; libsmtp_temp < g_list_length(libsmtp_session->CC);\
 				libsmtp_temp++)
 	{
-		libsmtp_temp_glist=g_list_nth (libsmtp_session->CC, libsmtp_temp);
+		libsmtp_temp_glist = g_list_nth (libsmtp_session->CC, libsmtp_temp);
+
 		g_string_sprintf (libsmtp_temp_gstring, "rcpt to: %s\r\n", \
-				(char *)libsmtp_temp_glist->data);
+				strchr((char *)libsmtp_temp_glist->data, '<'));
 
 
 		/* Every recipient gets sent to the server */
@@ -396,7 +397,7 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
 	{
 		libsmtp_temp_glist=g_list_nth (libsmtp_session->BCC, libsmtp_temp);
 		g_string_sprintf (libsmtp_temp_gstring, "rcpt to: %s\r\n", \
-				(char *)libsmtp_temp_glist->data);
+				strchr((char *)libsmtp_temp_glist->data, '<'));
 
 
 		/* Every recipient gets sent to the server */
