@@ -2,7 +2,7 @@
 
   main.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,10 +27,6 @@
 
 #include "main.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 GB_INTERFACE GB EXPORT;
 
 GB_DESC *GB_CLASSES[] EXPORT =
@@ -47,8 +43,3 @@ int EXPORT GB_INIT(void)
 void EXPORT GB_EXIT(void)
 {
 }
-
-#ifdef _cpluscplus
-}
-#endif
-
