@@ -1,23 +1,23 @@
 /***************************************************************************
 
-  CSerialPort.c
+	CSerialPort.c
 
-  (c) 2003-2004 Daniel Campos Fernández <dcamposf@gmail.com>
+	(c) 2003-2004 Daniel Campos Fernández <dcamposf@gmail.com>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-  MA 02110-1301, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -112,15 +112,15 @@ if (THIS->signals._signal != new_signals._signal) \
 }
 	
 
-static void cb_change(int fd, int type, CSERIALPORT *_object)
+static int cb_change(intptr_t _object)
 {
 	SERIAL_SIGNAL new_signals;
-	struct timespec mywait;
+	//struct timespec mywait;
 
 	// Just sleeping a bit to reduce CPU waste
-	mywait.tv_sec = 0;
-	mywait.tv_nsec = 1000000; // 1 ms
-	nanosleep(&mywait, NULL);
+	//mywait.tv_sec = 0;
+	//mywait.tv_nsec = 1000000; // 1 ms
+	//nanosleep(&mywait, NULL);
 
 	/* Serial port signals status */
 	new_signals = get_signals(THIS);
@@ -131,6 +131,8 @@ static void cb_change(int fd, int type, CSERIALPORT *_object)
 	CHECK_SIGNAL(CTS);
 	CHECK_SIGNAL(DCD);
 	CHECK_SIGNAL(RNG);
+	
+	return FALSE;
 }
 
 static void cb_read(int fd, int type, CSERIALPORT *_object)
@@ -138,7 +140,7 @@ static void cb_read(int fd, int type, CSERIALPORT *_object)
 	GB.Raise(THIS, EVENT_Read, 0);
 }
 
-static void assign_callback(CSERIALPORT *_object)
+static void assign_callback(CSERIALPORT *_object, int polling)
 {
 	int port = THIS->port;
 
@@ -146,12 +148,15 @@ static void assign_callback(CSERIALPORT *_object)
 		GB.Watch(port, GB_WATCH_READ, (void *)cb_read, (intptr_t)THIS);
 	
 	if (GB.CanRaise(THIS, EVENT_DTR)
-	    || GB.CanRaise(THIS, EVENT_CTS)
-	    || GB.CanRaise(THIS, EVENT_DCD)
-	    || GB.CanRaise(THIS, EVENT_DSR)
-	    || GB.CanRaise(THIS, EVENT_RNG)
-	    || GB.CanRaise(THIS, EVENT_RTS))
-		GB.Watch(port, GB_WATCH_WRITE, (void *)cb_change, (intptr_t)THIS);
+			|| GB.CanRaise(THIS, EVENT_CTS)
+			|| GB.CanRaise(THIS, EVENT_DCD)
+			|| GB.CanRaise(THIS, EVENT_DSR)
+			|| GB.CanRaise(THIS, EVENT_RNG)
+			|| GB.CanRaise(THIS, EVENT_RTS))
+	{
+		// Polling is 50ms by default
+		GB.Every(polling, cb_change, (intptr_t)THIS);
+	}
 }
 
 static void release_callback(CSERIALPORT *_object)
@@ -171,7 +176,7 @@ static void close_serial_port(CSERIALPORT *_object)
 }
 
 /****************************************************************************
- 
+
 	Stream implementation
 
 ****************************************************************************/
@@ -245,7 +250,7 @@ int CSerialPort_stream_read(GB_STREAM *stream, char *buffer, int len)
 	int no_block = 0;
 	int bytes;
 
- 	if (!_object) 
+	if (!_object) 
 		return -1;
 	
 	if (ioctl(THIS->port, FIONREAD, &bytes)) 
@@ -312,23 +317,23 @@ static bool check_close(CSERIALPORT *_object)
 }
 
 /****************************************************************************
- 
+
 	SerialPort
 
 ****************************************************************************/
 
-BEGIN_PROPERTY(CSERIALPORT_Status)
+BEGIN_PROPERTY(SerialPort_Status)
 
-  GB.ReturnInteger(THIS->status);
+	GB.ReturnInteger(THIS->status);
 
 END_PROPERTY
 
 // Data Set Ready
 
-BEGIN_PROPERTY(CSERIALPORT_DSR)
+BEGIN_PROPERTY(SerialPort_DSR)
 
-  if (!THIS->status)
-  	GB.ReturnBoolean(0);
+	if (!THIS->status)
+		GB.ReturnBoolean(0);
 	else
 	{
 		THIS->signals = get_signals(THIS);
@@ -339,7 +344,7 @@ END_PROPERTY
 
 // Data Transmission Ready
 
-BEGIN_PROPERTY(CSERIALPORT_DTR)
+BEGIN_PROPERTY(SerialPort_DTR)
 
 	int ist;
 
@@ -370,7 +375,7 @@ END_PROPERTY
 
 // Ready to send
 
-BEGIN_PROPERTY(CSERIALPORT_RTS)
+BEGIN_PROPERTY(SerialPort_RTS)
 
 	int ist;
 	
@@ -401,10 +406,10 @@ END_PROPERTY
 
 // Clear to send
 
-BEGIN_PROPERTY(CSERIALPORT_CTS)
+BEGIN_PROPERTY(SerialPort_CTS)
 
-  if (!THIS->status)
-  	GB.ReturnBoolean(0);
+	if (!THIS->status)
+		GB.ReturnBoolean(0);
 	else
 	{
 		THIS->signals = get_signals(THIS);
@@ -415,10 +420,10 @@ END_PROPERTY
 
 // Data Carrier Detect
 
-BEGIN_PROPERTY(CSERIALPORT_DCD)
+BEGIN_PROPERTY(SerialPort_DCD)
 
-  if (!THIS->status)
-  	GB.ReturnBoolean(0);
+	if (!THIS->status)
+		GB.ReturnBoolean(0);
 	else
 	{
 		THIS->signals = get_signals(THIS);
@@ -429,10 +434,10 @@ END_PROPERTY
 
 // Ring
 
-BEGIN_PROPERTY(CSERIALPORT_RNG)
+BEGIN_PROPERTY(SerialPort_RNG)
 
-  if (!THIS->status)
-  	GB.ReturnBoolean(0);
+	if (!THIS->status)
+		GB.ReturnBoolean(0);
 	else
 	{
 		THIS->signals = get_signals(THIS);
@@ -443,7 +448,7 @@ END_PROPERTY
 
 // Gets / Sets serial port name (on Linux /dev/ttyS0, etc)
 
-BEGIN_PROPERTY(CSERIALPORT_Port)
+BEGIN_PROPERTY(SerialPort_Port)
 
 	if (READ_PROPERTY)
 		GB.ReturnString(THIS->portName);
@@ -458,7 +463,7 @@ END_PROPERTY
 
 // FlowControl : 1->CRTSCTS , 2-> XON/XOFF , 3-> XON/OFF plus CRTSCTS, 0 --> NONE
 
-BEGIN_PROPERTY(CSERIALPORT_FlowControl)
+BEGIN_PROPERTY(SerialPort_FlowControl)
 
 	int flow;
 
@@ -483,14 +488,14 @@ END_PROPERTY
 
 // Gets / Sets serial parity (E,O,N)
 
-BEGIN_PROPERTY(CSERIALPORT_Parity)
+BEGIN_PROPERTY(SerialPort_Parity)
 
-  int parity;
-  
-  if (READ_PROPERTY)
+	int parity;
+	
+	if (READ_PROPERTY)
 		GB.ReturnInteger(THIS->parity);
-  else
-  {
+	else
+	{
 		if (check_close(THIS))
 			return;
 
@@ -508,7 +513,7 @@ END_PROPERTY
 
 // Gets / Sets serial port Speed
 
-BEGIN_PROPERTY(CSERIALPORT_Speed)
+BEGIN_PROPERTY(SerialPort_Speed)
 
 	int speed;
 	
@@ -531,7 +536,7 @@ END_PROPERTY
 
 // Gets / Sets serial port Data Bits
 
-BEGIN_PROPERTY(CSERIALPORT_DataBits)
+BEGIN_PROPERTY(SerialPort_DataBits)
 
 	int value;
 	
@@ -554,7 +559,7 @@ END_PROPERTY
 
 // Gets / Sets serial port Stop Bits
 
-BEGIN_PROPERTY(CSERIALPORT_StopBits)
+BEGIN_PROPERTY(SerialPort_StopBits)
 
 	int value;
 	
@@ -577,7 +582,7 @@ END_PROPERTY
 
 // Gambas object "Constructor"
 
-BEGIN_METHOD_VOID(CSERIALPORT_new)
+BEGIN_METHOD_VOID(SerialPort_new)
 
 	THIS->portName = GB.NewZeroString("/dev/ttyS0");
 	THIS->speed = 19200;
@@ -590,7 +595,7 @@ END_METHOD
 
 // Gambas object "Destructor"
 
-BEGIN_METHOD_VOID(CSERIALPORT_free)
+BEGIN_METHOD_VOID(SerialPort_free)
 
 	close_serial_port(THIS);
 	GB.FreeString(&THIS->portName);
@@ -599,10 +604,11 @@ END_METHOD
 
 // To open the port
 
-BEGIN_METHOD_VOID(CSERIALPORT_Open)
+BEGIN_METHOD(SerialPort_Open, GB_INTEGER polling)
 
 	int err;
 	char buffer[8];
+	int polling = VARGOPT(polling, 50);
 
 	if (THIS->status)
 	{
@@ -620,7 +626,7 @@ BEGIN_METHOD_VOID(CSERIALPORT_Open)
 	THIS->signals = get_signals(THIS);
 	THIS->stream.desc = &SerialStream;
 	THIS->stream.tag = THIS;
-	assign_callback(THIS);
+	assign_callback(THIS, polling);
 
 	THIS->status = 1;
 
@@ -630,61 +636,61 @@ END_METHOD
 
 GB_DESC CSerialPortDesc[] =
 {
-  GB_DECLARE("SerialPort", sizeof(CSERIALPORT)),
+	GB_DECLARE("SerialPort", sizeof(CSERIALPORT)),
 
-  GB_INHERITS("Stream"),
+	GB_INHERITS("Stream"),
 
-  GB_CONSTANT("None", "i", 0),
-  
-  GB_CONSTANT("Hardware", "i", 1),
-  GB_CONSTANT("Software", "i", 2),
-  GB_CONSTANT("Both", "i", 3),
+	GB_CONSTANT("None", "i", 0),
+	
+	GB_CONSTANT("Hardware", "i", 1),
+	GB_CONSTANT("Software", "i", 2),
+	GB_CONSTANT("Both", "i", 3),
 
-  GB_CONSTANT("Even", "i", 1),
-  GB_CONSTANT("Odd", "i", 2),
-  
-  GB_CONSTANT("Bits1", "i", 1),
-  GB_CONSTANT("Bits2", "i", 2),
-  
-  GB_CONSTANT("Bits5", "i", 5),
-  GB_CONSTANT("Bits6", "i", 6),
-  GB_CONSTANT("Bits7", "i", 7),
-  GB_CONSTANT("Bits8", "i", 8),
-  
-  GB_EVENT("Read", NULL, NULL, &EVENT_Read),
-  GB_EVENT("DTRChange", NULL, "(CurrentValue)b", &EVENT_DTR),
-  GB_EVENT("DSRChange", NULL, "(CurrentValue)b", &EVENT_DSR),
-  GB_EVENT("RTSChange", NULL, "(CurrentValue)b", &EVENT_RTS),
-  GB_EVENT("CTSChange", NULL, "(CurrentValue)b", &EVENT_CTS),
-  GB_EVENT("DCDChange", NULL, "(CurrentValue)b", &EVENT_DCD),
-  GB_EVENT("RNGChange", NULL, "(CurrentValue)b", &EVENT_RNG),
+	GB_CONSTANT("Even", "i", 1),
+	GB_CONSTANT("Odd", "i", 2),
+	
+	GB_CONSTANT("Bits1", "i", 1),
+	GB_CONSTANT("Bits2", "i", 2),
+	
+	GB_CONSTANT("Bits5", "i", 5),
+	GB_CONSTANT("Bits6", "i", 6),
+	GB_CONSTANT("Bits7", "i", 7),
+	GB_CONSTANT("Bits8", "i", 8),
+	
+	GB_EVENT("Read", NULL, NULL, &EVENT_Read),
+	GB_EVENT("DTRChange", NULL, "(CurrentValue)b", &EVENT_DTR),
+	GB_EVENT("DSRChange", NULL, "(CurrentValue)b", &EVENT_DSR),
+	GB_EVENT("RTSChange", NULL, "(CurrentValue)b", &EVENT_RTS),
+	GB_EVENT("CTSChange", NULL, "(CurrentValue)b", &EVENT_CTS),
+	GB_EVENT("DCDChange", NULL, "(CurrentValue)b", &EVENT_DCD),
+	GB_EVENT("RNGChange", NULL, "(CurrentValue)b", &EVENT_RNG),
 
-  GB_METHOD("_new", NULL, CSERIALPORT_new, NULL),
-  GB_METHOD("_free", NULL, CSERIALPORT_free, NULL),
-  GB_METHOD("Open", NULL, CSERIALPORT_Open, NULL),
+	GB_METHOD("_new", NULL, SerialPort_new, NULL),
+	GB_METHOD("_free", NULL, SerialPort_free, NULL),
+	GB_METHOD("Open", NULL, SerialPort_Open, "[(Polling)i]"),
 
-  GB_PROPERTY("FlowControl","i",CSERIALPORT_FlowControl),
-  GB_PROPERTY("PortName", "s", CSERIALPORT_Port),
-  GB_PROPERTY("Parity", "i", CSERIALPORT_Parity),
-  GB_PROPERTY("Speed", "i", CSERIALPORT_Speed),
-  
-  GB_PROPERTY("DataBits", "i", CSERIALPORT_DataBits),
-  GB_PROPERTY("StopBits", "i", CSERIALPORT_StopBits),
-  GB_PROPERTY("DTR", "b", CSERIALPORT_DTR),
-  GB_PROPERTY("RTS", "b", CSERIALPORT_RTS),
-  GB_PROPERTY_READ("Status", "i", CSERIALPORT_Status),
-  GB_PROPERTY_READ("DSR", "b", CSERIALPORT_DSR),
-  GB_PROPERTY_READ("CTS", "b", CSERIALPORT_CTS),
-  GB_PROPERTY_READ("DCD", "b", CSERIALPORT_DCD),
-  GB_PROPERTY_READ("RNG", "b", CSERIALPORT_RNG),
-  
-  GB_CONSTANT("_IsControl", "b", TRUE),
-  GB_CONSTANT("_IsVirtual", "b", TRUE),
-  GB_CONSTANT("_Group", "s", "Network"),
-  GB_CONSTANT("_Properties", "s", "FlowControl{SerialPort.None;Hardware;Software;Both}=Hardware,PortName,Parity{SerialPort.None;Even;Odd}=None,Speed=19200,DataBits{SerialPort.Bits5;Bits6;Bits7;Bits8}=Bits8,StopBits{SerialPort.Bits1;Bits2}=Bits1"),
-  GB_CONSTANT("_DefaultEvent", "s", "Read"),
+	GB_PROPERTY("FlowControl","i",SerialPort_FlowControl),
+	GB_PROPERTY("PortName", "s", SerialPort_Port),
+	GB_PROPERTY("Parity", "i", SerialPort_Parity),
+	GB_PROPERTY("Speed", "i", SerialPort_Speed),
+	
+	GB_PROPERTY("DataBits", "i", SerialPort_DataBits),
+	GB_PROPERTY("StopBits", "i", SerialPort_StopBits),
+	GB_PROPERTY("DTR", "b", SerialPort_DTR),
+	GB_PROPERTY("RTS", "b", SerialPort_RTS),
+	GB_PROPERTY_READ("Status", "i", SerialPort_Status),
+	GB_PROPERTY_READ("DSR", "b", SerialPort_DSR),
+	GB_PROPERTY_READ("CTS", "b", SerialPort_CTS),
+	GB_PROPERTY_READ("DCD", "b", SerialPort_DCD),
+	GB_PROPERTY_READ("RNG", "b", SerialPort_RNG),
 
-  GB_END_DECLARE
+	GB_CONSTANT("_IsControl", "b", TRUE),
+	GB_CONSTANT("_IsVirtual", "b", TRUE),
+	GB_CONSTANT("_Group", "s", "Network"),
+	GB_CONSTANT("_Properties", "s", "FlowControl{SerialPort.None;Hardware;Software;Both}=Hardware,PortName,Parity{SerialPort.None;Even;Odd}=None,Speed=19200,DataBits{SerialPort.Bits5;Bits6;Bits7;Bits8}=Bits8,StopBits{SerialPort.Bits1;Bits2}=Bits1"),
+	GB_CONSTANT("_DefaultEvent", "s", "Read"),
+
+	GB_END_DECLARE
 };
 
 
