@@ -78,7 +78,7 @@ BEGIN_METHOD_VOID(GL_unknown)
 	
 	if (strncasecmp(name, "GL_", 3))
 	{
-		GB.Error(GB_ERR_NSYMBOL, name, "Gl");
+		GB.Error(GB_ERR_NSYMBOL, "Gl", name);
 		return;
 	}
 	
@@ -378,6 +378,12 @@ GB_DESC Cgl[] =
 	/* opengl constants */
 	/********************/
 	
+		GB_CONSTANT("FLOAT" , "i", GL_FLOAT),
+	GB_CONSTANT("INTEGER" , "i", GL_FLOAT),
+	GB_CONSTANT("UNSIGNED_BYTE", "i", GL_UNSIGNED_BYTE),
+	GB_CONSTANT("UNSIGNED_INT", "i", GL_UNSIGNED_INT),
+	GB_CONSTANT("DOUBLE", "i", GL_DOUBLE),
+
 	/* GetString */
 	GB_CONSTANT("EXTENSIONS", "i", GL_EXTENSIONS),
 	GB_CONSTANT("VENDOR", "i", GL_VENDOR),
@@ -396,13 +402,17 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("QUADS", "i", GL_QUADS),
 	GB_CONSTANT("QUAD_STRIP", "i", GL_QUAD_STRIP),
 	GB_CONSTANT("POLYGON", "i", GL_POLYGON),
+	GB_CONSTANT("VERTEX_ARRAY", "i", GL_VERTEX_ARRAY),
+  	GB_CONSTANT("COLOR_ARRAY", "i", GL_COLOR_ARRAY),
+	GB_CONSTANT("NORMAL_ARRAY", "i", GL_NORMAL_ARRAY),
+	GB_CONSTANT("TEXTURE_COORD_ARRAY", "i", GL_TEXTURE_COORD_ARRAY),
 
 	/* Matrix Mode */
 	GB_CONSTANT("MATRIX_MODE", "i", GL_MATRIX_MODE),
 	GB_CONSTANT("MODELVIEW", "i", GL_MODELVIEW),
 	GB_CONSTANT("PROJECTION", "i", GL_PROJECTION),
 	GB_CONSTANT("TEXTURE", "i", GL_TEXTURE),
-
+	
 	/* Points */
 	GB_CONSTANT("POINT_SMOOTH", "i", GL_POINT_SMOOTH),
 	GB_CONSTANT("POINT_SIZE", "i", GL_POINT_SIZE),
@@ -517,11 +527,11 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("SELECT", "i", GL_SELECT),
 
 	/* Feedback */
-	GB_CONSTANT("2D", "i", GL_2D),
-	GB_CONSTANT("3D", "i", GL_3D),
-	GB_CONSTANT("3D_COLOR", "i", GL_3D_COLOR),
-	GB_CONSTANT("3D_COLOR_TEXTURE", "i",  GL_3D_COLOR_TEXTURE),
-	GB_CONSTANT("4D_COLOR_TEXTURE", "i",  GL_4D_COLOR_TEXTURE),
+	GB_CONSTANT("_2D", "i", GL_2D),
+	GB_CONSTANT("_3D", "i", GL_3D),
+	GB_CONSTANT("_3D_COLOR", "i", GL_3D_COLOR),
+	GB_CONSTANT("_3D_COLOR_TEXTURE", "i",  GL_3D_COLOR_TEXTURE),
+	GB_CONSTANT("_4D_COLOR_TEXTURE", "i",  GL_4D_COLOR_TEXTURE),
 	GB_CONSTANT("POINT_TOKEN", "i", GL_POINT_TOKEN),
 	GB_CONSTANT("LINE_TOKEN", "i", GL_LINE_TOKEN),
 	GB_CONSTANT("LINE_RESET_TOKEN", "i", GL_LINE_RESET_TOKEN),
@@ -687,6 +697,8 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("DITHER", "i", GL_DITHER),
 	GB_CONSTANT("RGB", "i", GL_RGB),
 	GB_CONSTANT("RGBA", "i", GL_RGBA),
+	GB_CONSTANT("BGR", "i", GL_BGR),
+        GB_CONSTANT("BGRA", "i", GL_BGRA),
 
 	/* Implementation limits */
 	GB_CONSTANT("MAX_LIST_NESTING", "i", GL_MAX_LIST_NESTING),
@@ -868,6 +880,9 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("Q", "i", GL_Q),
 	GB_CONSTANT("TEXTURE_GEN_R", "i", GL_TEXTURE_GEN_R),
 	GB_CONSTANT("TEXTURE_GEN_Q", "i", GL_TEXTURE_GEN_Q),
+        GB_CONSTANT("GENERATE_MIPMAP", "i", GL_GENERATE_MIPMAP),        
+        GB_CONSTANT("TEXTURE_MAX_LEVEL", "i", GL_TEXTURE_MAX_LEVEL),
+        GB_CONSTANT("COMPRESSED_RGBA", "i", GL_COMPRESSED_RGBA),
 
 	/* Multitexture Mapping ARB*/
 	GB_CONSTANT("TEXTURE0_ARB", "i", GL_TEXTURE0_ARB),
@@ -875,6 +890,12 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("TEXTURE2_ARB", "i", GL_TEXTURE2_ARB),
 	GB_CONSTANT("TEXTURE3_ARB", "i", GL_TEXTURE3_ARB),
 	GB_CONSTANT("TEXTURE4_ARB", "i", GL_TEXTURE4_ARB),
+	
+	GB_CONSTANT("TEXTURE0", "i", GL_TEXTURE0),
+	GB_CONSTANT("TEXTURE1", "i", GL_TEXTURE1),
+	GB_CONSTANT("TEXTURE2", "i", GL_TEXTURE2),
+	GB_CONSTANT("TEXTURE3", "i", GL_TEXTURE3),
+	GB_CONSTANT("TEXTURE4", "i", GL_TEXTURE4),
 	GB_CONSTANT("COMBINE", "i", GL_COMBINE),
 	GB_CONSTANT("COMBINE_RGB", "i", GL_COMBINE_RGB),
 	GB_CONSTANT("COMBINE_ALPHA", "i", GL_COMBINE_ALPHA),
@@ -903,6 +924,9 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("CONSTANT_ARB", "i", GL_CONSTANT_ARB),
 	GB_CONSTANT("PRIMARY_COLOR_ARB", "i", GL_PRIMARY_COLOR_ARB),
 	GB_CONSTANT("PREVIOUS_ARB", "i", GL_PREVIOUS_ARB),
+	GB_CONSTANT("COMPRESSED_RGBA_S3TC_DXT1_EXT", "i", GL_COMPRESSED_RGBA_S3TC_DXT1_EXT),
+	GB_CONSTANT("COMPRESSED_RGBA_S3TC_DXT3_EXT", "i", GL_COMPRESSED_RGBA_S3TC_DXT3_EXT),
+	GB_CONSTANT("COMPRESSED_RGBA_S3TC_DXT5_EXT", "i", GL_COMPRESSED_RGBA_S3TC_DXT5_EXT),
 
 	/* 38; GL_ARB_texture_rectangle */
 	GB_CONSTANT("TEXTURE_RECTANGLE_ARB", "i", GL_TEXTURE_RECTANGLE_ARB),
@@ -913,6 +937,7 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("SAMPLER_2D_RECT_SHADOW_ARB", "i", GL_SAMPLER_2D_RECT_SHADOW_ARB),
 
 	/* 310; GL_EXT_framebuffer_object */
+	GB_CONSTANT("ELEMENT_ARRAY_BUFFER", "i", GL_ELEMENT_ARRAY_BUFFER),
 	GB_CONSTANT("FRAMEBUFFER_EXT", "i", GL_FRAMEBUFFER_EXT),
 	GB_CONSTANT("RENDERBUFFER_EXT", "i", GL_RENDERBUFFER_EXT),
 	GB_CONSTANT("STENCIL_INDEX1_EXT", "i", GL_STENCIL_INDEX1_EXT),
@@ -964,6 +989,25 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("MAX_COLOR_ATTACHMENTS_EXT", "i", GL_MAX_COLOR_ATTACHMENTS_EXT),
 	GB_CONSTANT("MAX_RENDERBUFFER_SIZE_EXT", "i", GL_MAX_RENDERBUFFER_SIZE_EXT),
 	GB_CONSTANT("INVALID_FRAMEBUFFER_OPERATION_EXT", "i", GL_INVALID_FRAMEBUFFER_OPERATION_EXT),
+	GB_CONSTANT("SAMPLES_PASSED", "i", GL_SAMPLES_PASSED),
+	GB_CONSTANT("QUERY_RESULT", "i", GL_QUERY_RESULT),
+	GB_CONSTANT("TRUE", "i", GL_TRUE),
+	GB_CONSTANT("FALSE", "i", GL_FALSE),
+	GB_CONSTANT("PRIMITIVES_GENERATED", "i", GL_PRIMITIVES_GENERATED),
+	GB_CONSTANT("TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN", "i", GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN),
+	//GB_CONSTANT("TIME_ELAPSED", "i",  GL_TIME_ELAPSED),
+	//GB_CONSTANT("TIMESTAMP", "i", ​ GL_TIMESTAMP​),
+	GB_CONSTANT("CURRENT_QUERY", "i",  GL_CURRENT_QUERY),
+	GB_CONSTANT("QUERY_COUNTER_BITS", "i",  GL_QUERY_COUNTER_BITS),
+
+    /* GL Array */
+    GB_CONSTANT("ARRAY_BUFFER", "i", GL_ARRAY_BUFFER),
+    GB_CONSTANT("STATIC_DRAW", "i", GL_STATIC_DRAW),
+
+
+
+   
+
 
 #if 0
 
