@@ -2,7 +2,7 @@
 
   gb.form.arrangement.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -137,10 +137,15 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 	WIDGET_TYPE wid;
 	int x, y, w, h, i;
 	int xc, yc, wc, hc;
+	#ifndef GET_MAX_SIZE
 	int wf, hf;
+	#endif
 	int dmax = 0, d;
 	int sexp, nexp;
-	bool swap, autoresize, has_expand_children = false;
+	bool swap, autoresize, invert;
+	#ifndef GET_MAX_SIZE
+	bool has_expand_children = false;
+	#endif
 	bool redo;
 	bool first;
 	void *ob;
@@ -198,7 +203,8 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 		//if (!strcmp(GET_OBJECT_NAME(_object), "HBox1"))
 		//	fprintf(stderr, "HBox1: child count: %d\n", gtk_count);
 
-		rtl = IS_RIGHT_TO_LEFT() ^ arr->invert;
+		invert = arr->invert;
+		rtl = IS_RIGHT_TO_LEFT() ^ invert;
 		rtlm = rtl ? -1 : 1;
 		swap = (arr->mode & 1) == 0; // means "vertical"
 
@@ -223,8 +229,10 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 			redo = false;
 
 			GET_WIDGET_CONTENTS(cont, xc, yc, wc, hc);
+			#ifndef GET_MAX_SIZE
 			wf = GET_WIDGET_W(cont) - wc;
 			hf = GET_WIDGET_H(cont) - hc;
+			#endif
 			
 			//fprintf(stderr, "cont: %s: %d %d %d %d (%d %d)\n", ((gControl *)_object)->name(), xc, yc, wc, hc, wf, hf);
 			
@@ -243,7 +251,8 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 			{
 				if (swap)
 				{
-					yc += indent;
+					if (!invert)
+						yc += indent;
 					hc -= indent;
 				}
 				else
@@ -273,8 +282,16 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 			y = yc;
 			w = h = 0;
 
-			if (rtl && !swap)
-				x += wc;
+			if (swap)
+			{
+				if (invert)
+					y += hc;
+			}
+			else
+			{
+				if (rtl)
+					x += wc;
+			}
 			
 			wid = 0;
 			RESET_CHILDREN_LIST();
@@ -318,7 +335,9 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 						sexp += spacing;
 					}
 					
+					#ifndef GET_MAX_SIZE
 					has_expand_children = nexp > 0;
+					#endif
 
 					if (autoresize)
 					{
@@ -353,7 +372,12 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 						if (!first)
 						{
 							if (swap)
-								y += spacing;
+							{
+								if (invert)
+									y -= spacing;
+								else
+									y += spacing;
+							}
 							else
 								x += spacing * rtlm;
 						}
@@ -379,8 +403,16 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 								if (w != GET_WIDGET_W(wid) || h != GET_WIDGET_H(wid))
 									redo = true;
 
-								MOVE_RESIZE_WIDGET(ob, wid, x, y, w, h);
-								y += h;
+								if (invert)
+								{
+									y -= h;
+									MOVE_RESIZE_WIDGET(ob, wid, x, y, w, h);
+								}
+								else
+								{
+									MOVE_RESIZE_WIDGET(ob, wid, x, y, w, h);
+									y += h;
+								}
 							}
 						}
 						else
@@ -432,7 +464,7 @@ void FUNCTION_NAME(void *_object) //(QFrame *cont)
 
 						if (swap)
 						{
-							if (rtl)
+							if (rtl || invert)
 							{
 								if ((y > yc) && ((y + (IS_EXPAND(ob) ? DESKTOP_SCALE : GET_WIDGET_H(wid))) > hc))
 								{
