@@ -2,7 +2,7 @@
 
   CClipboard.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,23 +25,22 @@
 
 #include "gambas.h"
 
-#include <qapplication.h>
-#include <qmime.h>
-#include <qclipboard.h>
-#include <qimage.h>
-#include <qevent.h>
-#include <qcolor.h>
-//Added by qt3to4:
+#include <QApplication>
+#include <QClipboard>
+#include <QImage>
+#include <QEvent>
+#include <QColor>
 #include <QDragMoveEvent>
 #include <QDropEvent>
 #include <QDragEnterEvent>
 #include <QMimeData>
+#include <QLineEdit>
+#include <QTextEdit>
+#include <QSpinBox>
 
 #include "CWidget.h"
 #include "CImage.h"
 #include "CClipboard.h"
-//#include "CListView.h"
-#include "CTreeView.h"
 
 CDRAG_INFO CDRAG_info = { 0 };
 bool CDRAG_dragging = false;
@@ -173,6 +172,55 @@ static void paste(const QMimeData *data, const char *fmt)
 
 ***************************************************************************/
 
+static GB_ARRAY _clipboard_formats = NULL;
+
+void CLIPBOARD_has_changed()
+{
+	GB.Unref(POINTER(&_clipboard_formats));
+	_clipboard_formats = NULL;
+}
+
+static GB_ARRAY load_clipboard_formats()
+{
+	if (!_clipboard_formats)
+	{
+		//qDebug("load clipboard formats");
+		GB.Array.New(&_clipboard_formats, GB_T_STRING, 0);
+		get_formats(QApplication::clipboard()->mimeData(), _clipboard_formats);
+		GB.Ref(_clipboard_formats);
+	}
+	
+	return _clipboard_formats;
+}
+
+static int get_clipboard_type()
+{
+	int i;
+	QString format;
+	
+	load_clipboard_formats();
+	
+	for (i = 0; i < GB.Array.Count(_clipboard_formats); i++)
+	{
+		format = *((char **)GB.Array.Get(_clipboard_formats, i));
+		if (format.startsWith("text/"))
+			return MIME_TEXT;
+		else if (format.startsWith("image/"))
+			return MIME_IMAGE;
+		else if (format == "application/x-qt-image")
+			return MIME_IMAGE;
+	}
+	
+	return MIME_UNKNOWN;
+}
+
+BEGIN_METHOD_VOID(Clipboard_exit)
+
+	CLIPBOARD_has_changed();
+
+END_METHOD
+
+
 BEGIN_METHOD_VOID(CCLIPBOARD_clear)
 
 	QApplication::clipboard()->clear();
@@ -182,25 +230,25 @@ END_METHOD
 
 BEGIN_PROPERTY(CCLIPBOARD_format)
 
-	GB.ReturnNewZeroString(TO_UTF8(get_format(QApplication::clipboard()->mimeData())));
+	load_clipboard_formats();
+	if (GB.Array.Count(_clipboard_formats) == 0)
+		GB.ReturnVoidString();
+	else
+		GB.ReturnString(*((char **)GB.Array.Get(_clipboard_formats, 0)));
 
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CCLIPBOARD_formats)
 
-	GB_ARRAY array;
-	
-	GB.Array.New(&array, GB_T_STRING, 0);
-	get_formats(QApplication::clipboard()->mimeData(), array);
-	GB.ReturnObject(array);
+	GB.ReturnObject(load_clipboard_formats());
 
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CCLIPBOARD_type)
 
-	GB.ReturnInteger(get_type(QApplication::clipboard()->mimeData()));
+	GB.ReturnInteger(get_clipboard_type());
 
 END_PROPERTY
 
@@ -256,10 +304,13 @@ BEGIN_METHOD(CCLIPBOARD_paste, GB_STRING format)
 
 END_METHOD
 
+
 GB_DESC CClipboardDesc[] =
 {
 	GB_DECLARE("Clipboard", 0), GB_VIRTUAL_CLASS(),
 
+	GB_STATIC_METHOD("_exit", NULL, Clipboard_exit, NULL),
+
 	GB_CONSTANT("None", "i", 0),
 	GB_CONSTANT("Text", "i", 1),
 	GB_CONSTANT("Image", "i", 2),
@@ -284,8 +335,9 @@ MyDragFrame::MyDragFrame(QWidget *parent) :
 {
 	setAutoFillBackground(true);
 	QPalette pal(palette());
-	pal.setColor(QPalette::Window, Qt::black);
+	pal.setColor(QPalette::Window, QColor(0, 0, 0, 128));
 	setPalette(pal);
+	//setWindowOpacity(0.5);
 }
 
 /*MyDragFrame::paintEvent(QPaintEvent *e)
@@ -311,6 +363,8 @@ static void hide_frame(CWIDGET *control)
 		delete _frame[i];
 		
 	_frame_visible = false;
+	GB.Unref(POINTER(&_frame_control));
+	_frame_control = NULL;
 }
 
 void CDRAG_hide_frame(CWIDGET *control)
@@ -339,6 +393,13 @@ static void show_frame(CWIDGET *control, int x, int y, int w, int h)
 	//x += p.x();
 	//y += p.y();
 	
+	if (control != _frame_control)
+	{
+		hide_frame(NULL);
+		_frame_control = control;
+		GB.Ref(control);
+	}
+	
 	if (!_frame_visible)
 	{
 		for (i = 0; i < 4; i++)
@@ -349,19 +410,17 @@ static void show_frame(CWIDGET *control, int x, int y, int w, int h)
 	//y -= 2;
 	//w += 4;
 	//h += 4;
-	if (w < 2 || h < 2)
+	if (w < 4 || h < 4)
 		return;
 	
 	_frame[0]->setGeometry(x, y, w, 2);
-	_frame[1]->setGeometry(x, y, 2, h);
-	_frame[2]->setGeometry(x + w - 2, y, 2, h);
 	_frame[3]->setGeometry(x, y + h - 2, w, 2);
+	_frame[1]->setGeometry(x, y + 2, 2, h - 4);
+	_frame[2]->setGeometry(x + w - 2, y + 2, 2, h - 4);
 	
 	for (i = 0; i < 4; i++)
 		_frame[i]->show();
 	
-	_frame_control = control;
-	
 	_frame_visible = true;
 }
 
@@ -456,7 +515,7 @@ void *CDRAG_drag(CWIDGET *source, GB_VARIANT_VALUE *data, GB_STRING *fmt)
 	//qDebug("end drag");
 	
 	hide_frame(NULL);
-	GB.Post((GB_POST_FUNC)post_exit_drag, 0);
+	GB.Post((GB_CALLBACK)post_exit_drag, 0);
 
 	if (CDRAG_destination)
 		GB.Unref(POINTER(&CDRAG_destination));
@@ -480,15 +539,20 @@ bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 	//qDebug("CDRAG_drag_enter: (%s %p) %p", GB.GetClassName(control), control, qobject_cast<MyListView *>(QWIDGET(control)));
 
 	// Hack for QScrollView
-	if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
-		((MyListView *)QWIDGET(control))->contentsDragEnterEvent((QDragEnterEvent *)e);
+	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
+		((MyListView *)QWIDGET(control))->contentsDragEnterEvent((QDragEnterEvent *)e);*/
 
 	if (!GB.CanRaise(control, EVENT_Drag))
 	{
 		if (GB.CanRaise(control, EVENT_DragMove) || GB.CanRaise(control, EVENT_Drop))
 			e->acceptProposedAction();
 		else
+		{
+			if (qobject_cast<QLineEdit *>(w) || qobject_cast<QTextEdit *>(w))
+				return false;
+
 			e->ignore();
+		}
 		return true;
 	}
 	
@@ -506,10 +570,29 @@ bool CDRAG_drag_enter(QWidget *w, CWIDGET *control, QDropEvent *e)
 	return cancel;
 }
 
+#define EXT(_ob) (((CWIDGET *)_ob)->ext)
+
+void CDRAG_drag_leave(CWIDGET *control)
+{
+	CDRAG_hide_frame(control);
+	
+	//while (EXT(control) && EXT(control)->proxy)
+	//	control = (CWIDGET *)(EXT(control)->proxy);
+
+__DRAG_LEAVE_TRY_PROXY:
+
+	GB.Raise(control, EVENT_DragLeave, 0);
+
+	if (EXT(control) && EXT(control)->proxy)
+	{
+		control = (CWIDGET *)(EXT(control)->proxy);
+		goto __DRAG_LEAVE_TRY_PROXY;
+	}
+}
+
 
 bool CDRAG_drag_move(QWidget *w, CWIDGET *control, QDropEvent *e)
 {
-	bool accepted;
 	bool cancel;
 	QPoint p;
 
@@ -517,7 +600,7 @@ bool CDRAG_drag_move(QWidget *w, CWIDGET *control, QDropEvent *e)
 
 	// Hack for QScrollView
 	
-	if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
+	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
 	{
 		accepted = e->isAccepted();
 		((MyListView *)QWIDGET(control))->contentsDragMoveEvent((QDragMoveEvent *)e);
@@ -525,7 +608,7 @@ bool CDRAG_drag_move(QWidget *w, CWIDGET *control, QDropEvent *e)
 			e->acceptProposedAction();
 		else
 			e->ignore();
-	}
+	}*/
 
 	if (!GB.CanRaise(control, EVENT_DragMove))
 	{
@@ -564,8 +647,8 @@ bool CDRAG_drag_drop(QWidget *w, CWIDGET *control, QDropEvent *e)
 		return false;
 
 	// Hack for QScrollView
-	if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
-		((MyListView *)QWIDGET(control))->contentsDropEvent((QDragMoveEvent *)e);
+	/*if (CWIDGET_test_flag(control, WF_SCROLLVIEW) && qobject_cast<MyListView *>(QWIDGET(control)))
+		((MyListView *)QWIDGET(control))->contentsDropEvent((QDragMoveEvent *)e);*/
 	
 	CDRAG_clear(true);
 	CDRAG_info.event = e;
@@ -598,6 +681,7 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CDRAG_exit)
 
+	hide_frame(NULL);
 	GB.Unref(POINTER(&_picture));
 
 END_METHOD
