@@ -2,7 +2,7 @@
 
   CContainer.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -251,8 +251,8 @@ static void resize_container(void *_object, QWidget *cont, int w, int h)
 #define IS_DESIGN(_object) (CWIDGET_test_flag(_object, WF_DESIGN) && CWIDGET_test_flag(_object, WF_DESIGN_LEADER))
 #define IS_WIDGET_VISIBLE(_widget) (_widget)->isVisible()
 
+//#define CAN_ARRANGE(_object) ((_object) && !CWIDGET_test_flag(_object, WF_DELETED) && (!GB.Is(_object, CLASS_Window) || (((CWINDOW *)_object)->opened)))
 #define CAN_ARRANGE(_object) ((_object) && ((CWIDGET *)(_object))->flag.shown && !CWIDGET_test_flag(_object, WF_DELETED))
-//(IS_WIDGET_VISIBLE(GET_CONTAINER(_object)) || IS_WIDGET_VISIBLE(GET_WIDGET(_object))))
 
 #if USE_CACHE
 
@@ -395,6 +395,7 @@ static void gms_move_resize_widget(QWidget *wid, int x, int y, int w, int h)
 #define RESIZE_CONTAINER(_object, _cont, _w, _h) (0)
 //#undef IS_WIDGET_VISIBLE
 //#define IS_WIDGET_VISIBLE(_cont) (1)
+#define GET_MAX_SIZE
 
 #include "gb.form.arrangement.h"
 
@@ -498,15 +499,17 @@ void MyFrame::setFrameStyle(int frame)
 	update();
 }
 
-void CCONTAINER_draw_frame(QPainter *p, int frame, QStyleOptionFrame &opt, QWidget *w)
+static void _draw_border(QPainter *p, int frame, QWidget *w, QStyleOptionFrame &opt)
 {
 	QStyle *style;
 	QStyleOptionFrameV3 optv3;
+	bool a;
+	QBrush save_brush;
 	//QRect rect = opt.rect;
-	
-	if (frame == 0)
+
+	if (frame == BORDER_NONE)
 		return;
-	
+
 	if (w)
 		style = w->style();
 	else
@@ -515,18 +518,30 @@ void CCONTAINER_draw_frame(QPainter *p, int frame, QStyleOptionFrame &opt, QWidg
 	switch (frame)
 	{
 		case BORDER_PLAIN:
-			qDrawPlainRect(p, opt.rect, CCOLOR_light_foreground());
-			//p->setPen(opt.palette.windowText().color());
+			//qDrawPlainRect(p, opt.rect, CCOLOR_light_foreground());
+			a = p->testRenderHint(QPainter::Antialiasing);
+			if (a)
+				p->setRenderHint(QPainter::Antialiasing, false);
+			p->setPen(CCOLOR_light_foreground());
+			p->setBrush(Qt::NoBrush);
+			opt.rect = QRect(opt.rect.x(), opt.rect.y(), opt.rect.width() - 1, opt.rect.height() - 1);
+			p->drawRect(opt.rect);
+			if (a)
+				p->setRenderHint(QPainter::Antialiasing, true);
 			break;
-			
+
 		case BORDER_SUNKEN:
 			optv3.rect = opt.rect;
 			optv3.state = opt.state | QStyle::State_Sunken;
 			optv3.frameShape = QFrame::StyledPanel;
+
+			save_brush = p->brush();
+			p->setBrush(QBrush());
 			style->drawPrimitive(QStyle::PE_Frame, &optv3, p, w);
+			p->setBrush(save_brush);
 			//style->drawControl(QStyle::CE_ShapedFrame, &optv3, p, w);
 			break;
-			
+
 		case BORDER_RAISED:
 			optv3.rect = opt.rect;
 			optv3.state = opt.state | QStyle::State_Raised;
@@ -537,41 +552,70 @@ void CCONTAINER_draw_frame(QPainter *p, int frame, QStyleOptionFrame &opt, QWidg
 			opt.state |= QStyle::State_Raised;
 			style->drawPrimitive(QStyle::PE_Frame, &opt, p, w);*/
 			break;
-			
+
 		case BORDER_ETCHED:
-			qDrawShadeRect(p, opt.rect, opt.palette, true, 1, 0);
-			/*p->setPen(opt.palette.shadow().color());
-			p->drawRect(rect);*/
+			optv3.rect = opt.rect;
+			//optv3.state = opt.state | QStyle::State_Raised;
+			optv3.frameShape = QFrame::StyledPanel;
+			style->drawPrimitive(QStyle::PE_FrameGroupBox, &optv3, p, w);
+			//qDrawShadeRect(p, opt.rect, opt.palette, true, 1, 0);
 			break;
-			
+
 		default:
 			return;
 	}
-	
-	/*if (rect.x() > 0)
-		p->drawLine(rect.x(), rect.y(), rect.x(), rect.y() + rect.height() - 1);
-	if (rect.x() > 0)
-		p->drawLine(rect.x(), rect.y(), rect.x() + rect.width() - 1, rect.y());
-	if (w->parentWidget())
+}
+
+void CCONTAINER_draw_border(QPainter *p, char frame, QWidget *wid)
+{
+	QStyleOptionFrame opt;
+	opt.init(wid);
+	opt.rect = QRect(0, 0, wid->width(), wid->height());
+	_draw_border(p, frame, wid, opt);
+}
+
+void CCONTAINER_draw_border_without_widget(QPainter *p, char frame, QStyleOptionFrame &opt)
+{
+	_draw_border(p, frame, NULL, opt);
+}
+
+int CCONTAINER_get_border_width(char border)
+{
+	switch (border)
 	{
-		int dx, dy;
-		
-		dx = rect.x() + rect.width() - 1;
-		dy = rect.y() + rect.height() - 1;
-		if (dx < (w->parentWidget()->width() - 1))
-			p->drawLine(dx, rect.y(), dx, dy);
-		if (dy < (w->parentWidget()->height() - 1))
-			p->drawLine(rect.x(), dy, dx, dy);
-	}*/
+		case BORDER_PLAIN:
+			return 1;
+			break;
+
+		case BORDER_SUNKEN:
+		case BORDER_RAISED:
+		case BORDER_ETCHED:
+
+			return qApp->style()->pixelMetric(QStyle::QStyle::PM_ComboBoxFrameWidth);
+			break;
+
+		default:
+			return 0;
+	}
+}
+
+void CCONTAINER_set_border(char *border, char new_border, QWidget *wid)
+{
+	int m;
+
+	if (new_border < BORDER_NONE || new_border > BORDER_ETCHED || new_border == *border)
+		return;
+
+	*border = new_border;
+	m = CCONTAINER_get_border_width(new_border);
+
+	wid->setContentsMargins(m, m, m, m);
+	wid->update();
 }
 
 void MyFrame::drawFrame(QPainter *p)
 {
-	QStyleOptionFrame opt;
-	opt.init(this);
-	opt.rect = QRect(0, 0, width(), height());
-
-	CCONTAINER_draw_frame(p, _frame, opt, this);
+	CCONTAINER_draw_border(p, _frame, this);
 }
 
 int MyFrame::frameWidth()
@@ -934,6 +978,14 @@ BEGIN_PROPERTY(UserContainer_Arrangement)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(UserContainer_Focus)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(CWIDGET_get_allow_focus(THIS));
+	else
+		CWIDGET_set_allow_focus(THIS, VPROP(GB_BOOLEAN));
+
+END_PROPERTY
 
 BEGIN_PROPERTY(Container_AutoResize)
 
@@ -1148,6 +1200,9 @@ BEGIN_PROPERTY(UserControl_Container)
 			GB.Error("Container must be a child control");
 		else
 		{
+			GB_COLOR bg = CWIDGET_get_background((CWIDGET *)current, true);
+			GB_COLOR fg = CWIDGET_get_foreground((CWIDGET *)current, true);
+
 			if (current)
 				CWIDGET_container_for(current, NULL);
 			CWIDGET_container_for(cont, THIS);
@@ -1157,6 +1212,8 @@ BEGIN_PROPERTY(UserControl_Container)
 			CWIDGET_update_design((CWIDGET *)THIS);
 			CCONTAINER_arrange(THIS);
 
+			CWIDGET_set_color((CWIDGET *)cont, bg, fg, true);
+
 			CWIDGET_register_proxy(THIS, cont);
 		}
 	}
@@ -1214,21 +1271,70 @@ BEGIN_PROPERTY(UserContainer_Design)
 END_PROPERTY
 
 
-BEGIN_METHOD(Container_Find, GB_INTEGER x; GB_INTEGER y)
+BEGIN_METHOD(Container_FindChild, GB_INTEGER x; GB_INTEGER y)
 
+	QObjectList list = CONTAINER->children();
+	int index = 0;
 	QWidget *w;
 	void *control;
 	
-	w = CONTAINER->childAt(VARG(x), VARG(y));
-	control = CWidget::get(w);
-	if (control == THIS)
-		control = NULL;
-	
-	GB.ReturnObject(control);
+	for(;;)
+	{
+		w = get_next_widget(list, index);
+		if (!w)
+			break;
+		if (w->geometry().contains(VARG(x), VARG(y)))
+		{
+			control = CWidget::get(w);
+			if (control)
+			{
+				GB.ReturnObject(control);
+				return;
+			}
+		}
+	}
+
+	GB.ReturnNull();
 
 END_METHOD
 
 
+BEGIN_METHOD(Container_unknown, GB_VALUE x; GB_VALUE y)
+
+	char *name = GB.GetUnknown();
+	int nparam = GB.NParam();
+	
+	if (strcasecmp(name, "Find"))
+	{
+		GB.Error(GB_ERR_NSYMBOL, GB.GetClassName(NULL), name);
+		return;
+	}
+	
+	if (nparam < 2)
+	{
+		GB.Error("Not enough argument");
+		return;
+	}
+	else if (nparam > 2)
+	{
+		GB.Error("Too many argument");
+		return;
+	}
+	
+	GB.Deprecated("gb.qt4", "Container.Find", "Container.FindChild");
+	
+	if (GB.Conv(ARG(x), GB_T_INTEGER) || GB.Conv(ARG(y), GB_T_INTEGER))
+		return;
+	
+	Container_FindChild(_object, _param);
+	
+	GB.ReturnConvVariant();
+
+END_METHOD
+
+
+//---------------------------------------------------------------------------
+
 GB_DESC CChildrenDesc[] =
 {
 	GB_DECLARE(".Container.Children", sizeof(CCONTAINER)), GB_VIRTUAL_CLASS(),
@@ -1256,7 +1362,8 @@ GB_DESC CContainerDesc[] =
 	GB_PROPERTY_READ("ClientH", "i", Container_Height),
 	GB_PROPERTY_READ("ClientHeight", "i", Container_Height),
 	
-	GB_METHOD("Find", "Control", Container_Find, "(X)i(Y)i"),
+	GB_METHOD("_unknown", "v", Container_unknown, "."),
+	GB_METHOD("FindChild", "Control", Container_FindChild, "(X)i(Y)i"),
 	
 	CONTAINER_DESCRIPTION,
 
@@ -1278,7 +1385,12 @@ GB_DESC CUserControlDesc[] =
 	GB_PROPERTY("_Container", "Container", UserControl_Container),
 	GB_PROPERTY("_AutoResize", "b", Container_AutoResize),
 	GB_PROPERTY("_Arrangement", "i", Container_Arrangement),
-	
+	GB_PROPERTY("_Padding", "i", Container_Padding),
+	GB_PROPERTY("_Spacing", "b", Container_Spacing),
+	GB_PROPERTY("_Margin", "b", Container_Margin),
+	GB_PROPERTY("_Indent", "b", Container_Indent),
+	GB_PROPERTY("_Invert", "b", Container_Invert),
+
 	USERCONTROL_DESCRIPTION,
 	
 	GB_END_DECLARE
@@ -1305,6 +1417,8 @@ GB_DESC CUserContainerDesc[] =
 	
 	GB_PROPERTY("Design", "b", UserContainer_Design),
 
+	//GB_PROPERTY("Focus", "b", UserContainer_Focus),
+
 	USERCONTAINER_DESCRIPTION,
 	
 	GB_END_DECLARE
