@@ -2,7 +2,7 @@
 
   CTextBox.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -385,39 +385,37 @@ static void combo_set_editable(void *_object, bool ed)
 	QString text;
 	bool hasFocus = COMBOBOX->hasFocus();
 	
+	if (ed == COMBOBOX->isEditable())
+		return;
+
 	COMBOBOX->blockSignals(true);
 	text = COMBOBOX->currentText();
 
 	if (ed)
 	{
-		if (!COMBOBOX->isEditable())
-		{
-			//CWidget::removeFilter(COMBOBOX);
-			COMBOBOX->setEditable(true);
-			COMBOBOX->setCompleter(0);
-			//CWidget::installFilter(COMBOBOX);
-			QObject::connect(COMBOBOX->lineEdit(), SIGNAL(returnPressed()), &CTextBox::manager, SLOT(onActivate()));
+		//CWidget::removeFilter(COMBOBOX);
+		COMBOBOX->setEditable(true);
+		COMBOBOX->setCompleter(0);
+		//CWidget::installFilter(COMBOBOX);
+		QObject::connect(COMBOBOX->lineEdit(), SIGNAL(returnPressed()), &CTextBox::manager, SLOT(onActivate()));
 
-			if (CWIDGET_test_flag(THIS, WF_DESIGN))
-			{
-				get(_object, &textbox);
-				//textbox->removeEventFilter(COMBOBOX);
-				COMBOBOX->setFocusProxy(0);
-			}
+		if (CWIDGET_test_flag(THIS, WF_DESIGN))
+		{
+			get(_object, &textbox);
+			//textbox->removeEventFilter(COMBOBOX);
+			COMBOBOX->setFocusProxy(0);
 		}
 	}
 	else
 	{
-		if (COMBOBOX->isEditable())
-		{
-			get(THIS, &textbox);
-			textbox->setFocusProxy(0);
-			COMBOBOX->setEditable(false);
-			COMBOBOX->update();
-		}
+		get(THIS, &textbox);
+		textbox->setFocusProxy(0);
+		COMBOBOX->setEditable(false);
+		COMBOBOX->update();
 	}
 	
 	combo_set_text(THIS, text);
+	CWIDGET_reset_color((CWIDGET *)THIS);
 
 	if (hasFocus)
 		COMBOBOX->setFocus();
@@ -680,6 +678,19 @@ BEGIN_PROPERTY(CCOMBOBOX_list)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(ComboBox_Border)
+
+	GET_TEXT_BOX();
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(COMBOBOX->hasFrame());
+	else
+		COMBOBOX->setFrame(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
+
+
 
 /***************************************************************************
 
@@ -833,6 +844,7 @@ GB_DESC CComboBoxDesc[] =
 	GB_PROPERTY("ReadOnly", "b", CCOMBOBOX_read_only),
 	GB_PROPERTY("Password", "b", CTEXTBOX_password),
 	GB_PROPERTY("MaxLength", "i", CTEXTBOX_max_length),
+	GB_PROPERTY("Border", "b", ComboBox_Border),
 
 	GB_PROPERTY_SELF("Selection", ".TextBox.Selection"),
 	GB_METHOD("Select", NULL, CTEXTBOX_sel_select, "[(Start)i(Length)i]"),
