@@ -2,7 +2,7 @@
 
   CWidget.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -61,9 +61,14 @@
 #include <QWheelEvent>
 #include <QHash>
 #include <QAbstractScrollArea>
-#include <Q3ScrollView>
 #include <QProgressBar>
 #include <QAbstractEventDispatcher>
+#include <QListWidget>
+#include <QComboBox>
+#include <QSpinBox>
+#include <QSet>
+#include <QScrollBar>
+#include <QLineEdit>
  
 #ifndef NO_X_WINDOW
 static QMap<int, int> _x11_to_qt_keycode;
@@ -76,10 +81,12 @@ static CWIDGET *_old_active_control = 0;
 static CWIDGET *_hovered = 0;
 static CWIDGET *_official_hovered = 0;
 
+QSet<CWIDGET *> *_enter_leave_set = NULL;
+
 #define EXT(_ob) (((CWIDGET *)_ob)->ext)
 
 #define ENSURE_EXT(_ob) (EXT(_ob) ? EXT(_ob) : alloc_ext((CWIDGET *)(_ob)))
-	
+
 #define HANDLE_PROXY(_ob) \
 	while (EXT(_ob) && EXT(_ob)->proxy) \
 		_ob = (typeof _ob)(EXT(_ob)->proxy);
@@ -185,7 +192,7 @@ static void set_design(CWIDGET *_object)
 	}
 }
 
-static void set_name(CWIDGET *_object, const char *name)
+void CWIDGET_set_name(CWIDGET *_object, const char *name)
 {
 	CWINDOW *window;
 	MyMainWindow *win = 0;
@@ -226,7 +233,12 @@ void *CWIDGET_get_parent(void *_object)
     return CWidget::get(parent);
 }
 
-static bool is_visible(void *_object)
+int CWIDGET_get_handle(void *_object)
+{
+	return (int)WIDGET->winId();
+}
+
+bool CWIDGET_is_visible(void *_object)
 {
 	return THIS->flag.visible; // || !QWIDGET(_object)->isHidden();
 }
@@ -266,8 +278,10 @@ static QWidget *get_viewport(QWidget *w)
 {
 	if (qobject_cast<QAbstractScrollArea *>(w))
 		return ((QAbstractScrollArea *)w)->viewport();
-	else if (qobject_cast<Q3ScrollView *>(w))
-		return ((Q3ScrollView *)w)->viewport();
+	//else if (qobject_cast<Q3ScrollView *>(w))
+	//	return ((Q3ScrollView *)w)->viewport();
+	//else if (qobject_cast<Q3ListView *>(w))
+	//	return ((Q3ListView *)w)->viewport();
 	else
 		return 0;
 }
@@ -283,11 +297,18 @@ void CWIDGET_update_design(CWIDGET *_object)
 
 void CWIDGET_init_name(CWIDGET *_object)
 {
+	static int n = 0;
 	char *name = GB.GetLastEventName();
+	
 	if (!name)
-		name = GB.GetClassName(THIS);
-	//qDebug("name: %p: %s", THIS, name);
-	set_name(THIS, name);
+	{
+		char buffer[16];
+		n++;
+		sprintf(buffer, "#%d", n);
+		CWIDGET_set_name(THIS, buffer);
+	}
+	else
+		CWIDGET_set_name(THIS, name);
 }
 
 bool CWIDGET_container_for(void *_object, void *container_for)
@@ -336,6 +357,25 @@ static void CWIDGET_leave(void *_object)
 	}
 }
 
+bool CWIDGET_get_allow_focus(void *_object)
+{
+	return WIDGET->focusPolicy() != Qt::NoFocus;
+}
+
+void CWIDGET_set_allow_focus(void *_object, bool f)
+{
+	if (f)
+	{
+		WIDGET->setFocusPolicy(GB.CanRaise(THIS, EVENT_MouseWheel) ? Qt::WheelFocus : Qt::StrongFocus);
+		WIDGET->setAttribute(Qt::WA_InputMethodEnabled, true);
+	}
+	else
+	{
+		WIDGET->setFocusPolicy(Qt::NoFocus);
+	}
+}
+
+
 void CWIDGET_new(QWidget *w, void *_object, bool no_show, bool no_filter, bool no_init)
 {
 	//QAbstractScrollArea *sa;
@@ -351,7 +391,7 @@ void CWIDGET_new(QWidget *w, void *_object, bool no_show, bool no_filter, bool n
 	if (!no_init)
 		CWIDGET_init_name(THIS);	
 
-	if (qobject_cast<QAbstractScrollArea *>(w) || qobject_cast<Q3ScrollView *>(w))
+	if (qobject_cast<QAbstractScrollArea *>(w)) // || qobject_cast<Q3ScrollView *>(w))
 		CWIDGET_set_flag(THIS, WF_SCROLLVIEW);
 
 	//w->setAttribute(Qt::WA_PaintOnScreen, true);
@@ -361,14 +401,14 @@ void CWIDGET_new(QWidget *w, void *_object, bool no_show, bool no_filter, bool n
 	//THIS->flag.fillBackground = GB.Is(THIS, CLASS_Container);
 	//w->setAutoFillBackground(THIS->flag.fillBackground);
 	
-	CCONTAINER_insert_child(THIS);
-
 	if (!no_show)
 	{
 		w->setGeometry(-16, -16, 8, 8);
 		CWIDGET_set_visible(THIS, true);
 		w->raise();
 	}
+	
+	CCONTAINER_insert_child(THIS);
 }
 
 
@@ -377,6 +417,50 @@ QString CWIDGET_Utf8ToQString(GB_STRING *str)
 	return QString::fromUtf8((const char *)(str->value.addr + str->value.start), str->value.len);
 }
 
+void *CWIDGET_enter_popup()
+{
+	void *save = _enter_leave_set;
+	
+	_enter_leave_set = new QSet<CWIDGET *>;
+	return save;
+}
+
+void CWIDGET_leave_popup(void *save)
+{
+	CWIDGET *_object;
+	QSetIterator<CWIDGET *> i(*_enter_leave_set);
+	
+	while (i.hasNext())
+	{
+		_object = i.next();
+		GB.Unref(POINTER(&_object));
+		if (_object)
+		{
+			if (THIS->flag.inside_later != THIS->flag.inside)
+			{
+				if (THIS->flag.inside_later)
+					CWIDGET_enter(THIS);
+				else
+					CWIDGET_leave(THIS);
+			}
+		}
+	}
+	
+	delete _enter_leave_set;
+	_enter_leave_set = (QSet<CWIDGET *>*) save;
+}
+
+static void insert_enter_leave_event(CWIDGET *control, bool in)
+{
+	control->flag.inside_later = in;
+	
+	if (_enter_leave_set->contains(control))
+		return;
+	
+	_enter_leave_set->insert(control);
+	GB.Ref(control);
+}
+
 static bool _post_check_hovered = false;
 static CWIDGET *_post_check_hovered_window = NULL;
 
@@ -483,6 +567,9 @@ static void CWIDGET_after_geometry_change(void *_object, bool arrange)
 			((MyDrawingArea *)((CWIDGET *)_object)->widget)->updateBackground();
 	}
 	
+	if (THIS->flag.ignore)
+		return;
+	
   arrange_parent(THIS);
 }
 
@@ -681,6 +768,7 @@ void CWIDGET_move_resize_cached(void *_object, int x, int y, int w, int h)
 }
 */
 
+#if 0
 void CWIDGET_check_hovered()
 {
 	//qDebug("CWIDGET_check_hovered: %p %s -> %p %s", _hovered, _hovered ? _hovered->name : 0, _official_hovered, _official_hovered ? _official_hovered->name : 0);
@@ -688,14 +776,15 @@ void CWIDGET_check_hovered()
 	if (_official_hovered != _hovered)
 	{
 		if (_official_hovered)
-			GB.Raise(_official_hovered, EVENT_Leave, NULL);
+			CWIDGET_leave(_official_hovered);
 		
 		if (_hovered)
-			GB.Raise(_hovered, EVENT_Enter, NULL);
+			CWIDGET_enter(_hovered);
 		
 		_official_hovered = _hovered;
 	}
 }
+#endif
 
 BEGIN_PROPERTY(Control_X)
 
@@ -763,9 +852,10 @@ BEGIN_PROPERTY(Control_Font)
 		THIS->font = CFONT_create(WIDGET->font(), 0, THIS);
 		GB.Ref(THIS->font);
 	}
-
+	
 	if (READ_PROPERTY)
 	{
+		*(((CFONT *)THIS->font)->font) = WIDGET->font();
 		GB.ReturnObject(THIS->font);
 	}
 	else
@@ -827,17 +917,10 @@ END_PROPERTY
 
 BEGIN_PROPERTY(Control_Hovered)
 
-	if (!is_visible(THIS))
-	{
+	if (!CWIDGET_is_visible(THIS))
 		GB.ReturnBoolean(false);
-		return;
-	}
-
-	QPoint m = QCursor::pos();
-	int x = WIDGET->mapToGlobal(QPoint(0, 0)).x(); 
-	int y = WIDGET->mapToGlobal(QPoint(0, 0)).y(); 
-
-	GB.ReturnBoolean(m.x() >= x && m.y() >= y && m.x() < (x + WIDGET->width()) && m.y() < (y + WIDGET->height()));
+	else
+		GB.ReturnBoolean(THIS->flag.inside);
 
 END_PROPERTY
 
@@ -952,7 +1035,7 @@ void CWIDGET_set_visible(CWIDGET *_object, bool v)
 BEGIN_PROPERTY(Control_Visible)
 
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(is_visible(THIS));
+		GB.ReturnBoolean(CWIDGET_is_visible(THIS));
 	else
 		CWIDGET_set_visible(THIS, VPROP(GB_BOOLEAN));
 
@@ -1327,36 +1410,23 @@ BEGIN_PROPERTY(CWIDGET_foreground)
 END_PROPERTY
 #endif
 
-static QWidget *get_color_widget(QWidget *w)
+
+static QWidget *get_color_widget(CWIDGET *_object)
 {
-	QWidget *view = get_viewport(w);
+	if (qobject_cast<MyScrollView *>(WIDGET))
+		return ((CSCROLLVIEW *)THIS)->container;
+	
+	QWidget *view = get_viewport(WIDGET);
 	if (view)
 		return view;
-	else
-		return w;
+
+	return WIDGET;
 }
 
-/*int get_real_background(CWIDGET *_object)
-{
-	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
-	if (THIS->bg == COLOR_DEFAULT && parent)
-		return get_real_background(parent);
-	else
-		return THIS->bg;
-}
-
-int get_real_foreground(CWIDGET *_object)
-{
-	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
-	if (THIS->fg == COLOR_DEFAULT && parent)
-		return get_real_foreground(parent);
-	else
-		return THIS->fg;
-}*/
 
 void CWIDGET_reset_color(CWIDGET *_object)
 {
-	int fg, bg;
+	GB_COLOR fg, bg;
 	QPalette palette;
 	QWidget *w;
 	
@@ -1364,42 +1434,74 @@ void CWIDGET_reset_color(CWIDGET *_object)
 	//qDebug("reset_color: %s", THIS->name);
 	//qDebug("set_color: (%s %p) bg = %08X (%d) fg = %08X (%d)", GB.GetClassName(THIS), THIS, THIS->bg, w->backgroundRole(), THIS->fg, w->foregroundRole());
 	
-	w = get_color_widget(WIDGET);
+	w = get_color_widget(THIS);
 	
 	if (!THIS_EXT || (THIS_EXT->bg == COLOR_DEFAULT && THIS_EXT->fg == COLOR_DEFAULT))
 	{
-		//CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
-		//if (parent)
-		//	w->setPalette(parent->widget->palette());
-		//else
 		w->setPalette(QPalette());
-		//WIDGET->setPalette(QPalette());
+		w->setAutoFillBackground(!THIS->flag.noBackground && THIS->flag.fillBackground);
 	}
 	else
 	{
-		palette = QPalette(); //w->palette());
 		bg = THIS_EXT->bg;
 		fg = THIS_EXT->fg;
 		
-		if (bg != COLOR_DEFAULT)
-			palette.setColor(w->backgroundRole(), QColor((QRgb)bg));
-		
-		if (fg != COLOR_DEFAULT)
+		if (qobject_cast<QComboBox *>(w))
 		{
-			palette.setColor(w->foregroundRole(), QColor((QRgb)fg));
-			//palette.setColor(QPalette::Text, QColor((QRgb)fg));
-			//palette.setColor(QPalette::WindowText, QColor((QRgb)fg));
-			//palette.setColor(QPalette::ButtonText, QColor((QRgb)fg));
-			/*palette.setColor(QPalette::WindowText, QColor((QRgb)fg));
-			palette.setColor(QPalette::Text, QColor((QRgb)fg));
-			palette.setColor(QPalette::ButtonText, QColor((QRgb)fg));*/
+			QComboBox *cb = (QComboBox *)w;
+			palette = QPalette();
+
+			if (bg != COLOR_DEFAULT)
+			{
+				if (cb->isEditable())
+					palette.setColor(QPalette::Base, QColor((QRgb)bg));
+				else
+					palette.setColor(QPalette::Button, QColor((QRgb)bg));
+			}
+
+			if (fg != COLOR_DEFAULT)
+			{
+				if (cb->isEditable())
+					palette.setColor(QPalette::Text, QColor((QRgb)fg));
+				else
+					palette.setColor(QPalette::ButtonText, QColor((QRgb)fg));
+			}
+
+			w->setPalette(palette);
 		}
+		else if (qobject_cast<QSpinBox *>(w))
+		{
+			palette = QPalette();
+
+			if (bg != COLOR_DEFAULT)
+			{
+				palette.setColor(QPalette::Base, QColor((QRgb)bg));
+			}
+
+			if (fg != COLOR_DEFAULT)
+			{
+				palette.setColor(QPalette::Text, QColor((QRgb)fg));
+			}
+
+			w->setPalette(palette);
+		}
+		else
+		{
+			palette = QPalette();
+		
+			if (bg != COLOR_DEFAULT)
+				palette.setColor(w->backgroundRole(), QColor((QRgb)bg));
 			
-		w->setPalette(palette);
-		//WIDGET->setPalette(palette);
-	}	
+			if (fg != COLOR_DEFAULT)
+				palette.setColor(w->foregroundRole(), QColor((QRgb)fg));
+		
+			w->setPalette(palette);
+
+			w->setAutoFillBackground(!THIS->flag.noBackground && (THIS->flag.fillBackground || ((THIS_EXT && THIS_EXT->bg != COLOR_DEFAULT) && w->backgroundRole() == QPalette::Window)));
+		}
+
+	}
 	
-	w->setAutoFillBackground(!THIS->flag.noBackground && (THIS->flag.fillBackground || ((THIS_EXT && THIS_EXT->bg != COLOR_DEFAULT) && w->backgroundRole() == QPalette::Window)));
 	//w->setAutoFillBackground(THIS->bg != COLOR_DEFAULT);
 	
 	if (GB.Is(THIS, CLASS_TextArea))
@@ -1412,8 +1514,11 @@ void CWIDGET_reset_color(CWIDGET *_object)
 		CWINDOW_define_mask((CWINDOW *)THIS);
 }
 
-void CWIDGET_set_color(CWIDGET *_object, int bg, int fg)
+
+void CWIDGET_set_color(CWIDGET *_object, int bg, int fg, bool handle_proxy)
 {
+	if (handle_proxy) { HANDLE_PROXY(_object); }
+
 	ENSURE_EXT(THIS);
 	THIS_EXT->bg = bg;
 	THIS_EXT->fg = fg;
@@ -1422,32 +1527,56 @@ void CWIDGET_set_color(CWIDGET *_object, int bg, int fg)
 }
 
 
-int CWIDGET_get_background(CWIDGET *_object)
+GB_COLOR CWIDGET_get_background(CWIDGET *_object, bool handle_proxy)
 {
+	if (handle_proxy) { HANDLE_PROXY(_object); }
+
 	return THIS_EXT ? THIS_EXT->bg : COLOR_DEFAULT;
-	/*
-	QWidget *w = get_color_widget(WIDGET);
-	
-	if (THIS->flag.default_bg)
-		return COLOR_DEFAULT;
-	else
-		return w->palette().color(w->backgroundRole()).rgb() & 0xFFFFFF;
-	*/
 }
-	
-int CWIDGET_get_foreground(CWIDGET *_object)
+
+
+int CWIDGET_get_real_background(CWIDGET *_object)
 {
-	return THIS_EXT ? THIS_EXT->fg : COLOR_DEFAULT;
-	/*
-	QWidget *w = get_color_widget(WIDGET);
+	GB_COLOR bg = CWIDGET_get_background(THIS);
 	
-	if (THIS->flag.default_fg)
-		return COLOR_DEFAULT;
+	if (bg != COLOR_DEFAULT)
+		return bg;
+
+	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
+	
+	if (parent)
+		return CWIDGET_get_real_background(parent);
 	else
-		return w->palette().color(w->foregroundRole()).rgb() & 0xFFFFFF;
-	*/
+		return QApplication::palette().color(QPalette::Window).rgb() & 0xFFFFFF;
 }
+
+
+GB_COLOR CWIDGET_get_foreground(CWIDGET *_object, bool handle_proxy)
+{
+	if (handle_proxy) { HANDLE_PROXY(_object); }
+
+	return THIS_EXT ? THIS_EXT->fg : COLOR_DEFAULT;
+}
+
+
+/*int CWIDGET_get_real_foreground(CWIDGET *_object, bool return_default)
+{
+	GB_COLOR fg = CWIDGET_get_foreground(THIS);
 	
+	if (fg != COLOR_DEFAULT)
+		return fg;
+
+	CWIDGET *parent = (CWIDGET *)CWIDGET_get_parent(THIS);
+	
+	if (parent)
+		return CWIDGET_get_real_foreground(parent, return_default);
+	else if (return_default)
+		return QApplication::palette().color(QPalette::WindowText).rgb() & 0xFFFFFF;
+	else
+		return COLOR_DEFAULT;
+}*/
+
+
 BEGIN_PROPERTY(Control_Background)
 
 	if (THIS_EXT && THIS_EXT->proxy)
@@ -1464,7 +1593,7 @@ BEGIN_PROPERTY(Control_Background)
 		GB.ReturnInteger(CWIDGET_get_background(THIS));
 	else
 	{
-		int col = VPROP(GB_INTEGER);
+		GB_COLOR col = VPROP(GB_INTEGER);
 		if (col != CWIDGET_get_background(THIS))
 			CWIDGET_set_color(THIS, col, CWIDGET_get_foreground(THIS));
 	}
@@ -1488,13 +1617,14 @@ BEGIN_PROPERTY(Control_Foreground)
 		GB.ReturnInteger(CWIDGET_get_foreground(THIS));
 	else
 	{
-		int col = VPROP(GB_INTEGER);
+		GB_COLOR col = VPROP(GB_INTEGER);
 		if (col != CWIDGET_get_foreground(THIS))
 			CWIDGET_set_color(THIS, CWIDGET_get_background(THIS), col);
 	}
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Control_Parent)
 
 	GB.ReturnObject(CWIDGET_get_parent(THIS));
@@ -1546,7 +1676,20 @@ BEGIN_PROPERTY(Control_Tooltip)
 	if (READ_PROPERTY)
 		GB.ReturnNewZeroString(TO_UTF8(WIDGET->toolTip()));
 	else
-		WIDGET->setToolTip(QSTRING_PROP());
+	{
+		QString tip = QSTRING_PROP();
+		if (THIS->flag.inside)
+		{
+			if (tip.isEmpty())
+				QToolTip::hideText();
+			else if (QToolTip::isVisible())
+			{
+				QToolTip::hideText();
+				QToolTip::showText(QCursor::pos(), tip, WIDGET);
+			}
+		}
+		WIDGET->setToolTip(tip);
+	}
 
 END_PROPERTY
 
@@ -1556,7 +1699,7 @@ BEGIN_PROPERTY(Control_Name)
 	if (READ_PROPERTY)
 		GB.ReturnString(THIS->name);
 	else
-		set_name(THIS, GB.ToZeroString(PROP(GB_STRING)));
+		CWIDGET_set_name(THIS, GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
@@ -1631,7 +1774,7 @@ BEGIN_METHOD(Control_Reparent, GB_OBJECT container; GB_INTEGER x; GB_INTEGER y)
 	if (GB.CheckObject(VARG(container)))
 		return;
 
-	show = is_visible(THIS);
+	show = CWIDGET_is_visible(THIS);
 	CWIDGET_set_visible(THIS, false);
 	WIDGET->setParent(QCONTAINER(VARG(container)));
 	WIDGET->move(p);
@@ -1644,19 +1787,28 @@ END_METHOD
 BEGIN_PROPERTY(Control_Drop)
 
 	if (READ_PROPERTY)
-		GB.ReturnBoolean(WIDGET->acceptDrops());
+		GB.ReturnBoolean(THIS->flag.drop);
 	else
 	{
-		WIDGET->setAcceptDrops(VPROP(GB_BOOLEAN));
+		THIS->flag.drop = VPROP(GB_BOOLEAN);
 		if (CWIDGET_test_flag(THIS, WF_SCROLLVIEW))
 			get_viewport(WIDGET)->setAcceptDrops(VPROP(GB_BOOLEAN));
+		else
+			WIDGET->setAcceptDrops(VPROP(GB_BOOLEAN));
 	}
 
 END_PROPERTY
 
+static bool has_tracking(CWIDGET *_object)
+{
+	HANDLE_PROXY(_object);
+	return THIS->flag.tracking;
+}
 
 BEGIN_PROPERTY(Control_Tracking)
 
+	HANDLE_PROXY(_object);
+
   if (READ_PROPERTY)
     GB.ReturnBoolean(THIS->flag.tracking);
   else
@@ -1755,7 +1907,7 @@ END_PROPERTY
 BEGIN_PROPERTY(CWIDGET_scrollbar)
 
 	QAbstractScrollArea *wid = qobject_cast<QAbstractScrollArea *>(WIDGET);
-	Q3ScrollView *sw = qobject_cast<Q3ScrollView *>(WIDGET);
+	//Q3ScrollView *sw = qobject_cast<Q3ScrollView *>(WIDGET);
 	int scroll;
 
 	if (wid)
@@ -1777,7 +1929,7 @@ BEGIN_PROPERTY(CWIDGET_scrollbar)
 			wid->setVerticalScrollBarPolicy( (scroll & 2) ? Qt::ScrollBarAsNeeded : Qt::ScrollBarAlwaysOff);
 		}
 	}
-	else if (sw)
+	/*else if (sw)
 	{
 		if (READ_PROPERTY)
 		{
@@ -1795,7 +1947,7 @@ BEGIN_PROPERTY(CWIDGET_scrollbar)
 			sw->setHScrollBarMode( (scroll & 1) ? Q3ScrollView::Auto : Q3ScrollView::AlwaysOff);
 			sw->setVScrollBarMode( (scroll & 2) ? Q3ScrollView::Auto : Q3ScrollView::AlwaysOff);
 		}
-	}
+	}*/
 
 END_PROPERTY
 
@@ -2150,11 +2302,6 @@ void CWidget::destroy()
 	
 	if (_post_check_hovered_window == THIS)
 		_post_check_hovered_window = NULL;
-	/*if (!_check_enter_leave)
-	{
-		_check_enter_leave = true;
-		QTimer::singleShot(0, &CWidget::manager, SLOT(checkEnterLeave));
-	}*/
 	
 	if (CWIDGET_active_control == THIS)
 		CWIDGET_active_control = NULL;
@@ -2184,7 +2331,7 @@ void CWidget::destroy()
 		GB.Free(POINTER(&THIS->ext));
 	}
 	
-	set_name(THIS, 0);
+	CWIDGET_set_name(THIS, 0);
 
 	dict.remove(w);
 
@@ -2282,6 +2429,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	bool cancel;
 	QPoint p;
 	void *jump;
+	bool parent_got_it;
 
 	//if (widget->isA("MyMainWindow"))
 	//	getDesignDebug(widget);
@@ -2329,6 +2477,10 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			}
 			else
 				goto _STANDARD;
+		case QEvent::TabletMove:
+		case QEvent::TabletPress:
+		case QEvent::TabletRelease:
+			jump = &&__TABLET; break;
 		default:
 			goto _STANDARD;
 	}
@@ -2353,8 +2505,15 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	{
 		QWidget *popup = qApp->activePopupWidget();
 		
-		if (real && (!popup || CWidget::getReal(popup)))
-			CWIDGET_enter(control);
+		//qDebug("enter %s real = %d popup = %p inside = %d", control->name, real, popup, control->flag.inside);
+		
+		if (real)
+		{
+			if (!popup || CWidget::getReal(popup))
+				CWIDGET_enter(control);
+			else if (_enter_leave_set)
+				insert_enter_leave_event(control, true);
+		}
 		
 		goto __NEXT;
 	}
@@ -2363,8 +2522,15 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	{
 		QWidget *popup = qApp->activePopupWidget();
 		
-		if (real && (!popup || CWidget::getReal(popup)))
-			CWIDGET_leave(control);
+		//qDebug("leave %s real = %d popup = %p inside = %d", control->name, real, popup, control->flag.inside);
+		
+		if (real)
+		{
+			if (!popup || CWidget::getReal(popup))
+				CWIDGET_leave(control);
+			else if (_enter_leave_set)
+				insert_enter_leave_event(control, false);
+		}
 		
 		goto __NEXT;
 	}
@@ -2392,10 +2558,17 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		//qDebug("Menu event! %p %d", control, EVENT_Menu);
 		if (GB.CanRaise(control, EVENT_Menu))
 		{
+			static int old = MENU_popup_count;
+
 			((QContextMenuEvent *)event)->accept();
+
 			GB.Raise(control, EVENT_Menu, 0);
-			return true;
+			if (MENU_popup_count != old)
+				return true;
+			else
+				goto __NEXT;
 		}
+
 		if (EXT(control) && EXT(control)->popup)
 		{
 			CWINDOW *window = CWidget::getWindow(control);
@@ -2405,9 +2578,9 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			return true;
 		}
 
-		if (EXT(control) && EXT(control)->proxy_for)
+		if (EXT(control) && EXT(control)->proxy)
 		{
-			control = (CWIDGET *)(EXT(control)->proxy_for);
+			control = (CWIDGET *)(EXT(control)->proxy);
 			goto __MENU_TRY_PROXY;
 		}
 		
@@ -2459,8 +2632,8 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			event_id = EVENT_MouseDown;
 			//state = mevent->buttons();
 			
-			CMOUSE_info.sx = p.x();
-			CMOUSE_info.sy = p.y();
+			MOUSE_info.sx = p.x();
+			MOUSE_info.sy = p.y();
 			//qDebug("MouseEvent: %d %d", mevent->x(), mevent->y());
 		}
 		else if (type == QEvent::MouseButtonDblClick)
@@ -2473,7 +2646,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			//state = mevent->buttons();
 		}
 
-		if (event_id == EVENT_MouseMove && mevent->buttons() == Qt::NoButton && !QWIDGET(control)->hasMouseTracking())
+		if (event_id == EVENT_MouseMove && mevent->buttons() == Qt::NoButton && !has_tracking(control))
 			goto _DESIGN;
 
 
@@ -2494,10 +2667,13 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			}*/
 			
 			CMOUSE_clear(true);
-			CMOUSE_info.x = p.x();
-			CMOUSE_info.y = p.y();
-			CMOUSE_info.button = mevent->buttons() | mevent->button();
-			CMOUSE_info.modifier = mevent->modifiers();
+			MOUSE_info.x = p.x();
+			MOUSE_info.y = p.y();
+			MOUSE_info.screenX = mevent->globalX();
+			MOUSE_info.screenY = mevent->globalY();
+			MOUSE_info.button = mevent->button();
+			MOUSE_info.state = mevent->buttons();
+			MOUSE_info.modifier = mevent->modifiers();
 
 			cancel = GB.Raise(control, event_id, 0); //, GB_T_INTEGER, p.x(), GB_T_INTEGER, p.y(), GB_T_INTEGER, state);
 
@@ -2508,7 +2684,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		}
 		
 		if (event_id == EVENT_MouseMove && !cancel && (mevent->buttons() != Qt::NoButton) && GB.CanRaise(control, EVENT_MouseDrag) && !CDRAG_dragging
-				&& ((abs(p.x() - CMOUSE_info.sx) + abs(p.y() - CMOUSE_info.sy)) > 8)) // QApplication::startDragDistance()))
+				&& ((abs(p.x() - MOUSE_info.sx) + abs(p.y() - MOUSE_info.sy)) > 8)) // QApplication::startDragDistance()))
 		{		
 			/*if (!design && CWIDGET_test_flag(control, WF_SCROLLVIEW))
 			{
@@ -2520,11 +2696,15 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			}*/
 			
 			CMOUSE_clear(true);
-			CMOUSE_info.x = p.x();
-			CMOUSE_info.y = p.y();
-			CMOUSE_info.button = mevent->buttons();
-			CMOUSE_info.modifier = mevent->modifiers();
+			MOUSE_info.x = p.x();
+			MOUSE_info.y = p.y();
+			MOUSE_info.screenX = mevent->globalX();
+			MOUSE_info.screenY = mevent->globalY();
+			MOUSE_info.button = mevent->button();
+			MOUSE_info.state = mevent->buttons();
+			MOUSE_info.modifier = mevent->modifiers();
 		
+			//qDebug("MouseDrag: %s", control->name);
 			cancel = GB.Raise(control, EVENT_MouseDrag, 0);
 			
 			CMOUSE_clear(false);
@@ -2532,6 +2712,9 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	
 		GB.Unref(POINTER(&control));
 		
+		if (!control)
+			return true;
+		
 		if (control->flag.grab && event_id == EVENT_MouseUp)
 			MyApplication::eventLoop->exit();
 		
@@ -2547,21 +2730,113 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		goto __NEXT;
 	}
 	
-	/*
-	__DBL_CLICK:
+	__TABLET:
 	{
+		QTabletEvent *tevent = (QTabletEvent *)event;
+
 		if (!original)
 			goto _DESIGN;
 
-		//GB.Raise(control, EVENT_DblClick, 0);
-		if (GB.CanRaise(control, EVENT_DblClick))
+		if (!real)
 		{
-			GB.Ref(control);
-			GB.Post((void (*)())post_dblclick_event, (intptr_t)control);
+			CWIDGET *cont = CWidget::get(widget);
+			if (CWIDGET_test_flag(cont, WF_SCROLLVIEW))
+			{
+				if (qobject_cast<QScrollBar *>(widget))
+					goto _STANDARD;
+				/*if (widget != get_viewport(QWIDGET(cont)))
+				{
+					if (!widget->objectName().isNull())
+						goto _STANDARD;
+				}*/
+			}
 		}
-		goto __NEXT;
+		
+		if (!control->flag.use_tablet)
+			goto __NEXT;
+		
+	__TABLET_TRY_PROXY:
+	
+		p.setX(tevent->globalX());
+		p.setY(tevent->globalY());
+		p = QWIDGET(control)->mapFromGlobal(p);
+		
+		if (type == QEvent::TabletPress)
+		{
+			//qDebug("MouseDown on %p (%s %p) %s%s", widget, control ? GB.GetClassName(control) : "-", control, real ? "REAL " : "", design ? "DESIGN " : "");
+
+			event_id = EVENT_MouseDown;
+			//state = mevent->buttons();
+			
+			//MOUSE_info.sx = p.x();
+			//MOUSE_info.sy = p.y();
+			
+			control->flag.tablet_pressed = true;
+			//qDebug("MouseEvent: %d %d", mevent->x(), mevent->y());
+		}
+		else if (type == QEvent::TabletMove)
+		{
+			//if (!control->flag.tracking && !control->flag.tablet_pressed)
+			//	return false;
+			
+			event_id = EVENT_MouseMove;
+		}
+		else //if (type == QEvent::TabletRelease)
+		{
+			event_id = EVENT_MouseUp;
+			//state = mevent->buttons();
+		}
+
+		//if (event_id == EVENT_MouseMove && mevent->buttons() == Qt::NoButton && !QWIDGET(control)->hasMouseTracking())
+		//	goto _DESIGN;
+
+
+		cancel = false;
+
+		if (GB.CanRaise(control, event_id))
+		{
+			//MOUSE_info.x = p.x();
+			//MOUSE_info.y = p.y();
+			//POINTER_info.screenX = tevent->globalX();
+			//POINTER_info.screenY = tevent->globalY();
+			//MOUSE_info.modifier = tevent->modifiers();
+			POINTER_info.tx = tevent->hiResGlobalX();
+			POINTER_info.ty = tevent->hiResGlobalY();
+			POINTER_info.pressure = tevent->pressure();
+			POINTER_info.rotation = tevent->rotation();
+			POINTER_info.xtilt = tevent->xTilt();
+			POINTER_info.ytilt = tevent->yTilt();
+			
+			switch(tevent->pointerType())
+			{
+				case QTabletEvent::Pen: POINTER_info.type = POINTER_PEN; break;
+				case QTabletEvent::Eraser: POINTER_info.type = POINTER_ERASER; break;
+				case QTabletEvent::Cursor: POINTER_info.type = POINTER_CURSOR; break;
+				default: POINTER_info.type = POINTER_MOUSE;
+			}
+
+			//cancel = GB.Raise(control, event_id, 0);
+
+			//CMOUSE_clear(false);
+		}
+		
+		//if (control->flag.grab && event_id == EVENT_MouseUp)
+		//	MyApplication::eventLoop->exit();
+		
+		if (event_id == EVENT_MouseUp)
+			control->flag.tablet_pressed = false;
+		
+		//if (cancel)
+		//	return true;
+		
+		if (EXT(control) && EXT(control)->proxy_for)
+		{
+			control = (CWIDGET *)(EXT(control)->proxy_for);
+			goto __TABLET_TRY_PROXY;
+		}
+		
+		return false; // We fill the information, and then expect Qt to generate a Mouse event from the Tablet event
 	}
-	*/
 	
 	__KEY:
 	{
@@ -2571,20 +2846,26 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		if (!real || !original)
 			goto _DESIGN;
 		#endif
+			
+		if (control->flag.no_keyboard)
+			goto _DESIGN;
 
 		event_id = (type == QEvent::KeyRelease) ? EVENT_KeyRelease : EVENT_KeyPress;
 		cancel = false;
+		parent_got_it = false;
 
 		#if QT_VERSION > 0x030005
 		if (!original && type != QEvent::InputMethod)
 			goto _DESIGN; //_ACCEL;
 		#endif
 
-		/*qDebug("QKeyEvent: %s: (%s %s) window:%d -> %d %s",
+		if (type == QEvent::KeyRelease && kevent->isAutoRepeat())
+			goto __NEXT;
+			
+		/*qDebug("QKeyEvent: %s: (%s %s) -> %d %s %s",
 			type == QEvent::KeyPress ? "KeyPress" : "KeyRelease",
 			GB.GetClassName(control), control->name,
-			((QWidget *)widget)->isWindow(),  
-			kevent->key(), (const char *)kevent->text().toLatin1());*/
+			kevent->key(), (const char *)kevent->text().toLatin1(), kevent->isAutoRepeat() ? "AR" : "--");*/
 
 		//qDebug("CWidget::eventFilter: KeyPress on %s %p", GB.GetClassName(control), control);
 			
@@ -2611,15 +2892,23 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 		}
 		#endif
 		
-		if (!cancel)
-			cancel = raise_key_event_to_parent_window(control, event_id);
+		GB.Ref(control);
+		
+		if (!parent_got_it)
+		{
+			parent_got_it = true;
+			if (!cancel)
+				cancel = raise_key_event_to_parent_window(control, event_id);
+		}
 		
 		if (!cancel)
 			cancel = GB.Raise(control, event_id, 0);
 
+		GB.Unref(POINTER(&control));
+		
 		CKEY_clear(false);
 
-		if (cancel && (type != QEvent::KeyRelease))
+		if ((cancel && (type != QEvent::KeyRelease)) || !control)
 			return true;
 
 		if (EXT(control) && EXT(control)->proxy_for)
@@ -2709,12 +2998,14 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			p = ((QWidget *)widget)->mapTo(QWIDGET(control), p);
 
 			CMOUSE_clear(true);
-			CMOUSE_info.x = p.x();
-			CMOUSE_info.y = p.y();
-			CMOUSE_info.button = ev->buttons();
-			CMOUSE_info.modifier = ev->modifiers();
-			CMOUSE_info.orientation = ev->orientation();
-			CMOUSE_info.delta = ev->delta();
+			MOUSE_info.x = p.x();
+			MOUSE_info.y = p.y();
+			MOUSE_info.screenX = ev->globalX();
+			MOUSE_info.screenY = ev->globalY();
+			MOUSE_info.state = ev->buttons();
+			MOUSE_info.modifier = ev->modifiers();
+			MOUSE_info.orientation = ev->orientation();
+			MOUSE_info.delta = ev->delta();
 
 			cancel = GB.Raise(control, EVENT_MouseWheel, 0);
 
@@ -2735,39 +3026,66 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 	
 	__DRAG_ENTER:
 	{
-		//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
+		if (!control->flag.drop)
+			goto __NEXT;
+
 		if (CDRAG_drag_enter((QWidget *)widget, control, (QDropEvent *)event))
 		{
 			if (!((QDropEvent *)event)->isAccepted())
 				CDRAG_hide_frame(control);
 			return true;
 		}
+		
 		goto __NEXT;
 	}
 	
 	__DRAG_MOVE:
 	{
-		//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
-		if (CDRAG_drag_move((QWidget *)widget, control, (QDropEvent *)event))
+		if (!control->flag.drop)
+			goto __NEXT;
+
+		for(;;)
 		{
-			if (!((QDropEvent *)event)->isAccepted())
-				CDRAG_hide_frame(control);
-			return true;
+			if (GB.CanRaise(control, EVENT_DragMove))
+			{
+				if (CDRAG_drag_move(QWIDGET(control), control, (QDropEvent *)event))
+				{
+					if (!((QDropEvent *)event)->isAccepted())
+						CDRAG_hide_frame(control);
+					break;
+				}
+			}
+
+			if (EXT(control) && EXT(control)->proxy)
+			{
+				control = (CWIDGET *)(EXT(control)->proxy);
+				continue;
+			}
+			else
+				break;
 		}
-		goto __NEXT;
-	}
-	
-	__DROP:
-	{
-		//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
-		if (CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event))
-			return true;
+		
 		goto __NEXT;
 	}
 	
 	__DRAG_LEAVE:
 	{
-		CDRAG_hide_frame(control);
+		if (!control->flag.drop)
+			goto __NEXT;
+
+		CDRAG_drag_leave(control);
+		goto __NEXT;
+	}
+	
+	__DROP:
+	{
+		if (!control->flag.drop)
+			goto __NEXT;
+
+		//if (!CWIDGET_test_flag(control, WF_NO_DRAG))
+		CDRAG_drag_leave(control);
+		if (CDRAG_drag_drop((QWidget *)widget, control, (QDropEvent *)event))
+			return true;
 		goto __NEXT;
 	}
 	
@@ -2818,12 +3136,8 @@ _STANDARD:
 
 /** Action *****************************************************************/
 
-#define HAS_ACTION(_control) CWIDGET_test_flag((CWIDGET *)(_control), WF_ACTION)
-#define SET_ACTION(_control, _flag) \
-	if (_flag) \
-		CWIDGET_set_flag((CWIDGET *)(_control), WF_ACTION); \
-	else \
-		CWIDGET_clear_flag((CWIDGET *)(_control), WF_ACTION);
+#define HAS_ACTION(_control) ((CWIDGET *)(_control))->flag.has_action
+#define SET_ACTION(_control, _flag) (((CWIDGET *)(_control))->flag.has_action = (_flag))
 
 #include "gb.form.action.h"
 
@@ -2964,6 +3278,7 @@ GB_DESC CControlDesc[] =
 	GB_EVENT("Drag", NULL, NULL, &EVENT_Drag),
 	GB_EVENT("DragMove", NULL, NULL, &EVENT_DragMove),
 	GB_EVENT("Drop", NULL, NULL, &EVENT_Drop),
+	GB_EVENT("DragLeave", NULL, NULL, &EVENT_DragLeave),
 
 	CONTROL_DESCRIPTION,
 
