@@ -2,7 +2,7 @@
 
   CWindow.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -98,7 +98,6 @@ DECLARE_EVENT(EVENT_Show);
 DECLARE_EVENT(EVENT_Hide);
 DECLARE_EVENT(EVENT_Title);
 DECLARE_EVENT(EVENT_Icon);
-//DECLARE_EVENT(EVENT_State);
 
 DECLARE_METHOD(CWINDOW_show);
 
@@ -127,6 +126,7 @@ bool CWINDOW_has_property(QWidget *w, Atom property)
 
 	return X11_window_has_property(w->winId(), property);
 }
+
 #endif
 
 // Fix a QT little boring visual bug on menubars
@@ -226,6 +226,7 @@ static bool emit_open_event(void *_object)
 	qDebug("emit_open_event: %s %p", GB.GetClassName(THIS), THIS);
 	#endif
 	//THIS->opening = true;
+	//WINDOW->configure();
 	GB.Raise(THIS, EVENT_Open, 0);
 	//THIS->opening = false;
 	if (CWIDGET_test_flag(THIS, WF_CLOSED))
@@ -253,10 +254,20 @@ static void handle_focus(CWINDOW *_object)
 	}
 }
 
+static void raise_resize_event(void *_object)
+{
+	if (WINDOW->width() != THIS->last_resize_w || WINDOW->height() != THIS->last_resize_h)
+	{
+		THIS->last_resize_w = WINDOW->width();
+		THIS->last_resize_h = WINDOW->height();
+		GB.Raise(THIS, EVENT_Resize, 0);
+	}
+}
+
 static void post_show_event(void *_object)
 {
 	GB.Raise(THIS, EVENT_Move, 0);
-	GB.Raise(THIS, EVENT_Resize, 0);
+	raise_resize_event(THIS);
 	handle_focus(THIS);
 }
 
@@ -305,11 +316,7 @@ void CWINDOW_ensure_active_window()
 }
 
 
-/***************************************************************************
-
-	Window
-
-***************************************************************************/
+//-- Window ---------------------------------------------------------------
 
 static void show_later(CWINDOW *_object)
 {
@@ -390,10 +397,6 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent)
 
 	THIS->container = container;
 	CWIDGET_new(win, (void *)_object, true);
-	//container->setPaletteBackgroundColor(Qt::yellow);
-	//container->setBackgroundOrigin(QWidget::WindowOrigin);
-
-	//qDebug("CWINDOW_new: %p: %s", THIS, GB.Debug.GetCurrentPosition());
 	
 	if (win)
 	{
@@ -448,7 +451,7 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent)
 	}
 	#endif
 
-
+	#if 1
 	if (THIS->embedded && !THIS->xembed)
 	{
 		/* ### This can call post_show_event() directly, whereas the function is not terminated */
@@ -462,6 +465,7 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent)
 	}
 	//else
 	//	THIS->hidden = TRUE;
+	#endif
 	
 	THIS->showMenuBar = true;
 
@@ -472,6 +476,8 @@ BEGIN_METHOD_VOID(CFORM_new)
 
 	if (!GB.Parent(_object))
 		GB.Attach(_object, _object, "Form");
+	
+	CWIDGET_set_name((CWIDGET *)THIS, GB.GetClassName((void *)THIS));
 
 END_METHOD
 
@@ -479,7 +485,7 @@ END_METHOD
 BEGIN_METHOD_VOID(CFORM_main)
 
 	CWINDOW *form = (CWINDOW *)GB.AutoCreate(GB.GetClass(NULL), 0);
-	//if (!((MyMainWindow *)form->widget.widget)->isHidden())
+	
 	if (!form->hidden)
 		CWINDOW_show(form, NULL);
 		
@@ -909,44 +915,6 @@ BEGIN_PROPERTY(CWINDOW_mask)
 
 END_PROPERTY
 
-#if 0
-BEGIN_PROPERTY(CWINDOW_state)
-
-	if (!THIS->toplevel)
-	{
-		if (READ_PROPERTY)
-			GB.ReturnInteger(0);
-	}
-	else
-	{
-		if (READ_PROPERTY)
-			GB.ReturnInteger(WINDOW->getState());
-		else
-			WINDOW->setState(VPROP(GB_INTEGER));
-	}
-
-END_PROPERTY
-
-static void show_window_state(void *_object)
-{
-	THIS->stateChange = false;
-	
-	if (WINDOW)
-	{
-		if (WINDOW->windowState() & Qt::WindowMinimized)
-			WINDOW->showMinimized();
-		else if (WINDOW->windowState() & Qt::WindowFullScreen)
-			WINDOW->showFullScreen();
-		else if (WINDOW->windowState() & Qt::WindowMaximized)
-			WINDOW->showMaximized();
-		else
-			WINDOW->showNormal();
-	}
-	
-	//qDebug("show_window_state %s %p", GB.GetClassName(THIS), THIS);
-	GB.Unref(POINTER(&_object));
-}
-#endif
 
 static void manage_window_state(void *_object, void *_param, Qt::WindowState state)
 {
@@ -958,32 +926,13 @@ static void manage_window_state(void *_object, void *_param, Qt::WindowState sta
 	else
 	{
 		if (READ_PROPERTY)
-			GB.ReturnBoolean(WINDOW->windowState() & state);
+			GB.ReturnBoolean(WINDOW->getState() & state);
 		else
 		{
 			if (VPROP(GB_BOOLEAN))
-				WINDOW->setWindowState(WINDOW->windowState() | state);
+				WINDOW->setState(WINDOW->getState() | state);
 			else
-				WINDOW->setWindowState(WINDOW->windowState() & ~state);
-			
-			if (WINDOW->isVisible())
-			{
-				if (WINDOW->windowState() & Qt::WindowMinimized)
-					WINDOW->showMinimized();
-				else if (WINDOW->windowState() & Qt::WindowFullScreen)
-					WINDOW->showFullScreen();
-				else if (WINDOW->windowState() & Qt::WindowMaximized)
-					WINDOW->showMaximized();
-				else
-					WINDOW->showNormal();
-			}
-			/*if (WINDOW->isVisible() && !THIS->stateChange)
-			{
-				THIS->stateChange = true;
-				//qDebug("post show_window_state %s %p", GB.GetClassName(THIS), THIS);
-				GB.Ref(THIS);
-				GB.Post((GB_POST_FUNC)show_window_state, (intptr_t)THIS);
-			}*/
+				WINDOW->setState(WINDOW->getState() & ~state);
 		}
 	}
 }
@@ -1223,15 +1172,17 @@ BEGIN_PROPERTY(CWINDOW_visible)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CWINDOW_control_count)
+BEGIN_PROPERTY(Window_Controls_Count)
 
 	QList<QWidget *> list = WINDOW->findChildren<QWidget *>();
 	int i;
 	int n = 0;
+	CWIDGET *control;
 
 	for (i = 0; i < list.count(); i++)
 	{
-		if (CWidget::getReal(list.at(i)))
+		control = CWidget::getReal(list.at(i));
+		if (control && !CWIDGET_check(control))
 			n++;
 	}
 
@@ -1240,7 +1191,7 @@ BEGIN_PROPERTY(CWINDOW_control_count)
 END_PROPERTY
 
 
-BEGIN_METHOD_VOID(CWINDOW_control_next)
+BEGIN_METHOD_VOID(Window_Controls_next)
 
 	QList<QWidget *> children = WINDOW->findChildren<QWidget *>();
 	CWIDGET *control;
@@ -1261,7 +1212,7 @@ BEGIN_METHOD_VOID(CWINDOW_control_next)
 		control = CWidget::getReal(children.at(index));
 		index++;
 	}
-	while (!control);
+	while (!control || CWIDGET_check(control));
 
 	ENUM(int) = index;
 	GB.ReturnObject(control);
@@ -1277,9 +1228,14 @@ BEGIN_METHOD(CWINDOW_reparent, GB_OBJECT container; GB_INTEGER x; GB_INTEGER y)
 END_METHOD
 
 
-BEGIN_METHOD(CWINDOW_get, GB_STRING name)
+BEGIN_METHOD(Window_Controls_get, GB_STRING name)
 
-	GB.ReturnObject(WINDOW->names[GB.ToZeroString(ARG(name))]);
+	CWIDGET *control = WINDOW->names[GB.ToZeroString(ARG(name))];
+	
+	if (!control || CWIDGET_check(control))
+		GB.ReturnNull();
+	else
+		GB.ReturnObject(control);
 
 END_METHOD
 
@@ -1420,9 +1376,9 @@ GB_DESC CWindowControlsDesc[] =
 {
 	GB_DECLARE(".Window.Controls", 0), GB_VIRTUAL_CLASS(),
 
-	GB_METHOD("_next", "Control", CWINDOW_control_next, NULL),
-	GB_METHOD("_get", "Control", CWINDOW_get, "(Name)s"),
-	GB_PROPERTY_READ("Count", "i", CWINDOW_control_count),
+	GB_METHOD("_next", "Control", Window_Controls_next, NULL),
+	GB_METHOD("_get", "Control", Window_Controls_get, "(Name)s"),
+	GB_PROPERTY_READ("Count", "i", Window_Controls_Count),
 
 	GB_END_DECLARE
 };
@@ -1461,7 +1417,7 @@ GB_DESC CWindowDesc[] =
 
 	GB_METHOD("_new", NULL, CWINDOW_new, "[(Parent)Control;]"),
 	GB_METHOD("_free", NULL, CWINDOW_free, NULL),
-	GB_METHOD("_get", "Control", CWINDOW_get, "(Name)s"),
+	GB_METHOD("_get", "Control", Window_Controls_get, "(Name)s"),
 
 	GB_METHOD("Close", "b", CWINDOW_close, "[(Return)i]"),
 	GB_METHOD("Raise", NULL, CWINDOW_raise, NULL),
@@ -1502,6 +1458,7 @@ GB_DESC CWindowDesc[] =
 	GB_PROPERTY("Margin", "b", Container_Margin),
 	GB_PROPERTY("AutoResize", "b", Container_AutoResize),
   GB_PROPERTY("Invert", "b", Container_Invert),
+  GB_PROPERTY("Indent", "b", Container_Indent),
 	
 	//GB_PROPERTY("Type", "i", CWINDOW_type),
 	GB_PROPERTY("Utility", "b", Window_Utility),
@@ -1525,9 +1482,8 @@ GB_DESC CWindowDesc[] =
 	GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
 	GB_EVENT("Title", NULL, NULL, &EVENT_Title),
 	GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
-	//GB_EVENT("State", NULL, NULL, &EVENT_State),
 	
-	GB_INTERFACE("Draw", &DRAW_Interface),
+	//GB_INTERFACE("Draw", &DRAW_Interface),
 
 	GB_END_DECLARE
 };
@@ -1570,15 +1526,13 @@ MyMainWindow::MyMainWindow(QWidget *parent, const char *name, bool embedded) :
 	QWidget::QWidget(parent, embedded ? Qt::Widget : Qt::Window)
 {
 	sg = 0;
-	//shown = false;
-	//border = BorderResizable;
 	_border = true;
 	_resizable = true;
-	//state = StateNormal;
 	_deleted = false;
 	_type = _NET_WM_WINDOW_TYPE_NORMAL;
 	_enterLoop = false;
 	_utility = false;
+	_state = windowState();
 	
 	//setAttribute(Qt::WA_KeyCompression, true);
 	//setAttribute(Qt::WA_InputMethodEnabled, true);
@@ -1659,6 +1613,8 @@ void MyMainWindow::showEvent(QShowEvent *e)
 {
 	CWINDOW *_object = (CWINDOW *)CWidget::get(this);
 	
+	//qDebug("showEvent: %s\n", GB.GetClassName(THIS));
+	
 	emit_open_event(THIS);
 	
 	//CWINDOW_fix_menubar(THIS);
@@ -1707,6 +1663,56 @@ void MyMainWindow::activateLater()
 	activateWindow();
 }
 
+void MyMainWindow::present()
+{
+	if (!isVisible())
+	{
+		//X11_window_startup(WINDOW->winId(), THIS->x, THIS->y, THIS->w, THIS->h);
+
+		if (isUtility() && _resizable)
+			setMinimumSize(THIS->minw, THIS->minh);
+		
+		if (getState() & Qt::WindowMinimized)
+			showMinimized();
+		else if (getState() & Qt::WindowFullScreen)
+			showFullScreen();
+		else if (getState() & Qt::WindowMaximized)
+			showMaximized();
+		else
+			show();
+		
+		if (isUtility() && _resizable)
+    	setSizeGrip(true);
+		else
+			setSizeGrip(false);
+		
+		if (hasBorder())
+		{
+			//MAIN_process_events();
+			//usleep(50000);
+			//_activate = TRUE;
+			//if (isToolbar())
+			//QTimer::singleShot(50, this, SLOT(activateLater()));
+			//else
+			activateWindow();
+		}
+	}
+	else
+	{
+		//_activate = true;
+		
+		if (getState() & Qt::WindowMinimized)
+		{
+			setState(windowState() & ~Qt::WindowMinimized);
+			//qDebug("_activate set #2");
+		}
+		
+		raise();
+		if (hasBorder())
+			activateWindow();
+	}
+}
+
 void MyMainWindow::showActivate(QWidget *transient)
 {
 	CWIDGET *_object = CWidget::get(this);
@@ -1724,7 +1730,6 @@ void MyMainWindow::showActivate(QWidget *transient)
 		{
 			if (newParentWidget && parentWidget() != newParentWidget)
 			{
-				//qDebug("showActivate");
 				doReparent(newParentWidget, pos());
 			}
 		}
@@ -1738,66 +1743,9 @@ void MyMainWindow::showActivate(QWidget *transient)
 		setWindowTitle(TO_QSTRING(GB.Application.Title()));
 
 	initProperties();
+	//::sleep(1);
 
-	if (!isVisible())
-	{
-		//GB.Raise(THIS, EVENT_Open, 0);
-
-		//X11_window_startup(WINDOW->winId(), THIS->x, THIS->y, THIS->w, THIS->h);
-
-		if (isUtility() && _resizable)
-		{
-			setMinimumSize(THIS->minw, THIS->minh);
-    	setSizeGrip(true);
-		}
-		else
-			setSizeGrip(false);
-		
-		if (windowState() & Qt::WindowMinimized)
-			showMinimized();
-		else if (windowState() & Qt::WindowFullScreen)
-			showFullScreen();
-		else if (windowState() & Qt::WindowMaximized)
-			showMaximized();
-		else
-			show();
-
-		/*if (_type == _NET_WM_WINDOW_TYPE_NORMAL
-			  || _type == _NET_WM_WINDOW_TYPE_DOCK
-			  || _type == _NET_WM_WINDOW_TYPE_TOOLBAR
-			  || _type == _NET_WM_WINDOW_TYPE_MENU
-			  || _type == _NET_WM_WINDOW_TYPE_UTILITY
-			  || _type == _NET_WM_WINDOW_TYPE_DIALOG
-			  || _type == _NET_WM_WINDOW_TYPE_DROPDOWN_MENU
-			  || _type == _NET_WM_WINDOW_TYPE_POPUP_MENU
-			  || _type == _NET_WM_WINDOW_TYPE_COMBO)
-		{*/
-		if (hasBorder())
-		{
-			//MAIN_process_events();
-			//usleep(50000);
-			//_activate = TRUE;
-			//if (isToolbar())
-			QTimer::singleShot(50, this, SLOT(activateLater()));
-			//else
-			//	activateWindow();
-		}
-	}
-	else
-	{
-		//_activate = true;
-		
-		if (windowState() & Qt::WindowMinimized)
-		{
-			setWindowState(windowState() & ~Qt::WindowMinimized);
-			//qDebug("_activate set #2");
-		}
-		
-		raise();
-		if (hasBorder())
-			activateWindow();
-	}
-
+	present();
 	afterShow();
 
 	#ifndef NO_X_WINDOW
@@ -1812,20 +1760,42 @@ void MyMainWindow::showActivate(QWidget *transient)
 	#endif
 }
 
+void on_error_show_modal(MODAL_INFO *info)
+{
+	qDebug("on_error_show_modal");
+	
+	info->that->_enterLoop = false;
+	MyApplication::eventLoop->exit();
+
+	GB.Debug.LeaveEventLoop();
+
+	MyApplication::eventLoop = info->old;
+	CWINDOW_Current = info->save;
+	
+	if (info->that && info->that->isPersistent())
+	{
+		info->that->setSizeGrip(false);
+		info->that->setWindowModality(Qt::NonModal);
+	}
+}
+
 void MyMainWindow::showModal(void)
 {
 	//Qt::WindowFlags flags = windowFlags() & ~Qt::WindowType_Mask;
 	CWIDGET *_object = CWidget::get(this);
 	bool persistent = CWIDGET_test_flag(THIS, WF_PERSISTENT);
-	CWINDOW *save = CWINDOW_Current;
 	//QPoint p = pos();
-	QEventLoop *old;
 	QEventLoop eventLoop;
+	GB_ERROR_HANDLER handler;
+	MODAL_INFO info;
 
 	if (isModal())
 		return;
 
-	old = MyApplication::eventLoop;
+	info.that = this;
+	info.old = MyApplication::eventLoop;
+	info.save = CWINDOW_Current;
+	
 	MyApplication::eventLoop = &eventLoop;
 
 	setWindowModality(Qt::ApplicationModal);
@@ -1838,7 +1808,7 @@ void MyMainWindow::showModal(void)
 
 	_enterLoop = false; // Do not call exitLoop() if we do not entered the loop yet!
 	
-	show();
+	present();
 	afterShow();
 	
 	#ifndef NO_X_WINDOW
@@ -1854,11 +1824,22 @@ void MyMainWindow::showModal(void)
 	
 	_enterLoop = true;
 	
+	GB.Debug.EnterEventLoop();
+
+	handler.handler = (GB_CALLBACK)on_error_show_modal;
+	handler.arg1 = (intptr_t)&info;
+	
+	GB.OnErrorBegin(&handler);
+	
 	eventLoop.exec();
+	
+	GB.OnErrorEnd(&handler);
+	
+	GB.Debug.LeaveEventLoop();
 	//eventLoop.processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 	
-	MyApplication::eventLoop = old;
-	CWINDOW_Current = save;
+	MyApplication::eventLoop = info.old;
+	CWINDOW_Current = info.save;
 		
 	if (persistent)
 	{
@@ -1875,6 +1856,7 @@ void MyMainWindow::showPopup(QPoint &pos)
 	CWIDGET *_object = CWidget::get(this);
 	bool persistent = CWIDGET_test_flag(THIS, WF_PERSISTENT);
 	CWINDOW *save = CWINDOW_Current;
+	void *save_popup;
 
 	if (isModal())
 		return;
@@ -1904,6 +1886,8 @@ void MyMainWindow::showPopup(QPoint &pos)
 	//handle_focus(THIS);
 	//activateWindow();
 	
+	save_popup = CWIDGET_enter_popup();
+	
 	_enterLoop = true;
 	
 	QEventLoop eventLoop;
@@ -1911,7 +1895,9 @@ void MyMainWindow::showPopup(QPoint &pos)
 	
 	old = MyApplication::eventLoop;
 	MyApplication::eventLoop = &eventLoop;
+	GB.Debug.EnterEventLoop();
 	eventLoop.exec();
+	GB.Debug.LeaveEventLoop();
 	MyApplication::eventLoop = old;
 	//eventLoop.exec();
 	//eventLoop.processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
@@ -1924,6 +1910,8 @@ void MyMainWindow::showPopup(QPoint &pos)
 		setWindowFlags(Qt::Window | flags);
 	}
 	
+	CWIDGET_leave_popup(save_popup);
+	
 	//CWIDGET_check_hovered();
 }
 
@@ -2114,16 +2102,7 @@ void MyMainWindow::resizeEvent(QResizeEvent *e)
 	//qDebug("THIS->h = %ld  THIS->container->height() = %ld  height() = %ld", THIS->h, THIS->container->height(), height());
 
 	if (THIS->opened)
-	{
-		/*w = THIS->w;
-		h = THIS->h;*/
-		GB.Raise(THIS, EVENT_Resize, 0);
-		/*if (w != THIS->w || h != THIS->h)
-		{
-			GB.Ref(THIS);
-			GB.Post((void (*)())post_resize_event, (intptr_t)THIS);
-		}*/
-	}
+		raise_resize_event(THIS);
 }
 
 
@@ -2183,6 +2162,7 @@ void MyMainWindow::keyPressEvent(QKeyEvent *e)
 		if (!test->isVisible() || !test->isEnabled())
 			return;
 
+		test->setFocus();
 		test->animateClick();
 		e->accept();
 	}
@@ -2330,7 +2310,10 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
 	e->accept();
 
 	if (isModal() && _enterLoop)
+	{
+		_enterLoop = false;
 		MyApplication::eventLoop->exit();
+	}
 
 	#if DEBUG_WINDOW
 	qDebug("THIS->opened <- false: %p: %s", THIS, GB.GetClassName(THIS));
@@ -2487,16 +2470,27 @@ void MyMainWindow::configure()
 {
 	CWINDOW *_object = (CWINDOW *)CWidget::get(this);
 	QMenuBar *menuBar = THIS->menuBar;
+	bool arrange = false;
+	QRect geom;
 	
 	//qDebug("THIS->menuBar = %p  menuBar() = %p", THIS->menuBar, menuBar());
 
 	if (menuBar && THIS->showMenuBar && !THIS->hideMenuBar)
 	{
-		//qDebug("action.count = %d", menuBar->actions().count());
-		//h = menuBar->sizeHint().height();
+		int h = menuBar->sizeHint().height();
+		
+		if (h == 0)
+			h = menuBar->height();
+		
 		menuBar->show();
-		THIS->container->setGeometry(0, menuBar->height(), this->width(), this->height() - menuBar->height());
-		menuBar->setGeometry(0, 0, this->width(), menuBar->height());
+		geom = QRect(0, h, this->width(), this->height() - h);
+		
+		if (THIS->container->geometry() != geom)
+		{
+			arrange = true;
+			THIS->container->setGeometry(geom);
+		}
+		menuBar->setGeometry(0, 0, this->width(), h);
 	}
 	else
 	{
@@ -2506,12 +2500,20 @@ void MyMainWindow::configure()
 			menuBar->lower();
 		}
 		//qDebug("configure: %s (%d %d)", GB.GetClassName(THIS), this->width(), this->height());
-		THIS->container->setGeometry(0, 0, this->width(), this->height());
-		//THIS->container->setGeometry(0, 0, THIS->w, THIS->h);
+		
+		geom = QRect(0, 0, this->width(), this->height());
+		
+		if (THIS->container->geometry() != geom)
+		{
+			arrange = true;
+			THIS->container->setGeometry(geom);
+		}
+		
 		THIS->container->raise();
 	}
 	
-	CCONTAINER_arrange(THIS);
+	if (arrange)
+		CCONTAINER_arrange(THIS);
 
 	//qDebug(">>> THIS->menuBar = %p  menuBar() = %p", THIS->menuBar, menuBar());
 
@@ -2569,6 +2571,18 @@ void MyMainWindow::changeEvent(QEvent *e)
 	}*/
 }
 
+Qt::WindowStates MyMainWindow::getState() const
+{
+	return isVisible() ? windowState() : _state;
+}
+
+void MyMainWindow::setState(Qt::WindowStates state)
+{
+	if (isVisible())
+		setWindowState(state);
+	else
+		_state = state;
+}
 
 /***************************************************************************
 
@@ -2675,12 +2689,14 @@ bool CWindow::eventFilter(QObject *o, QEvent *e)
 		{
 			MyMainWindow *w = (MyMainWindow *)o;
 
-			//qDebug("Show: %s %d (%d)", GB.GetClassName(THIS), !WINDOW->isHidden(), e->spontaneous());
-			
 			if (THIS->toplevel)
 				w->center();
 			
 			//handle_focus(THIS);
+			emit_open_event(THIS);
+			
+			//qDebug("eventFilter: Show: %s %d (%d) focus = %p", GB.GetClassName(THIS), !WINDOW->isHidden(), e->spontaneous(), THIS->focus);
+			
 			post_show_event(THIS);
 			//CWINDOW_define_mask(THIS);
 			
@@ -2764,7 +2780,7 @@ void CWindow::removeTopLevel(CWINDOW *_object)
   count = list.count();
 	
 	#if DEBUG_WINDOW
-  qDebug("removeTopLevel: count = %d (%p %s %s)", count, THIS, THIS->widget.name, THIS->embedded ? "E" : "W");
+	qDebug("removeTopLevel: count = %d (%p %s %s)", count, THIS, THIS->widget.name, THIS->embedded ? "E" : "W");
 	#endif
 
 	MAIN_check_quit();
