@@ -2,7 +2,7 @@
 
   cpaint_impl.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,6 +34,9 @@
 #include <QPainter>
 #include <QPen>
 #include <QBrush>
+#include <QVector>
+#include <QTextDocument>
+
 #ifndef NO_X_WINDOW
 #include <QX11Info>
 #endif
@@ -53,6 +56,29 @@
 #include "csvgimage.h"
 #include "cpaint_impl.h"
 
+class ClipInfo
+{
+public:
+	QPainterPath *path;
+	GB_RECT *rect;
+	
+	ClipInfo() { path = NULL; rect = NULL; }
+	~ClipInfo() { delete path; delete rect; }
+};
+
+typedef
+	struct {
+		QPainter *painter;
+		QPainterPath *path;
+		int fillRule;
+		QTransform *init;
+		float bx, by;
+		QPainterPath *clip;
+		GB_RECT *clipRect;
+		QList<ClipInfo *> *clipStack;
+	}
+	QT_PAINT_EXTRA;
+	
 #define EXTRA(d) ((QT_PAINT_EXTRA *)d->extra)
 
 #define COLOR_TO_INT(color) ((color).rgba() ^ 0xFF000000)
@@ -90,8 +116,9 @@ static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 	MyPaintEngine *engine = (MyPaintEngine *)device->paintEngine();
 	engine->patchFeatures();
 	
-	EXTRA(d)->path = 0;
-	EXTRA(d)->clip = 0;
+	//EXTRA(d)->path = NULL;
+	//EXTRA(d)->clip = NULL;
+	//EXTRA(d)->clipRect = NULL;
 	
 	EXTRA(d)->init = new QTransform();
 	*(EXTRA(d)->init) = PAINTER(d)->worldTransform();
@@ -104,13 +131,49 @@ static bool init_painting(GB_PAINT *d, QPaintDevice *device)
 	pen.setCapStyle(Qt::FlatCap);
 	pen.setJoinStyle(Qt::MiterJoin);
 	pen.setMiterLimit(10.0);
-	pen.setWidthF(2.0);
+	pen.setWidthF(1.0);
 	PAINTER(d)->setPen(pen);
 	PAINTER(d)->setBrush(Qt::black);
 	
 	return FALSE;
 }
 
+static QColor get_color(GB_PAINT *d, GB_COLOR col)
+{
+	if (col == GB_COLOR_DEFAULT)
+	{
+		if (GB.Is(d->device, CLASS_Control))
+			col = CWIDGET_get_real_background((CWIDGET *)d->device);
+		else
+			col = 0xFFFFFF;
+	}
+
+	return CCOLOR_make(col);
+}
+
+static void begin_clipping(GB_PAINT *d)
+{
+	if (CLIP(d))
+	{
+		GB_RECT *r;
+		QTransform save = PAINTER(d)->worldTransform();
+		PAINTER(d)->resetTransform();
+		r = EXTRA(d)->clipRect;
+		if (r)
+			PAINTER(d)->setClipRect(r->x, r->y, r->w, r->h);
+		else
+			PAINTER(d)->setClipPath(*CLIP(d));
+		PAINTER(d)->setWorldTransform(save);
+	}
+}
+
+static void end_clipping(GB_PAINT *d)
+{
+	if (CLIP(d))
+		PAINTER(d)->setClipping(false);
+}
+
+//---------------------------------------------------------------------------
 
 static int Begin(GB_PAINT *d)
 {
@@ -167,6 +230,9 @@ static int Begin(GB_PAINT *d)
 		if (init_painting(d, target))
 			return TRUE;
 		
+		if (wid->isCached())
+			PAINTER(d)->initFrom(wid);
+		
 		d->width = wid->width();
 		d->height = wid->height();
 		return FALSE;
@@ -237,13 +303,23 @@ static void End(GB_PAINT *d)
 static void Save(GB_PAINT *d)
 {
 	QT_PAINT_EXTRA *dx = EXTRA(d);
+	ClipInfo *ci;
 	
 	PAINTER(d)->save();
 	
 	if (!dx->clipStack)
-		dx->clipStack = new QList<QPainterPath *>;
+		dx->clipStack = new QList<ClipInfo *>;
 	
-	dx->clipStack->append(dx->clip ? new QPainterPath(*dx->clip) : NULL);
+	ci = new ClipInfo;
+	if (dx->clip)
+		ci->path = new QPainterPath(*dx->clip);
+	if (dx->clipRect)
+	{
+		ci->rect = new GB_RECT;
+		*ci->rect = *dx->clipRect;
+	}
+	
+	dx->clipStack->append(ci);
 }
 
 static void Restore(GB_PAINT *d)
@@ -254,10 +330,21 @@ static void Restore(GB_PAINT *d)
 	
 	if (dx->clipStack && !dx->clipStack->isEmpty())
 	{
-		QPainterPath *path = dx->clipStack->takeLast();
+		ClipInfo *ci = dx->clipStack->takeLast();
+		
 		delete dx->clip;
-		dx->clip = path ? new QPainterPath(*path) : NULL;
-		delete path;
+		dx->clip = ci->path ? new QPainterPath(*(ci->path)) : NULL;
+		
+		delete dx->clipRect;
+		if (ci->rect)
+		{
+			dx->clipRect = new GB_RECT;
+			*dx->clipRect = *ci->rect;
+		}
+		else
+			dx->clipRect = NULL;
+		
+		delete ci;
 	}
 }
 		
@@ -314,6 +401,15 @@ static void init_path(GB_PAINT *d)
 	if (!PATH(_d)) \
 		EXTRA(_d)->path = new QPainterPath();
 
+static void delete_clip_rect(GB_PAINT *d)
+{
+	if (EXTRA(d)->clipRect)
+	{
+		delete EXTRA(d)->clipRect;
+		EXTRA(d)->clipRect = NULL;
+	}
+}
+	
 static void Clip(GB_PAINT *d, int preserve)
 {
 	CHECK_PATH(d);
@@ -325,6 +421,7 @@ static void Clip(GB_PAINT *d, int preserve)
 	
 	delete EXTRA(d)->clip;
 	EXTRA(d)->clip = new QPainterPath(path);
+	delete_clip_rect(d);
 	
 	PRESERVE_PATH(d, preserve);
 }
@@ -333,6 +430,7 @@ static void ResetClip(GB_PAINT *d)
 {
 	delete CLIP(d);
 	EXTRA(d)->clip = 0;
+	delete_clip_rect(d);
 }
 
 static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransform &transform)
@@ -353,31 +451,18 @@ static void get_path_extents(QPainterPath *path, GB_EXTENTS *ext, const QTransfo
 
 static void ClipExtents(GB_PAINT *d, GB_EXTENTS *ext)
 {
-	get_path_extents(CLIP(d), ext, PAINTER(d)->transform());
-	/*if (EXTRA(d)->w > 0 && EXTRA(d)->h > 0)
+	GB_RECT *rect = EXTRA(d)->clipRect;
+	if (rect)
 	{
-		int x1 = EXTRA(d)->x;
-		int x2 = x1 + EXTRA(d)->w;
-		int y1 = EXTRA(d)->y;
-		int y2 = y1 + EXTRA(d)->h;
-			
-		if (ext->x2 > ext->x1 && ext->y2 > ext->y1)
-		{
-			if (x1 < ext->x1) ext->x1 = x1;
-			if (x2 > ext->x2) ext->x2 = x2;
-			if (y1 < ext->y1) ext->y1 = y1;
-			if (y2 > ext->y2) ext->y2 = y2;
-		}
-		else
-		{
-			ext->x1 = x1;
-			ext->x2 = x2;
-			ext->y1 = y1;
-			ext->y2 = y2;
-		}
-	}*/
+		ext->x1 = (float)rect->x;
+		ext->y1 = (float)rect->y;
+		ext->x2 = (float)(rect->x + rect->w);
+		ext->y2 = (float)(rect->y + rect->h);
+	}
+	else
+		get_path_extents(CLIP(d), ext, PAINTER(d)->transform());
 }
-	
+
 static void Fill(GB_PAINT *d, int preserve)
 {
 	CHECK_PATH(d);
@@ -500,6 +585,16 @@ static void FillRule(GB_PAINT *d, int set, int *value)
 		*value = EXTRA(d)->fillRule;
 }
 
+static void FillStyle(GB_PAINT *d, int set, int *style)
+{
+	/*if (set)
+	{
+		EXTRA(d)->fillRule = *value;
+	}
+	else
+		*value = EXTRA(d)->fillRule;*/
+}
+
 static void LineCap(GB_PAINT *d, int set, int *value)
 {
 	QPen pen = PAINTER(d)->pen();
@@ -643,7 +738,7 @@ static void ClosePath(GB_PAINT *d)
 }
 
 		
-static void Arc(GB_PAINT *d, float xc, float yc, float radius, float angle, float length)
+static void Arc(GB_PAINT *d, float xc, float yc, float radius, float angle, float length, bool pie)
 {
 	CREATE_PATH(d);
 
@@ -653,8 +748,35 @@ static void Arc(GB_PAINT *d, float xc, float yc, float radius, float angle, floa
 	angle = - angle;
 	length = - length;
 	
-	PATH(d)->arcMoveTo(rect, to_deg(angle));
+	if (pie)
+		PATH(d)->moveTo(xc, yc);
+	else
+		PATH(d)->arcMoveTo(rect, to_deg(angle));
+	
 	PATH(d)->arcTo(rect, to_deg(angle), to_deg(length));
+
+	if (pie)
+		PATH(d)->closeSubpath();
+}
+
+static void Ellipse(GB_PAINT *d, float x, float y, float width, float height, float angle, float length, bool pie)
+{
+	CREATE_PATH(d);
+
+	QRectF rect;
+	rect.setCoords((qreal)x, (qreal)y, (qreal)x + width, (qreal)y + height);
+	
+	angle = - angle;
+	length = - length;
+	
+	if (pie)
+		PATH(d)->moveTo(x + width / 2, y + height / 2);
+	else
+		PATH(d)->arcMoveTo(rect, to_deg(angle));
+	
+	PATH(d)->arcTo(rect, to_deg(angle), to_deg(length));
+	if (pie)
+		PATH(d)->closeSubpath();
 }
 
 static void Rectangle(GB_PAINT *d, float x, float y, float width, float height)
@@ -663,6 +785,21 @@ static void Rectangle(GB_PAINT *d, float x, float y, float width, float height)
 	PATH(d)->addRect((qreal)x, (qreal)y, (qreal)width, (qreal)height);
 }
 
+static void ClipRect(GB_PAINT *d, int x, int y, int w, int h)
+{
+	GB_RECT *rect;
+	ResetClip(d);
+	Rectangle(d, x, y, w, h);
+	Clip(d, FALSE);
+	
+	rect = new GB_RECT;
+	rect->x = x;
+	rect->y = y;
+	rect->w = w;
+	rect->h = h;
+	EXTRA(d)->clipRect = rect;
+}
+	
 static void GetCurrentPoint(GB_PAINT *d, float *x, float *y)
 {
 	if (!PATH(d))
@@ -695,6 +832,35 @@ static void CurveTo(GB_PAINT *d, float x1, float y1, float x2, float y2, float x
 	PATH(d)->cubicTo(QPointF((qreal)x1, (qreal)y1), QPointF((qreal)x2, (qreal)y2), QPointF((qreal)x3, (qreal)y3));
 }
 
+static QStringList text_sl;
+static QVector<int> text_w;
+static int text_line;
+
+static int get_text_width(QPainter *dp, QString &s)
+{
+	int w, width = 0;
+	int i;
+
+	text_sl = s.split('\n', QString::KeepEmptyParts);
+
+	text_w.resize(text_sl.count());
+
+	for (i = 0; i < (int)text_sl.count(); i++)
+	{
+		w = dp->fontMetrics().width(text_sl[i]);
+		if (w > width) width = w;
+		text_w[i] = w;
+	}
+
+	return width;
+}
+
+static int get_text_height(QPainter *dp, QString &s)
+{
+	text_line = dp->fontMetrics().height();
+	return text_line * (1 + s.count('\n'));
+}
+
 static QPainterPath *_draw_path;
 static float _draw_x, _draw_y;
 
@@ -704,24 +870,19 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	
 	GetCurrentPoint(d, &_draw_x, &_draw_y);
 	
+	if (w < 0 && h < 0)
+		_draw_y -= PAINTER(d)->fontMetrics().ascent();
+		
 	if (draw)
 	{
-		if (CLIP(d))
-		{
-			QTransform save = PAINTER(d)->worldTransform();
-			PAINTER(d)->resetTransform();
-			PAINTER(d)->setClipping(true);
-			PAINTER(d)->setClipPath(*CLIP(d));
-			PAINTER(d)->setWorldTransform(save);
-		}
+		begin_clipping(d);
 		
 		if (rich)
 			DRAW_rich_text(PAINTER(d), QString::fromUtf8(text, len), _draw_x, _draw_y, w, h, CCONST_alignment(align, ALIGN_TOP_NORMAL, true));	
 		else
 			DRAW_text(PAINTER(d), QString::fromUtf8(text, len), _draw_x, _draw_y, w, h, CCONST_alignment(align, ALIGN_TOP_NORMAL, true));	
 		
-		if (CLIP(d))
-			PAINTER(d)->setClipping(false);
+		end_clipping(d);
 	}
 	else
 	{
@@ -729,13 +890,12 @@ static void draw_text(GB_PAINT *d, bool rich, const char *text, int len, float w
 	
 		_draw_path = PATH(d);
 
-		if (w <= 0 || h <= 0)
-			_draw_y -= PAINTER(d)->fontMetrics().ascent();
-		
 		MyPaintDevice device;
 		QPainter p(&device);
 		
 		p.setFont(PAINTER(d)->font());
+		p.setPen(PAINTER(d)->pen());
+		p.setBrush(PAINTER(d)->brush());
 		
 		if (rich)
 			DRAW_rich_text(&p, QString::fromUtf8(text, len), 0, 0, w, h, CCONST_alignment(align, ALIGN_TOP_NORMAL, true));	
@@ -789,6 +949,28 @@ static void RichTextExtents(GB_PAINT *d, const char *text, int len, GB_EXTENTS *
 	get_text_extents(d, true, text, len, ext, width);
 }
 
+static void TextSize(GB_PAINT *d, const char *text, int len, float *w, float *h)
+{
+	QString s = QString::fromUtf8((const char *)text, len);  
+	*w = get_text_width(PAINTER(d), s);
+	*h = get_text_height(PAINTER(d), s);
+}
+
+static void RichTextSize(GB_PAINT *d, const char *text, int len, float sw, float *w, float *h)
+{
+	QTextDocument rt;
+	
+	rt.setDocumentMargin(0);
+	rt.setHtml(QString::fromUtf8((const char *)text, len));
+	rt.setDefaultFont(PAINTER(d)->font());
+	
+	if (sw > 0)
+		rt.setTextWidth(sw);
+	
+	*w = rt.idealWidth();
+	*h = rt.size().height();
+}
+
 static void Matrix(GB_PAINT *d, int set, GB_TRANSFORM matrix)
 {
 	QTransform *t = (QTransform *)matrix;
@@ -816,14 +998,117 @@ static void SetBrush(GB_PAINT *d, GB_BRUSH brush)
 	//PAINTER(d)->setBrushOrigin(QPointF((qreal)x, (qreal)y));
 }
 
-static void DrawImage(GB_PAINT *d, GB_IMAGE image, float x, float y, float w, float h)
+static void BrushOrigin(GB_PAINT *d, int set, float *x, float *y)
+{
+	if (set)
+	{
+		EXTRA(d)->bx = *x;
+		EXTRA(d)->by = *y;
+		PAINTER(d)->setBrushOrigin(*x, *y);
+	}
+	else
+	{
+		*x = EXTRA(d)->bx;
+		*y = EXTRA(d)->by;
+	}
+}
+
+static void Background(GB_PAINT *d, int set, GB_COLOR *color)
+{
+	if (set)
+	{
+		QBrush b(get_color(d, *color));
+		SetBrush(d, (GB_BRUSH)&b);
+	}
+	else
+	{
+		*color = COLOR_TO_INT(PAINTER(d)->brush().color());
+	}
+}
+
+
+static void Invert(GB_PAINT *d, int set, int *invert)
+{
+	if (set)
+	{
+		#if QT_VERSION >= QT_VERSION_CHECK(4, 5, 0)
+		PAINTER(d)->setCompositionMode(*invert ? QPainter::RasterOp_SourceXorDestination : QPainter::CompositionMode_SourceOver);
+		#else
+		fprintf(stderr, "gb.qt4: warning: Draw.Invert needs Qt 4.5\n");
+		#endif
+	}
+	else
+	{
+		#if QT_VERSION >= QT_VERSION_CHECK(4, 5, 0)
+		*invert = PAINTER(d)->compositionMode() == QPainter::RasterOp_SourceXorDestination; //QPainter::CompositionMode_Xor;
+		#else
+		*invert = FALSE;
+		#endif
+	}
+}
+
+static void DrawImage(GB_PAINT *d, GB_IMAGE image, float x, float y, float w, float h, float opacity, GB_RECT *source)
 {
 	QImage *img = CIMAGE_get((CIMAGE *)image);
 	QRectF rect(x, y, w, h);
 	
-	PAINTER(d)->drawImage(rect, *img);
+	begin_clipping(d);
+	
+	PAINTER(d)->setOpacity(opacity);
+	
+	if (source)
+	{
+		bool smooth = PAINTER(d)->testRenderHint(QPainter::SmoothPixmapTransform);
+		
+		if (w >= source->w && h >= source->h && w == (int)w && h == (int)h && ((int)w % source->w) == 0 && ((int)h % source->h) == 0)
+			PAINTER(d)->setRenderHint(QPainter::SmoothPixmapTransform, false);
+
+		QRectF srect(source->x, source->y, source->w, source->h);
+		PAINTER(d)->drawImage(rect, *img, srect);
+
+		PAINTER(d)->setRenderHint(QPainter::SmoothPixmapTransform, smooth);
+	}
+	else
+		PAINTER(d)->drawImage(rect, *img);
+	
+	PAINTER(d)->setOpacity(1.0);
+	
+	end_clipping(d);
 }
 		
+static void DrawPicture(GB_PAINT *d, GB_PICTURE picture, float x, float y, float w, float h, GB_RECT *source)
+{
+	QPixmap *pix = ((CPICTURE *)picture)->pixmap;
+	QRectF rect(x, y, w, h);
+	QRectF srect;
+	
+	if (source)
+		srect = QRectF(source->x, source->y, source->w, source->h);
+	else
+		srect = QRectF(0, 0, pix->width(), pix->height());
+
+	begin_clipping(d);
+	
+	PAINTER(d)->drawPixmap(rect, *pix, srect);
+
+	end_clipping(d);
+}
+
+static void GetPictureInfo(GB_PAINT *d, GB_PICTURE picture, GB_PICTURE_INFO *info)
+{
+	QPixmap *p = ((CPICTURE *)picture)->pixmap;
+	
+	info->width = p->width();
+	info->height = p->height();
+}
+
+static void FillRect(GB_PAINT *d, float x, float y, float w, float h, GB_COLOR color)
+{
+	begin_clipping(d);
+	PAINTER(d)->fillRect(x, y, w, h, get_color(d, color));
+	end_clipping(d);
+}
+
 static void BrushFree(GB_BRUSH brush)
 {
 	delete (QBrush *)brush;
@@ -837,9 +1122,10 @@ static void BrushColor(GB_BRUSH *brush, GB_COLOR color)
 
 static void BrushImage(GB_BRUSH *brush, GB_IMAGE image)
 {
-	QImage *img = CIMAGE_get((CIMAGE *)image);
+	QImage img(*CIMAGE_get((CIMAGE *)image));
 	
-	QBrush *br = new QBrush(*img);
+	img.detach();
+	QBrush *br = new QBrush(img);
 	*brush = (GB_BRUSH)br;
 }
 
@@ -917,6 +1203,11 @@ static void TransformCreate(GB_TRANSFORM *matrix)
 	*matrix = (GB_TRANSFORM)(new QTransform());
 }
 
+static void TransformCopy(GB_TRANSFORM *matrix, GB_TRANSFORM copy)
+{
+	*matrix = (GB_TRANSFORM)(new QTransform(*(QTransform *)copy));
+}
+
 static void TransformDelete(GB_TRANSFORM *matrix)
 {
 	delete (QTransform *)*matrix;
@@ -946,7 +1237,7 @@ static void TransformScale(GB_TRANSFORM matrix, float sx, float sy)
 static void TransformRotate(GB_TRANSFORM matrix, float angle)
 {
 	QTransform *t = (QTransform *)matrix;
-	t->rotate(to_deg(-angle));
+	t->rotateRadians(-angle);
 }
 
 static int TransformInvert(GB_TRANSFORM matrix)
@@ -971,8 +1262,22 @@ static void TransformMultiply(GB_TRANSFORM matrix, GB_TRANSFORM matrix2)
 	*t = *t * *t2;
 }
 
+static void TransformMap(GB_TRANSFORM matrix, double *x, double *y)
+{
+	qreal xx, yy;
 
-GB_PAINT_DESC PAINT_Interface = {
+	xx = *x;
+	yy = *y;
+
+	((QTransform *)matrix)->map(xx, yy, &xx, &yy);
+
+	*x = xx;
+	*y = yy;
+}
+
+
+GB_PAINT_DESC PAINT_Interface = 
+{
 	// Size of the GB_PAINT structure extra data
 	sizeof(QT_PAINT_EXTRA),
 	Begin,
@@ -981,9 +1286,12 @@ GB_PAINT_DESC PAINT_Interface = {
 	Restore,
 	Antialias,
 	Font,
+	Background,
+	Invert,
 	Clip,
 	ResetClip,
 	ClipExtents,
+	ClipRect,
 	Fill,
 	Stroke,
 	PathExtents,
@@ -991,6 +1299,7 @@ GB_PAINT_DESC PAINT_Interface = {
 	Dash,
 	DashOffset,
 	FillRule,
+	FillStyle,
 	LineCap,
 	LineJoin,
 	LineWidth,
@@ -999,6 +1308,7 @@ GB_PAINT_DESC PAINT_Interface = {
 	NewPath,
 	ClosePath,
 	Arc,
+	Ellipse,
 	Rectangle,
 	GetCurrentPoint,
 	MoveTo,
@@ -1006,11 +1316,17 @@ GB_PAINT_DESC PAINT_Interface = {
 	CurveTo,
 	Text,
 	TextExtents,
+	TextSize,
 	RichText,
 	RichTextExtents,
+	RichTextSize,
 	Matrix,
 	SetBrush,
+	BrushOrigin,
 	DrawImage,
+	DrawPicture,
+	GetPictureInfo,
+	FillRect,
 	{
 		BrushFree,
 		BrushColor,
@@ -1018,19 +1334,23 @@ GB_PAINT_DESC PAINT_Interface = {
 		BrushLinearGradient,
 		BrushRadialGradient,
 		BrushMatrix,
-	},
-	{
-		TransformCreate,
-		TransformDelete,
-		TransformInit,
-		TransformTranslate,
-		TransformScale,
-		TransformRotate,
-		TransformInvert,
-		TransformMultiply
 	}
 };
 
+GB_PAINT_MATRIX_DESC PAINT_MATRIX_Interface =
+{
+	TransformCreate,
+	TransformCopy,
+	TransformDelete,
+	TransformInit,
+	TransformTranslate,
+	TransformScale,
+	TransformRotate,
+	TransformInvert,
+	TransformMultiply,
+	TransformMap
+};
+
 void PAINT_begin(void *device)
 {
 	DRAW.Paint.Begin(device);
@@ -1059,15 +1379,7 @@ void PAINT_clip(int x, int y, int w, int h)
 {
 	GB_PAINT *d = (GB_PAINT *)DRAW.Paint.GetCurrent();
 	if (d)
-	{
-		/*PAINTER(d)->setClipRect(x, y, w, h);
-		EXTRA(d)->x = x;
-		EXTRA(d)->y = y;
-		EXTRA(d)->w = w;
-		EXTRA(d)->h = h;*/
-		Rectangle(d, x, y, w, h);
-		Clip(d, FALSE);
-	}
+		ClipRect(d, x, y, w, h);
 }
 
 /*************************************************************************/
@@ -1165,7 +1477,7 @@ void MyPaintEngine::drawTextItem(const QPointF &p, const QTextItem &textItem)
 {
 	//qDebug("MyPaintEngine::drawTextItem: %g %g [%s] '%s'", p.x() + _draw_x, p.y() + _draw_y, (const char *)textItem.font().toString().toUtf8(), (const char *)textItem.text().toUtf8());
 	//_draw_path->moveTo(p.x() + _draw_x, p.y() + _draw_y);
-	_draw_path->addText(p.x() + _draw_x, p.y() + _draw_y, textItem.font(), textItem.text());
+	_draw_path->addText(p.x() + _draw_x + painter()->worldTransform().dx(), p.y() + _draw_y + painter()->worldTransform().dy(), textItem.font(), textItem.text());
 }
 
 /*************************************************************************/
