@@ -2,7 +2,7 @@
 
   gdocument.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include "gview.h"
 #include "gdocument.h"
 
-//#define DEBUG_UNDO
+//#define DEBUG_UNDO 1
 
 #define GMAX(x, y) ((x) > (y) ? (x) : (y))
 #define GMIN(x, y) ((x) < (y) ? (x) : (y))
@@ -50,6 +50,7 @@ GLine::GLine()
 	alternate = false;
 	modified = false;
 	changed = false;
+	saved = false;
 	flag = 0;
 	tag = 0;
 	proc = false;
@@ -57,6 +58,7 @@ GLine::GLine()
 	highlight = NULL;
 	nobreak = false;
 	baptized = false;
+	tab = false;
 }
 
 GLine::~GLine()
@@ -86,6 +88,59 @@ bool GLine::isEmpty() const
 
 /**---- GCommand -----------------------------------------------------------*/
 
+struct GCommandDocument
+{
+public:
+	GEditor *view;
+	int cx;
+	int cy;
+	int sx;
+	int sy;
+	int sx2;
+	int sy2;
+	
+	GCommandDocument() { }
+	GCommandDocument(GDocument *doc);
+	void process(GDocument *doc) const;
+	void print() const;
+	bool equals(GCommandDocument *o) const;
+};
+
+GCommandDocument::GCommandDocument(GDocument *doc)
+{
+	view = doc->currentView();
+	
+	view->getCursor(&cy, &cx);
+	
+	if (doc->hasSelection())
+		doc->getSelection(&sy, &sx, &sy2, &sx2, false); // TODO: store insertMode
+	else
+		sx = sy = sx2 = sy2 = -1;
+}
+
+void GCommandDocument::process(GDocument *doc) const
+{
+	//qDebug("goto %d %d", cx, cy);
+	view->cursorGoto(cy, cx, false);
+	if (sx >= 0)
+	{
+		//qDebug("select %d %d %d %d", sx, sy, sx2, sy2);
+		doc->startSelection(view, sy, sx);
+		doc->endSelection(sy2, sx2);
+	}
+}
+
+void GCommandDocument::print() const
+{
+	qDebug("- %d %d [%d %d %d %d]", cx, cy, sx, sy, sx2, sy2);
+}
+
+bool GCommandDocument::equals(GCommandDocument *o) const
+{
+	return view == o->view && cx == o->cx && cy == o->cy 
+		&& sx == o->sx && sy == o->sy && sx2 == o->sx2 && sy2 == o->sy2;
+}
+
 class GCommand
 {
 public:
@@ -93,6 +148,7 @@ public:
 	{
 		None, Begin, End, Move, Insert, Delete, Indent, Unindent
 	};
+	GCommandDocument info;
 
 	virtual ~GCommand() { }
 	virtual Type type() const { return None; }
@@ -109,11 +165,12 @@ class GBeginCommand: public GCommand
 public:
 	bool _linked;
 	
-	GBeginCommand(bool linked = false) { _linked = linked; }
+	GBeginCommand(GCommandDocument *info, bool linked = false) { _linked = linked; this->info = *info; }
 	Type type() const { return Begin; }
-	void print() const { qDebug("Begin"); }
+	void print() const { qDebug("Begin"); info.print(); }
 	int nest() const { return 1; }
 	bool linked() const { return _linked; }
+	void process(GDocument *doc, bool undo) const { info.process(doc); }
 };
 
 class GEndCommand: public GCommand
@@ -135,13 +192,14 @@ public:
 	int x, y, x2, y2;
 	GString str;
 
-	GDeleteCommand(int y, int x, int y2, int x2, const GString & str)
+	GDeleteCommand(GCommandDocument *info, int y, int x, int y2, int x2, const GString &str)
 	{
 		this->x = x; this->y = y; this->x2 = x2; this->y2 = y2; this->str = str;
+		this->info = *info;
 	}
-
+	
 	Type type() const { return Delete; }
-	void print() const { qDebug("Delete: (%d %d)-(%d %d): '%s'", x, y, x2, y2, str.utf8()); }
+	void print() const { qDebug("Delete: (%d %d)-(%d %d): '%s'", x, y, x2, y2, str.utf8()); info.print(); }
 
 	bool merge(GCommand *c) const
 	{
@@ -149,13 +207,17 @@ public:
 			return false;
 
 		GDeleteCommand *o = (GDeleteCommand *)c;
+		
+		if (info.view != o->info.view)
+			return false;
 
-		if (x2 != o->x || y2 != o->y)
+		if (x2 != o->x || y2 != o->y || o->y != o->y2)
 			return false;
 
 		o->str.prepend(str);
 		o->x = x;
 		o->y = y;
+		//o->info = info;
 		return true;
 	}
 
@@ -165,15 +227,18 @@ public:
 			doc->insert(y, x, str);
 		else
 			doc->remove(y, x, y2, x2);
+		
+		info.process(doc);
 	}
 };
 
 class GInsertCommand: public GDeleteCommand
 {
 public:
-	GInsertCommand(int y, int x, int y2, int x2, const GString & str): GDeleteCommand(y, x, y2, x2, str) {}
+	GInsertCommand(GCommandDocument *info, int y, int x, int y2, int x2, const GString &str): 
+		GDeleteCommand(info, y, x, y2, x2, str) {}
 	Type type() const { return Insert; }
-	void print() const { qDebug("Insert: (%d %d)-(%d %d): '%s'", x, y, x2, y2, str.utf8()); }
+	void print() const { qDebug("Insert: (%d %d)-(%d %d): '%s'", x, y, x2, y2, str.utf8()); info.print(); }
 
 	bool merge(GCommand *c) const
 	{
@@ -185,6 +250,9 @@ public:
 
 		GInsertCommand *o = (GInsertCommand *)c;
 
+		if (o->info.view != info.view)
+			return false;
+		
 		if (o->str.length() && o->str.isNewLine(str.length() - 1))
 			return false;
 
@@ -214,6 +282,7 @@ void GDocument::init()
 	selector = NULL;
 	colorizeFrom = 0;
 	_disableColorize = 0;
+	_currentLine = -1;
 }
 
 GDocument::GDocument()
@@ -224,6 +293,7 @@ GDocument::GDocument()
 	readOnly = false;
 	highlightMode = None;
 	keywordsUseUpperCase = false;
+	_currentView = NULL;
 
 	//lines = new GArray<GLine>;
 	lines.setAutoDelete(true);
@@ -255,15 +325,25 @@ void GDocument::clear()
 }
 
 
-void GDocument::reset()
+void GDocument::reset(bool saved)
 {
-	for (uint i = 0; i < lines.count(); i++)
-		lines.at(i)->changed = false;
+	if (saved)
+	{
+		for (uint i = 0; i < lines.count(); i++)
+		{
+			lines.at(i)->saved |= lines.at(i)->changed;
+			lines.at(i)->changed = false;
+		}
+	}
+	else
+	{
+		for (uint i = 0; i < lines.count(); i++)
+			lines.at(i)->saved = lines.at(i)->changed = false;
+	}
 
 	updateViews();
 }
 
-
 GString GDocument::getText()
 {
 	GString tmp;
@@ -384,6 +464,7 @@ void GDocument::insert(int y, int x, const GString &text, bool doNotMove)
 	GString rest;
 	int yy;
 	int i, ns;
+	GCommandDocument info(this);
 
 	if (readOnly)
 	{
@@ -491,7 +572,7 @@ void GDocument::insert(int y, int x, const GString &text, bool doNotMove)
 	}
 	
 	begin();
-	addUndo(new GInsertCommand(ys, xs, y, x, text));
+	addUndo(new GInsertCommand(&info, ys, xs, y, x, text));
 	enableColorize();
 	end();
 
@@ -523,6 +604,7 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
 	GLine *l, *l2;
 	GString text, rest;
 	int y;
+	GCommandDocument info(this);
 
 	yAfter = y1;
 	xAfter = x1;
@@ -603,7 +685,7 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
 	}
 
 	begin();
-	addUndo(new GDeleteCommand(y1, x1, y2, x2, text));
+	addUndo(new GDeleteCommand(&info, y1, x1, y2, x2, text));
 	enableColorize();
 	end();
 
@@ -626,7 +708,7 @@ void GDocument::setText(const GString & text)
 	clearUndo();
 	undoLevel++; // Prevent textChanged emission
 	insert(0, 0, text, true);
-	reset();
+	reset(false);
 	undoLevel--;
 
 	blockUndo = false;
@@ -644,8 +726,14 @@ void GDocument::unsubscribe(GEditor *view)
 {
 	//qDebug("unsubscribe %p -> %p", view, this);
 	views.removeRef(view);
+	
 	if (views.count() == 0)
 		delete this;
+	else
+	{
+		if (_currentView == view)
+			_currentView = views.at(0);
+	}
 }
 
 void GDocument::subscribe(GEditor *view)
@@ -655,6 +743,9 @@ void GDocument::subscribe(GEditor *view)
 	views.append(view);
 	view->setNumRows(lines.count());
 	view->updateContents();
+	
+	if (!_currentView)
+		_currentView = view;
 }
 
 void GDocument::updateViews(int row, int count)
@@ -805,17 +896,17 @@ void GDocument::eraseSelection(bool insertMode)
 	if (!selector)
 		return;
 
+	begin();
 	getSelection(&y1, &x1, &y2, &x2, false);
 	selector = NULL;
 	if (!insertMode)
 		remove(y1, x1, y2, x2);
 	else
 	{
-		begin();
 		for (i = y1; i <= y2; i++)
 			remove(i, x1, i, x2);
-		end();
 	}
+	end();
 }
 
 void GDocument::clearUndo()
@@ -830,25 +921,50 @@ void GDocument::addUndo(GCommand *c)
 	if (blockUndo)
 		return;
 	
-	//fprintf(stderr, "addUndo: ");
-	//c->print();
+	#if DEBUG_UNDO
+		fprintf(stderr, "addUndo: ");
+		c->print();
+	#endif
 
 	if (!undoList.isEmpty())
 	{
 		if (c->merge(undoList.last()))
 		{
-			//qDebug("         MERGE");
+			#if DEBUG_UNDO
+			qDebug("         MERGE");
+			#endif
 			delete c;
 			return;
 		}
 		else if (c->remove(undoList.last()))
 		{
-			//qDebug("         REMOVE");
+			#if DEBUG_UNDO
+			qDebug("         REMOVE");
+			#endif
 			delete c;
 			delete undoList.take();
 			return;
 		}
 	}
+	
+	if (c->type() == GCommand::End && undoList.count() >= 2 && undoList.at(undoList.count() - 2)->type() == GCommand::Begin)
+	{
+		GCommand *o = undoList.take();
+		GCommand *begin = undoList.take();
+		#if DEBUG_UNDO
+		qDebug("         NO BEGIN / END");
+		#endif
+		o->info = begin->info;
+		delete begin;
+		delete c;
+		addUndo(o);
+		return;
+		/*else
+		{
+			undoList.append(begin);
+			undoList.append(o);
+		}*/
+	}
 
 	undoList.append(c);
 
@@ -864,13 +980,17 @@ void GDocument::begin(bool linked)
 	if (undoLevel == 0)
 		textHasChanged = false;
 	undoLevel++;  
-	if (!blockUndo) addUndo(new GBeginCommand(linked));
+	if (!blockUndo && (undoLevel == 1))
+	{
+		GCommandDocument info(this);
+		addUndo(new GBeginCommand(&info, linked));
+	}
 }
 
 void GDocument::end(bool linked)
 {
 	undoLevel--;
-	if (!blockUndo) addUndo(new GEndCommand(linked));
+	if (!blockUndo && undoLevel == 0) addUndo(new GEndCommand(linked));
 	if (undoLevel == 0 && textHasChanged)
 		emitTextChanged();
 }
@@ -1131,7 +1251,6 @@ int GDocument::convState(int state)
 		case EVAL_TYPE_OPERATOR: return GLine::Operator;
 		case EVAL_TYPE_DATATYPE: return GLine::Datatype;
 		case EVAL_TYPE_ERROR: return GLine::Error;
-		case EVAL_TYPE_ALTERNATE: return GLine::Alternate;
 		case EVAL_TYPE_HELP: return GLine::Help;
 		case EVAL_TYPE_PREPROCESSOR: return GLine::Preprocessor;
 		default: return GLine::Normal;
@@ -1246,7 +1365,7 @@ void GDocument::colorize(int y, bool force)
 	int updateFrom;
 	bool undo;
 
-	if (highlightMode == None)
+	if (highlightMode == None || highlightCallback == NULL)
 		return;
 
 	if (y < 0)
@@ -1306,6 +1425,8 @@ void GDocument::colorize(int y, bool force)
 
 		if (l->s.length())
 		{
+			GCommandDocument info(this);
+				
 			old = l->s;
 			GB.FreeArray(&l->highlight);
 			proc = l->proc;
@@ -1321,9 +1442,9 @@ void GDocument::colorize(int y, bool force)
 					undo = true;
 					begin();
 				}
-				addUndo(new GDeleteCommand(yy, 0, yy, old.length(), old));
+				addUndo(new GDeleteCommand(&info, yy, 0, yy, old.length(), old));
 				if (l->s.length())
-					addUndo(new GInsertCommand(yy, 0, yy, l->s.length(), l->s));
+					addUndo(new GInsertCommand(&info, yy, 0, yy, l->s.length(), l->s));
 				
 				//maxLength = GMAX(maxLength, (int)l->s.length());
 				updateLineWidth(yy);
@@ -1390,13 +1511,7 @@ void GDocument::colorize(int y, bool force)
 		updateViews(updateFrom, yy - updateFrom + 1);
 	
 	if (updateAll)
-	{
-		FOR_EACH_VIEW(v)
-		{
-			//if (v->getFlag(GEditor::ChangeBackgroundAtLimit))
-				v->updateContents();
-		}
-	}
+		updateContents();
 }
 
 void GDocument::colorizeAll()
@@ -1612,4 +1727,18 @@ void GDocument::enableColorize()
 		colorize(_disableColorizeStart);
 		_disableColorizeStart = -1;
 	}
-}
\ No newline at end of file
+}
+
+void GDocument::updateContents()
+{
+	FOR_EACH_VIEW(v)
+	{
+		v->updateContents();
+	}
+}
+
+void GDocument::setCurrentLine(int y)
+{
+	_currentLine = y;
+	updateContents();
+}
