@@ -2,7 +2,7 @@
 
   gstring.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,7 +30,8 @@ bool GString::hasUnicode() const
 {
 	for (uint i = 0; i < (uint)s.length(); i++)
 	{
-		if (s[i].unicode() & 0xFF00)
+		int c = s[i].unicode();
+		if (!isStandardChar(c))
 			return true;
 	}
 	
