@@ -2,7 +2,7 @@
 
   gview.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -88,7 +88,6 @@ static QColor defaultColors[GLine::NUM_STATE] =
 	QColor(0xFF, 0xFF, 0x00),
 	QColor(0xE8, 0xE8, 0xF8),
 	Qt::red,
-	Qt::black,
 	Qt::gray,
 	Qt::green
 };
@@ -97,7 +96,8 @@ static QColor defaultColors[GLine::NUM_STATE] =
 /**---- GEditor -----------------------------------------------------------*/
 
 QPixmap *GEditor::_cache = 0;
-QPixmap *GEditor::breakpoint = 0;
+QPixmap *GEditor::_breakpoint = 0;
+QPixmap *GEditor::_bookmark = 0;
 int GEditor::count = 0;
 QStyle *GEditor::_style = 0;
 
@@ -121,7 +121,6 @@ void GEditor::reset()
 	_posOutside = false;
 	_checkCache = true;
 	_ensureCursorVisibleLater = false;
-	_sameWidth = 0;
 	
 	foldClear();
 }
@@ -167,6 +166,8 @@ GEditor::GEditor(QWidget *parent)
 	_insertMode = false;
 	_cellw = _cellh = 0;
 	_oddLine = false;
+	_dblclick = false;
+	_firstLineNumber = 0;
 
 	for (i = 0; i < GLine::NUM_STATE; i++)
 	{
@@ -190,7 +191,6 @@ GEditor::GEditor(QWidget *parent)
 	setDocument(NULL);
 
 	setFont(QFont("monospace", QApplication::font().pointSize()));
-	updateFont();
 
 	blinkTimer = new QTimer(this);
 	connect(blinkTimer, SIGNAL(timeout()), this, SLOT(blinkTimerTimeout()));
@@ -209,19 +209,29 @@ GEditor::~GEditor()
 	if (count == 0)
 	{
 		delete _cache;
-		delete breakpoint;
+		delete _breakpoint;
+		delete _bookmark;
 		_cache = 0;
-		breakpoint = 0;
+		_breakpoint = 0;
+		_bookmark = 0;
 		delete _style;
 	}
 }
 
 void GEditor::setBreakpointPixmap(QPixmap *p)
 {
-	if (!breakpoint)
-		breakpoint = new QPixmap;
+	if (!_breakpoint)
+		_breakpoint = new QPixmap;
 		
-	*breakpoint = *p;
+	*_breakpoint = *p;
+}
+
+void GEditor::setBookmarkPixmap(QPixmap *p)
+{
+	if (!_bookmark)
+		_bookmark = new QPixmap;
+		
+	*_bookmark = *p;
 }
 
 void GEditor::setDocument(GDocument *d)
@@ -245,7 +255,8 @@ int GEditor::getStringWidth(const QString &s, int len, bool unicode) const
 {
 	int i;
 	ushort c;
-	int w = 0;
+	double w = 0;
+	int us = -1, ul = 0;
 	
 	//if (len < 0)
 	//	len = s.length();
@@ -254,17 +265,39 @@ int GEditor::getStringWidth(const QString &s, int len, bool unicode) const
 		return 0;
 	
 	if (_sameWidth && !unicode)
-		return len * _sameWidth;
+		return ROUND_WIDTH(len * _sameWidth);
 	
 	for (i = 0; i < len; i++)
 	{
 		c = s.at(i).unicode();
-		if (c & 0xFF00)
-			return fm.width(s, len);
+		if (c != 9 && !GString::isStandardChar(c))
+		{
+			if (us < 0)
+			{
+				us = i;
+				ul = 0;
+			}
+			ul++;
+		}
 		else
-			w += _charWidth[c];
+		{
+			if (us >= 0)
+			{
+				w += fm.width(s.mid(us), ul);
+				us = -1;
+			}
+			if (c == 9)
+				w = (ROUND_WIDTH(w) + _tabWidth) / _tabWidth * _tabWidth;
+			else
+				w += _charWidth[c];
+		}
 	}
 	
+	if (us >= 0)
+		w += fm.width(s.mid(us), ul);
+	
+	//qDebug("getStringWidth: %g / %d (%g %d)", w, fm.width(s, len), _sameWidth, unicode);
+	
 	return w;
 }
 
@@ -272,7 +305,7 @@ void GEditor::updateCache()
 {
 	//int nw = QMAX(cache->width(), QMIN(visibleWidth(), _cellw));
 	//int nh = QMAX(cache->height(), QMIN(visibleHeight(), _cellh));
-	int nw = QMAX(_cache->width(), visibleWidth() + _charWidth['m'] * 2);
+	int nw = QMAX(_cache->width(), ROUND_WIDTH(visibleWidth() + _charWidth['m'] * 2));
 	int nh = QMAX(_cache->height(), visibleHeight() + _cellh);
 	
 	if (nw > 0 && nh > 0 && (nw != _cache->width() || nh != _cache->height()))
@@ -282,9 +315,9 @@ void GEditor::updateCache()
 
 int GEditor::lineWidth(int y) const
 {
-	// Add 2 or a space width so that we see the cursor at end of line
+	// Add 2 or a character width so that we see the cursor at end of line
 	GLine *l = doc->lines.at(y);
-	return margin + getStringWidth(l->s.getString(), l->s.length(), l->unicode) + (_insertMode ? _charWidth[' '] : 2);
+	return margin + getStringWidth(l->s.getString(), l->s.length(), l->unicode) + (_insertMode ? _charWidth['W'] : 2);
 }
 
 
@@ -390,7 +423,7 @@ UPDATE_WIDTH:
 	
 void GEditor::updateHeight()
 {
-	_cellh = fm.ascent() + fm.descent() + 2;
+	_cellh = fm.ascent() + fm.descent() + 3;
 	//updateCache();
 	_checkCache = true;
 		
@@ -404,34 +437,91 @@ void GEditor::redrawContents()
 	//	repaintContents(contentsX(), contentsHeight(), visibleWidth(), visibleHeight() - contentsHeight() + contentsX(), true);
 }
 
+static double *get_char_width_table(QFontMetrics &fm, QFont &font)
+{
+	static QHash<QString, double *> cache;
+	
+	int i;
+	QString c, fd;
+	double *cw;
+	
+	fd = font.toString();
+	
+	if (cache.contains(fd))
+		return cache[fd];
+		
+	//qDebug("get_char_width_table: %s", (const char *)fd.toUtf8());
+
+	cw = new double[256];
+	
+	for (i = 0; i < 256; i++)
+	{
+		c = QChar(i);
+		c = c.repeated(64);
+		cw[i] = (double)fm.width(c) / 64;
+		//qDebug("_charWidth[%d] = %g", i, _charWidth[i]);
+	}
+	
+	cache.insert(fd, cw);
+	return cw;
+}
+
 void GEditor::updateFont()
 {
 	QFont f;
+	int i;
+	QString c;
 	
-	fm = fontMetrics();
-	for (int i = 0; i < 256; i++)
-		_charWidth[i] = fm.width(QChar(i));
+	normalFont = QFont(font());
+	normalFont.setKerning(false);
+	italicFont = QFont(font());
+	italicFont.setKerning(false);
+	italicFont.setItalic(true);
+	
+	fm = QFontMetrics(normalFont);
+	_ytext = fm.ascent() + 1;
+	
+	_charWidth = get_char_width_table(fm, normalFont);
 	
 	_sameWidth = _charWidth[' '];
+	_tabWidth = _charWidth['m'] * 8;
 	//_charWidth[9] = _sameWidth * 8;
 	
-	textOption.setTabStop(_sameWidth * 8);
-	textOption.setWrapMode(QTextOption::NoWrap);
-	
-	for (int i = 33; i < 256; i++)
+	for (i = 33; i < 127; i++)
 	{
 		if (_charWidth[i] != _sameWidth)
 		{
+			//qDebug("[%d] = %g!", i, _charWidth[i]);
 			_sameWidth = 0;
 			break;
 		}
 	}
 	
-	normalFont = font();
-	normalFont.setKerning(false);
-	italicFont = font();
-	italicFont.setKerning(false);
-	italicFont.setItalic(true);
+	if (_sameWidth)
+	{
+		for (i = 160; i < 256; i++)
+		{
+			if (i == 173)
+				continue;
+			if (_charWidth[i] != _sameWidth)
+			{
+				//qDebug("[%d] = %g!", i, _charWidth[i]);
+				_sameWidth = 0;
+				break;
+			}
+		}
+	}
+	
+	//qDebug("%p: sameWidth = %g tabWidth = %d", this, _sameWidth, _tabWidth);
+	
+	if (_sameWidth)
+	{
+		QString t = "AbCdEfGh01#@WwmM";
+		t = t.repeated(4);
+		
+		_sameWidth = (double)fm.width(t) / t.length();
+		//qDebug("_sameWidth -> %g", _sameWidth);
+	}
 	
 	updateMargin();
 	updateWidth();
@@ -449,10 +539,12 @@ void GEditor::changeEvent(QEvent *e)
 static int find_last_non_space(const QString &s)
 {
 	int i;
+	ushort c;
 	
 	for (i = s.length() - 1; i >= 0; i--)
 	{
-		if (s[i].unicode() > 32) // || s[i].unicode() == 9)
+		c = s[i].unicode();
+		if (c > 32 || c == 9)
 			return i;
 	}
 	return -1;
@@ -461,19 +553,22 @@ static int find_last_non_space(const QString &s)
 void GEditor::paintDottedSpaces(QPainter &p, int row, int ps, int ls)
 {
 	QPoint pa[ls];
-	int i, x, y, w;
+	int i, y;
+	double x, w;
 
-	x = lineWidth(row, ps) + 1;
-	y = fm.ascent();
 	w = _charWidth[' '];
+	x = lineWidth(row, ps) + w / 2;
+	y = _cellh / 2;
 	for (i = 0; i < ls; i++)
 	{
-		pa[i].setX(x);
+		pa[i].setX(ROUND_WIDTH(x));
 		pa[i].setY(y);
 		x += w;
 	}
 
+	p.setOpacity(0.5);
 	p.drawPoints(pa, i);
+	p.setOpacity(1);
 }
 
 static QColor merge_color(const QColor &ca, const QColor &cb)
@@ -503,8 +598,6 @@ static QColor merge_color(const QColor &ca, const QColor &cb)
 
 static QColor calc_color(QColor ca, QColor cb, QColor cd)
 {
-	int r = 1, g = 1, b = 1, n = 0;
-
 	if (!ca.isValid() && !cb.isValid())
 		return cd;
 	
@@ -514,12 +607,54 @@ static QColor calc_color(QColor ca, QColor cb, QColor cd)
 	if (!cb.isValid())
 		return ca;
 	
-	#define test(x) r *= x.red(); g *= x.green(); b *= x.blue(); n++;
+	return QColor((ca.red() + cb.red()) / 2, (ca.green() + cb.green()) / 2, (ca.blue() + cb.blue()) / 2);
+}
 
-	test(ca);
-	test(cb);
-	n = (n == 2) ? 255 : (n == 3) ? 255*255 : 1;
-	return QColor(r / n, g / n, b / n);
+void GEditor::drawTextWithTab(QPainter &p, int sx, int x, int y, const QString &s)
+{
+	int tp, tnp, yt = y;
+	
+	tp = 0;
+	
+	for(;;) //while (pos < s.length())
+	{
+		tnp = s.find('\t', tp);
+		if (tnp < 0)
+		{
+			p.drawText(x, y, s.mid(tp));
+			break;
+		}
+		
+		if (tnp > tp)
+		{
+			p.drawText(x, y, s.mid(tp, tnp - tp));
+			x += fm.width(s.mid(tp, tnp - tp));
+		}
+		p.setOpacity(0.5);
+		p.drawLine(x, yt - 2, x, yt);
+		p.drawLine(x + 1, yt, x + 2, yt);
+		p.setOpacity(1);
+		x = sx + ((x - sx) + _tabWidth) / _tabWidth * _tabWidth;
+		tp = tnp + 1;
+	}
+}
+
+static void highlight_text(QPainter &p, int x, int x2, int h, QColor color, QColor)
+{
+	//int i, j;
+	
+	//p.setPen(color);
+	
+	/*for (i = -1; i <= 1; i++)
+		for (j = -1; j <= 1; j++)
+			p.drawText(x + i, y + j, s);*/
+	//color.setAlpha(192);
+	//p.setCompositionMode(QPainter::CompositionMode_Overlay);
+	p.fillRect(x, 0, x2 - x, h, color);
+	//p.setCompositionMode(QPainter::CompositionMode_SourceOver);
+	//border.setAlpha(128);
+	//p.setPen(QColor(border));
+	//p.drawRect(x, 0, x2 - x, h);
 }
 
 void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax, int h, int xs1, int xs2, int row, QColor &fbg)
@@ -535,6 +670,7 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 	QColor bg;
 	bool draw_bg;
 	bool show_dots = getFlag(ShowDots);
+	int xs = x;
 
 	if (l->s.length() == 0)
 	{
@@ -552,7 +688,7 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 			if (xd2 > x2)
 				xd2 = x2;
 
-			p.fillRect(x, 0, nx - x, h, merge_color(styles[GLine::Alternate].color, fbg));
+			p.fillRect(x, 0, nx - x, h, merge_color(_altBackground, fbg));
 			
 			if (xd2 > xd1)
 				p.fillRect(xd1, 0, xd2 - xd1, h, styles[GLine::Selection].color);
@@ -589,8 +725,8 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 			else if (alternate)
 			{
 				draw_bg = true;
-				bg = styles[GLine::Alternate].color;
-				if (i == (GB.Count(l->highlight) - 1))
+				bg = _altBackground;
+				if (i == (GB.Count(l->highlight) - 1) && l->alternate)
 					nx = _cellw;
 			}
 			
@@ -617,6 +753,15 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 					p.fillRect(xd1, 0, xd2 - xd1, h, styles[GLine::Selection].color);
 			}
 			
+			// Highlight braces
+			if (getFlag(HighlightBraces))
+			{
+				if (row == y1m && x1m >= 0 && x1m >= pos && x1m < (pos + len))
+					highlight_text(p, lineWidth(y1m, x1m), lineWidth(y1m, x1m + 1), _cellh - 1, styles[GLine::Highlight].color, styles[GLine::Normal].color);
+				if (row == y2m && x2m >= 0 && x2m >= pos && x2m < (pos + len))
+					highlight_text(p, lineWidth(y2m, x2m), lineWidth(y2m, x2m + 1), _cellh - 1, styles[GLine::Highlight].color, styles[GLine::Normal].color);
+			}
+
 			if (ps >= 0 && pos >= ps)
 			{
 				if (show_dots)
@@ -633,17 +778,21 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 				if (st->bold)
 				{
 					QColor c = st->color;
-					c.setAlpha(160);
+					c.setAlpha(128);
 					p.setPen(c);
-					p.drawText(x + 1, y, sd);
+					drawTextWithTab(p, xs, x + 1, y, sd);
 				}
 				
 				p.setPen(st->color);
-				p.drawText(x, y, sd);
+				drawTextWithTab(p, xs, x, y, sd);
 			}
 			
 			if (st->underline)
+			{
+				p.setOpacity(0.5);
 				p.drawLine(x, y + 2, nx - 1, y + 2);
+				p.setOpacity(1);
+			}
 		}
 
 		pos += len;
@@ -653,7 +802,8 @@ void GEditor::paintText(QPainter &p, GLine *l, int x, int y, int xmin, int lmax,
 	if (pos < (int)l->s.length() && pos < (xmin + lmax))
 	{
 		p.setPen(styles[GLine::Normal].color);
-		p.drawText(x, y, l->s.mid(pos).getString());
+		//p.drawText(x, y, l->s.mid(pos).getString());
+		drawTextWithTab(p, xs, x, y, l->s.mid(pos).getString());
 		if (show_dots)
 			paintDottedSpaces(p, row, pos, QMIN(xmin + lmax - pos, (int)l->s.length() - pos));
 	}
@@ -699,7 +849,7 @@ void GEditor::paintShowString(QPainter &p, GLine *l, int x, int y, int xmin, int
 static void make_blend(QImage *pix, QColor start) //, bool loop = false)
 {
 	double r, g, b, a, da;
-	int n = pix->height() * 3 / 4;
+	int n = pix->height();
 	int i;
 	QPainter p;
 
@@ -708,19 +858,20 @@ static void make_blend(QImage *pix, QColor start) //, bool loop = false)
 	r = start.red();
 	g = start.green();
 	b = start.blue();
-	a = 255;
 
 	if (n == 0)
 		n = 1;
 
-	da = -(a / n);
+	a = 0;
+	da = (128 / ((n + 1) / 2) - 1);
 
 	p.begin(pix);
 
-	for (i = 0; i < n; i++)
+	for (i = 0; i < ((n + 1) / 2); i++)
 	{
-		QBrush brush(QColor((int)r, (int)g, (int)b, (int) a));
+		QBrush brush(QColor((int)r, (int)g, (int)b, qMin(255,(int)a)));
 		p.fillRect(0, i, pix->width(), 1, brush);
+		p.fillRect(0, n - i - 1, pix->width(), 1, brush);
 		a += da;
 	}
 
@@ -733,19 +884,6 @@ static void make_blend(QImage *pix, QColor start) //, bool loop = false)
 	return QColor(f(red), f(green), f(blue));
 }*/
 
-static void highlight_text(QPainter &p, int x, int y, int x2, int yy, QString s, QColor color)
-{
-	//int i, j;
-	
-	//p.setPen(color);
-	
-	/*for (i = -1; i <= 1; i++)
-		for (j = -1; j <= 1; j++)
-			p.drawText(x + i, y + j, s);*/
-		
-	p.fillRect(x, 0, x2 - x, yy, color);
-}
-
 void GEditor::paintCell(QPainter &p, int row, int)
 {
 	QRect ur;
@@ -831,7 +969,7 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	//	qDebug("%d: %d %d %d %d (%d %d)", row, ur.left(), ur.top(), ur.width(), ur.height(), xmin, lmax);
 
 	// Line background
-	if (l->flag & (1 << GLine::CurrentFlag))
+	if (realRow == doc->currentLine()) //l->flag & (1 << GLine::CurrentFlag))
 		a = styles[GLine::Current].color;
 
 	if (getFlag(ShowCurrentLine) && realRow == y)
@@ -849,7 +987,10 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	if (drawSep && getFlag(ChangeBackgroundAtLimit) && isEnabled())
 	{
 		if (ysep)
+		{
+			//int xsep = qMax(0, margin - qMin(_cellh, 12) - contentsX());
 			p.fillRect(0, ysep, visibleWidth(), _cellh - ysep, !odd ? _oddBackground : styles[GLine::Background].color);
+		}
 		
 		drawSep = false;
 	}
@@ -859,10 +1000,10 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	//p.translate(-ur.left(), 0);
 
 	p.translate(-contentsX(), 0);
-
+	
 	// Show string
 	if ((_showRow == realRow || _showString.length()))
-		paintShowString(p, l, margin, fm.ascent() + 1, xmin, lmax, _cellh, realRow);
+		paintShowString(p, l, margin, _ytext, xmin, lmax, _cellh, realRow);
 	
 	// Selection background
 	xs1 = 0;
@@ -897,65 +1038,25 @@ void GEditor::paintCell(QPainter &p, int row, int)
 		}
 	}
 
-	// Margin
-	if (margin && (margin > ur.left()))
-	{
-		//if (!l->flag)
-		//	p.fillRect(0, 0, margin, _cellh, color); //styles[GLine::Background].color);
-
-		if (getFlag(ShowModifiedLines))
-		{
-			if (l->changed)
-				p.fillRect(0, 0, margin - 2, _cellh, styles[GLine::Highlight].color);
-			//if (l->modified)
-			//	p.fillRect((margin - 2) / 2, _cellh / 2, 1, 1, styles[GLine::Breakpoint].color);
-		}
-		//else
-		//	p.fillRect(0, 0, margin - 2, _cellh, odd ? _oddBackground : styles[GLine::Background].color);
-		/*else if (getFlag(ShowCurrentLine))
-			p.fillRect(0, 0, margin - 2, _cellh, styles[GLine::Line].color);*/
-
-		//x1 = 0;
-
-		if (getFlag(ShowLineNumbers))
-		{
-			int n = realRow + 1;
-			if ((n % 10) == 0 || (l->proc && folded))
-				p.setPen(styles[GLine::Normal].color);
-			else
-				p.setPen(styles[GLine::Selection].color);
-			p.drawText(2, fm.ascent() + 1, QString::number(n).rightJustify(lineNumberLength));
-		}
-	}
-	
-	// Highlight braces
-	if (getFlag(HighlightBraces))
-	{
-		if (realRow == y1m && x1m >= 0)
-			highlight_text(p, lineWidth(y1m, x1m), fm.ascent(), lineWidth(y1m, x1m + 1), _cellh, l->s.getString().mid(x1m, 1), styles[GLine::Highlight].color);
-		if (realRow == y2m && x2m >= 0)
-			highlight_text(p, lineWidth(y2m, x2m), fm.ascent(), lineWidth(y2m, x2m + 1), _cellh, l->s.getString().mid(x2m, 1), styles[GLine::Highlight].color);
-		/*p.fillRect(x1m * charWidth + margin, 0, charWidth, _cellh, styles[GLine::Highlight].color);
-		p.fillRect(x2m * charWidth + margin, 0, charWidth, _cellh, styles[GLine::Highlight].color);*/
-	}
-
 	// Procedure separation
 	
 	if (drawSep)
 	{
+		int xsep = 0; //qMax(0, margin - qMin(_cellh, 12) - contentsX());
+		
 		p.translate(contentsX(), 0);
-
+		
 		if (getFlag(BlendedProcedureLimits))
 		{
 			if (!_blend_pattern)
 			{
-				_blend_pattern = new QImage(64, _cellh, QImage::Format_ARGB32_Premultiplied);
-				make_blend(_blend_pattern, styles[GLine::Line].color);
+				_blend_pattern = new QImage(64, _cellh / 2, QImage::Format_ARGB32_Premultiplied);
+				make_blend(_blend_pattern, styles[GLine::Selection].color);
 			}
 			
 			wsep = visibleWidth();
 			
-			for (i = 0; i < wsep; i += _blend_pattern->width())
+			for (i = xsep; i < wsep; i += _blend_pattern->width())
 				p.drawImage(i, ysep, *_blend_pattern, 0, 0, QMIN(wsep - i, _blend_pattern->width()), _cellh - ysep);
 			
 			if (xs1 && xs2)
@@ -976,21 +1077,69 @@ void GEditor::paintCell(QPainter &p, int row, int)
 			//QBrush brush(styles[GLine::Selection].color, Qt::Dense4Pattern);
 			//p.fillRect(0, 0, cache->width(), 1, brush);
 			p.setPen(styles[GLine::Selection].color);
-			p.drawLine(0, ysep, visibleWidth() - 1, ysep);
+			p.setOpacity(0.5);
+			p.drawLine(xsep, ysep, visibleWidth() - 1, ysep);
+			p.setOpacity(1);
 		}
 		
 		p.translate(-contentsX(), 0);
 	}
 
+	// Margin
+	if (margin && (margin > ur.left()))
+	{
+		//if (!l->flag)
+		//	p.fillRect(0, 0, margin, _cellh, color); //styles[GLine::Background].color);
+
+		if (getFlag(ShowModifiedLines))
+		{
+			if (l->changed || l->saved)
+			{
+				int w = qMax(2, margin - qMin(_cellh, 12) - 1);
+				p.setOpacity(0.5);
+				p.fillRect(0, 0, w, _cellh, styles[l->changed ? GLine::Breakpoint : GLine::Highlight].color);
+				p.setOpacity(1);
+			}
+			//if (l->modified)
+			//	p.fillRect((margin - 2) / 2, _cellh / 2, 1, 1, styles[GLine::Breakpoint].color);
+		}
+		//else
+		//	p.fillRect(0, 0, margin - 2, _cellh, odd ? _oddBackground : styles[GLine::Background].color);
+		/*else if (getFlag(ShowCurrentLine))
+			p.fillRect(0, 0, margin - 2, _cellh, styles[GLine::Line].color);*/
+
+		//x1 = 0;
+
+		if (getFlag(ShowLineNumbers) && (!drawSep || ysep == 0))
+		{
+			int n = realRow + 1 + _firstLineNumber;
+			if ((n % 10) == 0 || (l->proc && folded))
+				p.setPen(styles[GLine::Normal].color);
+			else
+				p.setPen(styles[GLine::Selection].color);
+			p.drawText(2, _ytext, QString::number(n).rightJustify(lineNumberLength));
+		}
+	}
+	
 	// Line text
 	if (!highlight)
 	{
+		// Highlight braces
+		if (getFlag(HighlightBraces))
+		{
+			if (realRow == y1m && x1m >= 0)
+				highlight_text(p, lineWidth(y1m, x1m), lineWidth(y1m, x1m + 1), _cellh - 1, styles[GLine::Highlight].color, styles[GLine::Normal].color);
+			if (realRow == y2m && x2m >= 0)
+				highlight_text(p, lineWidth(y2m, x2m), lineWidth(y2m, x2m + 1), _cellh - 1, styles[GLine::Highlight].color, styles[GLine::Normal].color);
+		}
+
 		if (l->s.length())
 		{
 			p.setPen(styles[GLine::Normal].color);
 			//p.drawText(margin + xmin * charWidth, fm.ascent() + 1, l->s.getString().mid(xmin, lmax));
-			//p.drawText(QRect(lineWidth(realRow, xmin), 1, visibleWidth(), _cellh), l->s.getString().mid(xmin, lmax), textOption);
-			p.drawText(lineWidth(realRow, xmin), 1 + fm.ascent(), l->s.getString().mid(xmin, lmax));
+			//p.drawText(QRect(lineWidth(realRow, xmin), 0, visibleWidth(), _cellh), l->s.getString().mid(xmin, lmax), textOption);
+			//p.drawText(lineWidth(realRow, xmin), fm.ascent(), l->s.getString().mid(xmin, lmax));
+			drawTextWithTab(p, lineWidth(realRow, 0), lineWidth(realRow, xmin), _ytext, l->s.getString().mid(xmin, lmax));
 			if (getFlag(ShowDots))
 			{
 				i = find_last_non_space(l->s.getString()) + 1;
@@ -1001,11 +1150,11 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	}
 	else
 	{
-		paintText(p, l, margin, fm.ascent() + 1, xmin, lmax, _cellh, xs1, xs2, realRow, color);
+		paintText(p, l, margin, _ytext, xmin, lmax, _cellh, xs1, xs2, realRow, color);
 	}
 	
 	// Folding symbol
-	if (margin && l->proc)
+	if (!getFlag(NoFolding) && margin && l->proc)
 	{
 		QPalette pal;
 		QStyleOption opt;
@@ -1022,23 +1171,33 @@ void GEditor::paintCell(QPainter &p, int row, int)
 	}
 	
 	// Breakpoint symbol
-	if ((l->flag & (1 << GLine::BreakpointFlag)) && breakpoint && !breakpoint->isNull())
+	if ((l->flag & (1 << GLine::BreakpointFlag)) && _breakpoint && !_breakpoint->isNull())
 	{
 		//p.fillRect(margin - 10, 0, 8, _cellh, styles[GLine::Breakpoint].color);
 		//updateBreakpoint(styles[GLine::Background].color.rgb(), styles[GLine::Breakpoint].color.rgb());
-		p.drawPixmap(margin - breakpoint->width() - 2, (_cellh - breakpoint->height()) / 2, *breakpoint);
+		p.drawPixmap(margin - _breakpoint->width() - 2, (_cellh - _breakpoint->height()) / 2, *_breakpoint);
+	}
+
+	// Bookmark symbol
+	if ((l->flag & (1 << GLine::BookmarkFlag)) && _bookmark && !_bookmark->isNull())
+	{
+		//p.fillRect(margin - 10, 0, 8, _cellh, styles[GLine::Breakpoint].color);
+		//updateBreakpoint(styles[GLine::Background].color.rgb(), styles[GLine::Breakpoint].color.rgb());
+		p.drawPixmap(margin - _bookmark->width() - 2, (_cellh - _bookmark->height()) / 2, *_bookmark);
 	}
 
 	// Text cursor
 	if (cursor && realRow == y)
 	{
+		QColor color = styles[GLine::Normal].color;
 		int xc = lineWidth(realRow, x);
 		int wc = 2;
 		//p.fillRect(QMIN((int)l->s.length(), x) * charWidth + margin, 0, 1, _cellh, styles[GLine::Normal].color);
 		if (_insertMode)
 			wc = lineWidth(realRow, x + 1) - xc;
 		
-		p.fillRect(xc, 0, wc, _cellh, styles[GLine::Normal].color);
+		color.setAlpha(160);
+		p.fillRect(xc, 0, wc, _cellh, color);
 	}
 
 	p.translate(contentsX(), 0);
@@ -1086,6 +1245,10 @@ void GEditor::drawContents(QPainter *p, int cx, int cy, int cw, int ch)
 	//p->setClipRect(cx, cy, cw, ch);
 	p->drawPixmap(contentsX(), rowfirst * _cellh, *_cache, 0, 0, _cellw, _cellh * (rowlast - rowfirst + 1)); //, _cellw, _cellh);
 	
+	//qDebug("%d %d", contentsX(), contentsY());
+	//p->setPen(styles[GLine::Normal].color);
+	//p->drawText(contentsX(), contentsY() + 16, QString::number(y) + " : " + QString::number(x));
+	
 	if (_blend_pattern)
 	{
 		delete _blend_pattern;
@@ -1391,7 +1554,7 @@ bool GEditor::cursorGoto(int ny, int nx, bool mark)
 		nx = 0;
 	else 
 	{
-		int xmax = _insertMode ? QMAX((_cellw / _charWidth[' ']) + 1, lineLength(largestLine)) : lineLength(ny);
+		int xmax = _insertMode ? QMAX((int)(_cellw / _charWidth[' ']) + 1, lineLength(largestLine)) : lineLength(ny);
 		
 		if (nx > xmax)
 			nx = xmax;
@@ -1442,9 +1605,11 @@ bool GEditor::cursorGoto(int ny, int nx, bool mark)
 
 void GEditor::insert(QString text)
 {
+	doc->begin();
 	doc->eraseSelection(_insertMode);
 	unfoldLine(y);
 	doc->insert(y, x, text);
+	doc->end();
 	cursorGoto(doc->yAfter, doc->xAfter, false);
 }
 
@@ -1516,7 +1681,11 @@ void GEditor::cursorUp(bool shift, bool ctrl, bool alt)
 		}
 	}
 	else if (ctrl)
-		cursorGoto(doc->getPreviousLimit(y), xx, shift);
+	{
+		int yl = doc->getPreviousLimit(y);
+		if (yl >= 0)
+			cursorGoto(yl, xx, shift);
+	}
 	else
 		cursorGoto(viewToReal(realToView(y) - 1), xx, shift);
 }
@@ -1568,7 +1737,14 @@ void GEditor::cursorDown(bool shift, bool ctrl, bool alt)
 	else if (!ctrl)
 		cursorGoto(QMIN(numLines() - 1, viewToReal(realToView(y) + 1)), xx, shift);
 	else
-		cursorGoto(doc->getNextLimit(y), xx, shift);
+	{
+		int yl = doc->getNextLimit(y);
+		
+		if (yl >= 0)
+			cursorGoto(yl, xx, shift);
+		else
+			cursorGoto(numLines(), 0, shift);
+	}
 }
 
 void GEditor::movePreviousSameIndent(bool shift)
@@ -1654,6 +1830,8 @@ void GEditor::del(bool ctrl)
 		return;
 	}
 	
+	doc->begin();
+	
 	if (x >= lineLength(y))
 	{
 		if (!_insertMode && y < (numLines() - 1))
@@ -1669,6 +1847,8 @@ void GEditor::del(bool ctrl)
 		else
 			doc->remove(y, x, y, x + 1);
 	}
+	
+	doc->end();
 }
 
 void GEditor::backspace(bool ctrl)
@@ -1683,6 +1863,8 @@ void GEditor::backspace(bool ctrl)
 		return;
 	}
 
+	doc->begin();
+	
 	indent = doc->getIndent(y);
 
 	if (x > 0 && x <= indent)
@@ -1712,6 +1894,8 @@ void GEditor::backspace(bool ctrl)
 				del(false);
 		}
 	}
+	
+	doc->end();
 }
 
 void GEditor::tab(bool back)
@@ -1728,8 +1912,11 @@ void GEditor::tab(bool back)
 		{
 			ins.fill(' ', tw - (x % tw));
 			insert(ins);
+			return;
 		}
-		return;
+
+		doc->startSelection(this, y, 0);
+		doc->endSelection(y + 1, 0);
 	}
 
 	doc->getSelection(&y1, &x1, &y2, &x2, _insertMode);
@@ -1847,14 +2034,14 @@ void GEditor::paste(bool mouse)
 
 void GEditor::undo()
 {
-	if (!doc->undo())
-		cursorGoto(doc->yAfter, doc->xAfter, false);
+	doc->undo();
+	//cursorGoto(doc->yAfter, doc->xAfter, false);
 }
 
 void GEditor::redo()
 {
-	if (!doc->redo())
-		cursorGoto(doc->yAfter, doc->xAfter, false);
+	doc->redo();
+	//cursorGoto(doc->yAfter, doc->xAfter, false);
 }
 
 void GEditor::selectAll()
@@ -2112,17 +2299,7 @@ void GEditor::mousePressEvent(QMouseEvent *e)
 	lastx = e->pos().x();
 	left = updateCursor();
 	
-	if (left)
-	{
-		if (doc->lines.at(ny)->proc)
-		{
-			if (isFolded(ny))
-				unfoldLine(ny);
-			else
-				foldLine(ny);
-		}
-	}
-	else
+	if (!left)
 		cursorGoto(ny, nx, shift);
 }
 
@@ -2152,18 +2329,19 @@ void GEditor::mouseDoubleClickEvent(QMouseEvent *e)
 {
 	int ny;
 	
+	_dblclick = true;
+	
 	if (left)
 	{
 		ny = posToLine(e->pos().y());
-		if (doc->lines.at(ny)->proc)
+		if (!getFlag(NoFolding) && doc->lines.at(ny)->proc)
 		{
 			if (isFolded(ny))
 				foldAll();
 			else
 				unfoldAll();
 		}
-		else
-			emit marginDoubleClicked(ny);
+		emit marginDoubleClicked(ny);
 		return;
 	}
 	
@@ -2191,14 +2369,27 @@ void GEditor::mouseReleaseEvent(QMouseEvent *e)
 		scrollTimer->stop();
 		startBlink();
 		copy(true);
-		return;
 	}
+	else
+	{
+		if (left && !_dblclick)
+		{
+			int ny = posToLine(e->pos().y());
+			if (!getFlag(NoFolding) && doc->lines.at(ny)->proc)
+			{
+				if (isFolded(ny))
+					unfoldLine(ny);
+				else
+					foldLine(ny);
+			}
+			emit marginClicked(ny);
+		}
 
-	if (left)
-		emit marginClicked(posToLine(e->pos().y()));
-
-	if (e->button() == Qt::MidButton)
-		paste(true);
+		if (e->button() == Qt::MidButton)
+			paste(true);
+	}
+	
+	_dblclick = false;
 }
 
 
@@ -2279,8 +2470,9 @@ void GEditor::blinkTimerTimeout()
 void GEditor::focusInEvent(QFocusEvent *e)
 {
 	startBlink();
-	ensureCursorVisible();
+	//ensureCursorVisible();
 	Q3ScrollView::focusInEvent(e);
+	doc->setCurrentView(this);
 }
 
 void GEditor::focusOutEvent(QFocusEvent *e)
@@ -2303,6 +2495,7 @@ void GEditor::scrollTimerTimeout()
 void GEditor::setStyle(int index, GHighlightStyle *style)
 {
 	int sat;
+	int gray;
 	
 	if (index < 0 || index >= GLine::NUM_STATE)
 		return;
@@ -2320,7 +2513,12 @@ void GEditor::setStyle(int index, GHighlightStyle *style)
 	if (index == GLine::Background)
 	{
 		_oddBackground = style->color;
+
 		sat = _oddBackground.saturation();
+		gray = 128 + (_oddBackground.value() - 128) * 0.8;
+
+		_altBackground = QColor(gray, gray, gray);
+		
 		if (_oddBackground.value() > 127)
 			_oddBackground.setHsv(_oddBackground.hue(), sat, _oddBackground.value() - 16);
 		else
@@ -2356,6 +2554,9 @@ void GEditor::setFlag(int f, bool v)
 	else
 		flags &= ~(1 << f);
 
+	if (getFlag(NoFolding))
+		unfoldAll();
+	
 	updateMargin();
 	updateContents();
 }
@@ -2363,25 +2564,30 @@ void GEditor::setFlag(int f, bool v)
 void GEditor::updateMargin()
 {
 	int charWidth = _charWidth['m'];
-	int nm = 0, lnl = 0;
+	int nm = 1, lnl = 0;
 	
 	if (!getFlag(HideMargin))
 	{
 		nm = 2;
 	
-		if (doc->getHighlightMode() != GDocument::None)
+		//if (doc->getHighlightMode() != GDocument::None)
 		{
-			if (breakpoint && !breakpoint->isNull())
-				nm += breakpoint->width() + 2;
-			else
-				nm += 8;
+			int bw = 8;
+			
+			if (_breakpoint && !_breakpoint->isNull())
+				bw = qMax(bw, _breakpoint->width() + 2);
+			
+			if (_bookmark && !_bookmark->isNull())
+				bw = qMax(bw, _bookmark->width() + 2);
+			
+			nm += bw;
 		}
-		else if (getFlag(ShowLineNumbers))
-			nm += 4;
+		//else if (getFlag(ShowLineNumbers))
+		//	nm += 4;
 		
 		if (getFlag(ShowLineNumbers))
 		{
-			int cnt = numLines();
+			int cnt = numLines() + _firstLineNumber;
 
 			while (cnt)
 			{
@@ -2522,15 +2728,17 @@ QVariant GEditor::inputMethodQuery(Qt::InputMethodQuery property) const
 			return QRect(px, py, 1, _cellh);
 		}
 		case Qt::ImFont:
-				return font();
+			return font();
 		case Qt::ImCursorPosition:
-				return QVariant(x);
+			return QVariant(x);
 		case Qt::ImSurroundingText:
-				return QVariant(doc->getLine(y).getString());
+			return QVariant(doc->getLine(y).getString());
 		case Qt::ImCurrentSelection:
-				return QVariant(doc->getSelectedText(_insertMode).getString());
+			return QVariant(QString());
+		case Qt::ImAnchorPosition:
+			return QVariant(x);
 		default:
-				return QVariant();
+			return QVariant();
 	}
 }
 
@@ -2634,6 +2842,9 @@ void GEditor::foldLine(int row, bool no_refresh)
 	GFoldedProc *fp;
 	int ny;
 	
+	if (getFlag(NoFolding))
+		return;
+	
 	if (!doc->hasLimit(row))
 		row = doc->getPreviousLimit(row);
 	
@@ -2838,6 +3049,9 @@ void GEditor::foldAll()
 {
 	int row;
 	
+	if (getFlag(NoFolding))
+		return;
+	
 	row = 0;
 	for(;;)
 	{
@@ -2866,6 +3080,9 @@ void GEditor::foldRemove(int y1, int y2)
 	GFoldedProc *fp;
 	int n;
 	
+	if (getFlag(NoFolding))
+		return;
+	
 	if (y2 < 0)
 	{
 		unfoldLine(y1);
@@ -2898,6 +3115,9 @@ void GEditor::foldInsert(int y, int n)
 	uint i;
 	GFoldedProc *fp;
 	
+	if (getFlag(NoFolding))
+		return;
+	
 	if (n == 0)
 	{
 		unfoldLine(y);
@@ -3064,3 +3284,9 @@ void GEditor::setBorder(bool b)
 	style()->polish(this);
 	updateViewportAttributes();
 }
+
+void GEditor::setLineOffset(int l)
+{
+	_firstLineNumber = l;
+	update();
+}
