@@ -2,7 +2,7 @@
 
   main.cpp
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -63,7 +63,6 @@
 #include "CButton.h"
 #include "CContainer.h"
 #include "CLabel.h"
-#include "CListBox.h"
 #include "CTextBox.h"
 #include "CTextArea.h"
 #include "CPictureBox.h"
@@ -76,8 +75,6 @@
 #include "CCheckBox.h"
 #include "CFrame.h"
 #include "CRadioButton.h"
-#include "CTreeView.h"
-#include "CListView.h"
 #include "CTabStrip.h"
 #include "CDialog.h"
 #include "CPicture.h"
@@ -93,7 +90,6 @@
 #include "CScrollBar.h"
 #include "CMovieBox.h"
 #include "CSpinBox.h"
-#include "CSplitter.h"
 #include "CWatcher.h"
 #include "cprinter.h"
 #include "csvgimage.h"
@@ -112,7 +108,7 @@
 
 extern "C" {
 
-GB_INTERFACE GB EXPORT;
+const GB_INTERFACE *GB_PTR EXPORT;
 IMAGE_INTERFACE IMAGE EXPORT;
 
 }
@@ -125,6 +121,8 @@ int MAIN_scale = 6;
 int MAIN_x11_last_key_code = 0;
 #endif
 
+bool MAIN_debug_busy = false;
+
 GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Container;
 GB_CLASS CLASS_UserControl;
@@ -146,8 +144,6 @@ static int _no_destroy = 0;
 static QTranslator *_translator = NULL;
 static bool _application_keypress = false;
 static GB_FUNCTION _application_keypress_func;
-static QWidget *_mouseGrabber = 0;
-static QWidget *_keyboardGrabber = 0;
 static bool _check_quit_posted = false;
 
 #ifndef NO_X_WINDOW
@@ -156,6 +152,10 @@ static void (*_x11_event_filter)(XEvent *) = 0;
 
 static QHash<void *, void *> _link_map;
 
+static QPointer<QWidget> MAIN_mouseGrabber = 0;
+static QPointer<QWidget> MAIN_keyboardGrabber = 0;
+
+
 //static MyApplication *myApp;
 
 /***************************************************************************
@@ -330,6 +330,16 @@ MyApplication::MyApplication(int &argc, char **argv)
 {
 }
 
+void MyApplication::initClipboard()
+{
+  QObject::connect(clipboard(), SIGNAL(dataChanged()), qApp, SLOT(clipboardHasChanged()));
+}
+
+void MyApplication::clipboardHasChanged()
+{
+	CLIPBOARD_has_changed();
+}
+
 static bool QT_EventFilter(QEvent *e)
 {
 	bool cancel;
@@ -515,18 +525,18 @@ void MyTimer::timerEvent(QTimerEvent *e)
 
 static void release_grab()
 {
-	_mouseGrabber = QWidget::mouseGrabber();
-	_keyboardGrabber = QWidget::keyboardGrabber();
+	MAIN_mouseGrabber = QWidget::mouseGrabber();
+	MAIN_keyboardGrabber = QWidget::keyboardGrabber();
 	
-	if (_mouseGrabber) 
+	if (MAIN_mouseGrabber) 
 	{
 		//qDebug("releaseMouse");
-		_mouseGrabber->releaseMouse();
+		MAIN_mouseGrabber->releaseMouse();
 	}
-	if (_keyboardGrabber)
+	if (MAIN_keyboardGrabber)
 	{
 		//qDebug("releaseKeyboard");
-		_keyboardGrabber->releaseKeyboard();
+		MAIN_keyboardGrabber->releaseKeyboard();
 	}
 	
 	#ifndef NO_X_WINDOW
@@ -541,18 +551,18 @@ static void release_grab()
 
 static void unrelease_grab()
 {
-	if (_mouseGrabber)
+	if (MAIN_mouseGrabber)
 	{
 		//qDebug("grabMouse");
-		_mouseGrabber->grabMouse();
-		_mouseGrabber = 0;
+		MAIN_mouseGrabber->grabMouse();
+		MAIN_mouseGrabber = 0;
 	}
 	
-	if (_keyboardGrabber)
+	if (MAIN_keyboardGrabber)
 	{
 		//qDebug("grabKeyboard");
-		_keyboardGrabber->grabKeyboard();
-		_keyboardGrabber = 0;
+		MAIN_keyboardGrabber->grabKeyboard();
+		MAIN_keyboardGrabber = 0;
 	}
 }
 
@@ -561,7 +571,7 @@ static bool must_quit(void)
 	#if DEBUG_WINDOW
 	qDebug("must_quit: Window = %d Watch = %d in_event_loop = %d", CWindow::count, CWatch::count, in_event_loop);
 	#endif
-	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0;
+	return CWINDOW_must_quit() && CWatch::count == 0 && in_event_loop && MAIN_in_message_box == 0 && TRAYICON_count == 0;
 }
 
 static void check_quit_now(intptr_t param)
@@ -589,7 +599,7 @@ void MAIN_check_quit(void)
 	if (_check_quit_posted)
 		return;
 	
-	GB.Post((GB_POST_FUNC)check_quit_now, 0);
+	GB.Post((GB_CALLBACK)check_quit_now, 0);
 	_check_quit_posted = true;
 }
 
@@ -633,6 +643,8 @@ static void QT_Init(void)
 
 	MAIN_update_scale();
 
+	qApp->installEventFilter(&CWidget::manager);
+
 	MyApplication::setEventFilter(true);
 	
 	if (GB.GetFunction(&_application_keypress_func, (void *)GB.Application.StartupClass(), "Application_KeyPress", "", "") == 0)
@@ -641,8 +653,6 @@ static void QT_Init(void)
 		MyApplication::setEventFilter(true);
 	}
 
-	qApp->installEventFilter(&CWidget::manager);
-
 	//qt_x11_set_global_double_buffer(false);
 
 	//Q3StyleSheet::defaultSheet()->item("tt")->setFontFamily("Monospace");
@@ -650,6 +660,8 @@ static void QT_Init(void)
 
 	qApp->setQuitOnLastWindowClosed(false);
 	
+	MyApplication::initClipboard();
+	
 	init = true;
 }
 
@@ -691,8 +703,8 @@ static void init_lang(char *lang, bool rtl)
 	delete _translator;
 	_translator = NULL;
 	
-	if (strcmp(lang, "C"))
-		qDebug("gb.qt4: warning: unable to load Qt translation: %s", lang);
+	//if (strcmp(lang, "C"))
+	//	qDebug("gb.qt4: warning: unable to load Qt translation: %s", lang);
 	
 	goto __SET_DIRECTION;
 
@@ -729,15 +741,27 @@ static int X11_error_handler(Display *d, XErrorEvent *e)
 }
 #endif
 
-static void hook_main(int *argc, char **argv)
+static void *_old_hook_main;
+
+static void hook_main(int *argc, char ***argv)
 {
-	//QApplication::setGraphicsSystem("opengl");
-	new MyApplication(*argc, argv);
+	new MyApplication(*argc, *argv);
 	
 	QT_Init();
 	init_lang(GB.System.Language(), GB.System.IsRightToLeft());
 	
 	//_old_handler = XSetErrorHandler(X11_error_handler);
+	
+	CALL_HOOK_MAIN(_old_hook_main, argc, argv);
+}
+
+
+static void hook_quit()
+{
+	CWINDOW_close_all();
+	CWINDOW_delete_all();
+
+	qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
 }
 
 
@@ -754,10 +778,7 @@ static void hook_loop()
 	else
 		MAIN_check_quit();
 	
-	CWINDOW_close_all();
-	CWINDOW_delete_all();
-
-	qApp->processEvents(QEventLoop::ExcludeUserInputEvents | QEventLoop::DeferredDeletion, 0);
+	hook_quit();
 }
 
 
@@ -809,24 +830,6 @@ static void hook_post(void)
 }
 
 
-static void hook_quit()
-{
-	QWidgetList list;
-	int i;
-
-	//qApp->closeAllWindows();
-
-	list = QApplication::topLevelWidgets();
-
-	for (i = 0; i < list.count(); i++)
-		list.at(i)->close();
-
-	for (i = 0; i < list.count(); i++)
-		list.at(i)->deleteLater();
-}
-
-
-
 static void hook_error(int code, char *error, char *where)
 {
 	QString msg;
@@ -841,12 +844,12 @@ static void hook_error(int code, char *error, char *where)
 	if (code > 0)
 	{
 		msg = msg + "[%1] %2.<br>%3";
-		msg = msg.arg(code).arg(error).arg(where);
+		msg = msg.arg(code).arg(TO_QSTRING(error)).arg(where);
 	}
 	else
 	{
 		msg = msg + "%1.<br>%2";
-		msg = msg.arg(error).arg(where);
+		msg = msg.arg(TO_QSTRING(error)).arg(where);
 	}
 
 	release_grab();
@@ -965,21 +968,11 @@ const char *QT_ToUTF8(const QString &str)
 	return res;
 }
 
-static void *QT_CreateFont(const QFont &f, FONT_FUNC func, void *object)
-{
-	return CFONT_create(f, func, object);
-}
-
 static void *QT_CreatePicture(const QPixmap &p)
 {
 	return CPICTURE_create(&p);
 }
 
-static void *QT_GetDrawInterface()
-{
-	return (void *)&DRAW_Interface;
-}
-
 void MyApplication::linkDestroyed(QObject *qobject)
 {
 	void *object = _link_map.value(qobject, 0);
@@ -1005,11 +998,11 @@ extern "C" {
 GB_DESC *GB_CLASSES[] EXPORT =
 {
 	CBorderDesc, CColorDesc,
-	CAlignDesc, CArrangeDesc, CScrollDesc, CKeyDesc, CLineDesc, CFillDesc, CSelectDesc,
+	CAlignDesc, CArrangeDesc, CScrollDesc, CKeyDesc, CSelectDesc,
 	CMessageDesc,
 	CImageDesc, CPictureDesc,
 	CFontDesc, CFontsDesc,
-	CMouseDesc, CCursorDesc,
+	CMouseDesc, CCursorDesc, CPointerDesc,
 	CClipboardDesc, CDragDesc,
 	StyleDesc, ScreenDesc, ScreensDesc, DesktopDesc,
 	ApplicationDesc,
@@ -1021,16 +1014,8 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CCheckBoxDesc, CRadioButtonDesc,
 	CTextBoxSelectionDesc, CTextBoxDesc, CComboBoxItemDesc, CComboBoxDesc,
 	CTextAreaSelectionDesc, CTextAreaDesc,
-	CListBoxItemDesc, CListBoxDesc,
-	CListViewItemDesc, CListViewDesc,
-	CTreeViewItemDesc, CTreeViewDesc,
-	CColumnViewItemDesc, CColumnViewColumnDesc, CColumnViewColumnsDesc, CColumnViewDesc,
-	//CListView2ItemDesc, CListView2Desc,
-	//CTreeView2ItemDesc, CTreeView2Desc,
-	//CColumnView2ItemDesc, CColumnView2ColumnDesc, CColumnView2ColumnsDesc, CColumnView2Desc,
 	CFrameDesc, CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
-	CHSplitDesc, CVSplitDesc,
-	CTabChildrenDesc, CTabDesc, CTabStripDesc,
+	CTabStripContainerChildrenDesc, CTabStripContainerDesc, CTabStripDesc,
 	CScrollViewDesc,
 	CDrawingAreaDesc,
 	CProgressDesc, CSliderDesc, CSpinBoxDesc, CMovieBoxDesc, CScrollBarDesc,
@@ -1058,14 +1043,14 @@ void *GB_QT4_1[] EXPORT = {
 	(void *)CWIDGET_border_full,
 	(void *)CWIDGET_scrollbar,
 	(void *)Control_Font,
-	(void *)QT_CreateFont,
+	(void *)CFONT_create,
+	(void *)CFONT_set,
 	(void *)QT_CreatePicture,
 	//(void *)QT_MimeSourceFactory,
 	(void *)QT_GetPixmap,
 	(void *)QT_ToUTF8,
 	(void *)QT_EventFilter,
 	(void *)QT_Notify,
-	(void *)QT_GetDrawInterface,
 	(void *)CCONST_alignment,
 	(void *)QT_Link,
 	(void *)QT_GetLink,
@@ -1090,19 +1075,25 @@ static void myMessageHandler(QtMsgType type, const char *msg )
 #endif
 #endif
 
-const char *GB_INCLUDE EXPORT = "gb.draw";
+const char *GB_INCLUDE EXPORT = "gb.draw,gb.gui.base";
 
 int EXPORT GB_INIT(void)
 {
 	// Do not disable GLib support
 	
-	char *env = getenv("KDE_FULL_SESSION");
+	char *env;
+	
+	env = getenv("KDE_FULL_SESSION");
 	if (env && !strcasecmp(env, "true"))
 		putenv((char *)"QT_NO_GLIB=1");
 	
+	env = getenv("GB_GUI_BUSY");
+	if (env && atoi(env))
+		MAIN_debug_busy = true;
+	
 	//putenv((char *)"QT_SLOW_TOPLEVEL_RESIZE=1");
 
-	GB.Hook(GB_HOOK_MAIN, (void *)hook_main);
+	_old_hook_main = GB.Hook(GB_HOOK_MAIN, (void *)hook_main);
 	GB.Hook(GB_HOOK_LOOP, (void *)hook_loop);
 	GB.Hook(GB_HOOK_WAIT, (void *)hook_wait);
 	GB.Hook(GB_HOOK_TIMER, (void *)hook_timer);
@@ -1112,7 +1103,9 @@ int EXPORT GB_INIT(void)
 	GB.Hook(GB_HOOK_ERROR, (void *)hook_error);
 	GB.Hook(GB_HOOK_LANG, (void *)hook_lang);
 
-	GB.LoadComponent("gb.draw");
+	GB.Component.Load("gb.draw");
+	GB.Component.Load("gb.image");
+	GB.Component.Load("gb.gui.base");
 	GB.GetInterface("gb.image", IMAGE_INTERFACE_VERSION, &IMAGE);
   IMAGE.SetDefaultFormat(GB_IMAGE_BGRA);
 	DRAW_init();
@@ -1169,6 +1162,16 @@ int EXPORT GB_INFO(const char *key, void **value)
 		*value = (void *)x11_set_event_filter;
 		return TRUE;
 	}
+	else if (!strcasecmp(key, "GET_HANDLE"))
+	{
+		*value = (void *)CWIDGET_get_handle;
+		return TRUE;
+	}
+	else if (!strcasecmp(key, "TIME"))
+	{
+		*value = (void *)QX11Info::appTime();
+		return TRUE;
+	}
 	else
 		return FALSE;
 }
@@ -1178,7 +1181,7 @@ int EXPORT GB_INFO(const char *key, void **value)
 extern Time	qt_x_time;
 #endif*/
 
-static void activate_main_window(int value)
+static void activate_main_window(intptr_t value)
 {
 	CWINDOW *active;
 	
@@ -1219,7 +1222,7 @@ void EXPORT GB_SIGNAL(int signal, void *param)
 			break;
 			
 		case GB_SIGNAL_DEBUG_CONTINUE:
-			GB.Post((GB_POST_FUNC)activate_main_window, 0);
+			GB.Post((GB_CALLBACK)activate_main_window, 0);
 			unrelease_grab();
 			break;
 	}
