@@ -3,6 +3,7 @@
   CXSLT.h
 
   (c) 2004 Daniel Campos Fernández <danielcampos@netcourrier.com>
+  (c) 2012 Adrien Prokopowicz <prokopy@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,25 +25,10 @@
 #ifndef __CXSLT_H
 #define __CXSLT_H
 
-#include "gambas.h"
+#include "../gbinterface.h"
 
 #ifndef __CXSLT_C
-
 extern GB_DESC CXsltDesc[];
-
-#else
-
-#define THIS ((CXSLT *)_object)
-
 #endif
 
-
-typedef struct 
-{
-    GB_BASE ob;
- 
-    
-}  CXSLT;
-
-
 #endif
