@@ -2,7 +2,7 @@
 
   gb_file.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,4 +21,8 @@
 
 ***************************************************************************/
 
+#include "gb_common.h"
+#include "gb_error.h"
+#include "gb_alloc.h"
+
 #include "gb_file_temp.h"
