@@ -2,7 +2,7 @@
 
   gbc_class.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ typedef
 	struct {
 		TYPE type;
 		int value;
+		int line;
 		}
 	PACKED
 	CLASS_SYMBOL_INFO;
@@ -46,8 +47,9 @@ typedef
 		CLASS_SYMBOL_INFO local;
 		int class;
 		int unknown;
-		unsigned used : 1;
-		unsigned _reserved : 31;
+		unsigned global_used : 1;
+		unsigned local_used : 1;
+		unsigned _reserved : 30;
 		}
 	PACKED
 	CLASS_SYMBOL;
@@ -88,16 +90,22 @@ typedef
 
 		char nparam;                   // Maximum number of arguments
 		char npmin;                    // Minimum number of arguments
-		char vararg;                   // If this function accepts extra arguments
-		char _reserved;
+		unsigned vararg : 1;           // If this function accepts extra arguments
+		unsigned fast : 1;             // If this function is jit compiled
+		unsigned _reserved : 14;
 		short nlocal;                  // Local variable count
 		short nctrl;                   // Control structure variable count
+		
 		uint64_t byref;                // Byref mask
 		PARAM *local;                  // Datatypes of local variables
 		PARAM *param;                  // Datatypes of arguments
 
 		PATTERN *start;                // Starts compilation from there
+		
 		int line;	                     // ...which is this line
+		short stack;                   // Needed stack
+		short _reserved2;
+		
 		ushort *code;                  // Compile bytecode
 		
 		short *pos_line;               // Bytecode position of each code line
@@ -105,12 +113,10 @@ typedef
 		ushort ncode;                  // Number of instructions
 		ushort ncode_max;              // Size of the bytecode allocation
 
-		short last_code;               // Last compiled bytecode position
-		short last_code2;              // Last last compiled bytecode position
-		short stack;                   // Needed stack
-		short finally;                 // FINALLY position
-		short catch;                   // CATCH position
-		short _reserved2;
+		ushort last_code;              // Last compiled bytecode position
+		ushort last_code2;             // Last last compiled bytecode position
+		ushort finally;                // FINALLY position
+		ushort catch;                  // CATCH position
 		}
 	PACKED
 	FUNCTION;
@@ -132,7 +138,8 @@ typedef
 		int name;                      // Function name index in class symbol table
 		PARAM *param;                  // Argument list
 		short nparam;                  // Number of arguments
-		short _reserved;
+		unsigned vararg : 1;           // Variable number of arguments
+		unsigned _reserved : 15;
 		int library;                   // Library name index
 		int alias;                     // Real function name index
 		}
@@ -188,7 +195,8 @@ typedef
 		unsigned autocreate : 1;       // class is auto-creatable
 		unsigned optional : 1;         // class is optional
 		unsigned nocreate : 1;         // class cannot be instantiated
-		unsigned _reserved : 12;
+		unsigned all_fast : 1;         // all methods have the Fast option (JIT)
+		unsigned _reserved : 11;
 		VARIABLE *stat;                // static variables
 		VARIABLE *dyn;                 // dynamic variables
 		CONSTANT *constant;            // constants
@@ -212,10 +220,12 @@ typedef
 #define FUNCTION_is_procedure(func)  (TYPE_get_id((func)->type) == T_VOID)
 #define FUNCTION_is_static(func)     (TYPE_is_static((func)->type))
 
+	
 void CLASS_create(CLASS **result);
 void CLASS_delete(CLASS **class);
 
 CLASS_SYMBOL *CLASS_declare(CLASS *class, int index, bool global);
+void CLASS_check_unused_global(CLASS *class);
 
 void CLASS_add_function(CLASS *class, TRANS_FUNC *decl);
 void CLASS_add_event(CLASS *class, TRANS_EVENT *decl);
@@ -233,7 +243,8 @@ int CLASS_add_array(CLASS *class, TRANS_ARRAY *array);
 
 int CLASS_get_array_class(CLASS *class, int type, int value);
 
-void FUNCTION_add_pos_line(void);
+void FUNCTION_add_last_pos_line(void);
+void FUNCTION_add_all_pos_line(void);
 char *FUNCTION_get_fake_name(int func);
 
 int CLASS_add_symbol(CLASS *class, const char *name);
@@ -241,7 +252,7 @@ int CLASS_add_symbol(CLASS *class, const char *name);
 void CLASS_sort_declaration(CLASS *class);
 void CLASS_check_properties(CLASS *class);
 
-/* gbc_dump.c */
+// gbc_dump.c
 
 void CLASS_dump(void);
 void CLASS_export(void);
