@@ -2,7 +2,7 @@
 
   gbc_header.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -123,6 +123,9 @@ static void analyze_function_desc(TRANS_FUNC *func, int flag)
 
 	for(;;)
 	{
+		if (func->nparam >= MAX_PARAM_FUNC)
+			THROW("Too many arguments");
+
 		param = &func->param[func->nparam];
 		CLEAR(param);
 
@@ -139,7 +142,7 @@ static void analyze_function_desc(TRANS_FUNC *func, int flag)
 			look++;
 		}
 
-		if (PATTERN_is(*look, RS_3PTS))
+		if (PATTERN_is(*look, RS_3PTS) && !(flag & HF_NO_3PTS))
 		{
 			look++;
 			if (!PATTERN_is(*look, RS_RBRA))
@@ -166,11 +169,25 @@ static void analyze_function_desc(TRANS_FUNC *func, int flag)
 			look++;
 		}
 
+		if (PATTERN_is(*look, RS_LBRA))
+		{
+			param->ignore = TRUE;
+			look++;
+		}
+		
 		if (!PATTERN_is_identifier(*look))
 			THROW("Syntax error. The &1 argument is not a valid identifier", TRANS_get_num_desc(func->nparam + 1));
-
+		
 		param->index = PATTERN_index(*look);
 		look++;
+		
+		if (param->ignore)
+		{
+			if (!PATTERN_is(*look, RS_RBRA))
+				THROW(E_MISSING, "')'");
+			look++;
+		}
+
 		JOB->current = look;
 
 		if (!TRANS_type(TT_NOTHING, &ttyp))
@@ -376,7 +393,7 @@ static bool header_extern(TRANS_EXTERN *trans)
 	CLEAR(trans);
 
 	JOB->current = look;
-	analyze_function_desc((TRANS_FUNC *)trans, HF_NO_3PTS);
+	analyze_function_desc((TRANS_FUNC *)trans, HF_NO_BYREF);
 
 	if (PATTERN_is(*JOB->current, RS_AS))
 	{
@@ -596,8 +613,17 @@ static bool header_function(TRANS_FUNC *func)
 	bool is_proc = FALSE;
 	bool is_static = FALSE;
 	bool is_public = FALSE;
+	bool is_fast = FALSE;
 
-	/* static ? */
+	// fast ?
+	
+	if (PATTERN_is(*look, RS_FAST))
+	{
+		is_fast = TRUE;
+		look++;
+	}
+	
+	// static ?
 
 	if (JOB->is_module)
 	{
@@ -609,7 +635,7 @@ static bool header_function(TRANS_FUNC *func)
 		look++;
 	}
 
-	/* public ou static ? */
+	// public ou static ?
 
 	is_public = JOB->is_module && JOB->public_module;
 
@@ -635,7 +661,6 @@ static bool header_function(TRANS_FUNC *func)
 	JOB->current = look;
 	analyze_function_desc(func, HF_NORMAL);
 
-	//if (!is_proc)
 	if (PATTERN_is(*JOB->current, RS_AS))
 	{
 		if (!TRANS_type(TT_NOTHING, &ttyp))
@@ -647,6 +672,8 @@ static bool header_function(TRANS_FUNC *func)
 	TYPE_set_kind(&func->type, TK_FUNCTION);
 	if (is_static) TYPE_set_flag(&func->type, TF_STATIC);
 	if (is_public) TYPE_set_flag(&func->type, TF_PUBLIC);
+	
+	func->fast = is_fast;
 
 	// Check special methods
 
@@ -721,7 +748,7 @@ static bool header_function(TRANS_FUNC *func)
 	}
 
 
-	/* on saute le corps de la fonction */
+	// We ignore function body
 
 	if (!PATTERN_is_newline(*(JOB->current)))
 		THROW("Syntax error at function declaration");
@@ -792,8 +819,11 @@ static bool header_inherits(void)
 
 	if (JOB->class->parent != NO_SYMBOL)
 		THROW("Cannot inherit twice");
-
+	
 	index = PATTERN_index(*JOB->current);
+	
+	if (strcasecmp(TABLE_get_symbol_name(JOB->class->table, index), JOB->class->name) == 0)
+		THROW("Cannot inherit itself");
 
 	JOB->class->parent = CLASS_add_class(JOB->class, index);
 	/*printf("JOB->class->parent = %d\n", JOB->class->parent);*/
@@ -837,6 +867,14 @@ static bool header_option(void)
 			return TRUE;
 		}
 	}
+	
+	if (PATTERN_is(JOB->current[0], RS_FAST))
+	{
+		JOB->current++;
+		JOB->class->all_fast = TRUE;
+		
+		return TRUE;
+	}
 
 	return FALSE;
 }
