@@ -2,7 +2,7 @@
 
   gbc_read.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include "gbc_compile.h"
 #include "gbc_class.h"
 #include "gbc_preprocess.h"
+#include "gbc_help.h"
 #include "gbc_read.h"
 
 //#define DEBUG
@@ -84,6 +85,7 @@ static void READ_init(void)
 	unsigned char i;
 	
 	JOB->line = 1;
+	JOB->max_line = FORM_FIRST_LINE - 1;
 	JOB->column = TRUE;
 	
 	if (!is_init)
@@ -141,7 +143,7 @@ static void READ_exit(void)
 		{
 			local = TRUE;
 		}
-		else
+		else if (*name != '.')
 		{
 			if (TABLE_find_symbol(JOB->class->table, name, len, &index))
 			{
@@ -438,6 +440,10 @@ static void add_newline()
 	if (action == PREP_LINE)
 		comp->line = PREP_next_line;
 	
+	// Void lines must act as void help comments
+	if (comp->line > 0 && PATTERN_is_newline(get_last_pattern()))
+		HELP_add_at_current_line("\n");
+	
 	add_pattern(RT_NEWLINE, comp->line);
 	comp->line++;
 	
@@ -569,6 +575,10 @@ READ_NUMBER:
 		while (isdigit(car))
 			car = next_char();
 	}
+	else if (toupper(car) == 'I')
+	{
+		car = next_char();
+	}
 
 	goto END;
 
@@ -737,7 +747,7 @@ static void add_identifier()
 			}
 			else
 			{
-				if (last_type)
+				if (last_type || PATTERN_is(last_pattern, RS_OPEN))
 					can_be_reserved = TRUE;
 				else
 					can_be_reserved = FALSE;
@@ -1087,13 +1097,18 @@ void READ_do(void)
 		continue;
 
 	__COMMENT:
-			
-		do
+
+		source_ptr++;
+		
+		if (HELP_is_help_comment(source_ptr))
+			HELP_add_at_current_line(source_ptr);
+		
+		car = get_char();
+		while (car != '\n')
 		{
 			source_ptr++;
 			car = get_char();
 		}
-		while (car != '\n');
 
 		_begin_line = FALSE;
 		continue;
