@@ -2,7 +2,7 @@
 
   gbc_trans_ctrl.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ static void control_set_value(int value)
 }
 
 
-static int control_get_value()
+static int control_get_value(void)
 {
 	if (ctrl_level <= 0)
 		return 0;
@@ -78,16 +78,16 @@ static int control_get_value()
 }
 
 
-static void control_add_pos(short **tab_pos, short pos)
+static void control_add_pos(ushort **tab_pos, ushort pos)
 {
 	if (!(*tab_pos))
 		ARRAY_create(tab_pos);
 
-	*((short *)ARRAY_add(tab_pos)) = pos;
+	*((ushort *)ARRAY_add(tab_pos)) = pos;
 }
 
 
-static void control_add_current_pos()
+static void control_add_current_pos(void)
 {
 	control_add_pos(&current_ctrl->pos, CODE_get_current_pos());
 }
@@ -98,7 +98,7 @@ static void control_add_this_pos(ushort pos)
 }
 
 
-static void control_jump_each_pos_with(short *tab_pos)
+static void control_jump_each_pos_with(ushort *tab_pos)
 {
 	int i;
 
@@ -110,7 +110,7 @@ static void control_jump_each_pos_with(short *tab_pos)
 }
 
 
-static void control_jump_each_pos()
+static void control_jump_each_pos(void)
 {
 	control_jump_each_pos_with(current_ctrl->pos);
 }
@@ -158,7 +158,7 @@ static TRANS_CTRL *control_get_inner_with(void)
 }
 
 
-static void add_goto(int index, bool gosub)
+static void add_goto(int index, int mode)
 {
 	TRANS_GOTO *info;
 
@@ -170,16 +170,19 @@ static void add_goto(int index, bool gosub)
 	info->pos = CODE_get_current_pos();
 	info->ctrl_id = (ctrl_level == 0) ? 0 : current_ctrl->id;
 	info->line = JOB->line;
-	info->gosub = gosub;
+	info->gosub = mode == RS_GOSUB;
+	info->on_goto = mode == RS_NONE;
 
 	#ifdef DEBUG_GOTO
 		printf("add_goto: ctrl_id = %d (%ld)\n", info->ctrl_id, ARRAY_count(goto_info));
 	#endif
 
-	if (gosub)
-		CODE_gosub();
-	else
+	if (mode == RS_GOSUB)
+		CODE_gosub(ctrl_local);
+	else if (mode == RS_GOTO)
 		CODE_jump();
+	else
+		CODE_nop();
 }
 
 
@@ -228,7 +231,7 @@ static void control_enter(int type)
 }
 
 
-static void control_leave()
+static void control_leave(void)
 {
 	control_jump_each_pos_with(current_ctrl->pos_break);
 
@@ -290,7 +293,7 @@ static void check_try(const char *name)
 	}
 }
 
-void TRANS_control_init()
+void TRANS_control_init(void)
 {
 	ctrl_level = 0;
 	ctrl_id = 0;
@@ -307,7 +310,7 @@ void TRANS_control_init()
 }
 
 
-void TRANS_control_exit()
+void TRANS_control_exit(void)
 {
 	int i;
 	CLASS_SYMBOL *sym;
@@ -356,7 +359,7 @@ void TRANS_control_exit()
 					id = ctrl_parent[id - 1];
 				}
 			}
-
+			
 			CODE_jump_length(goto_info[i].pos, label->pos);
 		}
 
@@ -540,8 +543,11 @@ static void trans_else_if(void)
 }
 
 
-void TRANS_if()
+void TRANS_if(void)
 {
+	PATTERN *look;
+	bool has_else;
+	
 	control_enter(RS_IF);
 
 	trans_if();
@@ -549,12 +555,35 @@ void TRANS_if()
 	if (PATTERN_is_newline(*JOB->current))
 		return;
 
+	has_else = FALSE;
+	look = JOB->current;
+	for(;;)
+	{
+		look++;
+		if (PATTERN_is_newline(*look))
+			break;
+		if (PATTERN_is(*look, RS_ELSE))
+		{
+			has_else = TRUE;
+			*look = PATTERN_make(RT_NEWLINE, 0);
+			break;
+		}
+	}
+	
 	TRANS_statement();
+	
+	if (has_else)
+	{
+		JOB->current++;
+		trans_else();
+		TRANS_statement();
+	}
+	
 	TRANS_endif();
 }
 
 
-void TRANS_else()
+void TRANS_else(void)
 {
 	control_check(RS_IF, "ELSE without IF", "ELSE");
 
@@ -568,7 +597,7 @@ void TRANS_else()
 }
 
 
-void TRANS_endif()
+void TRANS_endif(void)
 {
 	control_check(RS_IF, "ENDIF without IF", "ENDIF");
 	trans_endif();
@@ -576,7 +605,7 @@ void TRANS_endif()
 }
 
 
-void TRANS_goto()
+void TRANS_goto(void)
 {
 	int index;
 
@@ -588,10 +617,10 @@ void TRANS_goto()
 	index = PATTERN_index(*JOB->current);
 	JOB->current++;
 
-	add_goto(index, FALSE);
+	add_goto(index, RS_GOTO);
 }
 
-void TRANS_gosub()
+void TRANS_gosub(void)
 {
 	int index;
 
@@ -603,9 +632,55 @@ void TRANS_gosub()
 	index = PATTERN_index(*JOB->current);
 	JOB->current++;
 
-	add_goto(index, TRUE);
+	add_goto(index, RS_GOSUB);
 }
 
+void TRANS_on_goto_gosub(void)
+{
+	bool gosub;
+	int pos, n;
+	int index;
+	
+	TRANS_expression(FALSE);
+	
+	if (TRANS_is(RS_GOTO))
+		gosub = FALSE;
+	else if (TRANS_is(RS_GOSUB))
+		gosub = TRUE;
+	else
+		THROW(E_SYNTAX);
+
+	check_try(gosub ? "ON GOSUB" : "ON GOTO");
+	
+	pos = CODE_get_current_pos();
+	CODE_nop();
+	
+	for (n = 1;; n++)
+	{
+		if (!PATTERN_is_identifier(*JOB->current))
+			THROW(E_SYNTAX);
+		
+		index = PATTERN_index(*JOB->current);
+		JOB->current++;
+
+		add_goto(index, RS_NONE);
+		
+		if (!TRANS_is(RS_COMMA))
+			break;
+		
+		if (n == 127)
+			THROW("Too many labels");
+	}
+	
+	pos = CODE_set_current_pos(pos);
+	CODE_on(n);
+	CODE_set_current_pos(pos);
+	
+	if (gosub)
+		CODE_gosub(ctrl_local);
+	else
+		CODE_jump();
+}
 
 void TRANS_do(int type)
 {
@@ -722,6 +797,7 @@ void TRANS_case(void)
 	int i;
 	short pos;
 	short local;
+	bool like;
 
 	control_check(RS_SELECT, "CASE without SELECT", "CASE");
 	
@@ -733,7 +809,9 @@ void TRANS_case(void)
 	local = current_ctrl->local;
 
 	control_enter(RS_CASE);
-
+	
+	like = TRANS_is(RS_LIKE);
+	
 	for(i = 0; ; i++)
 	{
 		if (i > MAX_CASE_EXPR)
@@ -746,18 +824,18 @@ void TRANS_case(void)
 		TRANS_expression(FALSE);
 		CODE_op(C_EQ, 2);*/
 		
-		if (TRANS_is(RS_TO))
-		{
-			CODE_push_local(local);
-			TRANS_expression(FALSE);
-			CODE_op(C_LE, 0, 2, TRUE);
-		}
-		else if (TRANS_is(RS_LIKE))
+		if (like)
 		{
 			CODE_push_local(local);
 			TRANS_expression(FALSE);
 			CODE_op(C_LIKE, 0, 2, TRUE);
 		}
+		else if (TRANS_is(RS_TO))
+		{
+			CODE_push_local(local);
+			TRANS_expression(FALSE);
+			CODE_op(C_LE, 0, 2, TRUE);
+		}
 		else
 		{
 			TRANS_expression(FALSE);
@@ -882,7 +960,8 @@ void TRANS_for(void)
 
 	control_enter(RS_FOR);
 
-	TRANS_affectation(FALSE);
+	if (!TRANS_affectation(FALSE))
+		THROW(E_SYNTAX);
 
 	if (!CODE_check_pop_local_last(&local))
 		THROW("Loop variable must be local");
@@ -985,6 +1064,8 @@ void TRANS_for_each(void)
 		if (!CODE_popify_last())
 			THROW("Invalid assignment");
 
+		TRANS_want(RS_IN, "IN");
+
 		JOB->current = save;
 	}
 	else
@@ -1149,22 +1230,28 @@ void TRANS_end_with(void)
 void TRANS_raise(void)
 {
 	CLASS_SYMBOL *sym;
+	int index;
 	int np;
 
 	if (!PATTERN_is_identifier(*JOB->current))
 		THROW("Syntax error in event name");
 
-	sym = CLASS_get_symbol(JOB->class, PATTERN_index(*JOB->current));
+	index = PATTERN_index(*JOB->current);
+	sym = CLASS_get_symbol(JOB->class, index);
 
 	JOB->current++;
 
-	if (TYPE_get_kind(sym->global.type) != TK_EVENT)
-		THROW("Unknown event");
+	if (TYPE_get_kind(sym->global.type) == TK_EVENT)
+	{
+		if (TYPE_is_static(JOB->func->type))
+			THROW("Cannot raise events in static function");
 
-	if (TYPE_is_static(JOB->func->type))
-		THROW("Cannot raise events in static function");
-
-	CODE_push_event(sym->global.value);
+		CODE_push_event(sym->global.value);
+	}
+	else
+	{
+		CODE_push_unknown_event(CLASS_add_unknown(JOB->class, index));
+	}
 
 	np = 0;
 
@@ -1190,9 +1277,4 @@ void TRANS_raise(void)
 
 	if (TRANS_in_affectation == 0)
 		CODE_drop();
-
-	/*if (TRANS_is(RS_AS))
-		TRANS_reference();
-	else
-		CODE_drop();*/
 }
