@@ -2,7 +2,7 @@
 
 	gb_error.c
 
-	(c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "gbx_exec.h"
 #include "gbx_api.h"
 #include "gbx_stack.h"
+#include "gbx_project.h"
 #include "gb_error.h"
 
 //#define DEBUG_ERROR 1
@@ -46,7 +47,7 @@ int ERROR_depth = 0;
 
 static int _lock = 0;
 
-static const char *_message[71] =
+static const char *const _message[72] =
 {
 	/*  0 E_UNKNOWN */ "Unknown error",
 	/*  1 E_MEMORY */ "Out of memory",
@@ -59,7 +60,7 @@ static const char *_message[71] =
 	/*  8 E_ILLEGAL */ "Illegal instruction",
 	/*  9 E_NFUNC */ "Not a function",
 	/* 10 E_CSTATIC */ ".1Class '&1' is not creatable",
-	/* 11 E_NSYMBOL */ ".2Unknown symbol '&1' in class '&2'",
+	/* 11 E_NSYMBOL */ ".2Unknown symbol '&2' in class '&1'",
 	/* 12 E_NOBJECT */ "Not an object",
 	/* 13 E_NULL */ "Null object",
 	/* 14 E_STATIC */ ".2'&1.&2' is static",
@@ -118,7 +119,8 @@ static const char *_message[71] =
 	/* 67 E_SERIAL */ "Serialization error",
 	/* 68 E_CHILD */ ".2Cannot run child process: &1&2",
 	/* 69 E_USER */ "Unknown user or group",
-	/* 70 E_NEMPTY */ "Directory is not empty"
+	/* 70 E_NEMPTY */ "Directory is not empty",
+	/* 71 E_UTYPE */ "Unsupported datatype"
 };
 
 #if DEBUG_ERROR
@@ -144,6 +146,8 @@ void ERROR_debug(const char *msg, ...)
 		err = err->prev;
 	}
 	fprintf(stderr, "NULL\n");
+	
+	va_end(args);
 }
 #endif
 
@@ -260,7 +264,7 @@ void ERROR_leave(ERROR_CONTEXT *err)
 
 void ERROR_propagate()
 {
-	ERROR_HANDLER *ph;
+	ERROR_HANDLER *ph, *prev;
 	#if DEBUG_ERROR
 	ERROR_debug("ERROR_propagate: %p %d %s (ret = %d)\n", ERROR_current, ERROR_current->info.code, ERROR_current->info.msg, ERROR_current->ret);
 	#endif
@@ -277,8 +281,9 @@ void ERROR_propagate()
 			break;
 		
 		//fprintf(stderr, "ERROR_propagate: %p @ %p (%p)\n", ERROR_handler, ERROR_handler->context, ERROR_current);
-		ERROR_handler = ph->prev;
-		(*ph->handler)();
+		prev = ph->prev;
+		(*ph->handler)(ph->arg1, ph->arg2);
+		ERROR_handler = prev;
 	}
 	
 	longjmp(ERROR_current->env, 1);
@@ -462,9 +467,16 @@ void THROW(int code, ...)
 
 	ERROR_define((char *)(intptr_t)code, arg);
 
+	va_end(args);
+	
 	PROPAGATE();
 }
 
+void THROW_CLASS(void *class, char *arg1, char *arg2)
+{
+	THROW(E_CLASS, CLASS_get_name((CLASS *)class), arg1, arg2);
+}
+
 void THROW_ILLEGAL()
 {
 	THROW(E_ILLEGAL);
@@ -472,6 +484,9 @@ void THROW_ILLEGAL()
 
 void THROW_STACK()
 {
+	#if DEBUG_STACK
+	fprintf(stderr, "THROW STACK!\n");
+	#endif
 	THROW(E_STACK);
 }
 
@@ -511,6 +526,19 @@ void THROW_SYSTEM(int err, const char *path)
 	}
 }
 
+void ERROR_fatal(const char *error, ...)
+{
+	va_list args;
+
+	va_start(args, error);
+	fputs(EXEC_arch ? "gbr" GAMBAS_VERSION_STRING : "gbx" GAMBAS_VERSION_STRING, stderr);
+	fputs(": ", stderr);
+	vfprintf(stderr, error, args);
+	va_end(args);
+	putc('\n', stderr);
+	_exit(1);
+}
+
 void ERROR_panic(const char *error, ...)
 {
 	va_list args;
@@ -519,15 +547,17 @@ void ERROR_panic(const char *error, ...)
 
 	fflush(NULL);
 
-	fprintf(stderr, "\n** INTERNAL ERROR **\n** ");
+	fprintf(stderr, "\n** Oops! Internal error! **\n** ");
 	vfprintf(stderr, error, args);
+	
+	va_end(args);
+	
 	putc('\n', stderr);
 	if (ERROR_current->info.code)
 	{
 		ERROR_print();
 	}
-	fprintf(stderr, "** Program aborting. Sorry! :-(\n");
-	/*abort();*/
+	fprintf(stderr, "** Program aborting. Sorry! :-(\n** Please send a bug report at gambas@users.sourceforge.net\n");
 	_exit(1);
 }
 
@@ -573,11 +603,6 @@ void ERROR_print(void)
 {
 	static bool lock = FALSE;
 	
-	ERROR_print_at(stderr, FALSE, TRUE);
-	
-	if (ERROR_backtrace)
-		DEBUG_print_backtrace(ERROR_backtrace);
-
 	if (EXEC_main_hook_done && !EXEC_debug && EXEC_Hook.error && !lock)
 	{
 		lock = TRUE;
@@ -585,22 +610,34 @@ void ERROR_print(void)
 		HOOK(error)(ERROR_current->info.code, ERROR_current->info.msg, DEBUG_get_position(ERROR_current->info.cp, ERROR_current->info.fp, ERROR_current->info.pc));
 		lock = FALSE;
 	}
+
+	ERROR_print_at(stderr, FALSE, TRUE);
+	
+	if (ERROR_backtrace)
+		DEBUG_print_backtrace(ERROR_backtrace);
 }
 
-void ERROR_save(ERROR_INFO *save, ERROR_INFO *last)
+static void ERROR_copy(ERROR_INFO *save, ERROR_INFO *last)
 {
 	ERROR_reset(save);
 	*save = ERROR_current->info;
-	CLEAR(&ERROR_current->info);
 
 	if (last)
 	{
 		ERROR_reset(last);
 		*last = ERROR_last;
-		CLEAR(&ERROR_last);
 	}
 }
 
+void ERROR_save(ERROR_INFO *save, ERROR_INFO *last)
+{
+	ERROR_copy(save, last);
+
+	CLEAR(&ERROR_current->info);
+	if (last)
+		CLEAR(&ERROR_last);
+}
+
 void ERROR_restore(ERROR_INFO *save, ERROR_INFO *last)
 {
 	ERROR_reset(&ERROR_current->info);
@@ -628,6 +665,14 @@ void ERROR_set_last(bool bt)
 	}
 }
 
+void ERROR_define_last(void)
+{
+	ERROR_reset(&ERROR_current->info);
+	ERROR_current->info = ERROR_last;
+	if (ERROR_last.free)
+		STRING_ref(ERROR_last.msg);
+}
+
 void ERROR_warning(const char *warning, ...)
 {
 	va_list args;
@@ -638,6 +683,9 @@ void ERROR_warning(const char *warning, ...)
 
 	fprintf(stderr, "gbx" GAMBAS_VERSION_STRING ": warning: ");
 	vfprintf(stderr, warning, args);
+	
+	va_end(args);
+	
 	putc('\n', stderr);
 }
 
@@ -650,4 +698,41 @@ void ERROR_exit(void)
 {
 	ERROR_reset(&ERROR_last);
 	STACK_free_backtrace(&ERROR_backtrace);
-}
\ No newline at end of file
+}
+
+void ERROR_hook(void)
+{
+	static bool no_rec = FALSE;
+	
+	ERROR_INFO save = { 0 };
+	ERROR_INFO last = { 0 };
+	CLASS_DESC_METHOD *handle_error;
+	
+	if (no_rec)
+		return;
+	
+	if (PROJECT_class && PROJECT_class->state)
+	{
+		handle_error = (CLASS_DESC_METHOD *)CLASS_get_symbol_desc_kind(PROJECT_class, "Application_Error", CD_STATIC_METHOD, 0);
+		
+		if (handle_error)
+		{
+			no_rec = TRUE;
+			ERROR_copy(&save, &last);
+			
+			TRY
+			{
+				EXEC_public_desc(PROJECT_class, NULL, handle_error, 0);
+			}
+			CATCH
+			{
+				ERROR_save(&save, &last);
+			}
+			END_TRY
+
+			ERROR_restore(&save, &last);
+			no_rec = FALSE;
+		}
+	}
+}
+
