@@ -2,7 +2,7 @@
 
 	gbx_api.c
 
-	(c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "gb_common_buffer.h"
 #include "gb_error.h"
 #include "gb_alloc.h"
+#include "gb_list.h"
 
 #include <stdarg.h>
 
@@ -55,6 +56,12 @@
 #include "gbx_c_observer.h"
 #include "gbx_debug.h"
 #include "gbx_c_file.h"
+#include "gbx_extern.h"
+#include "gbx_compare.h"
+#include "gbx_subr.h"
+#include "gbx_math.h"
+#include "gbx_struct.h"
+#include "gbx_signal.h"
 
 #include "gambas.h"
 #include "gbx_api.h"
@@ -66,7 +73,7 @@ typedef
 		}
 	GB_API_FUNCTION;
 
-void *GAMBAS_Api[] =
+const void *const GAMBAS_Api[] =
 {
 	(void *)GB_VERSION,
 
@@ -78,6 +85,7 @@ void *GAMBAS_Api[] =
 	(void *)COMPONENT_exist,
 	(void *)GB_CurrentComponent,
 	(void *)COMPONENT_get_info,
+	(void *)COMPONENT_signal,
 
 	(void *)GB_Push,
 	(void *)GB_GetFunction,
@@ -85,8 +93,11 @@ void *GAMBAS_Api[] =
 	(void *)GB_GetClassInterface,
 	(void *)GB_GetProperty,
 	(void *)GB_SetProperty,
+	(void *)GB_Serialize,
+	(void *)GB_UnSerialize,
 
 	(void *)WATCH_one_loop,
+	(void *)GB_Wait,
 	(void *)EVENT_post,
 	(void *)EVENT_post2,
 	(void *)CTIMER_every,
@@ -100,7 +111,6 @@ void *GAMBAS_Api[] =
 	(void *)GB_GetLastEventName,
 	(void *)CTIMER_raise,
 	(void *)GB_Stopped,
-	(void *)COMPONENT_signal,
 
 	(void *)GB_NParam,
 	(void *)GB_Conv,
@@ -108,6 +118,9 @@ void *GAMBAS_Api[] =
 
 	(void *)GB_Error,
 	(void *)ERROR_propagate,
+	(void *)GB_Deprecated,
+	(void *)GB_OnErrorBegin,
+	(void *)GB_OnErrorEnd,
 
 	(void *)GB_GetClass,
 	(void *)GB_GetClassName,
@@ -122,13 +135,15 @@ void *GAMBAS_Api[] =
 	(void *)GB_Detach,
 	(void *)GB_Attach,
 	(void *)OBJECT_parent,
+	(void *)GB_Create,
 	(void *)GB_New,
 	(void *)CLASS_auto_create,
 	(void *)GB_CheckObject,
 
 	(void *)GB_GetEnum,
 	(void *)GB_StopEnum,
-	(void *)GB_ListEnum,
+	(void *)GB_BeginEnum,
+	(void *)GB_EndEnum,
 	(void *)GB_NextEnum,
 	(void *)GB_StopAllEnum,
 
@@ -140,9 +155,12 @@ void *GAMBAS_Api[] =
 	(void *)GB_ReturnDate,
 	(void *)GB_ReturnObject,
 	(void *)GB_ReturnNull,
+	(void *)GB_ReturnSingle,
 	(void *)GB_ReturnFloat,
 	(void *)GB_ReturnVariant,
 	(void *)GB_ReturnConvVariant,
+	(void *)GB_ReturnBorrow,
+	(void *)GB_ReturnRelease,
 	(void *)GB_ReturnPtr,
 	(void *)GB_ReturnSelf,
 
@@ -157,6 +175,7 @@ void *GAMBAS_Api[] =
 	(void *)GB_NewZeroString,
 	(void *)GB_TempString,
 	(void *)GB_FreeString,
+	(void *)STRING_free_later,
 	(void *)STRING_extend,
 	(void *)STRING_add,
 	(void *)STRING_add_char,
@@ -180,6 +199,7 @@ void *GAMBAS_Api[] =
 	(void *)GB_TempFile,
 	(void *)GB_CopyFile,
 	(void *)GB_BrowseProject,
+	(void *)GB_StatFile,
 
 	(void *)GB_Store,
 	(void *)GB_StoreString,
@@ -188,6 +208,7 @@ void *GAMBAS_Api[] =
 	(void *)VALUE_read,
 	(void *)GB_BorrowValue,
 	(void *)GB_ReleaseValue,
+	(void *)COMPARE_variant,
 
 	(void *)DATE_split,
 	(void *)DATE_make,
@@ -209,12 +230,6 @@ void *GAMBAS_Api[] =
 	(void *)ARRAY_insert_many,
 	(void *)ARRAY_remove_many,
 
-	(void *)GB_SubCollectionNew,
-	(void *)GB_SubCollectionAdd,
-	(void *)GB_SubCollectionRemove,
-	(void *)GB_SubCollectionGet,
-	(void *)GB_SubCollectionContainer,
-
 	(void *)GB_tolower,
 	(void *)GB_toupper,
 	(void *)strcasecmp,
@@ -231,6 +246,8 @@ void *GAMBAS_Api[] =
 	(void *)GB_SystemDomainName,
 	(void *)GB_IsRightToLeft,
 	(void *)GB_SystemPath,
+	(void *)FILE_init,
+	(void *)GB_SystemDebug,
 
 	(void *)GB_ArrayNew,
 	(void *)GB_ArrayCount,
@@ -255,20 +272,31 @@ void *GAMBAS_Api[] =
 	(void *)GB_StreamGet,
 	(void *)GB_StreamSetBytesRead,
 	(void *)GB_StreamSetSwapping,
+	(void *)GB_StreamSetAvailableNow,
 	(void *)GB_StreamBlock,
 	(void *)GB_StreamRead,
 	(void *)GB_StreamWrite,
+	(void *)STREAM_get_readable,
+	(void *)STREAM_eof,
 
 	(void *)STRING_start_len,
 	(void *)STRING_end,
 	(void *)STRING_make,
 	
 	(void *)DEBUG_get_current_position,
+	(void *)DEBUG_enter_event_loop,
+	(void *)DEBUG_leave_event_loop,
+	
+	(void *)SIGNAL_register,
+	(void *)SIGNAL_unregister,
+	
+	(void *)LIST_insert,
+	(void *)LIST_remove,
 
 	NULL
 };
 
-void *GAMBAS_DebugApi[] =
+const void *GAMBAS_DebugApi[] =
 {
 	(void *)GB_DebugGetExec,
 	(void *)STACK_get_frame,
@@ -289,11 +317,177 @@ void *GAMBAS_DebugApi[] =
 	NULL
 };
 
+void *GAMBAS_JitApi[] =
+{
+	(void *)EXEC_release,
+	(void *)RELEASE_many,
+	
+	(void *)EXEC_push_unknown,
+	(void *)EXEC_push_array,
+	
+	(void *)EXEC_pop_unknown,
+	(void *)EXEC_pop_array,
+	
+	(void *)CENUM_create,
+	(void *)EXEC_enum_first,
+	(void *)EXEC_enum_next,
+	
+	(void *)EXEC_new,
+	(void *)EXEC_enter_quick,
+	(void *)EXEC_enter,
+	(void *)EXEC_native_quick,
+	(void *)EXEC_native,
+	(void *)EXEC_call_native,
+	(void *)EXEC_function_real,
+	(void *)EXEC_leave_keep,
+	(void *)EXEC_leave_drop,
+	(void *)EXEC_function_loop,
+	
+	(void *)EXEC_special,
+	(void *)EXEC_object_variant,
+	(void *)EXEC_object_other,
+	
+	(void *)EXTERN_get_function_info,
+	(void *)EXTERN_call,
+	(void *)EXTERN_make_callback,
+	
+	(void *)STRING_new,
+	(void *)STRING_new_temp_value,
+	(void *)STRING_free_real,
+	(void *)STRING_free_later,
+	(void *)STRING_compare,
+	(void *)STRING_equal_ignore_case_same,
+	(void *)STRING_conv,
+	
+	(void *)OBJECT_comp_value,
+	(void *)COMPARE_object,
+	(void *)CLASS_inherits,
+	(void *)OBJECT_create,
+	(void *)CLASS_free,
+	(void *)SYMBOL_find,
+	
+	(void *)CARRAY_get_array_class,
+	(void *)CARRAY_get_data_multi,
+	(void *)CARRAY_create_static,
+	(void *)CSTRUCT_create_static,
+	
+	(void *)REGEXP_scan,
+	
+	(void *)VALUE_convert,
+	(void *)VALUE_to_string,
+	(void *)VALUE_convert_float,
+	(void *)VALUE_convert_variant,
+	(void *)VALUE_convert_string,
+	(void *)VALUE_is_null,
+	(void *)VALUE_undo_variant,
+	
+	(void *)NUMBER_from_string,
+	(void *)NUMBER_int_to_string,
+	
+	(void *)LOCAL_format_number,
+	
+	(void *)DATE_to_string,
+	(void *)DATE_from_string,
+	(void *)DATE_comp,
+	(void *)DATE_timer,
+	(void *)DATE_now,
+	(void *)DATE_add,
+	(void *)DATE_diff,
+	
+	(void *)randomize,
+	(void *)rnd,
+	
+	(void *)THROW,
+	(void *)ERROR_panic,
+	(void *)ERROR_propagate,
+	(void *)ERROR_reset,
+	(void *)ERROR_set_last,
+	(void *)ERROR_lock,
+	(void *)ERROR_unlock,
+	
+	(void *)TYPE_get_name,
+	
+	(void *)SUBR_not,
+	(void *)SUBR_and_,
+	(void *)SUBR_cat,
+	(void *)SUBR_file,
+	(void *)SUBR_like,
+	(void *)SUBR_string,
+	(void *)SUBR_upper,
+	(void *)SUBR_instr,
+	(void *)SUBR_subst,
+	(void *)SUBR_replace,
+	(void *)SUBR_split,
+	(void *)SUBR_strcomp,
+	(void *)SUBR_sconv,
+	(void *)SUBR_abs,
+	(void *)SUBR_int,
+	(void *)SUBR_fix,
+	(void *)SUBR_sgn,
+	(void *)SUBR_min_max,
+	(void *)SUBR_choose,
+	(void *)SUBR_bit,
+	(void *)SUBR_is_type,
+	(void *)SUBR_hex_bin,
+	(void *)SUBR_val,
+	(void *)SUBR_format,
+	(void *)SUBR_year,
+	(void *)SUBR_week,
+	(void *)SUBR_date,
+	(void *)SUBR_time,
+	(void *)SUBR_eval,
+	(void *)SUBR_debug,
+	(void *)SUBR_wait,
+	(void *)SUBR_open,
+	(void *)SUBR_close,
+	(void *)SUBR_input,
+	(void *)SUBR_linput,
+	(void *)SUBR_print,
+	(void *)SUBR_read,
+	(void *)SUBR_write,
+	(void *)SUBR_flush,
+	(void *)SUBR_lock,
+	(void *)SUBR_inp_out,
+	(void *)SUBR_eof,
+	(void *)SUBR_lof,
+	(void *)SUBR_seek,
+	(void *)SUBR_kill,
+	(void *)SUBR_move,
+	(void *)SUBR_exist,
+	(void *)SUBR_access,
+	(void *)SUBR_stat,
+	(void *)SUBR_dfree,
+	(void *)SUBR_temp,
+	(void *)SUBR_isdir,
+	(void *)SUBR_dir,
+	(void *)SUBR_rdir,
+	(void *)SUBR_exec,
+	(void *)SUBR_alloc,
+	(void *)SUBR_free,
+	(void *)SUBR_realloc,
+	(void *)SUBR_strptr,
+	(void *)SUBR_collection,
+	(void *)SUBR_tr,
+	(void *)SUBR_quote,
+	(void *)SUBR_unquote,
+	(void *)SUBR_ptr,
+	
+	(void *)CLASS_load_from_jit,
+	(void *)CLASS_run_inits,
+	
+	(void *)DEBUG_get_current_position,
+	(void *)NULL, // Later set to DEBUG.Profile.Add
+	
+	(void *)EXEC_quit,
+	NULL
+};
+
 
 bool GAMBAS_DoNotRaiseEvent = FALSE;
 bool GAMBAS_StopEvent = FALSE;
 
 static bool _event_stopped = FALSE;
+static int _raise_event_level = 0;
 
 #define CATCH_ERROR \
 	bool ret = FALSE; \
@@ -397,7 +591,7 @@ static void push(int nval, va_list args)
 
 			case T_OBJECT:
 				SP->_object.object = va_arg(args, void *);
-				OBJECT_REF(SP->_object.object, "push");
+				OBJECT_REF(SP->_object.object);
 				break;
 
 			default:
@@ -420,20 +614,106 @@ void GB_Push(int nval, ...)
 	va_end(args);
 }
 
-
-void GB_GetProperty(void *object, const char *property)
+static void error(int code, CLASS *class, const char *name)
 {
-	static bool init = FALSE;
-	static GB_FUNCTION func;
-	
-	if (!init)
+	GB_Error((char *)(intptr_t)code, CLASS_get_name(class), name);
+}
+
+static CLASS_DESC *get_desc(CLASS *class, const char *name)
+{
+	int index;
+
+	index = CLASS_find_symbol(class, name);
+
+	if (index == NO_SYMBOL)
 	{
-		GB_GetFunction(&func, (void *)CLASS_find("Object"), "GetProperty", NULL, NULL);
-		init = TRUE;
+		error(E_NSYMBOL, class, name);
+		return NULL;
 	}
+	else
+		return class->table[index].desc;
+}
+
+void GB_GetProperty(void *object, const char *name)
+{
+	CLASS_DESC *desc;
+	CLASS *class;
+	char type;
+
+	//if (GB_CheckObject(object))
+	//	return;
+
+	if (OBJECT_is_class(object))
+	{
+		class = (CLASS *)object;
+		object = NULL;
+	}
+	else
+	{
+		class = OBJECT_class(object);
+	}
+
+	desc = get_desc(class, name);
+
+	if (!desc)
+		return;
+
+	type = CLASS_DESC_get_type(desc);
+
+	if (type == CD_PROPERTY || type == CD_PROPERTY_READ || type == CD_VARIABLE)
+	{
+		if (!object)
+		{
+			error(E_DYNAMIC, class, name);
+			return;
+		}
+	}
+	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_PROPERTY_READ || type == CD_STATIC_VARIABLE || type == CD_CONSTANT)
+	{
+		if (object)
+		{
+			error(E_STATIC, class, name);
+			return;
+		}
+	}
+	else
+	{
+		error(E_NPROPERTY, class, name);
+		return;
+	}
+
+	if (type == CD_VARIABLE)
+		VALUE_read(&TEMP, (char *)object + desc->variable.offset, desc->property.type);
+	else if (type == CD_STATIC_VARIABLE)
+		VALUE_read(&TEMP, (char *)class->stat + desc->variable.offset, desc->property.type);
+	else if (type == CD_CONSTANT)
+		VALUE_read(&TEMP, (void *)&desc->constant.value, desc->constant.type);
+	else
+	{
+		if (desc->property.native)
+		{
+			if (EXEC_call_native(desc->property.read, object, desc->property.type, 0))
+			{
+				EXEC_set_native_error(TRUE);
+				return;
+			}
+		}
+		else
+		{
+			EXEC.class = desc->property.class;
+			EXEC.object = object;
+			EXEC.nparam = 0;
+			EXEC.native = FALSE;
+			EXEC.index = (int)(intptr_t)desc->property.read;
+			//EXEC.func = &class->load->func[(long)desc->property.read];
 	
-	GB_Push(2, GB_T_OBJECT, object, GB_T_STRING, property, strlen(property));
-	GB_Call(&func, 2, FALSE);
+			EXEC_function_keep();
+	
+			TEMP = *RP;
+			UNBORROW(RP);
+			RP->type = T_VOID;
+		}
+	}
 }
 
 void GB_SetProperty(void *object, const char *property, ...)
@@ -555,16 +835,18 @@ static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam)
 
 // If nparam < 0, the args are already on the stack
 
-static void *_GB_Raise_unref = NULL;
-static char _GB_Raise_nparam = 0;
 static GB_RAISE_HANDLER *_GB_Raise_handler = NULL;
 
 static void error_GB_Raise()
 {
-	RELEASE_MANY(SP, _GB_Raise_nparam);
-	OBJECT_UNREF(_GB_Raise_unref, "error_GB_Raise");
+	void *object = (void *)ERROR_handler->arg1;
 	
-	if (_GB_Raise_handler)
+	RELEASE_MANY(SP, (int)(ERROR_handler->arg2));
+	OBJECT_UNREF(object);
+	
+	_raise_event_level--;
+	
+	if (_GB_Raise_handler && _GB_Raise_handler->level == _raise_event_level)
 	{
 		(*_GB_Raise_handler->callback)(_GB_Raise_handler->data);
 		_GB_Raise_handler = _GB_Raise_handler->old;
@@ -574,6 +856,7 @@ static void error_GB_Raise()
 void GB_RaiseBegin(GB_RAISE_HANDLER *handler)
 {
 	handler->old = _GB_Raise_handler;
+	handler->level = _raise_event_level;
 	_GB_Raise_handler = handler;
 }
 
@@ -590,8 +873,6 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 	va_list args;
 	bool arg;
 	COBSERVER *obs;
-	void *save_GB_Raise_unref;
-	char save_GB_Raise_nparam;
 
 	if (GAMBAS_DoNotRaiseEvent)
 		return FALSE;
@@ -599,17 +880,15 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 	if (!OBJECT_is_valid(object) || !OBJECT_has_events(object))
 		return FALSE;
 
-	save_GB_Raise_unref = _GB_Raise_unref;
-	save_GB_Raise_nparam = _GB_Raise_nparam;
+	OBJECT_REF(object);
+
+	arg = nparam < 0;
+	nparam = abs(nparam);
 	
-	ON_ERROR(error_GB_Raise)
+	_raise_event_level++;
+	
+	ON_ERROR_2(error_GB_Raise, object, nparam)
 	{
-		OBJECT_REF(object, "GB_Raise");
-		_GB_Raise_unref = object;
-	
-		arg = nparam < 0;
-		nparam = abs(nparam);
-	
 		if (!arg)
 		{
 			va_start(args, nparam);
@@ -617,8 +896,6 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 			va_end(args);
 		}
 	
-		_GB_Raise_nparam = nparam;
-		
 		result = FALSE;
 	
 		// Observers before
@@ -639,7 +916,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 			
 			if (!OBJECT_is_valid(parent))
 			{
-				OBJECT_detach(object);
+				OBJECT_detach((OBJECT *)obs);
 				continue;
 			}
 			
@@ -697,7 +974,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 			
 			if (!OBJECT_is_valid(parent))
 			{
-				OBJECT_detach(object);
+				OBJECT_detach((OBJECT *)obs);
 				continue;
 			}
 			
@@ -710,13 +987,12 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 __RETURN:
 	
 		RELEASE_MANY(SP, nparam);
-		OBJECT_UNREF(object, "GB_Raise");	
+		OBJECT_UNREF(object);	
 	}
 	END_ERROR
 
-	_GB_Raise_unref = save_GB_Raise_unref;
-	_GB_Raise_nparam = save_GB_Raise_nparam;
-
+	_raise_event_level--;
+	
 	return result;
 }
 
@@ -768,14 +1044,14 @@ bool GB_GetFunction(GB_FUNCTION *_func, void *object, const char *name, const ch
 
 		if (nparam)
 		{
-			ALLOC(&tsign, nparam * sizeof(TYPE), "GB_GetFunction");
+			ALLOC(&tsign, nparam * sizeof(TYPE));
 			tsign = TYPE_transform_signature(&tsign, sign, nparam);
 		}
 
 		error = TYPE_compare_signature(desc->method.signature, desc->method.npmax, tsign, nparam);
 
 		if (nparam)
-			FREE(&tsign, "GB_GetFunction");
+			FREE(&tsign);
 
 		if (error)
 		{
@@ -800,7 +1076,7 @@ bool GB_GetFunction(GB_FUNCTION *_func, void *object, const char *name, const ch
 		}
 	}
 
-	func->object = object;
+	func->object = kind == CD_STATIC_METHOD ? NULL : object;
 	func->desc = &desc->method;
 
 	if (!func->desc)
@@ -810,7 +1086,7 @@ bool GB_GetFunction(GB_FUNCTION *_func, void *object, const char *name, const ch
 
 _NOT_FOUND:
 
-	GB_Error("Unable to find method &1 in class &2. &3", name, class->name, err);
+	GB_Error("Unable to find method &1 in class &2. &3", name, CLASS_get_name(class), err);
 	func->object = NULL;
 	func->desc = NULL;
 	return TRUE;
@@ -885,7 +1161,7 @@ int GB_GetEvent(void *class, char *name)
 	if (!cd)
 		return (-1);
 	else
-		return *cd->index;
+		return cd->index;
 }
 
 
@@ -919,6 +1195,18 @@ const char *GB_GetUnknown(void)
 }
 
 
+void GB_OnErrorBegin(GB_ERROR_HANDLER *handler)
+{
+	handler->prev = ERROR_handler;
+	handler->context = ERROR_current;
+	ERROR_handler = (ERROR_HANDLER *)handler;
+}
+
+void GB_OnErrorEnd(GB_ERROR_HANDLER *handler)
+{
+	ERROR_handler = (ERROR_HANDLER *)handler->prev;
+}
+
 void GB_Error(const char *error, ...)
 {
 	va_list args;
@@ -935,11 +1223,32 @@ void GB_Error(const char *error, ...)
 
 	for (i = 0; i < 4; i++)
 		arg[i] = va_arg(args, char *);
+	
+	va_end(args);
 
 	ERROR_define(error, arg);
 	EXEC_set_native_error(TRUE);
 }
 
+void GB_Deprecated(const char *msg, const char *func, const char *repl)
+{
+	if (EXEC_debug)
+	{
+		if (repl)
+			GB_Error("&1: &2 is deprecated. Use &3 instead", msg, func, repl);
+		else
+			GB_Error("&1: &2 is deprecated. Do not use it anymore", msg, func);
+	}
+	else
+	{
+		DEBUG_where();
+		if (repl)
+			fprintf(stderr, "%s: %s is deprecated. Use %s instead\n", msg, func, repl);
+		else
+			fprintf(stderr, "%s: %s is deprecated. Do not use it anymore\n", msg, func);
+	}
+}
+
 #if DEBUG_REF
 
 #include <execinfo.h>
@@ -972,7 +1281,7 @@ void GB_Ref(void *object)
 		#if DEBUG_REF
 		//print_stack_backtrace();
 		#endif
-		OBJECT_REF(object, "GB_Ref");
+		OBJECT_REF(object);
 	}
 
 	#if TRACE_MEMORY
@@ -993,7 +1302,7 @@ void GB_Unref(void **object)
 		#if DEBUG_REF
 		//print_stack_backtrace();
 		#endif
-		OBJECT_UNREF(*object, "GB_Unref");
+		OBJECT_UNREF(*object);
 	}
 	
 	#if TRACE_MEMORY
@@ -1016,11 +1325,11 @@ void GB_UnrefKeep(void **object, int delete)
 		#endif
 		if (delete)
 		{
-			OBJECT_UNREF(*object, "GB_UnrefKeep");
+			OBJECT_UNREF(*object);
 		}
 		else
 		{
-			OBJECT_UNREF_KEEP(*object, "GB_UnrefKeep");
+			OBJECT_UNREF_KEEP(*object);
 		}
 	}
 
@@ -1073,10 +1382,17 @@ void *GB_GetEnum(void)
 
 static void *_enum_object;
 
-void GB_ListEnum(void *enum_object)
+void *GB_BeginEnum(void *enum_object)
 {
+	CENUM *old = EXEC_enum;
 	EXEC_enum = NULL;
 	_enum_object = enum_object;
+	return old;
+}
+
+void GB_EndEnum(void *old_enum)
+{
+	EXEC_enum = old_enum;
 }
 
 
@@ -1087,16 +1403,17 @@ bool GB_NextEnum(void)
 		EXEC_enum = CENUM_get_next(EXEC_enum);
 		if (!EXEC_enum)
 			return TRUE;
-		if (EXEC_enum->enum_object == _enum_object)
+		if (EXEC_enum->enum_object == _enum_object && !EXEC_enum->stop)
 			return FALSE;
 	}
 }
 
 void GB_StopAllEnum(void *enum_object)
 {
-	GB_ListEnum(enum_object);
+	void *save = GB_BeginEnum(enum_object);
 	while (!GB_NextEnum())
 		GB_StopEnum();
+	GB_EndEnum(save);
 }
 
 
@@ -1179,6 +1496,8 @@ __VOID:
 __VARIANT:
 __FUNCTION:
 __NULL:
+
+	va_end(args);
 	return;
 }
 
@@ -1274,9 +1593,20 @@ void GB_ReturnVariant(GB_VARIANT_VALUE *val)
 
 void GB_ReturnConvVariant(void)
 {
+	BORROW(&TEMP);
 	VALUE_conv(&TEMP, T_VARIANT);
+	UNBORROW(&TEMP);
 }
 
+void GB_ReturnBorrow(void)
+{
+	BORROW(&TEMP);
+}
+
+void GB_ReturnRelease(void)
+{
+	UNBORROW(&TEMP);
+}
 
 void GB_ReturnPtr(GB_TYPE type, void *value)
 {
@@ -1401,7 +1731,7 @@ void *GB_GetClass(void *object)
 char *GB_GetClassName(void *object)
 {
 	CLASS *class = GB_GetClass(object);
-	return class->name;
+	return CLASS_get_name(class);
 }
 
 
@@ -1481,9 +1811,9 @@ void GB_StoreObject(GB_OBJECT *src, void **dst)
 		object = NULL;
 
 	if (object)
-		OBJECT_REF(object, "GB_StoreObject");
+		OBJECT_REF(object);
 
-	OBJECT_UNREF(*dst, "GB_StoreObject");
+	OBJECT_UNREF(*dst);
 	*dst = object;
 }
 
@@ -1521,10 +1851,18 @@ void GB_Watch(int fd, int flag, void *callback, intptr_t param)
 }
 
 
-void *GB_New(void *class, const char *name, void *parent)
+void *GB_Create(void *class, const char *name, void *parent)
 {
 	void *object;
 	
+	object = OBJECT_new(class, name, parent);
+	OBJECT_UNREF_KEEP(object);
+	
+	return object;
+}
+
+void *GB_New(void *class, const char *name, void *parent)
+{
 	if (name && !parent)
 	{
 		parent = OP;
@@ -1533,17 +1871,20 @@ void *GB_New(void *class, const char *name, void *parent)
 	}
 
 	if (!((CLASS *)class)->no_create)
-		object = OBJECT_create(class, name, parent, 0);
-	else
 	{
-		object = OBJECT_new(class, name, parent);
-		OBJECT_UNREF_KEEP(object, "GB_New");
+		int nparam = 0;
+		if (!name && parent)
+		{
+			nparam = (int)(intptr_t)parent;
+			parent = NULL;
+		}
+		
+		return OBJECT_create(class, name, parent, nparam);
 	}
-	
-	return object;
+	else
+		return GB_Create(class, name, parent);
 }
 
-
 bool GB_CheckObject(void *object)
 {
 	CLASS *class;
@@ -1607,17 +1948,17 @@ void *GB_Eval(void *expr, void *func)
 
 void GB_Alloc(void **addr, int len)
 {
-	ALLOC(addr, len, "GB_Alloc");
+	ALLOC(addr, len);
 }
 
 void GB_Free(void **addr)
 {
-	FREE(addr, "GB_Free");
+	FREE(addr);
 }
 
 void GB_Realloc(void **addr, int len)
 {
-	REALLOC(addr, len, "GB_Realloc");
+	REALLOC(addr, len);
 }
 
 
@@ -1806,12 +2147,17 @@ GB_STREAM *GB_StreamGet(void *object)
 
 void GB_StreamSetBytesRead(GB_STREAM *stream, int length)
 {
-	STREAM_eff_read = length;
+	STREAM_eff_read += length;
 }
 
-void GB_StreamSetSwapping(GB_STREAM *stream, int swap)
+void GB_StreamSetSwapping(GB_STREAM *stream, int v)
 {
-	((STREAM *)stream)->common.swap = swap;
+	((STREAM *)stream)->common.swap = v;
+}
+
+void GB_StreamSetAvailableNow(GB_STREAM *stream, int v)
+{
+	((STREAM *)stream)->common.available_now = v;
 }
 
 bool GB_StreamBlock(GB_STREAM *stream, int block)
@@ -1918,25 +2264,16 @@ int GB_FindFile(const char *dir, int recursive, int follow, void (*found)(const
 	
 	return ret;
 }
+#endif
 
-int GB_StatFile(const char *path, GB_FILE_STAT *info, int follow)
+bool GB_StatFile(const char *path, GB_FILE_STAT *info, bool follow)
 {
-	int ret = 0;
-	
-	TRY
+	CATCH_ERROR
 	{
 		FILE_stat(path, (FILE_STAT *)info, follow);
 	}
-	CATCH
-	{
-		ret = 1;
-		GAMBAS_Error = TRUE;
-	}
-	END_TRY;
-	
-	return ret;
+	END_CATCH_ERROR
 }
-#endif
 
 char *GB_RealFileName(const char *name, int len)
 {
@@ -2007,6 +2344,11 @@ void *GB_DebugGetExec(void)
 	return &EXEC_current;
 }
 
+bool GB_SystemDebug(void)
+{
+	return EXEC_debug;
+}
+
 
 bool GB_ExistClass(const char *name)
 {
@@ -2019,12 +2361,68 @@ bool GB_ExistClassLocal(const char *name)
 	return CLASS_look(name, strlen(name)) != NULL;
 }
 
-/*void GB_AddString(char **ptr, const char *src, int len)
+void GB_Wait(int delay)
 {
-	*ptr = STRING_add(*ptr, src, len);
+	struct timespec rem;
+	double wait;
+	double stop, time;
+
+	DEBUG_enter_event_loop();
+		
+	if (delay == 0)
+	{
+		HOOK_DEFAULT(wait, WATCH_wait)(0);
+	}
+	else
+	{
+		wait = delay / 1000.0;
+		
+		DATE_timer(&stop, FALSE);
+		stop += wait;
+
+		for(;;)
+		{
+			HOOK_DEFAULT(wait, WATCH_wait)((int)(wait * 1000 + 0.5));
+
+			if (DATE_timer(&time, FALSE))
+				break;
+
+			wait = stop - time;
+			if (wait <= 0.0)
+				break;
+			
+			rem.tv_sec = 0;
+			rem.tv_nsec = 1000000;
+			nanosleep(&rem, &rem);
+		}
+	}
+
+	DEBUG_leave_event_loop();
 }
 
-void GB_ExtendString(char **ptr, int new_len)
+bool GB_Serialize(const char *path, GB_VALUE *value)
 {
-	*ptr = STRING_extend(*ptr, new_len);
-}*/
+	STREAM stream;
+	
+	CATCH_ERROR
+	{
+		STREAM_open(&stream, path, ST_CREATE);
+		STREAM_write_type(&stream, T_VARIANT, (VALUE *)value);
+		STREAM_close(&stream);
+	}
+	END_CATCH_ERROR
+}
+
+bool GB_UnSerialize(const char *path, GB_VALUE *value)
+{
+	STREAM stream;
+	
+	CATCH_ERROR
+	{
+		STREAM_open(&stream, path, ST_READ);
+		STREAM_read_type(&stream, T_VARIANT, (VALUE *)value);
+		STREAM_close(&stream);
+	}
+	END_CATCH_ERROR
+}
+
