@@ -2,7 +2,7 @@
 
   gbx_api.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ void *GB_Hook(int type, void *hook);
 bool GB_LoadComponent(const char *name);
 const char *GB_CurrentComponent(void);
 
+void GB_Wait(int);
 void GB_Push(int nval, ...);
 bool GB_CanRaise(void *object, int event_id);
 bool GB_Raise(void *object, int event_id, int nparam, ...);
@@ -51,7 +52,13 @@ const char *GB_GetUnknown(void);
 void GB_GetProperty(void *object, const char *property);
 void GB_SetProperty(void *object, const char *property, ...);
 
+bool GB_Serialize(const char *path, GB_VALUE *value);
+bool GB_UnSerialize(const char *path, GB_VALUE *value);
+
 void GB_Error(const char *msg, ...);
+void GB_Deprecated(const char *msg, const char *func, const char *repl);
+void GB_OnErrorBegin(GB_ERROR_HANDLER *handler);
+void GB_OnErrorEnd(GB_ERROR_HANDLER *handler);
 
 void GB_Ref(void *object);
 void GB_Unref(void **object);
@@ -59,7 +66,8 @@ void GB_UnrefKeep(void **object, int);
 
 void GB_StopEnum(void);
 void *GB_GetEnum(void);
-void GB_ListEnum(void *);
+void *GB_BeginEnum(void *);
+void GB_EndEnum(void *);
 bool GB_NextEnum(void);
 void GB_StopAllEnum(void *);
 
@@ -77,7 +85,10 @@ void GB_ReturnPtr(GB_TYPE type, void *value);
 void GB_ReturnDate(GB_DATE *date);
 void GB_ReturnSelf(void *object);
 void GB_ReturnVariant(GB_VARIANT_VALUE *value);
+
 void GB_ReturnConvVariant(void);
+void GB_ReturnBorrow(void);
+void GB_ReturnRelease(void);
 
 void GB_ReturnString(char *str);
 void GB_ReturnVoidString(void);
@@ -103,7 +114,7 @@ char *GB_TempDir(void);
 char *GB_TempFile(const char *pattern);
 bool GB_CopyFile(const char *src, const char *dst);
 //int GB_FindFile(const char *dir, int recursive, int follow, void (*found)(const char *));
-//int GB_StatFile(const char *path, GB_FILE_STAT *info, int follow);
+bool GB_StatFile(const char *path, GB_FILE_STAT *info, bool follow);
 void GB_BrowseProject(GB_BROWSE_CALLBACK func);
 
 int GB_IsMissing(int param);
@@ -120,6 +131,7 @@ void GB_ReleaseValue(GB_VALUE *value);
 
 void GB_Watch(int fd, int flag, void *callback, intptr_t param);
 
+void *GB_Create(void *class_name, const char *name, void *parent);
 void *GB_New(void *class_name, const char *name, void *parent);
 bool GB_CheckObject(void *object);
 bool GB_Is(void *object, void *class);
@@ -138,6 +150,7 @@ char *GB_SystemCharset(void);
 char *GB_SystemDomainName(void);
 bool GB_IsRightToLeft(void);
 char *GB_SystemPath(void);
+bool GB_SystemDebug(void);
 
 void *GB_Eval(void *, void *);
 
@@ -180,6 +193,7 @@ void *GB_Add(void *pdata);
 GB_STREAM *GB_StreamGet(void *);
 void GB_StreamSetBytesRead(GB_STREAM *, int);
 void GB_StreamSetSwapping(GB_STREAM *, int);
+void GB_StreamSetAvailableNow(GB_STREAM *, int);
 bool GB_StreamBlock(GB_STREAM *, int);
 int GB_StreamRead(GB_STREAM *stream, void *addr, int len);
 int GB_StreamWrite(GB_STREAM *stream, void *addr, int len);
@@ -187,12 +201,6 @@ int GB_StreamWrite(GB_STREAM *stream, void *addr, int len);
 int GB_tolower(int c);
 int GB_toupper(int c);
 
-void GB_SubCollectionNew(GB_SUBCOLLECTION *subcollection, GB_SUBCOLLECTION_DESC *desc, void *container);
-void GB_SubCollectionAdd(void *_object, const char *key, int len, void *value);
-void GB_SubCollectionRemove(void *_object, const char *key, int len);
-void *GB_SubCollectionContainer(void *_object);
-void *GB_SubCollectionGet(void *_object, const char *key, int len);
-
 void *GB_DebugGetClass(const char *name);
 void *GB_DebugGetExec(void);
 
@@ -201,6 +209,7 @@ void *GB_DebugGetExec(void);
 #ifndef __GBX_API_C
 EXTERN void *GAMBAS_Api[];
 EXTERN void *GAMBAS_DebugApi[];
+EXTERN void *GAMBAS_JitApi[];
 EXTERN unsigned int GAMBAS_MissingParam;
 EXTERN bool GAMBAS_DoNotRaiseEvent;
 EXTERN bool GAMBAS_StopEvent;
