@@ -2,7 +2,7 @@
 
   gbx_archive.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ ARCHIVE *ARCHIVE_create(const char *name, const char *path)
 {
   ARCHIVE *arch;
 
-  ALLOC_ZERO(&arch, sizeof(ARCHIVE), "ARCHIVE_create");
+  ALLOC_ZERO(&arch, sizeof(ARCHIVE));
 
   arch->name = name;
 	arch->path = path;
@@ -75,7 +75,7 @@ ARCHIVE *ARCHIVE_create(const char *name, const char *path)
   return arch;
 }
 
-static void load_exported_class(ARCHIVE *arch)
+void ARCHIVE_load_exported_class(ARCHIVE *arch)
 {
   /*STREAM stream;*/
   char *buffer;
@@ -84,11 +84,19 @@ static void load_exported_class(ARCHIVE *arch)
   CLASS *class;
 	CLASS **exported = NULL;
 	int i;
+	COMPONENT *current;
 
+	if (arch->exported_classes_loaded)
+		return;
+	
+	current = COMPONENT_current;
+	
+	COMPONENT_current = (COMPONENT *)arch->current_component;
+	
   /* COMPONENT_current is set => it will look in the archive */
 
   #if DEBUG_COMP
-    fprintf(stderr, "load_exported_class: %s (component: %s)\n", arch->name, COMPONENT_current ? COMPONENT_current->name : "?");
+		fprintf(stderr, "load_exported_class: %s (component: %s)\n", arch->name, COMPONENT_current ? COMPONENT_current->name : "?");
   #endif
 
 	if (!FILE_exist(".list"))
@@ -135,12 +143,20 @@ static void load_exported_class(ARCHIVE *arch)
   }
 
 	for (i = 0; i < ARRAY_count(exported); i++)
-		CLASS_load_without_init(exported[i]);
+	{
+		//fprintf(stderr, "load %p %s\n", exported[i], exported[i]->name);
+		CLASS_load(exported[i]);
+	}
 	
 	ARRAY_delete(&exported);
-  FREE(&buffer, "load_exported_class");
+  FREE(&buffer);
+	
+	arch->exported_classes_loaded = TRUE;
+	
+	COMPONENT_current = current;
 }
 
+#if 0
 static void load_component(char *name)
 {
 	COMPONENT *comp;
@@ -148,6 +164,7 @@ static void load_component(char *name)
 	comp = COMPONENT_create(name);
 	COMPONENT_load(comp);
 }
+#endif
 
 /*static void load_dependencies(ARCHIVE *arch)
 {
@@ -159,14 +176,6 @@ static void load_component(char *name)
 	FREE(&buffer, "load_dependencies");
 }*/
 
-static void load_archive(ARCHIVE *arch, const char *path) //, bool dep)
-{
-	arch->arch = ARCH_open(path);
-	load_exported_class(arch);
-	//if (dep)
-	//	load_dependencies(arch);
-}
-
 static char *exist_library(const char *dir, const char *name)
 {
 	char *path;
@@ -178,7 +187,7 @@ static char *exist_library(const char *dir, const char *name)
 		return NULL;
 }
 
-void ARCHIVE_load(ARCHIVE *arch) //, bool dep)
+void ARCHIVE_load(ARCHIVE *arch, bool load_exp)
 {
   char *path;
 
@@ -202,7 +211,11 @@ void ARCHIVE_load(ARCHIVE *arch) //, bool dep)
 		sprintf(path, ARCH_PATTERN, COMPONENT_path, arch->name);
 	}
 
-	load_archive(arch, path); //, dep);
+	arch->arch = ARCH_open(path);
+	arch->current_component = COMPONENT_current;
+	
+	if (load_exp)
+		ARCHIVE_load_exported_class(arch); //, dep);
 }
 
 
@@ -219,7 +232,7 @@ void ARCHIVE_create_main(const char *path)
 
 void ARCHIVE_load_main()
 {
-	load_exported_class(ARCHIVE_main);
+	ARCHIVE_load_exported_class(ARCHIVE_main);
 }
 
 
@@ -233,7 +246,7 @@ void ARCHIVE_delete(ARCHIVE *arch)
   TABLE_delete(&arch->classes);
   STRING_free(&arch->domain);
 
-  FREE(&arch, "ARCHIVE_delete");
+  FREE(&arch);
 }
 
 
@@ -374,8 +387,8 @@ void ARCHIVE_stat(ARCHIVE *arch, const char *path, FILE_STAT *info)
   info->mtime = (int)buf.st_mtime;
   info->ctime = (int)buf.st_mtime;
   info->hidden = (*FILE_get_name(path) == '.');
-  info->uid = buf.st_uid;
-  info->gid = buf.st_gid;
+  info->uid = (int)buf.st_uid;
+  info->gid = (int)buf.st_gid;
 }
 
 
