@@ -2,7 +2,7 @@
 
   gbx_c_array.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ static int get_dim(CARRAY *_object)
 {
 	int d;
 
-	if (LIKELY(THIS->dim == NULL))
+	if (THIS->dim == NULL)
 		return 1;
 	else
 	{
@@ -114,7 +114,7 @@ static int get_dim(CARRAY *_object)
 
 static int get_bound(CARRAY *_object, int d)
 {
-	if (LIKELY(THIS->dim == NULL))
+	if (THIS->dim == NULL)
 		return THIS->count;
 	else
 	{
@@ -279,7 +279,7 @@ static void release_one(CARRAY *_object, int i)
 	else if (THIS->type == T_VARIANT)
 		VARIANT_free(&(((VARIANT *)(THIS->data))[i]));
 	else if (TYPE_is_object(THIS->type))
-		OBJECT_UNREF(((void **)(THIS->data))[i], "release_one");
+		OBJECT_UNREF(((void **)(THIS->data))[i]);
 }
 
 static void release_static(TYPE type, void *data, int start, int end)
@@ -299,7 +299,7 @@ static void release_static(TYPE type, void *data, int start, int end)
 	else if (TYPE_is_object(type))
 	{
 		for (i = start; i < end; i++)
-			OBJECT_UNREF(((void **)data)[i], "release");
+			OBJECT_UNREF(((void **)data)[i]);
 	}
 }
 
@@ -393,7 +393,7 @@ CARRAY *CARRAY_create_static(CLASS *class, void *ref, CLASS_ARRAY *desc, void *d
 	array->data = data;
 	array->ref = ref;
 	array->count = get_count(desc->dim);
-	OBJECT_REF(ref, "CARRAY_create_static");
+	OBJECT_REF(ref);
 	array->dim = desc->dim;
 	array->size = CLASS_sizeof_ctype(class, desc->ctype);
 	
@@ -436,7 +436,6 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 
 	TYPE type;
 	CLASS *klass;
-	uint64_t size;
 	int max_size;
 	int inc;
 	GB_INTEGER *sizes = ARG(size);
@@ -468,7 +467,7 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 
 	if (nsize <= 1)
 	{
-		size = VARGOPT(size, 0);
+		int size = VARGOPT(size, 0);
 		if (size > max_size)
 			THROW(E_MEMORY);
 		if (size < 0)
@@ -481,6 +480,8 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 		ARRAY_create_with_size(&THIS->data, THIS->size, inc);
 		if (size > 0)
 			ARRAY_add_many_void(&THIS->data, size);
+		
+		THIS->count = size;
 	}
 	else
 	{
@@ -490,7 +491,7 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 			return;
 		}
 
-		size = 1;
+		uint64_t size = 1;
 		for (i = 0; i < nsize; i++)
 		{
 			VALUE_conv_integer((VALUE *)&sizes[i]);
@@ -504,7 +505,7 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 				THROW(E_MEMORY);
 		}
 
-		ALLOC_ZERO(&THIS->dim, nsize * sizeof(int), "Array_new");
+		ALLOC_ZERO(&THIS->dim, nsize * sizeof(int));
 
 		for (i = 0; i < nsize; i++)
 			THIS->dim[i] = sizes[i].value;
@@ -512,9 +513,10 @@ BEGIN_METHOD(Array_new, GB_INTEGER size)
 
 		ARRAY_create_with_size(&THIS->data, THIS->size, 8);
 		ARRAY_add_many_void(&THIS->data, (int)size);
+		
+		THIS->count = size;
 	}
 
-	THIS->count = size;
 	
 END_METHOD
 
@@ -537,7 +539,7 @@ BEGIN_METHOD_VOID(Array_free)
 	
 	ARRAY_delete(&THIS->data);
 
-	FREE(&THIS->dim, "CARRAY_free");
+	FREE(&THIS->dim);
 
 END_METHOD
 
@@ -598,7 +600,7 @@ static bool copy_remove(CARRAY *_object, int start, int length, bool copy, bool
 		if (THIS->dim)
 		{
 			nsize = get_dim(THIS);
-			ALLOC_ZERO(&array->dim, nsize * sizeof(int), "copy_remove");
+			ALLOC_ZERO(&array->dim, nsize * sizeof(int));
 
 			for (i = 0; i < nsize; i++)
 				array->dim[i] = THIS->dim[i];
@@ -1021,6 +1023,7 @@ IMPLEMENT_find(Long, GB_LONG)
 IMPLEMENT_find(Float, GB_FLOAT)
 IMPLEMENT_find(Single, GB_SINGLE)
 IMPLEMENT_find(Date, GB_DATE)
+IMPLEMENT_find(Variant, GB_VARIANT)
 
 static int find_object(CARRAY *_object, void *value, int start, bool byref)
 {
@@ -1395,48 +1398,66 @@ END_METHOD
 
 static CARRAY *_converted_array;
 
-static void error_array_convert()
+static void error_convert()
 {
-	OBJECT_UNREF(_converted_array, "error_array_convert");
+	OBJECT_UNREF(_converted_array);
 }
 
-static void *array_convert(CARRAY *src, CLASS *class)
+static bool _convert(CARRAY *src, CLASS *class, VALUE *conv)
 {
 	CARRAY *array;
 	int i;
 	void *data;
 	VALUE temp;
+	int dim;
 	
-	if (!CLASS_inherits(class, CLASS_Array))
-		return NULL;
+	if (!src || !TYPE_is_pure_object((TYPE)class))
+		return TRUE;
+	
+	CLASS_load(class); // Force creation of array classes
+	
+	if (!class->is_array) //CLASS_inherits(class, CLASS_Array))
+		return TRUE;
 	
 	_converted_array = array = OBJECT_create(class, NULL, NULL, 0);
 	
-	ARRAY_add_many_void(&array->data, src->count);
-	array->count = src->count;
-		
-	ON_ERROR(error_array_convert)
+	if (src->count)
 	{
-		for (i = 0; i < src->count; i++)
+		ARRAY_add_many_void(&array->data, src->count);
+		array->count = src->count;
+		
+		ON_ERROR(error_convert)
 		{
-			data = CARRAY_get_data(src, i);
-			VALUE_read(&temp, data, src->type);
-			BORROW(&temp);
-			data = CARRAY_get_data(array, i);
-			VALUE_write(&temp, data, array->type);
-			RELEASE(&temp);
+			for (i = 0; i < src->count; i++)
+			{
+				data = CARRAY_get_data(src, i);
+				VALUE_read(&temp, data, src->type);
+				BORROW(&temp);
+				data = CARRAY_get_data(array, i);
+				VALUE_write(&temp, data, array->type);
+				RELEASE(&temp);
+			}
 		}
+		END_ERROR
 	}
-	END_ERROR
 	
-	return array;
+	dim = get_dim(src);
+	if (dim > 1)
+	{
+		ALLOC(&array->dim, dim * sizeof(int));
+		for (i = 0; i < dim; i++)
+			array->dim[i] = src->dim[i];
+	}
+	
+	conv->_object.object = array;
+	return FALSE;
 }
 
 #else
 
 #include "gbx_c_array.h"
 
-#define array_convert NULL
+#define _convert NULL
 
 #endif /* #ifndef GBX_INFO */
 
@@ -1484,7 +1505,7 @@ GB_DESC NATIVE_Array[] =
 	GB_METHOD("Delete", "Array", Array_Extract, "(Start)i[(Length)i]"),
 	GB_METHOD("Fill", NULL, Array_Fill, "(Value)v[(Start)i(Length)i]"),*/
 	
-	GB_INTERFACE("_convert", array_convert),
+	GB_INTERFACE("_convert", _convert),
 
 	GB_END_DECLARE
 };
@@ -1870,6 +1891,8 @@ GB_DESC NATIVE_VariantArray[] =
 	GB_METHOD("Add", NULL, Array_Variant_Add, "(Value)v[(Index)i]"),
 	GB_METHOD("Push", NULL, Array_Variant_Push, "(Value)v"),
 	GB_METHOD("_put", NULL, Array_Variant_put, "(Value)v(Index)i."),
+	GB_METHOD("Find", "i", Array_Variant_Find, "(Value)v[(Start)i]"),
+	GB_METHOD("Exist", "b", Array_Variant_Exist, "(Value)v"),
 
 	GB_METHOD("Pop", "v", Array_Pop, NULL),
 	GB_METHOD("_get", "v", Array_get, "(Index)i."),
@@ -2163,4 +2186,6 @@ void GB_ArrayRemove(GB_ARRAY array, int index)
 	copy_remove((CARRAY *)array, index, 1, FALSE, TRUE);
 }
 
+
+
 #endif
