@@ -2,7 +2,7 @@
 
   gbx_c_class.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,24 +44,29 @@
 
 static CLASS_DESC_SYMBOL *_current_symbol = NULL;
 
+static void error(int code, CLASS *class, const char *name)
+{
+	GB_Error((char *)(intptr_t)code, CLASS_get_name(class), name);
+}
+
 static CLASS_DESC *get_desc(CLASS *class, const char *name)
 {
 	int index;
 
 	index = CLASS_find_symbol(class, name);
 
-	//fprintf(stderr, "CLASS_find_symbol('%s', '%s') = %ld\n", class->name, name, index);
+	//fprintf(stderr, "CLASS_find_symbol('%s', '%s') = %ld\n", CLASS_get_name(class), name, index);
 
 	if (index == NO_SYMBOL)
 	{
-		GB_Error((char *)E_NSYMBOL, name, class->name);
+		error(E_NSYMBOL, class, name);
 		return NULL;
 	}
 	else
 		return class->table[index].desc;
 }
 
-/**** Components ***********************************************************/
+//---- Components ---------------------------------------------------------
 
 BEGIN_METHOD(Components_get, GB_STRING name)
 
@@ -129,7 +134,7 @@ BEGIN_METHOD(Component_IsLoaded, GB_STRING name)
 END_METHOD
 
 
-/**** Classes **************************************************************/
+//---- Classes ------------------------------------------------------------
 
 BEGIN_METHOD(Classes_get, GB_STRING name)
 
@@ -165,14 +170,7 @@ BEGIN_METHOD(Class_Load, GB_STRING name)
 
 END_METHOD
 
-
-BEGIN_PROPERTY(Class_Name)
-
-	GB_ReturnConstZeroString(OBJECT(CLASS)->name);
-
-END_PROPERTY
-
-
+/*
 BEGIN_METHOD_VOID(Classes_next)
 
 	TABLE *table = CLASS_get_table();
@@ -198,6 +196,39 @@ BEGIN_METHOD_VOID(Classes_next)
 	}
 
 END_METHOD
+*/
+
+BEGIN_METHOD_VOID(Classes_next)
+
+	CLASS **pcurrent = (CLASS **)GB_GetEnum();
+	CLASS *class;
+
+	class = *pcurrent;
+	
+	for(;;)
+	{
+		if (!class)
+			class = CLASS_get_list();
+		else
+			class = class->next;
+		
+		if (!class)
+		{
+			GB_StopEnum();
+			break;
+		}
+
+		if (class->state)
+		{
+			GB_ReturnObject(class);
+			break;
+		}
+		
+	}
+
+	*pcurrent = class;
+	
+END_METHOD
 
 
 BEGIN_PROPERTY(Classes_count)
@@ -206,7 +237,15 @@ BEGIN_PROPERTY(Classes_count)
 
 END_PROPERTY
 
-/**** Class ****************************************************************/
+
+//---- Class --------------------------------------------------------------
+
+BEGIN_PROPERTY(Class_Name)
+
+	GB_ReturnConstZeroString(OBJECT(CLASS)->name);
+
+END_PROPERTY
+
 
 BEGIN_PROPERTY(Class_Count)
 
@@ -214,30 +253,35 @@ BEGIN_PROPERTY(Class_Count)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Class_Hidden)
 
-	GB_ReturnBoolean(*(OBJECT(CLASS)->name) == '.');
+	GB_ReturnBoolean(*(CLASS_get_name(OBJECT(CLASS))) == '.');
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Class_Native)
 
 	GB_ReturnBoolean(CLASS_is_native(OBJECT(CLASS)));
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Class_Component)
 
 	GB_ReturnObject(OBJECT(CLASS)->component);
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Class_Parent)
 
 	GB_ReturnObject(OBJECT(CLASS)->parent);
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Class_Symbols)
 
 	CLASS *class = OBJECT(CLASS);
@@ -259,6 +303,7 @@ BEGIN_PROPERTY(Class_Symbols)
 
 END_PROPERTY
 
+
 BEGIN_METHOD(Class_get, GB_STRING name)
 
 	CLASS *class = OBJECT(CLASS);
@@ -280,12 +325,23 @@ BEGIN_METHOD(Class_get, GB_STRING name)
 END_METHOD
 
 
+BEGIN_METHOD(Class_Exist, GB_STRING name)
+
+	CLASS *class = OBJECT(CLASS);
+	const char *name = GB_ToZeroString(ARG(name));
+
+	GB_ReturnBoolean(name != NULL && CLASS_get_symbol(class, name) != NULL);
+
+END_METHOD
+
+
 BEGIN_PROPERTY(Class_Instance)
 
 	GB_ReturnObject(OBJECT(CLASS)->instance);
 
 END_PROPERTY
 
+
 BEGIN_METHOD_VOID(Class_AutoCreate)
 
 	CLASS *class = OBJECT(CLASS);
@@ -297,6 +353,7 @@ BEGIN_METHOD_VOID(Class_AutoCreate)
 
 END_METHOD
 
+
 BEGIN_METHOD(Class_New, GB_OBJECT params)
 
 	CLASS *class = OBJECT(CLASS);
@@ -319,13 +376,13 @@ BEGIN_METHOD(Class_New, GB_OBJECT params)
 	}
 
 	object = EXEC_create_object(class, np, NULL);
-	OBJECT_UNREF_KEEP(object, "Class_New");
+	OBJECT_UNREF_KEEP(object);
 	GB_ReturnObject(object);
 
 END_METHOD
 
 
-/**** Symbol ***************************************************************/
+//---- Symbol -------------------------------------------------------------
 
 BEGIN_PROPERTY(Symbol_Name)
 
@@ -425,7 +482,10 @@ BEGIN_PROPERTY(Symbol_Signature)
 	char *sign = CLASS_DESC_get_signature(cds->desc);
 
 	if (sign)
+	{
+		STRING_free_later(sign);
 		GB_ReturnString(sign);
+	}
 	else
 		GB_ReturnVoidString();
 
@@ -452,92 +512,19 @@ BEGIN_PROPERTY(Symbol_Value)
 END_PROPERTY
 
 
-/**** Object ***************************************************************/
+//---- Object -------------------------------------------------------------
 
 BEGIN_METHOD(Object_GetProperty, GB_OBJECT object; GB_STRING property)
 
 	const char *name;
-	CLASS_DESC *desc;
-	CLASS *class;
-	char type;
 	void *object = VARG(object);
 
 	if (GB_CheckObject(object))
 		return;
 
-	if (OBJECT_is_class(object))
-	{
-		class = (CLASS *)object;
-		object = NULL;
-	}
-	else
-	{
-		class = OBJECT_class(object);
-	}
-
 	name = GB_ToZeroString(ARG(property));
-	desc = get_desc(class, name);
 
-	if (!desc)
-		return;
-
-	type = CLASS_DESC_get_type(desc);
-
-	if (type == CD_PROPERTY || type == CD_PROPERTY_READ || type == CD_VARIABLE)
-	{
-		if (!object)
-		{
-			GB_Error((char *)E_DYNAMIC, class->name, name);
-			return;
-		}
-	}
-	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_PROPERTY_READ || type == CD_STATIC_VARIABLE || type == CD_CONSTANT)
-	{
-		if (object)
-		{
-			GB_Error((char *)E_STATIC, class->name, name);
-			return;
-		}
-	}
-	else
-	{
-		GB_Error((char *)E_NPROPERTY, class->name, name);
-		return;
-	}
-
-	if (type == CD_VARIABLE)
-		VALUE_read(&TEMP, (char *)object + desc->variable.offset, desc->property.type);
-	else if (type == CD_STATIC_VARIABLE)
-		VALUE_read(&TEMP, (char *)class->stat + desc->variable.offset, desc->property.type);
-	else if (type == CD_CONSTANT)
-		VALUE_read(&TEMP, (void *)&desc->constant.value, desc->constant.type);
-	else
-	{
-		if (desc->property.native)
-		{
-			if (EXEC_call_native(desc->property.read, object, desc->property.type, 0))
-			{
-				EXEC_set_native_error(TRUE);
-				return;
-			}
-		}
-		else
-		{
-			EXEC.class = desc->property.class;
-			EXEC.object = object;
-			EXEC.nparam = 0;
-			EXEC.native = FALSE;
-			EXEC.index = (int)(intptr_t)desc->property.read;
-			//EXEC.func = &class->load->func[(long)desc->property.read];
-	
-			EXEC_function_keep();
-	
-			TEMP = *RP;
-			UNBORROW(RP);
-			RP->type = T_VOID;
-		}
-	}
-	
+	GB_GetProperty(object, name);
 	GB_ReturnConvVariant();
 
 END_METHOD
@@ -579,7 +566,7 @@ BEGIN_METHOD(Object_SetProperty, GB_OBJECT object; GB_STRING property; GB_VARIAN
 		{
 			if (!class->auto_create)
 			{
-				GB_Error((char *)E_DYNAMIC, class->name, name);
+				error(E_DYNAMIC, class, name);
 				return;
 			}
 			
@@ -590,18 +577,18 @@ BEGIN_METHOD(Object_SetProperty, GB_OBJECT object; GB_STRING property; GB_VARIAN
 	{
 		if (object)
 		{
-			GB_Error((char *)E_STATIC, class->name, name);
+			error(E_STATIC, class, name);
 			return;
 		}
 	}
 	else if (type == CD_PROPERTY_READ  || type == CD_STATIC_PROPERTY_READ)
 	{
-		GB_Error((char *)E_NWRITE, class->name, name);
+		error(E_NWRITE, class, name);
 		return;
 	}
 	else
 	{
-		GB_Error((char *)E_NPROPERTY, class->name, name);
+		error(E_NPROPERTY, class, name);
 		return;
 	}
 
@@ -642,6 +629,7 @@ BEGIN_METHOD(Object_SetProperty, GB_OBJECT object; GB_STRING property; GB_VARIAN
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Attach, GB_OBJECT object; GB_OBJECT parent; GB_STRING name)
 
 	void *object = VARG(object);
@@ -667,6 +655,7 @@ BEGIN_METHOD(Object_Attach, GB_OBJECT object; GB_OBJECT parent; GB_STRING name)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Detach, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -681,6 +670,7 @@ BEGIN_METHOD(Object_Detach, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Parent, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -692,6 +682,7 @@ BEGIN_METHOD(Object_Parent, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Class, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -703,6 +694,7 @@ BEGIN_METHOD(Object_Class, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Type, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -714,6 +706,7 @@ BEGIN_METHOD(Object_Type, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Is, GB_OBJECT object; GB_STRING class)
 
 	void *object = VARG(object);
@@ -732,6 +725,7 @@ BEGIN_METHOD(Object_Is, GB_OBJECT object; GB_STRING class)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 
 	int i;
@@ -746,7 +740,7 @@ BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 	
 	if (GB_GetFunction(&func, object, name, NULL, NULL))
 	{
-		GB_Error((char *)E_NSYMBOL, name, OBJECT_class(object)->name);
+		error(E_NSYMBOL, OBJECT_class(object), name);
 		return;
 	}
 
@@ -771,12 +765,14 @@ BEGIN_METHOD(Object_Call, GB_OBJECT object; GB_STRING method; GB_OBJECT params)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_IsValid, GB_OBJECT object)
 
 	GB_ReturnBoolean(OBJECT_is_valid(VARG(object)));
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Lock, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -788,6 +784,7 @@ BEGIN_METHOD(Object_Lock, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Unlock, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -799,6 +796,7 @@ BEGIN_METHOD(Object_Unlock, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_IsLocked, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -810,6 +808,7 @@ BEGIN_METHOD(Object_IsLocked, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_Count, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -822,6 +821,7 @@ BEGIN_METHOD(Object_Count, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_SizeOf, GB_OBJECT object)
 
 	void *object = VARG(object);
@@ -833,6 +833,7 @@ BEGIN_METHOD(Object_SizeOf, GB_OBJECT object)
 
 END_METHOD
 
+
 BEGIN_METHOD(Object_New, GB_STRING class; GB_OBJECT params)
 
 	CLASS *class = CLASS_find(GB_ToZeroString(ARG(class)));
@@ -861,21 +862,47 @@ BEGIN_METHOD(Object_New, GB_STRING class; GB_OBJECT params)
 	}
 
 	object = EXEC_create_object(class, np, NULL);
-	OBJECT_UNREF_KEEP(object, "Object_New");
+	OBJECT_UNREF_KEEP(object);
 	GB_ReturnObject(object);
 
 END_METHOD
 
+
 BEGIN_PROPERTY(Object_Address)
 
 	GB_ReturnPointer(VPROP(GB_OBJECT));
 
 END_PROPERTY
 
+
+BEGIN_METHOD(Object_CanRaise, GB_OBJECT object; GB_STRING name)
+
+	void *object = VARG(object);
+	CLASS *class;
+	char *name;
+	int index;
+
+	if (!object)
+	{
+		GB_Error((char *)E_NULL);
+		return;
+	}
+
+	class = OBJECT_class(object);
+	name = GB_ToZeroString(ARG(name));
+	index = GB_GetEvent(class, name);
+	//fprintf(stderr, "Object.CanRaise: %s %d\n", name, index);
+	if (index < 0)
+		GB_ReturnBoolean(FALSE);
+	else
+		GB_ReturnBoolean(GB_CanRaise(object, index));
+	
+END_METHOD
+
 #endif
 
 
-/***************************************************************************/
+//-------------------------------------------------------------------------
 
 GB_DESC NATIVE_Symbol[] =
 {
@@ -935,7 +962,7 @@ GB_DESC NATIVE_Class[] =
 
 	GB_STATIC_METHOD("Load", "Class", Class_Load, "(Name)s"),
 
-	GB_METHOD("_get", ".Symbol", Class_get, "(Name)s"),
+	GB_METHOD("_get", ".Symbol", Class_get, "(Symbol)s"),
 
 	GB_PROPERTY_READ("Name", "s", Class_Name),
 	GB_PROPERTY_READ("Hidden", "b", Class_Hidden),
@@ -947,6 +974,7 @@ GB_DESC NATIVE_Class[] =
 	GB_PROPERTY_READ("Symbols", "String[]", Class_Symbols),
 	GB_METHOD("AutoCreate", "o", Class_AutoCreate, NULL),
 	GB_METHOD("New", "o", Class_New, "[(Arguments)Array;]"),
+	GB_METHOD("Exist", "b", Class_Exist, "(Symbol)s"),
 
 	GB_CONSTANT("Variable", "i", 1),
 	GB_CONSTANT("Property", "i", 2),
@@ -978,6 +1006,7 @@ GB_DESC NATIVE_Object[] =
 	GB_STATIC_METHOD("Count", "i", Object_Count, "(Object)o"),
 	GB_STATIC_METHOD("SizeOf", "i", Object_SizeOf, "(Object)o"),
 	GB_STATIC_METHOD("Address", "p", Object_Address, "(Object)o"),
+	GB_STATIC_METHOD("CanRaise", "b", Object_CanRaise, "(Object)o(Event)s"),
 
 	GB_END_DECLARE
 };
