@@ -2,7 +2,7 @@
 
   gbx_c_file.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ CFILE *CFILE_create(STREAM *stream, int mode)
 	CFILE *file;
 
 	file = OBJECT_new(CLASS_File, NULL, NULL);
-	OBJECT_UNREF_KEEP(file, "CFILE_new");
+	OBJECT_UNREF_KEEP(file);
 
 	if (stream)
 	{
@@ -114,7 +114,7 @@ static CFILE *create_default_stream(FILE *file, int mode)
 	stream.buffer.file = file;
 	STREAM_check_blocking(&stream);
 	ob = CFILE_create(&stream, mode);
-	OBJECT_REF(ob, "create_default_stream");
+	OBJECT_REF(ob);
 	return ob;
 }
 
@@ -127,9 +127,9 @@ void CFILE_init(void)
 
 void CFILE_exit(void)
 {
-	OBJECT_UNREF(CFILE_in, "CFILE_exit");
-	OBJECT_UNREF(CFILE_out, "CFILE_exit");
-	OBJECT_UNREF(CFILE_err, "CFILE_exit");
+	OBJECT_UNREF(CFILE_in);
+	OBJECT_UNREF(CFILE_out);
+	OBJECT_UNREF(CFILE_err);
 }
 
 void CFILE_init_watch(void)
@@ -263,7 +263,7 @@ END_PROPERTY
 static char *get_file_user(CFILE *_object)
 {
 	struct passwd *pwd;
-	uid_t uid = THIS_STAT->info.uid;
+	uid_t uid = (uid_t)THIS_STAT->info.uid;
 
 	if (uid == 0)
 		return "root";
@@ -290,7 +290,7 @@ END_PROPERTY
 static char *get_file_group(CFILE *_object)
 {
 	struct group *grp;
-	gid_t gid = THIS_STAT->info.gid;
+	gid_t gid = (gid_t)THIS_STAT->info.gid;
 
 	if (gid == 0)
 		return "root";
@@ -487,7 +487,7 @@ static void return_path(void)
 			tmp = STRING_add_char(tmp, '/');
 	}
 	
-	if (*_basename)
+	if (_basename && *_basename)
 		tmp = STRING_add(tmp, _basename, 0);
 	
 	if (*_ext)
@@ -628,9 +628,10 @@ BEGIN_METHOD(File_Load, GB_STRING path)
 		{
 			rlen = len;
 
-			str = STRING_new_temp(NULL, rlen);
-
-			STREAM_read(&stream, str, rlen);
+			str = STRING_new(NULL, rlen);
+			rlen = STREAM_read_max(&stream, str, rlen);
+			str = STRING_extend(str, rlen);
+			STRING_free_later(str);
 		}
 		
 		STREAM_close(&stream);
@@ -731,6 +732,7 @@ END_METHOD
 
 BEGIN_METHOD_VOID(Stream_free)
 
+	STREAM_close(CSTREAM_stream(THIS_STREAM));
 	GB_StoreVariant(NULL, POINTER(&(THIS_STREAM->tag)));
 
 END_METHOD
@@ -740,9 +742,14 @@ BEGIN_METHOD(Stream_ReadLine, GB_STRING escape)
 	char *escape;
 	char *str;
 	
-	escape = GB_ToZeroString(ARG(escape));
-	if (!*escape)
+	if (MISSING(escape))
 		escape = NULL;
+	else
+	{
+		escape = GB_ToZeroString(ARG(escape));
+		if (!*escape)
+			escape = NULL;
+	}
 
 	str = STREAM_line_input(CSTREAM_stream(THIS_STREAM), escape);
 	STRING_free_later(str);
