@@ -2,7 +2,7 @@
 
   gbx_c_process.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,8 +71,11 @@ DECLARE_EVENT(EVENT_Read);
 DECLARE_EVENT(EVENT_Error);
 DECLARE_EVENT(EVENT_Kill);
 
-static CPROCESS *RunningProcessList = NULL;
-static int _pipe_child[2];
+static CPROCESS *_running_process_list = NULL;
+static int _running_process = 0;
+static int _ignore_process = 0;
+
+static SIGNAL_CALLBACK *_SIGCHLD_callback;
 static bool _init = FALSE;
 
 static int _last_status = 0;
@@ -80,8 +83,6 @@ static int _last_status = 0;
 static int _last_child_error = 0;
 static int _last_child_error_errno = 0;
 
-static SIGNAL_HANDLER _SIGCHLD_handler;
-
 static void init_child(void);
 static void exit_child(void);
 
@@ -100,9 +101,57 @@ static const char *const _child_error[] = {
 	"cannot initialize pseudo-terminal: ",
 	"cannot plug standard input: ",
 	"cannot plug standard output and standard error: ",
-	"cannot exec program: "
+	"cannot run executable: "
 };
 
+//-------------------------------------------------------------------------
+
+static void close_fd(int *pfd)
+{
+	int fd = *pfd;
+	
+	if (fd >= 0)
+	{
+		#ifdef DEBUG_ME
+		fprintf(stderr, "unwatch & close: %d\n", fd);
+		#endif
+		GB_Watch(fd, GB_WATCH_NONE, NULL, 0);
+		close(fd);
+		*pfd = -1;
+	}
+}
+
+static void add_process_to_running_list(CPROCESS *process)
+{
+	if (_running_process_list)
+		_running_process_list->prev = process;
+
+	process->next = _running_process_list;
+	process->prev = NULL;
+
+	_running_process_list = process;
+
+	process->running = TRUE;
+	_running_process++;
+}
+
+static void remove_process_from_running_list(CPROCESS *process)
+{
+	if (process->prev)
+		process->prev->next = process->next;
+
+	if (process->next)
+		process->next->prev = process->prev;
+
+	if (process == _running_process_list)
+		_running_process_list = process->next;
+
+	process->running = FALSE;
+	_running_process--;
+	if (process->ignore)
+		_ignore_process--;
+}
+
 static void callback_write(int fd, int type, CPROCESS *process)
 {
 	#ifdef DEBUG_ME
@@ -113,17 +162,16 @@ static void callback_write(int fd, int type, CPROCESS *process)
 	{
 		int n = read(fd, COMMON_buffer, 256);
 		if (n > 0)
-		{
 			process->result = STRING_add(process->result, COMMON_buffer, n);
-			CSTREAM_stream(process)->process.read_something = TRUE;
-		}
 	}
-	else if (!STREAM_is_closed(CSTREAM_stream(process)) && !STREAM_eof(CSTREAM_stream(process))) //process->running &&
+	else if (GB_CanRaise(process, EVENT_Read) && !STREAM_is_closed(CSTREAM_stream(process)) && !STREAM_eof(CSTREAM_stream(process))) //process->running &&
 		GB_Raise(process, EVENT_Read, 0);
+	else
+		close_fd(&process->out);
 }
 
 
-static int callback_error(int fd, int type, CPROCESS *process)
+static bool callback_error(int fd, int type, CPROCESS *process)
 {
 	char buffer[256];
 	int n;
@@ -142,7 +190,10 @@ static int callback_error(int fd, int type, CPROCESS *process)
 
 	/*printf("callback_error: (%d) %.*s\n", n, n, buffer);*/
 
-	GB_Raise(process, EVENT_Error, 1, GB_T_STRING, buffer, n);
+	if (GB_CanRaise(process, EVENT_Error))
+		GB_Raise(process, EVENT_Error, 1, GB_T_STRING, buffer, n);
+	else
+		close_fd(&process->err);
 
 	return FALSE;
 
@@ -179,22 +230,9 @@ static void exit_process(CPROCESS *_object)
 		THIS->in = -1;
 	}
 
-	if (THIS->out >= 0)
-	{
-		GB_Watch(THIS->out, GB_WATCH_NONE, NULL, 0);
-		close(THIS->out);
-		THIS->out = -1;
-	}
+	close_fd(&THIS->out);
+	close_fd(&THIS->err);
 
-	if (THIS->err >= 0)
-	{
-		GB_Watch(THIS->err, GB_WATCH_NONE, NULL, 0);
-		close(THIS->err);
-		THIS->err = -1;
-	}
-
-	// Why that ? This should be done only at process creation
-	// update_stream(THIS);
 	STREAM_close(&THIS->ob.stream);
 }
 
@@ -257,7 +295,8 @@ static void throw_last_child_error()
 static void stop_process_after(CPROCESS *_object)
 {
 	STREAM *stream;
-	//long long len;
+	int64_t len, len2;
+	bool do_exit_process = FALSE;
 
 	#ifdef DEBUG_ME
 	fprintf(stderr, "stop_process_after: %p\n", _object);
@@ -267,30 +306,44 @@ static void stop_process_after(CPROCESS *_object)
 	{
 		prepare_child_error(THIS);
 		exit_process(THIS);
-		OBJECT_detach((OBJECT *)THIS);
+		//OBJECT_detach((OBJECT *)THIS);
 	}
 
 	/* Vidage du tampon d'erreur */
 	if (THIS->err >= 0)
+	{
 		while (callback_error(THIS->err, 0, THIS) == 0);
+		do_exit_process = TRUE;
+	}
 
 	/* Vidage du tampon de sortie */
 	if (THIS->out >= 0)
 	{
 		stream = CSTREAM_stream(THIS);
-		while (!STREAM_eof(stream))
+		if (!STREAM_is_closed(stream))
 		{
-			stream->process.read_something = FALSE;
-			callback_write(THIS->out, 0, THIS);
-			//GB_Raise(THIS, EVENT_Read, 0);
-			if (!stream->process.read_something)
-				break;
+			while (!STREAM_eof(stream))
+			{
+				STREAM_lof(stream, &len);
+				callback_write(THIS->out, 0, THIS);
+				if (STREAM_is_closed(stream))
+					break;
+				STREAM_lof(stream, &len2);
+				if (len == len2)
+					break;
+			}
 		}
+
+		do_exit_process = TRUE;
 	}
 
-	GB_Raise(THIS, EVENT_Kill, 0);
+	if (do_exit_process)
+		exit_process(THIS);
 
-	exit_process(THIS);
+	#ifdef DEBUG_ME
+	fprintf(stderr, "Raising Kill event for %p: parent = %p  can raise = %d\n", THIS, OBJECT_parent(THIS), GB_CanRaise(THIS, EVENT_Kill));
+	#endif
+	GB_Raise(THIS, EVENT_Kill, 0);
 
 	OBJECT_detach((OBJECT *)THIS);
 
@@ -310,24 +363,13 @@ static void stop_process(CPROCESS *process)
 
 	/* Remove from running process list */
 
-	if (process->prev)
-		process->prev->next = process->next;
-
-	if (process->next)
-		process->next->prev = process->prev;
-
-	if (process == RunningProcessList)
-		RunningProcessList = process->next;
-
-	process->running = FALSE;
-
-	/*printf("** stop_process\n");*/
-
+	remove_process_from_running_list(process);
 	stop_process_after(process);
 
-	OBJECT_UNREF(process, "stop_process"); /* Le processus ne tourne plus */
+	OBJECT_UNREF(process);
 
-	if (!RunningProcessList)
+	//if (!_running_process_list)
+	if (_running_process <= _ignore_process)
 		exit_child();
 }
 
@@ -342,6 +384,10 @@ static void abort_child(int error)
 	
 	save_errno = errno;
 	
+	#ifdef DEBUG_ME
+	fprintf(stderr, "abort_child: %d %d\n", error, save_errno);
+	#endif
+	
 	snprintf(path, sizeof(path), FILE_TEMP_DIR "/%d.child", (int)getuid(), (int)getppid(), (int)getpid());
 	
 	fd = open(path, O_CREAT | O_WRONLY, 0600);
@@ -397,8 +443,10 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	CARRAY *array;
 	int i, n;
 	sigset_t sig, old;
+	bool pwd;
+	int save_errno;
 
-	/* for terminal */
+	// for virtual terminal
 	int fd_master = -1;
 	int fd_slave;
 	char *slave = NULL;
@@ -406,8 +454,6 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	//struct termios termios_check;
 	struct termios termios_master;
 
-	init_child();
-
 	if (mode & PM_SHELL)
 	{
 		#ifdef DEBUG_ME
@@ -434,7 +480,7 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		if (n == 0)
 			return;
 
-		ALLOC(&argv, sizeof(*argv) * (n + 1), "run_process");
+		ALLOC(&argv, sizeof(*argv) * (n + 1));
 		memcpy(argv, array->data, sizeof(*argv) * n);
 		argv[n] = NULL;
 
@@ -444,6 +490,12 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 				argv[i] = "";
 		}
 
+		if (*argv[0] == '/' && !FILE_exist(argv[0]))
+		{
+			FREE(&argv);
+			THROW(E_NEXIST);
+		}
+		
 		#ifdef DEBUG_ME
 		{
 			int i;
@@ -456,20 +508,6 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		#endif
 	}
 
-	// Adding to the running process list
-
-	OBJECT_REF(process, "run_process");
-
-	if (RunningProcessList)
-		RunningProcessList->prev = process;
-
-	process->next = RunningProcessList;
-	process->prev = NULL;
-
-	RunningProcessList = process;
-
-	process->running = TRUE;
-
 	if (mode & PM_STRING)
 	{
 		process->to_string = TRUE;
@@ -480,12 +518,12 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	if (mode & PM_TERM)
 	{
 		#ifdef OS_OPENBSD
-		if (openpty(&fd_master, &fd_slave, NULL, NULL, NULL)<0)
-			THROW_SYSTEM(errno, NULL);
+		if (openpty(&fd_master, &fd_slave, NULL, NULL, NULL) < 0)
+			goto __ABORT_ERRNO;
 		#else
 		fd_master = posix_openpt(O_RDWR | O_NOCTTY);
 		if (fd_master < 0)
-			THROW_SYSTEM(errno, NULL);
+			goto __ABORT_ERRNO;
 
 		grantpt(fd_master);
 		unlockpt(fd_master);
@@ -500,13 +538,22 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		/* Create pipes */
 
 		if ((mode & PM_WRITE) && pipe(fdin) != 0)
-			THROW_SYSTEM(errno, NULL);
+			goto __ABORT_ERRNO;
 
 		if ((mode & PM_READ) && (pipe(fdout) != 0 || pipe(fderr) != 0))
-			THROW_SYSTEM(errno, NULL);
+			goto __ABORT_ERRNO;
 	}
 
-	/* Block SIGCHLD */
+	// Adding to the running process list
+
+	add_process_to_running_list(process);
+	OBJECT_REF(process);
+	
+	// Start the SIGCHLD callback
+	
+	init_child();
+
+	// Block SIGCHLD and fork
 
 	sigemptyset(&sig);
 
@@ -518,9 +565,11 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 	{
 		stop_process(process);
 		sigprocmask(SIG_SETMASK, &old, &sig);
-		THROW_SYSTEM(errno, NULL);
+		goto __ABORT_ERRNO;
 	}
 
+	// parent process
+	
 	if (pid)
 	{
 		process->pid = pid;
@@ -532,13 +581,13 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 		if (mode & PM_TERM)
 		{
 			if (tcgetattr(fd_master, &termios_master))
-				THROW_SYSTEM(errno, NULL);
+				goto __ABORT_ERRNO;
 				
 			cfmakeraw(&termios_master);
 			//termios_master.c_lflag &= ~ECHO;
 
 			if (tcsetattr(fd_master, TCSANOW, &termios_master))
-				THROW_SYSTEM(errno, NULL);
+				goto __ABORT_ERRNO;
 		}
 
 		if (mode & PM_WRITE)
@@ -570,6 +619,9 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 				process->err = fderr[0];
 			}
 
+			#ifdef DEBUG_ME
+			fprintf(stderr, "watch: out = %d err = %d\n", process->out, process->err);
+			#endif
 			GB_Watch(process->out, GB_WATCH_READ, (void *)callback_write, (intptr_t)process);
 			if (process->err >= 0)
 			{
@@ -580,12 +632,15 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 
 		if ((mode & PM_SHELL) == 0)
 		{
-			FREE(&argv, "run_process");
+			FREE(&argv);
 		}
 
 		sigprocmask(SIG_SETMASK, &old, &sig);
 	}
-	else /* child */
+	
+	// child process
+	
+	else
 	{
 		//bool stdin_isatty = isatty(STDIN_FILENO);
 		
@@ -658,8 +713,7 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 			}
 		}
 
-		// Return to the parent working directory
-		FILE_chdir(PROJECT_oldcwd);
+		pwd = FALSE;
 		
 		if (env)
 		{
@@ -670,26 +724,38 @@ static void run_process(CPROCESS *process, int mode, void *cmd, CARRAY *env)
 				str = ((char **)env->data)[i];
 				if (putenv(str))
 					ERROR_warning("cannot set environment string: %s", str);
+				if (strncmp(str, "PWD=", 4) == 0 && chdir(&str[4]) == 0)
+					pwd = TRUE;
 			}
 		}
 		
+		// Return to the parent working directory if the PWD environment variable has not been set
+		if (!pwd)
+			FILE_chdir(PROJECT_oldcwd);
+		
 		execvp(argv[0], (char **)argv);
 		//execve(argv[0], (char **)argv, environ);
 		abort_child(CHILD_CANNOT_EXEC);
 	}
 
 	update_stream(process);
+	return;
+	
+__ABORT_ERRNO:
+
+	save_errno = errno;
+	stop_process(process);
+	THROW_SYSTEM(save_errno, NULL);
 }
 
 
-static void callback_child(int fd, int type, void *data)
+static void callback_child(int signum, intptr_t data)
 {
 	int status;
 	CPROCESS *process, *next;
-	int buffer;
-
-	/*old = signal(SIGCHLD, signal_child);*/
+	//int buffer;
 
+#if 0
 	for(;;)
 	{
 		if (read(fd, (char *)&buffer, 1) == 1)
@@ -697,12 +763,12 @@ static void callback_child(int fd, int type, void *data)
 		if (errno != EINTR)
 			ERROR_panic("Cannot read from SIGCHLD pipe: %s", strerror(errno));
 	}
-
+#endif
 	#ifdef DEBUG_ME
-	fprintf(stderr, "<< callback_child\n");
+	fprintf(stderr, ">> callback_child\n");
 	#endif
 
-	for (process = RunningProcessList; process; )
+	for (process = _running_process_list; process; )
 	{
 		next = process->next;
 
@@ -728,42 +794,10 @@ static void callback_child(int fd, int type, void *data)
 	throw_last_child_error();
 	
 	#ifdef DEBUG_ME
-	fprintf(stderr, ">> callback_child\n");
+	fprintf(stderr, "<< callback_child\n");
 	#endif
 }
 
-
-static void signal_child(int sig, siginfo_t *info, void *context)
-{
-	#ifdef DEBUG_ME
-	fprintf(stderr, "SIGCHLD: _pipe_child[0] = %d\n", _pipe_child[0]);
-	#endif
-	char buffer;
-	int save_errno;
-
-	save_errno = errno;
-			
-	if (_init)
-	{
-		//fprintf(stderr, "SIGCHLD\n");
-		
-		buffer = 42;
-		for(;;)
-		{
-			if (write(_pipe_child[1], &buffer, 1) == 1)
-				break;
-			
-			if (errno != EINTR)
-				ERROR_panic("Cannot write into SIGCHLD pipe: %s", strerror(errno));
-		}
-	}
-	
-	SIGNAL_previous(&_SIGCHLD_handler, sig, info, context);
-	
-	errno = save_errno;
-}
-
-
 static void init_child(void)
 {
 	if (_init)
@@ -773,19 +807,8 @@ static void init_child(void)
 	fprintf(stderr, "init_child()\n");
 	#endif
 
-	if (pipe(_pipe_child) != 0)
-		ERROR_panic("Cannot create SIGCHLD pipes: %s", strerror(errno));
-
-	fcntl(_pipe_child[0], F_SETFD, FD_CLOEXEC);
-	fcntl(_pipe_child[1], F_SETFD, FD_CLOEXEC);
-
-	#ifdef DEBUG_ME
-	fprintf(stderr, "_pipe_child[0] = %d\n", _pipe_child[0]);
-	#endif
-
-	SIGNAL_install(&_SIGCHLD_handler, SIGCHLD, signal_child);
+	_SIGCHLD_callback = SIGNAL_register(SIGCHLD, callback_child, 0);
 	
-	GB_Watch(_pipe_child[0], GB_WATCH_READ, (void *)callback_child, 0);
 	_init = TRUE;
 }
 
@@ -798,15 +821,19 @@ static void exit_child(void)
 	fprintf(stderr, "exit_child()\n");
 	#endif
 
-	GB_Watch(_pipe_child[0], GB_WATCH_NONE, NULL, 0);
-	close(_pipe_child[0]);
-	close(_pipe_child[1]);
-	
-	SIGNAL_uninstall(&_SIGCHLD_handler);
+	SIGNAL_unregister(SIGCHLD, _SIGCHLD_callback);
+
 	_init = FALSE;
 }
 
 
+static CPROCESS *_CPROCESS_create_process;
+
+static void error_CPROCESS_create()
+{
+	OBJECT_UNREF(_CPROCESS_create_process);
+}
+
 CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env)
 {
 	CPROCESS *process;
@@ -816,12 +843,16 @@ CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env)
 	//if (!name || !*name)
 	//	name = "Process";
 
-	process = OBJECT_new(CLASS_Process, name, OP  ? (OBJECT *)OP : (OBJECT *)CP);
+	_CPROCESS_create_process = process = OBJECT_new(CLASS_Process, name, OP  ? (OBJECT *)OP : (OBJECT *)CP);
 
-	init_process(process);
-	run_process(process, mode, cmd, env);
+	ON_ERROR(error_CPROCESS_create)
+	{
+		init_process(process);
+		run_process(process, mode, cmd, env);
+	}
+	END_ERROR
 
-	OBJECT_UNREF_KEEP(process, "CPROCESS_create");
+	OBJECT_UNREF_KEEP(process);
 
 	if (!name || !*name)
 		STREAM_blocking(CSTREAM_stream(process), TRUE);
@@ -831,44 +862,46 @@ CPROCESS *CPROCESS_create(int mode, void *cmd, char *name, CARRAY *env)
 	return process;
 }
 
-static CPROCESS *_error_CPROCESS_wait_for_process;
-
-static void error_CPROCESS_wait_for()
+static void error_CPROCESS_wait_for(CPROCESS *process)
 {
-	OBJECT_UNREF(_error_CPROCESS_wait_for_process, "CPROCESS_wait_for");
+	OBJECT_UNREF(process);
 }
 
-void CPROCESS_wait_for(CPROCESS *process)
+void CPROCESS_wait_for(CPROCESS *process, int timeout)
 {
 	int ret;
+	int sigfd;
 
 	#ifdef DEBUG_ME
-	printf("Waiting for %d\n", process->pid);
+	fprintf(stderr, "Waiting for %d\n", process->pid);
 	#endif
 
-	OBJECT_REF(process, "CPROCESS_wait_for");
-	_error_CPROCESS_wait_for_process = process;
+	OBJECT_REF(process);
 	
-	ON_ERROR(error_CPROCESS_wait_for)
+	sigfd = SIGNAL_get_fd();
+
+	ON_ERROR_1(error_CPROCESS_wait_for, process)
 	{
 		while (process->running)
 		{
-			//HOOK_DEFAULT(wait, WATCH_wait)(0);
-			#ifdef DEBUG_ME
-			fprintf(stderr, "watching _pipe_child[0] = %d\n", _pipe_child[0]);
-			#endif
-			ret = WATCH_process(_pipe_child[0], process->out);
-			if (ret == _pipe_child[0])
-				callback_child(_pipe_child[0], GB_WATCH_READ, 0);
-			else if (ret == process->out)
+			ret = WATCH_process(sigfd, process->out, timeout);
+
+			if (ret & WP_OUTPUT)
 				callback_write(process->out, GB_WATCH_READ, process);
-			else
+
+			if (ret & WP_END)
+				SIGNAL_raise_callbacks(sigfd, GB_WATCH_READ, 0);
+
+			if (ret & WP_TIMEOUT)
+				break;
+
+			if (ret == 0)
 				usleep(1000);
 		}
 	}
 	END_ERROR
 	
-	OBJECT_UNREF(process, "CPROCESS_wait_for");
+	OBJECT_UNREF(process);
 
 	#if 0
 	{
@@ -889,7 +922,7 @@ void CPROCESS_wait_for(CPROCESS *process)
 	sigprocmask(SIG_SETMASK, &old, NULL);*/
 
 	#ifdef DEBUG_ME
-	printf("Got it !\n");
+	fprintf(stderr, "Waiting for: got it !\n");
 	#endif
 }
 
@@ -899,10 +932,12 @@ BEGIN_METHOD_VOID(Process_exit)
 
 	//fprintf(stderr, "Process_exit\n");
 
-	while (RunningProcessList)
-		stop_process(RunningProcessList);
+	while (_running_process_list)
+		stop_process(_running_process_list);
 
 	exit_child();
+	
+	STRING_free(&CPROCESS_shell);
 
 END_METHOD
 
@@ -917,7 +952,7 @@ END_METHOD
 BEGIN_METHOD_VOID(Process_free)
 
 	#ifdef DEBUG_ME
-	printf("Process_free %p\n", THIS);
+	fprintf(stderr, "Process_free %p\n", THIS);
 	#endif
 
 	exit_process(THIS);
@@ -942,7 +977,7 @@ BEGIN_METHOD_VOID(Process_Kill)
 	else
 		kill(THIS->pid, SIGKILL);
 	
-	CPROCESS_wait_for(THIS);
+	//CPROCESS_wait_for(THIS);
 
 END_METHOD
 
@@ -1031,9 +1066,36 @@ BEGIN_PROPERTY(Process_LastValue)
 
 END_PROPERTY
 
-BEGIN_METHOD_VOID(Process_Wait)
+BEGIN_METHOD(Process_Wait, GB_FLOAT timeout)
 
-	CPROCESS_wait_for(THIS);
+	CPROCESS_wait_for(THIS, (int)(VARGOPT(timeout, 0.0) * 1000));
+
+END_METHOD
+
+BEGIN_PROPERTY(Process_Ignore)
+
+	if (READ_PROPERTY)
+		GB_ReturnBoolean(THIS->ignore);
+	else
+	{
+		bool ignore = VPROP(GB_BOOLEAN);
+		if (THIS->ignore != ignore)
+		{
+			THIS->ignore = ignore;
+			if (ignore)
+			{
+				_ignore_process++;
+				if (_running_process <= _ignore_process)
+					exit_child();
+			}
+			else
+			{
+				_ignore_process--;
+				if (_running_process > _ignore_process)
+					init_child();
+			}
+		}
+	}
 
 END_METHOD
 
@@ -1063,10 +1125,9 @@ GB_DESC NATIVE_Process[] =
 
 	GB_METHOD("Kill", NULL, Process_Kill, NULL),
 	GB_METHOD("Signal", NULL, Process_Signal, NULL),
-	GB_METHOD("Wait", NULL, Process_Wait, NULL),
+	GB_METHOD("Wait", NULL, Process_Wait, "[(Timeout)f]"),
 
-	//GB_METHOD("Send", NULL, CPROCESS_send, "(Input)s"),
-	//GB_METHOD("_print", NULL, CPROCESS_send, "(Input)s"),
+	GB_PROPERTY("Ignore", "b", Process_Ignore),
 
 	GB_EVENT("Read", NULL, NULL, &EVENT_Read),
 	GB_EVENT("Error", NULL, "(Error)s", &EVENT_Error),
