@@ -2,7 +2,7 @@
 
   gbx_c_string.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -68,8 +68,9 @@ int STRING_get_utf8_char_length(unsigned char c)
 	return utf8_get_char_length(c);
 }
 
-static int utf8_get_length(const char *str, int len)
+static int utf8_get_length(const char *sstr, int len)
 {
+	const uchar *str = (const uchar *)sstr;
 	int ulen;
 	int i;
 
@@ -84,8 +85,9 @@ static int utf8_get_length(const char *str, int len)
 	return ulen;
 }
 
-static uint utf8_to_unicode(const char *str, int len)
+static uint utf8_to_unicode(const char *sstr, int len)
 {
+	const uchar *str = (const uchar *)sstr;
 	uint unicode;
 	
 	switch (len)
@@ -132,8 +134,10 @@ _INVALID:
 	return UNICODE_INVALID;
 }
 
-static void utf8_from_unicode(uint code, char *str)
+static void utf8_from_unicode(uint code, char *sstr)
 {
+	uchar *str = (uchar *)sstr;
+	
 	if (code < 0x80)
 		str[0] = code;
 	else if (code < 0x800)
@@ -178,49 +182,132 @@ static void utf8_from_unicode(uint code, char *str)
 /***************************************************************************/
 
 char *STRING_utf8_current = NULL;
-#define UTF8_POS_COUNT 256
-static short _utf8_pos[UTF8_POS_COUNT];
-static int _utf8_last_pos;
 
-static int utf8_get_pos(const char *str, int len, int index)
+#define UTF8_MAX_COUNT 256
+#define UTF8_MAX_CACHE 64
+
+struct {
+	ushort pos[UTF8_MAX_COUNT];
+	ushort last_pos;
+	ushort cnext;
+	int lpos;
+	int lindex;
+	int cindex[UTF8_MAX_CACHE];
+	int cpos[UTF8_MAX_CACHE];
+	}
+_utf8 = { { 0 } };
+
+static int utf8_get_pos(const char *ref, const char *sstr, int len, int index)
 {
-	int i, pos;
+	const uchar *str = (const uchar *)sstr;
+	int i, j, pos;
+	int min_index, min_i;
 	
 	if (index <= 0)
 		return 0;
 	
-	if (str != STRING_utf8_current)
+	//fprintf(stderr, "utf8_get_pos: %p %d %d\n", sstr, len, index);
+	
+	if (ref != STRING_utf8_current || sstr != STRING_utf8_current)
 	{
-		STRING_utf8_current = (char *)str;
-		_utf8_last_pos = 0;
-		_utf8_pos[0] = 0;
+		//fprintf(stderr, "STRING_utf8_current = %p\n", STRING_utf8_current);
+		STRING_utf8_current = (char *)ref;
+		CLEAR(&_utf8);
 	}
 	
-	if (index > len)
-		index = len;
+	if (index < UTF8_MAX_COUNT)
+	{
+		if (index <= _utf8.last_pos)
+			return _utf8.pos[index];
+		
+		pos = _utf8.pos[_utf8.last_pos];
+		
+		for(;;)
+		{
+			if (pos >= len)
+				return len;
+
+			pos += utf8_get_char_length(str[pos]);
+			_utf8.pos[++_utf8.last_pos] = pos;
+
+			if (index == _utf8.last_pos)
+				return pos;
+		}
+	}
 	
-	if (index <= _utf8_last_pos)
-		return _utf8_pos[index];
+	//fprintf(stderr, "index = %d\n", index);
 	
-	pos = _utf8_pos[_utf8_last_pos];
+	if (index == _utf8.lindex)
+		return _utf8.lpos;
 	
-	while (_utf8_last_pos < (UTF8_POS_COUNT - 1))
+	min_index = 0;
+	min_i = -1;
+	
+	for (j = 0; j < UTF8_MAX_CACHE; j++)
+	{
+		i = (_utf8.cnext + UTF8_MAX_CACHE - j - 1) % UTF8_MAX_CACHE;
+		
+		if (_utf8.cindex[i] == 0)
+			break;
+		
+		if ((index >= _utf8.cindex[i]) && (index < (_utf8.cindex[i] + 256)))
+		{
+			//fprintf(stderr, "use cache %d (%d)\n", i, _utf8.cindex[i]);
+			pos = _utf8.cpos[i];
+			j = _utf8.cindex[i];
+			goto __CALC_POS;
+		}
+		else if (_utf8.cindex[i] > min_index && _utf8.cindex[i] < index)
+		{
+			min_index = _utf8.cindex[i];
+			min_i = i;
+		}
+	}
+	
+	j = index & ~0xFF;
+
+	if (min_i < 0)
+	{
+		pos = 0;
+		i = 0;
+	}
+	else
+	{
+		pos = _utf8.cpos[min_i];
+		i = _utf8.cindex[min_i];
+	}
+	
+	//fprintf(stderr, "add cache %d: %d / %d\n", _utf8.cnext, j, index - i);
+
+	for (; i < j; i++)
 	{
 		if (pos >= len)
-			return len;
+		{
+			pos = len;
+			break;
+		}
 		pos += utf8_get_char_length(str[pos]);
-		_utf8_pos[++_utf8_last_pos] = pos;
-		if (index == _utf8_last_pos)
-			return pos;
 	}
 	
-	for (i = UTF8_POS_COUNT - 1; i < index; i++)
+	_utf8.cindex[_utf8.cnext] = j;
+	_utf8.cpos[_utf8.cnext] = pos;
+	_utf8.cnext = (_utf8.cnext + 1) % UTF8_MAX_CACHE;
+
+__CALC_POS:
+	
+	for (i = j; i < index; i++)
 	{
 		if (pos >= len)
-			return len;
+		{
+			pos = len;
+			break;
+		}
 		pos += utf8_get_char_length(str[pos]);
 	}
 	
+	_utf8.lindex = index;
+	_utf8.lpos = pos;
+	
 	return pos;
 }
 
@@ -238,18 +325,18 @@ static int byte_to_index(const char *str, int len, int byte)
 	return utf8_get_length(str, byte);
 }
 
-static int index_to_byte(const char *str, int len, int index)
+static int index_to_byte(const char *ref, const char *str, int len, int index)
 {
 	if (index <= 0)
 		return 0;
 
-	return utf8_get_pos(str, len, index - 1) + 1;
+	return utf8_get_pos(ref, str, len, index - 1) + 1;
 }
 
 
 BEGIN_METHOD(String_Pos, GB_STRING str; GB_INTEGER index)
 
-	GB_ReturnInteger(index_to_byte(STRING(str), LENGTH(str), VARG(index)));
+	GB_ReturnInteger(index_to_byte(VARG(str).addr, STRING(str), LENGTH(str), VARG(index)));
 
 END_METHOD
 
@@ -271,6 +358,7 @@ END_METHOD
 static void String_Mid(ushort code)
 {
 	char *str;
+	char *ref;
 	int start, length;
 	int len, ulen, upos;
 	bool null;
@@ -288,10 +376,11 @@ static void String_Mid(ushort code)
 	if (null)
 		goto _SUBR_MID_FIN;
 	
-	str = PARAM->_string.addr + PARAM->_string.start;
+	ref = PARAM->_string.addr;
+	str = ref + PARAM->_string.start;
 	len = PARAM->_string.len;
 	
-	ulen = utf8_get_pos(str, len, start);
+	ulen = utf8_get_pos(ref, str, len, start);
 	if (ulen >= len)
 	{
 		VOID_STRING(PARAM);
@@ -315,10 +404,12 @@ static void String_Mid(ushort code)
 			length += utf8_get_length(str, len) - start;
 		
 		if (length == 1)
+		{
 			ulen = utf8_get_char_length(str[ulen]);
+		}
 		else
 		{
-			upos = utf8_get_pos(str, len, start + length);
+			upos = utf8_get_pos(ref, str, len, start + length);
 			if (upos > len)
 				upos = len;
 			ulen = upos - ulen;
@@ -342,6 +433,7 @@ _SUBR_MID_FIN:
 static void String_Left(ushort code)
 {
 	int val;
+	char *ref;
 	char *str;
 	int len, ulen;
 
@@ -357,13 +449,14 @@ static void String_Left(ushort code)
 			val = PARAM[1]._integer.value;
 		}
 	
-		str = PARAM->_string.addr + PARAM->_string.start;
+		ref = PARAM->_string.addr;
+		str = ref + PARAM->_string.start;
 		len = PARAM->_string.len;
 	
 		if (val < 0)
 			val += utf8_get_length(str, len);
 		
-		ulen = utf8_get_pos(str, len, val);
+		ulen = utf8_get_pos(ref, str, len, val);
 		PARAM->_string.len = ulen;
 	}
 
@@ -376,6 +469,7 @@ static void String_Right(ushort code)
 {
 	int val;
 	char *str;
+	char *ref;
 	int len, ulen;
 
 	SUBR_ENTER();
@@ -390,7 +484,8 @@ static void String_Right(ushort code)
 			val = PARAM[1]._integer.value;
 		}
 	
-		str = PARAM->_string.addr + PARAM->_string.start;
+		ref = PARAM->_string.addr;
+		str = ref + PARAM->_string.start;
 		len = PARAM->_string.len;
 	
 		if (val < 0)
@@ -398,7 +493,7 @@ static void String_Right(ushort code)
 		else
 			val = utf8_get_length(str, len) - val;
 		
-		ulen = utf8_get_pos(str, len, val);
+		ulen = utf8_get_pos(ref, str, len, val);
 		
 		PARAM->_string.start += ulen;
 		PARAM->_string.len -= ulen;
@@ -416,7 +511,7 @@ bool STRING_convert_to_unicode(wchar_t **pwstr, int *pwlen, const char *str, int
 	int i, lc;
 	wchar_t *wstr;
 	
-	result = STRING_new_temp(NULL, wlen * sizeof(wchar_t) + 3);
+	result = STRING_new_temp(NULL, (wlen  + 1) * sizeof(wchar_t) - 1);
 	wstr = (wchar_t *)result;
 	
 	for (i = 0; i < wlen; i++)
@@ -551,6 +646,41 @@ BEGIN_METHOD(String_Upper, GB_STRING str)
 END_METHOD
 
 
+BEGIN_METHOD(String_UCaseFirst, GB_STRING str)
+
+	char *str = STRING(str);
+	int len = LENGTH(str);
+	char *result;
+	int lc;
+	wchar_t wc;
+	
+	if (len <= 0)
+	{
+		GB_ReturnVoidString();
+		return;
+	}
+	
+	result = STRING_new_temp(str, len);
+	
+	lc = utf8_get_char_length(result[0]);
+	
+	if (lc == 1)
+	{
+		result[0] = toupper(result[0]);
+	}
+	else
+	{
+		wc = (wchar_t)utf8_to_unicode(result, lc);
+		wc = towupper(wc);
+		// We suppose that the conversion does not change the number of bytes!
+		utf8_from_unicode((uint)wc, result);
+	}
+
+	GB_ReturnString(result);
+
+END_METHOD
+
+
 BEGIN_METHOD(String_Chr, GB_INTEGER code)
 
 	char temp[8];
@@ -576,19 +706,22 @@ BEGIN_METHOD(String_Code, GB_STRING str; GB_INTEGER index)
 	
 	str = STRING(str);
 	len = LENGTH(str);
-	pos = utf8_get_pos(str, len, index - 1);
+	pos = utf8_get_pos(VARG(str).addr, str, len, index - 1);
 	lc = utf8_get_char_length(str[pos]);
 	
 	GB_ReturnInteger(utf8_to_unicode(&str[pos], lc));
 	
 END_METHOD
 
-static void string_search(const char *str, int len, const char *pattern, int lenp, int start, bool right, bool nocase)
+static void string_search(const char *str, const char *ref, int len, const char *pattern, int lenp, int start, bool right, bool nocase)
 {
 	int pos;
 
+	if (lenp == 0)
+		goto __ERROR;
+
 	if (start)
-		start = index_to_byte(str, len, start);
+		start = index_to_byte(ref, str, len, start);
 
 	if (!nocase)
 	{
@@ -624,13 +757,13 @@ __ERROR:
 
 BEGIN_METHOD(String_Instr, GB_STRING str; GB_STRING pattern; GB_INTEGER start; GB_INTEGER mode)
 
-	string_search(STRING(str), LENGTH(str), STRING(pattern), LENGTH(pattern), VARGOPT(start, 0), FALSE, VARGOPT(mode, GB_COMP_BINARY) == GB_COMP_NOCASE);
+	string_search(STRING(str), VARG(str).addr, LENGTH(str), STRING(pattern), LENGTH(pattern), VARGOPT(start, 0), FALSE, VARGOPT(mode, GB_COMP_BINARY) == GB_COMP_NOCASE);
 
 END_METHOD
 
 BEGIN_METHOD(String_RInstr, GB_STRING str; GB_STRING pattern; GB_INTEGER start; GB_INTEGER mode)
 
-	string_search(STRING(str), LENGTH(str), STRING(pattern), LENGTH(pattern), VARGOPT(start, 0), TRUE, VARGOPT(mode, GB_COMP_BINARY) == GB_COMP_NOCASE);
+	string_search(STRING(str), VARG(str).addr, LENGTH(str), STRING(pattern), LENGTH(pattern), VARGOPT(start, 0), TRUE, VARGOPT(mode, GB_COMP_BINARY) == GB_COMP_NOCASE);
 
 END_METHOD
 
@@ -716,6 +849,8 @@ GB_DESC NATIVE_String[] =
 	GB_STATIC_METHOD("Upper$", "s", String_Upper, "(String)s"),
 	GB_STATIC_METHOD("UCase", "s", String_Upper, "(String)s"),
 	GB_STATIC_METHOD("UCase$", "s", String_Upper, "(String)s"),
+	GB_STATIC_METHOD("UCaseFirst", "s", String_UCaseFirst, "(String)s"),
+	GB_STATIC_METHOD("UCaseFirst$", "s", String_UCaseFirst, "(String)s"),
 	GB_STATIC_METHOD("Lower", "s", String_Lower, "(String)s"),
 	GB_STATIC_METHOD("Lower$", "s", String_Lower, "(String)s"),
 	GB_STATIC_METHOD("LCase", "s", String_Lower, "(String)s"),
