@@ -2,7 +2,7 @@
 
   gbx_class.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include "gbx_variant.h"
 #include "gbx_number.h"
 #include "gbx_c_array.h"
+#include "gbx_jit.h"
 
 #include "gambas.h"
 
@@ -81,6 +82,10 @@ TABLE *CLASS_get_table(void)
 	return &_global_table;
 }
 
+CLASS *CLASS_get_list(void)
+{
+	return _classes;
+}
 
 static void exit_class(CLASS *class, bool native)
 {
@@ -111,15 +116,15 @@ static void unload_class(CLASS *class)
 	#endif
 
 	if (class->free_name)
-		FREE(&class->name, "unload_class");
+		FREE(&class->name);
 
 	if (class->is_struct)
 	{
-		FREE(&class->load->dyn, "unload_class");
+		FREE(&class->load->dyn);
 		if (class->debug)
-			FREE(&class->load->global, "unload_class");
-		FREE(&class->load, "unload_class");
-		FREE(&class->data, "unload_class");
+			FREE(&class->load->global);
+		FREE(&class->load);
+		FREE(&class->data);
 	}
 	else if (!CLASS_is_native(class))
 	{
@@ -127,14 +132,14 @@ static void unload_class(CLASS *class)
 			
 			if (class->load)
 			{
-				FREE(&class->load->desc, "unload_class");
-				FREE(&class->load->cst, "unload_class");
-				FREE(&class->load->class_ref, "unload_class");
-				FREE(&class->load->unknown, "unload_class");
-				FREE(&class->load->event, "unload_class");
-				FREE(&class->load->ext, "unload_class");
-				FREE(&class->load->local, "unload_class");
-				FREE(&class->load->array, "unload_class");
+				FREE(&class->load->desc);
+				FREE(&class->load->cst);
+				FREE(&class->load->class_ref);
+				FREE(&class->load->unknown);
+				FREE(&class->load->event);
+				FREE(&class->load->ext);
+				FREE(&class->load->local);
+				FREE(&class->load->array);
 				
 				if (class->debug)
 				{
@@ -144,61 +149,92 @@ static void unload_class(CLASS *class)
 					for (i = 0; i < class->load->n_func; i++)
 					{
 						func = &class->load->func[i];
-						FREE(&func->debug->local, "unload_class");
+						FREE(&func->debug->local);
 					}
 					
-					FREE(&class->load->global, "unload_class");
-					FREE(&class->load->debug, "unload_class");
+					FREE(&class->load->global);
+					FREE(&class->load->debug);
 				}
 				
-				FREE(&class->load->func, "unload_class");
+				FREE(&class->load->func);
 			}
 
 		#endif
 		
-		FREE(&class->load, "unload_class");
-		if (!class->mmapped)
-			FREE(&class->data, "unload_class");
+		if (class->load)
+			FREE(&class->load->prof);
+		
+		FREE(&class->jit_functions);
+		
+		FREE(&class->load);
+		//if (!class->mmapped)
+		FREE(&class->data);
 	}
 	else
 	{
-		FREE(&class->signature, "unload_class");
-		FREE(&class->data, "unload_class");
+		FREE(&class->signature);
+		FREE(&class->data);
 	}
 
 	if (class->free_event)
-		FREE(&class->event, "unload_class");
+		FREE(&class->event);
 
-	FREE(&class->stat, "unload_class");
+	FREE(&class->stat);
 
-	FREE(&class->table, "unload_class");
-	FREE(&class->sort, "unload_class");
+	FREE(&class->table);
+	FREE(&class->sort);
 
 	class->state = CS_NULL;
 }
 
-static CLASS *class_replace_global(const char *name)
+#define SWAP_FIELD(_v, _s, _d, _f) (_v = _s->_f, _s->_f = _d->_f, _d->_f = _v)
+
+static void class_replace_global(const char *name)
 {
-	CLASS_SYMBOL *csym;
-	char *new_name;
+	CLASS_DESC_SYMBOL *cds;
 	CLASS *class, *parent;
-	CLASS *new_class;
+	char *old_name;
+	CLASS *old_class;
 	int len;
-	int index;
+	//int index;
+	CLASS swap;
+	char *swap_name;
+	bool swap_free_name;
+	int nprefix;
+	int i;
 
 	class = CLASS_find_global(name);
 	if (class->state)
 	{
+		//fprintf(stderr, "class_replace_global: %p %s\n", class, name);
+		
 		len = strlen(name);
 
-		ALLOC(&new_name, len + 2, "class_replace_global");
-		snprintf(new_name, len + 2, ">%s", name);
-		new_class = class_replace_global(new_name);
-		FREE(&new_name, "class_replace_global");
+		nprefix = 0;
+		parent = class;
+		do
+		{
+			nprefix++;
+			parent = parent->parent;
+		}
+		while (parent);
+		
+		ALLOC(&old_name, len + nprefix + 1);
+		for (i = 0; i < nprefix; i++)
+			old_name[i] = '>';
+		strcpy(&old_name[i], name);
+		
+		old_class = CLASS_find_global(old_name);
+		//fprintf(stderr, "-> %p %s\n", old_class, old_name);
+		
+		FREE(&old_name);
+		/*FREE(&old_class->name, "class_replace_global");
+		old_class->free_name = FALSE;
+		old_class->name = class->name;*/
 
-		new_name = (char *)new_class->name;
+		//new_name = (char *)new_class->name;
 
-		if (TABLE_find_symbol(&_global_table, name, len, &index))
+		/*if (TABLE_find_symbol(&_global_table, name, len, &index))
 		{
 			csym = (CLASS_SYMBOL *)TABLE_get_symbol(&_global_table, index);
 			csym->class = new_class;
@@ -211,9 +247,28 @@ static CLASS *class_replace_global(const char *name)
 		}
 
 		new_class->name = class->name;
-		class->name = new_name;
+		class->name = new_name;*/
 		
-		for(;;)
+		swap = *class;
+		*class = *old_class;
+		*old_class = swap;
+		
+		SWAP_FIELD(swap_name, class, old_class, name);
+		SWAP_FIELD(swap_free_name, class, old_class, free_name);
+		SWAP_FIELD(parent, class, old_class, next);
+		SWAP_FIELD(i, class, old_class, count);
+		SWAP_FIELD(i, class, old_class, ref);
+		
+		for (i = 0; i < old_class->n_desc; i++)
+		{
+			cds = &old_class->table[i];
+			if (cds->desc && cds->desc->method.class == class)
+				cds->desc->method.class = old_class;
+		}
+		
+		CLASS_inheritance(class, old_class, FALSE);
+		
+		/*for(;;)
 		{
 			class->override = new_class;
 			parent = class->parent;
@@ -222,12 +277,17 @@ static CLASS *class_replace_global(const char *name)
 			class = parent;
 		}
 
-		class = new_class;
+		class = new_class;*/
 	}
 
-	return class;
+	//return class;
 }
 
+static void release_class(CLASS *class)
+{
+	OBJECT_release(class, NULL);
+	class->exit = TRUE;
+}
 
 void CLASS_clean_up(bool silent)
 {
@@ -251,7 +311,7 @@ void CLASS_clean_up(bool silent)
 	for (class = _classes; class; class = class->next)
 	{
 		if (class->instance)
-			OBJECT_UNREF(class->instance, "CLASS_exit");
+			OBJECT_UNREF(class->instance);
 		if (!CLASS_is_native(class) && class->state)
 		{
 			/*printf("Must free: %s\n", class->name);*/
@@ -288,13 +348,12 @@ void CLASS_clean_up(bool silent)
 				#if DEBUG_LOAD
 				fprintf(stderr, "Freeing %s\n", class->name);
 				#endif
-				OBJECT_release(class, NULL);
-				class->exit = TRUE;
+				release_class(class);
 				n++;
 			}
 		}
 
-		if (n == nb) /* On n'a rien pu faire */
+		if (n == nb) // nothing could be done
 			break;
 	}
 
@@ -309,16 +368,24 @@ void CLASS_clean_up(bool silent)
 	{
 		if (!silent)
 			ERROR_warning("circular references detected:");
-		for (class = _classes; class; class = class->next)
+
+		for(;;)
 		{
-			if (!CLASS_is_native(class) && class->state && !class->exit)
+			nb = n;
+
+			for (class = _classes; class; class = class->next)
 			{
-				if (!silent)
-					fprintf(stderr, "gbx" GAMBAS_VERSION_STRING ": % 5d %s\n", class->count, class->name);
-				OBJECT_release(class, NULL);
-				class->exit = TRUE;
-				n++;
+				if (!CLASS_is_native(class) && class->state && !class->exit && (!class->array_class || class->array_class->exit)) // && !class->astruct_class)
+				{
+					if (!silent)
+						fprintf(stderr, "gbx" GAMBAS_VERSION_STRING ": % 5d %s\n", class->count, class->name);
+					release_class(class);
+					n++;
+				}
 			}
+
+			if (n == nb) // nothing could be done
+				break;
 		}
 	}
 
@@ -351,7 +418,7 @@ void CLASS_exit()
 	while (class)
 	{
 		next = class->next;
-		FREE(&class, "CLASS_exit");
+		FREE(&class);
 		class = next;
 	}
 
@@ -369,6 +436,12 @@ CLASS *CLASS_look(const char *name, int len)
 	fprintf(stderr, "CLASS_look: %s in %s\n", name, _global ? "global" : "local");
 	#endif
 
+	/*if (strncasecmp(name, "listbox", 7) == 0)
+	{
+		name = "MyListBox";
+		len = 9;
+	}*/
+	
 	//if (CP && CP->component && CP->component->archive)
 	if (!_global && !ARCHIVE_get_current(&arch))
 	{
@@ -441,7 +514,7 @@ CLASS *CLASS_find(const char *name)
 		csym = (CLASS_SYMBOL *)TABLE_get_symbol(&_global_table, index);
 	}
 
-	ALLOC_ZERO(&class, sizeof(CLASS), "CLASS_find");
+	ALLOC_ZERO(&class, sizeof(CLASS));
 	csym->class = class;
 	//class->state = CS_NULL;
 	//class->count = 0;
@@ -450,7 +523,7 @@ CLASS *CLASS_find(const char *name)
 	class->next = _classes;
 	_classes = class;
 
-	ALLOC(&class->name, len + 1, "CLASS_find");
+	ALLOC(&class->name, len + 1);
 	strcpy((char *)class->name, name);
 
 	csym->sym.name = class->name;
@@ -644,17 +717,14 @@ char *CLASS_DESC_get_signature(CLASS_DESC *cd)
 	return res;
 }
 
-// The _free method can be called during a conversion, so we must save the EXEC structure
-
-static void *_CLASS_free_object;
-static EXEC_GLOBAL *_CLASS_free_exec;
+// NOTE: The _free method can be called during a conversion, so we must save the EXEC structure
 
 static void error_CLASS_free()
 {
-	void *object = _CLASS_free_object;
+	void *object = (void *)ERROR_handler->arg1;
+	EXEC = *((EXEC_GLOBAL *)ERROR_handler->arg2);
 	((OBJECT *)object)->ref = 0;
 	OBJECT_release(OBJECT_class(object), object);
-	EXEC = *_CLASS_free_exec;
 }
 
 void CLASS_free(void *object)
@@ -662,24 +732,17 @@ void CLASS_free(void *object)
 	CLASS *class = OBJECT_class(object);
 	EXEC_GLOBAL save = EXEC;
 
-	void *save_object = _CLASS_free_object;
-	void *save_exec = _CLASS_free_exec;
-	
-	_CLASS_free_object = object;
-	_CLASS_free_exec = &save;
-	
-	ON_ERROR(error_CLASS_free)
+	ON_ERROR_2(error_CLASS_free, object, &save)
 	{
-		((OBJECT *)object)->ref = 1; /* Prevents anybody from freeing the object ! */
+		((OBJECT *)object)->ref = 1; // Prevents anybody from freeing the object!
+		
 		EXEC_special_inheritance(SPEC_FREE, class, object, 0, TRUE);
+		
 		((OBJECT *)object)->ref = 0;
 		EXEC = save;
 		OBJECT_release(class, object);
 	}
 	END_ERROR
-	
-	_CLASS_free_object = save_object;
-	_CLASS_free_exec = save_exec;
 }
 
 #if DEBUG_REF
@@ -838,7 +901,7 @@ void CLASS_sort(CLASS *class)
 	
 	_sorted_class = class;
 	
-	ALLOC(&sym, sizeof(ushort) * class->n_desc, "CLASS_sort");
+	ALLOC(&sym, sizeof(ushort) * class->n_desc);
 
 	for (i = 0; i < class->n_desc; i++)
 		sym[i] = i;
@@ -858,27 +921,31 @@ void CLASS_sort(CLASS *class)
 		{
 			s = (SYMBOL *)&class->table[sym[i]];
 
-			fprintf(stderr, "[%d] %.*s\n", i, (int)s->len, s->name);
+			fprintf(stderr, "[%d] %.*s (%d)\n", i, (int)s->len, s->name, sym[i]);
 		}
+		fputc('\n', stderr);
 	}
 	#endif
 }
 
-void CLASS_inheritance(CLASS *class, CLASS *parent)
+void CLASS_inheritance(CLASS *class, CLASS *parent, bool in_jit_compilation)
 {
 	if (class->parent != NULL)
-		THROW(E_CLASS, class->name, "Multiple inheritance", "");
+		THROW_CLASS(class, "Multiple inheritance", "");
 
 	class->parent = parent;
 	parent->has_child = TRUE;
 
 	TRY
 	{
-		CLASS_load(class->parent);
+		if (!in_jit_compilation)
+			CLASS_load(class->parent);
+		else
+			JIT.LoadClass(class->parent);
 	}
 	CATCH
 	{
-		THROW(E_CLASS, class->name, "Cannot load parent class: ", STRING_new_temp_zero(ERROR_current->info.msg));
+		THROW_CLASS(class, "Cannot load parent class: ", STRING_new_temp_zero(ERROR_current->info.msg));
 	}
 	END_TRY
 
@@ -902,7 +969,7 @@ void CLASS_inheritance(CLASS *class, CLASS *parent)
 
 const char *CLASS_DESC_get_type_name(const CLASS_DESC *desc)
 {
-	switch (desc->gambas.val4)
+	switch (desc->gambas.val3._int[1])
 	{
 		case CD_PROPERTY_ID: return desc->gambas.val2 < 0 ? "r" : "p";
 		case CD_VARIABLE_ID: return "v";
@@ -996,7 +1063,7 @@ void CLASS_make_description(CLASS *class, const CLASS_DESC *desc, int n_desc, in
 	// Make the description symbol table
 
 	if (class->n_desc)
-		ALLOC(&class->table, sizeof(CLASS_DESC_SYMBOL) * class->n_desc, "CLASS_make_description");
+		ALLOC(&class->table, sizeof(CLASS_DESC_SYMBOL) * class->n_desc);
 
 	i = 0;
 
@@ -1051,14 +1118,14 @@ void CLASS_make_description(CLASS *class, const CLASS_DESC *desc, int n_desc, in
 						#if DEBUG_DESC
 						fprintf(stderr, "type = '%c' parent_type = '%c'\n", type, parent_type);
 						#endif
-						THROW(E_OVERRIDE, parent->name, cds->name, class->name);
+						THROW(E_OVERRIDE, CLASS_get_name(parent), cds->name, CLASS_get_name(class));
 					}
 					
 					if (!CLASS_is_native(class) && strcasecmp(name, "_new"))
 					{
 						//fprintf(stderr, "check_signature: %s\n", name);
 						if (check_signature(type, &desc[j], cds->desc))
-							THROW(E_OVERRIDE, parent->name, cds->name, class->name);
+							THROW(E_OVERRIDE, CLASS_get_name(parent), cds->name, CLASS_get_name(class));
 					}
 					
 					check = TRUE;
@@ -1167,7 +1234,7 @@ void CLASS_calc_info(CLASS *class, int n_event, int size_dynamic, bool all, int
 
 	class->size_stat = size_static;
 	if (size_static)
-		ALLOC_ZERO(&class->stat, class->size_stat, "CLASS_calc_info");
+		ALLOC_ZERO(&class->stat, class->size_stat);
 	else
 		class->stat = NULL;
 		
@@ -1194,7 +1261,7 @@ void CLASS_make_event(CLASS *class, int *first)
 		}
 	}
 
-	ALLOC_ZERO(&class->event, sizeof(CLASS_EVENT) * class->n_event, "CLASS_make_event");
+	ALLOC_ZERO(&class->event, sizeof(CLASS_EVENT) * class->n_event);
 
 	if (class->parent != NULL)
 	{
@@ -1243,7 +1310,7 @@ void *CLASS_auto_create(CLASS *class, int nparam)
 			return ob;
 		}
 
-		OBJECT_UNREF(class->instance, "CLASS_auto_create");
+		OBJECT_UNREF(class->instance);
 		class->instance = NULL;
 	}
 
@@ -1258,8 +1325,13 @@ void *CLASS_auto_create(CLASS *class, int nparam)
 	return class->instance;
 }
 
+//#define SET_OPTIONAL_OPERATOR(_class, _op, _func) if (!CLASS_has_operator(_class, _op)) (_class)->operators[_op] = (EXEC_no_operator##_func)
+
 void CLASS_search_special(CLASS *class)
 {
+	static int _operator_strength = 0;
+	int sym;
+	
 	class->special[SPEC_NEW] = CLASS_get_symbol_index_kind(class, "_new", CD_METHOD, 0);
 	class->special[SPEC_FREE] = CLASS_get_symbol_index_kind(class, "_free", CD_METHOD, 0);
 	class->special[SPEC_GET] = CLASS_get_symbol_index_kind(class, "_get", CD_METHOD, CD_STATIC_METHOD);
@@ -1272,27 +1344,45 @@ void CLASS_search_special(CLASS *class)
 	class->special[SPEC_COMPARE] = CLASS_get_symbol_index_kind(class, "_compare", CD_METHOD, 0);
 	class->special[SPEC_ATTACH] = CLASS_get_symbol_index_kind(class, "_attach", CD_METHOD, 0);
 	
-	class->special[SPEC_CONVERT] = CLASS_get_symbol_index_kind(class, "_@_convert", CD_CONSTANT, 0);
-
+	sym = CLASS_get_symbol_index_kind(class, "_@_convert", CD_CONSTANT, 0);
+	if (sym != NO_SYMBOL)
+	{
+		class->has_convert = TRUE;
+		class->convert = CLASS_get_desc(class, sym)->constant.value._pointer;
+	}
+	
+	sym = CLASS_get_symbol_index_kind(class, "_@_operator", CD_CONSTANT, 0);
+	if (sym != NO_SYMBOL)
+	{
+		class->has_operators = TRUE;
+		class->operators = CLASS_get_desc(class, sym)->constant.value._pointer;
+		_operator_strength++;
+		CLASS_set_operator_strength(class, _operator_strength);
+	}
+	
 	if (class->special[SPEC_NEXT] != NO_SYMBOL)
 		class->enum_static = CLASS_DESC_get_type(CLASS_get_desc(class, class->special[SPEC_NEXT])) == CD_STATIC_METHOD;
+	if (class->special[SPEC_UNKNOWN] != NO_SYMBOL)
+		class->unknown_static = CLASS_DESC_get_type(CLASS_get_desc(class, class->special[SPEC_UNKNOWN])) == CD_STATIC_METHOD;
+	if (class->special[SPEC_PROPERTY] != NO_SYMBOL)
+		class->property_static = CLASS_DESC_get_type(CLASS_get_desc(class, class->special[SPEC_PROPERTY])) == CD_STATIC_METHOD;
 }
 
 
 CLASS *CLASS_check_global(char *name)
 {
-	CLASS *class, *parent;
+	CLASS *class;
 
 	class = CLASS_find_global(name);
 	if (class->state)
 	{
 		if (COMPONENT_current && class->component == COMPONENT_current)
-			ERROR_panic("Class '%s' declared twice in the component '%s'.", class->name, class->component->name);
+			ERROR_panic("Class '%s' declared twice in the component '%s'.", CLASS_get_name(class), class->component->name);
 
-		parent = class;
-		class = class_replace_global(name);
-
-		CLASS_inheritance(class, parent);
+		if (class->has_child)
+			THROW(E_CLASS, name, "Overriding an already inherited class is forbidden", "");
+			
+		class_replace_global(name);
 	}
 
 	return class;
@@ -1357,7 +1447,7 @@ void CLASS_create_array_class(CLASS *class)
 	TYPE_joker = array_type;
 	array_type->array_class = class;
 
-	ALLOC(&desc, sizeof(GB_DESC) * ARRAY_TEMPLATE_NDESC, "CLASS_create_array_class");
+	ALLOC(&desc, sizeof(GB_DESC) * ARRAY_TEMPLATE_NDESC);
 	memcpy(desc, NATIVE_TemplateArray, sizeof(GB_DESC) * ARRAY_TEMPLATE_NDESC);
 	((CLASS_DESC_GAMBAS *)desc)->name = class->name;
 
@@ -1407,7 +1497,7 @@ void CLASS_create_array_of_struct_class(CLASS *class)
 	TYPE_joker = array_type;
 	array_type->astruct_class = class;
 
-	ALLOC(&desc, sizeof(GB_DESC) * ARRAY_OF_STRUCT_TEMPLATE_NDESC, "CLASS_create_array_of_struct_class");
+	ALLOC(&desc, sizeof(GB_DESC) * ARRAY_OF_STRUCT_TEMPLATE_NDESC);
 	memcpy(desc, NATIVE_TemplateArrayOfStruct, sizeof(GB_DESC) * ARRAY_OF_STRUCT_TEMPLATE_NDESC);
 	((CLASS_DESC_GAMBAS *)desc)->name = class->name;
 
@@ -1442,3 +1532,13 @@ int CLASS_sizeof(CLASS *class)
 	else
 		return class->size - sizeof(OBJECT);
 }
+
+char *CLASS_get_name(CLASS *class)
+{
+	char *name = class->name;
+	
+	while (*name == '>')
+		name++;
+	
+	return name;
+}
