@@ -2,7 +2,7 @@
 
 	gbx_class_native.c
 
-	(c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -82,7 +82,7 @@ CLASS *CLASS_register_class(GB_DESC *ptr, CLASS *class)
 		return NULL;
 		
 	if (class->error)
-		THROW(E_CLASS, class->name, "Loading has already failed", "");
+		THROW_CLASS(class, "Loading has already failed", "");
 
 	class->error = TRUE;
 	
@@ -125,7 +125,7 @@ CLASS *CLASS_register_class(GB_DESC *ptr, CLASS *class)
 		switch ((intptr_t)desc->gambas.name)
 		{
 			case (intptr_t)GB_INHERITS_ID:
-				CLASS_inheritance(class, CLASS_find((const char *)desc->gambas.type));
+				CLASS_inheritance(class, CLASS_find((const char *)desc->gambas.type), FALSE);
 				break;
 
 			case (intptr_t)GB_AUTO_CREATABLE_ID:
@@ -174,8 +174,16 @@ CLASS *CLASS_register_class(GB_DESC *ptr, CLASS *class)
 					desc->constant.type = T_CSTRING;
 				else if (desc->constant.type == T_FLOAT || desc->constant.type == T_SINGLE)
 				{
-					if (NUMBER_from_string(NB_READ_FLOAT, desc->constant.value._string, strlen(desc->constant.value._string), &value))
-						THROW(E_CLASS, class->name, "Bad constant", "");
+					if (desc->gambas.val1 == 0)
+					{
+						value._float.value = desc->gambas.val3._double;
+					}
+					else
+					{
+						if (NUMBER_from_string(NB_READ_FLOAT, desc->constant.value._string, strlen(desc->constant.value._string), &value))
+							THROW_CLASS(class, "Bad constant", "");
+					}
+					
 					if (desc->constant.type == T_SINGLE)
 						desc->constant.value._single = (float)value._float.value;
 					else
@@ -243,48 +251,64 @@ CLASS *CLASS_register_class(GB_DESC *ptr, CLASS *class)
 
 	if (nsign)
 	{
-		ALLOC(&class->signature, sizeof(TYPE) * nsign, "CLASS_register");
+		ALLOC(&class->signature, sizeof(TYPE) * nsign);
 		sign = class->signature;
-	}
-
-	for (i = first; i < class->n_desc; i++)
-	{
-		desc = class->table[i].desc;
-
-		//fprintf(stderr, "[%.*s]\n", class->table[i].len, class->table[i].name);
 		
-		switch (CLASS_DESC_get_type(desc))
+		for (i = first; i < class->n_desc; i++)
 		{
-			case CD_METHOD:
-			case CD_STATIC_METHOD:
+			desc = class->table[i].desc;
 
-				if (desc->method.npmax)
-				{
-					desc->method.signature =
-						TYPE_transform_signature(&sign, (char *)desc->method.signature, desc->method.npmax);
-				}
-				break;
+			//fprintf(stderr, "[%.*s]\n", class->table[i].len, class->table[i].name);
+			
+			switch (CLASS_DESC_get_type(desc))
+			{
+				case CD_METHOD:
+				case CD_STATIC_METHOD:
 
-			case CD_EVENT:
+					if (desc->method.npmax)
+					{
+						desc->method.signature =
+							TYPE_transform_signature(&sign, (char *)desc->method.signature, desc->method.npmax);
+					}
+					break;
 
-				if (desc->event.npmax)
-				{
-					desc->event.signature =
-						TYPE_transform_signature(&sign, (char *)desc->event.signature, desc->event.npmax);
-				}
+				case CD_EVENT:
 
-				event = &class->event[first_event];
-				event->name = class->table[i].name;
-				if (desc->event.index)
-					*desc->event.index = first_event;
+					if (desc->event.npmax)
+					{
+						desc->event.signature =
+							TYPE_transform_signature(&sign, (char *)desc->event.signature, desc->event.npmax);
+					}
 
-				event->type = desc->event.type;
-				event->param = (CLASS_PARAM *)desc->event.signature;
-				event->n_param = desc->event.npmax;
+					break;
+			}
+		}
+	}
+	
+	if (class->n_event && (!class->parent || class->n_event > class->parent->n_event))
+	{
+		for (i = first; i < class->n_desc; i++)
+		{
+			desc = class->table[i].desc;
 
-				first_event++;
+			switch (CLASS_DESC_get_type(desc))
+			{
+				case CD_EVENT:
 
-				break;
+					event = &class->event[first_event];
+					event->name = class->table[i].name;
+					if (desc->event.index)
+						*((int *)desc->event.index) = first_event;
+					desc->event.index = first_event;
+
+					event->type = desc->event.type;
+					event->param = (CLASS_PARAM *)desc->event.signature;
+					event->n_param = desc->event.npmax;
+
+					first_event++;
+
+					break;
+			}
 		}
 	}
 
