@@ -2,7 +2,7 @@
 
   gbx_component.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,20 +34,19 @@
 typedef
   struct _COMPONENT {
     void *class;
-    int ref;
+    intptr_t ref;
     LIST list;
+		LIST load;
     char *name;
     LIBRARY *library;
     ARCHIVE *archive;
 		unsigned order : 8;
-    unsigned preload : 1;
     unsigned loaded : 1;
     unsigned user : 1;       // user library
     unsigned warning : 1;    // Set when the bytecode warning was displayed by the class loader for this component
     unsigned loading : 1;    // component is being loaded
-    unsigned _reserved : 19;
+    unsigned _reserved : 20;
     }
-  PACKED
   COMPONENT;
 
 #ifndef __GBX_COMPONENT_C
@@ -67,9 +66,12 @@ COMPONENT *COMPONENT_find(const char *name);
 bool COMPONENT_exist(const char *name);
 
 void COMPONENT_load(COMPONENT *comp);
-void COMPONENT_load_all(void);
 void COMPONENT_unload(COMPONENT *comp);
 
+void COMPONENT_load_all(void);
+void COMPONENT_load_all_finish(void);
+
+
 COMPONENT *COMPONENT_next(COMPONENT *comp);
 
 void COMPONENT_translation_must_be_reloaded(void);
@@ -80,4 +82,6 @@ void COMPONENT_signal(int signal, void *param);
 
 bool COMPONENT_get_info(const char *key, void **value);
 
+void COMPONENT_exec(const char *name, int argc, char **argv);
+
 #endif
