@@ -2,7 +2,7 @@
 
   gbx_event.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -42,20 +42,27 @@ static EVENT_POST *_post_list = NULL;
 
 static void check_event_method(CLASS *class, const char *name, CLASS_DESC_METHOD *desc, CLASS_EVENT *event)
 {
-	if (event->n_param < desc->npmin)
+	int n = event->n_param;
+	
+	if (n < desc->npmin)
 	{
 		/*printf("event->n_param = %d  desc->npmin = %d  desc->npmax = %d\n", event->n_param, desc->npmin, desc->npmax);*/
-		THROW(E_EVENT, class->name, name, "Too many arguments");
+		THROW(E_EVENT, CLASS_get_name(class), name, "Too many arguments");
 	}
 
-	if (event->n_param > desc->npmax)
-		THROW(E_EVENT, class->name, name, "Not enough arguments");
+	if (n > desc->npmax)
+	{
+		if (!desc->npvar)
+			THROW(E_EVENT, CLASS_get_name(class), name, "Not enough arguments");
+		
+		n = desc->npmax;
+	}
 
-	if (TYPE_compare_signature(desc->signature, event->n_param, (TYPE *)event->param, event->n_param))
-		THROW(E_EVENT, class->name, name, "Type mismatch");
+	if (TYPE_compare_signature(desc->signature, n, (TYPE *)event->param, n))
+		THROW(E_EVENT, CLASS_get_name(class), name, "Type mismatch");
 
 	if (desc->type != T_VOID)
-		THROW(E_EVENT, class->name, name, "Not a procedure");
+		THROW(E_EVENT, CLASS_get_name(class), name, "Not a procedure");
 }
 
 
@@ -164,7 +171,7 @@ void EVENT_exit()
 	{
 		ep = _post_list;
 		_post_list = _post_list->list.next;
-		FREE(&ep, "EVENT_exit");
+		FREE(&ep);
 	}
 	
 	STRING_free(&EVENT_Name);
@@ -178,7 +185,7 @@ static void post(void (*func)(), int nparam, intptr_t param, intptr_t param2)
 	/*printf("EVENT_post\n");
 	fflush(NULL);*/
 
-	ALLOC(&ep, sizeof(EVENT_POST), "EVENT_post");
+	ALLOC(&ep, sizeof(EVENT_POST));
 
 	ep->func = func;
 	ep->nparam = nparam;
@@ -204,18 +211,12 @@ void EVENT_post2(void (*func)(), intptr_t param, intptr_t param2)
 static void post_event(void *object, int event)
 {
 	GB_Raise(object, event, 0);
-	OBJECT_UNREF(object, "post_event");
+	OBJECT_UNREF(object);
 }
 
 void EVENT_post_event(void *object, int event)
 {
-	/*if (!GB_CanRaise(object, event))
-	{
-		fprintf(stderr, "EVENT_post_event: cannot raise event\n");
-		return;
-	}*/
-	
-	OBJECT_REF(object, "EVENT_post_event");
+	OBJECT_REF(object);
 	post((void (*)())post_event, 2, (intptr_t)object, (intptr_t)event);
 }
 
@@ -237,7 +238,7 @@ bool EVENT_check_post(void)
 		save = *ep;
 		LIST_remove(&_post_list, _post_list, &_post_list->list);
 
-		FREE(&ep, "EVENT_check_post");
+		FREE(&ep);
 
 		if (save.nparam == 1)
 			(*save.func)(save.param);
