@@ -2,7 +2,7 @@
 
   gbx_exec.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -62,12 +62,16 @@ bool EXEC_big_endian;
 CENUM *EXEC_enum;
 
 bool EXEC_debug = FALSE; // debugging mode
+bool EXEC_task = FALSE; // I am a background task
 bool EXEC_profile = FALSE; // profiling mode
+bool EXEC_profile_instr = FALSE; // profiling mode at instruction level
+const char *EXEC_profile_path = NULL; // profile file path
 bool EXEC_arch = FALSE; // executing an archive
 bool EXEC_fifo = FALSE; // debugging through a fifo
 const char *EXEC_fifo_name = NULL; // fifo name
 bool EXEC_keep_library = FALSE; // do not unload libraries
-
+bool EXEC_string_add = FALSE; // next '&' operator is done for a '&='
+uchar EXEC_quit_value = 0;
 EXEC_HOOK EXEC_Hook = { NULL };
 EXEC_GLOBAL EXEC;
 bool EXEC_main_hook_done = FALSE;
@@ -121,11 +125,11 @@ __VARIANT:
 	if (value->_variant.vtype == T_STRING)
 		STRING_ref(value->_variant.value._string);
 	else if (TYPE_is_object(value->_variant.vtype))
-		OBJECT_REF(value->_variant.value._object, "BORROW");
+		OBJECT_REF(value->_variant.value._object);
 	return;
 
 __FUNCTION:
-	OBJECT_REF(value->_function.object, "BORROW");
+	OBJECT_REF(value->_function.object);
 	return;
 
 __STRING:
@@ -147,7 +151,7 @@ void UNBORROW(VALUE *value)
 
 	if (UNLIKELY(TYPE_is_object(type)))
 	{
-		OBJECT_UNREF_KEEP(value->_object.object, "UNBORROW");
+		OBJECT_UNREF_KEEP(value->_object.object);
 		return;
 	}
 	
@@ -157,11 +161,11 @@ __VARIANT:
 	if (value->_variant.vtype == T_STRING)
 		STRING_unref_keep(&value->_variant.value._string);
 	else if (TYPE_is_object(value->_variant.vtype))
-		OBJECT_UNREF_KEEP(value->_variant.value._object, "UNBORROW");
+		OBJECT_UNREF_KEEP(value->_variant.value._object);
 	return;
 
 __FUNCTION:
-	OBJECT_UNREF_KEEP(value->_function.object, "UNBORROW");
+	OBJECT_UNREF_KEEP(value->_function.object);
 	return;
 
 __STRING:
@@ -185,11 +189,11 @@ __VARIANT:
 	if (value->_variant.vtype == T_STRING)
 		STRING_unref(&value->_variant.value._string);
 	else if (TYPE_is_object(value->_variant.vtype))
-		OBJECT_UNREF(value->_variant.value._object, "RELEASE");
+		OBJECT_UNREF(value->_variant.value._object);
 	return;
 
 __FUNCTION:
-	OBJECT_UNREF(value->_function.object, "RELEASE");
+	OBJECT_UNREF(value->_function.object);
 	return;
 
 __STRING:
@@ -217,7 +221,7 @@ void RELEASE_many(VALUE *value, int n)
 	
 		if (UNLIKELY(TYPE_is_object(type)))
 		{
-			OBJECT_UNREF(value->_object.object, "RELEASE");
+			OBJECT_UNREF(value->_object.object);
 			continue;
 		}
 		
@@ -227,11 +231,11 @@ void RELEASE_many(VALUE *value, int n)
 		if (value->_variant.vtype == T_STRING)
 			STRING_unref(&value->_variant.value._string);
 		else if (TYPE_is_object(value->_variant.vtype))
-			OBJECT_UNREF(value->_variant.value._object, "RELEASE");
+			OBJECT_UNREF(value->_variant.value._object);
 		continue;
 	
 	__FUNCTION:
-		OBJECT_UNREF(value->_function.object, "RELEASE");
+		OBJECT_UNREF(value->_function.object);
 		continue;
 	
 	__STRING:
@@ -302,7 +306,7 @@ void EXEC_release_return_value(void)
 
 
 #define print_register() \
-	printf("| SP = %d  BP = %d  FP = %p  PC = %p  EC = %p\n", SP - (VALUE *)STACK_base, BP - (VALUE *)STACK_base, FP, PC, EC)
+	fprintf(stderr, "| SP = %d  BP = %d  FP = %p  PC = %p  EC = %p\n", (int)(SP - (VALUE *)STACK_base), (int)(BP - (VALUE *)STACK_base), FP, PC, EC)
 
 static ushort exec_enter_can_quick(void)
 {
@@ -420,21 +424,21 @@ void EXEC_enter(void)
 	CLASS *class = EXEC.class;
 
 	#if DEBUG_STACK
-	printf("\n| >> EXEC_enter(%s, %ld, %d)\n", EXEC.class->name, EXEC.index, nparam);
+	fprintf(stderr, "\n| >> EXEC_enter(%s, %d, %d)\n", EXEC.class->name, EXEC.index, nparam);
 	print_register();
 	#endif
 
 	func = &class->load->func[EXEC.index];
 	#if DEBUG_STACK
 	if (func->debug)
-		printf(" | >> %s\n", func->debug->name);
+		fprintf(stderr, " | >> %s\n", func->debug->name);
 	#endif
 
 	#if SHOW_FUNCTION
 	if (func->debug)
 		fprintf(stderr, "%s.%s\n", EXEC.class->name, func->debug->name);
 	#endif
-		
+	
 	// Check number of arguments
 
 	if (UNLIKELY(nparam < func->npmin))
@@ -472,7 +476,8 @@ void EXEC_enter(void)
 				SP++;
 			}
 			
-			EXEC.nparam = func->n_param;
+			//EXEC.nparam = func->n_param;
+			nparam = func->n_param;
 		}
 	}
 
@@ -484,7 +489,7 @@ void EXEC_enter(void)
 
 	BP = SP;
 	if (func->vararg)
-		PP = SP - (EXEC.nparam - func->n_param);
+		PP = SP - (nparam - func->n_param);
 	else
 		PP = SP;
 	FP = func;
@@ -493,6 +498,8 @@ void EXEC_enter(void)
 	CP = class;
 	EP = NULL;
 	GP = NULL;
+	
+	PROFILE_ENTER_FUNCTION();
 
 	if (func->error)
 	{
@@ -505,7 +512,7 @@ void EXEC_enter(void)
 		EC = NULL;
 
 	// Reference the object so that it is not destroyed during the function call
-	OBJECT_REF(OP, "EXEC_enter");
+	OBJECT_REF(OP);
 
 	// Local variables initialization
 
@@ -526,7 +533,7 @@ void EXEC_enter(void)
 	RP->type = T_VOID;
 
 	#if DEBUG_STACK
-	printf("| << EXEC_enter()\n");
+	fprintf(stderr, "| << EXEC_enter()\n");
 	print_register();
 	#endif
 }
@@ -555,14 +562,14 @@ void EXEC_enter_quick(void)
 	CLASS *class = EXEC.class;
 
 	#if DEBUG_STACK
-	printf("\n| >> EXEC_enter_quick(%s, %ld, %d)\n", EXEC.class->name, EXEC.index, EXEC.nparam);
+	fprintf(stderr, "\n| >> EXEC_enter_quick(%s, %d, %d)\n", EXEC.class->name, EXEC.index, EXEC.nparam);
 	print_register();
 	#endif
 
 	func = &class->load->func[EXEC.index];
 	#if DEBUG_STACK
 	if (func->debug)
-		printf(" | >> %s\n", func->debug->name);
+		fprintf(stderr, " | >> %s\n", func->debug->name);
 	#endif
 
 	#if SHOW_FUNCTION
@@ -589,9 +596,11 @@ void EXEC_enter_quick(void)
 		EC = PC + func->error;
 	else
 		EC = NULL;
+	
+	PROFILE_ENTER_FUNCTION();
 
 	/* reference the object so that it is not destroyed during the function call */
-	OBJECT_REF(OP, "EXEC_enter_quick");
+	OBJECT_REF(OP);
 
 	/* local variables initialization */
 
@@ -612,7 +621,7 @@ void EXEC_enter_quick(void)
 	RP->type = T_VOID;
 
 	#if DEBUG_STACK
-	printf("| << EXEC_enter()\n");
+	fprintf(stderr, "| << EXEC_enter()\n");
 	print_register();
 	#endif
 }
@@ -669,9 +678,12 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	RELEASE_MANY(SP, n);
 	SP -= nparam;
 	SP += nb;
-	OBJECT_UNREF(OP, "EXEC_leave");
+	OBJECT_UNREF(OP);
 	SP -= nb;
+	
+	PROFILE_LEAVE_FUNCTION();
 	STACK_pop_frame(&EXEC_current);
+	
 	PC += nbyref + 1;
 	return nb;
 }
@@ -720,8 +732,9 @@ static int exec_leave_byref(ushort *pc, int nparam)
 	RELEASE_MANY(SP, n); \
 	SP -= nparam; \
 	SP += nb; \
-	OBJECT_UNREF(OP, "EXEC_leave"); \
+	OBJECT_UNREF(OP); \
 	SP -= nb; \
+	PROFILE_LEAVE_FUNCTION(); \
 	STACK_pop_frame(&EXEC_current); \
 	PC += nbyref + 1; \
 	goto __RETURN_VALUE;
@@ -730,19 +743,21 @@ static int exec_leave_byref(ushort *pc, int nparam)
 void EXEC_leave_drop()
 {
 	int nparam;
-	VALUE ret;
+	//VALUE ret;
 	ushort *pc;
 	int n, nb;
 
 #if DEBUG_STACK
-	printf("| >> EXEC_leave\n");
+	fprintf(stderr, "| >> EXEC_leave\n");
 	print_register();
 #endif
 
 	/* Save the return value. It can be erased by OBJECT_UNREF() */
 
 	//ret = *RP;
-	VALUE_copy(&ret, RP);
+	EXEC_release_return_value();
+
+	//VALUE_copy(&ret, RP);
 	
   pc = STACK_get_previous_pc();
   nparam = FP->n_param;
@@ -755,18 +770,17 @@ void EXEC_leave_drop()
 		n = nparam + (SP - PP);
 		RELEASE_MANY(SP, n);
 
-		OBJECT_UNREF(OP, "EXEC_leave");
+		OBJECT_UNREF(OP);
+		PROFILE_LEAVE_FUNCTION();
 		STACK_pop_frame(&EXEC_current);
 	}
 
-	EXEC_release_return_value();
-
 	SP += nb;
 	
 #if DEBUG_STACK
-	printf("| << EXEC_leave()\n");
+	fprintf(stderr, "| << EXEC_leave()\n");
 	print_register();
-	printf("\n");
+	fprintf(stderr, "\n");
 #endif
 	return;
 }
@@ -779,7 +793,7 @@ void EXEC_leave_keep()
 	int n, nb;
 
 #if DEBUG_STACK
-	printf("| >> EXEC_leave\n");
+	fprintf(stderr, "| >> EXEC_leave\n");
 	print_register();
 #endif
 
@@ -797,7 +811,8 @@ void EXEC_leave_keep()
 		n = nparam + (SP - PP);
 		RELEASE_MANY(SP, n);
 
-		OBJECT_UNREF(OP, "EXEC_leave");
+		OBJECT_UNREF(OP);
+		PROFILE_LEAVE_FUNCTION();
 		STACK_pop_frame(&EXEC_current);
 	}
 
@@ -806,7 +821,7 @@ void EXEC_leave_keep()
 		if (SP[-1].type == T_FUNCTION)
 		{
 			SP--;
-			OBJECT_UNREF(SP->_function.object, "EXEC_leave");
+			OBJECT_UNREF(SP->_function.object);
 		}
 		
 		COPY_VALUE(SP, &ret);
@@ -815,13 +830,17 @@ void EXEC_leave_keep()
 			VALUE_conv_variant(SP);
 		SP++;
 	}
+	else
+	{
+		VALUE_copy(RP, &ret);
+	}
 
 	SP += nb;
 	
 #if DEBUG_STACK
-	printf("| << EXEC_leave()\n");
+	fprintf(stderr, "| << EXEC_leave()\n");
 	print_register();
-	printf("\n");
+	fprintf(stderr, "\n");
 #endif
 	return;
 }
@@ -838,6 +857,7 @@ void EXEC_function_real()
 	STACK_push_frame(&EXEC_current, 0);
 
 	PC = NULL;
+	GP = NULL;
 
 	ON_ERROR(error_EXEC_function_real)
 	{
@@ -845,7 +865,33 @@ void EXEC_function_real()
 	}
 	END_ERROR
 	
-	EXEC_function_loop();
+	if (FP->fast)
+		EXEC_jit_function_loop();
+	else
+		EXEC_function_loop();
+}
+
+void EXEC_jit_function_loop()
+{
+	TRY
+	{
+		(*CP->jit_functions[EXEC.index])();
+	}
+	CATCH
+	{
+		ERROR_set_last(TRUE);
+		
+		ERROR_lock();
+		while (PC != NULL && EC == NULL)
+			EXEC_leave_drop();
+		ERROR_unlock();
+		
+		STACK_pop_frame(&EXEC_current);
+		PROPAGATE();
+	}
+	END_TRY
+	
+	STACK_pop_frame(&EXEC_current);
 }
 
 void EXEC_function_loop()
@@ -886,6 +932,12 @@ void EXEC_function_loop()
 					#endif
 					ERROR_set_last(FALSE);
 					
+					// No need to unwind the Gosub stack until the TRY stack position, because TRY GOSUB is forbidden
+					/*while (GP > EP)
+					{
+						...
+					}*/
+					
 					// The stack is popped until reaching the stack position before the TRY
 					while (SP > EP)
 						POP();
@@ -909,6 +961,9 @@ void EXEC_function_loop()
 					while (SP > (BP + FP->n_local + FP->n_ctrl))
 						POP();
 					
+					// Reset the Gosub stack pointer as all Gosub control variables saved have been released
+					GP = NULL;
+					
 					PC = EC;
 					EC = NULL;
 					retry = TRUE;
@@ -927,6 +982,9 @@ void EXEC_function_loop()
 
 					if (EXEC_debug && !STACK_has_error_handler())
 					{
+						ERROR_hook();
+						DEBUG.Main(TRUE);
+						
 						if (TP && TC)
 						{
 							ERROR_lock();
@@ -942,7 +1000,6 @@ void EXEC_function_loop()
 							ERROR_unlock();
 						}
 
-						DEBUG.Main(TRUE);
 						retry = TRUE;
 					}
 					else
@@ -951,10 +1008,24 @@ void EXEC_function_loop()
 						// - A stack frame that has an error handler.
 						// - A void stack frame created by EXEC_enter_real()
 						
+						// First we leave stack frames for JIT functions
+						// on top of the stack. We have just propagated
+						// past these some lines below.
 						ERROR_lock();
-						while (PC != NULL && EC == NULL)
+						while (PC != NULL && EC == NULL && FP->fast)
 							EXEC_leave_drop();
 						ERROR_unlock();
+						
+						// We can only leave stack frames for non-JIT functions.
+						ERROR_lock();
+						while (PC != NULL && EC == NULL && !FP->fast)
+							EXEC_leave_drop();
+						ERROR_unlock();
+
+						// If the JIT function has set up an exception handler, call that now.
+						// If not, we must still propagate past that JIT function.
+						if (PC != NULL && FP->fast)
+							PROPAGATE();
 
 						// If we got the void stack frame, then we remove it and raise the error again
 						if (PC == NULL)
@@ -1018,7 +1089,8 @@ void EXEC_function_loop()
 		while (retry);
 	}
 
-	STACK_pop_frame(&EXEC_current);
+	if (PC == NULL)
+		STACK_pop_frame(&EXEC_current);
 }
 
 
@@ -1073,11 +1145,8 @@ void EXEC_native_check(bool defined)
 	else \
 	{ \
 		TYPE __type = (_type); \
-		if (TYPE_is_pure_object(_type) && TEMP.type != T_NULL && TEMP.type != T_VOID) \
+		if (TYPE_is_pure_object(__type) && TEMP.type != T_NULL && TEMP.type != T_VOID) \
 		{ \
-			while (((CLASS *)__type)->override) \
-				__type = (TYPE)(((CLASS *)__type)->override); \
-			\
 			if (TEMP.type == T_CLASS) \
 				TEMP._class.class = (CLASS *)__type; \
 			else \
@@ -1122,7 +1191,7 @@ void EXEC_native_quick(void)
 		RELEASE_MANY(SP, nparam);
 
 		SP--;
-		OBJECT_UNREF(SP->_function.object, "EXEC_native (FUNCTION)");
+		OBJECT_UNREF(SP->_function.object);
 
 		SP->type = T_VOID;
 		SP->_void.ptype = T_NULL;
@@ -1134,14 +1203,14 @@ void EXEC_native_quick(void)
 		RELEASE_MANY(SP, nparam);
 
 		SP--;
-		OBJECT_UNREF(SP->_function.object, "EXEC_native (FUNCTION)");
+		OBJECT_UNREF(SP->_function.object);
 		COPY_VALUE(SP, &ret);
 		SP++;
 	}
 
 
 	#if DEBUG_STACK
-	printf("| << EXEC_native: %s (%p)\n", desc->name, &desc);
+	fprintf(stderr, "| << EXEC_native: %s (%p)\n", desc->name, &desc);
 	#endif
 }
 
@@ -1165,7 +1234,7 @@ void EXEC_native(void)
 	VALUE ret;
 
 	#if DEBUG_STACK
-	printf("| >> EXEC_native: %s.%s (%p)\n", EXEC.class->name, desc->name, &desc);
+	fprintf(stderr, "| >> EXEC_native: %s.%s (%p)\n", EXEC.class->name, desc->name, &desc);
 	#endif
 
 	ON_ERROR(error_EXEC_native)
@@ -1272,7 +1341,7 @@ void EXEC_native(void)
 		if (use_stack)
 		{
 			SP--;
-			OBJECT_UNREF(SP->_function.object, "EXEC_native (FUNCTION)");
+			OBJECT_UNREF(SP->_function.object);
 		}
 		
 		PROPAGATE();
@@ -1287,7 +1356,7 @@ void EXEC_native(void)
 		if (use_stack)
 		{
 			SP--;
-			OBJECT_UNREF(SP->_function.object, "EXEC_native (FUNCTION)");
+			OBJECT_UNREF(SP->_function.object);
 		}
 		
 		SP->type = T_VOID;
@@ -1303,7 +1372,7 @@ void EXEC_native(void)
 		if (use_stack)
 		{
 			SP--;
-			OBJECT_UNREF(SP->_function.object, "EXEC_native (FUNCTION)");
+			OBJECT_UNREF(SP->_function.object);
 		
 			if (PCODE_is_variant(*PC) && ret.type != T_VOID)
 				VALUE_conv_variant(&ret);
@@ -1315,7 +1384,7 @@ void EXEC_native(void)
 
 
 	#if DEBUG_STACK
-	printf("| << EXEC_native: %s (%p)\n", desc->name, &desc);
+	fprintf(stderr, "| << EXEC_native: %s (%p)\n", desc->name, &desc);
 	#endif
 }
 
@@ -1429,7 +1498,7 @@ __FUNCTION:
 		EXEC_special(SPEC_UNKNOWN, val->_function.class, val->_function.object, 0, FALSE);
 
 		object = val->_function.object;
-		OBJECT_UNREF(object, "EXEC_object (FUNCTION)");
+		OBJECT_UNREF(object);
 
 		SP--;
 		//*val = *SP;
@@ -1738,7 +1807,7 @@ void *EXEC_create_object(CLASS *class, int np, char *event)
 	CLASS_load(class);
 
 	if (UNLIKELY(class->no_create))
-		THROW(E_CSTATIC, class->name);
+		THROW(E_CSTATIC, CLASS_get_name(class));
 
 	object = OBJECT_new(class, event, ((OP == NULL) ? (OBJECT *)CP : (OBJECT *)OP));
 
@@ -1757,7 +1826,7 @@ void *EXEC_create_object(CLASS *class, int np, char *event)
 	CATCH
 	{
 		// _free() methods should not be called, but we must
-		OBJECT_UNREF(object, "EXEC_create_object");
+		OBJECT_UNREF(object);
 		PROPAGATE();
 //     SP--; /* class */
 //     SP->type = T_NULL;
@@ -1791,8 +1860,8 @@ void EXEC_new(void)
 	if (SP->type == T_CLASS)
 	{
 		class = SP->_class.class;
-		if (UNLIKELY(class->override != NULL))
-			class = class->override;
+		//if (UNLIKELY(class->override != NULL))
+		//	class = class->override;
 	}
 	else if (TYPE_is_string(SP->type))
 	{
@@ -1810,7 +1879,7 @@ void EXEC_new(void)
 	CLASS_load(class);
 
 	if (UNLIKELY(class->no_create))
-		THROW(E_CSTATIC, class->name);
+		THROW(E_CSTATIC, CLASS_get_name(class));
 
 	if (event)
 	{
@@ -1857,7 +1926,7 @@ void EXEC_new(void)
 	{
 		EVENT_leave_name(save);
 		// _free() methods should not be called, but we must
-		OBJECT_UNREF(object, "EXEC_new");
+		OBJECT_UNREF(object);
 		//(*class->free)(class, object);
 		SP--; /* class */
 		SP->type = T_NULL;
@@ -1887,7 +1956,7 @@ void *EXEC_auto_create(CLASS *class, bool ref)
 		THROW(E_IOBJECT);*/
 
 	if (ref)
-		OBJECT_REF(object, "EXEC_auto_create");
+		OBJECT_REF(object);
 	
 	return object;
 }
@@ -1906,3 +1975,32 @@ void EXEC_dup(int n)
 		n--;
 	}
 }
+
+void EXEC_push_complex(void)
+{
+	static void *(*func)(double) = NULL;
+	void *ob;
+	
+	SP--;
+	if (SP->type < T_INTEGER || SP->type > T_FLOAT)
+		THROW_ILLEGAL();
+	SP++;
+	
+	if (!func)
+	{
+		if (COMPONENT_get_info("PUSH_COMPLEX", POINTER(&func)))
+		{
+			COMPONENT_load(COMPONENT_create("gb.complex"));
+			if (COMPONENT_get_info("PUSH_COMPLEX", POINTER(&func)))
+				THROW(E_MATH);
+		}
+	}
+	
+	SP--;
+	VALUE_conv_float(SP);
+	ob = (*func)(SP->_float.value);
+	SP->_object.object = ob;
+	SP->_object.class = OBJECT_class(ob);
+	OBJECT_REF(ob);
+	SP++;
+}
