@@ -2,7 +2,7 @@
 
   gbx_exec_push.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -73,12 +73,12 @@ _PUSH_GENERIC:
 
 	// The first time we access a symbol, we must not be virtual to find it
 	val = &SP[-1];
-	if (LIKELY(defined && object && !VALUE_is_super(val)))
+	if (defined && object && !VALUE_is_super(val))
 		index = CLASS_find_symbol(val->_object.class, name);
 	else
 		index = CLASS_find_symbol(class, name);
 
-	if (UNLIKELY(index == NO_SYMBOL))
+	if (index == NO_SYMBOL)
 	{
 		//index = CLASS_find_symbol(class, name);
 
@@ -86,22 +86,35 @@ _PUSH_GENERIC:
 		{
 			if (defined && object && !VALUE_is_super(val))
 				class = val->_object.class;
-			THROW(E_NSYMBOL, name, class->name);
+			THROW(E_NSYMBOL, CLASS_get_name(class), name);
+		}
+
+		if (class->unknown_static && object)
+			THROW(E_STATIC, CLASS_get_name(class), name);
+		else if (!class->unknown_static && object == NULL)
+		{
+			if (!class->auto_create)
+				THROW(E_DYNAMIC, CLASS_get_name(class), name);
+			object = EXEC_auto_create(class, TRUE);
 		}
 
 		if (class->special[SPEC_PROPERTY] != NO_SYMBOL)
 		{
 			EXEC_unknown_name = name;
-			EXEC_special(SPEC_PROPERTY, class, object, 0, FALSE);
-			VALUE_conv_boolean(&SP[-1]);
-			SP--;
-			if (SP->_boolean.value)
+			if (!EXEC_special(SPEC_PROPERTY, class, class->property_static ? NULL : object, 0, FALSE))
 			{
+				VALUE_conv_boolean(&SP[-1]);
 				SP--;
-				EXEC_special(SPEC_UNKNOWN, class, object, 0, FALSE);
-				VALUE_conv_variant(&SP[-1]);
-				OBJECT_UNREF(object, "EXEC_push_unknown");
-				goto _FIN;
+				if (SP->_boolean.value)
+				{
+					//SP--; Keep the object on the stack so that it is automatically freed if an error is raised
+					EXEC_special(SPEC_UNKNOWN, class, class->unknown_static ? NULL : object, 0, FALSE);
+					VALUE_conv_variant(&SP[-1]);
+					SP--;
+					SP[-1] = *SP;
+					OBJECT_UNREF(object);
+					goto _FIN;
+				}
 			}
 		}
 
@@ -165,8 +178,8 @@ _PUSH_GENERIC:
 
 			if (object == NULL)
 			{
-				if (UNLIKELY(!class->auto_create))
-					THROW(E_DYNAMIC, class->name, name);
+				if (!class->auto_create)
+					THROW(E_DYNAMIC, CLASS_get_name(class), name);
 				object = EXEC_auto_create(class, TRUE);
 				*PC |= 7;
 			}
@@ -183,7 +196,7 @@ _PUSH_GENERIC:
 		case CD_STRUCT_FIELD:
 
 			if (object == NULL)
-				THROW(E_DYNAMIC, class->name, name);
+				THROW(E_DYNAMIC, CLASS_get_name(class), name);
 			
 			if (defined) 
 			{
@@ -196,7 +209,7 @@ _PUSH_GENERIC:
 		case CD_STATIC_VARIABLE:
 
 			if (object)
-				THROW(E_STATIC, class->name, name);
+				THROW(E_STATIC, CLASS_get_name(class), name);
 
 			if (defined) *PC |= 3;
 
@@ -210,8 +223,8 @@ _PUSH_GENERIC:
 
 			if (object == NULL)
 			{
-				if (UNLIKELY(!class->auto_create))
-					THROW(E_DYNAMIC, class->name, name);
+				if (!class->auto_create)
+					THROW(E_DYNAMIC, CLASS_get_name(class), name);
 				object = EXEC_auto_create(class, TRUE);
 				*PC |= 8;
 			}
@@ -229,7 +242,7 @@ _PUSH_GENERIC:
 		case CD_STATIC_PROPERTY_READ:
 
 			if (object)
-				THROW(E_STATIC, class->name, name);
+				THROW(E_STATIC, CLASS_get_name(class), name);
 
 			if (defined) *PC |= 4;
 
@@ -242,8 +255,8 @@ _PUSH_GENERIC:
 
 			if (object == NULL)
 			{
-				if (UNLIKELY(!class->auto_create))
-					THROW(E_DYNAMIC, class->name, name);
+				if (!class->auto_create)
+					THROW(E_DYNAMIC, CLASS_get_name(class), name);
 				object = EXEC_auto_create(class, TRUE);
 				*PC |= 9;
 			}
@@ -261,7 +274,7 @@ _PUSH_GENERIC:
 
 			if (object)
 			{
-				OBJECT_UNREF(object, "EXEC_push_unknown");
+				OBJECT_UNREF(object);
 				object = NULL;
 			}
 
@@ -276,7 +289,7 @@ _PUSH_GENERIC:
 
 			if (object)
 			{
-				OBJECT_UNREF(object, "EXEC_push_unknown");
+				OBJECT_UNREF(object);
 				object = NULL;
 			}
 
@@ -289,7 +302,7 @@ _PUSH_GENERIC:
 
 		default:
 
-			THROW(E_NSYMBOL, name, class->name);
+			THROW(E_NSYMBOL, CLASS_get_name(class), name);
 	}
 
 	
@@ -312,13 +325,22 @@ _PUSH_CONST_STRING_2:
 	SP--;
 	SP->type = T_CSTRING;
 	
-	if (desc->constant.translate)
-		SP->_string.addr = (char *)LOCAL_gettext(desc->constant.value._string);
+	if (!desc->constant.value._string)
+	{
+		SP->_string.addr = NULL;
+		SP->_string.len = 0;
+	}
 	else
-		SP->_string.addr = (char *)desc->constant.value._string;
+	{
+		if (desc->constant.translate)
+			SP->_string.addr = (char *)LOCAL_gettext(desc->constant.value._string);
+		else
+			SP->_string.addr = (char *)desc->constant.value._string;
+	
+		SP->_string.len = strlen(SP->_string.addr);
+	}
 	
 	SP->_string.start = 0;
-	SP->_string.len = strlen(SP->_string.addr);
 	SP++;
 	goto _FIN_DEFINED_NO_BORROW;
 
@@ -410,7 +432,7 @@ _PUSH_STATIC_METHOD:
 
 	if (object)
 	{
-		OBJECT_UNREF(object, "EXEC_push_unknown");
+		OBJECT_UNREF(object);
 		object = NULL;
 	}
 
@@ -458,7 +480,7 @@ _PUSH_EXTERN:
 
 	if (object)
 	{
-		OBJECT_UNREF(object, "EXEC_push_unknown");
+		OBJECT_UNREF(object);
 		object = NULL;
 	}
 
@@ -491,7 +513,7 @@ _PUSH_UNKNOWN_METHOD:
 	SP->_function.defined = FALSE;
 	SP++;
 
-	//OBJECT_REF(&object, "EXEC_push_unknown: UNKNOWN");
+	//OBJECT_REF(&object);
 
 	goto _FIN;
 
@@ -506,7 +528,7 @@ _FIN_DEFINED_NO_BORROW:
 
 	// SP[-1] was the object and it has been erased. So we must unref it manually,
 	// unless we are calling a static method (see above)
-	OBJECT_UNREF(object, "EXEC_push_unknown");
+	OBJECT_UNREF(object);
 
 _FIN:
 
@@ -537,18 +559,25 @@ __PUSH_GENERIC:
 
 	defined = EXEC_object(val, &class, &object);
 	
-	// The first time we access a symbol, we must not be virtual to find it
-	if (LIKELY(defined && object && !VALUE_is_super(val)))
-		class = val->_object.class;
-	
 	fast = 3;
-	
-	if (LIKELY(defined))
+
+	if (defined)
 	{
 		if (class->quick_array == CQA_ARRAY)
 			fast = 1;
 		else if (class->quick_array == CQA_COLLECTION)
 			fast = 2;
+		else
+		{
+			// Check the symbol existance, but *not virtually*
+			if (object && !VALUE_is_super(val))
+			{
+				CLASS *nvclass = val->_object.class;
+
+				if (nvclass->special[SPEC_GET] == NO_SYMBOL)
+					THROW(E_NARRAY, CLASS_get_name(nvclass));
+			}
+		}
 	}
 
 	*PC |= fast << 6;
@@ -579,7 +608,7 @@ __PUSH_QUICK_ARRAY:
 	
 	VALUE_conv_integer(&val[1]);
 	
-	if (LIKELY(np == 1))
+	if (np == 1)
 	{
 		data = CARRAY_get_data((CARRAY *)object, val[1]._integer.value);
 	}
@@ -612,7 +641,7 @@ __PUSH_QUICK_END:
 	
 	SP = val;
 	PUSH();
-	OBJECT_UNREF(object, "EXEC_push_array");
+	OBJECT_UNREF(object);
 	return;
 	
 __PUSH_ARRAY:
@@ -621,14 +650,49 @@ __PUSH_ARRAY:
 	
 __PUSH_ARRAY_2:
 
-	if (UNLIKELY(EXEC_special(SPEC_GET, class, object, np, FALSE)))
-		THROW(E_NARRAY, class->name);
+	if (EXEC_special(SPEC_GET, class, object, np, FALSE))
+		THROW(E_NARRAY, CLASS_get_name(class));
 
-	OBJECT_UNREF(object, "EXEC_push_array");
+	OBJECT_UNREF(object);
 	SP--;
 	//SP[-1] = SP[0];
 	VALUE_copy(&SP[-1], &SP[0]);
 
-	if (UNLIKELY(!defined))
+	if (!defined)
 		VALUE_conv_variant(&SP[-1]);
 }
+
+int EXEC_push_unknown_event(bool unknown)
+{
+	int index;
+	CLASS_DESC *desc;
+	const char *name;
+	
+	if (unknown)
+	{
+		name = CP->load->unknown[PC[1]];
+		// The ':' is already in the name, thanks to the compiler.
+		index = CLASS_find_symbol(CP, name);
+		if (index == NO_SYMBOL)
+			THROW(E_DYNAMIC, CLASS_get_name(CP), name);
+
+		desc = CP->table[index].desc;
+		if (CLASS_DESC_get_type(desc) != CD_EVENT)
+			THROW(E_DYNAMIC, CLASS_get_name(CP), name);
+		
+		PC[1] = index;
+		PC[0] &= ~1;
+	}
+	else
+	{
+		desc = CP->table[PC[1]].desc;
+	}
+	
+	index = desc->event.index;
+	
+	//if (desc->event.class->parent)
+	//	index += desc->event.class->parent->n_event;
+	
+	return index;
+}
+
