@@ -2,7 +2,7 @@
 
   gbx_math.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,14 +23,18 @@
 
 #define __GBX_MATH_C
 
+#include "gb_common.h"
+
 #include <math.h>
 #include <time.h>
 #include <sys/time.h>
 
-#include "gb_common.h"
 #include "gb_hash.h"
 #include "gbx_math.h"
 
+const double MATH_pow10_double[] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000};
+static const uint _pow10_uint[] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000, 1000000000};
+
 /* This is a twisted generalized feedback shift register
    that generates pseudo-random numbers.
 
@@ -126,37 +130,6 @@ double fix(double x)
     return -floor(fabs(x));
 }
 
-#if 0
-double frexp10_old(double x, int *exp)
-{
-  long double l, f;
-
-  if (x == 0.0)
-  {
-    *exp = 0;
-    return x;
-  }
-
-  l = modfl(log10l(fabsl(x)), &f);
-  
-  if (f == 0.0)
-    l = x;
-  else
-    l = powl(10, l);
-
-  if (x < 0.0) l = -l;
-
-  while (l >= 1.0)
-  {
-    l /= 10.0;
-    f++;
-  }
-
-  *exp = (int)f;
-  return l;
-}
-#endif
-
 double frexp10(double x, int *exp)
 {
 	int p;
@@ -168,7 +141,7 @@ double frexp10(double x, int *exp)
 	}
 	
 	p = (int)log10(x);
-	x /= pow(10, p);
+	x /= pow10(p);
 
 	if (x >= 1)
 	{
@@ -230,36 +203,6 @@ double exp2(double x)
 }
 #endif
 
-#ifndef HAVE_LOG10L
-long double log10l(long double x)
-{
-	return log10((double) x);
-}
-#endif
-
-#ifndef HAVE_FABSL
-long double fabsl(long double x)
-{
-	return fabs((double) x);
-}
-#endif
-
-#ifndef HAVE_POWL
-long double powl(long double x, long double y)
-{
-	return pow((double) x, (double) y);
-}
-#endif
-
-#ifndef HAVE_MODFL
-long double modfl(long double x, long double *iptr)
-{
-	double val;
-	return modf((double)x, &val);
-	*iptr = val;
-}
-#endif
-
 static int nbits(uint n)
 {
   uint c;
@@ -281,3 +224,17 @@ void MATH_init(void)
 		
 	HASH_seed = seed;
 }
+
+uint64_t pow10_uint64_p(int n)
+{
+	uint64_t v = 1;
+
+	while (n > 8)
+	{
+		v *= 100000000;
+		n -= 8;
+	}
+	v *= _pow10_uint[n];
+	return v;
+}
+
