@@ -2,7 +2,7 @@
 
   gbx_object.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -52,9 +52,13 @@ typedef
 #define OBJECT_event(_object) ((OBJECT_EVENT *)((intptr_t *)_object + ((OBJECT *)(_object))->class->off_event / sizeof(intptr_t)))
 #define OBJECT_is(_object, _class) (OBJECT_class(_object) == _class)
 #define OBJECT_is_class(_object) OBJECT_is(_object, CLASS_Class)
-#define OBJECT_class(_object) ((_object) ? ((OBJECT *)_object)->class : NULL)
-#define OBJECT_count(_object) ((_object) ? ((OBJECT *)_object)->ref : 0)
+#define OBJECT_class(_object) (((OBJECT *)_object)->class)
+#define OBJECT_class_null(_object) ((_object) ? ((OBJECT *)_object)->class : NULL)
+#define OBJECT_count(_object) (((OBJECT *)_object)->ref)
 
+#define OBJECT_are_null(_o1, _o2) (((intptr_t)(_o1) | (intptr_t)(_o2)) == 0)
+#define OBJECT_are_not_null(_o1, _o2) ((_o1) && (_o2))
+	
 void *OBJECT_new(CLASS *class, const char *name, OBJECT *parent);
 void OBJECT_attach(OBJECT *ob, OBJECT *parent, const char *name);
 void OBJECT_detach(OBJECT *ob);
@@ -103,6 +107,8 @@ static INLINE CLASS *OBJECT_class(void *object)
 
 EXTERN const char *OBJECT_ref_where;
 
+char *OBJECT_where_am_i(const char *file, int line, const char *func);
+
 #define OBJECT_ref(_object) \
 { \
 	if (_object) \
@@ -158,9 +164,9 @@ EXTERN const char *OBJECT_ref_where;
 	} \
 }
 
-#define OBJECT_REF(_ob, _where) { OBJECT_ref_where = "REF @" _where; OBJECT_ref(_ob); }
-#define OBJECT_UNREF(_ob, _where) { OBJECT_ref_where = "UNREF @" _where; OBJECT_unref(_ob); }
-#define OBJECT_UNREF_KEEP(_ob, _where) { OBJECT_ref_where = "UNREF_KEEP @" _where; OBJECT_unref_keep(_ob); }
+#define OBJECT_REF(_ob) { OBJECT_ref_where = OBJECT_where_am_i(__FILE__, __LINE__, __func__); OBJECT_ref(_ob); }
+#define OBJECT_UNREF(_ob) { OBJECT_ref_where = OBJECT_where_am_i(__FILE__, __LINE__, __func__); OBJECT_unref(_ob); }
+#define OBJECT_UNREF_KEEP(_ob) { OBJECT_ref_where = OBJECT_where_am_i(__FILE__, __LINE__, __func__); OBJECT_unref_keep(_ob); }
 
 #else /* DEBUG_REF */
 
@@ -200,9 +206,9 @@ EXTERN const char *OBJECT_ref_where;
 }
 
 
-#define OBJECT_REF(_ob, _where) OBJECT_ref(_ob)
-#define OBJECT_UNREF(_ob, _where) OBJECT_unref(_ob)
-#define OBJECT_UNREF_KEEP(_ob, _where) OBJECT_unref_keep(_ob)
+#define OBJECT_REF(_ob) OBJECT_ref(_ob)
+#define OBJECT_UNREF(_ob) OBJECT_unref(_ob)
+#define OBJECT_UNREF_KEEP(_ob) OBJECT_unref_keep(_ob)
 
 #endif /* DEBUG_REF */
 
@@ -222,6 +228,4 @@ static INLINE void OBJECT_put(VALUE *value, void *object)
 	value->_object.object = object;
 }
 
-
-
 #endif
