@@ -2,7 +2,7 @@
 
   gbx_stream.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -74,7 +74,11 @@ void STREAM_exit(void)
 	STREAM_close(&_temp_stream);
 }
 
-#define wait_for_fd_ready_to_read(_fd) WATCH_process(_fd, -1)
+static void wait_for_fd_ready_to_read(fd)
+{
+	if (fd >= 0)
+		WATCH_process(fd, -1, 0);
+}
 
 bool STREAM_in_archive(const char *path)
 {
@@ -90,13 +94,15 @@ bool STREAM_in_archive(const char *path)
 	return FALSE;
 }
 
-static int STREAM_get_readable(STREAM *stream, int *len)
+int STREAM_get_readable(STREAM *stream, int *len)
 {
 	int fd;
 	off_t off;
 	off_t end;
 
 	fd = STREAM_handle(stream);
+	if (fd < 0)
+		return TRUE;
 	
 //_IOCTL:
 
@@ -242,7 +248,7 @@ void STREAM_release(STREAM *stream)
 		#if DEBUG_STREAM
 		fprintf(stderr, "Stream %p [%d]: Free buffer\n", stream, stream->common.tag);
 		#endif
-		FREE(&stream->common.buffer, "STREAM_close");
+		FREE(&stream->common.buffer);
 		stream->common.buffer_pos = 0;
 		stream->common.buffer_len = 0;
 	}
@@ -250,18 +256,21 @@ void STREAM_release(STREAM *stream)
 	STREAM_cancel(stream);
 }
 
+static void stop_watching(STREAM *stream, int mode)
+{
+	int fd = STREAM_handle(stream);
+	if (fd >= 0)
+		GB_Watch(fd, mode, NULL, 0);
+}
+
 void STREAM_close(STREAM *stream)
 {
-	int fd;
-
 	STREAM_release(stream);
 
 	if (!stream->type)
 		return;
 
-	fd = STREAM_handle(stream);
-	if (fd >= 0)
-		GB_Watch(fd, GB_WATCH_NONE, NULL, 0);
+	stop_watching(stream, GB_WATCH_NONE);
 
 	if (stream->common.standard || !(*(stream->type->close))(stream))
 		stream->type = NULL;
@@ -275,9 +284,11 @@ void STREAM_close(STREAM *stream)
 
 void STREAM_flush(STREAM *stream)
 {
+	STREAM_end(stream);
+	
 	if (!stream->type)
 		THROW(E_CLOSED);
-		
+	
 	(*(stream->type->flush))(stream);
 }
 
@@ -300,7 +311,7 @@ void STREAM_read(STREAM *stream, void *addr, int len)
 {
 	STREAM_eff_read = 0;
 
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 	
 	if (len <= 0)
@@ -311,10 +322,13 @@ void STREAM_read(STREAM *stream, void *addr, int len)
 	
 	while ((*(stream->type->read))(stream, addr, len))
 	{
+		if (errno == EINTR)
+			continue;
+
+		stop_watching(stream, GB_WATCH_READ);
+
 		switch(errno)
 		{
-			case EINTR:
-				break;
 			case 0:
 			case EAGAIN:
 				THROW(E_EOF);
@@ -347,10 +361,13 @@ char STREAM_getchar(STREAM *stream)
 		if (!ret)
 			break;
 		
+		if (errno == EINTR)
+			continue;
+
+		stop_watching(stream, GB_WATCH_READ);
+
 		switch(errno)
 		{
-			case EINTR:
-				continue;
 			case 0:
 			case EAGAIN:
 				THROW(E_EOF);
@@ -409,11 +426,15 @@ int STREAM_read_max(STREAM *stream, void *addr, int len)
 		
 			if (!err)
 				break;
-				
+
+			if (errno == EINTR)
+				continue;
+
+			if (errno == 0)
+				stop_watching(stream, GB_WATCH_READ);
+
 			switch(errno)
 			{
-				case EINTR:
-					continue;
 				case 0:
 				case EAGAIN:
 					return STREAM_eff_read;
@@ -431,7 +452,7 @@ int STREAM_read_max(STREAM *stream, void *addr, int len)
 
 void STREAM_write(STREAM *stream, void *addr, int len)
 {
-	if (!stream->type)
+	if (STREAM_is_closed_for_writing(stream))
 		THROW(E_CLOSED);
 	
 	if (len <= 0)
@@ -469,7 +490,7 @@ void STREAM_write_zeros(STREAM *stream, int len)
 
 void STREAM_write_eol(STREAM *stream)
 {
-	if (!stream->type)
+	if (STREAM_is_closed_for_writing(stream))
 		THROW(E_CLOSED);
 		
 	switch(stream->common.eol)
@@ -485,7 +506,7 @@ int64_t STREAM_tell(STREAM *stream)
 {
 	int64_t pos;
 
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	if (stream->type->tell(stream, &pos))
@@ -498,7 +519,7 @@ int64_t STREAM_tell(STREAM *stream)
 
 void STREAM_seek(STREAM *stream, int64_t pos, int whence)
 {
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	if (stream->type->seek(stream, pos, whence))
@@ -536,16 +557,24 @@ static void fill_buffer(STREAM *stream, char *addr)
 			err = (*(stream->type->read))(stream, addr, STREAM_BUFFER_SIZE);
 			
 			if ((flags & O_NONBLOCK) == 0)
+			{
+				int save_errno = errno;
 				fcntl(fd, F_SETFL, flags);
+				errno = save_errno;
+			}
 		}
 	
 		if (!err)
 			break;
-			
+
+		if (errno == EINTR)
+			continue;
+
+		if (STREAM_eff_read == 0)
+			stop_watching(stream, GB_WATCH_READ);
+
 		switch(errno)
 		{
-			case EINTR:
-				continue;
 			case 0:
 			case EAGAIN:
 			case EINPROGRESS:
@@ -604,7 +633,7 @@ static char *input(STREAM *stream, bool line, char *escape)
 		#if DEBUG_STREAM
 		fprintf(stderr, "Stream %p [%d]: Alloc buffer\n", stream, stream->common.tag);
 		#endif
-		ALLOC(&buffer, STREAM_BUFFER_SIZE, "input");
+		ALLOC(&buffer, STREAM_BUFFER_SIZE);
 		buffer_pos = 0;
 		buffer_len = 0;
 	}
@@ -803,7 +832,7 @@ static void read_value_ctype(STREAM *stream, CLASS *class, CTYPE ctype, void *ad
 		if (CLASS_is_struct(class))
 		{
 			CSTRUCT *structure = (CSTRUCT *)OBJECT_create(class, NULL, NULL, 0);
-			OBJECT_REF(structure, "read_value_ctype");
+			OBJECT_REF(structure);
 			read_structure(stream, class, (char *)structure + sizeof(CSTRUCT));
 			*((void **)addr) = structure;
 			return;
@@ -1039,15 +1068,13 @@ void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value)
 
 			if (stream->type == &STREAM_memory)
 			{
-				size_t slen;
+				ssize_t slen;
 				if (CHECK_strlen(stream->memory.addr + stream->memory.pos, &slen))
 					THROW(E_READ);
 				len = (int)slen;
 			}
 			else
-			{
 				len = read_length(stream);
-			}
 
 			if (len > 0)
 			{
@@ -1055,7 +1082,9 @@ void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value)
 				STREAM_read(stream, value->_string.addr, len);
 			}
 			else
-				value->type = T_NULL;
+			{
+				STRING_void_value(value);
+			}
 
 			break;
 
@@ -1214,8 +1243,17 @@ void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value)
 		case T_STRING:
 		case T_CSTRING:
 
-			write_length(stream, value->_string.len);
-			STREAM_write(stream, value->_string.addr + value->_string.start, value->_string.len);
+			if (stream->type == &STREAM_memory)
+			{
+				STREAM_write(stream, value->_string.addr + value->_string.start, value->_string.len);
+				buffer._byte = 0;
+				STREAM_write(stream, &buffer._byte, 1);
+			}
+			else
+			{
+				write_length(stream, value->_string.len);
+				STREAM_write(stream, value->_string.addr + value->_string.start, value->_string.len);
+			}
 			break;
 			
 		case T_OBJECT:
@@ -1253,9 +1291,9 @@ void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value)
 						structure = CSTRUCT_create_static(array, (CLASS *)array->type, data);
 						temp._object.class = OBJECT_class(structure);
 						temp._object.object = structure;
-						OBJECT_REF(structure, "STREAM_write_type");
+						OBJECT_REF(structure);
 						STREAM_write_type(stream, T_OBJECT, &temp);
-						OBJECT_UNREF(structure, "STREAM_write_type");
+						OBJECT_UNREF(structure);
 					}
 				}
 				else
@@ -1356,7 +1394,7 @@ void STREAM_load(const char *path, char **buffer, int *rlen)
 
 	*rlen = len;
 
-	ALLOC(buffer, *rlen, "STREAM_load");
+	ALLOC(buffer, *rlen);
 
 	STREAM_read(&stream, *buffer, *rlen);
 	STREAM_close(&stream);
@@ -1415,7 +1453,7 @@ void STREAM_unmap(char *addr, int len)
 
 int STREAM_handle(STREAM *stream)
 {
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	if (stream->type->handle)
@@ -1425,17 +1463,17 @@ int STREAM_handle(STREAM *stream)
 }
 
 
-void STREAM_lock(STREAM *stream)
+bool STREAM_lock(STREAM *stream)
 {
 	int64_t pos;
 	int fd;
 
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	fd = STREAM_handle(stream);
 	if (fd < 0)
-		THROW(E_LOCK);
+		return TRUE;
 
 	pos = lseek(fd, 0, SEEK_CUR);
 	if (pos < 0)
@@ -1449,24 +1487,26 @@ void STREAM_lock(STREAM *stream)
 		if (lockf(fd, F_TLOCK, 0))
 		{
 			if (errno == EAGAIN)
-				THROW(E_LOCK);
+				return TRUE;
 			else
 				goto __ERROR;
 		}
 		
 	#else
 	
-		fprintf(stderr, "locking is not implemented\n");
+		ERROR_warning("locking is not implemented");
 	
 	#endif
 
 	if (lseek(fd, pos, SEEK_SET) < 0)
 		goto __ERROR;
 
-	return;
+	return FALSE;
 
 __ERROR:
+
 	THROW_SYSTEM(errno, NULL);
+	return TRUE;
 }
 
 
@@ -1476,7 +1516,7 @@ void STREAM_lof(STREAM *stream, int64_t *len)
 	int fd;
 	int ilen;
 
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	if (stream->type->lof)
@@ -1495,7 +1535,7 @@ void STREAM_lof(STREAM *stream, int64_t *len)
 
 bool STREAM_eof(STREAM *stream)
 {
-	if (!stream->type)
+	if (STREAM_is_closed(stream))
 		THROW(E_CLOSED);
 		
 	if (stream->common.buffer && stream->common.buffer_pos < stream->common.buffer_len)
@@ -1589,7 +1629,7 @@ void STREAM_cancel(STREAM *stream)
 		return;
 
 	STREAM_close(stream->common.redirect);
-	FREE(&stream->common.redirect, "STREAM_cancel");
+	FREE(&stream->common.redirect);
 	stream->common.redirected = FALSE;
 }
 
@@ -1599,7 +1639,7 @@ void STREAM_begin(STREAM *stream)
 	
 	if (!stream->common.redirect)
 	{
-		ALLOC_ZERO(&stream->common.redirect, sizeof(STREAM), "STREAM_begin");
+		ALLOC_ZERO(&stream->common.redirect, sizeof(STREAM));
 		STREAM_open(stream->common.redirect, NULL, ST_STRING | ST_WRITE);
 	}
 
