@@ -2,7 +2,7 @@
 
   gbx_string.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -64,18 +64,18 @@ static void print_where()
 static void *_my_malloc(int size)
 {
 	void *ptr;
-	ALLOC(&ptr, size, "_my_malloc");
+	ALLOC(&ptr, size);
 	return ptr;
 }
 
 static void _my_free(void *ptr)
 {
-	IFREE(ptr, "_my_free");
+	IFREE(ptr);
 }
 
 static void *_my_realloc(void *ptr, int size)
 {
-	REALLOC(&ptr, size, "_my_realloc");
+	REALLOC(&ptr, size);
 	return ptr;
 }
 #else
@@ -198,12 +198,13 @@ void STRING_free_real(char *ptr)
 	int pool = (size / SIZE_INC) - 1;
 
 	if (STRING_utf8_current == ptr)
+	{
+		//fprintf(stderr, "free STRING_utf8_current (%p)\n", ptr);
 		STRING_utf8_current = NULL;
+	}
 	
 	MEMORY_count--;
 	
-	//fprintf(stderr, "STRING_free_real: %p '%s'\n", ptr, ptr);
-	
 	if (pool < POOL_SIZE)
 	{
 		if (_pool_count[pool] < POOL_MAX)		
@@ -411,14 +412,14 @@ char *STRING_extend(char *str, int new_len)
 	{
 		sstr = alloc_string(new_len);
 		#ifdef DEBUG_ME
-			fprintf(stderr, "STRING_extend: NULL -> %p / %p\n", str, str->data);
+			fprintf(stderr, "STRING_extend: NULL -> %p / %p\n", sstr, sstr->data);
 		#endif
 	}
 	else
 	{
 		sstr = realloc_string(STRING_from_ptr(str), new_len);
 		#ifdef DEBUG_ME
-			fprintf(stderr, "STRING_extend: %p / %p -> %p / %p\n", STRING_from_ptr(*ptr), *ptr, str, str->data);
+			fprintf(stderr, "STRING_extend: %p / %p -> %p / %p\n", STRING_from_ptr(str), str, sstr, sstr->data);
 		#endif
 	}
 
@@ -426,6 +427,24 @@ char *STRING_extend(char *str, int new_len)
 }
 
 
+bool STRING_extend_will_realloc(char *str, int new_len)
+{
+	STRING *sstr;
+	int size;
+	int new_size;
+	
+	if (!str)
+		return new_len != 0;
+	
+	sstr = STRING_from_ptr(str);
+	
+	if (new_len == sstr->len)
+		return FALSE;
+		
+	size = REAL_SIZE(sstr->len + 1 + sizeof(STRING));
+	new_size = REAL_SIZE(new_len + 1 + sizeof(STRING));
+	return size != new_size;
+}
 
 
 void STRING_new_temp_value(VALUE *value, const char *src, int len)
@@ -1186,6 +1205,7 @@ char *STRING_conv_file_name(const char *name, int len)
 	struct passwd *info;
 	char *dir;
 	char *user;
+	int err;
 
 	if (!name)
 		return "";
@@ -1227,8 +1247,11 @@ char *STRING_conv_file_name(const char *name, int len)
 	if (LOCAL_is_UTF8)
 		result = STRING_new_temp(name, len);
 	else
-		STRING_conv(&result, name, len, SC_UTF8, LOCAL_encoding, TRUE);
-
+	{
+		err = STRING_conv(&result, name, len, SC_UTF8, LOCAL_encoding, FALSE);
+		if (err)
+			result = STRING_new_temp(name, len);
+	}
 	//fprintf(stderr, "STRING_conv_file_name: %s\n", result);
 
 	if (result)
