@@ -2,7 +2,7 @@
 
   gbx_subr.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -147,11 +147,11 @@ char *SUBR_get_string(VALUE *param)
 }
 
 
-char *SUBR_copy_string(VALUE *param)
+/*char *SUBR_copy_string(VALUE *param)
 {
   SUBR_check_string(param);
   return STRING_copy_from_value_temp(param);
-}
+}*/
 
 
 void SUBR_get_string_len(VALUE *param, char **str, int *len)
