@@ -2,7 +2,7 @@
 
 	gbx_subr_file.c
 
-	(c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -58,9 +58,9 @@ static void push_stream(void **list, CSTREAM *stream)
 {
 	CSTREAM_NODE *slot;
 
-	ALLOC(&slot, sizeof(CSTREAM_NODE), "push_stream");
+	ALLOC(&slot, sizeof(CSTREAM_NODE));
 	slot->stream = stream;
-	//OBJECT_REF(stream, "push_stream");
+	//OBJECT_REF(stream);
 
 	slot->next = *list;
 	*list = slot;
@@ -78,7 +78,7 @@ static CSTREAM *pop_stream(void **list)
 	stream = ((CSTREAM_NODE *)*list)->stream;
 	slot = *list;
 	*list = slot->next;
-	FREE(&slot, "pop_stream");
+	FREE(&slot);
 
 	return stream;
 }
@@ -114,8 +114,7 @@ static STREAM *get_default(intptr_t val)
 	return stream;
 }
 
-#define get_stream(_value, _can_default) \
-({ \
+#define _get_stream(_value, _can_default) \
 	STREAM *stream; \
 	\
 	VARIANT_undo(_value); \
@@ -134,7 +133,11 @@ static STREAM *get_default(intptr_t val)
 			VALUE_conv_object((_value), (TYPE)CLASS_Stream); \
 			stream = NULL; \
 		} \
-	} \
+	}
+
+#define get_stream(_value, _can_default) \
+({ \
+	_get_stream(_value, _can_default); \
 	\
 	if (STREAM_is_closed(stream)) \
 		THROW(E_CLOSED); \
@@ -142,6 +145,15 @@ static STREAM *get_default(intptr_t val)
 	stream; \
 })
 
+#define get_stream_for_writing(_value, _can_default) \
+({ \
+	_get_stream(_value, _can_default); \
+	\
+	if (STREAM_is_closed_for_writing(stream)) \
+		THROW(E_CLOSED); \
+	\
+	stream; \
+})
 
 static char *get_path(VALUE *param)
 {
@@ -174,6 +186,28 @@ void SUBR_open(ushort code)
 		
 		STREAM_open(&stream, (char *)addr, mode | ST_MEMORY);
 	}
+	else if (mode & ST_STRING)
+	{
+		char *str;
+
+		STREAM_open(&stream, NULL, mode);
+
+		if (!VALUE_is_null(PARAM))
+		{
+			str = SUBR_get_string(PARAM);
+			
+			if (mode & ST_WRITE)
+			{
+				stream.string.buffer = STRING_new(str, STRING_length(str));
+			}
+			else
+			{
+				stream.string.buffer = str;
+				STRING_ref(str);
+			}
+			stream.string.size = STRING_length(str);
+		}
+	}
 	else
 	{
 		STREAM_open(&stream, get_path(PARAM), mode);
@@ -189,11 +223,34 @@ void SUBR_open(ushort code)
 
 void SUBR_close(void)
 {
+	STREAM *stream;
+
 	SUBR_ENTER_PARAM(1);
 
-	STREAM_close(get_stream(PARAM, FALSE));
+	stream = get_stream(PARAM, FALSE);
 
-	SUBR_LEAVE_VOID();
+	if (stream->type == &STREAM_string)
+	{
+		char *buffer = stream->string.buffer;
+
+		RETURN->type = T_STRING;
+		RETURN->_string.addr = buffer;
+		RETURN->_string.start = 0;
+		RETURN->_string.len = stream->string.size;
+
+		STRING_ref(buffer);
+		STREAM_close(stream);
+		STRING_free_later(buffer);
+
+		//fprintf(stderr, "buffer ref = %d\n", STRING_from_ptr(buffer)->ref);
+
+		SUBR_LEAVE();
+	}
+	else
+	{
+		STREAM_close(stream);
+		SUBR_LEAVE_VOID();
+	}
 }
 
 
@@ -223,7 +280,7 @@ void SUBR_print(ushort code)
 	if (NPARAM < 1)
 		THROW(E_NEPARAM);
 
-	_stream = get_stream(PARAM, TRUE);
+	_stream = get_stream_for_writing(PARAM, TRUE);
 
 	//PRINT_init(print_it, FALSE);
 
@@ -455,7 +512,7 @@ void SUBR_write(ushort code)
 
 	SUBR_ENTER_PARAM(3);
 
-	stream = get_stream(PARAM, TRUE);
+	stream = get_stream_for_writing(PARAM, TRUE);
 
 	if (code & 0x3F)
 	{
@@ -517,7 +574,7 @@ void SUBR_stat(ushort code)
 	FILE_stat(path, &info, follow);
 
 	cstat = OBJECT_new(CLASS_Stat, NULL, NULL);
-	OBJECT_UNREF_KEEP(cstat, "SUBR_stat");
+	OBJECT_UNREF_KEEP(cstat);
 	cstat->info = info;
 	cstat->path = STRING_new_zero(path);
 
@@ -694,9 +751,31 @@ void SUBR_kill(ushort code)
 }
 
 
-void SUBR_mkdir(void)
+void SUBR_mkdir(ushort code)
 {
-	SUBR_kill(1);
+	SUBR_ENTER_PARAM(1);
+	
+	switch (code & 0xFF)
+	{
+		case 0: // Deprecated Mkdir
+			SUBR_kill(1);
+			return;
+			
+		case 1: // Even
+			VALUE_conv(PARAM, T_LONG);
+			PARAM->type = GB_T_BOOLEAN;
+			PARAM->_boolean.value = (PARAM->_long.value & 1) == 0 ? -1 : 0;
+			break;
+
+		case 2: // Odd
+			VALUE_conv(PARAM, T_LONG);
+			PARAM->type = GB_T_BOOLEAN;
+			PARAM->_boolean.value = (PARAM->_long.value & 1) != 0 ? -1 : 0;
+			break;
+		
+		default:
+			THROW_ILLEGAL();
+	}
 }
 
 
@@ -767,17 +846,17 @@ void SUBR_link(ushort code)
 	
 	switch (code & 0xFF)
 	{
-		case 0:
+		case 0: // Deprecated Link
 			SUBR_move(2);
 			return;
 			
-		case 1:
+		case 1: // IsNan
 			VALUE_conv(PARAM, T_FLOAT);
 			PARAM->type = GB_T_BOOLEAN;
 			PARAM->_boolean.value = isnan(PARAM->_float.value) ? -1 : 0;
 			break;
 
-		case 2:
+		case 2: // IsInf
 			VALUE_conv(PARAM, T_FLOAT);
 			PARAM->type = GB_T_INTEGER;
 			PARAM->_integer.value = isinf(PARAM->_float.value);
@@ -865,8 +944,12 @@ void SUBR_lock(ushort code)
 			THROW(E_BADPATH);
 
 		STREAM_open(&stream, path, ST_WRITE | ST_CREATE | ST_DIRECT);
-		STREAM_lock(&stream);
-		
+		if (STREAM_lock(&stream))
+		{
+			STREAM_close(&stream);
+			THROW(E_LOCK);
+		}
+
 		file = CFILE_create(&stream, ST_WRITE | ST_CREATE | ST_DIRECT);
 		OBJECT_put(RETURN, file);
 		SUBR_LEAVE();
@@ -892,14 +975,14 @@ void SUBR_inp_out(ushort code)
 	{
 		stream = pop_stream(where);
 		if (stream)
-			OBJECT_UNREF(stream, "SUBR_inp_out");
+			OBJECT_UNREF(stream);
 		return;
 	}
 
 	VALUE_conv_object(PARAM, (TYPE)CLASS_Stream);
 
 	stream = PARAM->_object.object;
-	OBJECT_REF(stream, "SUBR_inp_out");
+	OBJECT_REF(stream);
 
 	push_stream(where, stream);
 
@@ -915,7 +998,7 @@ static void free_list(void **list)
 		stream = pop_stream(list);
 		if (!stream)
 			return;
-		OBJECT_UNREF(stream, "free_list");
+		OBJECT_UNREF(stream);
 	}
 }
 
