@@ -2,7 +2,7 @@
 
   gbx_subr_string.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,6 +37,25 @@
 #include "gbx_c_array.h"
 #include "gbx_local.h"
 #include "gbx_compare.h"
+#include "gb.pcre.h"
+
+//static int _count = 0;
+
+static PCRE_INTERFACE PCRE;
+
+static void init_pcre()
+{
+	static bool init = FALSE;
+	
+	if (init)
+		return;
+		
+	COMPONENT_load(COMPONENT_create("gb.pcre"));
+	LIBRARY_get_interface_by_name("gb.pcre", PCRE_INTERFACE_VERSION, &PCRE);
+	init = TRUE;
+}
+
+//---------------------------------------------------------------------------
 
 void SUBR_cat(ushort code)
 {
@@ -51,10 +70,57 @@ void SUBR_cat(ushort code)
 		len = PARAM[0]._string.len ;
 		VALUE_conv_string(&PARAM[1]);
 		len2 = PARAM[1]._string.len;
+		
+		#if 0
+		if (EXEC_string_add)
+		{
+			EXEC_string_add = FALSE;
+			
+			str = PARAM[0]._string.addr;
+			
+			if (0 && PARAM[0].type == T_STRING && PARAM[0]._string.start == 0 && STRING_length(str) == len)
+			{
+				if (str && !STRING_extend_will_realloc(str, len + len2))
+				{
+					//_count++;
+					//fprintf(stderr, "[%d] &= optimization: str = %p (%d) param2 = %p (%d)\n", _count, str, len, PARAM[1]._string.addr + PARAM[1]._string.start, len2);
+					str = STRING_add(str, PARAM[1]._string.addr + PARAM[1]._string.start, len2);
+					/*if (str != PARAM[0]._string.addr)
+					{
+						//fprintf(stderr, "--> %p !\n", str);
+						BREAKPOINT();
+					}*/
+					RELEASE_STRING(&PARAM[1]);
+
+					SP -= 2;
+					//SP->type = T_STRING;
+					//SP->_string.addr = str;
+					//SP->_string.start = 0;
+					SP->_string.len += len2;
+					SP++;
+					return;
+				}
+			}
+			/*else
+			{
+				if (PARAM[0].type != T_STRING)
+					fprintf(stderr, "PARAM[0].type == %ld\n", PARAM[0].type);
+				else if (PARAM[0]._string.start)
+					fprintf(stderr, "PARAM[0]._string.start == %d\n", PARAM[0]._string.start);
+				else if (STRING_length(str) != len)
+					fprintf(stderr, "len == %d / %d\n", len, STRING_length(str));
+			}*/
+		}
+		#endif
+		
 		str = STRING_new(NULL, len + len2);
+
+		//fprintf(stderr, "normal: str = %p p0 = %p p1 = %p\n", str, PARAM[0]._string.addr + PARAM[0]._string.start, PARAM[1]._string.addr + PARAM[1]._string.start);
+		
 		memcpy(str, PARAM[0]._string.addr + PARAM[0]._string.start, len);
-		RELEASE_STRING(&PARAM[0]);
 		memcpy(&str[len], PARAM[1]._string.addr + PARAM[1]._string.start, len2);
+		
+		RELEASE_STRING(&PARAM[0]);
 		RELEASE_STRING(&PARAM[1]);
 
 		SP -= 2;
@@ -149,25 +215,29 @@ void SUBR_file(ushort code)
 	i = NPARAM;
 	while (i--)
 	{
-		VALUE_get_string(PARAM, &addr, &len);
-		
-		if (len > 0)
+		if (PARAM->type != T_NULL)
 		{
-			if (ptr > str)
+			VALUE_get_string(PARAM, &addr, &len);
+			
+			if (len > 0)
 			{
-				if (!slash && *addr != '/')
-					*ptr++ = '/';
-				else if (slash && *addr == '/')
-					ptr--;
+				if (ptr > str)
+				{
+					if (!slash && *addr != '/')
+						*ptr++ = '/';
+					else if (slash && *addr == '/')
+						ptr--;
+				}
+				
+				slash = addr[len - 1] == '/';
+				
+				memcpy(ptr, addr, len);
+				ptr += len;
 			}
-			
-			slash = addr[len - 1] == '/';
-			
-			memcpy(ptr, addr, len);
-			ptr += len;
-		}
 
-		RELEASE_STRING(PARAM);
+			RELEASE_STRING(PARAM);
+		}
+		
 		PARAM++;
 	}
 
@@ -301,7 +371,9 @@ void SUBR_upper(ushort code)
 	SUBR_ENTER_PARAM(1);
 	
 	if (SUBR_check_string(PARAM))
-		STRING_void_value(&SP[-1]);
+	{
+		VOID_STRING(&SP[-1]);
+	}
 	else
 	{
 		len = PARAM->_string.len;
@@ -430,7 +502,7 @@ __FOUND:
 
 void SUBR_like(ushort code)
 {
-	static const void *jump[] = { &&__LIKE, &&__BEGINS, &&__ENDS, &&__RETURN };
+	static const void *jump[] = { &&__LIKE, &&__BEGINS, &&__ENDS, &&__MATCH };
 	char *pattern;
 	char *string;
 	int len_pattern, len_string;
@@ -463,6 +535,12 @@ __ENDS:
 	else if (len_pattern <= len_string)
 		ret = STRING_equal_same(string + len_string - len_pattern, pattern, len_pattern);
 	goto __RETURN;
+	
+__MATCH:
+
+	init_pcre();
+	ret = PCRE.Match(string, len_string, pattern, len_pattern, 0, 0);
+	goto __RETURN;
 
 __RETURN:
 	
@@ -848,7 +926,7 @@ void SUBR_tr(void)
 
 void SUBR_quote(ushort code)
 {
-	static void *jump[4] = { &&__QUOTE, &&__SHELL, &&__HTML, &&__QUOTE };
+	static void *jump[4] = { &&__QUOTE, &&__SHELL, &&__HTML, &&__BASE64 };
 	char *str;
 	int lstr;
 	int i;
@@ -960,6 +1038,36 @@ __HTML:
 	}
 	
 	goto __END;
+	
+__BASE64:
+	{
+		static const char base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+		uchar *in;
+		char *out = buf;
+		
+		for (i = 0; i < (lstr - 2); i += 3)
+		{
+			in = (uchar *)&str[i];
+			out[0] = base64[in[0] >> 2];
+			out[1] = base64[((in[0] & 0x03) << 4) | ((in[1] & 0xF0) >> 4)];
+			out[2] = base64[((in[1] & 0x0F) << 2) | ((in[2] & 0xC0) >> 6)];
+			out[3] = base64[in[2] & 0x3F];
+			STRING_make(out, 4);
+		}
+
+		if (i < lstr)
+		{
+			in = (uchar *)&str[i];
+			lstr -= i;
+			out[0] = base64[in[0] >> 2];
+			out[1] = base64[((in[0] & 0x03) << 4) | ((in[1] & 0xF0) >> 4)];
+			out[2] = (lstr > 1 ? base64[((in[1] & 0x0F) << 2) | ((in[2] & 0xC0) >> 6) ] : '=');
+			out[3] = (lstr > 2 ? base64[in[2] & 0x3F] : '=');
+			STRING_make(out, 4);
+		}
+	}
+	
+	goto __END;
 
 __END:
 
@@ -983,15 +1091,17 @@ static int read_hex_digit(unsigned char c)
 		return 0;
 }
 
-void SUBR_unquote(void)
+void SUBR_unquote(ushort code)
 {
+	static void *jump[2] = { &&__UNQUOTE, &&__UNBASE64 };
+	
 	char *str;
 	int lstr;
 	int i;
 	unsigned char c;
 	
 	SUBR_ENTER_PARAM(1);
-	
+
 	VALUE_conv_string(&PARAM[0]);
 	
 	str = PARAM->_string.addr + PARAM->_string.start;
@@ -999,6 +1109,10 @@ void SUBR_unquote(void)
 	
 	STRING_start_len(lstr);
 	
+	goto *jump[code & 0x3];
+
+__UNQUOTE:
+	
 	if (lstr >= 2 && str[0] == '"' && str[lstr - 1] == '"')
 	{
 		str++;
@@ -1034,6 +1148,50 @@ void SUBR_unquote(void)
 		STRING_make_char(c);
 	}
 	
+	goto __END;
+	
+__UNBASE64:
+
+	{
+		char buf[4];
+		unsigned char n = 0;
+		
+		for (i = 0; i < lstr; i++)
+		{
+			c = str[i];
+			if (c >= 'A' && c <= 'Z')
+				c = c - 'A';
+			else if (c >= 'a' && c <= 'z')
+				c = c - 'a' + 26;
+			else if (c >= '0' && c <= '9')
+				c = c - '0' + 52;
+			else if (c == '+')
+				c = 62;
+			else if (c == '/')
+				c = 63;
+			else if (c == '=')
+				break;
+			else
+				continue;
+			
+			switch (n & 3)
+			{
+				case 0: buf[0] = c << 2; break;
+				case 1: buf[0] |= c >> 4; buf[1] = c << 4; break;
+				case 2: buf[1] |= c >> 2; buf[2] = c << 6; break;
+				case 3: buf[2] |= c; STRING_make(buf, 3); break;
+			}
+			n++;
+		}
+
+		if ((n & 3) > 1)
+			STRING_make(buf, (n & 3) - 1);
+	}
+	
+	goto __END;
+	
+__END:
+	
 	RETURN->type = T_STRING;
 	RETURN->_string.addr = STRING_end_temp();
 	RETURN->_string.start = 0;
