@@ -2,7 +2,7 @@
 
   gbx_type.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@ typedef
     unsigned char id;
     short value;
     }
-  PACKED
   CTYPE;
 
 #define CTYPE_is_static(type)     ((type).flag & TF_STATIC)
@@ -60,7 +59,10 @@ EXTERN void *TYPE_joker;
 EXTERN const size_t TYPE_sizeof_memory_tab[];
 #endif
 
+#define TYPE_is_void(type)         ((type) == T_VOID)
+#define TYPE_is_null(type)         ((type) == T_NULL)
 #define TYPE_is_object(type)       ((type) >= T_OBJECT)
+#define TYPE_is_object_null(type)  ((type) >= T_NULL)
 #define TYPE_is_pure_object(type)  ((type) > T_OBJECT)
 #define TYPE_is_boolean(type)      ((type) == T_BOOLEAN)
 #define TYPE_is_integer(type)      ((type) >= T_BOOLEAN && (type) <= T_INTEGER)
@@ -73,10 +75,10 @@ EXTERN const size_t TYPE_sizeof_memory_tab[];
 #define TYPE_is_number_date(type)  ((type) >= T_BOOLEAN && (type) <= T_DATE)
 #define TYPE_is_string(type)       ((type) == T_STRING || (type) == T_CSTRING)
 #define TYPE_is_function(type)     ((type) == T_FUNCTION)
-#define TYPE_is_null(type)         ((type) == T_NULL)
-#define TYPE_is_object_null(type)  ((type) >= T_OBJECT || (type) == T_NULL)
 #define TYPE_is_pointer(type)      ((type) == T_POINTER)
-#define TYPE_is_void(type)         ((type) == T_VOID)
+
+#define TYPE_are_objects(_t1, _t2) (TYPE_is_object(_t1) && TYPE_is_object(_t2))
+//#define TYPE_are_not_objects(_t1, _t2) (((_t1) | (_t2)) < T_OBJECT)
 
 size_t TYPE_sizeof(TYPE type);
 #define TYPE_sizeof_memory(_type) (TYPE_is_object(_type) ? sizeof(void *) : TYPE_sizeof_memory_tab[_type])
