@@ -2,7 +2,7 @@
 
   gbx_value.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -75,7 +75,6 @@ typedef
 		#endif
     int64_t value;
     }
-  PACKED
   VALUE_LONG;
 
 typedef
@@ -83,7 +82,6 @@ typedef
     TYPE type;
     char *value;
     }
-  PACKED
   VALUE_POINTER;
 
 typedef
@@ -91,7 +89,6 @@ typedef
     TYPE type;
     float value;
     }
-  PACKED
   VALUE_SINGLE;
 
 typedef
@@ -102,7 +99,6 @@ typedef
 		#endif
     double value;
     }
-  PACKED
   VALUE_FLOAT;
 
 typedef
@@ -131,7 +127,6 @@ typedef
     char defined;
     short index;
     }
-  PACKED
   VALUE_FUNCTION;
 
 enum
@@ -218,10 +213,10 @@ typedef
 	void (*VALUE_CONVERT_FUNC)(VALUE *);
 	
 #define VALUE_copy(_dst, _src) \
-	(_dst)->_void.type = (_src)->_void.type; \
-	(_dst)->_void.ptype = (_src)->_void.ptype; \
-	(_dst)->_void.value[0] = (_src)->_void.value[0]; \
-	(_dst)->_void.value[1] = (_src)->_void.value[1];
+	((_dst)->_void.type = (_src)->_void.type, \
+	(_dst)->_void.ptype = (_src)->_void.ptype, \
+	(_dst)->_void.value[0] = (_src)->_void.value[0], \
+	(_dst)->_void.value[1] = (_src)->_void.value[1])
 
 #define VALUE_is_equal(_v1, _v2) (*_v1 == *v2)
 	
@@ -238,7 +233,6 @@ void VALUE_convert_integer(VALUE *value);
 void VALUE_convert_float(VALUE *value);
 void VALUE_convert_string(VALUE *value);
 void VALUE_convert_variant(VALUE *value);
-void VALUE_convert_object(VALUE *value, TYPE type);
 
 void VALUE_read(VALUE *value, void *addr, TYPE type);
 void VALUE_write(VALUE *value, void *addr, TYPE type);
