@@ -2,7 +2,7 @@
 
   CConnection.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 
 static CCONNECTION *_current = NULL;
 
-static GB_SUBCOLLECTION_DESC _databases_desc =
+static SUBCOLLECTION_DESC _databases_desc =
 {
   ".Connection.Databases",
   (void *)CDATABASE_get,
@@ -49,7 +49,7 @@ static GB_SUBCOLLECTION_DESC _databases_desc =
   (void *)CDATABASE_release
 };
 
-static GB_SUBCOLLECTION_DESC _users_desc =
+static SUBCOLLECTION_DESC _users_desc =
 {
   ".Connection.Users",
   (void *)CUSER_get,
@@ -58,7 +58,7 @@ static GB_SUBCOLLECTION_DESC _users_desc =
   (void *)CUSER_release
 };
 
-static GB_SUBCOLLECTION_DESC _tables_desc =
+static SUBCOLLECTION_DESC _tables_desc =
 {
   ".Connection.Tables",
   (void *)CTABLE_get,
@@ -193,6 +193,7 @@ BEGIN_METHOD(CCONNECTION_new, GB_STRING url)
 
 	THIS->desc.host = GB.NewZeroString(url);
 	THIS->desc.name = GB.NewZeroString(name);
+	return;
 
 __BAD_URL:
 
@@ -592,7 +593,7 @@ BEGIN_METHOD(CCONNECTION_quote, GB_STRING name; GB_BOOLEAN is_table)
   CHECK_DB();
   CHECK_OPEN();
 
-	if (VARGOPT(is_table, FALSE) && THIS->db.flags.schema)
+	if (VARGOPT(is_table, FALSE)) // && THIS->db.flags.schema)
 		GB.ReturnNewZeroString(DB_GetQuotedTable(THIS->driver, &THIS->db, GB.ToZeroString(ARG(name))));
 	else
 	{
@@ -669,7 +670,7 @@ BEGIN_PROPERTY(CCONNECTION_databases)
   CHECK_DB();
   CHECK_OPEN();
 
-  GB.SubCollection.New(&THIS->databases, &_databases_desc, THIS);
+  GB_SubCollectionNew(&THIS->databases, &_databases_desc, THIS);
   GB.ReturnObject(THIS->databases);
 
 END_PROPERTY
@@ -680,7 +681,7 @@ BEGIN_PROPERTY(CCONNECTION_users)
   CHECK_DB();
   CHECK_OPEN();
 
-  GB.SubCollection.New(&THIS->users, &_users_desc, THIS);
+  GB_SubCollectionNew(&THIS->users, &_users_desc, THIS);
   GB.ReturnObject(THIS->users);
 
 END_PROPERTY
@@ -691,7 +692,7 @@ BEGIN_PROPERTY(CCONNECTION_tables)
   CHECK_DB();
   CHECK_OPEN();
 
-  GB.SubCollection.New(&THIS->tables, &_tables_desc, THIS);
+  GB_SubCollectionNew(&THIS->tables, &_tables_desc, THIS);
   GB.ReturnObject(THIS->tables);
 
 END_PROPERTY
