@@ -2,7 +2,7 @@
 
   CConnection.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include "gambas.h"
 #include "gb.db.h"
+#include "c_subcollection.h"
 
 #ifndef __CCONNECTION_C
 extern GB_DESC CConnectionDesc[];
@@ -42,10 +43,10 @@ typedef
     DB_DRIVER *driver;
     DB_DATABASE db;
     DB_DESC desc;
-    GB_SUBCOLLECTION databases;
-    GB_SUBCOLLECTION tables;
-    GB_SUBCOLLECTION views;
-    GB_SUBCOLLECTION users;
+    CSUBCOLLECTION *databases;
+    CSUBCOLLECTION *tables;
+    CSUBCOLLECTION *views;
+    CSUBCOLLECTION *users;
     int limit;
     int trans;
 		bool ignore_charset;
