@@ -2,7 +2,7 @@
 
   CResult.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ static CBLOB *make_blob(CRESULT *result, int field);
 static void set_blob(CBLOB *_object, char *data, int length);
 
 
-static GB_SUBCOLLECTION_DESC _fields_desc =
+static SUBCOLLECTION_DESC _fields_desc =
 {
 	".Result.Fields",
 	(void *)CRESULTFIELD_get,
@@ -707,6 +707,7 @@ BEGIN_METHOD(CRESULT_delete, GB_BOOLEAN keep)
 
 	DB_INFO *info = &THIS->info;
 	int *pos;
+	void *save_enum;
 
 	if (check_available(THIS))
 		return;
@@ -735,13 +736,14 @@ BEGIN_METHOD(CRESULT_delete, GB_BOOLEAN keep)
 				THIS->count--;
 				reload_buffer(THIS);
 
-				GB.ListEnum(THIS);
+				save_enum = GB.BeginEnum(THIS);
 				while (!GB.NextEnum())
 				{
 					pos = (int *)GB.GetEnum();
 					if (*pos > THIS->pos)
 						(*pos)--;
 				}
+				GB.EndEnum(save_enum);
 			}
 
 			break;
@@ -759,7 +761,7 @@ END_METHOD
 
 BEGIN_PROPERTY(CRESULT_fields)
 
-	GB.SubCollection.New(&THIS->fields, &_fields_desc, THIS);
+	GB_SubCollectionNew(&THIS->fields, &_fields_desc, THIS);
 	GB.ReturnObject(THIS->fields);
 
 END_PROPERTY
@@ -807,6 +809,28 @@ GB_DESC CResultDesc[] =
 
 /** Blob *******************************************************************/
 
+static bool _convert_blob(CBLOB *_object, GB_TYPE type, GB_VALUE *conv)
+{
+	if (BLOB)
+	{
+		switch (type)
+		{
+			case GB_T_STRING:
+			case GB_T_CSTRING:
+				conv->_string.value.addr = BLOB->data;
+				conv->_string.value.start = 0;
+				conv->_string.value.len = BLOB->length;
+				conv->type = GB_T_CSTRING;
+				return FALSE;
+				
+			default:
+				return TRUE;
+		}
+	}
+	else
+		return TRUE;
+}
+
 /*static int check_blob(CBLOB *_object)
 {
 	return check_result(BLOB->result) || (BLOB->result->pos != BLOB->pos);
@@ -825,9 +849,13 @@ static CBLOB *make_blob(CRESULT *result, int field)
 	//BLOB->pos = result->pos;
 	BLOB->data = NULL;
 	BLOB->length = 0;
+	BLOB->constant = TRUE;
 
 	if (result->handle && result->pos >= 0)
+	{
 		result->driver->Result.Blob(result->handle, result->pos, field, BLOB);
+		set_blob(BLOB, BLOB->data, BLOB->length);
+	}
 
 	//fprintf(stderr, "make_blob: [%d] %d (%d) -> %p\n", result->pos, field, BLOB->length, BLOB);
 
@@ -837,14 +865,12 @@ static CBLOB *make_blob(CRESULT *result, int field)
 
 static void set_blob(CBLOB *_object, char *data, int length)
 {
-	//fprintf(stderr, "set_blob: %p %ld\n", BLOB, length);
-
-	if (!BLOB->constant)
+	if (!BLOB->constant && BLOB->data)
 		GB.FreeString((char **)&BLOB->data);
 
 	if (data && length)
 	{
-		*((char **)&BLOB->data) = GB.NewString(data, length);
+		BLOB->data = GB.NewString(data, length);
 		BLOB->constant = FALSE;
 	}
 
@@ -905,6 +931,7 @@ GB_DESC CBlobDesc[] =
 	//GB_PROPERTY_READ("Result", "Result", CBLOB_result),
 	GB_PROPERTY("Data", "s", CBLOB_data),
 	GB_PROPERTY_READ("Length", "i", CBLOB_length),
+	GB_INTERFACE("_convert", &_convert_blob),
 	//GB_METHOD("_unknown", "v", CBLOB_unknown, "v"),
 
 	GB_END_DECLARE
