@@ -2,7 +2,7 @@
 
   cpaint.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,8 +25,6 @@
 
 #include "gb.image.h"
 #include "main.h"
-#include "crect.h"
-#include "CDraw.h"
 #include "cpaint.h"
 
 static GB_PAINT *_current = NULL;
@@ -36,16 +34,33 @@ static GB_PAINT *_current = NULL;
 #define XTHIS ((PAINT_EXTENTS *)_object)
 
 #define MTHIS ((PAINT_MATRIX *)_object)
-#define MPAINT (MTHIS->desc)
+#define MPAINT (_matrix_desc)
 
 #define BTHIS ((PAINT_BRUSH *)_object)
 #define BPAINT (BTHIS->desc)
 
 #define CHECK_DEVICE() if (check_device()) return
 
+#define RAD(_deg) ((_deg) * M_PI / 180)
+
+static GB_PAINT_MATRIX_DESC *_matrix_desc = NULL;
+
+static void load_matrix_interface()
+{
+	if (!_matrix_desc)
+	{
+		_matrix_desc = (GB_PAINT_MATRIX_DESC *)GB.GetClassInterface(GB.FindClass("Image"), "PaintMatrix");
+		if (!_matrix_desc)
+		{
+			fprintf(stderr, "gb.draw: error: unable to find PaintMatrix interface\n");
+			abort();
+		}
+	}
+}
+
 static bool check_device()
 {
-	if (!_current || !_current->opened)
+	if (!_current || !_current->extra)
 	{
 		GB.Error("No current device");
 		return TRUE;
@@ -56,8 +71,10 @@ static bool check_device()
 
 GB_PAINT *PAINT_get_current()
 {
-	check_device();
-	return _current;
+	if (check_device())
+		return NULL;
+	else
+		return _current;
 }
 
 GB_PAINT *PAINT_from_device(void *device)
@@ -82,12 +99,16 @@ bool PAINT_open(GB_PAINT *paint)
 	GB.Alloc(POINTER(&paint->extra), paint->desc->size);
 	memset(paint->extra, 0, paint->desc->size);
 	paint->opened = !paint->desc->Begin(paint);
+
+	if (!paint->opened)
+		GB.Free(POINTER(&paint->extra));
+	
 	return !paint->opened;
 }
 
 void PAINT_close(GB_PAINT *paint)
 {
-	if (paint->opened)
+	if (paint->opened && !paint->other)
 	{
 		//fprintf(stderr, "PAINT_close: %p\n", paint);
 		paint->desc->End(paint);
@@ -104,6 +125,7 @@ bool PAINT_begin(void *device)
 	
 	klass = GB.GetClass(device);
 	desc = (GB_PAINT_DESC *)GB.GetClassInterface(klass, "Paint");
+	load_matrix_interface();
 
 	if (!desc)
 	{
@@ -120,17 +142,28 @@ bool PAINT_begin(void *device)
 	paint->device = device;
 	paint->brush = NULL;
 	paint->opened = FALSE;
-
+	paint->other = FALSE;
+	paint->tag = NULL;
+	
 	paint->previous = _current;
 	_current = paint;
 	
-	paint->draw = DRAW_from_device(device);
-	if (paint->draw)
-		DRAW_close(paint->draw);
+	//paint->draw = DRAW_from_device(device);
+	//if (paint->draw)
+	//	DRAW_close(paint->draw);
 	
 	if (other)
 	{
 		paint->extra = other->extra;
+		paint->opened = TRUE;
+		paint->other = TRUE;
+		paint->width = other->width;
+		paint->height = other->height;
+		paint->resolutionX = other->resolutionX;
+		paint->resolutionY = other->resolutionY;
+		paint->brush = other->brush;
+		if (paint->brush)
+			GB.Ref(paint->brush);
 	}
 	else
 	{
@@ -153,17 +186,18 @@ void PAINT_end()
 	
 	PAINT_close(paint);
 
-	if (paint->draw)
-		DRAW_open(paint->draw);
+	//if (paint->draw)
+	//	DRAW_open(paint->draw);
 	
 	if (paint->brush)
 		GB.Unref(POINTER(&paint->brush));
 	GB.Unref(POINTER(&paint->device));
+	GB.StoreObject(NULL, POINTER(&(paint->tag)));
 	GB.Free(POINTER(&paint));
 }
 
 
-/**** PaintExtents *********************************************************/
+//---- PaintExtents ---------------------------------------------------------
 
 #define IMPLEMENT_EXTENTS_PROPERTY(_method, _field) \
 BEGIN_PROPERTY(_method) \
@@ -217,54 +251,90 @@ GB_DESC PaintExtentsDesc[] =
 	GB_END_DECLARE
 };
 
-/**** PaintMatrix **********************************************************/
 
-static PAINT_MATRIX *create_matrix(GB_PAINT_DESC *desc, GB_TRANSFORM *transform)
+//---- PaintMatrix ----------------------------------------------------------
+
+static bool _do_not_init = FALSE;
+
+static PAINT_MATRIX *create_matrix(GB_TRANSFORM transform)
 {
 	PAINT_MATRIX *matrix;
+	_do_not_init = TRUE;
 	matrix = GB.New(GB.FindClass("PaintMatrix"), NULL, NULL);
-	matrix->desc = desc;
+	_do_not_init = FALSE;
 	matrix->transform = transform;
 	return matrix;
 }
 
+BEGIN_METHOD(PaintMatrix_new, GB_FLOAT xx; GB_FLOAT xy; GB_FLOAT yx; GB_FLOAT yy; GB_FLOAT x0; GB_FLOAT y0)
+
+	load_matrix_interface();
+
+	if (_do_not_init)
+		return;
+	
+	MPAINT->Create(&MTHIS->transform);
+	MPAINT->Init(MTHIS->transform, VARGOPT(xx, 1.0), VARGOPT(xy, 0.0), VARGOPT(yx, 0.0), VARGOPT(yy, 1.0), VARGOPT(x0, 0.0), VARGOPT(y0, 0.0));
+
+END_METHOD
+
+BEGIN_METHOD(PaintMatrix_call, GB_FLOAT xx; GB_FLOAT xy; GB_FLOAT yx; GB_FLOAT yy; GB_FLOAT x0; GB_FLOAT y0)
+
+	GB_TRANSFORM transform;
+	
+	MPAINT->Create(&transform);
+	MPAINT->Init(transform, VARGOPT(xx, 1.0), VARGOPT(xy, 0.0), VARGOPT(yx, 0.0), VARGOPT(yy, 1.0), VARGOPT(x0, 0.0), VARGOPT(y0, 0.0));
+	GB.ReturnObject(create_matrix(transform));
+
+END_METHOD
+
 BEGIN_METHOD_VOID(PaintMatrix_free)
 
-	MPAINT->Transform.Delete(&MTHIS->transform);
+	if (MTHIS->transform)
+		MPAINT->Delete(&MTHIS->transform);
+
+END_METHOD
+
+BEGIN_METHOD_VOID(PaintMatrix_Copy)
+
+	GB_TRANSFORM transform;
+
+	MPAINT->Copy(&transform, MTHIS->transform);
+	GB.ReturnObject(create_matrix(transform));
 
 END_METHOD
 
 BEGIN_METHOD_VOID(PaintMatrix_Reset)
 
-	MPAINT->Transform.Init(MTHIS->transform, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0);
+	MPAINT->Init(MTHIS->transform, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0);
 	RETURN_SELF();
 
 END_METHOD
 
 BEGIN_METHOD(PaintMatrix_Translate, GB_FLOAT tx; GB_FLOAT ty)
 
-	MPAINT->Transform.Translate(MTHIS->transform, (float)VARG(tx), (float)VARG(ty));
+	MPAINT->Translate(MTHIS->transform, (float)VARG(tx), (float)VARG(ty));
 	RETURN_SELF();
 
 END_METHOD
 
 BEGIN_METHOD(PaintMatrix_Scale, GB_FLOAT sx; GB_FLOAT sy)
 
-	MPAINT->Transform.Scale(MTHIS->transform, (float)VARG(sx), (float)VARG(sy));
+	MPAINT->Scale(MTHIS->transform, (float)VARG(sx), (float)VARG(sy));
 	RETURN_SELF();
 
 END_METHOD
 
 BEGIN_METHOD(PaintMatrix_Rotate, GB_FLOAT angle)
 
-	MPAINT->Transform.Rotate(MTHIS->transform, (float)VARG(angle));
+	MPAINT->Rotate(MTHIS->transform, (float)VARG(angle));
 	RETURN_SELF();
 
 END_METHOD
 
 BEGIN_METHOD_VOID(PaintMatrix_Invert)
 
-	if (MPAINT->Transform.Invert(MTHIS->transform))
+	if (MPAINT->Invert(MTHIS->transform))
 		GB.ReturnNull();
 	else
 		RETURN_SELF();
@@ -278,29 +348,50 @@ BEGIN_METHOD(PaintMatrix_Multiply, GB_OBJECT matrix)
 	if (GB.CheckObject(matrix))
 		return;
 
-	MPAINT->Transform.Multiply(MTHIS->transform, matrix->transform);
+	MPAINT->Multiply(MTHIS->transform, matrix->transform);
 	RETURN_SELF();
 
 END_METHOD
 
+BEGIN_METHOD(PaintMatrix_Map, GB_OBJECT point)
+
+	GEOM_POINTF *point = VARG(point);
+	double x, y;
+
+	if (GB.CheckObject(point))
+		return;
+
+	x = point->x;
+	y = point->y;
+
+	MPAINT->Map(MTHIS->transform, &x, &y);
+
+	GB.ReturnObject(GEOM.CreatePointF(x, y));
+
+END_METHOD
+
 GB_DESC PaintMatrixDesc[] = 
 {
-	GB_DECLARE("PaintMatrix", sizeof(PAINT_MATRIX)), GB_NOT_CREATABLE(),
-
+	GB_DECLARE("PaintMatrix", sizeof(PAINT_MATRIX)),
+	
+	GB_METHOD("_new", NULL, PaintMatrix_new, "[(XX)f(XY)f(YX)f(YY)f(X0)f(Y0)f]"),
+	GB_STATIC_METHOD("_call", "PaintMatrix", PaintMatrix_call, "[(XX)f(XY)f(YX)f(YY)f(X0)f(Y0)f]"),
 	GB_METHOD("_free", NULL, PaintMatrix_free, NULL),
 
+	GB_METHOD("Copy", "PaintMatrix", PaintMatrix_Copy, NULL),
 	GB_METHOD("Reset", "PaintMatrix", PaintMatrix_Reset, NULL),
 	GB_METHOD("Translate", "PaintMatrix", PaintMatrix_Translate, "(TX)f(TY)f"),
 	GB_METHOD("Scale", "PaintMatrix", PaintMatrix_Scale, "(SX)f(SY)f"),
 	GB_METHOD("Rotate", "PaintMatrix", PaintMatrix_Rotate, "(Angle)f"),
 	GB_METHOD("Invert", "PaintMatrix", PaintMatrix_Invert, NULL),
 	GB_METHOD("Multiply", "PaintMatrix", PaintMatrix_Multiply, "(Matrix)PaintMatrix;"),	
+	GB_METHOD("Map", "PointF", PaintMatrix_Map, "(Point)PointF;"),
 
 	GB_END_DECLARE	
 };
 
 
-/**** PaintBrush ***********************************************************/
+//---- PaintBrush -----------------------------------------------------------
 
 BEGIN_METHOD_VOID(PaintBrush_free)
 
@@ -315,9 +406,9 @@ BEGIN_PROPERTY(PaintBrush_Matrix)
 	
 	if (READ_PROPERTY)
 	{
-		BPAINT->Transform.Create(&transform);
+		MPAINT->Create(&transform);
 		BPAINT->Brush.Matrix(BTHIS->brush, FALSE, transform);
-		GB.ReturnObject(create_matrix(BPAINT, transform));
+		GB.ReturnObject(create_matrix(transform));
 	}
 	else
 	{
@@ -340,11 +431,11 @@ BEGIN_METHOD(PaintBrush_Translate, GB_FLOAT tx; GB_FLOAT ty)
 
 	GB_TRANSFORM transform;
 
-	BPAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	BPAINT->Brush.Matrix(BTHIS->brush, FALSE, transform);
-	BPAINT->Transform.Translate(transform, (float)VARG(tx), (float)VARG(ty));
+	MPAINT->Translate(transform, (float)VARG(tx), (float)VARG(ty));
 	BPAINT->Brush.Matrix(BTHIS->brush, TRUE, transform);
-	BPAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
@@ -352,11 +443,11 @@ BEGIN_METHOD(PaintBrush_Scale, GB_FLOAT sx; GB_FLOAT sy)
 
 	GB_TRANSFORM transform;
 
-	BPAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	BPAINT->Brush.Matrix(BTHIS->brush, FALSE, transform);
-	BPAINT->Transform.Scale(transform, (float)VARG(sx), (float)VARG(sy));
+	MPAINT->Scale(transform, (float)VARG(sx), (float)VARG(sy));
 	BPAINT->Brush.Matrix(BTHIS->brush, TRUE, transform);
-	BPAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
@@ -364,11 +455,11 @@ BEGIN_METHOD(PaintBrush_Rotate, GB_FLOAT angle)
 
 	GB_TRANSFORM transform;
 
-	BPAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	BPAINT->Brush.Matrix(BTHIS->brush, FALSE, transform);
-	BPAINT->Transform.Rotate(transform, (float)VARG(angle));
+	MPAINT->Rotate(transform, (float)VARG(angle));
 	BPAINT->Brush.Matrix(BTHIS->brush, TRUE, transform);
-	BPAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
@@ -389,9 +480,10 @@ GB_DESC PaintBrushDesc[] =
 };
 
 
-/**** Paint ****************************************************************/
+//---- Paint ----------------------------------------------------------------
 
-BEGIN_METHOD(Paint_begin, GB_OBJECT device)
+
+BEGIN_METHOD(Paint_Begin, GB_OBJECT device)
 
 	void *device = VARG(device);
 
@@ -403,7 +495,7 @@ BEGIN_METHOD(Paint_begin, GB_OBJECT device)
 END_METHOD
 
 
-BEGIN_METHOD_VOID(Paint_end)
+BEGIN_METHOD_VOID(Paint_End)
 
 	PAINT_end();
 	
@@ -417,6 +509,7 @@ BEGIN_METHOD_VOID(Paint_exit)
 
 END_METHOD
 
+
 BEGIN_PROPERTY(Paint_Device)
 	
 	if (THIS)
@@ -426,6 +519,7 @@ BEGIN_PROPERTY(Paint_Device)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Paint_Width)
 
 	CHECK_DEVICE();
@@ -433,6 +527,7 @@ BEGIN_PROPERTY(Paint_Width)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Paint_Height)
 
 	CHECK_DEVICE();
@@ -440,6 +535,7 @@ BEGIN_PROPERTY(Paint_Height)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Paint_ResolutionX)
 
 	CHECK_DEVICE();
@@ -447,6 +543,7 @@ BEGIN_PROPERTY(Paint_ResolutionX)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Paint_ResolutionY)
 
 	CHECK_DEVICE();
@@ -460,6 +557,7 @@ BEGIN_METHOD_VOID(_method) \
 	PAINT->_api(THIS); \
 END_METHOD
 
+
 #define IMPLEMENT_METHOD_PRESERVE(_method, _api) \
 BEGIN_METHOD(_method, GB_BOOLEAN preserve) \
 	CHECK_DEVICE(); \
@@ -497,9 +595,16 @@ END_METHOD
 #define IMPLEMENT_PROPERTY_INTEGER(_property, _api) \
 	IMPLEMENT_PROPERTY(_property, _api, int, GB_INTEGER, GB.ReturnInteger)
 
+#define IMPLEMENT_PROPERTY_UNSIGNED_INTEGER(_property, _api) \
+	IMPLEMENT_PROPERTY(_property, _api, uint, GB_INTEGER, GB.ReturnInteger)
+
 #define IMPLEMENT_PROPERTY_FLOAT(_property, _api) \
 	IMPLEMENT_PROPERTY(_property, _api, float, GB_FLOAT, GB.ReturnFloat)
 
+IMPLEMENT_PROPERTY_BOOLEAN(Paint_Invert, Invert)
+IMPLEMENT_PROPERTY_INTEGER(Paint_FillStyle, FillStyle)
+IMPLEMENT_PROPERTY_UNSIGNED_INTEGER(Paint_Background, Background)
+
 IMPLEMENT_PROPERTY_BOOLEAN(Paint_Antialias, Antialias)
 IMPLEMENT_METHOD(Paint_Save, Save)
 IMPLEMENT_METHOD(Paint_Restore, Restore)
@@ -510,10 +615,11 @@ IMPLEMENT_METHOD_PRESERVE(Paint_Fill, Fill)
 IMPLEMENT_METHOD_PRESERVE(Paint_Stroke, Stroke)
 IMPLEMENT_PROPERTY_EXTENTS(Paint_PathExtents, PathExtents)
 
+
 BEGIN_PROPERTY(Paint_ClipRect)
 
 	GB_EXTENTS ext;
-	CRECT *rect;
+	GEOM_RECT *rect;
 	int w, h;
 	
 	CHECK_DEVICE();
@@ -531,7 +637,7 @@ BEGIN_PROPERTY(Paint_ClipRect)
 			return;
 		}
 		
-		rect = CRECT_create();
+		rect = GEOM.CreateRect();
 		rect->x = ceilf(ext.x1);
 		rect->y = ceilf(ext.y1);
 		rect->w = w;
@@ -541,18 +647,16 @@ BEGIN_PROPERTY(Paint_ClipRect)
 	}
 	else
 	{
-		rect = (CRECT *)VPROP(GB_OBJECT);
-		PAINT->ResetClip(THIS);
+		rect = (GEOM_RECT *)VPROP(GB_OBJECT);
 		if (rect)
-		{
-			PAINT->NewPath(THIS);
-			PAINT->Rectangle(THIS, rect->x, rect->y, rect->w, rect->h);
-			PAINT->Clip(THIS, FALSE);
-		}
+			PAINT->ClipRect(THIS, rect->x, rect->y, rect->w, rect->h);
+		else
+			PAINT->ResetClip(THIS);
 	}
 
 END_PROPERTY
 
+
 BEGIN_METHOD(Paint_PathContains, GB_FLOAT x; GB_FLOAT y)
 
 	CHECK_DEVICE();
@@ -569,6 +673,8 @@ IMPLEMENT_PROPERTY_FLOAT(Paint_MiterLimit, MiterLimit)
 
 BEGIN_PROPERTY(Paint_Brush)
 
+	CHECK_DEVICE();
+	
 	if (READ_PROPERTY)
 		GB.ReturnObject(THIS->brush);
 	else
@@ -586,6 +692,35 @@ BEGIN_PROPERTY(Paint_Brush)
 
 END_PROPERTY
 
+
+BEGIN_PROPERTY(Paint_BrushOrigin)
+
+	float x, y;
+	
+	CHECK_DEVICE();
+
+	if (READ_PROPERTY)
+	{
+		PAINT->BrushOrigin(THIS, FALSE, &x, &y);
+		GB.ReturnObject(GEOM.CreatePointF(x, y));
+	}
+	else
+	{
+		GEOM_POINT *p = VPROP(GB_OBJECT);
+		if (!p)
+			x = y = 0.0;
+		else
+		{
+			x = p->x;
+			y = p->y;
+		}
+			
+		PAINT->BrushOrigin(THIS, TRUE, &x, &y);
+	}
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(Paint_Dash)
 
 	GB_ARRAY array;
@@ -619,6 +754,7 @@ BEGIN_PROPERTY(Paint_Dash)
 		if (!count)
 		{
 			PAINT->Dash(THIS, TRUE, NULL, &count);
+			//THIS->lineStyle = GB_PAINT_LINE_STYLE_SOLID;
 		}
 		else
 		{
@@ -627,15 +763,18 @@ BEGIN_PROPERTY(Paint_Dash)
 				dashes[i] = (float)*((double *)GB.Array.Get(array, i));
 			PAINT->Dash(THIS, TRUE, &dashes, &count);
 			GB.Free(POINTER(&dashes));
+			//THIS->lineStyle = GB_PAINT_LINE_STYLE_CUSTOM;
 		}
 	}
 
 END_PROPERTY
 
+
 IMPLEMENT_PROPERTY_FLOAT(Paint_DashOffset, DashOffset)
 IMPLEMENT_METHOD(Paint_NewPath, NewPath)
 IMPLEMENT_METHOD(Paint_ClosePath, ClosePath)
 
+
 BEGIN_PROPERTY(Paint_X)
 
 	float x, y;
@@ -645,6 +784,7 @@ BEGIN_PROPERTY(Paint_X)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Paint_Y)
 
 	float x, y;
@@ -654,13 +794,66 @@ BEGIN_PROPERTY(Paint_Y)
 
 END_PROPERTY
 
-BEGIN_METHOD(Paint_Arc, GB_FLOAT xc; GB_FLOAT yc; GB_FLOAT radius; GB_FLOAT angle; GB_FLOAT length)
+
+BEGIN_METHOD(Paint_Arc, GB_FLOAT xc; GB_FLOAT yc; GB_FLOAT radius; GB_FLOAT angle; GB_FLOAT length; GB_BOOLEAN pie)
 
 	CHECK_DEVICE();
-	PAINT->Arc(THIS, VARG(xc), VARG(yc), VARG(radius), VARGOPT(angle, 0.0), VARGOPT(length, MISSING(angle) ? M_PI * 2 : 0.0));
+	
+	bool pie = VARGOPT(pie, FALSE);
+	float angle = VARGOPT(angle, 0.0);
+	float length = VARGOPT(length, MISSING(angle) ? M_PI * 2 : 0.0);
+	
+	if (MISSING(length) || length == 0.0)
+		pie = FALSE;
+	
+	PAINT->Arc(THIS, VARG(xc), VARG(yc), VARG(radius), angle, length, pie);
 
 END_METHOD
 
+
+BEGIN_METHOD(Paint_Ellipse, GB_FLOAT x; GB_FLOAT y; GB_FLOAT width; GB_FLOAT height; GB_FLOAT angle; GB_FLOAT length; GB_BOOLEAN pie)
+
+	CHECK_DEVICE();
+
+	bool pie = VARGOPT(pie, FALSE);
+	float angle = VARGOPT(angle, 0.0);
+	float length = VARGOPT(length, MISSING(angle) ? M_PI * 2 : 0.0);
+	
+	if (MISSING(length) || length == 0.0)
+		pie = FALSE;
+	
+	PAINT->Ellipse(THIS, VARG(x), VARG(y), VARG(width), VARG(height), angle, length, pie);
+
+END_METHOD
+
+
+BEGIN_METHOD(Paint_Polygon, GB_OBJECT points)
+
+	GB_ARRAY points = VARG(points);
+	int i, n;
+	double *p;
+	
+	CHECK_DEVICE();
+	
+	if (!points)
+		return;
+	
+	n = GB.Array.Count(points);
+	if (n < 4) 
+		return;
+
+	CHECK_DEVICE();
+	
+	p = (double *)GB.Array.Get(points, 0);
+	
+	PAINT->MoveTo(THIS, p[0], p[1]);
+	for (i = 2; i < n; i+= 2)
+		PAINT->LineTo(THIS, p[i], p[i + 1]);
+	PAINT->LineTo(THIS, p[0], p[1]);
+	
+END_METHOD
+
+
 BEGIN_METHOD(Paint_CurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y2; GB_FLOAT x3; GB_FLOAT y3)
 
 	CHECK_DEVICE();
@@ -668,6 +861,7 @@ BEGIN_METHOD(Paint_CurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y2;
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RelCurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y2; GB_FLOAT x3; GB_FLOAT y3)
 
 	float x, y;
@@ -677,6 +871,7 @@ BEGIN_METHOD(Paint_RelCurveTo, GB_FLOAT x1; GB_FLOAT y1; GB_FLOAT x2; GB_FLOAT y
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_LineTo, GB_FLOAT x; GB_FLOAT y)
 
 	CHECK_DEVICE();
@@ -684,6 +879,7 @@ BEGIN_METHOD(Paint_LineTo, GB_FLOAT x; GB_FLOAT y)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RelLineTo, GB_FLOAT x; GB_FLOAT y)
 
 	float fx, fy;
@@ -693,6 +889,7 @@ BEGIN_METHOD(Paint_RelLineTo, GB_FLOAT x; GB_FLOAT y)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_MoveTo, GB_FLOAT x; GB_FLOAT y)
 
 	CHECK_DEVICE();
@@ -700,6 +897,7 @@ BEGIN_METHOD(Paint_MoveTo, GB_FLOAT x; GB_FLOAT y)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RelMoveTo, GB_FLOAT x; GB_FLOAT y)
 
 	float fx, fy;
@@ -709,15 +907,43 @@ BEGIN_METHOD(Paint_RelMoveTo, GB_FLOAT x; GB_FLOAT y)
 
 END_METHOD
 
-BEGIN_METHOD(Paint_Rectangle, GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h)
+
+BEGIN_METHOD(Paint_Rectangle, GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_FLOAT radius)
 
 	CHECK_DEVICE();
-	PAINT->Rectangle(THIS, VARG(x), VARG(y), VARG(w), VARG(h));
+	
+	float x = VARG(x);
+	float y = VARG(y);
+	float w = VARG(w);
+	float h = VARG(h);
+	float r = VARGOPT(radius, 0.0);
+	
+	if (r <= 0.0)
+		PAINT->Rectangle(THIS, x, y, w, h);
+	else
+	{
+		r = Min(r, Min(w, h) / 2);
+		float r2 = r * (1-0.55228475);
+
+		//PAINT->NewPath(THIS);
+		
+		PAINT->MoveTo(THIS, x + r, y);
+		PAINT->LineTo(THIS, x + w - r, y);
+		PAINT->CurveTo(THIS, x + w - r2, y, x + w, y + r2, x + w, y + r);
+		PAINT->LineTo(THIS, x + w, y + h - r);
+		PAINT->CurveTo(THIS, x + w, y + h - r2, x + w - r2, y + h, x + w - r, y + h);
+		PAINT->LineTo(THIS, x + r, y + h);
+		PAINT->CurveTo(THIS, x + r2, y + h, x, y + h - r2, x, y + h - r);
+		PAINT->LineTo(THIS, x, y + r);
+		PAINT->CurveTo(THIS, x, y + r2, x + r2, y, x + r, y);
+	}
 
 END_METHOD
 
+
 IMPLEMENT_PROPERTY(Paint_Font, Font, GB_FONT, GB_OBJECT, GB.ReturnObject)
 
+
 BEGIN_METHOD(Paint_Text, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
@@ -729,6 +955,7 @@ BEGIN_METHOD(Paint_Text, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_
 	
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
@@ -740,6 +967,7 @@ BEGIN_METHOD(Paint_RichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w;
 	
 END_METHOD
 
+
 BEGIN_METHOD(Paint_DrawText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
@@ -751,6 +979,7 @@ BEGIN_METHOD(Paint_DrawText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w;
 	
 END_METHOD
 
+
 BEGIN_METHOD(Paint_DrawRichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER align)
 
 	CHECK_DEVICE();
@@ -762,6 +991,7 @@ BEGIN_METHOD(Paint_DrawRichText, GB_STRING text; GB_FLOAT x; GB_FLOAT y; GB_FLOA
 	
 END_METHOD
 
+
 BEGIN_METHOD(Paint_TextExtents, GB_STRING text)
 
 	PAINT_EXTENTS *extents;
@@ -783,6 +1013,7 @@ BEGIN_METHOD(Paint_TextExtents, GB_STRING text)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RichTextExtents, GB_STRING text; GB_FLOAT width)
 
 	PAINT_EXTENTS *extents;
@@ -796,6 +1027,39 @@ BEGIN_METHOD(Paint_RichTextExtents, GB_STRING text; GB_FLOAT width)
 
 END_METHOD
 
+
+BEGIN_METHOD(Paint_TextSize, GB_STRING text)
+
+	float w, h;
+	GEOM_RECTF *size;
+
+	CHECK_DEVICE();
+	PAINT->TextSize(THIS, STRING(text), LENGTH(text), &w, &h);
+	size = GEOM.CreateRectF();
+	size->w = w;
+	size->h = h;
+	GB.ReturnObject(size);
+
+END_METHOD
+
+
+BEGIN_METHOD(Paint_RichTextSize, GB_STRING text; GB_FLOAT width)
+
+	float w, h;
+	GEOM_RECTF *size;
+	
+	w = VARGOPT(width, -1);
+	
+	CHECK_DEVICE();
+	PAINT->RichTextSize(THIS, STRING(text), LENGTH(text), w, &w, &h);
+	size = GEOM.CreateRectF();
+	size->w = w;
+	size->h = h;
+	GB.ReturnObject(size);
+
+END_METHOD
+
+
 static PAINT_BRUSH *make_brush(GB_PAINT *d, GB_BRUSH brush)
 {
 	PAINT_BRUSH *that;
@@ -806,6 +1070,7 @@ static PAINT_BRUSH *make_brush(GB_PAINT *d, GB_BRUSH brush)
 	return that;
 }
 
+
 BEGIN_METHOD(Paint_Color, GB_INTEGER color)
 
 	GB_BRUSH brush;
@@ -817,6 +1082,7 @@ BEGIN_METHOD(Paint_Color, GB_INTEGER color)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_Image, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y)
 
 	GB_BRUSH brush;
@@ -832,20 +1098,23 @@ BEGIN_METHOD(Paint_Image, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y)
 	if (!MISSING(x) || !MISSING(y))
 	{
 		GB_TRANSFORM transform;
-		PAINT->Transform.Create(&transform);
-		PAINT->Transform.Translate(transform, VARGOPT(x, 0.0), VARGOPT(y, 0.0));
+		MPAINT->Create(&transform);
+		MPAINT->Translate(transform, VARGOPT(x, 0.0), VARGOPT(y, 0.0));
 		PAINT->Brush.Matrix(brush, TRUE, transform);
-		PAINT->Transform.Delete(&transform);
+		MPAINT->Delete(&transform);
 	}
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_LinearGradient, GB_FLOAT x0; GB_FLOAT y0; GB_FLOAT x1; GB_FLOAT y1; GB_OBJECT colors; GB_OBJECT positions; GB_INTEGER extend)
 
 	GB_BRUSH brush;
 	GB_ARRAY positions, colors;
 	int nstop;
 	
+	CHECK_DEVICE();
+
 	positions = (GB_ARRAY)VARG(positions);
 	if (GB.CheckObject(positions))
 		return;
@@ -862,12 +1131,15 @@ BEGIN_METHOD(Paint_LinearGradient, GB_FLOAT x0; GB_FLOAT y0; GB_FLOAT x1; GB_FLO
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_RadialGradient, GB_FLOAT cx; GB_FLOAT cy; GB_FLOAT radius; GB_FLOAT fx; GB_FLOAT fy; GB_OBJECT colors; GB_OBJECT positions; GB_INTEGER extend)
 
 	GB_BRUSH brush;
 	GB_ARRAY positions, colors;
 	int nstop;
 	
+	CHECK_DEVICE();
+
 	positions = (GB_ARRAY)VARG(positions);
 	if (GB.CheckObject(positions))
 		return;
@@ -884,6 +1156,7 @@ BEGIN_METHOD(Paint_RadialGradient, GB_FLOAT cx; GB_FLOAT cy; GB_FLOAT radius; GB
 
 END_METHOD
 
+
 BEGIN_PROPERTY(Paint_Matrix)
 
 	GB_TRANSFORM transform;
@@ -893,9 +1166,9 @@ BEGIN_PROPERTY(Paint_Matrix)
 	
 	if (READ_PROPERTY)
 	{
-		PAINT->Transform.Create(&transform);
+		MPAINT->Create(&transform);
 		PAINT->Matrix(THIS, FALSE, transform);
-		GB.ReturnObject(create_matrix(PAINT, transform));
+		GB.ReturnObject(create_matrix(transform));
 	}
 	else
 	{
@@ -908,6 +1181,7 @@ BEGIN_PROPERTY(Paint_Matrix)
 
 END_PROPERTY
 
+
 BEGIN_METHOD_VOID(Paint_Reset)
 
 	CHECK_DEVICE();
@@ -915,49 +1189,55 @@ BEGIN_METHOD_VOID(Paint_Reset)
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_Translate, GB_FLOAT tx; GB_FLOAT ty)
 
 	GB_TRANSFORM transform;
 
 	CHECK_DEVICE();
-	PAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	PAINT->Matrix(THIS, FALSE, transform);
-	PAINT->Transform.Translate(transform, (float)VARG(tx), (float)VARG(ty));
+	MPAINT->Translate(transform, (float)VARG(tx), (float)VARG(ty));
 	PAINT->Matrix(THIS, TRUE, transform);
-	PAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_Scale, GB_FLOAT sx; GB_FLOAT sy)
 
 	GB_TRANSFORM transform;
 
 	CHECK_DEVICE();
-	PAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	PAINT->Matrix(THIS, FALSE, transform);
-	PAINT->Transform.Scale(transform, (float)VARG(sx), (float)VARG(sy));
+	MPAINT->Scale(transform, (float)VARG(sx), (float)VARG(sy));
 	PAINT->Matrix(THIS, TRUE, transform);
-	PAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
+
 BEGIN_METHOD(Paint_Rotate, GB_FLOAT angle)
 
 	GB_TRANSFORM transform;
 
 	CHECK_DEVICE();
-	PAINT->Transform.Create(&transform);
+	MPAINT->Create(&transform);
 	PAINT->Matrix(THIS, FALSE, transform);
-	PAINT->Transform.Rotate(transform, (float)VARG(angle));
+	MPAINT->Rotate(transform, (float)VARG(angle));
 	PAINT->Matrix(THIS, TRUE, transform);
-	PAINT->Transform.Delete(&transform);
+	MPAINT->Delete(&transform);
 
 END_METHOD
 
-BEGIN_METHOD(Paint_DrawImage, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y; GB_FLOAT width; GB_FLOAT height)
+
+BEGIN_METHOD(Paint_DrawImage, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y; GB_FLOAT width; GB_FLOAT height; GB_FLOAT opacity; GB_OBJECT source)
 
 	GB_IMG *image;
 	float x, y, w, h;
+	float opacity = VARGOPT(opacity, 1.0);
+	GEOM_RECT *source = (GEOM_RECT *)VARGOPT(source, NULL);
 
 	CHECK_DEVICE();
 	
@@ -968,60 +1248,258 @@ BEGIN_METHOD(Paint_DrawImage, GB_OBJECT image; GB_FLOAT x; GB_FLOAT y; GB_FLOAT
 	
 	x = VARG(x);
 	y = VARG(y);
-	w = VARGOPT(width, image->width);
-	h = VARGOPT(height, image->height);
+	w = VARGOPT(width, -1);
+	h = VARGOPT(height, -1);
 	
 	if (GB.CheckObject(VARG(image)))
 		return;
 	
+	if (w < 0) w = image->width;
+	if (h < 0) h = image->height;
+	
 	if (w <= 0.0 || h <= 0.0)
 		return;
 	
 	if (image->width <= 0 || image->height <= 0)
 		return;
 	
-	PAINT->DrawImage(THIS, VARG(image), x, y, w, h);
+	PAINT->DrawImage(THIS, VARG(image), x, y, w, h, opacity, source ? (GB_RECT *)&source->x : NULL);
 
 END_METHOD
 
+
+BEGIN_METHOD(Paint_DrawPicture, GB_OBJECT picture; GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_OBJECT source)
+
+	GB_PICTURE picture = VARG(picture);
+	GB_PICTURE_INFO info;
+	GEOM_RECT *source = (GEOM_RECT *)VARGOPT(source, NULL);
+	float x, y, w, h;
+
+	CHECK_DEVICE();
+
+	if (GB.CheckObject(picture))
+		return;
+
+	PAINT->GetPictureInfo(THIS, picture, &info);
+
+	x = VARG(x);
+	y = VARG(y);
+	w = VARGOPT(w, -1);
+	h = VARGOPT(h, -1);
+
+	if (w < 0) w = info.width;
+	if (h < 0) h = info.height;
+	
+	if (info.width <= 0 || info.height <= 0)
+		return;
+	
+	PAINT->DrawPicture(THIS, picture, x, y, w, h, source ? (GB_RECT *)&source->x : NULL);
+
+END_METHOD
+
+
+BEGIN_METHOD(Paint_ZoomImage, GB_OBJECT image; GB_INTEGER zoom; GB_INTEGER x; GB_INTEGER y; GB_INTEGER grid; GB_OBJECT source)
+
+	GB_IMAGE image = VARG(image);
+	GB_IMG *info = (GB_IMG *)image;
+	GEOM_RECT *source = (GEOM_RECT *)VARGOPT(source, NULL);
+	int zoom;
+	int x, y, sx, sy, sw, sh;
+	int i, j, xr, yr;
+	bool border;
+	GB_COLOR borderColor;
+	int antialias = FALSE;
+	GB_RECT rect;
+	float opacity = 1.0; //VARGOPT(opacity, 1.0);
+
+	CHECK_DEVICE();
+
+	if (GB.CheckObject(image))
+		return;
+
+	zoom = VARG(zoom);
+	if (zoom < 1)
+	{
+		GB.Error("Bad zoom factor");
+		return;
+	}
+	
+	x = VARGOPT(x, 0);
+	y = VARGOPT(y, 0);
+
+	if (source)
+	{
+		sx = source->x;
+		sy = source->y;
+		sw = source->w;
+		sh = source->h;
+	}
+	else
+	{
+		sx = sy = 0;
+		sw = info->width;
+		sh = info->height;
+	}
+
+	DRAW_NORMALIZE(x, y, sw, sh, sx, sy, sw, sh, info->width, info->height);
+
+	//DRAW->Fill.GetOrigin(THIS, &ox, &oy);
+	
+	PAINT->Save(THIS);
+	PAINT->Antialias(THIS, TRUE, &antialias);
+	
+	borderColor = VARGOPT(grid, GB_COLOR_DEFAULT);
+	border = borderColor != GB_COLOR_DEFAULT;
+
+	rect.x = sx;
+	rect.y = sy;
+	rect.w = sw;
+	rect.h = sh;
+	
+	PAINT->DrawImage(THIS, image, x, y, sw * zoom, sh * zoom, opacity, &rect);
+	
+	if (border && zoom >= 3)
+	{
+		for (i = sx, xr = x; i < (sx + sw); i++, xr += zoom)
+			PAINT->FillRect(THIS, xr, y, 1, sh * zoom, borderColor);
+		
+		for (j = sy, yr = y; j < (sy + sh); j++, yr += zoom)
+			PAINT->FillRect(THIS, x, yr, sw * zoom, 1, borderColor);
+	}
+	
+	PAINT->Restore(THIS);
+
+END_METHOD
+
+
+#if 0
+BEGIN_PROPERTY(Paint_LineStyle)
+
+	int v;
+	int count;
+	float dashes[6];
+
+	CHECK_DEVICE();
+
+	if (READ_PROPERTY)
+	{
+		GB.ReturnInteger(THIS->lineStyle);
+		return;
+	}
+	
+	v = VPROP(GB_INTEGER);
+	
+	switch (v)
+	{
+		case GB_PAINT_LINE_STYLE_NONE:
+			break;
+			
+		case GB_PAINT_LINE_STYLE_SOLID:
+			PAINT->Dash(THIS, TRUE, NULL, &count);
+			break;
+			
+		case GB_PAINT_LINE_STYLE_DASH:
+			dashes[0] = 3; dashes[1] = 3; count = 2;
+			PAINT->Dash(THIS, TRUE, dashes, &count);
+			break;
+			
+		case GB_PAINT_LINE_STYLE_DOT:
+			dashes[0] = 1; dashes[1] = 3; count = 2;
+			PAINT->Dash(THIS, TRUE, dashes, &count);
+			break;
+			
+		case GB_PAINT_LINE_STYLE_DASH_DOT:
+			dashes[0] = 3; dashes[1] = 3; dashes[2] = 3; dashes[3] = 1; dacount = 4;
+			PAINT->Dash(THIS, TRUE, dashes, &count);
+			break;
+			
+		case GB_PAINT_LINE_STYLE_DASH_DOT_DOT:
+			dashes[0] = 3; dashes[1] = 3; dashes[2] = 3; dashes[3] = 1; dashes[4] = 3; dashes[5] = 1; dacount = 6;
+			PAINT->Dash(THIS, TRUE, dashes, &count);
+			break;
+			
+		default:
+			return;
+	}
+
+	THIS->lineStyle = v;
+
+END_PROPERTY
+#endif
+
+BEGIN_METHOD(Paint_FillRect, GB_FLOAT x; GB_FLOAT y; GB_FLOAT w; GB_FLOAT h; GB_INTEGER color)
+
+	CHECK_DEVICE();
+	PAINT->FillRect(THIS, VARG(x), VARG(y), VARG(w), VARG(h), VARG(color));
+	
+END_METHOD
+
+
+BEGIN_PROPERTY(Paint_Tag)
+
+	CHECK_DEVICE();
+	
+	if (READ_PROPERTY)
+		GB.ReturnObject(THIS->tag);
+	else
+		GB.StoreObject(PROP(GB_OBJECT), POINTER(&(THIS->tag)));
+
+END_PROPERTY
+
+//-- Style API --------------------------------------------------------------
+
+#define GET_COORD() \
+	int x, y, w, h; \
+\
+	CHECK_DEVICE(); \
+\
+	x = VARG(x); \
+	y = VARG(y); \
+	w = VARG(w); \
+	h = VARG(h); \
+\
+	if (w < 1 || h < 1) \
+		return;
+
+
 GB_DESC PaintDesc[] =
 {
 	GB_DECLARE("Paint", 0), GB_VIRTUAL_CLASS(),
 
 	GB_STATIC_METHOD("_exit", NULL, Paint_exit, NULL),
 
-	GB_CONSTANT("ExtendPad", "i",                  GB_PAINT_EXTEND_PAD),
-	GB_CONSTANT("ExtendRepeat", "i",               GB_PAINT_EXTEND_REPEAT),
-	GB_CONSTANT("ExtendReflect", "i",              GB_PAINT_EXTEND_REFLECT),
+	GB_CONSTANT("ExtendPad", "i",        GB_PAINT_EXTEND_PAD),
+	GB_CONSTANT("ExtendRepeat", "i",     GB_PAINT_EXTEND_REPEAT),
+	GB_CONSTANT("ExtendReflect", "i",    GB_PAINT_EXTEND_REFLECT),
 
-	GB_CONSTANT("FillRuleWinding", "i",            GB_PAINT_FILL_RULE_WINDING),
-	GB_CONSTANT("FillRuleEvenOdd", "i",            GB_PAINT_FILL_RULE_EVEN_ODD),
+	GB_CONSTANT("FillRuleWinding", "i",  GB_PAINT_FILL_RULE_WINDING),
+	GB_CONSTANT("FillRuleEvenOdd", "i",  GB_PAINT_FILL_RULE_EVEN_ODD),
 
-	GB_CONSTANT("LineCapButt", "i",                GB_PAINT_LINE_CAP_BUTT),
-	GB_CONSTANT("LineCapRound", "i",               GB_PAINT_LINE_CAP_ROUND),
-	GB_CONSTANT("LineCapSquare", "i",              GB_PAINT_LINE_CAP_SQUARE),
+	GB_CONSTANT("LineCapButt", "i",      GB_PAINT_LINE_CAP_BUTT),
+	GB_CONSTANT("LineCapRound", "i",     GB_PAINT_LINE_CAP_ROUND),
+	GB_CONSTANT("LineCapSquare", "i",    GB_PAINT_LINE_CAP_SQUARE),
 	
-	GB_CONSTANT("LineJoinMiter", "i",              GB_PAINT_LINE_JOIN_MITER),
-	GB_CONSTANT("LineJoinRound", "i",              GB_PAINT_LINE_JOIN_ROUND),
-	GB_CONSTANT("LineJoinBevel", "i",              GB_PAINT_LINE_JOIN_BEVEL),
+	GB_CONSTANT("LineJoinMiter", "i",    GB_PAINT_LINE_JOIN_MITER),
+	GB_CONSTANT("LineJoinRound", "i",    GB_PAINT_LINE_JOIN_ROUND),
+	GB_CONSTANT("LineJoinBevel", "i",    GB_PAINT_LINE_JOIN_BEVEL),
 	
-	GB_CONSTANT("OperatorClear", "i",              GB_PAINT_OPERATOR_CLEAR),
-  GB_CONSTANT("OperatorSource", "i",             GB_PAINT_OPERATOR_SOURCE),
-	GB_CONSTANT("OperatorOver", "i",               GB_PAINT_OPERATOR_OVER),
-	GB_CONSTANT("OperatorIn", "i",                 GB_PAINT_OPERATOR_IN),
-	GB_CONSTANT("OperatorOut", "i",                GB_PAINT_OPERATOR_OUT),
-	GB_CONSTANT("OperatorATop", "i",               GB_PAINT_OPERATOR_ATOP),
-	GB_CONSTANT("OperatorDest", "i",               GB_PAINT_OPERATOR_DEST),
-	GB_CONSTANT("OperatorDestOver", "i",           GB_PAINT_OPERATOR_DEST_OVER),
-	GB_CONSTANT("OperatorDestIn", "i",             GB_PAINT_OPERATOR_DEST_IN),
-	GB_CONSTANT("OperatorDestOut", "i",            GB_PAINT_OPERATOR_DEST_OUT),
-	GB_CONSTANT("OperatorDestATop", "i",           GB_PAINT_OPERATOR_DEST_ATOP),
-	GB_CONSTANT("OperatorXor", "i",                GB_PAINT_OPERATOR_XOR),
-	GB_CONSTANT("OperatorAdd", "i",                GB_PAINT_OPERATOR_ADD),
-	GB_CONSTANT("OperatorSaturate", "i",           GB_PAINT_OPERATOR_SATURATE),
+	GB_CONSTANT("OperatorClear", "i",    GB_PAINT_OPERATOR_CLEAR),
+  GB_CONSTANT("OperatorSource", "i",   GB_PAINT_OPERATOR_SOURCE),
+	GB_CONSTANT("OperatorOver", "i",     GB_PAINT_OPERATOR_OVER),
+	GB_CONSTANT("OperatorIn", "i",       GB_PAINT_OPERATOR_IN),
+	GB_CONSTANT("OperatorOut", "i",      GB_PAINT_OPERATOR_OUT),
+	GB_CONSTANT("OperatorATop", "i",     GB_PAINT_OPERATOR_ATOP),
+	GB_CONSTANT("OperatorDest", "i",     GB_PAINT_OPERATOR_DEST),
+	GB_CONSTANT("OperatorDestOver", "i", GB_PAINT_OPERATOR_DEST_OVER),
+	GB_CONSTANT("OperatorDestIn", "i",   GB_PAINT_OPERATOR_DEST_IN),
+	GB_CONSTANT("OperatorDestOut", "i",  GB_PAINT_OPERATOR_DEST_OUT),
+	GB_CONSTANT("OperatorDestATop", "i", GB_PAINT_OPERATOR_DEST_ATOP),
+	GB_CONSTANT("OperatorXor", "i",      GB_PAINT_OPERATOR_XOR),
+	GB_CONSTANT("OperatorAdd", "i",      GB_PAINT_OPERATOR_ADD),
+	GB_CONSTANT("OperatorSaturate", "i", GB_PAINT_OPERATOR_SATURATE),
 
-	GB_STATIC_METHOD("Begin", NULL, Paint_begin, "(Device)o"),
-	GB_STATIC_METHOD("End", NULL, Paint_end, NULL),
+	GB_STATIC_METHOD("Begin", NULL, Paint_Begin, "(Device)o"),
+	GB_STATIC_METHOD("End", NULL, Paint_End, NULL),
 	
 	GB_STATIC_PROPERTY_READ("Device", "o", Paint_Device),
 	GB_STATIC_PROPERTY_READ("W", "f", Paint_Width),
@@ -1031,6 +1509,11 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_PROPERTY_READ("ResolutionX", "i", Paint_ResolutionX),
 	GB_STATIC_PROPERTY_READ("ResolutionY", "i", Paint_ResolutionY),
 	GB_STATIC_PROPERTY("AntiAlias", "b", Paint_Antialias),
+	
+	GB_STATIC_PROPERTY("_Invert", "b", Paint_Invert),
+	GB_STATIC_PROPERTY("_FillStyle", "i", Paint_FillStyle),
+	GB_STATIC_PROPERTY("Background", "i", Paint_Background),
+	GB_STATIC_PROPERTY("_Tag", "o", Paint_Tag),
 
 	GB_STATIC_METHOD("Save", NULL, Paint_Save, NULL),
 	GB_STATIC_METHOD("Restore", NULL, Paint_Restore, NULL),
@@ -1051,18 +1534,19 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_METHOD("Stroke", NULL, Paint_Stroke, "[(Preserve)b]"),
 	//GB_STATIC_PROPERTY_READ("StrokeExtents", "PaintExtents", Paint_StrokeExtents),
 	//GB_STATIC_METHOD("InStroke", "b", Paint_InStroke, "(X)f(Y)f"),
-	GB_STATIC_METHOD("DrawImage", NULL, Paint_DrawImage, "(Image)Image;(X)f(Y)f[(Width)f(Height)f]"),
 
 	GB_STATIC_PROPERTY_READ("PathExtents", "PaintExtents", Paint_PathExtents),
 	GB_STATIC_METHOD("PathContains", "b", Paint_PathContains, "(X)f(Y)f"),
 	
 	GB_STATIC_PROPERTY("Brush", "PaintBrush", Paint_Brush),
+	GB_STATIC_PROPERTY("BrushOrigin", "PointF", Paint_BrushOrigin),
 	GB_STATIC_PROPERTY("Dash", "Float[]", Paint_Dash),
 	GB_STATIC_PROPERTY("DashOffset", "f", Paint_DashOffset),
 	GB_STATIC_PROPERTY("FillRule", "i", Paint_FillRule),
 	GB_STATIC_PROPERTY("LineCap", "i", Paint_LineCap),
 	GB_STATIC_PROPERTY("LineJoin", "i", Paint_LineJoin),
 	GB_STATIC_PROPERTY("LineWidth", "f", Paint_LineWidth),
+	//GB_STATIC_PROPERTY("LineStyle", "i", Paint_LineStyle),
 	GB_STATIC_PROPERTY("MiterLimit", "f", Paint_MiterLimit),
 	GB_STATIC_PROPERTY("Operator", "i", Paint_Operator),
 	//GB_STATIC_PROPERTY("Tolerance", "f", CAIRO_tolerance),
@@ -1073,8 +1557,11 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_PROPERTY_READ("X", "f", Paint_X),
 	GB_STATIC_PROPERTY_READ("Y", "f", Paint_Y),
 
-	GB_STATIC_METHOD("Rectangle", NULL, Paint_Rectangle, "(X)f(Y)f(Width)f(Height)f"),
-	GB_STATIC_METHOD("Arc", NULL, Paint_Arc, "(XC)f(YC)f(Radius)f[(Angle)f(Length)f]"),
+	GB_STATIC_METHOD("Rectangle", NULL, Paint_Rectangle, "(X)f(Y)f(Width)f(Height)f[(Radius)f]"),
+	GB_STATIC_METHOD("FillRect", NULL, Paint_FillRect, "(X)f(Y)f(Width)f(Height)f(Color)i"),
+	GB_STATIC_METHOD("Arc", NULL, Paint_Arc, "(XC)f(YC)f(Radius)f[(Angle)f(Length)f(Pie)b]"),
+	GB_STATIC_METHOD("Ellipse", NULL, Paint_Ellipse, "(X)f(Y)f(Width)f(Height)f[(Angle)f(Length)f(Pie)b]"),
+	GB_STATIC_METHOD("Polygon", NULL, Paint_Polygon, "(Points)Float[];"),
 
 	GB_STATIC_METHOD("CurveTo", NULL, Paint_CurveTo, "(X1)f(Y1)f(X2)f(Y2)f(X3)f(Y3)f"),
 	GB_STATIC_METHOD("RelCurveTo", NULL, Paint_RelCurveTo, "(X1)f(Y1)f(X2)f(Y2)f(X3)f(Y3)f"),
@@ -1084,7 +1571,9 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_METHOD("RelMoveTo", NULL, Paint_RelMoveTo, "(X)f(Y)f"),
 	GB_STATIC_PROPERTY("Font", "Font", Paint_Font),
 	GB_STATIC_METHOD("Text", NULL, Paint_Text, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
+	GB_STATIC_METHOD("TextSize", "RectF", Paint_TextSize, "(Text)s"),
 	GB_STATIC_METHOD("RichText", NULL, Paint_RichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
+	GB_STATIC_METHOD("RichTextSize", "RectF", Paint_RichTextSize, "(Text)s[(Width)f]"),
 	GB_STATIC_METHOD("DrawText", NULL, Paint_DrawText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
 	GB_STATIC_METHOD("DrawRichText", NULL, Paint_DrawRichText, "(Text)s[(X)f(Y)f(Width)f(Height)f(Alignment)i)]"),
 	GB_STATIC_METHOD("TextExtents", "PaintExtents", Paint_TextExtents, "(Text)s"),
@@ -1101,6 +1590,12 @@ GB_DESC PaintDesc[] =
 	GB_STATIC_METHOD("Translate", NULL, Paint_Translate, "(TX)f(TY)f"),
 	GB_STATIC_METHOD("Scale", NULL, Paint_Scale, "(SX)f(SY)f"),
 	GB_STATIC_METHOD("Rotate", NULL, Paint_Rotate, "(Angle)f"),
+	
+	//GB_STATIC_METHOD("Clear", NULL, Paint_Clear, NULL),
+	
+	GB_STATIC_METHOD("DrawImage", NULL, Paint_DrawImage, "(Image)Image;(X)f(Y)f[(Width)f(Height)f(Opacity)f(Source)Rect;]"),
+	GB_STATIC_METHOD("DrawPicture", NULL, Paint_DrawPicture, "(Picture)Picture;(X)f(Y)f[(Width)f(Height)f(Source)Rect;]"),
+	GB_STATIC_METHOD("ZoomImage", NULL, Paint_ZoomImage, "(Image)Image;(Zoom)i(X)i(Y)i[(Grid)i(Source)Rect;]"),
 
 	GB_END_DECLARE
 };
