@@ -2,7 +2,7 @@
 
   eval_read.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -393,6 +393,10 @@ READ_NUMBER:
 		while (isdigit(car))
 			car = next_char();
 	}
+	else if (toupper(car) == 'I')
+	{
+		car = next_char();
+	}
 
 	goto END;
 
@@ -744,7 +748,7 @@ static void add_identifier()
 			}
 			else
 			{
-				if (last_type)
+				if (last_type || PATTERN_is(last_pattern, RS_OPEN))
 					can_be_reserved = TRUE;
 				else
 					can_be_reserved = FALSE;
