@@ -2,7 +2,7 @@
 
   main.c
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,9 +25,13 @@
 
 #include "main.h"
 
+#define USE_NOTHING 0
+#define USE_GB_QT4 1
+#define USE_GB_GTK 2
+
 GB_INTERFACE GB EXPORT;
 
-// Prevents gbi2 from complaining
+// Prevents gbi3 from complaining
 
 GB_DESC *GB_CLASSES[] EXPORT =
 {
@@ -38,31 +42,52 @@ char *GB_INCLUDE EXPORT = "gb.qt4";
 
 int EXPORT GB_INIT(void)
 {
-	char *comp;
+	int use = USE_NOTHING;
 	char *env;
-	
+	const char *comp;
+	const char *comp2;
+
 	env = getenv("GB_GUI");
 	if (env)
 	{
-		comp = env;
-		if (!GB.LoadComponent(env))
-			return FALSE;
+		if (strcmp(env, "gb.qt4") == 0)
+			use = USE_GB_QT4;
+		else if (strcmp(env, "gb.gtk") == 0)
+			use = USE_GB_GTK;
 	}
 	
-	comp = "gb.gtk";
-	
-	env = getenv("KDE_FULL_SESSION");
-	
-	if (env && !strcmp(env, "true"))
+	if (use == USE_NOTHING)
 	{
-		env = getenv("KDE_SESSION_VERSION");
-		if (env && !strcmp(env, "4"))
-			comp = "gb.qt4";
-	}
+		use = USE_GB_GTK;
 		
-	if (GB.LoadComponent(comp))
-		fprintf(stderr, "gb.gui: unable to load '%s' component\n", comp);
+		env = getenv("KDE_FULL_SESSION");
+		
+		if (env && !strcmp(env, "true"))
+		{
+			env = getenv("KDE_SESSION_VERSION");
+			if (env && !strcmp(env, "4"))
+				use = USE_GB_QT4;
+		}
+	}
+
+	comp = use == USE_GB_QT4 ? "gb.qt4" : "gb.gtk";
+
+	if (GB.Component.Load(comp))
+	{
+		comp2 = use == USE_GB_QT4 ? "gb.gtk" : "gb.qt4";
+
+		if (GB.Component.Load(comp2))
+		{
+			fprintf(stderr, "gb.gui: error: unable to find any GUI component. Please install the 'gb.qt4' or 'gb.gtk' component\n");
+			exit(1);
+		}
+
+		fprintf(stderr, "gb.gui: warning: '%s' component not found, using '%s' instead\n", comp, comp2);
+		comp = comp2;
+	}
   
+  setenv("GB_GUI", comp, TRUE);
+
   return 0;
 }
 
