@@ -2,7 +2,7 @@
 
   gb_file_temp.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ static void push_path(void **list, const char *path)
 {
 	FILE_PATH *slot;
 
-	ALLOC(&slot, sizeof(FILE_PATH), "push_path");
+	ALLOC(&slot, sizeof(FILE_PATH));
 	slot->path = STRING_new_zero(path);
 
 	slot->next = *list;
@@ -118,7 +118,7 @@ static char *pop_path(void **list)
 	path = ((FILE_PATH *)*list)->path;
 	slot = *list;
 	*list = ((FILE_PATH *)*list)->next;
-	FREE(&slot, "pop_path");
+	FREE(&slot);
 
 	//printf("pop_path: %s\n", path);
 	return path;
@@ -178,14 +178,31 @@ void FILE_remove_temp_file(void)
 	rmdir(FILE_make_temp(NULL, NULL));
 }
 
+void FILE_remove_temp_file_pid(pid_t pid)
+{
+	snprintf(file_buffer, sizeof(file_buffer), FILE_TEMP_DIR, (int)getuid(), (int)pid);
+	FILE_recursive_dir(file_buffer, NULL, remove_temp_file, 0, FALSE);
+	rmdir(FILE_make_temp(NULL, NULL));
+}
+
 void FILE_init(void)
 {
+	struct stat info;
+	
 	FILE_remove_temp_file();
 	
 	snprintf(file_buffer, sizeof(file_buffer), FILE_TEMP_PREFIX, (int)getuid());
-	mkdir(file_buffer, S_IRWXU);
-	snprintf(file_buffer, sizeof(file_buffer), FILE_TEMP_DIR, (int)getuid(), (int)getpid());
-	mkdir(file_buffer, S_IRWXU);
+	(void)mkdir(file_buffer, S_IRWXU);
+	
+	if (lstat(file_buffer, &info) == 0 && S_ISDIR(info.st_mode) && chown(file_buffer, getuid(), getgid()) == 0 && chmod(file_buffer, S_IRWXU) == 0)
+	{
+		snprintf(file_buffer, sizeof(file_buffer), FILE_TEMP_DIR, (int)getuid(), (int)getpid());
+		(void)mkdir(file_buffer, S_IRWXU);
+		if (lstat(file_buffer, &info) == 0 && S_ISDIR(info.st_mode) && chown(file_buffer, getuid(), getgid()) == 0 && chmod(file_buffer, S_IRWXU) == 0)
+			return;
+	}
+
+	ERROR_fatal("cannot initialize interpreter temporary directory. Do you try to hijack Gambas?");
 }
 
 void FILE_exit(void)
@@ -254,7 +271,7 @@ const char *FILE_cat(const char *path, ...)
 		if (len > 0)
 		{
 			if ((p + len) > &file_buffer[PATH_MAX])
-				return NULL;
+				THROW(E_TOOLONG);
 
 			if (p != path)
 			{
@@ -276,6 +293,8 @@ const char *FILE_cat(const char *path, ...)
 		add_slash = ((!end_slash) && (*path != 0) && (*path != '/'));
 	}
 
+	va_end(args);
+	
 	file_buffer_length = p - file_buffer;
 	return file_buffer;
 }
@@ -295,6 +314,24 @@ int FILE_buffer_length(void)
 	return file_buffer_length;
 }
 
+static void init_file_buffer(const char *path)
+{
+	int len;
+	
+	if (path == file_buffer)
+		return;
+	
+	len = strlen(path);
+	
+	if (len > PATH_MAX)
+		THROW(E_TOOLONG);
+	
+	strcpy(file_buffer, path);
+	file_buffer_length = len;
+}
+
+//#define INIT_FILE_BUFFER(_path) (init_file_buffer(), _path = file_buffer)
+
 const char *FILE_get_dir(const char *path)
 {
 	char *p;
@@ -305,8 +342,7 @@ const char *FILE_get_dir(const char *path)
 	if (path[0] == '/' && path[1] == 0)
 		return "/";
 
-	if (file_buffer != path)
-		strcpy(file_buffer, path);
+	init_file_buffer(path);
 
 	p = rindex(file_buffer, '/');
 
@@ -357,13 +393,9 @@ const char *FILE_set_ext(const char *path, const char *ext)
 {
 	char *p;
 
-	if (path != file_buffer)
-	{
-		strcpy(file_buffer, path);
-		path = file_buffer;
-	}
+	init_file_buffer(path);
 
-	p = (char *)FILE_get_ext(path);
+	p = (char *)FILE_get_ext(file_buffer);
 
 	if (!ext)
 	{
@@ -371,13 +403,13 @@ const char *FILE_set_ext(const char *path, const char *ext)
 			p[-1] = 0;
 		else
 			*p = 0;
-		return path;
+		return file_buffer;
 	}
 
 	if (&p[strlen(ext)] >= &file_buffer[PATH_MAX])
-		return path;
+		THROW(E_TOOLONG);
 
-	if (p == path || p[-1] != '.')
+	if (p == file_buffer || p[-1] != '.')
 		*p++ = '.';
 
 	if (*ext == '.')
@@ -386,7 +418,7 @@ const char *FILE_set_ext(const char *path, const char *ext)
 	strcpy(p, ext);
 
 	file_buffer_length = -1;
-	return path;
+	return file_buffer;
 }
 
 
@@ -396,15 +428,13 @@ const char *FILE_get_basename(const char *path)
 
 	path = FILE_get_name(path);
 
-	if (file_buffer != path)
-		strcpy(file_buffer, path);
-
+	init_file_buffer(path);
+	
 	p = rindex(file_buffer, '.');
 	if (p)
 		*p = 0;
 
 	file_buffer_length = -1;
-
 	return file_buffer;
 }
 
@@ -694,8 +724,9 @@ bool FILE_dir_next(char **path, int *len)
 	#else
 	if (file_attr)
 	{
-		strcpy(p, file_path);
-		p += strlen(file_path);
+		init_file_buffer(file_path);
+		p += file_buffer_length;
+		
 		if (p[-1] != '/' && (file_buffer[1] || file_buffer[0] != '/'))
 			*p++ = '/';
 	}
@@ -715,6 +746,10 @@ bool FILE_dir_next(char **path, int *len)
 		if (name[0] == '.' && (name[1] == 0 || (name[1] == '.' && name[2] == 0)))
 			continue;
 
+		len_entry = strlen(name);
+		if ((len_entry + file_buffer_length) > PATH_MAX)
+			continue;
+			
 		if (file_attr)
 		{
 			#ifdef _DIRENT_HAVE_D_TYPE
@@ -732,8 +767,6 @@ bool FILE_dir_next(char **path, int *len)
 			#endif
 		}
 
-		len_entry = strlen(name);
-
 		if (file_pattern == NULL)
 			break;
 
@@ -767,11 +800,10 @@ void FILE_recursive_dir(const char *dir, void (*found)(const char *), void (*aft
 	char *temp;
 	bool is_dir;
 
-	if (!FILE_is_dir(dir))
-		return;
-
 	if (!dir || *dir == 0)
 		dir = ".";
+	else if (!FILE_is_dir(dir))
+		return;
 
 	STRING_free(&file_rdir_path);
 	file_rdir_path = STRING_new_zero(dir);
@@ -873,8 +905,7 @@ void FILE_make_path_dir(const char *path)
 	if (FILE_is_relative(path))
 		return;
 
-	if (path != file_buffer)
-		strcpy(file_buffer, path);
+	init_file_buffer(path);
 
 	for (i = 1;; i++)
 	{
@@ -884,7 +915,7 @@ void FILE_make_path_dir(const char *path)
 		if (c == '/')
 		{
 			file_buffer[i] = 0;
-			mkdir(file_buffer, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+			(void)mkdir(file_buffer, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 			file_buffer[i] = c;
 		}
 		c++;
@@ -906,7 +937,7 @@ void FILE_copy(const char *src, const char *dst)
 	if (FILE_exist(dst))
 		THROW(E_EXIST, dst);
 
-	ALLOC(&buf, MAX_IO, "FILE_copy");
+	ALLOC(&buf, MAX_IO);
 
 	TRY
 	{
@@ -926,7 +957,7 @@ void FILE_copy(const char *src, const char *dst)
 		STREAM_close(&stream_src);
 		STREAM_close(&stream_dst);
 
-		FREE(&buf, "FILE_copy");
+		FREE(&buf);
 	}
 	CATCH
 	{
@@ -934,7 +965,7 @@ void FILE_copy(const char *src, const char *dst)
 			STREAM_close(&stream_src);
 		if (stream_dst.type)
 			STREAM_close(&stream_dst);
-		FREE(&buf, "FILE_copy");
+		FREE(&buf);
 
 		PROPAGATE();
 	}
@@ -1015,6 +1046,65 @@ time_t FILE_get_time(const char *path)
 		return (time_t)-1L;
 }
 
+bool FILE_copy(const char *src, const char *dst)
+{
+	int src_fd;
+	int dst_fd;
+	ssize_t len;
+	char *buf = NULL;
+	int save_errno;
+	struct stat info;
+
+	fprintf(stderr, "FILE_copy: %s -> %s\n", src, dst);
+	
+	if (stat(src, &info))
+		return TRUE;
+	
+	src_fd = open(src, O_RDONLY);
+	if (src_fd < 0)
+	{
+		fprintf(stderr, "open src failed\n");
+		return TRUE;
+	}
+	
+	dst_fd = creat(dst, info.st_mode);
+	if (dst_fd < 0)
+	{
+		fprintf(stderr, "open dst failed\n");
+		save_errno = errno;
+		close(src_fd);
+		errno = save_errno;
+		return TRUE;
+	}
+	
+	ALLOC(&buf, MAX_IO);
+
+	for(;;)
+	{
+		len = read(src_fd, buf, MAX_IO);
+		if (len == 0)
+			break;
+		if (len < 0 && errno == EINTR)
+			continue;
+		if (write(dst_fd, buf, len) < 0)
+		{
+			save_errno = errno;
+			close(src_fd);
+			close(dst_fd);
+			unlink(dst);
+			errno = save_errno;
+			IFREE(buf);
+			return TRUE;
+		}
+	}
+	
+	close(src_fd);
+	close(dst_fd);
+	IFREE(buf);
+	
+	return FALSE;
+}
+
 #endif
 
 const char *FILE_getcwd(const char *subdir)
@@ -1048,15 +1138,20 @@ const char *FILE_find_gambas(void)
 {
 	const char *path;
 
-	if (FILE_exist(GAMBAS_LINK_PATH))
+	path = getenv("GB_PATH");
+
+	if (!path || !*path)
 	{
-		path = FILE_readlink(GAMBAS_LINK_PATH);
-		if (!path)
-			path = GAMBAS_LINK_PATH;
-	}
-	else
-	{
-		path = GAMBAS_PATH "/gbx" GAMBAS_VERSION_STRING;
+		if (FILE_exist(GAMBAS_LINK_PATH))
+		{
+			path = FILE_readlink(GAMBAS_LINK_PATH);
+			if (!path)
+				path = GAMBAS_LINK_PATH;
+		}
+		else
+		{
+			path = GAMBAS_PATH "/gbx" GAMBAS_VERSION_STRING;
+		}
 	}
 
 	return path;
