@@ -2,7 +2,7 @@
 
   gb_reserved.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -139,6 +139,7 @@ typedef
 		RS_CONTINUE,
 		RS_GOTO,
 		RS_GOSUB,
+		RS_ON,
 		RS_ME,
 		RS_LAST,
 		RS_TRY,
@@ -305,12 +306,11 @@ typedef
 		short flag;
 		unsigned char value;
 		unsigned char read_switch;
-		unsigned char priority;
+		unsigned short priority;
 		short code;
 		short subcode;
 		void (*func)();
 		}
-	PACKED
 	COMP_INFO;
 
 typedef
@@ -321,7 +321,6 @@ typedef
 		short min_param;
 		short max_param;
 		}
-	PACKED
 	SUBR_INFO;
 
 #ifndef __RESERVED_C
