@@ -2,7 +2,7 @@
 
   gb_reserved_keyword.h
 
-  (c) 2000-2012 Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2013 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ COMP_INFO COMP_res_info[] =
 	{ "Class",        RSF_IDENT,                0,          2     },
 	{ "Function",     RSF_IDENT,                0,          4     },
 	{ "Struct",       RSF_PREV,                 0,          3     },
-	{ "Const",        0,                        0,          5     },
+	{ "Const",        RSF_IDENT,                0,          5     },
 	{ "Private",      RSF_IDENT|RSF_PUB,                          },
 	{ "Public",       RSF_IDENT|RSF_PUB,                          },
 	{ "Static",       RSF_PUB                                     },
@@ -86,6 +86,7 @@ COMP_INFO COMP_res_info[] =
 	{ "Continue"                                                  },
 	{ "Goto"                                                      },
 	{ "GoSub"                                                     },
+	{ "On"                                                        },
 	{ "Me",           0,                        0,          1     },
 	{ "Last",         0,                        0,          1     },
 	{ "Try"                                                       },
@@ -198,6 +199,8 @@ COMP_INFO COMP_res_info[] =
 	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,    C_LIKE, 5     }, // NOT BEGINS
 	{ "Ends",         RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,    C_LIKE, 2     },
 	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,    C_LIKE, 6     }, // NOT ENDS
+	{ "Match",        RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,    C_LIKE, 3     },
+	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,    C_LIKE, 7     }, // NOT MATCH
 
 	{ "+=",           RSF_ASGN,                 RS_PLUS                                },
 	{ "-=",           RSF_ASGN,                 RS_MINUS                               },
@@ -331,7 +334,7 @@ SUBR_INFO COMP_subr_info[] =
 	{ "IIf",               31,  0,  3,    },
 	{ "Choose",            32,  0,  1, 63 },
 
-	{ ".Array",            33,  0,  1, 63 },   /* Needed for Eval("[...]") */
+	{ ".Array",            33,  0,  0, 63 },   /* Needed for Eval("[...]") */
 
 	{ "ATan2",             34,  1,  2     },
 	{ "Atn2",              34,  1,  2     },
@@ -417,8 +420,8 @@ SUBR_INFO COMP_subr_info[] =
 	{ "WeekDay",           47,  7,  1     },
 	{ "Week",              48,  0,  0,  3 },
 
-	{ "Date",              49,  0,  0,  6 },
-	{ "Time",              50,  0,  0,  3 },
+	{ "Date",              49,  0,  0,  7 },
+	{ "Time",              50,  0,  0,  4 },
 
 	{ "DateAdd",           51,  0,  3     },
 	{ "DateDiff",          51,  1,  3     },
@@ -457,6 +460,11 @@ SUBR_INFO COMP_subr_info[] =
 	{ ".Kill",             69,  0,  1     },
 	{ ".Mkdir",            69,  1,  1     },
 	{ ".Rmdir",            69,  2,  1     },
+#endif
+	//{ ".Mkdir",            70,  0,  1     }, // The old Mkdir from 3.0
+	{ "Even",              70,  1,  1     },
+	{ "Odd",               70,  2,  1     },
+#ifndef __EVAL_RESERVED_C
 	{ ".Move",             72,  0,  2     },
 	{ ".Copy",             72,  1,  2     },
 	{ ".Link",             72,  2,  2     },
@@ -511,9 +519,13 @@ SUBR_INFO COMP_subr_info[] =
 	{ "Shell$",            92,  1,  1     },
 	{ "Html",              92,  2,  1     },
 	{ "Html$",             92,  2,  1     },
+	{ "Base64",            92,  3,  1     },
+	{ "Base64$",           92,  3,  1     },
 	
-	{ "Unquote",           93,  0,  1     },
-	{ "Unquote$",          93,  0,  1     },
+	{ "UnQuote",           93,  0,  1     },
+	{ "UnQuote$",          93,  0,  1     },
+	{ "UnBase64",          93,  1,  1     },
+	{ "UnBase64$",         93,  1,  1     },
 	
 	{ "MkBool",            94,  1,  1     },
 	{ "MkBool$",           94,  1,  1     },
